// Author: Sergey Linev, GSI   7/12/2016

/*************************************************************************
 * Copyright (C) 1995-2018, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifdef __CLING__

#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;

#pragma link C++ class TWebMenuItem+;
#pragma link C++ class TWebCheckedMenuItem+;
#pragma link C++ class TWebMenuArgument+;
#pragma link C++ class TWebArgsMenuItem+;
#pragma link C++ class TWebMenuItems+;

#pragma link C++ class TWebObjectOptions+;
#pragma link C++ class TWebPadOptions+;
#pragma link C++ class TWebPadClick+;

#pragma link C++ class TWebPadPainter+;
#pragma link C++ class TWebPainting+;
#pragma link C++ class TWebSnapshot+;
#pragma link C++ class TPadWebSnapshot+;
#pragma link C++ class TCanvasWebSnapshot+;

#pragma link C++ class TWebCanvas+;

#pragma link C++ class TWebControlBar+;

#endif
