/* @(#)root/graf:$Id$ */

/*************************************************************************
 * Copyright (C) 1995-2000, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifdef __CLING__

#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;

#pragma link C++ global gHistImagePalette;
#pragma link C++ global gWebImagePalette;


#pragma link C++ class TArc+;
#pragma link C++ class TArrow+;
#pragma link C++ class TAttImage;
#pragma link C++ class TBox-;
#pragma link C++ class TCandle-;
#pragma link C++ class TCrown+;
#pragma link C++ class TCurlyArc+;
#pragma link C++ class TCurlyLine+;
#pragma link C++ class TCutG-;
#pragma link C++ class TDiamond+;
#pragma link C++ class TEllipse-;
#pragma link C++ class TFrame+;
#pragma link C++ class TGaxis-;
#pragma link C++ class TGraphPolar+;
#pragma link C++ class TGraphPolargram+;
#pragma link C++ class TGraphQQ+;
#pragma link C++ class TGaxis-;
#pragma link C++ class TImage;
#pragma link C++ class TImagePlugin;
#pragma link C++ class TImagePalette;
#pragma link C++ class TLatex+;
#pragma link C++ class TMathText+;
#pragma link C++ class TLegend+;
#pragma link C++ class TLegendEntry+;
#pragma link C++ class TLine-;
#pragma link C++ class TLink+;
#pragma link C++ class TMarker-;
#pragma link C++ class TPaletteEditor;
#pragma link C++ class TPave-;
#pragma link C++ class TPaveLabel+;
#pragma link C++ class TPaveStats-;
#pragma link C++ class TPaveText-;
#pragma link C++ class TPavesText+;
#pragma link C++ class TPie+;
#pragma link C++ class TPieSlice+;
#pragma link C++ class TPoints+;
#pragma link C++ class TPolyLine-;
#pragma link C++ class TText-;
#pragma link C++ class TTF;
#pragma link C++ class TWbox+;

#pragma link C++ enum TImage::EImageFileTypes;
#pragma link C++ enum TImage::EText3DType;
#pragma link C++ enum TImage::ECharType;
#pragma link C++ enum TImage::ETileType;
#pragma link C++ enum TImage::ECoordMode;
#pragma link C++ enum TImage::EColorChan;

#pragma link C++ function operator+(const TImage&,const TImage&);
#pragma link C++ function operator/(const TImage&,const TImage&);

#endif
