#ifndef roofit_roofitcore_RooFit_RooSetProxy_h
#define roofit_roofitcore_RooFit_RooSetProxy_h

#include <RooArgList.h>
#include <RooCollectionProxy.h>

using RooSetProxy = RooCollectionProxy<RooArgSet>;

#endif
