/*!
 * OpenUI5
 * (c) Copyright 2009-2025 SAP SE or an SAP affiliate company.
 * Licensed under the Apache License, Version 2.0 - see LICENSE.txt.
 */

@import "../../../../../sap/ui/core/themes/base/base.less";
@import "../../../../../sap/ui/core/themes/base/global.less";

@import "AlignedFlowLayout.less";
@import "ColumnLayout.less";
@import "DynamicSideContent.less";
@import "FixFlex.less";
@import "Form.less";
@import "FormLayout.less";
@import "Grid.less";
@import "CSSGrid.less";
@import "GridLayout.less";
@import "HorizontalLayout.less";
@import "ResponsiveFlowLayout.less";
@import "ResponsiveGridLayout.less";
@import "ResponsiveLayout.less";
@import "ResponsiveSplitter.less";
@import "SimpleForm.less";
@import "Splitter.less";
@import "VerticalLayout.less";
@import "BlockLayoutRow.less";
@import "BlockLayoutCell.less";
@import "BlockLayout.less";
