/* NOTE: This file was generated as an optimized version of "library.source.less" for the Theme Designer. */

/*!
 * OpenUI5
 * (c) Copyright 2009-2025 SAP SE or an SAP affiliate company.
 * Licensed under the Apache License, Version 2.0 - see LICENSE.txt.
 */

@import "../base/library.less";
@import "../../../../../../Base/baseLib/sap_fiori_3/base.less"; /* ORIGINAL IMPORT PATH: "../../../../../sap/ui/core/themes/sap_fiori_3/base.less" */

@import "../../../../../sap/ui/core/themes/sap_fiori_3/global.less";

/* START "FormLayout.less" */
/* ========================================= */
/* CSS for control sap.ui.layout/FormLayout  */
/* Fiori 3 theme                             */
/* ========================================= */

.sapUiFormLayout {
	font-family: @sapUiDesktopFontFamily;
}

.sapUiFormLayout.sapUiFormBackgrTranslucent > section {
	background-color: fade(@sapUiGroupContentBackground, @sapUiTranslucentBGOpacity);
}

.sapUiFormLayout.sapUiFormBackgrSolid > section {
	background-color: @sapUiGroupContentBackground;
}

.sapUiForm .sapUiFormTitle {
	border-width: 1px;
	border-color: @sapUiGroupTitleBorderColor;
	font-family: @sapUiFontHeaderFamily;
	height: 2.75rem;
	line-height: 2.75rem;
}

.sapUiFormM .sapUiFormContainerTitle > .sapUiFormTitle {
	height: 2.75rem;
	line-height: 2.75rem;
}

.sapUiSizeCompact .sapUiFormContainerTitle > .sapUiFormTitle {
	height: 2rem;
	line-height: 2rem;
}

.sapUiFormTitle > .sapUiBtn {
	bottom: 0.25rem;
}

/* Title levels should be all rendered in same style. Levels can be changed by application to meet page structure without visual effect */
/* Form Titles are rendered using H4 size*/
.sapUiFormTitleH1,
.sapUiFormTitleH2,
.sapUiFormTitleH3,
.sapUiFormTitleH4,
.sapUiFormTitleH5,
.sapUiFormTitleH6 {
	font-size: @sapMFontHeader4Size;
}

/* FormContainer Titles are rendered using H4 size*/
.sapUiFormContainerTitle .sapUiFormTitleH1,
.sapUiFormContainerTitle .sapUiFormTitleH2,
.sapUiFormContainerTitle .sapUiFormTitleH3,
.sapUiFormContainerTitle .sapUiFormTitleH4,
.sapUiFormContainerTitle .sapUiFormTitleH5,
.sapUiFormContainerTitle .sapUiFormTitleH6 {
	font-size: @sapMFontHeader6Size;
	color: @sapUiGroupTitleTextColor;
	font-weight: bold;
}

/* Toolbar */
.sapUiFormContainerToolbar > div:first-child > .sapMTitle{
	font-weight: bold;
}

.sapUiFormContainerToolbar > div:first-child >.sapMTitle.sapMTitleStyleAuto{
	font-size: @sapMFontHeader6Size;
}

.sapUiFormContainerToolbar > .sapMTB-Transparent-CTX.sapMTB.sapMIBar {
	padding: 0 0.25rem 0 0;
}

/* END "FormLayout.less" */

/* START "ColumnLayout.less" */
/* ================================================ */
/* CSS for control sap.ui.layout/form/ColumnLayout  */
/* Fiori 3 theme                                    */
/* ================================================ */

/* Background */
.sapUiFormBackgrTranslucent > .sapUiFormCLContent {
	background-color: fade(@sapUiGroupContentBackground, @sapUiTranslucentBGOpacity);
}

.sapUiFormBackgrSolid > .sapUiFormCLContent {
	background-color: @sapUiGroupContentBackground;
}

/* Toolbar */
.sapUiFormCL.sapUiFormToolbar > div:first-child {
	height: 2.75rem; /* to have always 3rem on Form header */
	line-height: 2.75rem;
	padding-right: 1rem;
}

/* edit mode */
/* labels next to field */
.sapUiFormEdit > .sapUiFormCLSmallColumns .sapUiFormElementLbl:not(.sapUiFormCLCellsS12),
.sapUiFormEdit > .sapUiFormCLWideColumns .sapUiFormElementLbl:not(.sapUiFormCLCellsL12){
	padding-top: 0.875rem;
}
/* END "ColumnLayout.less" */

/* START "GridLayout.less" */
/* ========================================= */
/* CSS for control sap.ui.layout/GridLayout  */
/* Fiori 3 theme                             */
/* ========================================= */

.sapUiGrid {
	font-family: @sapUiDesktopFontFamily;
}

.sapUiGrid.sapUiFormBackgrTranslucent > tbody > tr {
	background-color: fade(@sapUiGroupContentBackground, @sapUiTranslucentBGOpacity);
}

.sapUiGrid.sapUiFormBackgrSolid > tbody > tr {
	background-color: @sapUiGroupContentBackground;
}

.sapUiGrid > tbody > tr.sapUiGridTitle {
	background-color: transparent;
}

/* Title */
.sapUiGridHeader > .sapUiFormTitle > .sapUiBtnS.sapUiBtnLite {
	bottom: 0.3125rem;
}

/* Toolbar */
.sapUiGrid.sapUiFormToolbar th > div {
	height: 2.75rem;
	line-height: 2.75rem;
}

.sapUiGridHeader.sapUiFormContainerToolbar > .sapMTB-Transparent-CTX.sapMTB.sapMIBar {
	padding: 0;
}

/* alignment of Labels */
.sapUiFormEdit.sapUiFormM > .sapUiGrid > tbody > tr > td.sapUiFormElementLbl {
	padding-top: 0.875rem;
}

.sapUiFormEdit.sapUiFormM > .sapUiGrid > tbody > tr.sapUiGridConteinerFirstRow > td.sapUiFormElementLbl {
	padding-top: 2.875rem;
}

.sapUiFormEdit.sapUiFormM >.sapUiGrid > tbody > tr:first-child > td.sapUiFormElementLbl, /* if no Form title */
.sapUiFormEdit[aria-labelledby].sapUiFormM > .sapUiGrid > tbody > tr:nth-child(2) > td.sapUiFormElementLbl {
	padding-top: 1.875rem;
}

/* END "GridLayout.less" */

/* START "ResponsiveGridLayout.less" */
/* =================================================== */
/* CSS for control sap.ui.layout/ResponsiveGridLayout  */
/* Fiori 3 theme                                       */
/* =================================================== */

/* Toolbar */
.sapUiFormResGrid.sapUiFormToolbar > div:first-child {
	background-color: transparent;
	height: 2.75rem;
	line-height: 2.75rem;
	padding-right: 1rem;
}

/* Grid */
.sapUiFormResGrid.sapUiFormBackgrTranslucent > div {
	background-color: fade(@sapUiGroupContentBackground, @sapUiTranslucentBGOpacity);
}

.sapUiFormResGrid.sapUiFormBackgrSolid > div {
	background-color: @sapUiGroupContentBackground;
}

/* alignment of Labels */
.sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanS12),
.sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanM12),
.sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanL12),
.sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanXL12){
	padding-top: 0.875rem;
}

/* END "ResponsiveGridLayout.less" */

/* START "ResponsiveLayout.less" */
/* =============================================== */
/* CSS for control sap.ui.layout/ResponsiveLayout  */
/* Fiori 3 theme                                   */
/* =============================================== */

.sapUiFormResLayout.sapUiFormBackgrTranslucent > div {
	background-color: fade(@sapUiGroupContentBackground, @sapUiTranslucentBGOpacity);
}

.sapUiFormResLayout.sapUiFormBackgrSolid > div {
	background-color: @sapUiGroupContentBackground;
}

.sapUiRLContainer > .sapUiFormTitle > .sapUiBtn {
	bottom: 0.3125rem;
}

/* Toolbar */
.sapUiFormResLayout.sapUiFormToolbar > div:first-child {
	background-color: transparent;
	height: 2.75rem;
	line-height: 2.75rem;
}

.sapUiRLContainer.sapUiFormContainerToolbar > .sapMTB-Transparent-CTX.sapMTB.sapMIBar {
	padding: 0;
}

/* alignment of Label */
.sapUiFormEdit.sapUiFormM .sapUiRLElementWithLabel > .sapUiRFLRow:not(.sapUiRFLCompleteRow) > .sapUiRFLContainer:first-child > .sapUiRFLContainerContent{
	padding-top: 0.875rem;
}

html[data-sap-ui-browser^="ff"] .sapUiFormEdit.sapUiFormM .sapUiRLElementWithLabel > .sapUiRFLRow:not(.sapUiRFLCompleteRow) > .sapUiRFLContainer:first-child > .sapUiRFLContainerContent{
	padding-top: 0.8125rem;
}

/* END "ResponsiveLayout.less" */

/* START "ResponsiveSplitter.less" */
/* ================================================= */
/* CSS for control sap.ui.layout/ResponsiveSplitter  */
/* Fiori 3 theme                                     */
/* ================================================= */

@_sap_ui_layout_ResponsiveSplitter_PaginatorHeight: 2.75rem;
@_sap_ui_layout_ResponsiveSplitter_PaginatorButtonWidth: 2.25rem;
@_sap_ui_layout_ResponsiveSplitter_PaginatorButtonHeight: 2.25rem;
/* END "ResponsiveSplitter.less" */

/* START "Splitter.less" */
/* ================================================= */
/* CSS for control sap.ui.layout/Splitter            */
/* Fiori 3 theme                                     */
/* ================================================= */

@_sap_ui_layout_Splitter_BarBorder: none;
/* END "Splitter.less" */

/* START "BlockLayout.less" */
/* ========================================== */
/* CSS for control sap.ui.layout/BlockLayout  */
/* Fiori 3 theme                              */
/* ========================================== */

@_sap_ui_layout_BlockLayout_FontSizeLandXL: @sapMFontHeader2Size;

/* ============ Custom Cell Background Color ============== */
@_sap_ui_layout_BlockLayout_BlockColorAccentType1A: lighten(@sapUiAccent1, 53);
@_sap_ui_layout_BlockLayout_BlockColorAccentType1B: lighten(@sapUiAccent1, 51);
@_sap_ui_layout_BlockLayout_BlockColorAccentType1C: lighten(@sapUiAccent1, 49);
@_sap_ui_layout_BlockLayout_BlockColorAccentType1D: lighten(@sapUiAccent1, 47);
@_sap_ui_layout_BlockLayout_BlockColorAccentType1E: @sapUiAccent1;
@_sap_ui_layout_BlockLayout_BlockColorAccentType1F: darken(@sapUiAccent1, 3);

@_sap_ui_layout_BlockLayout_BlockColorAccentType2A: lighten(@sapUiAccent2, 44);
@_sap_ui_layout_BlockLayout_BlockColorAccentType2B: lighten(@sapUiAccent2, 42);
@_sap_ui_layout_BlockLayout_BlockColorAccentType2C: lighten(@sapUiAccent2, 40);
@_sap_ui_layout_BlockLayout_BlockColorAccentType2D: lighten(@sapUiAccent2, 38);
@_sap_ui_layout_BlockLayout_BlockColorAccentType2E: @sapUiAccent2;
@_sap_ui_layout_BlockLayout_BlockColorAccentType2F: darken(@sapUiAccent2, 4);

@_sap_ui_layout_BlockLayout_BlockColorAccentType3A: lighten(@sapUiAccent3, 49);
@_sap_ui_layout_BlockLayout_BlockColorAccentType3B: lighten(@sapUiAccent3, 47);
@_sap_ui_layout_BlockLayout_BlockColorAccentType3C: lighten(@sapUiAccent3, 45);
@_sap_ui_layout_BlockLayout_BlockColorAccentType3D: lighten(@sapUiAccent3, 43);
@_sap_ui_layout_BlockLayout_BlockColorAccentType3E: @sapUiAccent3;
@_sap_ui_layout_BlockLayout_BlockColorAccentType3F: darken(@sapUiAccent3, 6);

@_sap_ui_layout_BlockLayout_BlockColorAccentType4A: lighten(@sapUiAccent4, 50);
@_sap_ui_layout_BlockLayout_BlockColorAccentType4B: lighten(@sapUiAccent4, 48);
@_sap_ui_layout_BlockLayout_BlockColorAccentType4C: lighten(@sapUiAccent4, 46);
@_sap_ui_layout_BlockLayout_BlockColorAccentType4D: lighten(@sapUiAccent4, 44);
@_sap_ui_layout_BlockLayout_BlockColorAccentType4E: @sapUiAccent4;
@_sap_ui_layout_BlockLayout_BlockColorAccentType4F: darken(@sapUiAccent4, 6);

@_sap_ui_layout_BlockLayout_BlockColorAccentType5A: lighten(@sapUiAccent5, 35);
@_sap_ui_layout_BlockLayout_BlockColorAccentType5B: lighten(@sapUiAccent5, 33);
@_sap_ui_layout_BlockLayout_BlockColorAccentType5C: lighten(@sapUiAccent5, 31);
@_sap_ui_layout_BlockLayout_BlockColorAccentType5D: lighten(@sapUiAccent5, 29);
@_sap_ui_layout_BlockLayout_BlockColorAccentType5E: @sapUiAccent5;
@_sap_ui_layout_BlockLayout_BlockColorAccentType5F: darken(@sapUiAccent5, 6);

@_sap_ui_layout_BlockLayout_BlockColorAccentType6A: lighten(@sapUiAccent6, 55);
@_sap_ui_layout_BlockLayout_BlockColorAccentType6B: lighten(@sapUiAccent6, 53);
@_sap_ui_layout_BlockLayout_BlockColorAccentType6C: lighten(@sapUiAccent6, 51);
@_sap_ui_layout_BlockLayout_BlockColorAccentType6D: lighten(@sapUiAccent6, 49);
@_sap_ui_layout_BlockLayout_BlockColorAccentType6E: @sapUiAccent6;
@_sap_ui_layout_BlockLayout_BlockColorAccentType6F: darken(@sapUiAccent6, 10);

@_sap_ui_layout_BlockLayout_BlockColorAccentType7A: lighten(@sapUiAccent7, 67);
@_sap_ui_layout_BlockLayout_BlockColorAccentType7B: lighten(@sapUiAccent7, 65);
@_sap_ui_layout_BlockLayout_BlockColorAccentType7C: lighten(@sapUiAccent7, 63);
@_sap_ui_layout_BlockLayout_BlockColorAccentType7D: lighten(@sapUiAccent7, 61);
@_sap_ui_layout_BlockLayout_BlockColorAccentType7E: @sapUiAccent7;
@_sap_ui_layout_BlockLayout_BlockColorAccentType7F: darken(@sapUiAccent7, 3);

@_sap_ui_layout_BlockLayout_BlockColorAccentType8A: lighten(@sapUiAccent8, 64);
@_sap_ui_layout_BlockLayout_BlockColorAccentType8B: lighten(@sapUiAccent8, 62);
@_sap_ui_layout_BlockLayout_BlockColorAccentType8C: lighten(@sapUiAccent8, 60);
@_sap_ui_layout_BlockLayout_BlockColorAccentType8D: lighten(@sapUiAccent8, 58);
@_sap_ui_layout_BlockLayout_BlockColorAccentType8E: darken(@sapUiAccent8, 7);
@_sap_ui_layout_BlockLayout_BlockColorAccentType8F: darken(@sapUiAccent8, 10);

@_sap_ui_layout_BlockLayout_BlockColorAccentType9A: @sapUiGroupContentBackground;
@_sap_ui_layout_BlockLayout_BlockColorAccentType9B: darken(@sapUiGroupContentBackground,2);
@_sap_ui_layout_BlockLayout_BlockColorAccentType9C: darken(@sapUiGroupContentBackground,4);
@_sap_ui_layout_BlockLayout_BlockColorAccentType9D: darken(@sapUiGroupContentBackground,6);
@_sap_ui_layout_BlockLayout_BlockColorAccentType9E: darken(@sapUiGroupContentBackground, 54);
@_sap_ui_layout_BlockLayout_BlockColorAccentType9F: darken(@sapUiGroupContentBackground, 58);

@_sap_ui_layout_BlockLayout_BlockColorAccentType10A: darken(@sapUiGroupContentBackground,8);
@_sap_ui_layout_BlockLayout_BlockColorAccentType10B: darken(@sapUiGroupContentBackground,10);
@_sap_ui_layout_BlockLayout_BlockColorAccentType10C: darken(@sapUiGroupContentBackground,12);
@_sap_ui_layout_BlockLayout_BlockColorAccentType10D: darken(@sapUiGroupContentBackground,14);
@_sap_ui_layout_BlockLayout_BlockColorAccentType10E: darken(@sapUiGroupContentBackground, 60);
@_sap_ui_layout_BlockLayout_BlockColorAccentType10F: darken(@sapUiGroupContentBackground, 64);

@_sap_ui_layout_BlockLayout_BlockColorAccentType11A: lighten(@sapUiHighlight, 65);
@_sap_ui_layout_BlockLayout_BlockColorAccentType11B: lighten(@sapUiHighlight, 63);
@_sap_ui_layout_BlockLayout_BlockColorAccentType11C: lighten(@sapUiHighlight, 61);
@_sap_ui_layout_BlockLayout_BlockColorAccentType11D: lighten(@sapUiHighlight, 59);
@_sap_ui_layout_BlockLayout_BlockColorAccentType11E: lighten(@sapUiHighlight, 7);
@_sap_ui_layout_BlockLayout_BlockColorAccentType11F: darken(@sapUiHighlight, 2);

@_sap_ui_layout_BlockLayout_BackgroundDashboardMargin: 0.5rem;
@_sap_ui_layout_BlockLayout_BackgroundDashboardEdgeChildMargin: 1rem;
@_sap_ui_layout_BlockLayout_BackgroundDashboardAfterScrollbarMargin: 0.75rem;

@_sap_ui_layout_BlockLayout_LinkTextShadow: none;

@_sap_ui_layout_BlockLayout_BackgroundDashboardColor: @sapUiTileBackground;
@_sap_ui_layout_BlockLayout_BackgroundDashboardBorderRadius: 0.25rem;

// Create classes which would just set background color of the BlockLayout cells
.generateCellColorSets (@backgroundColorSet, @backgroundColorShade) {
	@css-param: "@{backgroundColorSet}@{backgroundColorShade}"; // Concat the params
	@css-selector: ~"sapUiBlockLayoutCellColor@{css-param}"; // Create the css selector string
	@color-param: "_sap_ui_layout_BlockLayout_BlockColorAccentType@{css-param}"; // Build the color param name

	// Build the real CSS selector.
	// The output would be something like this: .sapUiBlockLayoutCellColor1A { background-color: "#996b00"; }
	.@{css-selector} {
		background-color: @@color-param;

		.sapUiBlockCellContent {
			color: contrast(@@color-param, @sapUiBaseText, @sapUiContentContrastTextColor, @sapUiContentContrastTextThreshold);

			.sapMText,
			.sapMLabel,
			.sapUiBlockCellTitle{
				color: inherit;
			}
		}
	}
	.sapContrast.@{css-selector},
	.sapContrast .@{css-selector},
	.sapContrastPlus.@{css-selector},
	.sapContrastPlus .@{css-selector} {
		background-color: inherit; // Read from the

		.sapUiBlockCellContent {
			color: inherit;
		}
	}
}

// Define a loop and go trough 11 color sets and for each set apply 4 or 6 shades (depending on the theme)
.loop-cell-colors(@i: 1) when (@i =< 11) {
	.generateCellColorSets(@i, "A");
	.generateCellColorSets(@i, "B");
	.generateCellColorSets(@i, "C");
	.generateCellColorSets(@i, "D");
	.generateCellColorSets(@i, "E"); // only for Fiori 3 theme
	.generateCellColorSets(@i, "F"); // only for Fiori 3 theme

	.loop-cell-colors(@i + 1); // Execute next loop iteration
}

//Execute Loop through color sets
.loop-cell-colors(1);

/**
 * Mixin to set contrast containers for Light background
 */
.setLightContrastContainers (@evenCell, @oddCell) {
	// Match every even cell
	.sapUiBlockLayoutCell.sapContrast:nth-child(2n),
	.sapUiBlockLayoutCell.sapContrastPlus:nth-child(2n) {
		.sapUiBlockCellContent {
			background-color: @oddCell;
		}
	}

	.sapUiBlockLayoutCell.sapContrast:nth-child(2n+1),
	.sapUiBlockLayoutCell.sapContrastPlus:nth-child(2n+1) {
		.sapUiBlockCellContent {
			background-color: @evenCell;
		}
	}
}

//Link as title
.sapUiBlockCellTitle .sapMLnk {color: darken(@sapUiLink, 4);}
.sapUiBlockCellTitle .sapMLnk:visited {color: darken(@sapUiLinkVisited, 4);}
.sapUiBlockCellTitle .sapMLnk:hover:not(.sapMLnkDsbl) {color: darken(@sapUiLinkHover, 4);}
// disabled and emphasized links: we do not support this at all here.

/* ============ Background type: Light ============== */
.sapUiBlockLayoutBackgroundLight {
	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet1,
	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet3 {
		.setCellsColorDoubles(lighten(@sapUiHighlight, 61), lighten(@sapUiHighlight, 65));
		.setLightContrastContainers(lighten(@sapUiHighlight, 61), lighten(@sapUiHighlight, 65));
	}

	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet1Inverted,
	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet3Inverted {
		.setCellsColorDoubles(lighten(@sapUiHighlight, 65), lighten(@sapUiHighlight, 61));
		.setLightContrastContainers(lighten(@sapUiHighlight, 65), lighten(@sapUiHighlight, 61));
	}

	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet2,
	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet4 {
		.setCellsColorDoubles(lighten(@sapUiHighlight, 63), lighten(@sapUiHighlight, 59));
		.setLightContrastContainers(lighten(@sapUiHighlight, 63), lighten(@sapUiHighlight, 59));
	}

	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet2Inverted,
	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet4Inverted {
		.setCellsColorDoubles(lighten(@sapUiHighlight, 59), lighten(@sapUiHighlight, 63));
		.setLightContrastContainers(lighten(@sapUiHighlight, 59), lighten(@sapUiHighlight, 63));
	}
}

/* ============ Background type: Mixed  - Deprecated as of version 1.50 ============== */
.sapUiBlockLayoutBackgroundMixed {
	.sapUiBlockLayoutBackgroundLight; // Use the same rules as in sapUiBlockLayoutBackgroundLight

	&.sapUiBlockLayoutSizeL, &.sapUiBlockLayoutSizeXL {
		.sapUiBlockLayoutRow {
			.sapContrast, .sapContrastPlus {
				&.sapUiBlockLayoutCell {
					background-color: darken(@sapUiBaseColor, 12);
				}

				&.sapUiBlockLayoutCell.sapUiBlockLayoutBackgroundContrast2 {
					background-color: darken(@sapUiBaseColor, 4);
				}
			}
		}
	}
}

/* ============ Background type: Accent ============== */
.sapUiBlockLayoutBackgroundAccent {
	.sapUiBlockCellContent,
	.sapUiBlockCellTitle,
	.sapUiBlockCellContent .sapMText {
		color: @sapUiContentContrastTextColor;
	}

	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet1 {
		.setCellsColorDoubles(darken(@sapUiAccent7, 8), darken(@sapUiAccent7, 11)); // Colors A & B
	}

	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet1Inverted {
		.setCellsColorDoubles(darken(@sapUiAccent7, 11), darken(@sapUiAccent7, 8)); // Colors B & A
	}

	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet2 {
		.setCellsColorDoubles(darken(@sapUiAccent6, 13), darken(@sapUiAccent6, 26)); // Colors C & D
	}

	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet2Inverted {
		.setCellsColorDoubles(darken(@sapUiAccent6, 26), darken(@sapUiAccent6, 13)); // Colors D & C
	}

	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet3 {
		.setCellsColorDoubles(@sapUiAccent5, darken(@sapUiAccent5, 6)); // Colors E & F
	}

	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet3Inverted {
		.setCellsColorDoubles(darken(@sapUiAccent5, 6), @sapUiAccent5); // Colors F & E
	}

	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet4 {
		.setCellsColorDoubles(darken(@sapUiGroupContentBackground, 2), darken(@sapUiGroupContentBackground, 0)); // Colors G & H
	}

	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet4Inverted {
		.setCellsColorDoubles(darken(@sapUiGroupContentBackground, 0), darken(@sapUiGroupContentBackground, 2)); // Colors H & G
	}

	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet1,
	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet2,
	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet3,
	.sapUiBlockLayoutRow.sapUiBlockLayoutBackgroundColorSet4 {
		.sapUiBlockLayoutCell.sapUiBlockLayoutBackgroundColorSetGray1 {
			background-color: darken(@sapUiGroupContentBackground, 0); //color: H
		}

		.sapUiBlockLayoutCell.sapUiBlockLayoutBackgroundColorSetGray2 {
			background-color: darken(@sapUiGroupContentBackground, 2); //color: G
		}
	}

	.sapUiBlockLayoutBackgroundColorSet4,
	.sapUiBlockLayoutBackgroundColorSet4Inverted,
	.sapUiBlockLayoutBackgroundColorSetGray1,
	.sapUiBlockLayoutBackgroundColorSetGray2 {
		.sapUiBlockCellContent,
		.sapUiBlockCellTitle,
		.sapUiBlockCellContent .sapMText {
			color: @sapUiGroupTitleTextColor;
		}
	}
}
/* END "BlockLayout.less" */
