// Author: Sergey Linev

/*************************************************************************
 * Copyright (C) 1995-2022, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifdef __CLING__

#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;

#pragma link C++ class ROOT::RGeomNodeBase+;
#pragma link C++ class ROOT::RGeomNode+;
#pragma link C++ class ROOT::RGeoItem+;
#pragma link C++ class ROOT::RGeomVisible+;
#pragma link C++ class ROOT::RGeomRenderInfo+;
#pragma link C++ class ROOT::RGeomRawRenderInfo+;
#pragma link C++ class ROOT::RGeomShapeRenderInfo+;
#pragma link C++ class ROOT::RGeomDescription+;
#pragma link C++ class ROOT::RGeomDrawing+;
#pragma link C++ class ROOT::RGeomNodeInfo+;
#pragma link C++ class ROOT::RGeomConfig+;
#pragma link C++ class ROOT::RGeoPainter+;
#pragma link C++ class ROOT::RGeomHierarchy+;
#pragma link C++ class ROOT::RGeomViewer+;

#endif
