/*************************************************************************
 * Copyright (C) 1995-2021, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * Author: Danilo Piparo - CERN                                          *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifdef __CLING__

#pragma link C++ namespace ROOT::VecOps;

#pragma link C++ class ROOT::Internal::VecOps::SmallVectorBase-;

#pragma link C++ class ROOT::VecOps::RVec<bool>-;

#pragma link C++ class ROOT::VecOps::RVec<float>-;
#pragma link C++ class ROOT::VecOps::RVec<double>-;

#pragma link C++ class ROOT::VecOps::RVec<char>-;
#pragma link C++ class ROOT::VecOps::RVec<short>-;
#pragma link C++ class ROOT::VecOps::RVec<int>-;
#pragma link C++ class ROOT::VecOps::RVec<long>-;
#pragma link C++ class ROOT::VecOps::RVec<long long>-;

#pragma link C++ class ROOT::VecOps::RVec<unsigned char>-;
#pragma link C++ class ROOT::VecOps::RVec<unsigned short>-;
#pragma link C++ class ROOT::VecOps::RVec<unsigned int>-;
#pragma link C++ class ROOT::VecOps::RVec<unsigned long>-;
#pragma link C++ class ROOT::VecOps::RVec<unsigned long long>-;

#pragma link C++ class ROOT::VecOps::RVec<std::string>-;

#endif
