/* NOTE: This file was generated as an optimized version of "library.source.less" for the Theme Designer. */

/*!
 * OpenUI5
 * (c) Copyright 2009-2025 SAP SE or an SAP affiliate company.
 * Licensed under the Apache License, Version 2.0 - see LICENSE.txt.
 */

@import "../../../../../../Base/baseLib/baseTheme/base.less"; /* ORIGINAL IMPORT PATH: "../../../../../sap/ui/core/themes/base/base.less" */

@import "../../../../../sap/ui/core/themes/base/global.less";

/* START "AlignedFlowLayout.less" */
.sapUiAFLayout {
	position: relative;
	display: flex;
	flex-wrap: wrap;
	margin: 0;
	padding: 0;
}

.sapUiAFLayoutNoContent {
	justify-content: flex-end;
}

.sapUiAFLayoutItem,
.sapUiAFLayoutEnd {
	overflow: hidden;
}

.sapUiAFLayoutItem {
	flex-grow: 1;
	flex-shrink: 0;
	/* text-align: center;  TODO: ? good for galleries, maybe not good for other usages... maybe a property? */
}

.sapUiAFLayoutItem > * {
	max-width: 100%;
}

/* the dummy elements for the last line */
.sapUiAFLayoutSpacer {
	height: 0;
	margin-top: 0 !important; /* override custom CSS added to the sapUiAFLayoutItem class */
	visibility: hidden;
}

.sapUiAFLayoutEnd {
	position: absolute;
	right: 0;
	bottom: 0;
	text-align: end;
	display: inline-block;
}

.sapUiAFLayoutEnd:only-child {
	position: static;
}

.sapUiAFLayoutOneLine > .sapUiAFLayoutSpacer {
	display: none;
}

.sapUiAFLayoutOneLine > .sapUiAFLayoutSpacer:last-child {
	flex-basis: auto !important; /* needs to override inline style */
	flex-grow: 0;
}

/* mixin call for creating container content padding classes (arguments: rootContainer, contentSelector) */
.sapUiContainerContentPadding(~".sapUiAFLayout");

/* END "AlignedFlowLayout.less" */

/* START "ColumnLayout.less" */
/* ================================================ */
/* CSS for control sap.ui.layout/form/ColumnLayout  */
/* Base theme                                       */
/* ================================================ */

@_sap_ui_layout_ColumnLayout_formColumnMaxXL: 6;
@_sap_ui_layout_ColumnLayout_formColumnMaxL: 4;
@_sap_ui_layout_ColumnLayout_formColumnMaxM: 3;
@_sap_ui_layout_ColumnLayout_formColumnMaxS: 1;
@_sap_ui_layout_ColumnLayout_formColumnElementCols: 12;
@_sap_ui_layout_ColumnLayout_formColumnElementOneCol: 100% / @_sap_ui_layout_ColumnLayout_formColumnElementCols;

.sapUiFormCL{
	white-space: initial; /* eg, if in Splitter where whit-space is set to nowrap */
}

/* Title */
.sapUiFormCL > .sapUiFormTitle{
	box-sizing: border-box;
	padding: 0 1rem;
}

.sapUiFormCLContainer > .sapUiFormTitle{
	padding: 0 0.25rem;
}

.sapUiFormCL > .sapUiFormTitle > img,
.sapUiFormCLContainer > .sapUiFormTitle > img,
.sapUiFormCL > .sapUiFormTitle > span,
.sapUiFormCLContainer > .sapUiFormTitle > span{
	margin-right: 0.5rem;
}

/* Toolbar */
.sapUiFormCL.sapUiFormToolbar > div:first-child {
	height: 3rem; /* to have always 3rem on Form header */
	line-height: 3rem;
	padding: 0 0.75rem;
}

/* Containers */
.sapUiFormCLContainer{
	min-height: 1px;
	overflow: hidden;
	padding: 1rem 0.75rem;
	box-sizing: border-box;
}

/* on real phones use smaller space between containers */
html.sap-phone .sapUiFormCL > .sapUiFormCLContent > .sapUiFormCLContainer:not(:first-child){
	padding-top: 0.5rem;
}

html.sap-phone .sapUiFormCL > .sapUiFormCLContent > .sapUiFormCLContainer:not(:last-child){
	padding-bottom: 0.5rem;
}

/* In Cozy mode put first FormContainers with title or toolbar 0.5rem on top  otherwise 1rem*/
.sapUiFormCL > .sapUiFormContainerTitle,
.sapUiFormCL > .sapUiFormContainerToolbar,
.sapUiFormCLMedia-Std-Phone > .sapUiFormCLContent > .sapUiFormCLContainerSFirstRow.sapUiFormContainerTitle,
.sapUiFormCLMedia-Std-Phone > .sapUiFormCLContent > .sapUiFormCLContainerSFirstRow.sapUiFormContainerToolbar,
.sapUiFormCLMedia-Std-Tablet > .sapUiFormCLContent > .sapUiFormCLContainerMFirstRow.sapUiFormContainerTitle,
.sapUiFormCLMedia-Std-Tablet > .sapUiFormCLContent > .sapUiFormCLContainerMFirstRow.sapUiFormContainerToolbar,
.sapUiFormCLMedia-Std-Desktop > .sapUiFormCLContent > .sapUiFormCLContainerLFirstRow.sapUiFormContainerTitle,
.sapUiFormCLMedia-Std-Desktop > .sapUiFormCLContent > .sapUiFormCLContainerLFirstRow.sapUiFormContainerToolbar,
.sapUiFormCLMedia-Std-LargeDesktop > .sapUiFormCLContent > .sapUiFormCLContainerXLFirstRow.sapUiFormContainerTitle,
.sapUiFormCLMedia-Std-LargeDesktop > .sapUiFormCLContent > .sapUiFormCLContainerXLFirstRow.sapUiFormContainerToolbar{
	padding-top: 0.5rem;
}

.sapUiSizeCompact .sapUiFormCL > .sapUiFormContainerTitle,
.sapUiSizeCompact .sapUiFormCL > .sapUiFormContainerToolbar,
.sapUiSizeCompact .sapUiFormCLMedia-Std-Phone > .sapUiFormCLContent > .sapUiFormCLContainerSFirstRow.sapUiFormContainerTitle,
.sapUiSizeCompact .sapUiFormCLMedia-Std-Phone > .sapUiFormCLContent > .sapUiFormCLContainerSFirstRow.sapUiFormContainerToolbar,
.sapUiSizeCompact .sapUiFormCLMedia-Std-Tablet > .sapUiFormCLContent > .sapUiFormCLContainerMFirstRow.sapUiFormContainerTitle,
.sapUiSizeCompact .sapUiFormCLMedia-Std-Tablet > .sapUiFormCLContent > .sapUiFormCLContainerMFirstRow.sapUiFormContainerToolbar,
.sapUiSizeCompact .sapUiFormCLMedia-Std-Desktop > .sapUiFormCLContent > .sapUiFormCLContainerLFirstRow.sapUiFormContainerTitle,
.sapUiSizeCompact .sapUiFormCLMedia-Std-Desktop > .sapUiFormCLContent > .sapUiFormCLContainerLFirstRow.sapUiFormContainerToolbar,
.sapUiSizeCompact .sapUiFormCLMedia-Std-LargeDesktop > .sapUiFormCLContent > .sapUiFormCLContainerXLFirstRow.sapUiFormContainerTitle,
.sapUiSizeCompact .sapUiFormCLMedia-Std-LargeDesktop > .sapUiFormCLContent > .sapUiFormCLContainerXLFirstRow.sapUiFormContainerToolbar{
	padding-top: 1rem;
}

/* Container content */
.sapUiFormCLContainer > .sapUiFormCLContainerCont{
	column-gap: 1.5rem;
	font-size: 0; /* otherwise there will be a spacing between elements */
}

.sapUiFormCLContainerColl > .sapUiFormCLContainerCont{
	display: none;
}

/* Stand Alone Container */
.sapUiFormCL > .sapUiFormCLContainer{
	width: 100%;
}

/* More containers */
.sapUiFormCLContent > .sapUiFormCLContainer{
	width: 100%; /* default, overwritten by precise width*/
	float: left;
}

/* do not set width to 100% and column-count to 1 as this is default */
.createContainerWidth(@used, @max, @size, @name)  when (@used > 0) {
	.createContainerWidth((@used - 1), @max, @size, @name);
	.sapUiFormCLMedia-Std-@{size} > .sapUiFormCLContent.sapUiFormCLColumns@{name}@{max} > .sapUiFormCLContainer@{name}@{used}{
		width: 100% / @max * @used;
	}
}

.createContainerRules(@max, @size, @name)  when (@max > 1) {
	.createContainerRules((@max - 1), @size, @name);
	.createContainerWidth((@max - 1), @max, @size, @name);

	.sapUiFormCLMedia-Std-@{size} > .sapUiFormCLContainer@{name}@{max} > .sapUiFormCLContainerCont,
	.sapUiFormCLMedia-Std-@{size} > .sapUiFormCLContent > .sapUiFormCLContainer@{name}@{max} > .sapUiFormCLContainerCont{
		column-count: @max;
	}
}

/* XL */
.sapUiFormCLMedia-Std-LargeDesktop > .sapUiFormCLContent > .sapUiFormCLContainerXLBreak{
	clear: both;
}

.createContainerRules(@_sap_ui_layout_ColumnLayout_formColumnMaxXL, LargeDesktop, XL);

/* L */
.sapUiFormCLMedia-Std-Desktop > .sapUiFormCLContent > .sapUiFormCLContainerLBreak{
	clear: both;
}

.createContainerRules(@_sap_ui_layout_ColumnLayout_formColumnMaxL, Desktop, L);

/* M */
.sapUiFormCLMedia-Std-Tablet > .sapUiFormCLContent > .sapUiFormCLContainerMBreak{
	clear: both;
}
.createContainerRules(@_sap_ui_layout_ColumnLayout_formColumnMaxM, Tablet, M);

/* S */

/* Elements */
.sapUiFormCLElement{
	width: 100%; /* to use the whole column */
	display: inline-block; /* otherwise it wraps inside element -> label in one column, field in another */
	font-size: @sapUiFontSize; /* restore default font size */
}

.sapUiFormCLContent:after,
.sapUiFormCLElement:after {
/* CLEAR FIX */
	clear: both;
	content: " ";
	display: block;
	font-size: 0;
	height: 0;
	visibility: hidden;
}

.sapUiFormCLElement > * {
	float: left;
	padding: 0 0.25rem;
	box-sizing: border-box;
	overflow: hidden;
}

.createColumnRules(@counter, @size, @name)  when (@counter > 0) {
	.createColumnRules((@counter - 1), @size, @name);
	.sapUiFormCL@{size}Columns .sapUiFormCLElement > .sapUiFormCLCells@{name}@{counter}{
		width: @_sap_ui_layout_ColumnLayout_formColumnElementOneCol * @counter;
	}
	.sapUiFormCL@{size}Columns .sapUiFormCLElement > .sapUiFormCLCell@{name}Space@{counter}{
		margin-left: @_sap_ui_layout_ColumnLayout_formColumnElementOneCol * @counter;
	}
}

/* small columns */
.createColumnRules(@_sap_ui_layout_ColumnLayout_formColumnElementCols, Small, S);

.sapUiFormCLSmallColumns .sapUiFormCLElement > .sapUiFormCLCellSBreak{
	clear: both;
}

/* wide columns - Labels next to field */
.createColumnRules(@_sap_ui_layout_ColumnLayout_formColumnElementCols, Wide, L);

.sapUiFormCLWideColumns .sapUiFormCLElement > .sapUiFormCLCellLBreak{
	clear: both;
}

/* edit mode */
/* labels next to field */
.sapUiFormEdit > .sapUiFormCLSmallColumns .sapUiFormElementLbl:not(.sapUiFormCLCellsS12),
.sapUiFormEdit > .sapUiFormCLWideColumns .sapUiFormElementLbl:not(.sapUiFormCLCellsL12){
	padding-top: 1rem;
}

.sapUiSizeCompact .sapUiFormEdit > .sapUiFormCLSmallColumns .sapUiFormElementLbl:not(.sapUiFormCLCellsS12),
.sapUiSizeCompact .sapUiFormEdit > .sapUiFormCLWideColumns .sapUiFormElementLbl:not(.sapUiFormCLCellsL12){
	padding-top: 0.5rem;
}

/* labels above field */
/* As is is not clear what FormElement will be on top of a column we can not use only rules for first label or field */
.sapUiFormEdit .sapUiFormCLSmallColumns .sapUiFormCLElement{
	padding-top: 0.625rem;
}

.sapUiFormEdit .sapUiFormCLSmallColumns .sapUiFormElementLbl.sapUiFormCLCellsS12,
.sapUiFormEdit .sapUiFormCLWideColumns .sapUiFormElementLbl.sapUiFormCLCellsL12{
	padding-bottom: 0.125rem;
}

.sapUiFormEdit .sapUiFormCLSmallColumns .sapUiFormCLContainer > .sapUiFormTitle,
.sapUiFormEdit .sapUiFormCLSmallColumns .sapUiFormContainerToolbar > div:first-child  {
	margin-bottom: -0.125rem;
}

.sapUiSizeCompact .sapUiFormEdit .sapUiFormCLSmallColumns .sapUiFormCLContainer > .sapUiFormTitle,
.sapUiSizeCompact .sapUiFormEdit .sapUiFormCLSmallColumns .sapUiFormContainerToolbar > div:first-child  {
	margin-bottom: 0.375rem;
}

.sapUiFormEdit .sapUiFormCLSmallColumns .sapUiFormCLContainer:not(.sapUiFormContainerTitle):not(.sapUiFormContainerToolbar) {
	padding-top: 0.375rem;
}

html.sap-phone .sapUiFormEdit .sapUiFormCLSmallColumns .sapUiFormCLContainer:not(.sapUiFormContainerTitle):not(.sapUiFormContainerToolbar){
	padding-top: 0;
	margin-top: -0.125rem;
}

/* Additional styling to prevent rendering issues in Safari in combination with column-count and overflow:hidden */
html[data-sap-ui-browser^="sf"] .sapUiFormEdit .sapUiFormCLElement {
	transform: translateX(0);
}

/* display mode */
/* in display only mode add padding to controls as they don't have one */
.sapUiForm:not(.sapUiFormEdit) .sapUiFormCLElement > *{
	padding-top: 0.25rem;
	padding-bottom: 0.25rem;
	font-size: 1px; /* as browser adds some px to text to meet font-size */
	margin: 0; /* As <dd> would add a margin */
}

.sapUiForm:not(.sapUiFormEdit) .sapUiFormCLElement > * .sapUiIcon:not(.sapMBtnIcon),
.sapUiForm:not(.sapUiFormEdit) .sapUiFormCLElement > * .sapMCbBg{
	font-size: @sapUiFontSize; /* to set default font size on icons */
}

/* labels above field */
.sapUiForm:not(.sapUiFormEdit) .sapUiFormCLSmallColumns .sapUiFormCLElement{
	padding-top: 0.5rem;
}

.sapUiForm:not(.sapUiFormEdit) .sapUiFormCLSmallColumns .sapUiFormElementLbl.sapUiFormCLCellsS12,
.sapUiForm:not(.sapUiFormEdit) .sapUiFormCLWideColumns .sapUiFormElementLbl.sapUiFormCLCellsL12{
	padding-bottom: 0;
}

.sapUiForm:not(.sapUiFormEdit) .sapUiFormCLSmallColumns .sapUiFormCLContainer > .sapUiFormTitle,
.sapUiForm:not(.sapUiFormEdit) .sapUiFormCLSmallColumns .sapUiFormContainerToolbar > div:first-child  {
	margin-bottom: -0.25rem;
}

.sapUiSizeCompact .sapUiForm:not(.sapUiFormEdit) .sapUiFormCLSmallColumns .sapUiFormCLContainer > .sapUiFormTitle,
.sapUiSizeCompact .sapUiForm:not(.sapUiFormEdit) .sapUiFormCLSmallColumns .sapUiFormContainerToolbar > div:first-child  {
	margin-bottom: 0.25rem;
}

.sapUiForm:not(.sapUiFormEdit) .sapUiFormCLSmallColumns .sapUiFormCLContainer:not(.sapUiFormContainerTitle):not(.sapUiFormContainerToolbar) {
	padding-top: 0.25rem;
}

html.sap-phone .sapUiForm:not(.sapUiFormEdit) .sapUiFormCLSmallColumns .sapUiFormCLContainer:not(.sapUiFormContainerTitle):not(.sapUiFormContainerToolbar){
	padding-top: 0;
	margin-top: -0.25rem;
}

/* CHROME Dummy */
.sapUiFormCLElementDummy {
	visibility: hidden;
	width: 100%;
	display: inline-block;
}
/* END "ColumnLayout.less" */

/* START "DynamicSideContent.less" */
/* ================================================= */
/* CSS for control sap.ui.layout/DynamicSideContent  */
/* Base theme                                        */
/* ================================================= */

.sapUiDSC {
	.sapUiDSCM:not(.sapUiHidden),
	.sapUiDSCS:not(.sapUiHidden) {
		position: relative;
	}
	&:after {
		clear: both;
		content: " ";
		display: block;
		font-size: 0;
		height: 0;
		visibility: hidden;
	}

	> div[class^="sapUiDSCSpan"] {
		float: left;
		overflow: auto;
	}

	> aside[class^="sapUiDSCSpan"] {
		float: left;
		overflow: auto;
	}

	> div.sapUiDSCMCFixed {
		width: ~"calc(100% - 340px)";
		float: left;
	}

	> aside.sapUiDSCSCFixed,
	> div.sapUiDSCSCFixed {
		width: 340px;
		float: left;
	}

	> .sapUiDSCSpan1 {
		width: 8.333%;
	}

	> .sapUiDSCSpan2 {
		width: 16.666%;
	}

	> .sapUiDSCSpan3 {
		width: 25%;
	}

	> .sapUiDSCSpan4 {
		width: 33.333%;
	}

	> .sapUiDSCSpan5 {
		width: 41.666%;
	}

	> .sapUiDSCSpan6 {
		width: 50%;
	}

	> .sapUiDSCSpan7 {
		width: 58.333%;
	}

	> .sapUiDSCSpan8 {
		width: 66.666%;
	}

	> .sapUiDSCSpan9 {
		width: 75%;
	}

	> .sapUiDSCSpan10 {
		width: 83.333%;
	}

	> .sapUiDSCSpan11 {
		width: 91.666%;
	}

	> .sapUiDSCSpan12 {
		width: 100%;
	}
}

/* mixin call for creating container content padding classes (arguments: rootContainer, mainContent) */
.sapUiContainerContentPadding(~".sapUiDSC", ~"> div:first-child");
.sapUiContainerContentPadding(~".sapUiDSC", ~"> div:last-child");

/* mixin call for creating container content padding classes (arguments: rootContainer, sideContent) */
.sapUiContainerContentPadding(~".sapUiDSC", ~"> aside");
/* END "DynamicSideContent.less" */

/* START "FixFlex.less" */
/* ====================================== */
/* CSS for control sap.ui.layout/FixFlex  */
/* Base theme                             */
/* ====================================== */

/* Class for vertical (default) layout
   ========================================================================== */
.sapUiFixFlex {
	box-sizing: border-box;

	display: -webkit-box;
	display: -webkit-flex;
	display: flex;

	-webkit-box-direction: normal;
	-webkit-box-orient: vertical;
	-webkit-flex-direction: column;
	flex-direction: column;

	-webkit-box-align: start;
	-webkit-align-items: flex-start;
	align-items: flex-start;

	height: 100%;
}

/* Flexible child */
.sapUiFixFlexFlexible {
	box-sizing: border-box;

	/* Fixes bug in Chrome 48 where flexbox is broken
		https://github.com/angular/material/issues/6841 */
	min-height: 0;
	min-width: 0;

	-webkit-box-flex: 1;
	-webkit-flex: 1;
	flex: 1;

	overflow: hidden;
	position: relative;
	width: 100%;
}

.sapUiFixFlexFlexibleContainer {
	position: absolute;
	top: 0;
	right: 0;
	bottom: 0;
	left: 0;

	& > .sapUiFixFlex {
		position: absolute;
		top: 0;
		right: 0;
		bottom: 0;
		left: 0;
	}
}

/* Fixed child */
.sapUiFixFlexFixed {
	box-sizing: border-box;

	position: relative;
	overflow: hidden;
	width: 100%;
}

/* Class for horizontal layout
   ========================================================================== */

.sapUiFixFlexRow {
	-webkit-box-direction: normal;
	-webkit-box-orient: horizontal;
	-webkit-flex-direction: row;
	flex-direction: row;

	& > .sapUiFixFlexFlexible,
	& > .sapUiFixFlexFixed {
		width: auto;
		height: 100%;
	}
}

/* Styles for scrolling
   ========================================================================== */
.sapUiFixFlexInnerScrolling {
	& > .sapUiFixFlexFlexible {
		-webkit-overflow-scrolling: touch;
		overflow: auto;

		.sapUiFixFlexFlexibleContainerGrowing {
			overflow-y: visible;
			width: 100%;
			position: static;
		}
	}

	&.sapUiFixFlexRow {
		& > .sapUiFixFlexFlexible {
			overflow-y: hidden;
		}
	}
}

.sapUiFixFlexScrolling {
	-webkit-overflow-scrolling: touch;
	display: block;

	& > .sapUiFixFlexFlexible {
		display: block;

		& > .sapUiFixFlexFlexibleContainer {
			position: static;
			height: 100%;
		}
	}

	&.sapUiFixFlexRow {
		white-space: nowrap;
		& > .sapUiFixFlexFlexible,
		& > .sapUiFixFlexFixed {
			display: inline-block;
		}
	}
}

/* END "FixFlex.less" */

/* START "Form.less" */
/* =================================== */
/* CSS for control sap.ui.layout/Form  */
/* Base theme                          */
/* =================================== */

@_sap_ui_layout_Form_FormContainerExpImageURL: url("sap-icon://slim-arrow-right");
@_sap_ui_layout_Form_FormContainerColImageURL: url("sap-icon://slim-arrow-down");

html.sap-tablet .sapUiFioriObjectPage > section > .sapUiForm,
html.sap-desktop .sapUiFioriObjectPage > section > .sapUiForm {
	margin: 1rem 0;
	padding: 0 1rem;
	box-sizing: border-box;
}

.sapUiFormElementLbl {
	font-size: 1px; /* To prevent larger DIV because of invisible strut character*/
}
/* END "Form.less" */

/* START "FormLayout.less" */
/* ========================================= */
/* CSS for control sap.ui.layout/FormLayout  */
/* Base theme                                */
/* ========================================= */

@_sap_ui_layout_FormLayout_FormTitleSize: ~"H4";
@_sap_ui_layout_FormLayout_FormSubTitleSize: ~"H5";

.sapUiFormLayout {
	border: 1px solid @sapUiGroupTitleBorderColor;
	font-size: @sapUiDesktopFontSize;
}

.sapUiFormLayout > .sapUiFormContainer {
	display: block;
}

.sapUiFormTitle {
	border-bottom: 1px solid @sapUiGroupTitleBorderColor;
	overflow: hidden;
	width: 100%;
	font-synthesis: none;
}

/* Text part */
.sapUiFormTitleH1,
.sapUiFormTitleH2,
.sapUiFormTitleH3,
.sapUiFormTitleH4,
.sapUiFormTitleH5,
.sapUiFormTitleH6 {
	overflow: hidden;
	white-space: nowrap;
	text-overflow: ellipsis;
	display: block;
	width: 100%;
	font-weight: @sapUiFontHeaderWeight;
	color: @sapUiGroupTitleTextColor;
	margin: 0;
}

.sapUiForm .sapUiFormTitle {
	height: 3rem;
	line-height: 3rem;
}

.sapUiFormContainerTitle > .sapUiFormTitle {
	position: relative;
	border: none;
	box-sizing: border-box;
	margin-bottom: 0.5rem;
	height: 2rem;
	line-height: 2rem;
}

.sapUiFormContainerTitle > .sapUiFormTitle.sapUiFormTitleExpandable{
	overflow: visible; /* to allow to put expander outside */
	padding-left: 2.5rem;
}

.sapUiFormContainerTitle > .sapUiFormTitle.sapUiFormTitleExpandable > button{
	position: absolute;
	left: -0.25rem;
	padding-right: 0.5rem;
}

.sapUiFormTitle.sapUiFormTitleEmph {
	border-color: @sapUiGroupTitleBorderColor;
}

/* Title levels - can not use TextView CSS from commons, because also used in mobile */
.sapUiFormTitleH1 {
	font-size: @sapMFontHeader1Size;
	line-height: 122%;
}

.sapUiFormTitleH2 {
	font-size: @sapMFontHeader2Size;
	line-height: 122%;
}

.sapUiFormTitleH3 {
	font-size: @sapMFontHeader3Size;
	line-height: 122%;
}

.sapUiFormTitleH4 {
	font-size: @sapMFontHeader4Size;
}

.sapUiFormTitleH5 {
	font-size: @sapMFontHeader5Size;
}

.sapUiFormTitleH6 {
	font-size: @sapMFontHeader6Size;
	color: @sapUiGroupTitleTextColor;
}

.sapUiFormTitle> img {
	vertical-align: middle;
}

/* Toolbar */
.sapUiFormContainerToolbar > div:first-child {
	margin-bottom: 0.5rem;
}

.sapUiFormToolbar > div:first-child >.sapMTitle.sapMTitleStyleAuto{
	font-size: @sapMFontHeader4Size;
}

.sapUiFormContainerToolbar > div:first-child >.sapMTitle.sapMTitleStyleAuto{
	font-size: @sapMFontHeader5Size;
}

.sapUiFormM .sapUiFormContainerTitle > .sapUiFormTitle {
	height: 3rem;
	line-height: 3rem;
}

.sapUiSizeCompact .sapUiFormContainerTitle > .sapUiFormTitle {
	height: 2rem;
	line-height: 2rem;
}

/* SemanticFormElement */
.sapUiForm .sapUiFormDelimiter { /* add Form class to make sure to overwrite sapMText setting */
	white-space: pre-wrap; /* To render whitespaces in display mode if controls are rendered */
}
/* END "FormLayout.less" */

/* START "Grid.less" */
/* UI5 RESPONSIVE GRID */
// Variables
@_sap_ui_layout_Grid_RespGridCols: 12;
@_sap_ui_layout_Grid_RespGridCols-1: @_sap_ui_layout_Grid_RespGridCols - 1;
@_sap_ui_layout_Grid_RespGridOneCol: 100% / @_sap_ui_layout_Grid_RespGridCols;
@_sap_ui_layout_Grid_RespGridMediaPhone: sapUiRespGridMedia-Std-Phone;
@_sap_ui_layout_Grid_RespGridMediaTablet: sapUiRespGridMedia-Std-Tablet;
@_sap_ui_layout_Grid_RespGridMediaDesktop: sapUiRespGridMedia-Std-Desktop;
@_sap_ui_layout_Grid_RespGridMediaXL: sapUiRespGridMedia-Std-LargeDesktop;
@_sap_ui_layout_Grid_RespGridMediaPhoneSize: 320px;
@_sap_ui_layout_Grid_RespGridMediaTabletSize: 600px;
@_sap_ui_layout_Grid_RespGridMediaDesktopSize: 1024px;
@_sap_ui_layout_Grid_RespGridMediaXLSize: 1440px;
@_sap_ui_layout_Grid_RespGridPxInRem: 16;
@_sap_ui_layout_Grid_RespGridVisible: sapUiRespGridVisible;
@_sap_ui_layout_Grid_RespGridHidden: sapUiRespGridHidden;

div[class*="sapUiRespGrid"] > div[class*="sapUiRespGridSpan"] {
    float: left;
	min-height: 1px; // BCP: 1770523047
}

div.sapUiRespGridOverflowHidden[class*="sapUiRespGrid"] > div[class*="sapUiRespGridSpan"] {
	overflow: hidden;
}

/* CLEAR FIX */

div[class*="sapUiRespGrid"]:after {
    clear: both;
    content: " ";
    display: block;
    font-size: 0;
    height: 0;
    visibility: hidden;
}

/* GRID HORIZONTAL SPACING - 0.5rem */

.sapUiRespGrid.sapUiRespGridHSpace05 {
    padding: 0 0.25rem;
}

.sapUiRespGrid.sapUiRespGridHSpace05 > div {
    margin: 0 0.25rem;
}

/* GRID HORIZONTAL SPACING - 1rem */

.sapUiRespGrid.sapUiRespGridHSpace1 {
    padding: 0 0.5rem;
}

.sapUiRespGrid.sapUiRespGridHSpace1 > div {
    margin: 0 0.5rem;
}

/* GRID HORIZONTAL SPACING - 2rem */

.sapUiRespGrid.sapUiRespGridHSpace2 {
    padding: 0 1rem;
}

.sapUiRespGrid.sapUiRespGridHSpace2 > div {
    margin: 0 1rem;
}

/* GRID VERTICAL SPACING - general */

.sapUiRespGrid.sapUiRespGridVSpace0 > div {
    margin-bottom: 0;
}

.sapUiRespGrid.sapUiRespGridVSpace05 > div {
    margin-bottom: 0.5rem;
}

.sapUiRespGrid.sapUiRespGridVSpace1 > div {
    margin-bottom: 1rem;
}

.sapUiRespGrid.sapUiRespGridVSpace2 > div {
    margin-bottom: 2rem;
}

.sapUiRespGrid[class*="sapUiRespGridVSpace0"] > .sapUiRespGridSpanInvisible,
.sapUiRespGrid[class*="sapUiRespGridVSpace05"] > .sapUiRespGridSpanInvisible,
.sapUiRespGrid[class*="sapUiRespGridVSpace1"] > .sapUiRespGridSpanInvisible,
.sapUiRespGrid[class*="sapUiRespGridVSpace2"] > .sapUiRespGridSpanInvisible {
	margin-bottom: 0;
}

/* GENERAL BREAK */

.sapUiRespGridBreak { clear: both; }

.sapUiRespGridWriteSpanClass(@classFor, @hspace, @span, @screenSize) when (@classFor = Span) {
    @a:  @_sap_ui_layout_Grid_RespGridOneCol * @span;
    width: @a - 100% * @_sap_ui_layout_Grid_RespGridPxInRem * @hspace / @screenSize;
    .sapUiRespGrid-getCalcFuncSpan(@a, @hspace);
}

.sapUiRespGridWriteIndentClass(@classFor, @hspace, @span, @screenSize) when (@classFor = Indent) {
    @a:  @_sap_ui_layout_Grid_RespGridOneCol * @span;
    margin-left: @a + (100% * @_sap_ui_layout_Grid_RespGridPxInRem * @hspace/2) / @screenSize ;
    .sapUiRespGrid-getCalcFuncIndent(@a, @hspace);
}

// Get the values and how I cheat LESS processor with the commented calc function
.sapUiRespGrid-getCalcFuncSpan(@a, @hspace)  when (@hspace > 0) {
   @c: "calc(";
   @b: " - ";
   @d: "-webkit-calc(";
   @e: "rem)";
   @par1: ~"@{d}@{a}@{b}@{hspace}@{e}";
   @par2: ~"@{c}@{a}@{b}@{hspace}@{e}";
        width: @par1;
        width: @par2;
}

// Get the values and how I cheat LESS processor with the commented calc function
.sapUiRespGrid-getCalcFuncIndent(@a, @hspace)  when (@hspace > 0) {
   @c: "calc(";
   @b: " + ";
   @d: "-webkit-calc(";
   @e: "rem)";
   @f: @hspace/2;
   @par1: ~"@{d}@{a}@{b}@{f}@{e}";
   @par2: ~"@{c}@{a}@{b}@{f}@{e}";
        margin-left: @par1;
        margin-left: @par2;
}


//Write Span and Indent classes
.sapUiRespGridWriteClasses(@counter, @media, @classFor, @name, @classHSpace, @hspace, @screenSize) when (@counter > 0) {
  .sapUiRespGridWriteClasses((@counter - 1), @media, @classFor, @name, @classHSpace, @hspace, @screenSize);
  .@{media}.sapUiRespGridHSpace@{classHSpace} > .sapUiRespGrid@{classFor}@{name}@{counter} {
      .sapUiRespGridWriteSpanClass(@classFor, @hspace, @counter,  @screenSize);
      .sapUiRespGridWriteIndentClass(@classFor, @hspace, @counter,  @screenSize);
    }
}

// MOVE LEFT
.sapUiRespGridWriteMoveFwd(@counter, @media, @name) when (@counter > 0) {
  .sapUiRespGridWriteMoveFwd((@counter - 1), @media, @name);
  .@{media} > .sapUiRespGridFwd@{name}@{counter} { left: @_sap_ui_layout_Grid_RespGridOneCol * @counter; }
}

// MOVE RIGHT
.sapUiRespGridWriteMoveBwd(@counter, @media, @name) when (@counter > 0) {
  .sapUiRespGridWriteMoveBwd((@counter - 1), @media, @name);
  .@{media} > .sapUiRespGridBwd@{name}@{counter} { right: @_sap_ui_layout_Grid_RespGridOneCol * @counter; }
}

.sapUiRespGridWriteBreak(@media, @name){
  .@{media} > .sapUiRespGridBreak@{name} { clear: both }
}


.sapUiRespGridWriteCSS(@media, @name, @screenSize) {

    /*SPANS */
    .sapUiRespGridWriteClasses(@_sap_ui_layout_Grid_RespGridCols, @media, Span, @name, 0, 0, @screenSize);
    .sapUiRespGridWriteClasses(@_sap_ui_layout_Grid_RespGridCols, @media, Span, @name, ~"05", 0.5, @screenSize);
    .sapUiRespGridWriteClasses(@_sap_ui_layout_Grid_RespGridCols, @media, Span, @name, 1, 1, @screenSize);
    .sapUiRespGridWriteClasses(@_sap_ui_layout_Grid_RespGridCols, @media, Span, @name, 2, 2, @screenSize);

    /*INDENTS */
    .sapUiRespGridWriteClasses(@_sap_ui_layout_Grid_RespGridCols-1, @media, Indent, @name, 0, 0, @screenSize);
    .sapUiRespGridWriteClasses(@_sap_ui_layout_Grid_RespGridCols-1, @media, Indent, @name, ~"05", 0.5, @screenSize);
    .sapUiRespGridWriteClasses(@_sap_ui_layout_Grid_RespGridCols-1, @media, Indent, @name, 1, 1, @screenSize);
    .sapUiRespGridWriteClasses(@_sap_ui_layout_Grid_RespGridCols-1, @media, Indent, @name, 2, 2, @screenSize);

	/* MOVE LEFT */
	.sapUiRespGridWriteMoveFwd(@_sap_ui_layout_Grid_RespGridCols-1, @media, @name);

	/* MOVE RIGHT */
	.sapUiRespGridWriteMoveBwd(@_sap_ui_layout_Grid_RespGridCols-1, @media, @name);

	/* BREAK */
	.sapUiRespGridWriteBreak(@media, @name);
}

/* =============================== */
/*   S M A L L   D E V I C E S     */
/* =============================== */

.sapUiRespGridWriteCSS(@_sap_ui_layout_Grid_RespGridMediaPhone, S, @_sap_ui_layout_Grid_RespGridMediaPhoneSize);
.@{_sap_ui_layout_Grid_RespGridMediaPhone} > .@{_sap_ui_layout_Grid_RespGridHidden}S { display: none }


/* =============================== */
/*   M E D I U M   D E V I C E S   */
/* =============================== */

.sapUiRespGridWriteCSS(@_sap_ui_layout_Grid_RespGridMediaTablet, M, @_sap_ui_layout_Grid_RespGridMediaTabletSize);
.@{_sap_ui_layout_Grid_RespGridMediaTablet} > .@{_sap_ui_layout_Grid_RespGridHidden}M { display: none }


/* ============================= */
/*   L A R G E   D E V I C E S   */
/* ============================= */

.sapUiRespGridWriteCSS(@_sap_ui_layout_Grid_RespGridMediaDesktop, L, @_sap_ui_layout_Grid_RespGridMediaDesktopSize);
.@{_sap_ui_layout_Grid_RespGridMediaDesktop} > .@{_sap_ui_layout_Grid_RespGridHidden}L { display: none }


/* ====================================== */
/*  E X T R A   L A R G E   D E V I C E S */
/* ====================================== */

.sapUiRespGridWriteCSS(@_sap_ui_layout_Grid_RespGridMediaXL, XL, @_sap_ui_layout_Grid_RespGridMediaXLSize);
.@{_sap_ui_layout_Grid_RespGridMediaXL} > .@{_sap_ui_layout_Grid_RespGridHidden}XL { display: none }



/* =============== */
/*   E X T R A S   */
/* =============== */

/* HORIZONTAL POSITIONING GRID IN CONTAINER */

div.sapUiRespGrid.sapUiRespGridPosCenter {
       margin-left: auto;
       margin-right: auto;
}

div.sapUiRespGrid.sapUiRespGridPosRight {
       margin-left: auto;
       margin-right: 0;
}

/* MOVE FOWARD and BACKWARD */

div[class*="sapUiRespGrid"] > div[class*="sapUiRespGridFwd"] {
       position: relative;
}

div[class*="sapUiRespGrid"] > div[class*="sapUiRespGridBwd"] {
       float: right;
       position: relative;
}


/* explanation for percentages for browsers without calc support:
   16px = 1rem
   -------------------------
S  1.25%     =  4px / 320px
   2.5%      =  8px / 320px
   5%        = 16px / 320px
   10%       = 32px / 320px
M  0.16667%  =  4px / 600px
   1.3333%   =  8px / 600px
   2.6667%   = 16px / 600px
   5.3333%   = 32px / 600px
L  0.390625% =  4px / 1024px
   0.78125%  =  8px / 1024px
   1.5625%   = 16px / 1024px
   3.125%    = 32px / 1024px
   ------------------------- */
/* END "Grid.less" */

/* START "CSSGrid.less" */
/* ============================================= */
/* CSS for control sap.ui.layout/cssgrid.CSSGrid */
/* Base theme                                    */
/* ============================================= */

.sapUiLayoutCSSGrid {
	box-sizing: border-box;
}

.sapUiLayoutCSSGridBoxLayoutContainer {
	display: grid;
	grid-auto-rows: 1fr;
	grid-gap: 0.5rem 0.5rem;

	.sapUiLayoutCSSGridBoxLayoutGenerateBreakpoints(S);
	.sapUiLayoutCSSGridBoxLayoutGenerateBreakpoints(M);
	.sapUiLayoutCSSGridBoxLayoutGenerateBreakpoints(L);
	.sapUiLayoutCSSGridBoxLayoutGenerateBreakpoints(XL);
}

.sapUiLayoutCSSGridBoxLayoutContainer.sapFGridListGroup {
	grid-auto-rows: auto;
}

/* Mixins */

// Mixin to generate breakpoints. */
.sapUiLayoutCSSGridBoxLayoutGenerateBreakpoints(@size, @counter: 0, @i: 1) when (@i =< 12) {
	&.sapUiLayoutCSSGridBoxLayoutSize@{size}.sapUiLayoutCSSGridBoxLayoutSpan@{size}@{i} {
		grid-template-columns: repeat(auto-fit, minmax(calc(~"(100% - @{counter} * 0.5rem - 1px) / @{i}"), 1fr));
	}

	.sapUiLayoutCSSGridBoxLayoutGenerateBreakpoints(@size, (@counter + 1), (@i + 1));
}

.sapUiLayoutCSSGrid .sapUiBlockLayerTabbable {
	// There are "span" helpers, which purpose is to block navigation into the blocked controls content.
	// If the focus reaches the span before the blocked/busy control it is forwarded to the one after the control.
	// These spans should not be visible.
	position: absolute;
}

/* ResponsiveColumnLayout */
@_sap_ui_layout_CSSGrid_RCLColumns:
	S 4,
	M 8,
	ML 12,
	L 12,
	XL 16,
	XXL 20,
	XXXL 20;

.sapUiLayoutCSSGridRCL {

	.generateBreakpoint(@i: 1) when (@i <= length(@_sap_ui_layout_CSSGrid_RCLColumns)) {
		@pair: extract(@_sap_ui_layout_CSSGrid_RCLColumns, @i);
		@rangeName: extract(@pair, 1);
		@columns: extract(@pair, 2);
	
		.sapUiLayoutCSSGridRCL-Layout@{rangeName} &,
		.sapUiLayoutCSSGridRCL-Layout@{rangeName}& {
			grid-template-columns: repeat(@columns, minmax(0px, 1fr));
			grid-gap: 1rem;
		}
	
		.generateBreakpoint(@i + 1); // next iteration
	}

	.generateBreakpoint();
}

.sapUiLayoutCSSGridRCL {
	position: relative;
}
/* END "CSSGrid.less" */

/* START "GridLayout.less" */
/* ========================================= */
/* CSS for control sap.ui.layout/GridLayout  */
/* Base theme                                */
/* ========================================= */

.sapUiForm:not(.sapUiFormM) > .sapUiGrid {
	font-size: @sapUiDesktopFontSize;
}

.sapUiGrid td {
	vertical-align: top;
	overflow: hidden;
}

.sapUiGrid td.sapUiGridHeader {
	vertical-align: bottom;
}

.sapUiGrid th > .sapUiFormTitle,
.sapUiGrid td > .sapUiFormTitle {
	text-align: start;
	position: relative;
}

.sapUiGridHeader.sapUiFormContainerTitle > .sapUiFormTitle.sapUiFormTitleExpandable{
	padding-left: 2.25rem;
}

.sapUiGridHeader.sapUiFormContainerTitle > .sapUiFormTitle.sapUiFormTitleExpandable > button{
	left: -0.5rem;
}

/* Toolbar */
.sapUiGrid.sapUiFormToolbar th > div {
	text-align: start;
}

.sapUiGrid > tbody > tr > td {
	padding: 0.25rem 0 0.25rem 0.5rem;
}

.sapUiFormM > .sapUiGrid > tbody > tr > td {
	padding: 0 0 0 0.5rem;
}

.sapUiGrid > tbody > tr > td:first-child {
	padding-left: 1rem;
}

.sapUiGrid > tbody > tr.sapUiGridConteinerFirstRow > td {
	padding-top: 2rem;
}

.sapUiGrid > tbody > tr:first-child > td, /* if no Form title */
.sapUiForm[aria-labelledby] > .sapUiGrid > tbody > tr:nth-child(2) > td {
	padding-top: 1rem;
}

/* In Cozy mode put first FormContainers with title or toolbar 0.5rem on top */
.sapUiFormM > .sapUiGrid > tbody > tr:first-child > td.sapUiFormContainerTitle, /* if no Form title */
.sapUiFormM > .sapUiGrid > tbody > tr:first-child > td.sapUiFormContainerToolbar, /* if no Form title */
.sapUiFormM[aria-labelledby] > .sapUiGrid > tbody > tr:nth-child(2) > td.sapUiFormContainerTitle,
.sapUiFormM[aria-labelledby] > .sapUiGrid > tbody > tr:nth-child(2) > td.sapUiFormContainerToolbar {
	padding-top: 0.75rem;
}

.sapUiSizeCompact .sapUiFormM > .sapUiGrid > tbody > tr:first-child > td.sapUiFormContainerTitle, /* if no Form title */
.sapUiSizeCompact .sapUiFormM > .sapUiGrid > tbody > tr:first-child > td.sapUiFormContainerToolbar, /* if no Form title */
.sapUiSizeCompact .sapUiFormM[aria-labelledby] > .sapUiGrid > tbody > tr:nth-child(2) > td.sapUiFormContainerTitle,
.sapUiSizeCompact .sapUiFormM[aria-labelledby] > .sapUiGrid > tbody > tr:nth-child(2) > td.sapUiFormContainerToolbar {
	padding-top: 1rem;
}

.sapUiGrid > tbody > tr > td:last-child {
	padding-right: 1rem;
}

.sapUiGrid > tbody > tr:last-child > td {
	padding-bottom: 1rem;
}

.sapUiFormM > .sapUiGrid > tbody > tr:last-child > td {
	padding-bottom: 0.75rem;
}

.sapUiGrid > tbody > tr > td.sapUiGridLabelFull {
	padding-bottom: 0.125rem;
}

.sapUiFormEdit.sapUiFormM > .sapUiGrid > tbody > tr > td.sapUiFormElementLbl {
	padding-top: 1rem;
}

.sapUiFormEdit.sapUiFormM > .sapUiGrid > tbody > tr.sapUiGridConteinerFirstRow > td.sapUiFormElementLbl {
	padding-top: 3rem;
}

.sapUiFormEdit.sapUiFormM >.sapUiGrid > tbody > tr:first-child > td.sapUiFormElementLbl, /* if no Form title */
.sapUiFormEdit[aria-labelledby].sapUiFormM > .sapUiGrid > tbody > tr:nth-child(2) > td.sapUiFormElementLbl {
	padding-top: 2rem;
}

.sapUiSizeCompact .sapUiFormEdit.sapUiFormM > .sapUiGrid > tbody > tr > td.sapUiFormElementLbl {
	padding-top: 0.5rem;
}

.sapUiSizeCompact .sapUiFormEdit.sapUiFormM > .sapUiGrid > tbody > tr.sapUiGridConteinerFirstRow > td.sapUiFormElementLbl {
	padding-top: 2.5rem;
}

.sapUiSizeCompact .sapUiFormEdit.sapUiFormM > .sapUiGrid > tbody > tr:first-child > td.sapUiFormElementLbl, /* if no Form title */
.sapUiSizeCompact .sapUiFormEdit[aria-labelledby].sapUiFormM > .sapUiGrid > tbody > tr:nth-child(2) > td.sapUiFormElementLbl {
	padding-top: 1.5rem;
}

html[data-sap-ui-browser^="ff"].sapUiSizeCompact,
html[data-sap-ui-browser^="ff"] .sapUiSizeCompact {
	.sapUiFormEdit.sapUiFormM > .sapUiGrid > tbody > tr > td.sapUiFormElementLbl {
		padding-top: 0.4375rem;
	}

	.sapUiFormEdit.sapUiFormM > .sapUiGrid > tbody > tr.sapUiGridConteinerFirstRow > td.sapUiFormElementLbl {
		padding-top: 2.4375rem;
	}

	.sapUiFormEdit.sapUiFormM > .sapUiGrid > tbody > tr:first-child > td.sapUiFormElementLbl, /* if no Form title */
	.sapUiFormEdit[aria-labelledby].sapUiFormM > .sapUiGrid > tbody > tr:nth-child(2) > td.sapUiFormElementLbl {
		padding-top: 1.4375rem;
	}
}

.sapUiFormEdit.sapUiFormM > .sapUiGrid > tbody > tr > td.sapUiGridLabelFull {
	padding-top: 0;
}

.sapUiGridSpace {
	width: 1.5rem; /* as cells after separator have padding-left of 0.5rem */
}

.sapUiGrid .sapUiGridHeader {
	padding: 0 0 0 0.5rem;
}

/* Title */
.sapUiGrid th > .sapUiFormTitle {
	box-sizing: border-box;
	padding: 0 1rem 0 1rem;
}

.sapUiGrid .sapUiFormTitle > img,
.sapUiGrid .sapUiFormTitle > span {
	margin-right: 0.25rem;
}

/* Toolbar */
.sapUiGrid.sapUiFormToolbar th > div {
	height: 3rem; /* to have in compact mode and in commons always 3rem on Form header */
	line-height: 3rem;
	padding: 0 1rem 0 1rem;
	box-sizing: border-box;
}

.sapUiGrid > tbody > tr > .sapUiFormContainerToolbar > div {
	margin-bottom: 0.5rem;
}

/* for display mode add padding to elements to have spacing between */
.sapUiFormM:not(.sapUiFormEdit) > .sapUiGrid tr:not(.sapUiGridConteinerHeaderRow) > td{
	padding-top: 0.25rem;
	padding-bottom: 0.25rem;
}

.sapUiFormM:not(.sapUiFormEdit) > .sapUiGrid {
	font-size: 1px; /* as browser adds some px to text to meet font-size*/
}

.sapUiFormM:not(.sapUiFormEdit) > .sapUiGrid tr:not(.sapUiGridConteinerHeaderRow) > td .sapUiIcon:not(.sapMBtnIcon),
.sapUiFormM:not(.sapUiFormEdit) > .sapUiGrid tr:not(.sapUiGridConteinerHeaderRow) > td .sapMCbBg{
	font-size: @sapUiFontSize; /* to set default font size on icons */
}

.sapUiFormM:not(.sapUiFormEdit) > .sapUiGrid .sapUiGridConteinerFirstRow:not(.sapUiGridConteinerHeaderRow) > td,
.sapUiFormM:not(.sapUiFormEdit) > .sapUiGrid .sapUiGridConteinerHeaderRow + tr:not(.sapUiGridConteinerHeaderRow) > td{
	padding-top: 0;
}

.sapUiFormM:not(.sapUiFormEdit) > .sapUiGrid > tbody > tr:last-child > td {
	padding-bottom: 1rem;
}
/* END "GridLayout.less" */

/* START "HorizontalLayout.less" */
/* =============================================== */
/* CSS for control sap.ui.layout/HorizontalLayout  */
/* Base theme                                      */
/* =============================================== */

.sapUiHLayout {
	display: inline-block;
	white-space: normal;
}

.sapUiHLayoutNoWrap {
	white-space: nowrap;
}

.sapUiHLayoutChildWrapper {
	white-space: normal;
	display: inline-block;
	vertical-align: top;
}

/* mixin call for creating container content padding classes (arguments: rootContainer, contentSelector) */
.sapUiContainerContentPadding(~".sapUiHLayout");

/* add padding between cells as well */
.sapUiHLayout.sapUiContentPadding > .sapUiHLayoutChildWrapper:not(:last-child) {
	padding-right: 1rem;
	box-sizing: border-box;
}

@media (max-width: 599px) {
	.sapMSplitContainerHideMode > .sapMSplitContainerDetail .sapUiHLayout.sapUiResponsiveContentPadding ,
	.sapMSplitContainerPortrait > .sapMSplitContainerDetail .sapUiHLayout.sapUiResponsiveContentPadding ,
	.sapUiHLayout.sapUiResponsiveContentPadding  > .sapUiHLayoutChildWrapper:not(:last-child) {
		padding: 0;
	}
}
@media (min-width: 600px) and (max-width: 1023px) {
	.sapMSplitContainerHideMode > .sapMSplitContainerDetail .sapUiHLayout.sapUiResponsiveContentPadding ,
	.sapUiHLayout.sapUiResponsiveContentPadding  > .sapUiHLayoutChildWrapper:not(:last-child) {
		padding-right: 1rem;
		box-sizing: border-box;
	}
}
@media (min-width: 1024px) {
	.sapMSplitContainerHideMode > .sapMSplitContainerDetail .sapUiHLayout.sapUiResponsiveContentPadding ,
	.sapUiHLayout.sapUiResponsiveContentPadding  > .sapUiHLayoutChildWrapper:not(:last-child) {
		padding-right: 1rem;
		box-sizing: border-box;
	}
}

.sapMSplitContainerDetail .sapUiHLayout.sapUiResponsiveContentPadding  > .sapUiHLayoutChildWrapper:not(:last-child) {
	padding-right: 1rem;
}

/* fix for BCP: 1580166858
 * HorizontalLayout inside table cell doesn't respect the width of the cell but the width of the control that is inside it
 */
.sapMListTblCell .sapUiHLayout {
	width: 100%;
}
/* END "HorizontalLayout.less" */

/* START "ResponsiveFlowLayout.less" */
/* =================================================== */
/* CSS for control sap.ui.layout/ResponsiveFlowLayout  */
/* Base theme                                          */
/* =================================================== */

.sapUiRFL {
	position: relative;
	display: inline-block;
	width: 100%;
	box-sizing: border-box;
}

.sapUiRFLRow {
	position: relative;
	display: block;
	box-sizing: border-box;
}

.sapUiRFLContainer {
	position: relative;
	display: inline-block;
	box-sizing: border-box;
	vertical-align: top;
	overflow: hidden;
}

.sapUiRFLRow > .sapUiRFLContainer:first-child > .sapUiRFLContainerContent.sapUiRFLPaddingClass:first-child {
	padding-left: 0;
}

.sapUiRFLRow > .sapUiRFLContainer:last-child > .sapUiRFLContainerContent.sapUiRFLPaddingClass:last-child {
	padding-right: 0;
}

.sapUiRFLMultiContainerContent {
	white-space: nowrap;
}

.sapUiRFLMultiContent {
	display: inline-block;
	box-sizing: border-box;
}

.sapUiRFLMultiContainerContent > .sapUiRFLMultiContent.sapUiRFLPaddingClass:last-child {
	padding-right: 0;
}

.sapUiRFLPaddingClass {
	padding: 0 0 8px 8px;
}

.sapUiRFLFullLength > .sapUiRFLPaddingClass {
	padding-left: 0;
}

.sapUiRFLMultiContent.sapUiRFLPaddingClass {
	padding: 0 0 8px 8px;
}

.sapUiRFLMultiContainerContent > .sapUiRFLMultiContent.sapUiRFLPaddingClass:first-child {
	padding-left: 0;
}
/* END "ResponsiveFlowLayout.less" */

/* START "ResponsiveGridLayout.less" */
/* =================================================== */
/* CSS for control sap.ui.layout/ResponsiveGridLayout  */
/* Base theme                                          */
/* =================================================== */

/* Title */
.sapUiFormResGrid > .sapUiFormTitle{
	box-sizing: border-box;
	padding: 0 1rem 0 1rem;
}

.sapUiRGLContainer > .sapUiFormTitle{
	padding: 0 0.25rem;
}

.sapUiFormResGrid > .sapUiFormTitle > img,
.sapUiRGLContainer > .sapUiFormTitle > img,
.sapUiFormResGrid > .sapUiFormTitle > span,
.sapUiRGLContainer > .sapUiFormTitle > span{
	margin-right: 0.5rem;
}

/* Toolbar */
.sapUiFormResGrid.sapUiFormToolbar > div:first-child {
	height: 3rem; /* to have in compact mode and in commons always 3rem on Form header */
	line-height: 3rem;
	padding: 0 0.75rem 0 1rem;
}

/* Grid */
.sapUiFormResGrid > div {
	padding: 1rem 0.75rem; /* because fields have 0.25rem padding on left and right */
	box-sizing: border-box;
}

.sapUiFormM:not(.sapUiFormEdit) > .sapUiFormResGrid > div:not(.sapMTB) {
	padding-bottom: 0.75rem; /* as fields have 0.25rem */
}

.sapUiRGLContainerColl > .sapUiRGLContainerCont{
	display: none;
}

/* add paddings to content */
.sapUiFormResGridCont{
	padding-bottom: 2rem;
}

.sapUiFormResGridCont > div{
	padding: 0 0.25rem;
	box-sizing: border-box;
}

/* padding between rows for commons controls */
.sapUiForm:not(.sapUiFormM) .sapUiFormResGridCont > div{
	padding-bottom: 0.5rem;
}

.sapUiFormResGridCont > .sapUiFormElementLbl{
	min-height: @sapUiLineHeight;
}

/* Special styles for Mobile controls */
/* in display only mode add padding to controls as they don't have one */
.sapUiFormM:not(.sapUiFormEdit) .sapUiFormResGridCont > div{
	padding-top: 0.25rem;
	padding-bottom: 0.25rem;
	font-size: 1px; /* as browser adds some px to text to meet font-size */
}

.sapUiFormM:not(.sapUiFormEdit) .sapUiFormResGridCont > div .sapUiIcon:not(.sapMBtnIcon),
.sapUiFormM:not(.sapUiFormEdit) .sapUiFormResGridCont > div .sapMCbBg{
	font-size: @sapUiFontSize; /* to set default font size on icons*/
}

.sapUiFormM:not(.sapUiFormEdit) .sapUiFormResGridCont{
	margin-top: -0.25rem; /* as content has 0.25rem padding-top */
	padding-bottom: 1.75rem; /* as content has 0.25rem padding-bottom */
}

.sapUiFormM > .sapUiFormResGrid > .sapUiFormResGridMain.sapUiRespGridOverflowHidden > div[class*="sapUiFormResGrid"]{
	overflow: visible; /* to show top of title or tooolbar in cozy mode (because moved upwards) */
}

/* ================= */
/*   SMALL DEVICES   */
/* ================= */
html.sap-phone .sapUiFormResGridCont{
	/* on real phones use smaller space between containers */
	padding-bottom: 1rem;
}

html.sap-phone .sapUiFormM:not(.sapUiFormEdit) .sapUiFormResGridCont{
	/* on real phones use smaller space between containers */
	padding-bottom: 0.75rem;
}

/* ================================= */
/*   SMALL, MEDIUM and LARGE DEVICES */
/* ================================= */
/* add some space on top of full-size labels*/
.sapUiForm:not(.sapUiFormM) .sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanS12,
.sapUiForm:not(.sapUiFormM) .sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanM12,
.sapUiForm:not(.sapUiFormM) .sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanL12,
.sapUiForm:not(.sapUiFormM) .sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanXL12{
	padding-top: 0.375rem;
}

.sapUiFormM.sapUiFormEdit .sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanS12,
.sapUiFormM.sapUiFormEdit .sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanM12,
.sapUiFormM.sapUiFormEdit .sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanL12,
.sapUiFormM.sapUiFormEdit .sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanXL12{
	padding-top: 0.625rem;
}

.sapUiFormM:not(.sapUiFormEdit) .sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanS12,
.sapUiFormM:not(.sapUiFormEdit) .sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanM12,
.sapUiFormM:not(.sapUiFormEdit) .sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanL12,
.sapUiFormM:not(.sapUiFormEdit) .sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanXL12{
	padding-top: 0.75rem;
	padding-bottom: 0;
}

/* but not for first one */
.sapUiForm .sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanS12,
.sapUiForm .sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanM12,
.sapUiForm .sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanL12,
.sapUiForm .sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanXL12{
	padding-top: 0;
}

/* but not for first content in display mode */
.sapUiFormM:not(.sapUiFormEdit) .sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanS12,
.sapUiFormM:not(.sapUiFormEdit) .sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanM12,
.sapUiFormM:not(.sapUiFormEdit) .sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanL12,
.sapUiFormM:not(.sapUiFormEdit) .sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanXL12{
	padding-top: 0.25rem;
}

/* but not for first full-size label under title/toolbar in compact mode */
.sapUiSizeCompact .sapUiFormM.sapUiFormEdit .sapUiFormContainerTitle > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanS12,
.sapUiSizeCompact .sapUiFormM.sapUiFormEdit .sapUiFormContainerTitle > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanM12,
.sapUiSizeCompact .sapUiFormM.sapUiFormEdit .sapUiFormContainerTitle > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanL12,
.sapUiSizeCompact .sapUiFormM.sapUiFormEdit .sapUiFormContainerTitle > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanXL12,
.sapUiSizeCompact .sapUiFormM.sapUiFormEdit .sapUiFormContainerToolbar > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanS12,
.sapUiSizeCompact .sapUiFormM.sapUiFormEdit .sapUiFormContainerToolbar > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanM12,
.sapUiSizeCompact .sapUiFormM.sapUiFormEdit .sapUiFormContainerToolbar > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanL12,
.sapUiSizeCompact .sapUiFormM.sapUiFormEdit .sapUiFormContainerToolbar > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanXL12{
	padding-top: 0.5rem;
}

.sapUiSizeCompact .sapUiFormM:not(.sapUiFormEdit) .sapUiFormContainerTitle > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanS12,
.sapUiSizeCompact .sapUiFormM:not(.sapUiFormEdit) .sapUiFormContainerTitle > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanM12,
.sapUiSizeCompact .sapUiFormM:not(.sapUiFormEdit) .sapUiFormContainerTitle > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanL12,
.sapUiSizeCompact .sapUiFormM:not(.sapUiFormEdit) .sapUiFormContainerTitle > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanXL12,
.sapUiSizeCompact .sapUiFormM:not(.sapUiFormEdit) .sapUiFormContainerToolbar > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanS12,
.sapUiSizeCompact .sapUiFormM:not(.sapUiFormEdit) .sapUiFormContainerToolbar > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanM12,
.sapUiSizeCompact .sapUiFormM:not(.sapUiFormEdit) .sapUiFormContainerToolbar > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanL12,
.sapUiSizeCompact .sapUiFormM:not(.sapUiFormEdit) .sapUiFormContainerToolbar > .sapUiRGLContainerCont > .sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > div:first-child.sapUiFormElementLbl.sapUiRespGridSpanXL12{
	padding-top: 0.75rem;
}

.sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanS12,
.sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanM12,
.sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanL12,
.sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > .sapUiFormElementLbl.sapUiRespGridSpanXL12{
	padding-bottom: 0.125rem;
}

.sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanS12),
.sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanM12),
.sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanL12),
.sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanXL12){
	padding-top: 1rem;
}

.sapUiSizeCompact .sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanS12),
.sapUiSizeCompact .sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanM12),
.sapUiSizeCompact .sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanL12),
.sapUiSizeCompact .sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanXL12){
	padding-top: 0.5rem;
}

html[data-sap-ui-browser^="ff"] .sapUiSizeCompact .sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanS12),
html[data-sap-ui-browser^="ff"].sapUiSizeCompact .sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanS12),
html[data-sap-ui-browser^="ff"] .sapUiSizeCompact .sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanM12),
html[data-sap-ui-browser^="ff"].sapUiSizeCompact .sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanM12),
html[data-sap-ui-browser^="ff"] .sapUiSizeCompact .sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanL12),
html[data-sap-ui-browser^="ff"].sapUiSizeCompact .sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanL12),
html[data-sap-ui-browser^="ff"] .sapUiSizeCompact .sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanXL12),
html[data-sap-ui-browser^="ff"].sapUiSizeCompact .sapUiFormEdit.sapUiFormM .sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanXL12){
	padding-top: 0.4375rem;
}

/* on last row no padding-bottom on container content */
.sapUiFormM > .sapUiFormResGrid > .sapUiRGLContainer > .sapUiRGLContainerCont > .sapUiFormResGridCont, /* only one container(panel) */
.sapUiRespGridMedia-Std-Phone > .sapUiFormResGridLastRowM:last-child .sapUiFormResGridCont,
.sapUiRespGridMedia-Std-Tablet > .sapUiFormResGridLastRowM .sapUiFormResGridCont,
.sapUiRespGridMedia-Std-Desktop > .sapUiFormResGridLastRowL .sapUiFormResGridCont,
.sapUiRespGridMedia-Std-LargeDesktop > .sapUiFormResGridLastRowXL .sapUiFormResGridCont{
	padding-bottom: 0;
}

/* In Cozy mode put first FormContainers with title or toolbar 0.5rem on top */
.sapUiFormM .sapUiRespGridMedia-Std-Phone > .sapUiFormResGridFirstRowM:first-child > .sapUiFormContainerTitle,
.sapUiFormM .sapUiRespGridMedia-Std-Phone > .sapUiFormResGridFirstRowM:first-child > .sapUiFormContainerToolbar,
.sapUiFormM .sapUiRespGridMedia-Std-Tablet > .sapUiFormResGridFirstRowM > .sapUiFormContainerTitle,
.sapUiFormM .sapUiRespGridMedia-Std-Tablet > .sapUiFormResGridFirstRowM > .sapUiFormContainerToolbar,
.sapUiFormM .sapUiRespGridMedia-Std-Desktop > .sapUiFormResGridFirstRowL > .sapUiFormContainerTitle,
.sapUiFormM .sapUiRespGridMedia-Std-Desktop > .sapUiFormResGridFirstRowL > .sapUiFormContainerToolbar,
.sapUiFormM .sapUiRespGridMedia-Std-LargeDesktop > .sapUiFormResGridFirstRowXL > .sapUiFormContainerTitle,
.sapUiFormM .sapUiRespGridMedia-Std-LargeDesktop > .sapUiFormResGridFirstRowXL > .sapUiFormContainerToolbar{
	margin-top: -0.5rem;
}

/* only one container - reduce top padding*/
.sapUiFormM > .sapUiFormResGrid > .sapUiFormContainerTitle,
.sapUiFormM > .sapUiFormResGrid > .sapUiFormContainerToolbar{
	padding-top: 0.5rem;
}

.sapUiSizeCompact .sapUiRespGridMedia-Std-Phone > .sapUiFormResGridFirstRowM:first-child > .sapUiFormContainerTitle,
.sapUiSizeCompact .sapUiRespGridMedia-Std-Phone > .sapUiFormResGridFirstRowM:first-child > .sapUiFormContainerToolbar,
.sapUiSizeCompact .sapUiRespGridMedia-Std-Tablet > .sapUiFormResGridFirstRowM > .sapUiFormContainerTitle,
.sapUiSizeCompact .sapUiRespGridMedia-Std-Tablet > .sapUiFormResGridFirstRowM > .sapUiFormContainerToolbar,
.sapUiSizeCompact .sapUiRespGridMedia-Std-Desktop > .sapUiFormResGridFirstRowL > .sapUiFormContainerTitle,
.sapUiSizeCompact .sapUiRespGridMedia-Std-Desktop > .sapUiFormResGridFirstRowL > .sapUiFormContainerToolbar,
.sapUiSizeCompact .sapUiRespGridMedia-Std-LargeDesktop > .sapUiFormResGridFirstRowXL > .sapUiFormContainerTitle,
.sapUiSizeCompact .sapUiRespGridMedia-Std-LargeDesktop > .sapUiFormResGridFirstRowXL > .sapUiFormContainerToolbar{
	margin-top: 0;
}

.sapUiSizeCompact .sapUiFormM > .sapUiFormResGrid > .sapUiFormContainerTitle,
.sapUiSizeCompact .sapUiFormM > .sapUiFormResGrid > .sapUiFormContainerToolbar{
	padding-top: 1rem;
}

/* ============================ */
/*   MEDIUM and LARGE DEVICES   */
/* ============================ */
/* add paddings between containers */
.sapUiRespGridMedia-Std-Tablet.sapUiFormResGridMain > div:not(.sapUiRespGridSpanM12):not(.sapUiFormResGridLastContM),
.sapUiRespGridMedia-Std-Desktop.sapUiFormResGridMain > div:not(.sapUiRespGridSpanL12):not(.sapUiFormResGridLastContL),
.sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridMain > div:not(.sapUiRespGridSpanXL12):not(.sapUiFormResGridLastContXL){
	padding-right: 0.75rem; /* because fields have 0.25rem padding on left and right */
	box-sizing: border-box;
}

.sapUiRespGridMedia-Std-Tablet.sapUiFormResGridMain > div:not(.sapUiRespGridBreakM):not(.sapUiRespGridSpanM12),
.sapUiRespGridMedia-Std-Desktop.sapUiFormResGridMain > div:not(.sapUiRespGridBreakL):not(.sapUiRespGridSpanL12),
.sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridMain > div:not(.sapUiRespGridBreakXL):not(.sapUiRespGridSpanXL12){
	padding-left: 0.75rem; /* because fields have 0.25rem padding on left and right */
	box-sizing: border-box;
}
/* END "ResponsiveGridLayout.less" */

/* START "ResponsiveLayout.less" */
/* =============================================== */
/* CSS for control sap.ui.layout/ResponsiveLayout  */
/* Base theme                                      */
/* =============================================== */

.sapUiRLContainer.sapUiFormContainerTitle > .sapUiFormTitle.sapUiFormTitleExpandable{
	padding-left: 2.25rem;
}

.sapUiRLContainer.sapUiFormContainerTitle > .sapUiFormTitle.sapUiFormTitleExpandable > button{
	left: -0.5rem;
}
.sapUiRLContainerColl > .sapUiRLContainerCont{
	display: none;
}

/* make labels before fields right aligned */
.sapUiRLElementWithLabel > .sapUiRFLRow:first-child > .sapUiRFLContainer:first-child:not(.sapUiRFLFullLength) > div{
	text-align: end;
}

.sapUiFormResLayout > div {
	padding: 1rem;
	box-sizing: border-box;
}

.sapUiFormM .sapUiRFLPaddingClass,
.sapUiFormM .sapUiRFLMultiContent.sapUiRFLPaddingClass {
	padding: 0 0 0 0.5rem;
}

.sapUiRLContainerCont > .sapUiRFL >.sapUiRFLRow >.sapUiRFLContainer{
	padding-right: 0.5rem;
}

.sapUiRLContainerCont > .sapUiRFL >.sapUiRFLRow >.sapUiRFLContainer:last-child{
	padding-right: 0;
}

.sapUiRLElementWithLabel > .sapUiRFLCompleteRow:first-child > .sapUiRFLContainer.sapUiRFLFullLength > .sapUiRFLContainerContent.sapUiRFLPaddingClass{
	padding-bottom: 0;
}

.sapUiFormEdit.sapUiFormM .sapUiRLElementWithLabel > .sapUiRFLRow:not(.sapUiRFLCompleteRow) > .sapUiRFLContainer:first-child > .sapUiRFLContainerContent{
	padding-top: 1rem;
}

html[data-sap-ui-browser^="ff"] .sapUiFormEdit.sapUiFormM .sapUiRLElementWithLabel > .sapUiRFLRow:not(.sapUiRFLCompleteRow) > .sapUiRFLContainer:first-child > .sapUiRFLContainerContent{
	padding-top: 0.9375rem;
}

.sapUiSizeCompact .sapUiFormEdit.sapUiFormM .sapUiRLElementWithLabel > .sapUiRFLRow:not(.sapUiRFLCompleteRow) > .sapUiRFLContainer:first-child > .sapUiRFLContainerContent{
	padding-top: 0.5rem;
}

html[data-sap-ui-browser^="ff"].sapUiSizeCompact .sapUiFormEdit.sapUiFormM .sapUiRLElementWithLabel > .sapUiRFLRow:not(.sapUiRFLCompleteRow) > .sapUiRFLContainer:first-child > .sapUiRFLContainerContent,
html[data-sap-ui-browser^="ff"] .sapUiSizeCompact .sapUiFormEdit.sapUiFormM .sapUiRLElementWithLabel > .sapUiRFLRow:not(.sapUiRFLCompleteRow) > .sapUiRFLContainer:first-child > .sapUiRFLContainerContent{
	padding-top: 0.4375rem;
}

.sapUiFormResLayout .sapUiRFLRow > .sapUiRFLContainer.sapUiRFLFullLength > .sapUiRFLContainerContent > .sapUiRLElement > .sapUiRFLRow:last-child > .sapUiRFLFullLength:last-child{
	padding-bottom: 0.625rem;
}
.sapUiForm > .sapUiFormResLayout > .sapUiRFL .sapUiRFLRow:last-child > .sapUiRFLContainer.sapUiRFLFullLength > .sapUiRFLContainerContent > .sapUiRLElement > .sapUiRFLRow:last-child > .sapUiRFLFullLength:last-child{
	/* last element in row has no padding bottom */
	padding-bottom: 0;
}

.sapUiRLElementFields > .sapUiRFLRow > .sapUiRFLContainer:first-child > .sapUiRFLContainerContent.sapUiRFLPaddingClass:first-child,
.sapUiRLElementFields > .sapUiRFLRow > .sapUiRFLContainer:first-child > .sapUiRFLContainerContent.sapUiRFLMultiContainerContent.sapUiRFLPaddingClass:first-child > .sapUiRFLMultiContent.sapUiRFLPaddingClass:first-child{
	padding-left: 0.5rem;
}
.sapUiRLElementFields > .sapUiRFLRow > .sapUiRFLContainer:first-child > .sapUiRFLContainerContent.sapUiRFLMultiContainerContent.sapUiRFLPaddingClass:first-child,
.sapUiRFLFullLength > .sapUiRFLContainerContent > .sapUiRLElementFields > .sapUiRFLRow > .sapUiRFLContainer:first-child > .sapUiRFLContainerContent.sapUiRFLPaddingClass:first-child,
.sapUiRFLFullLength > .sapUiRFLContainerContent > .sapUiRLElementFields > .sapUiRFLRow > .sapUiRFLContainer:first-child > .sapUiRFLContainerContent.sapUiRFLPaddingClass:first-child > .sapUiRFLMultiContent.sapUiRFLPaddingClass:first-child{
	padding-left: 0;
}

.sapUiRFLRow > .sapUiRFLContainer > .sapUiRFLContainerContent > .sapUiRLContainer{
	/* space between 2 containers, devide it to previous and next container to have the same size for all*/
	padding-left: 0.5rem;
	padding-right: 1rem;
}
.sapUiRFLRow > .sapUiRFLContainer:first-child > .sapUiRFLContainerContent > .sapUiRLContainer{
	padding-left: 0;
}
.sapUiRFLRow > .sapUiRFLContainer:last-child > .sapUiRFLContainerContent > .sapUiRLContainer{
	padding-right: 0;
}

/* full size labels in mobile */
.sapUiFormM .sapUiFormResLayout .sapUiRFLRow:first-child > .sapUiRFLContainer.sapUiRFLFullLength > .sapUiFormElementLbl{
	padding-top: 0.25rem;
}

/* full-size label under title/toolbar in compact mode */
.sapUiSizeCompact .sapUiFormM .sapUiFormContainerTitle > .sapUiRLContainerCont > .sapUiRFL > .sapUiRFLRow:first-child > .sapUiRFLContainer .sapUiRFLContainerContent > .sapUiRLElementWithLabel > .sapUiRFLRow:first-child > .sapUiRFLContainer.sapUiRFLFullLength > .sapUiFormElementLbl{
	padding-top: 0.5rem;
}

/* Title */
.sapUiFormResLayout > .sapUiFormTitle{
	box-sizing: border-box;
	padding: 0 1rem 0 1rem;
}

.sapUiFormResLayout .sapUiFormTitle > img,
.sapUiFormResLayout .sapUiFormTitle > span{
	margin-right: 0.1875rem;
}

/* Toolbar */
.sapUiFormResLayout.sapUiFormToolbar > div:first-child {
	height: 3rem; /* to have in compact mode and in commons always 3rem on Form header */
	line-height: 3rem;
	padding: 0 1rem 0 1rem;
}

/* vertical spacing between containers */
.sapUiFormResLayout > .sapUiRFL > .sapUiRFLRow > .sapUiRFLContainer > .sapUiRFLContainerContent > .sapUiRLContainer{
	padding-top: 2rem;
}

html.sap-phone .sapUiFormResLayout > .sapUiRFL > .sapUiRFLRow > .sapUiRFLContainer > .sapUiRFLContainerContent > .sapUiRLContainer{
	padding-top: 1rem;
}

.sapUiForm > .sapUiFormResLayout > .sapUiRFL > .sapUiRFLRow:first-child > .sapUiRFLContainer > .sapUiRFLContainerContent > .sapUiRLContainer{
	padding-top: 0;
}

/* In Cozy mode put first FormContainers with title or toolbar 0.5rem on top */
.sapUiFormM > .sapUiFormResLayout > .sapUiRFL > .sapUiRFLRow:first-child > .sapUiRFLContainer > .sapUiRFLContainerContent > .sapUiRLContainer.sapUiFormContainerTitle,
.sapUiFormM > .sapUiFormResLayout > .sapUiRFL > .sapUiRFLRow:first-child > .sapUiRFLContainer > .sapUiRFLContainerContent > .sapUiRLContainer.sapUiFormContainerToolbar{
	margin-top: -0.5rem;
}
.sapUiFormM > .sapUiFormResLayout > .sapUiRFL > .sapUiRFLRow:first-child > .sapUiRFLContainer{
	overflow: visible; /* to show expander and toolbar completely */
}

/* only one container - reduce top padding*/
.sapUiFormM > .sapUiFormResLayout > .sapUiFormContainerTitle,
.sapUiFormM > .sapUiFormResLayout > .sapUiFormContainerToolbar{
	padding-top: 0.5rem;
}

.sapUiSizeCompact .sapUiFormM > .sapUiFormResLayout > .sapUiRFL > .sapUiRFLRow:first-child > .sapUiRFLContainer > .sapUiRFLContainerContent > .sapUiRLContainer.sapUiFormContainerTitle,
.sapUiSizeCompact .sapUiFormM > .sapUiFormResLayout > .sapUiRFL > .sapUiRFLRow:first-child > .sapUiRFLContainer > .sapUiRFLContainerContent > .sapUiRLContainer.sapUiFormContainerToolbar{
	margin-top: 0;
}

.sapUiSizeCompact .sapUiFormM > .sapUiFormResLayout > .sapUiFormContainerTitle,
.sapUiSizeCompact .sapUiFormM > .sapUiFormResLayout > .sapUiFormContainerToolbar{
	padding-top: 1rem;
}

/* for display mode add padding to elements to have spacing between */
.sapUiFormM:not(.sapUiFormEdit) .sapUiRFLContainerContent{
	font-size: 1px; /* as browser adds some px to text to meet font-size*/
}

.sapUiFormM:not(.sapUiFormEdit) .sapUiRFLContainerContent .sapUiIcon:not(.sapMBtnIcon),
.sapUiFormM:not(.sapUiFormEdit) .sapUiRFLContainerContent .sapMCbBg{
	font-size: @sapUiFontSize; /* to set default font size on icons */
}

.sapUiFormM:not(.sapUiFormEdit) .sapUiRLElement > .sapUiRFLRow{
	padding-top: 0.25rem;
	padding-bottom: 0.25rem;
}

.sapUiFormM:not(.sapUiFormEdit) .sapUiRLContainerCont > .sapUiRFL > .sapUiRFLRow:first-child > .sapUiRFLContainer > .sapUiRFLContainerContent > .sapUiRLElement > .sapUiRFLRow:first-child{
	padding-top: 0;
}

.sapUiFormM:not(.sapUiFormEdit) .sapUiRLContainerCont > .sapUiRFL > .sapUiRFLRow:last-child > .sapUiRFLContainer > .sapUiRFLContainerContent > .sapUiRLElement > .sapUiRFLRow:last-child{
	padding-bottom: 0;
}

.sapUiFormM:not(.sapUiFormEdit) > .sapUiFormResLayout .sapUiRFLRow:first-child > .sapUiRFLContainer.sapUiRFLFullLength > .sapUiRFLContainerContent > .sapMLabel{
	padding-bottom: 0;
}

.sapUiFormM:not(.sapUiFormEdit) > .sapUiFormResLayout .sapUiRFLRow > .sapUiRFLContainer.sapUiRFLFullLength > .sapUiRFLContainerContent > .sapUiRLElement > .sapUiRFLRow:last-child > .sapUiRFLFullLength:last-child{
	padding-bottom: 0.25rem;
}

/* END "ResponsiveLayout.less" */

/* START "ResponsiveSplitter.less" */
/* ================================================= */
/* CSS for control sap.ui.layout/ResponsiveSplitter  */
/* Base theme                                        */
/* ================================================= */

@_sap_ui_layout_ResponsiveSplitter_PaginatorHeight: 2.5rem;
@_sap_ui_layout_ResponsiveSplitter_PaginatorButtonWidth: 2.5rem;
@_sap_ui_layout_ResponsiveSplitter_PaginatorButtonHeight: 2.375rem;
@_sap_ui_layout_ResponsiveSplitter_PaginatorButtonFocusBorderRadius: unset;
@_sap_ui_layout_ResponsiveSplitter_PaginatorSelectedButton: @sapUiContentIconColor;
@_sap_ui_layout_ResponsiveSplitter_PaginatorButtonActive: @sapUiContentContrastIconColor;
@_sap_ui_layout_ResponsiveSplitter_PaginatorButtonHoverShadow: none;

.sapUiResponsiveSplitter {
	width: 100%;
	height: 100%;
	overflow: hidden;
}

.sapUiResponsiveSplitterPage {
	width: 100%;
	height: 100%;
	overflow-x: hidden;
	overflow-y: auto;
}

.sapUiRSVisiblePaginator {
	& .sapUiResponsiveSplitterPaginator {
		width: 100%;
		display: flex;
		justify-content: center;
		box-sizing: border-box;
		outline: none;
		height: @_sap_ui_layout_ResponsiveSplitter_PaginatorHeight;
		background-color: @sapUiPageFooterBackground;
		border-top: 0.0625rem solid @sapUiPageFooterBorderColor;
		border-bottom: 0.0625rem solid transparent; // needed to properly align buttons vertically
	}

	& .sapUiResponsiveSplitterPage {
		height: calc(100% ~"-" @_sap_ui_layout_ResponsiveSplitter_PaginatorHeight);
	}

	& .sapUiResponsiveSplitterPaginator.sapUiResponsiveSplitterWithNavButtons {
		justify-content: space-between;
	}
}

.sapUiResponsiveSplitterPaginatorButtons {
	display: flex;
	align-items: center;
}

.sapUiResponsiveSplitterPaginatorButton {
	width: @_sap_ui_layout_ResponsiveSplitter_PaginatorButtonWidth;
	height: @_sap_ui_layout_ResponsiveSplitter_PaginatorButtonHeight;
	cursor: pointer;
	background-color: transparent;
	border-radius: @sapUiButtonBorderCornerRadius;
	position: relative;

	&::after {
		content: "";
		width: 0.25rem;
		height: 0.25rem;
		background-color: @sapUiContentNonInteractiveIconColor;
		margin: auto;
		display: block;
		margin-top: 16px;
		border-radius: 50%;
	};

	&:focus {
		outline: none;
	}

	&:focus::before {
		content: "";
		position: absolute;
		top: 0;
		bottom: 0;
		right: 0;
		left: 0;
		border: @sapUiContentFocusWidth @sapUiContentFocusStyle @sapUiContentFocusColor;
		border-radius: @_sap_ui_layout_ResponsiveSplitter_PaginatorButtonFocusBorderRadius;
	}

	&:hover {
		background-color: @sapUiButtonLiteHoverBackground;
		box-shadow: @_sap_ui_layout_ResponsiveSplitter_PaginatorButtonHoverShadow;

		&::after {
			background-color: @sapUiContentNonInteractiveIconColor;
		}
	}

	&:active {
		background-color: @sapUiButtonLiteActiveBackground;

		&::after {
			background-color: @_sap_ui_layout_ResponsiveSplitter_PaginatorButtonActive;
		}
	}
}

.sapUiResponsiveSplitterPaginatorSelectedButton {
	&::after {
		background-color: @_sap_ui_layout_ResponsiveSplitter_PaginatorSelectedButton;
		width: 0.5rem;
		height: 0.5rem;
		margin-top: 0.875rem;
	}

	&:hover {
		&::after {
			background-color: @_sap_ui_layout_ResponsiveSplitter_PaginatorSelectedButton;
		}
	}
}

.sapUiResponsiveSplitterHiddenElement {
	display: none;
}

.sapUiResponsiveSplitterPaginatorNavButton {
	width: 2.5rem;
	height: 100%;
	font-family: 'SAP-icons';
	cursor: pointer;

	&::after {
		position: absolute;
		margin-top: 11px;
		margin-left: 15px;
	}
}

.sapUiResponsiveSplitterPaginatorButtonBack {
	&::after {
		content: '\e067';
	}
}

.sapUiResponsiveSplitterPaginatorButtonForward {
	&::after {
		content: '\e066';
	}
}

.sapUiResponsiveSplitterHiddenBackButton,
 .sapUiResponsiveSplitterHiddenPaginatorButton {
	display: none !important;
	background-color: @sapUiSelected;
}

/* END "ResponsiveSplitter.less" */

/* START "SimpleForm.less" */
/* ========================================= */
/* CSS for control sap.ui.layout/SimpleForm  */
/* Base theme                                */
/* ========================================= */

.sapUiSimpleForm {
	width:100%;
	box-sizing: border-box;
}

html.sap-tablet .sapUiFioriObjectPage > section > .sapUiSimpleForm,
html.sap-desktop .sapUiFioriObjectPage > section > .sapUiSimpleForm {
	margin: 1rem 0;
	padding: 0 1rem;
}

.sapUiFormEdit-CTX .sapMSwtCont {
	vertical-align:middle;
}
/* END "SimpleForm.less" */

/* START "Splitter.less" */
/* ======================================= */
/* CSS for control sap.ui.layout/Splitter  */
/* Base theme                              */
/* ======================================= */

/**
 * The main Splitter element can have the following classes in addition to its main class
 * "sapUiLoSplitter":
 *   sapUiLoSplitterH        - It's a horizontal splitter
 *   sapUiLoSplitterV        - It's a vertical splitter
 * The splitter bars "sapUiLoSplitterBar" and "sapUiLoSplitterOverlayBar" can have the following
 * additional classes:
 *   sapUiLoSplitterNoResize
 *
**/

@_sap_ui_layout_Splitter_BarActiveDecorationColors: @sapUiHighlight, @sapUiHighlight;
@_sap_ui_layout_Splitter_BarDecorationColors: @sapUiHighlight, fade(@sapUiHighlight, 0);
@_sap_ui_layout_Splitter_BarBorder: 0.0625rem solid @sapUiGroupContentBorderColor;
@_sap_ui_layout_Splitter_BarHoverIconBorderColor: @sapUiButtonLiteBorderColor;
@_sap_ui_layout_Splitter_BarHoverIconBackground: @sapUiButtonLiteBackground;
@_sap_ui_layout_Splitter_GripButtonColor: @sapUiContentIconColor;

/********************************************* General ********************************************/

.sapUiLoSplitter {
	position: relative;
	width: 100%;
	height: 100%;
	overflow: hidden;
	display: block;
	box-sizing: border-box;

	.sapUiLoSplitterContent {
		overflow: auto;
	}
}

.sapUiLoSplitter > .sapUiLoSplitterBar,
.sapUiLoSplitterOverlay > .sapUiLoSplitterOverlayBar {
	color: @_sap_ui_layout_Splitter_GripButtonColor;
	-webkit-touch-callout: none;
	-webkit-user-select: none;
	-moz-user-select: none;
	user-select: none;
	background-color: @sapUiShellBackground;
	box-sizing: border-box;
}

.sapUiLoSplitter.sapUiLoSplitterH > .sapUiLoSplitterBar.sapUiLoSplitterNoResize,
.sapUiLoSplitter.sapUiLoSplitterV > .sapUiLoSplitterBar.sapUiLoSplitterNoResize {
	cursor: default;
	visibility: hidden;
}

.sapUiLoSplitter > .sapUiLoSplitterBar.sapUiLoSplitterNoResize > *,
.sapUiLoSplitterOverlay > .sapUiLoSplitterOverlayBar.sapUiLoSplitterNoResize > * {
	display: none;
}

.sapUiLoSplitterOverlay {
	display: none;
	position: absolute;
	top: 0;
	left: 0;
	bottom: 0;
	right: 0;
}

.sapUiLoSplitterOverlay > .sapUiLoSplitterOverlayBar {
	position: absolute;
}

/******************************************* Horizontal *******************************************/

.sapUiLoSplitterH.sapUiLoSplitter {
	white-space: nowrap;

	> .sapUiLoSplitterContent {
		white-space: normal;
		display: inline-block;
		vertical-align: top;
		height: 100%;
	}
}

/* Animated resizing is deprecated since version 1.21 */
.sapUiLoSplitterAnimated.sapUiLoSplitterH > .sapUiLoSplitterContent {
	transition: width 200ms ease 0ms;
}

.sapUiLoSplitter.sapUiLoSplitterH > .sapUiLoSplitterBar,
.sapUiLoSplitter.sapUiLoSplitterH > .sapUiLoSplitterOverlay .sapUiLoSplitterOverlayBar {
	cursor: col-resize;
	width: 1rem;
	height: 100%;
	display: inline-flex;
	flex-direction: column;
	justify-content: center;
	align-items: center;
	border-left: @_sap_ui_layout_Splitter_BarBorder;
	border-right: @_sap_ui_layout_Splitter_BarBorder;
	vertical-align: top;

	.sapUiLoSplitterBarDecorationBefore {
		width: 1rem;
		height: 4rem;
		background-image: linear-gradient(to top, @_sap_ui_layout_Splitter_BarDecorationColors);
		background-size: 0.0625rem 100%;
		background-repeat: no-repeat;
		background-position: center;
	}

	.sapUiLoSplitterBarDecorationAfter {
		width: 1rem;
		height: 4rem;
		height: 4rem;
		background-image: linear-gradient(to bottom, @_sap_ui_layout_Splitter_BarDecorationColors);
		background-size: 0.0625rem 100%;
		background-repeat: no-repeat;
		background-position: center;
	}

	.sapUiLoSplitterBarGrip {
		cursor: col-resize;
		width: 1.5rem;
		height: 2rem;
		line-height: 2rem;
		position: relative;
		z-index: 1;

		.sapUiLoSplitterBarGripIcon {
			cursor: col-resize;
			line-height: 1.625rem - @sapUiButtonBorderWidth - @sapUiButtonBorderWidth; // center the icon vertically
			box-sizing: border-box;
			height: 1.625rem;
			width: 1.5rem;
			margin-top: 3px;
			border: @sapUiButtonBorderWidth solid @sapUiButtonLiteBorderColor;
			border-radius: @sapUiButtonBorderCornerRadius;
			background: @sapUiButtonLiteBackground;
		}
	}

	&:hover {
		.sapUiLoSplitterBarDecorationBefore,
		.sapUiLoSplitterBarDecorationAfter {
			flex-grow: 1;
			transition: all 0.1s ease-in;
		}

		.sapUiLoSplitterBarGripIcon {
			background: @_sap_ui_layout_Splitter_BarHoverIconBackground;
			border-color: @_sap_ui_layout_Splitter_BarHoverIconBorderColor;
		}
	}
}

.sapUiLoSplitter.sapUiLoSplitterH > .sapUiLoSplitterOverlay .sapUiLoSplitterOverlayBar {
	.sapUiLoSplitterBarDecorationBefore {
		flex-grow: 1;
		background-image: linear-gradient(to top, @_sap_ui_layout_Splitter_BarActiveDecorationColors);
	}

	.sapUiLoSplitterBarDecorationAfter {
		flex-grow: 1;
		background-image: linear-gradient(to bottom ,@_sap_ui_layout_Splitter_BarActiveDecorationColors);
	}
}

/******************************************** Vertical ********************************************/

.sapUiLoSplitterV > .sapUiLoSplitterBar,
.sapUiLoSplitterV > .sapUiLoSplitterContent,
.sapUiLoSplitterV > .sapUiLoSplitterOverlay > .sapUiLoSplitterOverlayBar {
	display: block;
	width: 100%;
}

/* Animated resizing is deprecated since version 1.21 */
.sapUiLoSplitterAnimated.sapUiLoSplitterV > .sapUiLoSplitterContent {
	transition: height 200ms ease 0ms;
}

.sapUiLoSplitter.sapUiLoSplitterV > .sapUiLoSplitterBar,
.sapUiLoSplitter.sapUiLoSplitterV > .sapUiLoSplitterOverlay .sapUiLoSplitterOverlayBar {
	cursor: row-resize;
	height: 1rem;
	display: flex;
	justify-content: center;
	align-items: center;
	border-top: @_sap_ui_layout_Splitter_BarBorder;
	border-bottom: @_sap_ui_layout_Splitter_BarBorder;

	.sapUiLoSplitterBarDecorationBefore {
		width: 4rem;
		height: 1rem;
		background-image: linear-gradient(to left, @_sap_ui_layout_Splitter_BarDecorationColors);
		background-size: 100% 0.0625rem;
		background-repeat: no-repeat;
		background-position: center;
	}

	.sapUiLoSplitterBarDecorationAfter {
		width: 4rem;
		height: 1rem;
		background-image: linear-gradient(to right, @_sap_ui_layout_Splitter_BarDecorationColors);
		background-size: 100% 0.0625rem;
		background-repeat: no-repeat;
		background-position: center;
	}

	.sapUiLoSplitterBarGrip {
		cursor: row-resize;
		height: 1.5rem;
		line-height: 1.5rem;
		width: 2rem;
		position: relative;
		z-index: 1;

		.sapUiLoSplitterBarGripIcon {
			cursor: row-resize;
			box-sizing: border-box;
			width: 1.625rem;
			height: 1.5rem;
			line-height: 1.5rem - @sapUiButtonBorderWidth - @sapUiButtonBorderWidth; // center the icon vertically
			margin-left: 3px;
			border: @sapUiButtonBorderWidth solid @sapUiButtonLiteBorderColor;
			border-radius: @sapUiButtonBorderCornerRadius;
			background: @sapUiButtonLiteBackground;
		}
	}

	&:hover {
		.sapUiLoSplitterBarDecorationBefore,
		.sapUiLoSplitterBarDecorationAfter {
			flex-grow: 1;
			transition: all 0.1s ease-in;
		}

		.sapUiLoSplitterBarGripIcon {
			background: @_sap_ui_layout_Splitter_BarHoverIconBackground;
			border-color: @_sap_ui_layout_Splitter_BarHoverIconBorderColor;
		}
	}
}

.sapUiLoSplitter.sapUiLoSplitterV > .sapUiLoSplitterOverlay .sapUiLoSplitterOverlayBar {
	.sapUiLoSplitterBarDecorationBefore {
		flex-grow: 1;
		background-image: linear-gradient(to left, @_sap_ui_layout_Splitter_BarActiveDecorationColors);
	}

	.sapUiLoSplitterBarDecorationAfter {
		flex-grow: 1;
		background-image: linear-gradient(to right, @_sap_ui_layout_Splitter_BarActiveDecorationColors);
	}
}

/*********************************** Focus outline**********************************/

.sapUiLoSplitter > .sapUiLoSplitterBar,
.sapUiLoSplitter > .sapUiLoSplitterOverlay .sapUiLoSplitterOverlayBar {
	position: relative;

	&:focus {
		outline: none;
	}

	&:focus::after {
		content: "";
		display: block;
		position: absolute;
		top: 0;
		right: 0;
		bottom: 0;
		left: 0;
		border: @sapUiContentFocusWidth @sapUiContentFocusStyle @sapUiContentFocusColor;
	}
}
/* END "Splitter.less" */

/* START "VerticalLayout.less" */
/* ============================================= */
/* CSS for control sap.ui.layout/VerticalLayout  */
/* Base theme                                    */
/* ============================================= */

.sapUiVlt {
	display: inline-block;
	vertical-align: text-top;
}

/* mixin call for creating container content padding classes (arguments: rootContainer, contentSelector) */
.sapUiContainerContentPadding(~".sapUiVlt");

/* add padding between cells as well */
.sapUiVlt.sapUiContentPadding > .sapuiVltCell:not(:last-child) {
	padding-bottom: 1rem;
	box-sizing: border-box;
}

.sapUiVlt.sapUiContentPadding .sapUiVltCell:has(> .sapUiHiddenPlaceholder) {
	padding-bottom: 0;
}

@media (max-width: 599px) {
	.sapMSplitContainerHideMode > .sapMSplitContainerDetail .sapUiVlt.sapUiResponsiveContentPadding ,
	.sapMSplitContainerPortrait > .sapMSplitContainerDetail .sapUiVlt.sapUiResponsiveContentPadding ,
	.sapUiVlt.sapUiResponsiveContentPadding  > .sapuiVltCell:not(:last-child) {
		padding: 0;
	}
}
@media (min-width: 600px) and (max-width: 1023px) {
	.sapMSplitContainerHideMode > .sapMSplitContainerDetail .sapUiVlt.sapUiResponsiveContentPadding ,
	.sapUiVlt.sapUiResponsiveContentPadding  > .sapuiVltCell:not(:last-child) {
		padding-bottom: 1rem;
		box-sizing: border-box;
	}
}
@media (min-width: 1024px) {
	.sapMSplitContainerHideMode > .sapMSplitContainerDetail .sapUiVlt.sapUiResponsiveContentPadding ,
	.sapUiVlt.sapUiResponsiveContentPadding  > .sapuiVltCell:not(:last-child) {
		padding-bottom: 1rem;
		box-sizing: border-box;
	}
}

.sapMSplitContainerDetail .sapUiVlt.sapUiResponsiveContentPadding  > .sapuiVltCell:not(:last-child) {
	padding-bottom: 1rem;
}
/* END "VerticalLayout.less" */

/* START "BlockLayoutRow.less" */
/* ============================================= */
/* CSS for control sap.ui.layout/BlockLayoutRow  */
/* Base theme                                    */
/* ============================================= */

.sapUiBlockScrollingRow {
	display: flex;
	flex-wrap: nowrap;
	overflow-x: auto;
	overflow-y: hidden;

	.sapUiBlockScrollableCell {
		flex-shrink: 0;
		width: 40%;
	}
}

.sapUiBlockLayoutSizeL,
.sapUiBlockLayoutSizeXL {
	.sapUiBlockScrollingNarrowCells {
		.sapUiBlockScrollableCell {
			width: 25%;
		}
	}
}

.sapUiBlockLayoutSizeS {
	.sapUiBlockSmallCell {
		width: 100%;
	}

	.sapUiBlockMediumCell {
		width: 100%;
	}

	.sapUiBlockScrollingRow {
		.sapUiBlockScrollableCell:first-child .sapUiBlockCellContent {
			padding-left: 2rem;
		}

		.sapUiBlockScrollableCell:last-child .sapUiBlockCellContent {
			padding-right: 2rem;
		}

		.sapUiBlockScrollableCell {
			min-width: 90%;
		}
	}
}
/* END "BlockLayoutRow.less" */

/* START "BlockLayoutCell.less" */
/* ============================================== */
/* CSS for control sap.ui.layout/BlockLayoutCell  */
/* Base theme                                     */
/* ============================================== */

@_sap_ui_layout_BlockLayoutCell_background: transparent;

.sapUiBlockLayoutCell {
	display: flex;
	overflow: hidden;
}

.sapUiBlockCellCenteredContent > .sapUiBlockCellTitle {
	align-self: center;
	text-align: center;
	width: 100%;
}

.sapUiBlockCellLeft,
.sapUiBlockCellBegin {
	text-align: left;
}

.sapUiBlockCellRight,
.sapUiBlockCellEnd {
	text-align: right;
}

[dir=rtl] .sapUiBlockCellEnd {
	text-align: left;
}

[dir=rtl] .sapUiBlockCellBegin {
	text-align: right;
}

.sapUiBlockCellContent {
	width: 100%;
	padding: 1rem;
	overflow: hidden;
	box-sizing: border-box;
	background-color: @_sap_ui_layout_BlockLayoutCell_background;
}

.sapUiBlockCellTitle {
    font-weight: normal;
	font-size: @sapUiFontHeader2Size;
	margin-top: 0;
	margin-bottom: 1rem;
}

.sapUiBlockCellTitleNoContent {
	margin-bottom: 0;
}

/* mixin call for creating container content padding classes (arguments: rootContainer, contentSelector) */
.sapUiContainerContentPadding(~".sapUiBlockLayoutCell", ~"> .sapUiBlockCellContent");

/* override for standard padding: 2rem for BlockLayoutCell */
.sapUiBlockLayoutCell.sapUiContentPadding > .sapUiBlockCellContent {
	padding: 2rem;
}

/* END "BlockLayoutCell.less" */

/* START "BlockLayout.less" */
/* ========================================== */
/* CSS for control sap.ui.layout/BlockLayout  */
/* Base theme                                 */
/* ========================================== */

@_sap_ui_layout_BlockLayout_FontSizeLandXL: @sapMFontHeader1Size;

@_sap_ui_layout_BlockLayout_BackgroundDashboardMargin: 0.25rem;
@_sap_ui_layout_BlockLayout_BackgroundDashboardEdgeChildMargin: 0.5rem;
@_sap_ui_layout_BlockLayout_BackgroundDashboardAfterScrollbarMargin: 0.375rem;
@_sap_ui_layout_BlockLayout_BackgroundDashboardColor: @sapUiContentForegroundTextColor;
@_sap_ui_layout_BlockLayout_BackgroundDashboardBorderRadius: 0;
@_sap_ui_layout_BlockLayout_BackgroundDashboardCellShadow: @sapUiShadowLevel0;
@_sap_ui_layout_BlockLayout_BackgroundDashboardCellShadowOffset: 0;

@_sap_ui_layout_BlockLayout_LinkTextShadow: @sapUiShadowText;
@_sap_ui_layout_BlockLayout_CellTextColor: @sapUiContentForegroundTextColor;

.setCellsColorDoubles (@evenCell, @oddCell) {
	// Match every even cell
	.sapUiBlockLayoutEvenCell {
		background-color: @evenCell;
	}

	// Match every odd cell
	.sapUiBlockLayoutOddCell {
		background-color: @oddCell;
	}
}

.sapUiBlockLayout {
	.sapUiFormResLayout > div {
		background-color: inherit;
	}
}

.sapUiBlockCellContent {
	font-family: @sapUiFontFamily;
	font-weight: normal;
	font-size: 0.875rem;
	color: @_sap_ui_layout_BlockLayout_CellTextColor;
	padding: 1rem;

	> .sapUiIcon {
		font-size: 2.25rem;
	}

	.sapUiBlockCellTitle {
		font-size: @sapMFontHeader3Size;

		.sapMLnk {
			text-shadow: @_sap_ui_layout_BlockLayout_LinkTextShadow;
		}
	}
}

.sapUiBlockCellTitle,
.sapUiBlockCellTitle .sapMLnk {
	font-size: 1.25rem;
}

// Styles & Sizes for Tablet
.sapUiBlockLayoutSizeM:not(.sapUiBlockLayoutKeepFontSize) {
	.sapUiNoContentPadding .sapUiBlockCellContent {
		padding: 0;
	}
	.sapUiBlockCellContent {
		padding: 1rem 2rem;
		font-size: @sapMFontMediumSize;

		> .sapMText {
			font-size: @sapMFontMediumSize;
		}

		> .sapUiIcon {
			font-size: 3rem;
		}
	}
}

// Styles & Sizes for Desktop
.sapUiBlockLayoutSizeL:not(.sapUiBlockLayoutKeepFontSize) {
	.sapUiNoContentPadding .sapUiBlockCellContent {
		padding: 0;
	}
	.sapUiBlockCellContent {
		padding: 2rem 3rem;
		font-size: @sapMFontLargeSize;

		> .sapMText {
			font-size: @sapMFontLargeSize;
		}

		> .sapUiIcon {
			font-size: 3.75rem;
		}
	}

	.sapUiBlockCellTitle,
	.sapUiBlockCellTitle .sapMLnk {
		font-size: @_sap_ui_layout_BlockLayout_FontSizeLandXL;
	}
}

// Styles & Sizes for LargeDesktop
.sapUiBlockLayoutSizeXL:not(.sapUiBlockLayoutKeepFontSize) {
	.sapUiNoContentPadding .sapUiBlockCellContent {
		padding: 0;
	}
	.sapUiBlockCellContent {
		padding: 2rem 3rem;
		font-size: @sapMFontLargeSize;

		> .sapMText {
			font-size: @sapMFontLargeSize;
		}

		> .sapUiIcon {
			font-size: 4.5rem;
		}
	}

	.sapUiBlockCellTitle,
	.sapUiBlockCellTitle .sapMLnk {
		font-size: @_sap_ui_layout_BlockLayout_FontSizeLandXL;
	}
}

/* ============ Background type: Dashboard ============== */
.sapUiBlockLayoutBackgroundDashboard {
	.sapUiBlockLayoutCell {
		font-weight: normal;
		color: @_sap_ui_layout_BlockLayout_BackgroundDashboardColor;

		// use padding and negative margin to display box-shadow outside of the cell, because it has overflow: hidden
		padding: @_sap_ui_layout_BlockLayout_BackgroundDashboardCellShadowOffset;
		margin: -@_sap_ui_layout_BlockLayout_BackgroundDashboardCellShadowOffset;
	}

	.sapUiBlockCellContent {
		background-color: @sapUiGroupContentBackground;
		box-shadow: @_sap_ui_layout_BlockLayout_BackgroundDashboardCellShadow;
		margin: @_sap_ui_layout_BlockLayout_BackgroundDashboardMargin;
		border-radius: @_sap_ui_layout_BlockLayout_BackgroundDashboardBorderRadius;
	}

	.sapUiBlockScrollingNarrowCells {

		& + .sapUiBlockHorizontalCellsRow .sapUiBlockCellContent {
			margin-top: @_sap_ui_layout_BlockLayout_BackgroundDashboardAfterScrollbarMargin;
		}
	}

	&:not(.sapUiBlockLayoutSizeS) {
		.sapUiBlockLayoutCell:first-of-type {
			margin-left: 0;
			padding-left: 0;

			.sapUiBlockCellContent {
				margin-left: @_sap_ui_layout_BlockLayout_BackgroundDashboardEdgeChildMargin; // Adjust first cell in a row with 0.5rem on the left
			}
		}

		.sapUiBlockLayoutCell:last-of-type {
			margin-right: 0;
			padding-right: 0;

			.sapUiBlockCellContent {
				margin-right: @_sap_ui_layout_BlockLayout_BackgroundDashboardEdgeChildMargin; // Adjust last cell in a row with 0.5rem on the right
			}
		}

		.sapUiBlockLayoutRow:first-of-type {
			.sapUiBlockLayoutCell {
				margin-top: 0;
				padding-top: 0;
			}

			.sapUiBlockCellContent {
				margin-top: @_sap_ui_layout_BlockLayout_BackgroundDashboardEdgeChildMargin; // Adjust spacing at the top for all cells in the first row
			}	
		}

		.sapUiBlockLayoutRow:last-of-type {
			.sapUiBlockLayoutCell {
				margin-bottom: 0;
				padding-bottom: 0;
			}

			.sapUiBlockCellContent {
				margin-bottom: @_sap_ui_layout_BlockLayout_BackgroundDashboardEdgeChildMargin; // Adjust spacing at the bottom for all cells in the last row
			}
		}
	}

	// Block Layout with size S (Phone) puts every cell into a single row
	// Therefore spacing should be altered in this case
	&.sapUiBlockLayoutSizeS {
		.sapUiBlockCellContent {
			margin-left: 0.5rem;
			margin-right: 0.5rem;
		}

		.sapUiBlockLayoutRow:first-of-type {
			.sapUiBlockLayoutCell {
				margin-top: 0;
				padding-top: 0;

				&:first-of-type .sapUiBlockCellContent {
					margin-top: 0.5rem; // Add 0.5rem spacing for the first cell in the first row
				}
			}
		}

		.sapUiBlockLayoutRow:last-of-type {
			.sapUiBlockLayoutCell {
				margin-bottom: 0;
				padding-bottom: 0;
	
				&:last-of-type .sapUiBlockCellContent {
					margin-bottom: 0.5rem; // Add 0.5rem spacing for the last cell in the last row
				}
			}
		}
	}
}
/* END "BlockLayout.less" */

