/*
 * Project: RooFit
 * Author:
 *   Will Buttinger, RAL 2022
 *
 * Copyright (c) 2022, CERN
 *
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted according to the terms
 * listed in LICENSE (http://roofit.sourceforge.net/license.txt)
 */

#ifdef __CLING__

#pragma link off all globals;
#pragma link off all classes;

#pragma link C++ class ROOT::Experimental::XRooFit::xRooFit + ;
#pragma link C++ class ROOT::Experimental::XRooFit::xRooNode + ;
#pragma link C++ class ROOT::Experimental::XRooFit::xRooNode::InteractiveObject + ;
#pragma link C++ class ROOT::Experimental::XRooFit::xRooNLLVar + ;
#pragma link C++ class ROOT::Experimental::XRooFit::xRooBrowser + ;
#pragma link C++ class ROOT::Experimental::XRooFit::xRooHypoSpace + ;
#pragma link C++ class ROOT::Experimental::XRooFit::xRooFit::StoredFitResult + ;

#endif
