// @(#)root/mathcore:$Id$
// Authors: W. Brown, M. Fischler, L. Moneta    2005

 /**********************************************************************
  *                                                                    *
  * Copyright (c) 2005   ROOT CERN / FNAL MATHLIB Group              *
  *                                                                    *
  *                                                                    *
  **********************************************************************/

// Header file defining forward declarations for LorentzVector classes
//
// Created by: Lorenzo Moneta    at Thu May 12 11:19:02 2005
// Major revamp by: M. Fischler  at Tue Aug  9  2005
//
// Last update: Tue Aug  9  2005
//
#ifndef ROOT_Math_GenVector_LorentzRotationfwd
#define ROOT_Math_GenVector_LorentzRotationfwd  1


namespace ROOT {
  namespace Math {
    class LoretzRotation;
  } // end namespace Math
} // end namespace ROOT


#endif /* ROOT_Math_GenVector_LorentzRotationfwd  */
