/*************************************************************************
 * Copyright (C) 1995-2006, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifdef __CLING__

#pragma link C++ struct ROOT::Experimental::RFitPanelModel+;
#pragma link C++ struct ROOT::Experimental::RFitPanelModel::RComboBoxItem+;
#pragma link C++ struct ROOT::Experimental::RFitPanelModel::RMethodInfo+;
#pragma link C++ struct ROOT::Experimental::RFitPanelModel::RMinimezerAlgorithm+;
#pragma link C++ struct ROOT::Experimental::RFitPanelModel::RItemInfo+;
#pragma link C++ struct ROOT::Experimental::RFitPanelModel::RFuncPar+;
#pragma link C++ struct ROOT::Experimental::RFitPanelModel::RFuncParsList+;
#pragma link C++ class ROOT::Experimental::RFitPanel+;

#endif
