#ifndef CLAD_REVERSE_MODE_VISITOR_DIRECTION_KINDS
#define CLAD_REVERSE_MODE_VISITOR_DIRECTION_KINDS

namespace clad {
  namespace rmv {
    /// An enum to operate between forward and reverse passes.
  enum direction : int { forward, reverse };
  } // namespace rmv
} // namespace clad

#endif