/*!
 * OpenUI5
 * (c) Copyright 2009-2025 SAP SE or an SAP affiliate company.
 * Licensed under the Apache License, Version 2.0 - see LICENSE.txt.
 */

@import "../base/library.source.less";
@import "../../../../sap/ui/core/themes/sap_horizon_dark/base.less";
@import "../../../../sap/ui/core/themes/sap_horizon_dark/global.less";

@import "InfoLabel.less";
@import "SideNavigation.less";
@import "NavigationList.less"; // keep this order, it reuses variable @_sap_tnt_SideNavigation_BoxShadow defined in SideNavigation.less
@import "ToolHeader.less";
@import "ToolPage.less";