// @(#)root/mathcore:$Id$
// Authors: W. Brown, M. Fischler, L. Moneta    2005

#ifndef ROOT_Math_Vector4D
#define ROOT_Math_Vector4D

// defines typedefs to specific vectors and forward declarations
// define additional (to Cartesian) coordinate system types
#include "Math/Vector4Dfwd.h"

// generic LorentzVector class definition
#include "Math/GenVector/PxPyPzE4D.h"
#include "Math/GenVector/PtEtaPhiE4D.h"
#include "Math/GenVector/PxPyPzM4D.h"
#include "Math/GenVector/PtEtaPhiM4D.h"

#include "Math/GenVector/LorentzVector.h"

#endif








