// @(#)root/geom:$Id$
// Author : Andrei Gheata 10/06/02
/*************************************************************************
 * Copyright (C) 1995-2000, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifdef __CLING__

#pragma link C++ class TGeoPatternFinder + ;
#pragma link C++ class TGeoPatternX + ;
#pragma link C++ class TGeoPatternY + ;
#pragma link C++ class TGeoPatternZ + ;
#pragma link C++ class TGeoPatternParaX + ;
#pragma link C++ class TGeoPatternParaY + ;
#pragma link C++ class TGeoPatternParaZ + ;
#pragma link C++ class TGeoPatternTrapZ + ;
#pragma link C++ class TGeoPatternCylR + ;
#pragma link C++ class TGeoPatternCylPhi - ;
#pragma link C++ class TGeoPatternSphR + ;
#pragma link C++ class TGeoPatternSphTheta + ;
#pragma link C++ class TGeoPatternSphPhi + ;
#pragma link C++ class TGeoPatternHoneycomb + ;
#pragma link C++ class TGeoNodeCache + ;
#pragma link C++ class TGeoCacheState + ;
#pragma link C++ class TVirtualMagField + ;
#pragma link C++ class TGeoUniformMagField + ;
#pragma link C++ class TGeoGlobalMagField;
#pragma link C++ class TGeoBranchArray + ;
#pragma link C++ class TGeoExtension + ;
#pragma link C++ class TGeoRCExtension + ;
#pragma link C++ class TGeoParallelWorld + ;

#endif
