// @(#)root/mathcore:$Id$
// Authors: M. Fischler   2005

#ifndef ROOT_Math_GenVector_Boostfwd
#define ROOT_Math_GenVector_Boostfwd  1


namespace ROOT {

  namespace Math {

  /**
      Class describing a pure boost in some direction
   */

class Boost;

}  // namespace Math
}  // namespace ROOT

#endif  // ROOT_Math_GenVector_Boostfwd
