/* NOTE: This file was generated as an optimized version of "library.source.less" for the Theme Designer. */

/*!
 * OpenUI5
 * (c) Copyright 2009-2025 SAP SE or an SAP affiliate company.
 * Licensed under the Apache License, Version 2.0 - see LICENSE.txt.
 */

@import "../base/library.less";
@import "../../../../../../Base/baseLib/sap_fiori_3/base.less"; /* ORIGINAL IMPORT PATH: "../../../../../sap/ui/core/themes/sap_fiori_3/base.less" */

@import "../../../../../sap/ui/core/themes/sap_fiori_3/global.less";
/* START "./CodeEditor.less" */
/*
/ ============================================ /
/ CSS for control sap.ui.codeeditor/CodeEditor /
/ Fiori 3 theme                                /
/ ============================================ /
*/
.sapCEdThemeDefault .ace_editor .ace_marker-layer .ace_selection {
	background: #EAECEE;
}
/* END "./CodeEditor.less" */

