#ifdef __CLING__
#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;
#pragma link C++ class TNeuron+;
#pragma link C++ class TSynapse+;
#pragma link C++ class TMultiLayerPerceptron+;
#pragma link C++ class TMLPAnalyzer+;
#endif
