#ifdef __CLING__

#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;

#pragma link C++ class TSQLiteServer+;
#pragma link C++ class TSQLiteResult+;
#pragma link C++ class TSQLiteRow+;
#pragma link C++ class TSQLiteStatement+;

#endif
