#ifdef __CLING__

#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;
#pragma link off all namespaces;

#pragma link C++ nestedclass;


// other classes
#pragma link C++ class TMVA::TSpline2+;
#pragma link C++ class TMVA::TSpline1+;
#pragma link C++ class TMVA::PDF+;
#pragma link C++ class TMVA::BinaryTree+;
#pragma link C++ class TMVA::BinarySearchTreeNode+;
#pragma link C++ class TMVA::BinarySearchTree+;
#pragma link C++ class TMVA::Timer+;
#pragma link C++ class TMVA::RootFinder+;
#pragma link C++ class TMVA::CrossEntropy+;
#pragma link C++ class TMVA::DecisionTree+;
#pragma link C++ class TMVA::DecisionTreeNode+;
#pragma link C++ class TMVA::MisClassificationError+;
#pragma link C++ class TMVA::Node+;
#pragma link C++ class TMVA::SdivSqrtSplusB+;
#pragma link C++ class TMVA::SeparationBase+;
#pragma link C++ class TMVA::RegressionVariance+;
#pragma link C++ class TMVA::Tools+;
#pragma link C++ class TMVA::Reader+;
#pragma link C++ class TMVA::GeneticAlgorithm+;
#pragma link C++ class TMVA::GeneticGenes+;
#pragma link C++ class TMVA::GeneticPopulation+;
#pragma link C++ class TMVA::GeneticRange+;
#pragma link C++ class TMVA::GiniIndex+;
#pragma link C++ class TMVA::GiniIndexWithLaplace+;
#pragma link C++ class TMVA::SimulatedAnnealing+;
#pragma link C++ class TMVA::QuickMVAProbEstimator+;

#endif
