/*!
 * OpenUI5
 * (c) Copyright 2009-2025 SAP SE or an SAP affiliate company.
 * Licensed under the Apache License, Version 2.0 - see LICENSE.txt.
 *//*!
 * OpenUI5
 * (c) Copyright 2009-2025 SAP SE or an SAP affiliate company.
 * Licensed under the Apache License, Version 2.0 - see LICENSE.txt.
 */.sapUiAccKeysHighlighDom:first-letter{text-decoration:underline 20% black}.sapCEdThemeDefault .ace_editor{background-color:#29313a;color:#fafafa;font-family:"72 Mono",'Monaco','Menlo','Ubuntu Mono','Consolas','Source Code Pro','source-code-pro',monospace}.sapCEdThemeDefault .ace_editor .ace_gutter{background:#3a4552;color:#fafafa}.sapCEdThemeDefault .ace_editor .ace_print-margin{width:1px;background:#3a4552}.sapCEdThemeDefault .ace_editor .ace_cursor{color:#fafafa}.sapCEdThemeDefault .ace_editor .ace_marker-layer .ace_selection{background:#3a4552}.sapCEdThemeDefault .ace_editor .ace_marker-layer .ace_bracket{margin:-1px 0 0 -1px;border:1px solid #abe2ab}.sapCEdThemeDefault .ace_editor .ace_marker-layer .ace_active-line{background:transparent}.sapCEdThemeDefault .ace_editor .ace_gutter-active-line{background-color:#29313a}.sapCEdThemeDefault .ace_editor .ace_keyword,.sapCEdThemeDefault .ace_editor .ace_meta,.sapCEdThemeDefault .ace_editor .ace_storage,.sapCEdThemeDefault .ace_editor .ace_storage.ace_type,.sapCEdThemeDefault .ace_editor .ace_support.ace_type{color:#bb8aaf}.sapCEdThemeDefault .ace_editor .ace_keyword.ace_operator{color:#21c4c4}.sapCEdThemeDefault .ace_editor .ace_constant.ace_character,.sapCEdThemeDefault .ace_editor .ace_constant.ace_language,.sapCEdThemeDefault .ace_editor .ace_constant.ace_numeric,.sapCEdThemeDefault .ace_editor .ace_keyword.ace_other.ace_unit,.sapCEdThemeDefault .ace_editor .ace_support.ace_constant,.sapCEdThemeDefault .ace_editor .ace_variable.ace_parameter{color:#ffb98d}.sapCEdThemeDefault .ace_editor .ace_constant.ace_other{color:#bd8969}.sapCEdThemeDefault .ace_editor .ace_fold{background-color:#c3c5fd;border-color:#c3c5fd}.sapCEdThemeDefault .ace_editor .ace_entity.ace_name.ace_function,.sapCEdThemeDefault .ace_editor .ace_support.ace_function{color:#6bd3ff}.sapCEdThemeDefault .ace_editor .ace_support.ace_class,.sapCEdThemeDefault .ace_editor .ace_support.ace_type{color:#d88b8b}.sapCEdThemeDefault .ace_editor .ace_heading,.sapCEdThemeDefault .ace_editor .ace_markup.ace_heading,.sapCEdThemeDefault .ace_editor .ace_string{color:#aabc78}.sapCEdThemeDefault .ace_editor .ace_entity.ace_name.ace_tag,.sapCEdThemeDefault .ace_editor .ace_entity.ace_other.ace_attribute-name,.sapCEdThemeDefault .ace_editor .ace_meta.ace_tag,.sapCEdThemeDefault .ace_editor .ace_string.ace_regexp,.sapCEdThemeDefault .ace_editor .ace_variable{color:#f58c4b}.sapCEdThemeDefault .ace_editor .ace_comment{color:#b0bcc5}.sapCEdThemeDefault .ace_editor .ace_marker-layer .ace_error_bracket{border-color:#f88}.sapCEdThemeDefault .ace_editor .ace_fold-widget{background-color:#29313a}.sapCEdThemeDefault .ace_editor .ace_fold-widget:hover{border-color:#abe2ab}.sapCEdThemeDefault .ace_editor .ace_indent-guide{position:relative}.sapCEdThemeDefault .ace_editor .ace_indent-guide::after{content:" ";position:absolute;top:0.0625rem;right:0;left:0;bottom:0;border-inline-end:.0625rem dotted #abe2ab;margin-inline-end:-0.0625rem}.sapCEd[data-sap-ui-syntaxhints='false'] .ace_gutter-cell{background-image:none !important}.sapCEd[data-sap-ui-syntaxhints='false'] .ace_tooltip{display:none !important}.ace_text-input{position:absolute !important}
/* Inline theming parameters */
#sap-ui-theme-sap\.ui\.codeeditor{background-image:url('data:text/plain;utf-8,%7B%7D')}
