/* @(#)root/meta:$Id: 4660ec009138a70261265c65fc5a10398706fbd1 $ */

/*************************************************************************
 * Copyright (C) 1995-2000, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifdef __CLING__

#pragma link C++ nestedtypedefs;
#pragma link C++ nestedclasses;

#pragma link C++ namespace ROOT;

#pragma link C++ enum EProperty;

#pragma link C++ class TBaseClass-;
#pragma link C++ class TClass;
#pragma link C++ class TClassStreamer+;
#pragma link C++ class TMemberStreamer+;
#pragma link C++ class TDictAttributeMap+;
#pragma link C++ class TClassRef+;
#pragma link C++ class TClassGenerator+;
#pragma link C++ class TDataMember-;
#pragma link C++ class TOptionListItem+;
#pragma link C++ class TDataType+;
#pragma link C++ class TDictionary+;
#pragma link C++ class TEnumConstant+;
#pragma link C++ class TEnum+;
#pragma link C++ class TFunction;
#pragma link C++ class TFunctionTemplate+;
#pragma link C++ class ROOT::TSchemaRule+;
#pragma link C++ class ROOT::TSchemaRule::TSources+;
#pragma link C++ class ROOT::Detail::TSchemaRuleSet-;
#pragma link C++ options=version(0) class TVirtualObject-;
#pragma link C++ options=version(0) class TVirtualArray-;
#pragma link C++ class TGlobal;
#pragma link C++ class TMethod;
#pragma link C++ class TMethodArg;
#pragma link C++ class TMethodCall;
#pragma link C++ class TInterpreter;
#pragma link C++ class TClassMenuItem;
#pragma link C++ class TVirtualIsAProxy;
#pragma link C++ class TVirtualStreamerInfo-;
#pragma link C++ class TIsAProxy;
#pragma link C++ class TProtoClass+;
#pragma link C++ class TProtoClass::TProtoRealData+;
#pragma link C++ class TRealData;
#pragma link C++ class TStreamerArtificial-;
#pragma link C++ class TStreamerBase-;
#pragma link C++ class TStreamerBasicPointer-;
#pragma link C++ class TStreamerLoop-;
#pragma link C++ class TStreamerBasicType-;
#pragma link C++ class TStreamerObject-;
#pragma link C++ class TStreamerObjectAny-;
#pragma link C++ class TStreamerObjectPointer-;
#pragma link C++ class TStreamerObjectAnyPointer-;
#pragma link C++ class TStreamerString-;
#pragma link C++ class TStreamerSTL-;
#pragma link C++ class TStreamerSTLstring-;
#pragma link C++ class TStreamerElement-;
#pragma link C++ class TToggle;
#pragma link C++ class TToggleGroup;
#pragma link C++ class std::vector<std::pair<Int_t, Int_t> >+;
#pragma link C++ class TFileMergeInfo;
#pragma link C++ class TListOfFunctions+;
#pragma link C++ class TListOfFunctionsIter;
#pragma link C++ class TListOfFunctionTemplates+;
#pragma link C++ class TListOfDataMembers-;
#pragma link C++ class TListOfEnums+;
#pragma link C++ class TListOfEnumsWithLock+;
#pragma link C++ class TListOfEnumsWithLockIter;
#pragma link C++ class ROOT::Detail::TStatusBitsChecker-;
//for new protoclasses
#pragma link C++ class std::vector<TDataMember * >+;
#pragma link C++ class std::vector<TProtoClass::TProtoRealData >+;

#endif
