/*!
 * OpenUI5
 * (c) Copyright 2009-2025 SAP SE or an SAP affiliate company.
 * Licensed under the Apache License, Version 2.0 - see LICENSE.txt.
 */

@import "../base/library.source.less";
@import "../../../../sap/ui/core/themes/sap_horizon_dark/base.less";
@import "../../../../sap/ui/core/themes/sap_horizon_dark/global.less";

@import "AvatarGroup.less";
@import "Card.less";
@import "CardHeaders.less";
@import "DynamicPage.less";
@import "DynamicPageHeader.less";
@import "DynamicPageTitle.less";
@import "FlexibleColumnLayout.less";
@import "GridList.less";
@import "GridListItem.less";
@import "ShellBar.less";
@import "SidePanel.less";