html { height: 100%; }
body { min-height: 100%; margin: 0; overflow: hidden }
#BrowserDiv {
    position: absolute;
    left: 0;
    top: 0;
    bottom: 0;
    right: 0;
    padding: 1
}

textarea {
    font-family: Courier New, Courier, monospace !important;
    padding: 4px 4px !important;
    background-color: #2f3c47 !important;
    color: #fafafa !important;
}

.sapMTabContainer {
    padding-top: 0 !important;
}

#TopBrowserId--SplitCont-Master, #TopBrowserId--SplitCont-Detail {
    margin-top: 48px;
}
.sapMScrollContScroll {
    height: 100%;
}

.sapMBreadcrumbs {
    margin: 0 !important;
}

.imageViewer {
    position: absolute;
    top: 50%;
    left: 50%;
    max-height: 95%;
    max-width: 95%;
    transform: translate(-50%, -50%);
}

.masterExpanded {
    width: 40% !important;
}