/* NOTE: This file was generated as an optimized version of "library.source.less" for the Theme Designer. */

/*!
 * OpenUI5
 * (c) Copyright 2009-2025 SAP SE or an SAP affiliate company.
 * Licensed under the Apache License, Version 2.0 - see LICENSE.txt.
 */
@import "../../../../../Base/baseLib/baseTheme/base.less"; /* ORIGINAL IMPORT PATH: "../../../../sap/ui/core/themes/base/base.less" */

@import "../../../../sap/ui/core/themes/base/global.less";
/* START "AnchorBar.less" */
/* =================================== */
/* CSS for control sap.uxap/AnchorBar  */
/* Base theme                          */
/* =================================== */

/**********************************
Common Compact/Cozy
**********************************/

/* BAR */

.sapUxAPAnchorBar {
	height: 3rem;
}

.sapUxAPAnchorBar.sapMTB {
	padding-left: 2rem;
	box-shadow: inset 0 -0.125rem @sapUiObjectHeaderBorderColor;
}

/* M, L or splitview paddings */
.sapUxAPObjectPageLayout-Std-Phone,
html .sapUiContainer-Narrow {
	.sapUxAPAnchorBar.sapMTB {
		padding-left: 0;
	}
}

.sapUxAPObjectPageLayout-Std-Tablet .sapUxAPAnchorBar.sapMTB,
html .sapUiContainer-Medium .sapUxAPAnchorBar.sapMTB {
	padding-left: 1rem;
}

.sapUxAPObjectPageLayoutTitleClickEnabled .sapUxAPAnchorBar,
.sapUxAPObjectPageLayoutHeaderPinnable .sapUxAPAnchorBar {
	margin-top: 0.8125rem;
}

.sapUxAPAnchorBar {
	vertical-align: middle;
	position: relative;
	width: 100%;
	padding: 0;
}

.sapUxAPAnchorBarScrollContainer {
	width: 100%;
}

/* leave space for the overflow button to the right */
.sapUxAPObjectPageLayout:not(.sapUxAPObjectPageLayout-Std-Phone) .sapUxAPAnchorBarOverflow {
	padding-right: 4rem;
}

/* not display the select if the bar does not overflow */
.sapUxAPAnchorBarOverflow:not(.sapUxAPAnchorBarScrollRight):not(.sapUxAPAnchorBarScrollLeft) .sapUxAPHierarchicalSelect {
	display: none;
}

/* BAR ITEMS */

.sapUxAPAnchorBar .sapUxAPAnchorBarScrollContainer .sapMBarChild {
	display: inline-block;
}

.sapMIBar.sapUxAPAnchorBar .sapMBarChild:first-child {
	margin-left: 0;

	&.sapUxAPHierarchicalSelect {
		padding-left: 1rem;
	}
}
.sapUxAPAnchorBar .sapUxAPAnchorBarButton.sapMMenuBtn {
	max-width: 100%;

	.sapMSB {
		padding-right: 1.5rem;
		width: auto !important;
	}

	.sapMSBText {
		padding-left: 1rem;
	}

	.sapMSB .sapMSBText.sapMSBTextIE {
		width: auto !important;
	}

	.sapMSB .sapMSBText.sapMSBTextIE,
	.sapMSB .sapMSBText:not(.sapMSBTextIE) .sapMBtnInner {
		margin-right: 1.25rem;
	}

	.sapMSBArrow {
		padding-right: 0.25rem;
	}

	.sapMSBText:focus >  .sapMBtnInner {
		outline: none;
	}

	.sapMSB:focus .sapMSBInner::after {
		top: 0.125rem;
		bottom: 0.125rem;
	}
}

.sapUxAPAnchorBar .sapUxAPAnchorBarButton {
	height: 3rem;
	padding: 0;

	.sapMBtnInner {
		padding-left: 1rem;
		padding-right: 1rem;
		height: 3rem;

		.sapMBtnContent,
		.sapMBtnIcon {
			height: 2.8rem;
			line-height: 3rem;
		}
	}

	.sapMBtn,
	.sapMSBText .sapMBtnInner,
	.sapMSBArrow .sapMBtnInner {
		padding: 0;
	}
}

.sapUxAPAnchorBar.sapMIBar.sapMTB .sapMBarChild {
	margin-left: 0;
	margin-right: 0;
}

.sapUxAPAnchorBar .sapUxAPAnchorBarButton .sapMBtnIcon {
	font-size: 0.75rem;
	width: 1.5rem;
	right: 0;
}

.sapUxAPAnchorBar .sapUxAPAnchorBarButton > .sapMBtnIconLast > .sapMBtnContent {
	padding-right: 1.375rem;
}

.sapUxAPAnchorBarUpperCase {
	text-transform: uppercase;
}

/* visual in anchor bar */

.sapUxAPAnchorBarButton .sapMBtnInner {
	background: transparent !important;
	border: none !important;
	text-shadow: none !important;
}

.sapUxAPAnchorBarButton {
	border: none;
}

.sapUxAPAnchorBar .sapMBtnInner,
.sapUxAPAnchorBar .sapMBtnInner:hover {
	background: transparent;
	border: none;
}

/***********************/
/* BEGIN Scroll Arrows */
/***********************/

// Not displayed on phone
.sapUxAPObjectPageLayout-Std-Phone .sapUxAPAnchorBar .anchorBarArrowLeft,
.sapUxAPObjectPageLayout-Std-Phone .sapUxAPAnchorBar .anchorBarArrowRight,
.sapUxAPObjectPageLayout-Std-Phone .sapUxAPAnchorBar .sapUxAPAnchorBarScrollContainer,
html .sapUiContainer-Narrow .sapUxAPAnchorBar .anchorBarArrowLeft,
html .sapUiContainer-Narrow .sapUxAPAnchorBar .anchorBarArrowRight,
html .sapUiContainer-Narrow .sapUxAPAnchorBar .sapUxAPAnchorBarScrollContainer {
	display: none;
}

// Gradient mask + arrows container
.sapUxAPAnchorBar.sapUxAPAnchorBarScrollLeft .anchorBarArrowLeft,
.sapUxAPAnchorBar.sapUxAPAnchorBarScrollRight .anchorBarArrowRight {
	width: 3rem;
	height: 3rem;
	position: absolute;
	top: 0;
	z-index: 1;
}

.sapUxAPAnchorBar.sapUxAPAnchorBarScrollRight .anchorBarArrowRight {
	right: 2.5rem;
}

// Tablet alignment
.sapUxAPObjectPageLayout-Std-Tablet .sapUxAPAnchorBar.sapUxAPAnchorBarScrollLeft .anchorBarArrowLeft,
html .sapUiContainer-Medium .sapUxAPAnchorBar.sapUxAPAnchorBarScrollLeft .anchorBarArrowLeft {
	left: 0;
}

// Hides the gradient mask + arrows container when it is not needed
.sapUxAPAnchorBar:not(.sapUxAPAnchorBarScrollLeft) .anchorBarArrowLeft,
.sapUxAPAnchorBar:not(.sapUxAPAnchorBarScrollRight) .anchorBarArrowRight {
	display: none;
}

// Scroll buttons
.sapUxAPAnchorBar.sapUxAPAnchorBarScrollLeft .anchorBarArrow .sapMBtn,
.sapUxAPAnchorBar.sapUxAPAnchorBarScrollRight .anchorBarArrow .sapMBtn {
	width: 1.625rem;
	height: 1.625rem;
	position: absolute;
	top: 0.6875rem;
	padding: 0;
}

.sapUxAPAnchorBar .anchorBarArrowLeft .sapMBtn {
	left: 0;
}

.sapUxAPAnchorBar .anchorBarArrowRight .sapMBtn {
	right: 0;
}

// Inner button elements
.sapUxAPAnchorBar.sapUxAPAnchorBarScrollLeft .anchorBarArrow .sapMBtnInner,
.sapUxAPAnchorBar.sapUxAPAnchorBarScrollRight .anchorBarArrow .sapMBtnInner {
	min-width: 1.625rem;
	width: 1.625rem;
	height: 1.625rem;
	padding: 0;
	border-radius: 0.125rem;
}

.sapUxAPAnchorBar.sapUxAPAnchorBarScrollLeft .anchorBarArrow .sapMBtnIcon,
.sapUxAPAnchorBar.sapUxAPAnchorBarScrollRight .anchorBarArrow .sapMBtnIcon {
	position: relative;
	font-size: 1rem;
	width: 1.625rem;
	height: 1.625rem;
	line-height: 1.625rem;
}

.sapUxAPAnchorBar.sapUxAPAnchorBarScrollLeft .anchorBarArrow .sapMBtnIcon::before,
.sapUxAPAnchorBar.sapUxAPAnchorBarScrollRight .anchorBarArrow .sapMBtnIcon::before {
	position: absolute;
	left: 0;
	top: 0;
	width: 1.625rem;
	height: 1.625rem;
}

// Scroll button container
.sapUxAPAnchorBar .anchorBarArrow > .sapUiHLayoutChildWrapper {
	width: 2rem;
	height: 3rem;
	position: absolute;
}

.sapUxAPAnchorBar .anchorBarArrowLeft > .sapUiHLayoutChildWrapper {
	left: 0;
}

.sapUxAPAnchorBar .anchorBarArrowRight > .sapUiHLayoutChildWrapper {
	right: 0;
}

/*********************/
/* END Scroll Arrows */
/*********************/

/**********************************
Compact
**********************************/

.sapUiSizeCompact {
	.sapUxAPAnchorBar .sapUxAPAnchorBarButton.sapMMenuBtn {
		.sapMSB .sapMSBText.sapMSBTextIE,
		.sapMSB .sapMSBText:not(.sapMSBTextIE) .sapMBtnInner {
			margin-right: 0.75rem;
		}
	}

	.sapUxAPObjectPageLayoutTitleClickEnabled .sapUxAPAnchorBar,
	.sapUxAPObjectPageLayoutHeaderPinnable .sapUxAPAnchorBar{
		margin-top: 0.5rem;
	}
}

.sapUiSizeCompact .sapUxAPAnchorBar,
.sapUiSizeCompact .sapUxAPAnchorBar .sapUxAPAnchorBarButton {
	height: 2rem;
}

.sapUiSizeCompact .sapUxAPAnchorBar .sapUxAPAnchorBarButton .sapMBtnText > .sapMBtnIcon {
	top: 0.1rem
}

.sapUiSizeCompact .sapUxAPAnchorBarButton .sapMBtnInner {
	padding-bottom: 1px;
	height: 2rem;
}

.sapUiSizeCompact .sapUxAPAnchorBar.sapUxAPAnchorBarScrollLeft .anchorBarArrowLeft,
.sapUiSizeCompact .sapUxAPAnchorBar.sapUxAPAnchorBarScrollRight .anchorBarArrowRight {
	width: 2rem;
	height: 2rem;
}

// Scroll buttons
.sapUiSizeCompact .sapUxAPAnchorBar.sapUxAPAnchorBarScrollLeft .anchorBarArrow .sapMBtn,
.sapUiSizeCompact .sapUxAPAnchorBar.sapUxAPAnchorBarScrollRight .anchorBarArrow .sapMBtn {
	top: 0.125rem;
}

.sapUiSizeCompact .sapUxAPAnchorBar.sapUxAPAnchorBarScrollLeft .anchorBarArrow .sapMBtnIcon,
.sapUiSizeCompact .sapUxAPAnchorBar.sapUxAPAnchorBarScrollRight .anchorBarArrow .sapMBtnIcon {
	margin-left: 0;
	margin-right: 0;
}

.sapUiSizeCompact .sapUxAPAnchorBar .sapUxAPAnchorBarButton .sapMBtnInner .sapMBtnContent,
.sapUiSizeCompact .sapUxAPAnchorBar .sapUxAPAnchorBarButton .sapMBtnInner .sapMBtnIcon {
	height: 1.8rem;
	line-height: 1.8rem;
}

.sapUiSizeCompact .sapUxAPAnchorBar .sapUxAPAnchorBarButton .sapMBtnInner.sapMBtnIconLast {
	padding-right: 0;
}

.sapUiSizeCompact .sapUxAPAnchorBar .sapUxAPAnchorBarButton .sapMBtnIcon {
	right: 0;
	line-height: 1.5rem;
	margin: 0;
}

.sapUiSizeCompact .sapUxAPAnchorBarButton > sapMBtnIconLast > .sapMBtnContent {
	padding-right: 0;
}

.sapUiSizeCompact .sapUxAPAnchorBarButton .sapMBtnContent {
	line-height: 1.375rem;
}

// Scroll button container
.sapUiSizeCompact .sapUxAPAnchorBar .anchorBarArrow > .sapUiHLayoutChildWrapper {
	height: 2rem;
}

//Override showing focus of the internal button of MenuButton in IE and Edge
html[data-sap-ui-browser^="ie"],
html[data-sap-ui-browser^="ed"] {
	.sapUxAPAnchorBarButton.sapMMenuBtn .sapMSBInner .sapMBtnFocusDiv {
		border: 0 none;
	}
}

// Styles needed for the anchorBarButtonColor property
.sapMIBar.sapUxAPAnchorBar .sapMBarChild.sapUxAPAnchorBarButton.sapUxAPAnchorBarButtonColorPositive:hover:not(.sapMBtnDisabled) .sapMBtnInner .sapMBtnContent > bdi,
.sapMIBar.sapUxAPAnchorBar .sapMBarChild.sapUxAPAnchorBarButton.sapUxAPAnchorBarButtonColorPositive .sapMBtnInner .sapMBtnContent,
.sapUiSizeCompact .sapMIBar.sapUxAPAnchorBar .sapMBarChild.sapUxAPAnchorBarButton.sapUxAPAnchorBarButtonColorPositive .sapMBtnInner .sapMBtnContent  {
	color: @sapUiPositiveText;
}

.sapMIBar.sapUxAPAnchorBar .sapMBarChild.sapUxAPAnchorBarButton.sapUxAPAnchorBarButtonColorNegative:hover:not(.sapMBtnDisabled) .sapMBtnInner .sapMBtnContent > bdi,
.sapMIBar.sapUxAPAnchorBar .sapMBarChild.sapUxAPAnchorBarButton.sapUxAPAnchorBarButtonColorNegative .sapMBtnInner .sapMBtnContent,
.sapUiSizeCompact .sapMIBar.sapUxAPAnchorBar .sapMBarChild.sapUxAPAnchorBarButton.sapUxAPAnchorBarButtonColorNegative .sapMBtnInner .sapMBtnContent {
	color: @sapUiNegativeText;
}

.sapMIBar.sapUxAPAnchorBar .sapMBarChild.sapUxAPAnchorBarButton.sapUxAPAnchorBarButtonColorCritical:hover:not(.sapMBtnDisabled) .sapMBtnInner .sapMBtnContent > bdi,
.sapMIBar.sapUxAPAnchorBar .sapMBarChild.sapUxAPAnchorBarButton.sapUxAPAnchorBarButtonColorCritical .sapMBtnInner .sapMBtnContent,
.sapUiSizeCompact .sapMIBar.sapUxAPAnchorBar .sapMBarChild.sapUxAPAnchorBarButton.sapUxAPAnchorBarButtonColorCritical .sapMBtnInner .sapMBtnContent  {
	color: @sapUiCriticalText;
}

.sapMIBar.sapUxAPAnchorBar .sapMBarChild.sapUxAPAnchorBarButton.sapUxAPAnchorBarButtonColorNeutral:hover:not(.sapMBtnDisabled) .sapMBtnInner .sapMBtnContent > bdi,
.sapMIBar.sapUxAPAnchorBar .sapMBarChild.sapUxAPAnchorBarButton.sapUxAPAnchorBarButtonColorNeutral .sapMBtnInner .sapMBtnContent,
.sapUiSizeCompact .sapMIBar.sapUxAPAnchorBar .sapMBarChild.sapUxAPAnchorBarButton.sapUxAPAnchorBarButtonColorNeutral .sapMBtnInner .sapMBtnContent   {
	color: @sapUiNeutralText;
}
/* END "AnchorBar.less" */

/* START "BreadCrumbs.less" */
/* ===================================== */
/* CSS for control sap.uxap/BreadCrumbs  */
/* Base theme                            */
/* ===================================== */

.sapUxAPBreadCrumbs {
	margin: 0 0 0.5rem 0;
	height: 1.25rem;

	.sapMLnk, .sapMText, .sapUxAPTubeIcon, .sapMSlt {
		vertical-align: middle;
	}

	& .sapMSlt {
		border: 0;
		margin: 0;
		height: 1.25rem;

		.sapMSltArrow {
			margin-right: -0.5rem;
			background-color: transparent;
			line-height: 1.25rem;
		}
		.sapMSltLabel {
			padding-left: 0;
			padding-right: 1.5rem;
			line-height: 1.25rem;
		}
	}

	& .sapMSltHoverable:hover:not(.sapMSltPressed) > .sapMSltArrow {
		background-color: transparent;
	}

	.sapUxAPBreadCrumbsDots {
		color: @sapUiLink;
		font-weight: normal;
		font-size: 0.875rem;
		text-decoration: none;
	}

	div:nth-child(1) {
		white-space: nowrap;
		padding-right: 2rem;
	}

	& > ul {
		list-style-type: none;
		display: inline;
		padding: 0;
		margin: 0;

		li {
			display: inherit;
			padding: 0;
			margin: 0;
		}
	}

	.sapUxAPTubeIcon {
		margin: 0 0.5rem 0 0.5rem;
	}

	&.sapUxAPFullWidth {
		width: 100%;
	}

	.sapUxAPInvisible {
		visibility: hidden;
	}
	
	.sapMText:focus {
		outline: 0.0625rem dotted @sapUiContentFocusColor;
		outline-offset: 0.2rem;
	}

	.sapMSlt {
		background-color: transparent;

		.sapMSltLabel {
			color: @sapUiContentLabelColor;
		}
	}

	.sapMSltPressed > .sapMSltArrow::before {
		color: @sapUiContentIconColor;
	}
}

.sapUiSizeCompact .sapUxAPBreadCrumbs & .sapMSlt {
	height: 1.625rem;
}
/* END "BreadCrumbs.less" */

/* START "HierarchicalSelect.less" */
/* ============================================ */
/* CSS for control sap.uxap/HierarchicalSelect  */
/* Base theme                                   */
/* ============================================ */

.sapUxAPHierarchicalSelect {
	width: 2rem;
	margin: 0;

	.sapMSltLabel {
		padding: 0 2.5rem 0 1rem;
	}

	.sapMSltIcon {
		width: 1.625rem;
		height: 1.625rem;
		line-height: 1.625rem;
		font-size: 1rem;
		padding: 0;
		border-radius: 0.125rem;
		position: absolute;
		top: 0.6875rem;

		&::before {
			width: 1.625rem;
			height: 1.625rem;
			position: absolute;
			top: 0;
			left: 0;
		}
	}

	&.sapMSltHoverable.sapMSltIconOnly.sapMSltPressed.sapMSlt.sapMSltFocused > .sapMSltIcon {
		outline: none;
	}
}

.sapUiSizeCompact .sapUxAPHierarchicalSelect.sapMSlt,
.sapUiSizeCompact .sapUiContainer-Narrow .sapUxAPHierarchicalSelect.sapMSlt {
	height: 2rem;
	margin-top: 0;
	margin-bottom: 0;
}

.sapUiSizeCompact .sapUxAPHierarchicalSelect .sapMSltIcon {
	top: 0.125rem;
}

/* select label */

.sapUxAPHierarchicalSelect,
.sapUxAPHierarchicalSelect .sapMSltLabel,
.sapUxAPHierarchicalSelect .sapMSltArrow {
	line-height: 3rem;
}

.sapUiSizeCompact .sapUxAPHierarchicalSelect,
.sapUiSizeCompact .sapUxAPHierarchicalSelect .sapMSltLabel,
.sapUiSizeCompact .sapUxAPHierarchicalSelect .sapMSltArrow {
	line-height: 2rem;
}

.sapUxAPHierarchicalSelect .sapMSltArrow,
.sapUiSizeCompact .sapUxAPHierarchicalSelect .sapMSltArrow {
	width: 2.5rem;
}

.sapMSltPicker .sapUxAPHierarchicalSelect.sapMSltBar::after {
	font-family: "SAP-icons";
	speak: none;
	content: "\e03e";
}

.sapUiSizeCompact .sapMSltPicker .sapUxAPHierarchicalSelect.sapMSltBar::after {
	line-height: 2rem;
}

/* OVERVIEW BUTTON */
.sapUxAPHierarchicalSelect.sapMSltIconOnly {
	position: absolute;
	right: 0;
	top: 0;
	height: 3rem;
}

.sapUiSizeCompact .sapUxAPHierarchicalSelect.sapMSltIconOnly {
	height: 2rem;
}

/* inner list */

.sapUxAPHierarchicalSelectFirstLevel.sapMSelectListItem,
.sapUxAPHierarchicalSelectSecondLevel.sapMSelectListItem {
	height: auto;
	border-bottom: none;
}

.sapUxAPHierarchicalSelectFirstLevel.sapMSelectListItem,
.sapUxAPHierarchicalSelectSecondLevel.sapMSelectListItem {
	white-space: normal;
	line-height: normal;
	padding: 1rem 0.5rem 1rem 1rem;
}

.sapUiSizeCompact .sapUxAPHierarchicalSelectFirstLevel.sapMSelectListItem,
.sapUiSizeCompact .sapUxAPHierarchicalSelectSecondLevel.sapMSelectListItem {
	padding: 0.5rem;
}

.sapUxAPHierarchicalSelectSecondLevel.sapMSelectListItem {
	text-indent: 1.5rem;
}

.sapUxAPHierarchicalSelectUpperCase .sapMSltLabel,
.sapMSltPickerFirstLevelUpperCase .sapMInputBaseInner,
.sapMSltPickerFirstLevelUpperCase .sapUxAPHierarchicalSelectFirstLevel.sapMSelectListItem {
	text-transform: uppercase;
}

/* styling needed to achieve underline of the select label on phone */
.sapUxAPObjectPageLayout-Std-Phone, .sap-phone {

	.sapUxAPHierarchicalSelect {
		display: flex;
		 .sapMSltLabel {
			position: relative;
			padding-left: 0;
			display: inline-block;
			vertical-align: top;
			line-height: 2.2rem;
			width: auto;
			padding-right: 0;
			overflow: visible;
		 	min-width: 0;
			.sapMSelectListItemText {
				display: inline-block;
				width: 100%;
				text-overflow: ellipsis;
				overflow: hidden;
				box-sizing: border-box;
			}
		}

		.sapMSltArrow {
			position: static;
			flex-shrink: 0;
			margin-left: auto;
		}
	}

	.sapMSlt.sapUxAPHierarchicalSelect.sapMBarChild {
		height: 3rem;
		border: none;
		padding-left: 1rem;
	}
}

.sap-phone .sapUiSizeCompact,
.sapUiSizeCompact .sapUxAPObjectPageLayout-Std-Phone {
	.sapUxAPHierarchicalSelect .sapMSltLabel {
		padding-bottom: 0.3rem;
	}

	.sapMSlt.sapUxAPHierarchicalSelect {
		height: 2rem;
	}
}

/* IE and Edge ignores outline-offset. Use an overlay: */
/* TODO remove after the end of support for Internet Explorer */
html[data-sap-ui-browser^="ie"].sap-desktop,
html[data-sap-ui-browser^="ed"].sap-desktop {
	.sapMSlt.sapUxAPHierarchicalSelect:focus  > .sapMSltArrow {
		box-shadow: none;
	}

	.sapUxAPHierarchicalSelect.sapMSltHoverable.sapMSltIconOnly.sapMSlt:not(.sapMSltPressed):focus {
		&:before {
			border: none;
		}
		> .sapMSltIcon {
			outline: none;
			position: relative;
		}
		> .sapMSltIcon:after {
			content: " ";
			box-sizing: border-box;
			width: 100%;
			height: 100%;
			position: absolute;
			left: 0;
			top: 0;
			border: 1px dashed @sapUiContentFocusColor;
			pointer-events: none;
		}
	}
}

/* END "HierarchicalSelect.less" */

/* START "ObjectPageHeader.less" */
/* ========================================== */
/* CSS for control sap.uxap/ObjectPageHeader  */
/* Base theme                                 */
/* ========================================== */

.sapUxAPObjectPageHeaderIdentifier {
	background-color: @sapUiObjectHeaderBackground;

	.sapUxAPObjectPageHeaderIdentifierContainer {
		.sapUxAPObjectPageHeaderIdentifierDescription {
			font-size: @sapMFontMediumSize;
		}
	}

	.sapUxAPObjectPageHeaderIdentifierActions {
		&.sapUxAPObjectPageHeaderIdentifierActionsForce, .sapUxAPObjectPageHeaderStickied & {
			display: inline-block;
		}
	}
}

html .sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageHeaderIdentifierTitle,
html .sapUiContainer-Narrow .sapUxAPObjectPageHeaderIdentifierTitle {
	font-size: @sapMFontHeader4Size;
}

.sapUxAPObjectPageHeaderSeparator {
	background-color: @sapUiToolbarSeparatorColor;
}

h2.sapUxAPObjectPageHeaderIdentifierTitle,
h2.sapUxAPObjectPageHeaderIdentifierTitleForce {
	margin: 0;
	font-weight: normal;
}

/*
Expose class for rounded images
*/
.sapUxAPObjectPageHeaderObjectImage-Circle,
.sapUxAPObjectPageHeaderObjectImage-Circle .sapUxAPObjectPageHeaderObjectImage {
	border-radius: 100%;
}

.sapUxAPObjectPageHeaderPlaceholder {
	display: inline-block;
	width: 3rem;
	height: 3rem;
}

.sapUxAPObjectPageHeaderObjectImage.sapUiIcon,
.sapUxAPObjectPageHeaderPlaceholder .sapUiIcon {
	align: center;
	vertical-align: center;
}

.sapUxAPObjectPageHeaderActionButtonHideIcon {
	.sapMBtnIcon {
		display: none;
	}
	.sapMBtnContent {
		padding: 0;
	}
}
// Action Button END

/*
objectPageHeader Title
*/
.sapUxAPObjectPageHeaderIdentifier {
	padding-top: 0.5rem;
	padding-bottom: 0.5rem;
	position: relative;
	display: none;
	&.sapUxAPObjectPageHeaderIdentifierForce, .sapUxAPObjectPageHeaderStickied & {
		display: block;
	}

	.sapUxAPObjectChildPage {
		margin-top: -0.5rem;
	}

	.sapUxAPObjectPageHeaderObjectImageForce .sapUxAPObjectPageHeaderObjectImage,
	.sapUxAPObjectPageHeaderStickied & .sapUxAPObjectPageHeaderObjectImage {
		width: 3rem;
		height: 3rem;
		display: inline-block;
		margin-right: 1rem;

		& > .sapFAvatar {
			width: 3rem;
			height: 3rem;
		}
	}

	.sapUxAPObjectPageHeaderStickied & .sapMImg.sapUxAPObjectPageHeaderObjectImage {
		width: 3rem;
		height: 3rem;
	}

	.sapUxAPObjectPageHeaderObjectImage {
		width: 0;
		height: 0;
		opacity: 0;
		display: inline-block;

		&.sapMNoImg {
			display: none;
			margin-right: 0;
		}
	}

	.sapUxAPObjectPageHeaderIdentifierContainer {
		display: inline-block;
		line-height: 1.375rem;
		vertical-align: middle;
		width: 60%; /* Breaking point TBD */
		box-sizing: border-box;
		padding-top: 1rem;
		padding-bottom: 1rem;
	}

	.sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageHeaderIdentifierContainer {
		padding-top: 1rem;
		padding-bottom: 1rem;
	}

	.sapUxAPObjectPageHeaderIdentifierActions {
		position: absolute;
		top: 0;
		padding-top: 0.25rem;
		right: 2.25rem;
		display: inline-block;
		float: right;
		vertical-align: top;
		> * {
			margin-left: 0.5rem;
		}
		> *:not(.sapMSegB) {
			padding: 0;
		}
		> *:first-child {
			margin-left: 0;
		}
	}

	.sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageHeaderIdentifierActions {
		padding-top: 0.25rem;
	}

	.sapUxAPObjectPageHeaderObjectImageContainer {
		.sapMImg {
			vertical-align: middle;
		}
		.sapUiIcon {
			vertical-align: middle;
			font-size: 1.5rem;
		}
	}
}

.sapUxAPObjectPageNoNavigation .sapUxAPObjectPageHeaderStickied .sapUxAPObjectPageHeaderIdentifier {
	box-shadow: @sapUiShadowHeader;
}

html[data-sap-ui-animation="on"] .sapUxAPObjectPageHeaderObjectImageForce .sapUxAPObjectPageHeaderObjectImage,
html[data-sap-ui-animation="on"] .sapUxAPObjectPageHeaderStickied & .sapUxAPObjectPageHeaderObjectImage {
	-webkit-transition: opacity 0.5s;
	transition: opacity 0.5s;
	opacity: 1;
}

html[data-sap-ui-animation="off"] .sapUxAPObjectPageHeaderObjectImageForce .sapUxAPObjectPageHeaderObjectImage,
html[data-sap-ui-animation="off"] .sapUxAPObjectPageHeaderStickied & .sapUxAPObjectPageHeaderObjectImage  {
	opacity: 1;
}

.sapUxAPObjectPageHeaderIdentifier {
	padding-left: 3rem;
}

.sapUiSizeCompact .sapUxAPObjectPageHeaderIdentifier {
	min-height:2.5625rem;
}

.sapUxAPObjectPageLayout-Std-Tablet .sapUxAPObjectPageHeaderIdentifier,
html .sapUiContainer-Medium .sapUxAPObjectPageHeaderIdentifier {
	padding-left: 2rem;
}

.sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageHeaderIdentifier,
html .sapUiContainer-Narrow .sapUxAPObjectPageHeaderIdentifier {
	padding-left: 1rem;
}

.sapUxAPObjectPageHeaderIdentifier .sapUxAPObjectPageHeaderIdentifierDescription {
	display: none;
	word-break: break-word;
	vertical-align: bottom;
	padding-top: 0.5rem;

	.sapUxAPObjectPageHeaderStickied & {
		display: inline-block;
	}
	&.sapUxAPObjectPageHeaderIdentifierDescriptionForce{
		display: block;
	}
}

.sapUxAPObjectPageHeaderIdentifierTitle {
	font-size: 1.375rem;
	padding-right: 1rem;
	display: none;
	vertical-align: middle;
	line-height: normal;
	max-width: 100%;
	.sapUxAPObjectPageHeaderStickied & {
		display: inline-block;
	}
	&.sapUxAPObjectPageHeaderIdentifierTitleForce {
		display: inline-block;
	}
}

.sapUxAPObjectPageHeaderObjectImageContainerSub > img {
	padding-top: 0.5rem;
	padding-bottom: 0.5rem;
}

/* TODO remove after the end of support for Internet Explorer */
html[data-sap-ui-browser^=ie] .sapUxAPObjectPageHeaderObjectImageContainerSub > img {
	padding-top: 0;
	padding-bottom: 0;
}

.sapUxAPObjectPageHeaderTitle {
	position: absolute;
	padding-right: 0px;
	top: 0;
	right: 0;
	left: 0;
	z-index: 2;
}

.sapUxAPObjectPageHeaderTitleArrow {
	vertical-align: top;
	white-space: nowrap;
	margin-left: 0;
}

.sapUxAPObjectPageHeaderTitleTextRestrictedWidth {
	overflow: hidden;
	text-overflow: ellipsis;
	display:inline-block;
	max-width: 100%;
}

.sapUxAPObjectPageHeaderTitleTextWrappable {
	white-space: normal;
}

.sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageHeaderTitleTextWrappable,
html .sapUiContainer-Narrow .sapUxAPObjectPageHeaderTitleTextWrappable {
	word-break: break-word;
}

.sapUxAPObjectPageHeaderIdentifierTitle .sapMObjStatusMarker {
	font-family: SAP-icons;
	font-size: 1rem;
	white-space: nowrap;
}

.sapUxAPObjectPageHeaderIdentifierTitleInContent .sapMObjStatusMarker .sapUiIcon,
.sapUxAPObjectPageHeaderIdentifier .sapMObjStatusMarker .sapUiIcon {
	display: inline;
	margin-left: 0.5rem;
}

html[dir=rtl] .sapUxAPObjectPageHeaderIdentifierTitleInContent .sapMObjStatusMarker .sapUiIcon,
html[dir=rtl] .sapUxAPObjectPageHeaderIdentifier .sapMObjStatusMarker .sapUiIcon {
	margin-left: 0.5rem;
}

.sapUxAPObjectPageHeaderPlaceholder {
	position: relative;
	vertical-align: middle;
}

html[dir=rtl] .sapUxAPObjectPageHeaderPlaceholder.sapUxAPObjectPageHeaderObjectImage .sapUiIcon.sapUiIconMirrorInRTL,
.sapUxAPObjectPageHeaderPlaceholder.sapUxAPObjectPageHeaderObjectImage .sapUiIcon {
	position: absolute;
	top: 50%;
	left: 50%;
	-webkit-transform: translate(-50%, -50%);
	transform: translate(-50%, -50%);
}

.sapUxAPObjectPageHeaderTitleArrow .sapMBtn {
	margin-top: -0.325rem;
	margin-left: 0.125rem;
	height: 2rem;
	.sapMBtnInner {
		height: 1.625rem;
		min-width: 2rem;
	}
	.sapMBtnIcon {
		line-height: 1.5rem;
		width: 1rem;
	}
}

.sapUiSizeCompact .sapUxAPObjectPageHeaderTitleArrow .sapMBtn {
	margin-top: -0.25rem;
	.sapMBtnIcon {
		font-size: 1.375rem;
	}
}

.sapUxAPObjectPageHeaderTitleFollowArrow > .sapUxAPObjectPageHeaderIdentifierTitle {
	padding-right: 0.5rem;
}

.sapUxAPObjectPageHeaderIdentifierDescription {
	vertical-align: baseline;
	padding-top: 0.5rem;
}

.sapUxAPObjectPageHeaderTitleFollowArrow ~ .sapUxAPObjectPageHeaderIdentifierDescription {
	margin-top: -0.3125rem;
	padding-top: 0;
}

.sapUiSizeCompact .sapUxAPObjectPageHeaderTitleFollowArrow ~ .sapUxAPObjectPageHeaderIdentifierDescription {
	margin-top: 0.25rem;
	padding-top: 0;
}

.sapUxAPObjectPageHeaderIdentifierTitleInContent {
	display: block;
	padding-top: 0;
}

.sapUxAPObjectPageHeaderIdentifierSubTitleInContent {
	display: block;
	margin-top: -0.3125rem;
}

.sapUxAPObjectPageHeaderTitleFollowArrow ~ .sapUxAPObjectPageHeaderIdentifierSubTitleInContent {
	margin-top: -0.625rem;
}

.sapUxAPObjectPageHeaderNowrapMarkers {
	white-space: nowrap;
}

.sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageHeaderNowrapMarkers,
html .sapUiContainer-Narrow .sapUxAPObjectPageHeaderNowrapMarkers {
	white-space: normal;
	word-break: break-all;
}

.sapUxAPObjectPageHeaderChangesBtn,
.sapUxAPObjectPageHeaderLockBtn {
	white-space: nowrap;
	margin-left: 0.125rem;
}

html[dir=rtl] .sapUxAPObjectPageHeaderChangesBtn,
html[dir=rtl] .sapUxAPObjectPageHeaderLockBtn {
	margin-left: 1px;
	margin-right: 0rem;
}

.sapUxAPObjectPageHeaderMarks .sapMObjStatusMarker .sapUiIcon:first-child {
	margin-left: 0.4375rem;
}

html[dir=rtl] .sapUxAPObjectPageHeaderMarks .sapMObjStatusMarker .sapUiIcon:last-child {
	margin-right: 0;
	margin-left: 0.4375rem;
}

html[dir=rtl] .sapUxAPObjectPageHeaderMarks .sapMObjStatusMarker .sapUiIcon:first-child {
	margin-left: 0.5rem;
	margin-right: 0rem;
}

.sapUxAPObjectPageHeaderObjectImage.sapUxAPObjectPageHeaderPlaceholder.sapUxAPHidePlaceholder {
	display: none;
}

.sapUxAPObjectPageHeaderExpandButton {
	visibility: hidden;
}

.sapUxAPObjectPageHeaderStickied .sapUxAPObjectPageHeaderExpandButton {
	visibility: visible;
}

.sapUxAPObjectPageHeaderSideContentBtn {
	float: right;
}

.sapUxAPObjectPageHeaderIdentifier .sapUxAPObjectPageHeaderIdentifierActions > .sapUxAPObjectPageHeaderSideContentBtn > .sapMBtn {
	padding-left: 0.5rem;
	float: left;
}

.sapUxAPObjectPageHeaderSeparator {
	float: left;
	height: 2rem;
	margin-top: 0.5rem;
	width: 1px;
	visibility: hidden;
}


.sapUxAPObjectPageHeaderSideContentBtn > .sapUxAPObjectPageHeaderSeparator:only-of-type {
	visibility: visible;
}

.sapUiSizeCompact .sapUxAPObjectPageHeaderSeparator {
	height: 1.5rem;
	margin-top: 0.25rem;
}

.sapUiSizeCompact .sapUxAPObjectPageHeaderIdentifier .sapUxAPObjectPageHeaderIdentifierActions {
	padding-top: 0.75rem;
}

/* Fiori 2.0 adaptation */
.sapF2CollapsedHeader .sapUxAPObjectPageHeaderNavigation > .sapMBar {
	display: none;
}

.sapF2Adapted .sapUxAPObjectPageHeaderNavigation > .sapMIBar {
  background: transparent;
  box-shadow: none;
}

/* Phone Size */
.sapUxAPObjectPageLayout-Std-Phone {

	.sapUxAPObjectPageHeaderTitle.sapUxAPObjectPageHeaderSnappedTitleOnMobile > .sapFDynamicPageTitle {
		padding: 0 0.25rem 0 1rem;
		min-height: 2rem;
		height: 2rem;
	}
}

/* Tablet Size */
.sapUxAPObjectPageLayout-Std-Tablet {

	// Needed because on some high-end phones in landscape mode,
	// the width of the viewport is bigger than 600 (Tablet breakpoint).
	.sapUxAPObjectPageHeaderTitle.sapUxAPObjectPageHeaderSnappedTitleOnMobile > .sapFDynamicPageTitle {
		padding: 0 0.25rem 0 2rem;
		min-height: 2rem;
		height: 2rem;
	}
}
/* END "ObjectPageHeader.less" */

/* START "ObjectPageHeaderContent.less" */
/* ================================================= */
/* CSS for control sap.uxap/ObjectPageHeaderContent  */
/* Base theme                                        */
/* ================================================= */

.sapUxAPObjectPageHeaderContent {
	position: relative;

	.sapUxAPObjectPageHeaderContentItem {
		display: inline-block;
		vertical-align: top;
		margin-right: 2rem;
		margin-bottom: 1rem;
		line-height: 1.5rem;
	}


	.sapUxAPObjectPageHeaderContentItem.sapFAvatar .sapFAvatarMagnifyingGlass,
	.sapUxAPObjectPageHeaderContentItem .sapFAvatar .sapFAvatarMagnifyingGlass {
		line-height: 0.8125rem;
	}

	.sapUxAPObjectPageHeaderContentItem .sapMText,
	.sapUxAPObjectPageHeaderContentItem.sapMObjStatus:not(.sapMObjectStatusLarge) span {
		line-height: 1.5rem;
	}

	.sapUxAPObjectPageHeaderContentItem.sapUiRespGrid {
		display: inherit;
	}

	.sapUxAPObjectPageHeaderSeparatorContainer {
		position: relative;
		margin-right: 4rem;
		.sapUxAPObjectPageHeaderSeparatorBefore {
			width: 1px;
			background-color: @sapUiToolbarSeparatorColor;
			display: inline-block;
			position: absolute;
			top: 0;
			bottom: 1rem;
			margin-right: 1rem;
			left: -2rem;
		}
		.sapUxAPObjectPageHeaderSeparatorAfter {
			width: 1px;
			background-color: @sapUiToolbarSeparatorColor;
			display: inline-block;
			position: absolute;
			top: 0;
			bottom: 1rem;
			margin-left: 1rem;
			right: -2rem;
		}
	}

	&, > .sapUxAPObjectPageHeaderSeparatorContainer, > .sapUxAPObjectPageHeaderWidthContainer {
		.sapUiVlt .sapUiVltCell {
			> * {
				line-height: normal;
			}
		}
		.sapUiHLayoutChildWrapper {
			min-height: 2rem;
			min-width: 2rem;
			& > .sapMImg {
				vertical-align: middle;

			}
		}
		> .sapUiSimpleForm {

			.sapUiFormResLayout > div {
				padding: 0;
				background-color: transparent;
			}

			.sapUiFormResLayout .sapUiRFLRow > .sapUiRFLContainer.sapUiRFLFullLength > .sapUiRFLContainerContent > .sapUiRLElement > .sapUiRFLRow:last-child > .sapUiRFLFullLength:last-child {
				padding-bottom: 0;
			}

			.sapMText, .sapMLabel, .sapMLnk {
				font-family: @sapUiFontFamily;
				font-weight: normal;
				height: 1.5rem;
				line-height: 1.5rem;
			}
		}
		.sapMOH {
			margin: 0;
			padding: 0;
			line-height: inherit;
			.sapMOHTitleArrow {
				right: 0.5rem;
				margin-left: 0;
			}
		}


		.sapUxAPObjectPageHeaderContentItem {
			.sapMObjectAttributeDiv .sapMText,
			.sapMObjectAttributeDiv .sapMObjectAttributeTitle,
			.sapMObjectAttributeDiv .sapMObjectAttributeColon,
			.sapMObjectAttributeDiv .sapMObjectAttributeText {
				vertical-align: middle;
			}
		}

		.sapUiVltCell > .sapMObjectAttributeDiv, .sapUiVltCell > .sapMObjStatus {
			line-height: 1rem;
		}

		/* ProgressIndicator has display flex, but OPHeaderContent override it by setting display inline-block
		* for each content entry to order the entries horizontally.
		* In order to keep the display flex behaviour of the PI and the inline-block behaviour inside OPHeaderContent,
		* we use inline-flex.
		* NOTE: It`s needed in case PI is direct child of OPHeaderContent only.
		*/
		> .sapMPI {
			display: -webkit-inline-flex;
			display: inline-flex;
			margin-top: 0.25rem;
		}

		.sapMPI {
			width: 10rem !important; /* Progress indicator always use the size provide in the control otherwise... */
			height: 1rem;
			position: relative;
			margin-bottom: 1rem;
			.sap-phone & {
				width: 100% !important;
			}
		}
	}

	.sapUxAPObjectPageHeaderLayoutHiddenL,
	.sapUxAPObjectPageHeaderLayoutHiddenM,
	.sapUxAPObjectPageHeaderLayoutHiddenS {
		display: none;
	}

	.sapUxAPObjectPageHeaderObjectImage.sapMImgFocusable {
		margin-top: 1px;
	}
}

.sapUxAPObjectPageHeaderContent {
	padding-left: 3rem;
}

.sapUxAPObjectPageLayout-Std-Tablet .sapUxAPObjectPageHeaderContent,
html .sapUiContainer-Medium .sapUxAPObjectPageHeaderContent {
	padding-left: 2rem;
}

.sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageHeaderContent,
html .sapUiContainer-Narrow .sapUxAPObjectPageHeaderContent {
	padding-left: 1rem;
}

.sapUxAPObjectPageNoNavigation .sapUxAPObjectPageHeaderContent {
	box-shadow: @sapUiShadowHeader;
}

.sapUxAPObjectPageLayoutNoAnchorBar .sapUxAPObjectPageHeaderContent {
	box-shadow: var(--sapContent_HeaderShadow);
}

.sapUxAPObjectPageHeaderContentHidden {
	display: none;
}

/* content separators */
.sapUxAPObjectPageHeaderDetailsDesign-Dark .sapUxAPObjectPageHeaderSeparatorContainer {
	.sapUxAPObjectPageHeaderSeparatorBefore,
	.sapUxAPObjectPageHeaderSeparatorAfter {
		background-color: transparent;
	}
}

.sapUxAPObjectPageHeaderContentImageContainer .sapUiIcon,
.sapUxAPObjectPageHeaderContentImageContainer .sapMImg {
	margin: 0 1rem 0.5rem 0;
}

.sapUxAPObjectPageHeaderContentImageContainer .sapUxAPObjectPageHeaderPlaceholder .sapUiIcon {
	margin: 0;
}

.sapUxAPObjectPageHeaderContentImageContainer .sapMImg {
	height: 5rem;
	width: 5rem;
}

.sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageHeaderContentImageContainer .sapMImg,
html .sapUiContainer-Narrow .sapUxAPObjectPageHeaderContentImageContainer .sapMImg {
	height: 3rem;
	width: 3rem;
}

.sapUxAPObjectPageHeaderContentImageContainer > .sapMImg.sapUxAPObjectPageHeaderObjectImage.sapMNoImg {
	display: none;
	margin: 0;
}

/* In this particular case disply none doesn't work corectly in the current version of chrome (43.0.2357.132) */
html[data-sap-ui-browser^="cr4"] .sapUxAPObjectPageHeaderContentImageContainer > .sapMImg.sapUxAPObjectPageHeaderObjectImage.sapMNoImg {
	width: 0;
	height: 0;
	border: none;
	display: inline-block;
}

.sapUxAPObjectPageHeaderLayoutTitleContainer {
	display: inline-block;
}

.sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderPlaceholder {
	width: 5rem;
	height: 5rem;
	box-sizing: border-box;
	margin-right: 2rem;
	margin-bottom: 1rem;
}

.sapUxAPObjectPageLayout-Std-Tablet .sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderPlaceholder,
html .sapUiContainer-Medium .sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderPlaceholder {
	width: 5rem;
	height: 5rem;
}

.sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderPlaceholder,
html .sapUiContainer-Narrow .sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderPlaceholder {
	width: 3rem;
	height: 3rem;
	margin-right: 1rem;
	margin-bottom: 0.5rem;

	& > .sapFAvatar {
		width: 3rem;
		height: 3rem;
	}
}

.sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderContentImageContainer .sapUiIcon,
.sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderContentImageContainer .sapUxAPObjectPageHeaderPlaceholder.sapUxAPObjectPageHeaderObjectImage .sapUiIcon {
	font-size: 2.5rem;
}

.sapUxAPObjectPageLayout-Std-Tablet .sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderContentImageContainer .sapUiIcon,
.sapUxAPObjectPageLayout-Std-Tablet .sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderContentImageContainer .sapUxAPObjectPageHeaderPlaceholder.sapUxAPObjectPageHeaderObjectImage .sapUiIcon,
html .sapUiContainer-Medium .sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderContentImageContainer .sapUiIcon,
html .sapUiContainer-Narrow .sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderContentImageContainer .sapUxAPObjectPageHeaderPlaceholder.sapUxAPObjectPageHeaderObjectImage .sapUiIcon {
	font-size: 2.5rem;
}

.sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderContentImageContainer .sapUiIcon,
.sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderContentImageContainer .sapUxAPObjectPageHeaderPlaceholder.sapUxAPObjectPageHeaderObjectImage .sapUiIcon,
html .sapUiContainer-Narrow .sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderContentImageContainer .sapUiIcon,
html .sapUiContainer-Narrow .sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderContentImageContainer .sapUxAPObjectPageHeaderPlaceholder.sapUxAPObjectPageHeaderObjectImage .sapUiIcon {
	font-size: 1.5rem;
}

.sapUxAPObjectPageHeaderContent .sapSuiteDmc.sapUxAPObjectPageHeaderContentItem,
.sapUxAPObjectPageHeaderContent .sapSuiteBCContent.sapUxAPObjectPageHeaderContentItem,
.sapUxAPObjectPageHeaderContent .sapSuiteCmc.sapUxAPObjectPageHeaderContentItem,
.sapUxAPObjectPageHeaderContent .sapSuiteCmpChartContent.sapUxAPObjectPageHeaderContentItem,
.sapUxAPObjectPageHeaderContent .sapSuiteMac.sapUxAPObjectPageHeaderContentItem,
.sapUxAPObjectPageHeaderContent .suiteHBMC.sapUxAPObjectPageHeaderContentItem {
	line-height: normal;
}

.sapUxAPObjectPageHeaderContentFlexBox {
	display: -webkit-box; /* OLD - iOS 6-, Safari 3.1-6 */
	display: flex;
}

.sapUxAPObjectPageHeaderContentCellLeft {
	-webkit-flex-grow: 1;
	flex-grow: 1;
	min-width: 25%;
}

.sapUxAPObjectPageHeaderContentCellRight {
	-webkit-flex-grow: 0;
	-webkit-flex-shrink: 0;
	-webkit-align-self: flex-end;
	flex-grow: 0;
	flex-shrink: 0;
	padding-right: 1rem;
	align-self: flex-end;
}

.sapUxAPObjectPageHeaderDetailsHidden > * {
  visibility: hidden;
}

.sapUiLoSplitterH.sapUiLoSplitter .sapUxAPObjectPageHeaderContent {
	white-space: normal;
}

.sapUxAPObjectPageHeaderContentDesign-Light,
.sapUxAPObjectPageHeaderContentDesign-Dark {
	background-color: @sapUiObjectHeaderBackground;
}
/* END "ObjectPageHeaderContent.less" */

/* START "ObjectPageLayout.less" */
/* ========================================== */
/* CSS for control sap.uxap/ObjectPageLayout  */
/* Base theme                                 */
/* ========================================== */
@_sap_uxap_ObjectPageLayout_FooterHeight: 3rem;
@_sap_uxap_ObjectPageLayout_FooterHeight_Compact: 2.5rem;
@_sap_uxap_ObjectPageLayout_FooterMarginBottom: 0.5rem;

.sapUxAPObjectPageLayout {
	width: 100%;
	height: 100%;
	position: relative;
    display: -webkit-box;
    display: -webkit-flex;
    display: flex;
    -webkit-flex-direction: column;
    flex-direction: column;
	isolation: isolate;
	z-index: 0;
	white-space: normal; // BCP: 1780382804

	/* ObjectAttribute will not truncate inside ObjectPage but rather will go on more lines */
	.sapUxAPObjectPageHeaderDetails {
		.sapMObjectAttributeActive .sapMObjectAttributeTitle,
		.sapMObjectAttributeActive .sapMObjectAttributeText {
			display: inline;
			position: relative;
			white-space: pre-line;
			word-wrap: break-word;
		}
	}
}

.sapUxAPObjectPageLayout > .sapUiLocalBusyIndicator:focus  {
	z-index: 5;
}

.sapUxAPObjectPageLayoutFooterVisible .sapUxAPObjectPageWrapper {
	scroll-padding-bottom: ~"calc(@{_sap_uxap_ObjectPageLayout_FooterHeight} +  @{_sap_uxap_ObjectPageLayout_FooterMarginBottom} + 2px)";
}

.sapUiSizeCompact .sapUxAPObjectPageLayoutFooterVisible .sapUxAPObjectPageWrapper {
	scroll-padding-bottom: ~"calc(@{_sap_uxap_ObjectPageLayout_FooterHeight_Compact} +  @{_sap_uxap_ObjectPageLayout_FooterMarginBottom} + 2px)";
}

.sapUxAPObjectPageLayout:not(.sapUxAPObjectPageLayout-Std-Phone):not("sapUxAPObjectPageLayoutIconTabBar") .sapUxAPObjectPageSubSectionPromoted:not(.sapUxAPObjectPageSectionMultilineContent) .sapUxAPObjectPageSubSectionHeaderActions {
	position: absolute;
	top: 0;
	right: 1rem;
}
.sapUxAPObjectPageLayout:not(.sapUxAPObjectPageLayout-Std-Phone):not("sapUxAPObjectPageLayoutIconTabBar") .sapUxAPObjectPageSubSectionPromoted:not(.sapUxAPObjectPageSectionMultilineContent) .sapUxAPObjectPageSubSectionHeader {
	height: 0;
	padding: 0;
}

body > .sapUxAPObjectPageLayout,
.sapUiVltCell .sapUxAPObjectPageLayout {
	position: fixed;
}

.sapUxAPObjectPageHeaderTitle {
    flex-shrink: 0;
    -webkit-flex-shrink: 0;
}

.sapUxAPObjectPageLayoutTitleClickEnabled > .sapUxAPObjectPageHeaderStickied > .sapFDynamicPageTitle {
	box-shadow: @sapUiShadowHeader;
}

.sapUxAPObjectPageLayout:not(.sapUxAPObjectPageLayoutTitleClickEnabled) .sapFDynamicPageHeaderPinButton {
	margin-left: -1.25rem;
}

.sapUxAPObjectPageSubSectionPromoted:not(:first-of-type) {
	padding-top: 0.25rem;
}

.sapUxAPObjectPageWrapper {
    position:relative;
    height:100%;
    -webkit-flex-grow: 1;
    flex-grow: 1;
	overflow-y: auto;
	background-color: @sapUiGlobalBackgroundColor;
}

.sapUxAPObjectPageWrapperWillChangeScrollTop {
	/* Performance tweak */
	will-change: scroll-position;
}

.sapUxAPObjectPageScroll {
	position: relative;
}

.sapUxAPObjectPageContainerNoBar > .sapUxAPObjectPageSection:first-child:not(.sapUxAPObjectPageSectionNoTitle) {
	margin-top:  2rem;
}

/****
    sapUxAPObjectPageScroll copied from sapMScroll for now
****/

.sapUxAPObjectPageScrollV,
.sapUxAPObjectPageScrollH,
.sapUxAPObjectPageScrollV > div, .sapUxAPObjectPageScrollH > div {
	position: absolute;
}

.sapUxAPObjectPageScrollV > div, .sapUxAPObjectPageScrollH > div {
	box-sizing: border-box;
	-webkit-background-clip: padding-box;
	background-clip: padding-box;
	border-width: 0.0625rem;
	border-style: solid;
	border-color: rgba(255, 255, 255, 0.901961);
	border-radius: 0.1875rem;
	background-color: rgba(0, 0, 0, 0.498039)
}

.sapUxAPObjectPageScrollV {
	width: 0.438rem;
	right: 0.063rem;
	top: 0;
	bottom: 0
}

.sapUxAPObjectPageScrollV > div {
	width: 100%
}

.sapUxAPObjectPageScrollH {
	height: 0.4375rem;
	bottom: 0.0625rem;
	left: 0.125rem;
	right: 0.125rem
}

.sapUxAPObjectPageScrollH > div {
	height: 100%
}

.sapUxAPObjectChildPage {
	width: 0.5rem;
	height: 100%;
	position: absolute;
	left: 0;
	right: auto;
	z-index: 3000;
	background: @sapUiObjectHeaderBorderColor;
}

.sapUxAPObjectPageLayout.sapUxAPObjectPageLayoutTitleClickEnabled {
	// title hover
	.sapFDynamicPageTitle:hover,
	&.sapUxAPObjectPageTitleForceHovered .sapFDynamicPageTitle {
		cursor: pointer;
	}
}

// anchorBar
.sapUxAPObjectPageLayoutTitleClickEnabled .sapUxAPObjectPageNavigation .sapMITH,
.sapUxAPObjectPageLayoutHeaderPinnable .sapUxAPObjectPageNavigation .sapMITH {
	margin-top: 0.25rem;
}

.sapUiSizeCompact {
	.sapUxAPObjectPageLayoutTitleClickEnabled .sapUxAPObjectPageNavigation .sapMITH,
	.sapUxAPObjectPageLayoutHeaderPinnable .sapUxAPObjectPageNavigation .sapMITH {
		margin-top: 0.0625rem;
	}
}

// footer
.sapUxAPObjectPageLayout {
	.sapUxAPObjectPageFooter {
		position: absolute;
		height: @_sap_uxap_ObjectPageLayout_FooterHeight;
		bottom: 0;
		right: 0;
		left: 0;
		margin-left: 0.5rem;
		margin-right: 0.5rem;
		margin-bottom: @_sap_uxap_ObjectPageLayout_FooterMarginBottom;
		z-index:1001;

		.sapUxAPObjectPageFloatingFooter {
			border: 1px solid;
			border-radius: 0.25rem;
		}
	}
}

.sapUiSizeCompact .sapUxAPObjectPageLayout .sapUxAPObjectPageFooter {
	.sapUxAPObjectPageFloatingFooter {
		height: @_sap_uxap_ObjectPageLayout_FooterHeight_Compact;

		// Reflection: styles overwritten
		&.sapMBar-CTX {
			// BCP: 1870106645
			// When sap.m.Bar is used as footer (in compact) setting 2.5rem height
			// breaks the Bar`s content vertical alignment.
			// Fixing this internally in the sap.m.Bar (in the context of flaoting footer)
			// will require significant effort, so we set height of 2rem (the expected Bar height by the visual spec).
			height: 2rem;
		}
	}
	height: 2.5rem;
}

html[data-sap-ui-animation='on'] .sapUxAPObjectPageLayout {
	.sapUxAPObjectPageFloatingFooterShow {
		-webkit-animation: bounceShow 0.35s forwards ease-in-out;
		animation: bounceShow 0.35s forwards ease-in-out;
	}
	.sapUxAPObjectPageFloatingFooterHide {
		-webkit-animation: bounceHide 0.35s forwards ease-in-out;
		animation: bounceHide 0.35s forwards ease-in-out;
	}

	@-webkit-keyframes bounceShow {
		0% {
			-webkit-transform: translateY(100%);
			transform: translateY(100%);
			opacity: 0;
		}
		100% {
			opacity: 1;
		}
	}
	@keyframes bounceShow {
		0% {
			transform: translateY(100%);
			opacity: 0;
		}
		100% {
			opacity: 1;
		}
	}
	@-webkit-keyframes bounceHide {
		0% {
			-webkit-transform: translateY(-5%);
			transform: translateY(-5%);
			opacity: 1;
		}
		100% {
			-webkit-transform: translateY(100%);
			transform: translateY(100%);
			opacity: 0;
		}
	}
	@keyframes bounceHide {
		0% {
			transform: translateY(-5%);
			opacity: 1;
		}
		100% {
			transform: translateY(100%);
			opacity: 0;
		}
	}
}
/* Only on Windows 8 with IE 11 there's some weird incompatibility between the usage of the transform property
 in the sapMShell and the one in the objectPageWrapper.
 Also the z-index causes the scroll bars of the table to show in the headerTitle.
 For more information see internal incident 1670194272. */
 /* TODO remove after the end of support for Internet Explorer */
html[data-sap-ui-browser^="ie"] .sapMShell {

  .sapUxAPObjectPageHeaderTitle {
    z-index: 3;
  }
}

.ui-helper-clearfix:before,
.ui-helper-clearfix:after {
	content: "";
	display: table;
}

.ui-helper-clearfix:after {
	clear: both;
}

.ui-helper-clearfix {
	zoom: 1;
}

/* END "ObjectPageLayout.less" */

/* START "ObjectPageSection.less" */
/* =========================================== */
/* CSS for control sap.uxap/ObjectPageSection  */
/* Base theme                                  */
/* =========================================== */

.sapUxAPObjectPageSection {
	margin-left: 0;
	margin-bottom: 0;
	position: relative;
}

.sapUxAPObjectPageSectionTitleUppercase {
	text-transform: uppercase;
}

.sapUxAPObjectPageSectionHeader {
	box-sizing: border-box;
	height: 3rem;
	line-height: 3rem;
	padding: 0 3rem 0 3rem;
}

.sapUxAPObjectPageSectionHeading {
	padding: 0.5rem 3rem;
}

/* header height is involved in calculation of the scroll position
so we reset it when the section header is hidden */
.sapUxAPObjectPageSectionHeaderHidden {
	overflow: hidden;
	height: 0 !important;
	padding: 0 !important;
	border: none !important;
}

.sapUxAPObjectPageLayout-Std-Tablet .sapUxAPObjectPageSectionHeader,
html .sapUiContainer-Medium .sapUxAPObjectPageSectionHeader {
	padding: 0 0 0.5rem 2rem;
}

.sapUxAPObjectPageLayout-Std-Tablet .sapUxAPObjectPageSectionHeading {
	padding: 0.5rem 0 0.5rem 2rem;
}

.sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageSectionHeader,
html .sapUiContainer-Narrow .sapUxAPObjectPageSectionHeader {
	padding: 0 0 0.5rem 1rem;
}

.sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageSectionHeading {
	padding: 0.5rem 0 0.5rem 1rem;
}

.sapUxAPObjectPageSectionTitle {
	box-sizing: border-box;
	display: inline-block;
	height: 3rem;
	max-width: 100%;
	overflow: hidden;
	white-space: nowrap;
	text-overflow: ellipsis;
}

.sapUxAPSectionShowHideButton {
	float: right;
}

.sapUxAPSectionShowHideButton:not(:last-child) {
	float: right;
	margin-left: 0.5rem;
}

.sapUxAPObjectPageSection .sapUxAPObjectPageSectionContainer > .sapUiRespGrid,
.sapUxAPObjectPageSection .sapUxAPObjectPageSectionContainer > .sapUiRespGrid > .sapUiRespGridSpanXL12.sapUiRespGridSpanL12.sapUiRespGridSpanM12.sapUiRespGridSpanS12  {
	padding: 0;
	margin: 0;
	width: 100%;
}

.sapUiSizeCompact .sapUxAPSectionShowHideButton {
	margin-top: 0.5rem;
}

.sapUiSizeCompact .sapUxAPObjectPageSectionTitle {
	height: 2.5rem;
	line-height: 2rem;
	padding-bottom:0.5rem;
}

.sapUiSizeCompact .sapUxAPObjectPageSectionHeader {
	height: 2.5rem;
	line-height: 2rem;
}

html[data-sap-ui-browser^="ed"].sap-desktop .sapUxAPObjectPageSection:focus {
	outline-style: dashed;
}

/* IE ignores outline-offset. Use an overlay: */
/* TODO remove after the end of support for Internet Explorer */
html[data-sap-ui-browser^="ie"].sap-desktop {
	.sapUxAPObjectPageSection:focus {
		outline: none;
		position: relative;
	}
	.sapUxAPObjectPageSection:focus:before {
		content: " ";
		box-sizing: border-box;
		width: 100%;
		height: 100%;
		position: absolute;
		left: 0;
		top: 0;
		border: 0.0625rem dashed @sapUiContentFocusColor;
		pointer-events: none;
	}
}

// Styles needed for wrapTitle property
.sapUxAPObjectPageSection.sapUxAPObjectPageSectionWrapTitle
.sapUxAPObjectPageSectionHeader
.sapUxAPObjectPageSectionTitle  {
	height: auto;
	word-break: break-word;
	white-space: break-spaces;
	overflow: auto;
	line-height: inherit;
	border-bottom: none;
	position: relative;
	top: 1rem;
}

.sapUxAPObjectPageSection.sapUxAPObjectPageSectionWrapTitle
.sapUxAPObjectPageSectionHeader {
	height: auto;
	padding-bottom: 0;
}
/* END "ObjectPageSection.less" */

/* START "ObjectPageSubSection.less" */
/* ============================================== */
/* CSS for control sap.uxap/ObjectPageSubSection  */
/* Base theme                                     */
/* ============================================== */

.sapUxAPObjectPageSubSection {
	padding-top: 1rem;
	box-sizing: border-box;
}

.sapUxAPObjectPageSubSection.sapUxAPObjectPageSubSectionStashed .sapUxAPBlockContainer > .sapUiRespGrid:empty,
/* Blockbase is used */
.sapUxAPObjectPageSubSection.sapUxAPObjectPageSubSectionStashed .sapUxAPBlockContainer > .sapUiRespGrid > div > div:empty {
	height: 20vh;
}

.sapUxAPObjectPageSubSectionHeaderActions > .sapMBtn {
	margin-left: 0.5rem;
}

.sapUxAPObjectPageSubSectionHeader {
	height: auto;
	word-wrap: break-word;
	padding: 0 3rem 0 3rem;
	display: flex;
	justify-content: space-between;
}

.sapUxAPObjectPageSubSectionHeaderTitle {
	white-space: normal;
	line-height: 3rem;
	color: @sapUiGroupTitleTextColor;
	font-size: @sapMFontHeader4Size;
	font-weight: normal;
}

.sapUxAPSubSectionSeeMoreContainer {
	text-align: right;
	width: auto;
	float: right;
	height: auto;
}

.sapUxAPSubSectionSeeMoreButton {
	display: none;
}

.sapUxAPSubSectionSeeMoreButtonVisible {
	display: block;
}

/* public sapUxAPObjectPageSubSectionTransparentBackground class that sets background transparency */
.sapUxAPObjectPageSection .sapUxAPObjectPageSectionContainer .sapUxAPObjectPageSubSection.sapUxAPObjectPageSubSectionTransparentBackground .sapUxAPBlockContainer {
	background: transparent;
}

/* phone display */
.sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageSubSectionHeader,
html .sapUiContainer-Narrow .sapUxAPObjectPageSubSectionHeader {
	padding: 0 1rem;
}

.sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageSubSectionHeaderTitle,
html .sapUiContainer-Narrow .sapUxAPObjectPageSubSectionHeaderTitle {
	height: auto;
	overflow: auto;
	float: none;
	white-space: normal;
	word-wrap: break-word;
	word-break: break-all;
}

/* tablet display */

.sapUxAPObjectPageLayout-Std-Tablet .sapUxAPObjectPageSubSectionHeader,
html .sapUiContainer-Medium .sapUxAPObjectPageSubSectionHeader {
	padding: 0 2rem 0 2rem;
}

/* Those styles apply Desktop and Desktop-XL.
Instead of writing
".sapUxAPObjectPageLayout.sapUxAPObjectPageLayout-Std-Desktop,
.sapUxAPObjectPageLayout.sapUxAPObjectPageLayout-Std-Desktop-XL",
we exclude the phone and tablet breakpoints.
Doing so will result in less generated CSS, as well as less future maintanance if Desktop-XXL is implemented.
*/

.sapUxAPObjectPageLayout:not(.sapUxAPObjectPageLayout-Std-Phone):not(.sapUxAPObjectPageLayout-Std-Tablet) {
	.sapUxAPObjectPageSubSectionHeader.titleOnLeftLayout {
		box-sizing: border-box;
		width: 25%;
		margin-right: 0;
		display: inline-block;
		vertical-align: top;
		overflow: visible;
	}

	.sapUxAPObjectPageSubSectionHeader.titleOnLeftLayout + .sapUxAPBlockContainer {
		width: -webkit-calc(~"75% - 2rem");
		width: calc(~"75% - 2rem");
		box-sizing: border-box;
		display: inline-block;
		margin-left: 0;
	}

	.sapUxAPObjectPageSubSectionHeader.titleOnLeftLayout .sapUxAPObjectPageSubSectionHeaderTitle {
		height: auto;
		white-space: normal;
		word-wrap: break-word;
		line-height: normal;
	}

	.sapUxAPObjectPageSubSectionHeader.titleOnLeftLayout .sapUxAPObjectPageSubSectionHeaderActions {
		float: none;
		display: inline-block;
		margin-left: 0;
		width: 100%;
	}

	.sapUxAPObjectPageSubSectionHeader.titleOnLeftLayout .sapUxAPObjectPageSubSectionHeaderActions > .sapMBtn:first-child {
		margin-left: 0;
	}
}
/* remove the padding of form content, since form itself is already in a grid-cell that provides the required extra spacing */
/* (1) if the direct child is a form container with a title, a more specific selector is needed: */
.sapUxAPBlockContainer .sapUiFormCLContainer > div.sapUiFormContainerTitle {
	padding-top: 0;
}

/* (2) if the direct child is a form container with a title, a more specific selector is needed for the title itself: */
.sapUxAPBlockContainer .sapUiFormCLContainer > div.sapUiFormContainerTitle > .sapUiFormTitle {
	line-height: 2rem;
	margin-bottom: 0;
}

.sapUxAPBlockContainer {
	.sapUiFormResGrid>.sapUiFormTitle {
		padding: 0;
	}
	.sapUiFormResGrid.sapUiFormToolbar >.sapMTB {
		padding: 0;
	}
	.sapUxAPObjectPageSubSectionAlignContent {
		margin-left: -@sapUiMarginSmall;
		margin-right: -@sapUiMarginSmall;
	}
}

.sapUxAPBlockContainer {
	padding: 0 2rem;
}
.sapUxAPObjectPageLayout-Std-Tablet .sapUxAPBlockContainer {
	padding: 0 1rem;
}
.sapUxAPObjectPageLayout-Std-Phone .sapUxAPBlockContainer {
	padding: 0 1rem;
	.sapUxAPObjectPageSubSectionAlignContent {
		margin-left: -@sapUiMarginSmall;
		margin-right: -@sapUiMarginSmall;
	}
}


.sapUxAPObjectPageLayout-Std-Phone .sapUxAPObjectPageSubSection {
	.sapUiRespGrid.sapUiRespGridHSpace1 {
		.sapUiRespGridSpanS12 {
			width: 100%;
		}

		padding: 0;
	}
	.sapUiRespGrid.sapUiRespGridHSpace1 > div {
		margin: 0;
		width: 100%;
	}
	.sapUxAPBlockBase {
		margin-bottom: 1rem;
	}
}

.sapUxAPObjectPageSubSectionFitContainer {
	display: flex;
	flex-direction: column;
}

.sapUxAPObjectPageSubSectionFitContainer .sapUxAPBlockContainer .sapUxAPBlockBase {
	overflow-y: hidden;
}

.sapUxAPObjectPageSubSectionFitContainer {
	.sapUxAPBlockContainer,
	.sapUxAPBlockContainer > .sapUiRespGrid,
	.sapUxAPBlockContainer > .sapUiRespGrid > *,
	.sapUxAPBlockContainer .sapUxAPBlockBase {
		box-sizing: border-box;
		height: 100%;
		margin-bottom: 0;
		margin-top: 0;
	}
}

/* compact size */
.sapUiSizeCompact {
	.sapUxAPObjectPageSubSectionHeaderTitle {
		height: 2rem;
		line-height: 2rem;
	}
	.sapUxAPObjectPageLayout-Std-Phone .sapUxAPBlockContainer .sapUiFormResGrid > div,
	.sapUxAPObjectPageLayout-Std-Phone .sapUxAPBlockContainer .sapUiFormCLContainer {
		padding: 1rem 0;
	}
}

html[data-sap-ui-browser^="ed"].sap-desktop .sapUxAPObjectPageSubSectionFocusable:focus {
	outline-style: dashed;
}

/* IE ignores outline-offset. Use an overlay: */
/* TODO remove after the end of support for Internet Explorer */
html[data-sap-ui-browser^="ie"].sap-desktop {
	.sapUxAPObjectPageSubSectionFocusable:focus {
		outline: none;
		position: relative;
	}
	.sapUxAPObjectPageSubSectionFocusable:focus:before {
		content: " ";
		box-sizing: border-box;
		width: 100%;
		height: 100%;
		position: absolute;
		left: 0;
		top: 0;
		right: 0;
		bottom: 0;
		border: 0.0625rem dashed @sapUiContentFocusColor;
		pointer-events: none;
	}
}


/* END "ObjectPageSubSection.less" */

