/* NOTE: This file was generated as an optimized version of "library.source.less" for the Theme Designer. */

/*!
 * OpenUI5
 * (c) Copyright 2009-2025 SAP SE or an SAP affiliate company.
 * Licensed under the Apache License, Version 2.0 - see LICENSE.txt.
 */

@import "../../../../../Base/baseLib/baseTheme/base.less"; /* ORIGINAL IMPORT PATH: "../../../../sap/ui/core/themes/base/base.less" */

@import "../../../../sap/ui/core/themes/base/global.less";
/* START "shared.less" */
/* ====================== */
/* Shared CSS Base theme  */
/* ====================== */

@_sap_m_Badge_AttentionBackgroundColor: @sapUiContentBadgeBackground;

/* Common CSS for condensed table cell content controls */
.sapUiCondensedTableCellContent {
	box-sizing: border-box;
	padding: 0 0.5rem;
	line-height: 1.5rem;
	vertical-align: middle;
}

/* commons */
.sapMScrollbarV,
.sapMScrollbarH,
.sapMScrollbarV > div,
.sapMScrollbarH > div {
	position: absolute;
}

.sapMScrollbarV > div,
.sapMScrollbarH > div {
	box-sizing: border-box;
	-webkit-background-clip: padding-box;
	background-clip: padding-box;
	border-width: 0.0625rem;
	border-style: solid;
	border-color: rgba(255, 255, 255, 0.901961);
	border-radius: 0.1875rem;
	background-color: rgba(0, 0, 0, 0.498039);
}

/* vertical scrollbar */
.sapMScrollbarV {
	width: 0.438rem;
	right: 0.063rem;
	top: 0;
	bottom: 0;
}

.sapMScrollbarV > div {
	width: 100%;
}

/* horizontal scrollbar */
.sapMScrollbarH {
	height: 0.4375rem;
	bottom: 0.0625rem;
	left: 0.125rem;
	right: 0.125rem;
}

.sapMScrollbarH > div {
	height: 100%;
}

/* ObjectListItem and ObjectHeader share styles for markers such as flag and favorite */
.sapMObjStatusMarker {
	color: var(--sapContent_MarkerIconColor);
}

.sapMObjStatusMarker .sapUiIcon + .sapUiIcon,
.sapMObjStatusMarker .sapUiHiddenPlaceholder + .sapUiIcon {
	margin-left: .5rem;
}

.sapMObjStatusMarkerInvisible {
	visibility: hidden;
}

/* CURSORS */
.sapMPointer{
	cursor: pointer;
}

/* Enables native browser hyphenation where possible */
.sapUiHyphenation {
	-webkit-hyphens: auto;
	-moz-hyphens: auto;
	hyphens: auto;
}

.defaultHighlightedText {
	background: var(--sapContent_SearchHighlightColor);
	font-weight: bold;
}

/* Enables sap.m.IBadge implementations where possible */

@keyframes sapMBadgeAppearPulse {

	0% {transform: scale(.9); opacity:1;}

	100% {transform: scale(2);opacity: 0;}
}

@keyframes sapMBadgeAppearBounce {

	0% {transform: scale(1);}

	50% {transform: scale(1.4);}

	100% {transform: scale(1);}
}

@keyframes sapMBadgeDisappear {

	0% {transform: scale(1);}

	100% {transform: scale(0);}
}

@keyframes sapMBadgeUpdate {

	0% {transform: scale(1);}

	50% {transform: scale(1.2);}

	100% {transform: scale(1);}
}

@keyframes sapMBadgeAppearMotionSmall {

	0% {
		transform: scale(0.33); /* 0.125rem / 0.375rem */
		box-shadow: 0 0 0 0 rgba(0, 0, 0, 0);
	}

	20% {
		transform: scale(1.66); /* 0.625rem / 0.375rem */
	}

	40% {
		transform: scale(1.33); /* 0.5rem / 0.375rem */
		box-shadow: 0 0 0 1px fade(@_sap_m_Badge_AttentionBackgroundColor, 20%); /* 0.875rem pulse layer */
	}

	60% {
		transform: scale(1); /* 0.375rem */
		box-shadow: 0 0 0 0.3125rem fade(@_sap_m_Badge_AttentionBackgroundColor, 15%); /* 1rem pulse layer */
	}

	100% {
		box-shadow: 0 0 0 0.375rem fade(@_sap_m_Badge_AttentionBackgroundColor, 0%); /* 1.125rem pulse layer */
	}
}

@keyframes sapMBadgeAppearMotion {

	0% {
		transform: scale(0.25); /* 0.125rem / 0.5rem */
		box-shadow: 0 0 0 0 rgba(0, 0, 0, 0); /* required for Internet Explorer */
	}

	20% {
		transform: scale(1.25); /* 0.625rem / 0.5rem */
	}

	40% {
		transform: scale(1); /* 0.5rem */
		box-shadow: 0 0 0 1px fade(@_sap_m_Badge_AttentionBackgroundColor, 20%); /* 0.875rem pulse layer */
	}

	100% {
		box-shadow: 0 0 0 0.375rem fade(@_sap_m_Badge_AttentionBackgroundColor, 0%); /* 1.125rem pulse layer */
	}
}

.sapMBadge {
	overflow: visible;
	position: relative;

	.sapMBadgeDefault {
		position: absolute;
		top: -0.125rem;
		right: -0.25rem;
		z-index: 100;
		max-width: 100%;
		padding-right: 0.125rem;
		box-sizing: border-box;
		height: 100%;
		transform-origin: center center;
		pointer-events: none;

		&::after {
			border-radius: 1.125rem;
			display: block;
			padding: 0 0.3125rem;
			min-width: 0.5rem;
			height: 1.125rem;
			line-height: 1.125rem;
			font-size: var(--sapFontSmallSize);
			font-family: var(--sapFontFamily);
			text-align: center;
			background-color: var(--sapContent_BadgeBackground);
			border: 0.0625rem solid var(--sapContent_BadgeBackground);
			color: var(--sapContent_BadgeTextColor);
			text-overflow: ellipsis;
			white-space: nowrap;
			overflow: hidden;

			.badgeAccentColorMixin(10);
		}

		/* We need one more pseudo-element, attached to the badge indicator, in order */
		/* to show the pulse animation simultaneously with the bounce one. */
		&::before {
			position: absolute;
			border-radius: 1.125rem;
			display: block;
			padding: 0 0.3125rem;
			min-width: 0.5rem;
			height: 1.125rem;
			line-height: 1.125rem;
			font-size: var(--sapFontSmallSize);
			font-family: var(--sapFontFamily);
			background-color: var(--sapContent_BadgeBackground);
			border: 0.0625rem solid var(--sapContent_BadgeBackground);
			color: var(--sapContent_BadgeTextColor);
			text-overflow: ellipsis;
			overflow: hidden;
			opacity:0;

			.badgeAccentColorMixin(10);
		}

		.sapMBadgeTopLeft& {
			top: -0.125rem;
			left: -0.25rem;
			right: auto;
		}

		.sapMBadgeTopRight& {
			top: -0.125rem;
			right: -0.25rem;
		}

		.sapMBadgeInline& {
			left: 0;
			right: 0;
			top: 0;
		}
	}

	/* The attention badge is a single dot, which does not contain text or icons. */
	.sapMBadgeAttention {
		position: absolute;
		top: -0.25rem;
		right: -0.25rem;
		width: 0.5rem;
		height: 0.5rem;
		border-radius: 1.125rem;
		background-color: @_sap_m_Badge_AttentionBackgroundColor;
		border: 0.125rem solid var(--sapObjectHeader_Background);
	}
}

.sapMBadgeIndicator.sapMBadgeDefault::after {
	content: attr(data-badge);
}

.sapMBadgeIndicator.sapMBadgeDefault[data-badge='']::after {
	display: none;
}

html[data-sap-ui-animation='on'] {

	.sapMBadge.sapMBadgeAnimationTypeFull {

		.sapMBadgeDefault.sapMBadgeAnimationAdd::after {
			animation: sapMBadgeAppearBounce .1s;
		}

		.sapMBadgeDefault.sapMBadgeAnimationAdd::before {
			content: attr(data-badge);
			animation: sapMBadgeAppearPulse .2s;
		}

		.sapMBadgeAttention.sapMBadgeAnimationRemove,
		.sapMBadgeDefault.sapMBadgeAnimationRemove::after {
			animation: sapMBadgeDisappear .1s;
			animation-fill-mode: forwards;
			transform-origin: center;
		}
	}

	.sapMBadge:not(.sapMBadgeAnimationTypeNone) {

		.sapMBadgeDefault.sapMBadgeAnimationUpdate::after {
			animation: sapMBadgeUpdate .1s;
			animation-fill-mode: forwards;
		}
	}
}

html[data-sap-ui-animation='off'] {

	.sapMBadgeAttention.sapMBadgeAnimationRemove,
	.sapMBadgeDefault.sapMBadgeAnimationRemove::after {
		scale: 0;
		opacity: 0;
		display: none;
	}
}

.sapMBadgeAnimationTypeUpdate,
.sapMBadgeAnimationTypeNone {

	.sapMBadgeAttention.sapMBadgeAnimationRemove,
	.sapMBadgeDefault.sapMBadgeAnimationRemove::after {
		scale: 0;
		opacity: 0;
		display: none;
	}
}

.badgeAccentColorMixin(@n, @i: 1) when (@i =< @n) {
	@value: 'sapAccentColor@{i}';

	.sapMBadgeAccentColor@{i}& {
		background-color: @@value;
		border-color: @@value;
	}
	.badgeAccentColorMixin(@n, (@i + 1));
}
/* END "shared.less" */


/* START "AbstractContainer.less" */
//Change background color for the container control
.sapUiMAbstractContainerContent .sapMITB {
    padding-top: 1.75rem;
}

//Remove padding to align the ITB with the rest of the p13n related controls (left aligned)
.sapUiMAbstractContainerContent .sapMITH {
    padding: 0
}

.sapUiMAbstractContainerContent .wrapperDefaultList .sapMLIB{
    padding: 1.5rem;
}

.sapMAbstractContainer .sapMPage {
	display: flex;
	flex-direction: column;

	&>section {
		position: relative;
		top: 0 !important; // we need to overwrite the top argument for a section when using the AbstractContainer, otherwise top is overriden by other styles
	}

	>footer:not(.sapUiHidden) {
		position: relative;
	}
}
/* END "AbstractContainer.less" */

/* START "ActionListItem.less" */
/* ===================================== */
/* CSS for control sap.m/ActionListItem  */
/* Base theme                            */
/* ===================================== */

.sapMALI {
	display: -webkit-box;
	display: flex;
	height: 2.75rem;
}

.sapMALIText {
	-webkit-box-flex: 1;
	flex: auto;
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
	min-width: 3rem;
	display: block;
	text-align: center;
	color: @sapUiButtonTextColor;
}

.sapMALI > .sapMLIBContent {
	display: -webkit-box;
	display: flex;
	-webkit-box-pack: center;
	justify-content: center;
	white-space: nowrap;
	text-overflow: ellipsis;
}

.sapMALI .sapMLIBUnread,
.sapMALI .sapMLIBSelectS,
.sapMALI .sapMLIBSelectM,
.sapMALI .sapMLIBSelectD {
	display: none;
}

/*The active indicator of the list is not specific enough to overwrite this style*/
.sapMList .sapMLIB.sapMALI:not(.sapMLIBActive) {
	background: @sapUiButtonActionSelectBackground;
	border-color: @sapUiButtonActionSelectBorderColor;
}

html.sap-desktop .sapMList .sapMLIB.sapMALI:not(.sapMLIBActive) {
	background: @sapUiButtonActionSelectBackground;
	border-color: @sapUiButtonActionSelectBorderColor;
}

html.sap-desktop .sapMList .sapMLIB.sapMALI:hover:not(.sapMLIBActive) > .sapMLIBContent {
	background: @sapUiButtonLiteActionSelectHoverBackground;
}

/* Compact size */
.sapUiSizeCompact .sapMALI {
	height: 2rem;
}

.sapUiSizeCompact .sapMALI > .sapMLIBContent {
	line-height: 2rem;
}

.sapUiSizeCompact .sapMALIText {
	font-size: @sapMFontMediumSize;
}
/* END "ActionListItem.less" */

/* START "ActionSelect.less" */
/* =================================== */
/* CSS for control sap.m/ActionSelect  */
/* Base theme                          */
/* =================================== */

/* ----------------------------- */
/* button                        */
/* ----------------------------- */

.sapMActionSelectPicker .sapMBtn {
	display: block;
	width: 100%;
	height: 2.5rem;
	padding: 0;
	border-top-width: 0;
	border-bottom-width: 1px;
	text-shadow: var(--sapContent_TextShadow);
}

.sapMActionSelectPicker .sapMBtnInner {
	border: 1px solid var(--sapList_BorderColor);
	overflow: hidden;
}

.sapMActionSelectPicker .sapMSelectListItemBaseSelected.sapMActionSelectItemWithoutFocus {
	outline: none;
}
/* END "ActionSelect.less" */

/* START "ActionSheet.less" */
/* ================================== */
/* CSS for control sap.m/ActionSheet  */
/* Base theme                         */
/* ================================== */

/* Buffer width for the scroll bar */
@_sap_m_Popover_Scroll_Width_Buffer: 20px;

.sapMActionSheetDialog.sapMDialog {
	top: auto !important; /* We need to overwrite the inline top style */
	bottom: 0;
	width: 100%;
	max-width: 20rem;
	border-radius: 0.25rem 0.25rem 0 0;

	&.sapMDialogPhone {
		overflow: visible;
	}
}

.sapMActionSheet:focus {
	outline: none;
}

.sapMActionSheetDialog .sapMDialogScroll {
	display: block;
}

.sapMActionSheetButton > .sapMBtnInner {
	text-align: start;
}

.sapMActionSheetButton.sapMBtn:not(.sapMActionSheetCancelButton) > .sapMBtnInner {
	justify-content: flex-start;
}

.sapMActionSheetDialogWithTitle .sapMBarMiddle .sapMBarPH {
	/* The left/right padding should be 1rem, but there is already 0.25rem right/left padding from the Dialog title */
	padding: 0 0.75rem 0.5rem;
	justify-content: flex-start;
}

/* ====== BEGIN ====== */
/*Style for ActionSheet shown as a dialog sliding in from bottom*/
/* ====== ====== ===== */

.sapMDialog.sapMActionSheetDialog {
	background-color: transparent;
	box-shadow: none;
	border: none;
}

.sapMDialog.sapMActionSheetDialog > section {
	background-color: var(--sapGroup_ContentBackground);
	border-radius: 0.25rem 0.25rem 0 0;
}

.sapMActionSheetDialog .sapMActionSheetButton {
	width: 100%;
}

.sapMActionSheetDialog .sapMActionSheetButton.sapMActionSheetCancelButton > .sapMBtnInner {
	text-align: center;
}

.sapMActionSheetDialog .sapMBar.sapMHeader-CTX > .sapMBarMiddle > .sapMBarPH > .sapMDialogTitle {
	color: contrast(fade(@sapUiBlockLayerBackground, 60), @sapUiGroupTitleTextColor, @sapUiContentContrastTextColor, @sapUiContentContrastTextThreshold);
	font-size: @sapMFontMediumSize;
}

.sapMDialog.sapMActionSheetDialog > .sapMDialogSection {
	box-shadow: @sapUiShadowLevel1;
}

.sapMDialog.sapMActionSheetDialog > section {
	padding: 0.25rem 0.5rem;
}

.sapMDialog.sapMActionSheetDialog.sapMDialogPhone .sapMIBar.sapMHeader-CTX {
	background: transparent;
	box-shadow: none;
}

/* ====== BEGIN ====== */
/*Style for ActionSheet shown as a Popover*/
/* ====== ====== ===== */
.sapMActionSheetPopover.sapMPopover {
	box-shadow: @sapUiShadowLevel1;
}

.sapMActionSheetPopover .sapMActionSheetButton {
	display: block;
	width: 100%;
}

.sapUiSizeCompact {

	.sapMActionSheetPopover.sapMPopover {

		.sapMPopoverCont {
			padding: 0.1875rem 0.375rem;
			max-height: calc(~'100% - 0.375rem');
		}

		.sapMActionSheet .sapMActionSheetButton .sapMBtnInner.sapMBtnIconFirst {
			padding-left: 0.4375rem;
			padding-right: 0.4375rem;
		}
	}
}

.sapMActionSheetPopover.sapMPopover {

	.sapMPopoverCont {
		padding: 0.25rem 0.5rem;
		max-height: calc(~'100% - 0.5rem');
	}

	&.sapMPopoverVerticalScrollIncluded .sapMPopoverCont {
		padding: 0.25rem calc(~'0.5rem + ' @_sap_m_Popover_Scroll_Width_Buffer) 0.25rem 0.5rem;
	}
}

/* ====== END ====== */
/*Style for ActionSheet shown as a Popover*/
/* ====== ====== ===== */
/* END "ActionSheet.less" */

/* START "ActionTile.less" */
/* ================================== */
/* CSS for control sap.m/ActionTile  */
/* Base theme                         */
/* ================================== */

.sapMAT {
	width: 20rem;
	height: 23.125rem;
	border-radius: var(--sapTile_BorderCornerRadius);
	position: relative;
	padding: 1rem;
	box-shadow: var(--sapContent_Shadow0);
	background-color: var(--sapTile_Background);
	display: inline-block;
	box-sizing: border-box;
	user-select: none;
	vertical-align: top;
	text-decoration: none;

	.sapMGTHdrContent {
		padding: 0;
		height: 3.25rem;
		overflow: hidden;
		margin-bottom: 0.75rem;

		.sapMText {
			font-size: 1rem;
			font-weight: 700;
			color: var(--sapTile_TitleTextColor) !important;
			line-height: 1.5rem;
			cursor: pointer;
		}

		.sapMATHeaderContainer {
			display: flex;
			flex-direction: column;
			height: 100%;
			justify-content: center;
			gap: 0.25rem;

			.sapMGTTitle {
				line-height: 1.2;
				font-size: var(--sapFontSize);
			}

			.sapMTilePriorityValue {
				font-size: 0.875rem;

				&.High {
					color: var(--sapNegativeTextColor);
				}

				&.Medium {
					color: var(--sapCriticalTextColor);
				}

				&.Low {
					color: var(--sapPositiveTextColor);
				}

				&.VeryHigh {
					color: var(--sapIndicationColor_1_BorderColor);
				}
			}
		}
	}

	.sapMGTActionModeContainer {
		position: absolute;
		bottom: 0.25em;
		right: 1rem;
	}

	.sapMGTContent {
		height: 17.4rem;
		overflow: visible;

		.sapMTileCnt {
			display: flex;
			flex-direction: column;
			height: inherit;

			.sapMTileCntContent {
				margin-top: 0;
				padding: 0;

				.sapMFT {
					height: inherit;

					span {
						color: var(--sapTextColor);
						white-space: initial;
						word-break: break-all;
						display: -webkit-box;
						-webkit-line-clamp: 16;
						-webkit-box-orient: vertical;
						height: fit-content;
						line-height: 1rem;
						overflow: hidden;
						text-overflow: ellipsis;
						font-weight: 400;
						font-size: var(--sapFontSize);
						margin-bottom: 0.75rem;
						max-height: 16rem;
					}

					p + span {
						margin-bottom: 1rem;
						-webkit-line-clamp: 14;
						max-height: 14rem;
					}

					p {
						white-space: nowrap;
						overflow: hidden;
						text-overflow: ellipsis;
						color: var(--sapTextColor);
						font-weight: 400;
						font-size: var(--sapFontSize);
						margin: 0 0 1rem 0;
						line-height: 1.125rem;
					}

					p.sapMbrPresent::first-line {
						color: var(--sapContent_LabelColor);
					}

					p:not(.sapMbrPresent) {
						margin-bottom: 0.75rem;
						white-space: initial;
						word-break: break-all;
					}
				}
			}

			.sapMTileFtrCnt {
				position: absolute;
				bottom: 1rem;
				right: 0rem;
			}
		}
	}

	&.sapMATHideActionButton {

		.sapMTileCntContent {

			.sapMFT {

				span {
					-webkit-line-clamp: 14 !important;
				}

				p + span {
					-webkit-line-clamp: 12 !important;
				}
			}
		}

		.sapMGTActionModeContainer {
			display: none !important;
		}
	}
}

.sapMAT.sapMATHideActionButton {
	height: 20.75rem;
 }

.sapMAT:hover {
	background-color: var(--sapTile_Hover_Background);
	box-shadow:  var(--sapContent_Shadow2);
}

.sapMAT:active {
	background-color: var(--sapTile_Active_Background);
	box-shadow: none;
	outline: 0.0625rem solid var(--sapTile_Interactive_BorderColor);
}

.sapMAT:focus-visible {
	outline: none;
}

.sapMAT.sapMATAutoPointer {

	.sapMText {
		cursor: auto !important;
	}
}

 .sapMAT:focus {

	.sapMATFocusDiv {
		position: absolute;
		border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
		border-radius:  1rem;
		inset: 0px;
		pointer-events: none;
	}
}

/* Styles when Icon(headerImage) is present */

.sapMAT.sapMATHeaderImage {
	overflow: hidden;

	.sapMGTHdrContent {
		display: flex;
		flex-direction: row;
		align-items: center;
		gap: 0.5rem;
		height: 3.25rem;
		margin-bottom: 0.75rem;

		.sapMGTHdrIconImage.Neutral {
			color:  var(--sapNeutralTextColor);
		}

		.sapMGTHdrIconImage.Good {
			color: var(--sapPositiveTextColor);
		}

		.sapMGTHdrIconImage.Critical {
			color: var(--sapCriticalElementColor);
		}

		.sapMGTHdrIconImage.Error {
			color: var(--sapNegativeTextColor);
		}

		.sapMGTHdrIconImage.None {
			color: var(--sapTile_IconColor);
		}

		.sapMGTHdrIconImage.sapUiIcon {

			&::before {
				height: 2.75rem;
				font-weight: 400;
				font-size: 2.25rem;
				line-height: 2.4375rem;
				display: flex;
				align-items: end;
				text-align: center;
			}
		}

		.sapMATIconFrame {
			flex-shrink: 0;

			&.sapMGTIconFrameBadge {
				align-self: start;

				.sapFAvatarBadgeIconActiveArea {
					height: 3rem;
					width: 3rem;

					.sapFAvatarBadgeIcon {
						height: 1.125rem;
						width: 1.125rem;

						.sapUiIcon {
							font-size: 0.75rem;
						}
					}
				}
			}
		}
	}
}

.sapMAT:focus {

	.sapMGTFocusDiv {
		position: absolute !important;
		pointer-events: none;
		inset: 0;
		z-index: 2;
		border-radius: var(--sapTile_BorderCornerRadius);
		border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
	}

	.sapMATFocusDiv {
		position: absolute !important;
		pointer-events: none;
		inset: 0;
		z-index: 2;
		border-radius: var(--sapTile_BorderCornerRadius);
		border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
	}
}

.sapMAT.sapMATDynamicHeight {
	height: auto;
	display: inline-flex;
	flex-direction: column;
	padding-bottom: 0.4375rem;

	.sapMGTContent {
		height: auto;

		.sapMATC {
			height: auto;
			margin-bottom: 1.0625rem;
		}

		.sapMTileCnt {
			height: auto;

			.sapMTileCntContent {
				height: auto;
			}

			.sapMTileCntContent {

				.sapMFT {

					span {
						-webkit-line-clamp: initial;
						max-height: none;
					}

					p + span {
						-webkit-line-clamp: initial;
						max-height: none;
					}
				}
			}
		}
	}

	.sapMGTActionModeContainer {
		position: relative;
		height: fit-content;
		width: inherit;
		box-sizing: border-box;
		display: flex;
		justify-content: flex-end;
		padding-right: 2rem;
		inset: 0;
		margin-top: auto;
		top: 0.1875rem;
	}

	&.sapMATHideActionButton {
		padding-bottom: 1rem;

		.sapMTileCntContent {

			.sapMFT {

				span {
					-webkit-line-clamp: initial !important;
					max-height: none;
				}

				p + span {
					-webkit-line-clamp: initial !important;
					max-height: none;
				}
			}
		}

		.sapMGTContent {
			height: auto;

			.sapMATC {
				margin-bottom: 0;
			}
		}
	}
}

/* Loading State Styles */

.sapMAT.sapMATStateLoading {

	.sapMGTContentShimmerPlaceholderItem {
		padding: 0;
		height: 20rem;
		align-items: initial;
		flex-direction: column;
		justify-content: space-around;

		.sapMGTContentShimmerPlaceholderRows {
			flex-grow: 0;
		}
	}
}

.sapMAT.sapMATStateLoading.sapMATHideActionButton {

	.sapMGTContentShimmerPlaceholderItem {
		height: 17.5rem;
		justify-content: space-evenly;
		padding: 0px;

		.sapMGTContentShimmerPlaceholderRows {

			.sapMGTContentShimmerPlaceholderItemHeader,
			.sapMGTContentShimmerPlaceholderItemText {
				margin-top: 0;
			}

			.sapMGTContentShimmerPlaceholderItemText {
				margin-bottom: 0;
			}
		}
	}
}
/* END "ActionTile.less" */

/* START "ActionTileContent.less" */
/* ================================== */
/* CSS for control sap.m/ActionTileContent  */
/* Base theme                         */
/* ================================== */
.sapMATC {
    height: 14.6875rem;
     overflow: hidden;
     .sapMTilePriorityValue {
         font-size: 0.875rem;
         font-weight: 400;
         margin-bottom: 1rem;
         height: 1.625rem;
        display: flex;
        align-items: center;
         &.High {
             color: @sapNegativeTextColor;
             }
         &.Medium {
                 color: @sapCriticalTextColor;
             }
         &.Low {
             color: @sapPositiveTextColor;
             }
         &.VeryHigh {
                 color: @sapIndicationColor_1_BorderColor;
             }
     }
     .sapMContainer {
             display: flex;
             flex-direction: column;
             gap: 1rem;
         .sapMElementWrapper {
             display: flex;
             flex-direction: column;
             gap: 0.25rem;
             font-weight: 400;
             font-size: 0.875rem;
             white-space: nowrap;
             overflow: hidden;
             text-overflow: ellipsis;
             .sapMATCLabel {
                 color: @sapContent_LabelColor;
                 white-space: nowrap;
                 overflow: hidden;
                 text-overflow: ellipsis;
                 font-size: 14px;
                    font-weight: 400;
                    line-height: 1rem;
             }
             .sapMATCValue {
                 color: @sapTextColor;
                 white-space: nowrap;
                 overflow: hidden;
                 text-overflow: ellipsis;
                 font-size: 14px;
                font-weight: 400;
                line-height: 1rem;
                height: 1rem;
                .sapMText {
                    cursor: pointer;
                }
             }
         }
     }
    }
/* END "ActionTileContent.less" */

/* START "App.less" */
/* ========================== */
/* CSS for control sap.m/App  */
/* Base theme                 */
/* ========================== */

.sapMApp {
	width: 100%;
	height: 100%;
}
/* END "App.less" */

/* START "Avatar.less" */
/* ============================= */
/* CSS for control sap.m/Avatar  */
/* Base theme                    */
/* ============================= */

@_sap_m_Avatar_Font_Size_XL: 3rem;
@_sap_m_Avatar_Font_Size_L: 2.5rem;
@_sap_m_Avatar_Font_Size_M: 2rem;
@_sap_m_Avatar_Font_Size_S: 1.5rem;
@_sap_m_Avatar_Font_Size_XS: 1rem;
@_sap_m_Avatar_Hover_Box_Shadow_Offset: 0 0 0 0.0625rem;

.sapFAvatar {
	display: inline-block;
	position: relative;
	box-sizing: border-box;

	&.sapFAvatarBorder {
		border: 0.0625rem solid var(--sapGroup_ContentBorderColor);
	}

	&.sapMAvatarDisabled {
		opacity: var(--sapContent_DisabledOpacity);
	}
}

// Define a loop and go trough 10 color sets
.loop-colors(@i: 1) when (@i =< 10) {

	.generateColor(@i);

	.loop-colors(@i + 1); // Execute next loop iteration
}

// Execute Loop through color sets
.loop-colors(1);

// Create classes which would just set background color of Avatar
.generateColor (@accentIndex) {

	@css-selector: ~"sapFAvatarColorAccent@{accentIndex}"; // Create the css selector string
	@color-param: ~"sapUiAccent@{accentIndex}"; // Build the color param which is a sapUiAccent
	@border-param: ~"sapAvatar_@{accentIndex}_TextColor"; // Build the border param

	// Build the real CSS selector.
	// The output would be something like this: .sapFAvatar.sapFAvatarColorAccent6 {background-color: #0092d1;}
	.@{css-selector} {

		background-color: @@color-param;

		&.sapFAvatarFocusable:not(.sapMAvatarPressed):not(.sapMAvatarDisabled):hover {
			box-shadow: @_sap_m_Avatar_Hover_Box_Shadow_Offset @@border-param;
		}
	}
}

.sapFAvatarIcon {
	text-align: center;
	color: var(--sapContent_ContrastTextColor);

	.sapUiIcon {
		position: absolute;
		top: 50%;
		left: 50%;
		transform: translate(-50%, -50%);
	}
}

.sapFAvatarHiddenIcon {
	display: none;
}

.sapFAvatarColorTransparent {
	background-color: transparent;

	&.sapFAvatarIcon,
	& > .sapFAvatarInitialsHolder {
		color: var(--sapContent_IconColor);
	}

	&.sapFAvatarFocusable:not(.sapMAvatarPressed):not(.sapMAvatarDisabled):hover {
		box-shadow: @_sap_m_Avatar_Hover_Box_Shadow_Offset var(--sapContent_IconColor);
	}
}

.sapFAvatarColorTileIcon {
	background-color: var(--sapTile_IconColor);

	&.sapFAvatarFocusable:not(.sapMAvatarPressed):not(.sapMAvatarDisabled):hover {
		box-shadow: @_sap_m_Avatar_Hover_Box_Shadow_Offset var(--sapTile_IconColor);
	}
}

.sapFAvatarColorPlaceholder {
	background-color: var(--sapContent_ImagePlaceholderBackground);

	&.sapFAvatarFocusable:not(.sapMAvatarPressed):not(.sapMAvatarDisabled):hover {
		box-shadow: @_sap_m_Avatar_Hover_Box_Shadow_Offset var(--sapContent_ImagePlaceholderForegroundColor);
	}
}

.sapFAvatarInitialsHolder {
	color: var(--sapContent_ContrastTextColor);
	position: absolute;
	top: 50%;
	left: 50%;
	transform: translate(-50%, -50%);
}

.sapFAvatarImage {
	text-align: center;

	.sapFAvatarTypeIcon, .sapFAvatarInitialsHolder {
		display: none;
	}

	&.sapFAvatarFocusable:not(.sapMAvatarPressed):not(.sapMAvatarDisabled):hover {
		box-shadow: @_sap_m_Avatar_Hover_Box_Shadow_Offset var(--sapAvatar_10_TextColor);
	}
}

.sapFAvatarImageHolder {
	background: no-repeat center;
	display: block;
	position: absolute;
	left: 0;
	top: 0;
	right: 0;
	bottom: 0;

	.sapFAvatar:not(.sapFAvatarImage) & {
		display: none;
	}
}

.sapFAvatarImageCover {
	background-size: cover;
}

.sapFAvatarImageContain {
	background-size: contain;
}

.sapFAvatarCircle {
	border-radius: 50%;

	.sapMImg,
	.sapFAvatarImageHolder {
		border-radius: 50%;
	}
}

.sapFAvatarXS {
	height: 2rem;
	width: 2rem;
	font-size: @_sap_m_Avatar_Font_Size_XS;

	.sapFAvatarInitialsHolder {
		font-size: 0.75rem;
	}
}

.sapFAvatarS {
	height: 3rem;
	width: 3rem;
	font-size: @_sap_m_Avatar_Font_Size_S;

	.sapFAvatarInitialsHolder {
		font-size: 1.125rem;
	}
}

.sapFAvatarM {
	height: 4rem;
	width: 4rem;
	font-size: @_sap_m_Avatar_Font_Size_M;

	.sapFAvatarInitialsHolder {
		font-size: 1.625rem;
	}
}

.sapFAvatarL {
	height: 5rem;
	width: 5rem;
	font-size: @_sap_m_Avatar_Font_Size_L;

	.sapFAvatarInitialsHolder {
		font-size: 2rem;
	}
}

.sapFAvatarXL {
	height: 7rem;
	width: 7rem;
	font-size: @_sap_m_Avatar_Font_Size_XL;

	.sapFAvatarInitialsHolder {
		font-size: 2.75rem;
	}
}

.sapFAvatarFocusable.sapMAvatarPressed {
	background-color: var(--sapButton_Selected_Background);
	color: var(--sapButton_Selected_TextColor);
	outline-offset: 0.0625rem;

	&:hover {
		background-color: var(--sapButton_Selected_Hover_Background);
	}

	.sapUiRespGrid & {
		z-index: 100;
	}
}

.sapFAvatarFocusable:focus {
	outline: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
	outline-offset: 0.0625rem;

	.sapUiRespGrid & {
		z-index: 100;
	}
}

html:not(.sap-desktop) .sapFAvatarFocusable.sapMAvatarPressed {
	outline: none;
}

.sapFAvatarInitialsHolder {
	color: var(--sapContent_ImagePlaceholderForegroundColor);
}

// Apply LightBox Magnifying glass mixin to control
.lightBoxMagnifyingGlassMixing(~'.sapFAvatar', ~'.sapFAvatarMagnifyingGlass');


//restrinct inner elements of blocking press event of the whole control
.sapFAvatar * {
	pointer-events: none;
}

.sapFAvatarBadgeIconActiveArea {
	position: absolute;
	pointer-events: all;

	.sapFAvatarCircle & {
		width: 100%;
		height: 100%;
		bottom: 0;
		right: 0;
	}

	.sapFAvatarSquare & {
		width: ~'calc(100% + 0.125rem)';
		height: ~'calc(100% + 0.125rem)';
		bottom: -0.125rem;
		right:  -0.125rem;
	}

	.sapFAvatarSquare.sapFAvatarXL & {
		width: ~'calc(100% + 0.25rem)';
		height: ~'calc(100% + 0.25rem)';
		bottom: -0.25rem;
		right: -0.25rem;
	}

	.sapFAvatarSquare.sapFAvatarL & {
		width: ~'calc(100% + 0.1875rem)';
		height: ~'calc(100% + 0.1875rem)';
		bottom: -0.1875rem;
		right: -0.1875rem;
	}

	.sapFAvatarSquare.sapFAvatarCustom & {
		width: 104%;
		height: 104%;
		bottom: -4%;
		right: -4%;
	}
}

.generateBadgeColor(@i) when (@i =< 10) {
	@color: 'sapAccentColor@{i}';
	@background: 'sapAccentBackgroundColor@{i}';

	.sapFAvatarBadgeColorAccent@{i} .sapFAvatarBadgeIcon {
		box-shadow: @_sap_m_Avatar_Hover_Box_Shadow_Offset @@color;
		background-color: @@background;
		> .sapUiIcon {
			color: @@color;
		}
	}

	.generateBadgeColor(@i + 1);
}

.generateBadgeColor(1);

.sapFAvatarWarning .sapFAvatarBadgeIcon {
	background-color: var(--sapWarningBackground);
	box-shadow: @_sap_m_Avatar_Hover_Box_Shadow_Offset var(--sapWarningBorderColor);

	> .sapUiIcon {
		color: var(--sapCriticalTextColor);
	}
}

.sapFAvatarError .sapFAvatarBadgeIcon {
	background-color: var(--sapErrorBackground);
	box-shadow: @_sap_m_Avatar_Hover_Box_Shadow_Offset var(--sapErrorBorderColor);

	> .sapUiIcon {
		color: var(--sapNegativeTextColor);
	}
}

.sapFAvatarInformation .sapFAvatarBadgeIcon {
	background-color: var(--sapInformationBackground);
	box-shadow: @_sap_m_Avatar_Hover_Box_Shadow_Offset var(--sapInformationBorderColor);

	> .sapUiIcon {
		color: var(--sapInformativeTextColor);
	}
}

.sapFAvatarSuccess .sapFAvatarBadgeIcon {
	background-color: var(--sapSuccessBackground);
	box-shadow: @_sap_m_Avatar_Hover_Box_Shadow_Offset var(--sapSuccessBorderColor);

	> .sapUiIcon {
		color: var(--sapPositiveTextColor);
	}
}

.sapFAvatarBadgeIcon {
	display: inline-block;
	box-sizing: border-box;
	position: absolute;
	bottom: 0;
	right:  0;
	width: 1.125rem;
	height: 1.125rem;
	border-radius: 50%;
	font-size: 0.75rem;
	text-align: center;
	background-color: var(--sapButton_Emphasized_Background);
	box-shadow: @_sap_m_Avatar_Hover_Box_Shadow_Offset var(--sapButton_Emphasized_BorderColor);

	> .sapUiIcon {
		vertical-align: middle;
		color: var(--sapButton_Emphasized_TextColor);
	}

	&::after {
		content: '';
		display: inline-block;
		vertical-align: middle;
		height: 100%;
	}

	.sapFAvatarL & {
		width: 1.25rem;
		height: 1.25rem;
		font-size: 0.875rem;
	}

	.sapFAvatarXL & {
		width: 1.75rem;
		height: 1.75rem;
		font-size: 1rem;
	}

	.sapFAvatarCustom & {
		font-size: 16.5%;
		width: 25%;
		height: 25%;
	}
}

/* END "Avatar.less" */

/* START "Bar.less" */
/* ========================== */
/* CSS for control sap.m/Bar  */
/* Base theme                 */
/* ========================== */

@_sap_m_Bar_AppHeight: 3333px; /* Ux decision: never show the header, so make this value large enough even for zoom-out cases */
@_sap_m_Bar_HeaderHeight: 68px; /* should be 4.25rem */
@_sap_m_Bar_MinHeightForHeader: (@_sap_m_Bar_AppHeight + @_sap_m_Bar_HeaderHeight);

.sapMBar {
	display: block;
	width: 100%;
	position: relative;
	height: 3rem;
	line-height: 3rem;
	color: var(--sapPageHeader_TextColor);
	background-color: var(--sapPageHeader_Background);
	/* 1px down white shadow with 50% opacity */
	text-shadow: @sapUiShadowText;
	/* 1px inside border line at the bottom of a bar */
	box-shadow: var(--sapContent_HeaderShadow);
}

.sapMBarLeft.sapMBarContainer.sapMBarEmpty,
.sapMBarMiddle.sapMBarContainer.sapMBarEmpty,
.sapMBarRight.sapMBarContainer.sapMBarEmpty {
	padding: 0;
}

.sapMBarContainer {
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
}

/* fix for ff */
html[data-sap-ui-browser^="ff"].sap-desktop {

	.sapMBarContainer {
		display: flex;
		align-items: center;
	}

	.sapMBarPH {
		justify-content: center;
	}

	.sapMBarRight {
		justify-content: flex-end;
	}

	.sapMBarContainer > .sapMBarChild {
		flex : none;
	}

	.sapMBarContainer > .sapMButton,
	.sapMBarContainer > .sapMText,
	.sapMBarContainer > .sapMLabel {
		flex : initial;
	}
}

.sapMBarLeft,
.sapMBarRight {
	position: absolute;
	height: 100%;
}

.sapMBarLeft {
	left: 0px;
}

.sapMBarRight {
	text-align: end;
	right: 0px;
}

.sapMBarPH {
	display: inline-block;
	height: 3rem;
	vertical-align: top;
}

.sapMBarMiddle {
	text-align: center;
	width: 100%;
	height: 0px;
	top: 0px;
	left: 0px;
}

.sapMBar .sapMLabel,
.sapMBar .sapMLnk {
	display: inline;
}

.sapMBarFlexBoxWidth100 {
	width: 100%;
}

/* select control in a bar */
.sapMBar .sapMSlt {
	vertical-align: top;
}

.sapMBar:active {
	text-shadow: none;
}

/* this "sapMBrandingBar-CTX" CSS class can be set by any container control that comes with its own branding bar, so any pages inside will not display their branding bar */
.sapMBrandingBar-CTX .sapMBar.sapMPageHeader:before,
/* since Shell may run without sap.m and we can run without the shell this selector is currently in our lib */
.sapUiUfdShell .sapMBar.sapMPageHeader::before {
	display: none;
}

@media (min-height: @_sap_m_Bar_MinHeightForHeader) {
	/* Shell draws its own branding bar */
	.sap-desktop .sapMShell .sapMBar.sapMPageHeader::before {
		display: none;
	}
}

.sapMBar .sapMBarLeft > .sapMImg,
.sapMBar .sapMBarPH > .sapMImg,
.sapMBar .sapMBarRight > .sapMImg {
	vertical-align: middle;
}

.sapMBarLeft > .sapMBarChild,
.sapMBarRight > .sapMBarChild {
	flex-shrink: 0;
}

.sapMBarLeft > .sapMBarChild.sapMPI,
.sapMBarRight > .sapMBarChild.sapMPI {
	flex-shrink: 1;
}

.sapMBarLeft {
	display: inline-flex;
	align-items: center;
	justify-content: flex-start;
}

.sapMBarPH {
	display: inline-flex;
	align-items: center;
	justify-content: center;
}

.sapMBarRight {
	display: inline-flex;
	align-items: center;
	justify-content: flex-end;
}

/* Compact size */
.sapUiSizeCompact {

	.sapMBar {
		height: 2rem;
		line-height: 2rem;
	}

	.sapMBarLeft,
	.sapMBarRight,
	.sapMBarPH {
		height: 2rem;
		line-height: 2rem;
	}
}

.sapMBarRight > .sapMBarChild:not(.sapMTB) {
	margin-top: 0;
	margin-bottom: 0;
	margin-right: 0;
}

.sapUiSizeCompact {

	.sapMBarRight > .sapMBarChild:not(.sapMTB) {
		margin-bottom: 0;
		margin-right: 0;
	}
}

.sapMBarLeft,
.sapMIBar.sapMBar .sapMBarChild:first-child {
	margin-left: 0;
}

/*To replace margins, add padding to the containers*/
.sapMBarRight,
.sapMBarLeft,
.sapMBarPH {
	box-sizing: border-box;

	.sapMObjectAttributeDiv .sapMText {
		vertical-align: text-top;
	}
}

.sapMBarRight,
.sapMBarPH {
	padding-right: 0.25rem;
}

.sapMBarLeft,
.sapMBarPH {
	padding-left: 0.25rem;
}

.sapMIBar > .sapMBarLeft > .sapMBarChild:last-child,
.sapMIBar .sapMBarPH > .sapMBarChild:last-child {
	margin-right: 0;
}

.sapMIBar > .sapMBarLeft > .sapMBarChild.sapMGenericTag:last-child {
	margin-right: 0.0625rem;
}

/* Bar Title Alignment */
.sapMBarTitleAlignStart .sapMBarPH {
	justify-content: flex-start;
}

html[data-sap-ui-browser^="ff"].sap-desktop .sapMBarTitleAlignStart .sapMBarPH {
	justify-content: start;
}
/* END "Bar.less" */

/* START "BarBase.less" */
/* ============================== */
/* CSS for control sap.m/BarBase  */
/* Base theme                     */
/* ============================== */

.sapMIBar .sapMBarChild:first-child {
	margin-left: 0;
}

.sapMIBar .sapMBarChild {
	margin-left: 0.25rem;
}

.sapMIBar > .sapMBarChild:last-child {
	margin-right: 0.25rem;
}

.sapMIBar-CTX.sapMFooter-CTX {
	color: var(--sapPageFooter_TextColor);
	/* 1px down black shadow with 50% opacity */
	text-shadow: @sapUiShadowText;
	background-color: var(--sapPageFooter_Background);
	background-image: none;
	border-top: 1px solid var(--sapPageFooter_BorderColor);
	box-shadow: none;
}

.sapMIBar-CTX.sapMFooter-CTX:active {
	text-shadow: none;
}

.sapMIBar.sapMSubHeader-CTX {
	color: var(--sapPageHeader_TextColor);
	background-color: var(--sapPageHeader_Background);
}

html.sap-desktop .sapMIBar.sapMFooter-CTX {
	background-color: var(--sapPageFooter_Background);
	border-top: 1px solid var(--sapPageFooter_BorderColor);
}

.sapMIBar-CTX.sapMFooter-CTX.sapMTBClear,
html.sap-desktop .sapContrast.sapMIBar.sapMFooter-CTX.sapMTBClear {
	border-top: none;
}

/* footer separator inside of a SplitContainer control */
.sapMSplitContainerMaster .sapMFooter-CTX:not(.sapMIBar) {
	border-right: 1px solid var(--sapPageFooter_BorderColor);
}

.sapMIBar.sapMHeader-CTX {
	color: var(--sapPageHeader_TextColor);
	background-color: var(--sapPageHeader_Background);
	filter: none;
	background-image: none;
}

.sapMPopup-CTX .sapMIBar.sapMHeader-CTX { /* remove gradient in popups; restore grey header color when in popup on desktop */
	background-color: var(--sapPageHeader_Background);
	filter: none;
	background-image: none;
}
/* END "BarBase.less" */

/* START "Breadcrumbs.less" */
/* ================================== */
/* CSS for control sap.m/Breadcrumbs  */
/* Base theme                         */
/* ================================== */
@_sap_m_Breadcrumbs_MinWidth_OFT: 1px;

.sapMBreadcrumbs {
	white-space: nowrap;
	outline: none;
	margin: 0 0 0.5rem 0;

	> ol {
		display: flex;
		margin: 0;
		padding: 0;
		list-style-type: none;
	}

	.sapMSlt {
		border: 0;
		margin: 0;
		height: 1.2rem;
		padding-right: 0.125rem;

		.sapMSltIcon {
			margin: 0 -0.4rem 0 -0.3rem;
			vertical-align: top;
			position: relative;
			background-color: transparent;
			line-height: inherit;
			border: none;
		}
	}

	.sapMBreadcrumbsItem.sapMBreadcrumbsSelectItem >
	.sapMSlt.sapMSltIconOnly.sapMSltWithIcon.sapMSltHoverable.sapMSltWithArrow {
		background: transparent;
		border: none;

		&.sapMSltFocused,
		&:hover {

			.sapMSltIcon::before {
				color: @sapUiLinkHover;
				border-bottom: 0.0625rem solid @sapUiLinkHover;
			}
		}
	}

	.sapMSltIcon::before {
		font-family: var(--sapContent_IconFontFamily);
		vertical-align: super;
		color: var(--sapLinkColor);
		speak: none;
		content: "...\e1ef";
		font-size: @sapMFontMediumSize;
		padding-bottom: 0.0625rem;
	}

	.sapMLnk,
	.sapMBreadcrumbsCurrentLocation,
	.sapMSlt,
	.sapMBreadcrumbsSeparator {
		vertical-align: middle;
	}

	.sapMBreadcrumbsItem {
		-webkit-flex-shrink: 0;
		flex-shrink: 0;
	}

	/* set the flex to 1 for the breadcrumbsItem in order for the last item's text to truncate properly, but only if
	there's two items (the select and the last item in the breadcrumbs) or if there's only one item in the breadcrumbs
	and no select. In all other cases the texts should not truncate thus leaving the responsiveness to the breadcrumbs
	mechanism (moving items from the breadcrumbs into the select)*/
	.sapMBreadcrumbsSelectItem + .sapMBreadcrumbsItem:last-child:nth-child(2),
	.sapMBreadcrumbsItem:last-child:nth-child(1) {
		min-width: 1%;
		-webkit-flex: 1;
		-webkit-box-flex: 1;
		flex: 1 1 auto;
	}

	.sapMBreadcrumbsCurrentLocation {
		color: var(--sapContent_LabelColor);

		&:focus {
			outline: 1px dotted var(--sapContent_FocusColor);
			outline-offset: -1px;
		}
	}

	.sapMBreadcrumbsItem {
		display: inline-flex;
		align-items: center;
		padding: 0;
		margin: 0;
		vertical-align: middle;
		white-space: nowrap;
	}

	.sapMBreadcrumbsSeparator {
		padding: 0 0.25rem;
		font-size: @sapMFontMediumSize;
	}

	.sapMBreadcrumbsItem,
	.sapMBreadcrumbsSeparator {
		color: var(--sapContent_LabelColor);
	}

	.sapMBreadcrumbsItem .sapMBreadcrumbsLink {
		padding-top: 0.25rem;
		padding-bottom: 0.25rem;
	}
}

.sapMTB .sapMBreadcrumbs {
	flex-grow: 1; /* needed when Breadcrumbs is placed in a container with display: flex */
}

.sapUiSizeCompact .sapMBreadcrumbs .sapMSlt {
	margin: 0;
	height: 1.2rem;
}

/* make Breadcrumbs shrinkable when used in OTB */
.sapMOTB .sapMBreadcrumbs {
	min-width: @_sap_m_Breadcrumbs_MinWidth_OFT; /* needed for width calculations on initial render */
	max-width: 100%;
}

.sapMOTB .sapMBreadcrumbs .sapMBreadcrumbsItem:last-child {
	flex-basis: 2.5rem;
	flex-grow: 1;
	overflow: hidden;
}

/* prevent Breadcrumbs with long text from causing a horizontal scrollbar in OTB Menu */
.sapMOverflowToolbarMenu-CTX .sapMBreadcrumbs {
	max-width: 90vw;
}
/* END "Breadcrumbs.less" */

/* START "BusyDialog.less" */
/* ================================= */
/* CSS for control sap.m/BusyDialog  */
/* Base theme                        */
/* ================================= */

@_sap_m_BusyDialog_IndicatorMargin: 1em;
@_sap_m_BusyDialog_IndicatorMarginCompact: 0.5rem;

//this selectors need to be more specific from the Dialog's
.sapMBusyDialog.sapMDialog {
	min-width: 18.75rem !important;
	text-align: center;
	width: 18.75rem !important;

	.sapMBusyIndicator {
		margin: @_sap_m_BusyDialog_IndicatorMargin;
	}

	&.sapUiPopupWithPadding .sapMDialogScrollCont {
		padding: 0 !important;
	}

	.sapMDialogSection {
		overflow: hidden !important;
	}

	&.sapMBusyDialog-Light {
		//lightBusyDialog
		box-shadow: none;
		background: transparent;
		border: none;

		.sapMDialogSection {
			background: transparent;
		}
	}
}

.sapMLabel.sapMBusyDialogLabel {
	display: block;
	font-size: 1rem;
	line-height: 1.5em;
	margin: 1rem 1rem 0;
	text-align: center !important;
	white-space: normal;
}

// compact sizing
.sapUiSizeCompact .sapMBusyDialog {
	.sapMBusyDialogLabel {
		margin-top: 0.5rem;
	}

	.sapMBusyIndicator {
		margin: @_sap_m_BusyDialog_IndicatorMarginCompact;
	}
}
/* END "BusyDialog.less" */

/* START "BusyIndicator.less" */
/* ==================================== */
/* CSS for control sap.m/BusyIndicator  */
/* Base theme                           */
/* ==================================== */

.sapMBusyIndicator {
	display: inline-block;
	text-align: center;
	outline: none;

	.sapMBusyIndicatorBusyArea {
		min-width: 3em;
		min-height: 1em;
	}

	.sapMLabel {
		display: block;
	}

	.sapUiLocalBusyIndicator {
		background: none;
		font-size: inherit;
	}
}

.sapMBsyIndAnimation(@name) {
	animation: @name 1s linear 0s infinite normal none;
}

html[dir='rtl'][data-sap-ui-animation-mode="full"] .sapMBsyIndIcon {
	.sapMBsyIndAnimation(sapMBsyIndRotateRTL);
}

.sapMBsyIndIcon {
	vertical-align: middle;
	transform: rotate(0deg) translateZ(0);
	transition-duration: 0ms;
}

.sapMBsyIndIcon {
	.sapMBsyIndAnimation(sapMBsyIndRotate);
}

html[data-sap-ui-animation-mode="none"] .sapMBsyIndIcon {
	animation: none;
}

html[data-sap-ui-animation-mode="basic"],
html[data-sap-ui-animation-mode="minimal"] {

	.sapMBsyIndIcon {
		animation-duration: 12s;
	}
}

@keyframes sapMBsyIndRotate {

	from {
		transform:rotate(0deg) translateZ(0);
	}

	to {
		transform:rotate(360deg) translateZ(0);
	}
}

@keyframes sapMBsyIndRotateRTL {

	from {
		transform:rotate(0deg) translateZ(0);
	}

	to {
		transform:rotate(-360deg) translateZ(0);
	}
}
/* END "BusyIndicator.less" */

/* START "Button.less" */
/* ============================= */
/* CSS for control sap.m/Button  */
/* Base theme                    */
/* ============================= */

.sapMBtn {
	height: 3rem;
	margin: 0;
	padding: 0.250rem 0;
	vertical-align: top;
	border: none;
	/* should be always transparent since this is the background of the clickable area */
	background-color: transparent;
}

.sapMBtn:focus {
	outline: none;
}
.sapMBtn::-moz-focus-inner {
	padding: 0;
	border: 0
}
.sapMBtn:not(.sapMBtnDisabled) {
	cursor: pointer;
}

/* override the default cursor of .sapUiIcon */
.sapMBtn .sapMBtnIcon {
	cursor: inherit;
}

.sapMBtnBase {
	position: relative;
}

.sapMBtn .sapMBtnInner {
	display: flex;
	justify-content: center;
}

.sapMBtn .sapMBtnContent {
	display: inline-block;
	flex-shrink: 1;
}

/* --------------------- */
/* Inner area            */
/* --------------------- */
.sapMBtnInner {
	vertical-align: top;
	-webkit-appearance: none;
	-webkit-user-select: none;
	-moz-user-select: none;
	-webkit-background-clip: padding-box;
	background-clip: padding-box;
	box-sizing: border-box;
	font-family: var(--sapFontFamily);
	font-size: 0.875rem;
	height: var(--sapElement_Height);
	min-width: 2.500rem;
	margin: 0;
	padding: 0;
	text-align: center;
	position: relative;
	background-color: var(--sapButton_Background);
	border: 0.0625rem solid var(--sapButton_BorderColor);
	border-radius: 0.2rem;
	color: var(--sapButton_TextColor);
	text-shadow: @sapUiShadowText;
}

.sapMBtnInner::-moz-focus-inner {
	border: 0;
}
.sapMBtnInner:focus,
.sapMBtnContent > span:focus {
	outline: none;
}

/* paddings depends on the presence and placement of text and icons */
.sapMBtnInner.sapMBtnText {
	padding-left: 0.75rem;
	padding-right: 0.75rem;
}

/* ------------------- */
/* Content area        */
/* ------------------- */
.sapMBtnContent {
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
	vertical-align: top;
	line-height: 2.375rem;
}

/* unstyled button content line height should be normal on iOS devices to fit into the button */
html[data-sap-ui-os^="iOS"] .sapMBtnBase:not(.sapMBtn) .sapMBtnContent {
	line-height: normal;
}

/* ------------------- */
/* Button Icon         */
/* ------------------- */
.sapMBtnIcon {
	font-size: 1.375rem;
	line-height: 2.375rem;
	width: 2.375rem;
	text-align: center;
	margin: 0;
	padding: 0;
	vertical-align: top;
	max-height: 2.375rem;
	color: var(--sapButton_IconColor);
}

.sapMBtnInner .sapMBtnIcon.sapMImg {
	height: 2.375rem;
	max-width: 2.375rem;
	width: auto;
}

/* ------------------------------------------------ */
/* Buttons in a Bar                                 */
/* (add separators, remove top/bottom border)       */
/* ------------------------------------------------ */

.sapMIBar .sapMBtnInner {
	vertical-align: top;
}

/* ------------------------------------------------ */
/* Compact size                                     */
/* ------------------------------------------------ */

.sapUiSizeCompact {

	.sapMBtn {
		height: 2.000rem;
		padding: 0.1875rem 0;
	}

	.sapMBtnInner {
		height: var(--sapElement_Compact_Height);
		min-width: 2.0rem;
	}

	.sapMBtnContent {
		line-height: 1.500rem;
	}

	.sapMBtnIcon {
		line-height: 1.500rem;
		width: 1.000rem;
		font-size: 1.000rem;
		max-height: 1.500rem;
	}

	.sapMBtnInner > img {
		height: 1.250rem;
		width: auto;
		max-width: 1.250rem;
		margin-left: 0rem;
		margin-right: 0rem;
	}
}

/* ------------------------------------------------ */
/* Condensed size                                   */
/* ------------------------------------------------ */

.sapUiSizeCondensed .sapUiTableDataCell {

	.sapMBtn {
		height: 1.375rem;
		padding-top: 0;
		padding-bottom: 0;
	}

	.sapMBtnInner {
		height: 1.375rem;
		min-width: 2.0rem;
	}

	.sapMBtnInner.sapMBtnText,
	.sapMBtnInner1.sapMBtnIconFirst:not(.sapMBtnBack) {
		padding-left: 0.4375rem;
		padding-right: 0.4375rem;
	}

	.sapMBtnInner > img {
		height: 1.250rem;
		width: auto;
		max-width: 1.250rem;
		margin-left: 0.250rem;
		margin-right: 0.250rem;
	}

	.sapMIBar-CTX .sapMBtnInner {
		margin: 0;
	}

	.sapMBtnIcon {
		line-height: 1.250rem;
		font-size: 1.000rem;
		max-height: 1.250rem;
		width: 1.000rem;
		min-width: 1.000rem;
	}

	.sapMBtnContent {
		line-height: 1.250rem;
	}
}

/* ---------------------------------------------------------------------*/
/* Disabled, hover and active states:                                   */
/* ---------------------------------------------------------------------*/

.sapMBtnDisabled {
	text-shadow: none;
	cursor: default;
}

.sapMBtn:hover > .sapMBtnHoverable {
	background-image: none;
	background-color: var(--sapButton_Hover_Background);
	border-color: var(--sapButton_Hover_BorderColor);
}

.sapMBtn:hover:not(.sapMBtnDisabled):not(.sapMSBActive) > .sapMBtnHoverable:not(.sapMBtnDisabled):not(.sapMBtnActive):not(.sapMToggleBtnPressed):not(.sapMBtnTransparent) {

	&.sapMBtnDefault,
	&.sapMBtnGhost {
		color: var(--sapButton_Hover_TextColor);

		.sapUiIcon {
			color: var(--sapButton_Hover_TextColor);
		}
	}
}

:not(.sapMBtnDisabled) > .sapMBtnInner.sapMBtnActive {
	background-image: none;
	background-color: var(--sapButton_Active_Background);
	border-color: var(--sapButton_Active_BorderColor);
	color: var(--sapButton_Active_TextColor);
	text-shadow: none;
}

:not(.sapMBtnDisabled) > .sapMBtnInner.sapMBtnActive .sapMBtnIcon {
	color: var(--sapButton_Active_TextColor);
}

/* ---------------------------------------------------------------------*/
/* Special buttons have the same look on a page and in a header/footer. */
/* ---------------------------------------------------------------------*/

span.sapMBtnInner.sapMBtnAccept {
	background-image: none;
	background-color: var(--sapButton_Accept_Background);
	border-color: var(--sapButton_Accept_BorderColor);
	color: var(--sapButton_Accept_TextColor);

	.sapUiIcon {
		color: var(--sapButton_Accept_TextColor);
	}
}

span.sapMBtnInner.sapMBtnAttention {
	background-image: none;
	background-color: var(--sapButton_Attention_Background);
	border-color: var(--sapButton_Attention_BorderColor);
	color: var(--sapButton_Attention_TextColor);

	.sapUiIcon {
		color: var(--sapButton_Attention_TextColor);
	}
}

:not(.sapMBtnDisabled):not(.sapMSBActive) > span.sapMBtnInner.sapMBtnAccept:not(.sapMBtnActive):not(.sapMToggleBtnPressed) > .sapMBtnIcon {
	color: var(--sapButton_Accept_TextColor);
}

:not(.sapMBtnDisabled):not(.sapMSBActive) > span.sapMBtnInner.sapMBtnAttention:not(.sapMBtnActive):not(.sapMToggleBtnPressed) > .sapMBtnIcon {
	color: var(--sapButton_Attention_TextColor);
}

.sapMBtn:hover > .sapMBtnHoverable.sapMBtnAccept:not(.sapMBtnActive):not(.sapMBtnDisabled):not(.sapMToggleBtnPressed) {
	background-color: var(--sapButton_Accept_Hover_Background);
	border-color: var(--sapButton_Accept_Hover_BorderColor);
	color: var(--sapButton_Accept_Hover_TextColor);

	bdi {
		color: var(--sapButton_Accept_Hover_TextColor);
	}
}

.sapMBtn:hover > .sapMBtnHoverable.sapMBtnAttention:not(.sapMBtnActive):not(.sapMBtnDisabled):not(.sapMToggleBtnPressed) {
	background-color: var(--sapButton_Attention_Hover_Background);
	border-color: var(--sapButton_Attention_Hover_BorderColor);
	color: var(--sapButton_Attention_Hover_TextColor);

	bdi {
		color: var(--sapButton_Attention_Hover_TextColor);
	}
}

.sapMBtn:hover:not(.sapMBtnDisabled) > .sapMBtnHoverable.sapMBtnAttention:not(.sapMBtnActive):not(.sapMToggleBtnPressed) > .sapMBtnIcon,
.sapMBtn:hover:not(.sapMBtnDisabled) > span.sapMBtnInner.sapMBtnAttention:hover:not(.sapMBtnDisabled):not(.sapMBtnActive):not(.sapMToggleBtnPressed) .sapUiIcon,
:not(.sapMBtnDisabled):not(.sapMSBActive) > span.sapMBtnInner.sapMBtnAttention:not(.sapMBtnActive):not(.sapMBtnDisabled):not(.sapMToggleBtnPressed):hover > .sapMBtnIcon {
	color: var(--sapButton_Attention_Hover_TextColor);
}

.sapMBtn:hover:not(.sapMBtnDisabled) > .sapMBtnHoverable.sapMBtnAccept:not(.sapMBtnActive):not(.sapMToggleBtnPressed) > .sapMBtnIcon,
.sapMBtn:hover:not(.sapMBtnDisabled) > span.sapMBtnInner.sapMBtnAccept:hover:not(.sapMBtnDisabled):not(.sapMBtnActive):not(.sapMToggleBtnPressed) .sapUiIcon,
:not(.sapMBtnDisabled):not(.sapMSBActive) > span.sapMBtnInner.sapMBtnAccept:not(.sapMBtnActive):not(.sapMBtnDisabled):not(.sapMToggleBtnPressed):hover > .sapMBtnIcon {
	color: var(--sapButton_Accept_Hover_TextColor);
}

:not(.sapMBtnDisabled) > span.sapMBtnInner.sapMBtnAccept.sapMBtnActive {
	background-color: var(--sapButton_Accept_Active_Background);
	border-color: var(--sapButton_Accept_Active_BorderColor);
	text-shadow: none;
}

:not(.sapMBtnDisabled) > span.sapMBtnInner.sapMBtnAttention.sapMBtnActive {
	background-color: var(--sapButton_Attention_Active_Background);
	border-color: var(--sapButton_Attention_Active_BorderColor);
	text-shadow: none;
}

/* ---------------------------------------- */

span.sapMBtnInner.sapMBtnReject {
	background-image: none;
	background-color: var(--sapButton_Reject_Background);
	border-color: var(--sapButton_Reject_BorderColor);
	color: var(--sapButton_Reject_TextColor);

	.sapUiIcon {
		color: var(--sapButton_Reject_TextColor);
	}
}

:not(.sapMBtnDisabled):not(.sapMSBActive) > span.sapMBtnInner.sapMBtnReject:not(.sapMBtnActive):not(.sapMToggleBtnPressed) > .sapMBtnIcon {
	color: var(--sapButton_Reject_TextColor);
}

.sapMBtn:hover > .sapMBtnHoverable.sapMBtnReject:not(.sapMBtnActive):not(.sapMToggleBtnPressed) {
	background-color: var(--sapButton_Reject_Hover_Background);
	border-color: var(--sapButton_Reject_Hover_BorderColor);
	color: var(--sapButton_Reject_Hover_TextColor);

	bdi {
		color: var(--sapButton_Reject_Hover_TextColor);
	}
}

.sapMBtn:hover:not(.sapMBtnDisabled) > .sapMBtnHoverable.sapMBtnReject:not(.sapMBtnActive):not(.sapMToggleBtnPressed) > .sapMBtnIcon,
.sapMBtn:hover:not(.sapMBtnDisabled) > span.sapMBtnInner.sapMBtnReject:hover:not(.sapMBtnDisabled):not(.sapMBtnActive):not(.sapMToggleBtnPressed) .sapUiIcon,
:not(.sapMBtnDisabled):not(.sapMSBActive) > span.sapMBtnInner.sapMBtnReject:not(.sapMBtnActive):not(.sapMBtnDisabled):not(.sapMToggleBtnPressed):hover > .sapMBtnIcon {
	color: var(--sapButton_Reject_Hover_TextColor);
}

:not(.sapMBtnDisabled) > span.sapMBtnInner.sapMBtnReject.sapMBtnActive {
	background-color: var(--sapButton_Reject_Active_Background);
	border-color: var(--sapButton_Reject_Active_BorderColor);
	text-shadow: none;
}

/* ---------------------------------------- */

span.sapMBtnInner.sapMBtnEmphasized {
	background-image: none;
	background-color: var(--sapButton_Emphasized_Background);
	border-color: var(--sapButton_Emphasized_BorderColor);
	color: var(--sapButton_Emphasized_TextColor);
	text-shadow: 0 0 0.125rem var(--sapButton_Emphasized_TextShadow);

	.sapUiIcon, sapMBtnIcon {
		color: var(--sapButton_Emphasized_TextColor);
	}
}

:not(.sapMBtnDisabled) > span.sapMBtnInner.sapMBtnEmphasized.sapMBtnActive > .sapMBtnIcon {
	color: var(--sapButton_Emphasized_Active_TextColor);
}

.sapMBtn:hover > .sapMBtnHoverable.sapMBtnEmphasized:not(.sapMBtnActive):not(.sapMToggleBtnPressed) {
	background-color: var(--sapButton_Emphasized_Hover_Background);
	border-color: var(--sapButton_Emphasized_Hover_BorderColor);
	color: var(--sapButton_Emphasized_Hover_TextColor);

	.sapMBtnIcon {
		color: var(--sapButton_Emphasized_Hover_TextColor);
	}
}

:not(.sapMBtnDisabled) > span.sapMBtnInner.sapMBtnEmphasized.sapMBtnActive {
	background-color: var(--sapButton_Emphasized_Active_Background);
	border-color: var(--sapButton_Emphasized_Active_BorderColor);
	text-shadow: none;
	font-weight: var(--sapButton_Emphasized_FontWeight);
	color: var(--sapButton_Emphasized_TextColor);

	bdi {
		color: var(--sapButton_Active_TextColor);
	}
}

.sapMBtn .sapMBtnCritical,
.sapContrast span.sapMBtnInner.sapMBtnCritical {
	background: var(--sapButton_Critical_Background);
	border-color: var(--sapButton_Critical_BorderColor);

	.sapMBtnContent,
	.sapMBtnIcon {
		color: var(--sapButton_Critical_TextColor);
	}
}

.sapMBtn .sapMBtnNegative,
.sapContrast span.sapMBtnInner.sapMBtnNegative {
	background: var(--sapButton_Negative_Background);
	border-color: var(--sapButton_Negative_BorderColor);

	.sapMBtnContent,
	.sapMBtnIcon {
		color: var(--sapButton_Negative_TextColor);
	}
}

.sapMBtn .sapMBtnSuccess,
.sapContrast span.sapMBtnInner.sapMBtnSuccess {
	background: var(--sapButton_Success_Background);
	border-color: var(--sapButton_Success_BorderColor);

	.sapMBtnContent,
	.sapMBtnIcon {
		color: var(--sapButton_Success_TextColor);
	}
}

.sapMBtn .sapMBtnNeutral,
.sapContrast span.sapMBtnInner.sapMBtnNeutral {
	background: var(--sapButton_Neutral_Background);
	border-color: var(--sapButton_Neutral_BorderColor);

	.sapMBtnContent,
	.sapMBtnIcon {
		color: var(--sapButton_Neutral_TextColor);
	}
}

.sapMBtn .sapMBtnCritical.sapMBtnInner,
.sapMBtn .sapMBtnNegative.sapMBtnInner,
.sapMBtn .sapMBtnSuccess.sapMBtnInner,
.sapMBtn .sapMBtnNeutral.sapMBtnInner {
	text-shadow: var(--sapContent_ContrastTextShadow);
}

.sapMBtn:not(.sapMBtnDisabled):hover .sapMBtnCritical {
	background: var(--sapButton_Critical_Hover_Background);
	border-color: var(--sapButton_Critical_Hover_BorderColor);

	.sapMBtnContent,
	.sapMBtnIcon {
		color: var(--sapButton_Critical_Hover_TextColor);
	}
}

.sapMBtn:not(.sapMBtnDisabled):hover .sapMBtnNegative {
	background: var(--sapButton_Negative_Hover_Background);
	border-color: var(--sapButton_Negative_Hover_BorderColor);

	.sapMBtnContent,
	.sapMBtnIcon {
		color: var(--sapButton_Negative_Hover_TextColor);
	}
}

.sapMBtn:not(.sapMBtnDisabled):hover .sapMBtnSuccess {
	background: var(--sapButton_Success_Hover_Background);
	border-color: var(--sapButton_Success_Hover_BorderColor);

	.sapMBtnContent,
	.sapMBtnIcon {
		color: var(--sapButton_Success_Hover_TextColor);
	}
}

.sapMBtn:not(.sapMBtnDisabled):hover .sapMBtnNeutral {
	background: var(--apButton_Neutral_Hover_Background);
	border-color: var(--sapButton_Neutral_Hover_BorderColor);

	.sapMBtnContent,
	.sapMBtnIcon {
		color: var(--sapButton_Neutral_Hover_TextColor);
	}
}

.sapMBtn:not(.sapMBtnDisabled) .sapMBtnCritical.sapMBtnActive {
	background: var(--sapButton_Critical_Active_Background);
	border-color: var(--sapButton_Critical_Active_BorderColor);

	.sapMBtnContent,
	.sapMBtnIcon {
		color: var(--sapButton_Critical_Active_TextColor);
	}
}

.sapMBtn:not(.sapMBtnDisabled) .sapMBtnNegative.sapMBtnActive {
	background: var(--sapButton_Negative_Active_Background);
	border-color: var(--sapButton_Negative_Active_BorderColor);

	.sapMBtnContent,
	.sapMBtnIcon {
		color: var(--sapButton_Negative_Active_TextColor);
	}
}

.sapMBtn:not(.sapMBtnDisabled) .sapMBtnSuccess.sapMBtnActive {
	background: var(--sapButton_Success_Active_Background);
	border-color: var(--sapButton_Success_Active_BorderColor);

	.sapMBtnContent,
	.sapMBtnIcon {
		color: var(--sapButton_Success_Active_TextColor);
	}
}

.sapMBtn:not(.sapMBtnDisabled) .sapMBtnNeutral.sapMBtnActive {
	background: var(--sapButton_Neutral_Active_Background);
	border-color: var(--sapButton_Neutral_Active_BorderColor);

	.sapMBtnContent,
	.sapMBtnIcon {
		color: var(--sapButton_Neutral_Active_TextColor);
	}
}

/* ---------------------------------------------------------------------*/
/* Styles for buttons in Toolbar of type Info.                          */
/* ---------------------------------------------------------------------*/

.sapMTB-Info-CTX .sapMBtnInner,
.sapMTB-Info-CTX .sapMBtnInner:hover {
	color: var(--sapInfobar_TextColor);
	text-shadow: none;
}

.sapMIBar-CTX .sapMBtn:hover > .sapMBtnTransparent.sapMBtnHoverable:not(.sapMBtnActive):not(.sapMToggleBtnPressed) bdi {
	color: var(--sapContent_ContrastIconColor);
}

.sapMTB-Info-CTX .sapMBtn:hover > .sapMBtnTransparent.sapMBtnHoverable:not(.sapMBtnActive):not(.sapMToggleBtnPressed) bdi,
.sapMTB-Info-CTX .sapMBtn:hover:not(.sapMBtnDisabled) > .sapMBtnTransparent.sapMBtnHoverable:not(.sapMBtnActive):not(.sapMToggleBtnPressed):not(.sapMBtnEmphasized) .sapUiIcon {
	color: var(--sapList_TextColor);
}

/* Button, used as a focus dom ref for active toolbar */
.sapUiSizeCompact .sapMTBActiveButton,
.sapMTBActiveButton {
	position: relative;
	width: 0;
	height: 0;
	opacity: 0;
	z-index: -1;
	overflow: hidden;
	pointer-events: none;
}

/* ---------------------------------------------------------------------*/
/* Transparent buttons looks like normal one in a header/footer.        */
/* The following rules are overridden in a bar below:                   */
/* ---------------------------------------------------------------------*/

.sapMBtnBack,
.sapMBtnTransparent {
	background-image: none;
	background-color: var(--sapButton_Lite_Background);
	border-color: var(--sapButton_Lite_BorderColor);
	color: var(--sapButton_Lite_TextColor);
}

.sapMBtnGhost {
	background-image: none;
	border-color: fade(@sapUiButtonLiteTextColor, 50);
	background-color: transparent;
	color: var(--sapButton_Lite_TextColor);
}

.sapMBtnGhost.sapMBtnInner {
	text-shadow: @sapUiShadowText;
}

:not(.sapMBtnDisabled) .sapMBtnBack > .sapMBtnIcon,
:not(.sapMBtnDisabled) .sapMBtnTransparent > .sapMBtnIcon,
:not(.sapMBtnDisabled) .sapMBtnGhost > .sapMBtnIcon {
	color: var(--sapButton_IconColor);
}

.sapMBtn:hover > .sapMBtnBack.sapMBtnHoverable {
	background-color: var(--sapButton_Lite_Hover_Background);
	border-color: var(--sapButton_Lite_Hover_BorderColor);
}

.sapMBtn:hover:not(.sapMBtnDisabled) > .sapMBtnTransparent.sapMBtnHoverable:not(.sapMBtnActive):not(.sapMToggleBtnPressed):not(.sapMBtnEmphasized) {
	color: var(--sapButton_Lite_Hover_TextColor);

	.sapUiIcon {
		color: var(--sapButton_Lite_Hover_TextColor);
	}
}

.sapMBtn:hover:not(.sapMBtnDisabled) > .sapMBtnHoverable.sapMBtnTransparent:not(.sapMBtnActive):not(.sapMToggleBtnPressed) > .sapMBtnIcon,
*.sapMBtn:hover:not(.sapMBtnDisabled) > span.sapMBtnInner.sapMBtnTransparent:not(.sapMBtnActive):not(.sapMToggleBtnPressed) > .sapMBtnIcon,
:not(.sapMBtnDisabled):not(.sapMSBActive) > span.sapMBtnInner.sapMBtnTransparent:not(.sapMBtnActive):not(.sapMToggleBtnPressed):hover > .sapMBtnIcon {
	color: var(--sapButton_Lite_Hover_TextColor);
}

.sapMBtn:hover > .sapMBtnGhost.sapMBtnHoverable {
	background-color: fade(@sapUiButtonLiteTextColor, 20);
}

:not(.sapMBtnDisabled) > .sapMBtnBack.sapMBtnActive,
:not(.sapMBtnDisabled):hover > .sapMBtnBack.sapMBtnActive,
:not(.sapMBtnDisabled) > .sapMBtnTransparent.sapMBtnActive,
:not(.sapMBtnDisabled):hover > .sapMBtnTransparent.sapMBtnActive,
:not(.sapMBtnDisabled) > .sapMBtnGhost.sapMBtnActive,
:not(.sapMBtnDisabled):hover > .sapMBtnGhost.sapMBtnActive {
	background-color: var(--sapButton_Lite_Active_Background);
	border-color: var(--sapButton_Lite_Active_BorderColor);
	text-shadow: none;
}

/* Transparent button in Light Info toolbar/Facet filter */
.sapMTB-Info-CTX {

	:not(.sapMBtnDisabled) .sapMBtnBack > .sapMBtnIcon,
	:not(.sapMBtnDisabled) .sapMBtnTransparent > .sapMBtnIcon,
	:not(.sapMBtnDisabled) .sapMBtnGhost > .sapMBtnIcon {
		color: var(--sapInfobar_TextColor);
	}
}

/* ----------------------------------------------------------------- */
/* Default type Buttons in a Toolbar should be styled as transparent */
/* ----------------------------------------------------------------- */

/* transparent Toolbar Context */
.sapMTB-Transparent-CTX .sapMBtnDefault {
	background-image: none;
	background: var(--sapButton_Lite_Background);
}

.sapMTB-Transparent-CTX .sapMBtn:hover > .sapMBtnDefault.sapMBtnHoverable {
	background: var(--sapButton_Lite_Hover_Background);
}

/* ------------------------------------------------ */
/* Compact size                                     */
/* ------------------------------------------------ */

.sapUiSizeCompact {

	.sapMBtnInner {
		text-shadow: none;
	}

	.sapMBtnGhost {
		text-shadow: @sapUiShadowText;
	}

	.sapMBar-CTX .sapMBtnDefault.sapMBtnInner:not(.sapMBtnDisabled) {
		margin: 0;
		border-color: transparent;
	}

	.sapMBar-CTX .sapMBtnInner.sapMBtnGhost:not(.sapMBtnDisabled) {
		border-color: fade(@sapUiButtonLiteTextColor, 50)
	}

	.sapMBtnFocusDiv {
		left: 0;
		top: 0;
		right: 0;
		bottom: 0;
		border-radius: 0;
	}
}

/* ------------------------------------------------ */
/* Condensed size                                   */
/* ------------------------------------------------ */

.sapUiSizeCondensed .sapUiTableDataCell {

	.sapMBtnInner {
		text-shadow: none;
	}

	.sapMBar-CTX .sapMBtnInner {
		border-color: transparent;
	}

	.sapMBtnContent {
		line-height: 1.250rem;
	}

	.sapMBtnFocusDiv {
		left: 0;
		top: 0;
		right: 0;
		bottom: 0;
		border-radius: 0;
	}
}

/* ----------------------*/
/* Badges                */
/* ----------------------*/

.sapMBtn {
	overflow: visible;

	.sapMBadge {

		&.sapMBadgeTopRight .sapMBadgeIndicator {
			z-index: 1;
			top: -0.3125rem;
			right: -0.4375rem;
		}

		.sapMBadgeIndicator::after {
			text-overflow: clip;
			height: 0.875rem;
			line-height: 0.875rem;
			padding: 0 0.1875rem;
			background-color: var(--sapContent_BadgeBackground);
			border-color: var(--sapContent_BadgeBackground);
			color: var(--sapContent_BadgeTextColor);
		}
		.sapMBadgeIndicator.sapMBadgeAttention {
			width: 0.5rem;
			height: 0.5rem;
			top: -0.125rem;
			right: -0.125rem;
		}
	}
}

.sapUiSizeCompact {

	.sapMBtn {

		.sapMBadgeTopRight.sapMBadge {

			&.sapMBtnIconFirst:not(.sapMBtnText) {
				padding: 0 0.4375rem;
			}

			.sapMBadgeIndicator:not(.sapMBadgeAttention) {
				line-height: 2rem;
				display: inline-block;
				position: relative;
				top: 0.25rem;
				right: -0.25rem;
				padding-right: 0.0625rem;
			}

			.sapMBadgeIndicator.sapMBadgeAttention {
				width: 0.375rem;
				height: 0.375rem;
			}
		}
	}
}
/* END "Button.less" */

/* START "Carousel.less" */
/* =============================== */
/* CSS for control sap.m/Carousel  */
/* Base theme                      */
/* =============================== */

@_sap_m_Carousel_BackgroundColorSolid: var(--sapGroup_ContentBackground);
@_sap_m_Carousel_BackgroundColorTransparent: transparent;
@_sap_m_Carousel_BackgroundColorTranslucent: var(--sapGroup_ContentBackground);
@_sap_m_Carousel_MinWidth: 20rem;
@_sap_m_Carousel_PageIndicatorBarHeight: 3.5rem;
@_sap_m_Carousel_BottomArrowsOffset: calc(~'3.5rem + 1px');
@_sap_m_Carousel_ControlsNoArrowsHeight: 2rem;
@_sap_m_Carousel_NumericIndicatorTextColor: var(--sapContent_LabelColor);
@_sap_m_Carousel_PrevAndNextSize: 2rem;
@_sap_m_Carousel_PrevAndNextPosition: 0.625rem;
@_sap_m_Carousel_PrevAndNextFocusBorderRadius: unset;
@_sap_m_Carousel_PageIndicatorContainerPadding: 1rem;
@_sap_m_Carousel_PageIndicatorWidthSizeS: 12rem;
@_sap_m_Carousel_PageIndicatorWidthSizeM: 16rem;
@_sap_m_Carousel_PageIndicatorWidthSizeL: 20rem;
@_sap_m_Carousel_PageIndicatorWidthSizeXL: 24rem;
@_sap_m_Carousel_BulletInactiveBackground: var(--sapContent_NonInteractiveIconColor);
@_sap_m_Carousel_BulletInactiveBorderColor: var(--sapContent_NonInteractiveIconColor);
@_sap_m_Carousel_BulletActiveBackground: var(--sapSelectedColor);
@_sap_m_Carousel_BulletActiveBorder: @_sap_m_Carousel_BulletActiveBackground;
@_sap_m_Carousel_BulletInactiveWidth: 0.25rem;
@_sap_m_Carousel_BulletInactiveMargin: 0 0.375rem;
@_sap_m_Carousel_FocusBorderOffsetArrow: -0.125rem;
@_sap_m_Carousel_ContrastFocusBorder: var(--sapContent_FocusWidth) solid var(--sapContent_ContrastFocusColor);
@_sap_m_Carousel_FocusBorder: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);

.sapMCrsl {
	position: relative;
	overflow: hidden;
	box-sizing: border-box;
	min-width: @_sap_m_Carousel_MinWidth;
}

.sapMCrslBackground-Solid {
	background-color: @_sap_m_Carousel_BackgroundColorSolid;
}

.sapMCrslBackground-Transparent {
	background-color: @_sap_m_Carousel_BackgroundColorTransparent;
}

.sapMCrslBackground-Translucent {
	background-color: @_sap_m_Carousel_BackgroundColorTranslucent;
}

.sapMCrslItem:focus {
	position: relative;
}

.sapMCrslItem:focus,
.sapMCrslNoDataItem:focus {
	outline: none;

	&::before,
	&::after {
		position: absolute;
		content: ' ';
		top: 0;
		right: 0;
		bottom: 0;
		left: 0;
		pointer-events: none;
		z-index: 2;
	}

	&::before {
		border: @_sap_m_Carousel_ContrastFocusBorder;
	}

	&::after {
		border: @_sap_m_Carousel_FocusBorder;
	}
}

.sapMCrslNoDataItem {
	height: 100%;
}

.sapMCrsl.sapMCrslLeft {
	text-align: left;
}

.sapMCrsl.sapMCrslCenter {
	text-align: center;
}

.sapMCrsl.sapMCrslFluid > .sapMCrslInner > * {
	width: 100%;
}

.sapMCrsl.sapMCrslFluid.sapMCrslCenter > .sapMCrslInner >*:first-child {
	margin-left: 0;
}

.sapMCrslInner {
	height: 100%;
	position: relative;
	white-space: nowrap;
	text-align: begin;
	/* Removes inline-block gaps due to whitespace */
	font-size: 0;
	/* Configure  animations */
	transition-property: transform;
	transition-timing-function: cubic-bezier(0.46, 0, 0.44, 1);
	transition-duration: 0.5s;
}

/* for tablet and phone the duration is smaller */
@media all and (max-width: 1024px) {

	.sapMCrslInner {
		transition-duration: 0.3s;
	}
}

html[data-sap-ui-animation='off'] .sapMCrsl .sapMCrslInner {
	transition: none;
 }

.sapMCrslItemTable {
	height: 100%;
}

.sapMCrslItemTableCell {
	height: 100%;
}

.sapMCrslImg {
	text-align: center;
}

.sapMCrslImgNoArrows {
	text-align: center;
}

.sapMCrslImgNoArrows::before,
.sapMCrslImg::before {
	content: ' ';
	display: inline-block;
	height: 100%;
	vertical-align: middle;
}

.sapMCrslItemTableCell.sapMCrslImgNoArrows img,
.sapMCrslItemTableCell.sapMCrslImg img {
	vertical-align: middle;
	max-width: 100%;
	max-height: 100%;
}
.sapMCrsl.sapMCrslDragging .sapMCrslInner,
.sapMCrsl .sapMCrslInner.sapMCrslNoTransition {
	transition: none;
}

.sapMCrslInner.sapMCrslHideNonActive >:not(.sapMCrslActive) .sapMCrslItemTableCell {
	visibility: hidden;
}

.sapMCrslInner.sapMCrslInnerNoPages {
	display: flex;
	align-items: center;
}

.sapMCrslInner > * {
	display: inline-block;
	vertical-align: top;
	white-space: normal;
	/* Comment from Mobify-Carousel: 'We  must  reset  the  font-size here'. Needs explanation! */
	font-size: 1rem;
}

.sapMCrslFluid > .sapMCrslInner > * {
	box-sizing: border-box;
}

.sapMCrslItemTable,
.sapMCrslItem > .sapMScrollCont > .sapMScrollContScroll {
	height: 100%;
	width: 100%;
}

/* mixin call for creating container content padding classes (arguments: rootContainer, contentSelector) */
.sapUiContainerContentPadding(~'.sapMCrsl', ~'> .sapMCrslInner > .sapMCrslItem > .sapMScrollCont > .sapMScrollContScroll');

/*
  Carousel control base styling
*/
.sapMCrslControls {
	text-align: center;
	height: @_sap_m_Carousel_PageIndicatorBarHeight;
	display: flex;
	justify-content: center;
	width: 100%;
}

.sapMCrslControlsNoArrows {
	text-align: center;
	height: @_sap_m_Carousel_ControlsNoArrowsHeight;
	display: flex;
	justify-content: center;
	width: 100%;
}

.sapMCrslHud .sapMCrslArrow {
	z-index: 2;
	opacity: 1;
	display: block;
	position: absolute;
	margin: -1.5625px 0 0 0;
	box-shadow: var(--sapContent_Shadow0);

	&:hover,
	&:focus {
		opacity: 1;
		display: block;
	}
}

html.sap-desktop .sapMCrslHud .sapMCrslArrow {
	opacity: 0;
	display: none;
}

.sapMCrslPrev,
.sapMCrslNext {
	width: @_sap_m_Carousel_PrevAndNextSize;
	height: @_sap_m_Carousel_PrevAndNextSize;
	padding: 0;
	text-decoration: none;
	border-radius: 50%;
	transition: opacity 0.1s ease-in;
	box-sizing: border-box;
	background-color: var(--sapButton_Background);
	border: var(--sapButton_BorderWidth) solid var(--sapButton_BorderColor);
	color: var(--sapButton_IconColor);
	cursor: pointer;
	-webkit-user-select: none;
	-moz-user-select: -moz-none;
	user-select: none;
	-webkit-user-drag: none;
	-moz-user-drag: -moz-none;
	user-drag: none;

	&:hover {
		background: var(--sapButton_Hover_Background);
		border-color: var(--sapButton_Hover_BorderColor);
		color: var(--sapButton_Hover_TextColor);
	}

	&:active {
		background-color: var(--sapButton_Active_Background);
		border-color: var(--sapButton_Active_BorderColor);
		color: var(--sapButton_Active_TextColor);
		box-shadow: none;
	}
}

.sapMCrslHudMiddle .sapMCrslArrow {
	top: calc(~'50% - 1rem');
}

.sapMCrslHudTop,
.sapMCrslHudOverContentTop {

	.sapMCrslArrow {
		top: 50%;
	}
}

.sapMCrslHudBottom,
.sapMCrslHudOverContentBottom {

	.sapMCrslArrow {
		top: calc(~'50% - 2rem');
	}
}

.sapMCrslBulleted > span {
	text-indent: 2rem;
	line-height: 0;
	text-decoration: none;
	overflow: hidden;
	display: inline-block;
	width: @_sap_m_Carousel_BulletInactiveWidth;
	height: @_sap_m_Carousel_BulletInactiveWidth;
	margin: @_sap_m_Carousel_BulletInactiveMargin;
	transition: background-color 0.1s ease-in;
	border-radius: 50%;
	align-self: center;
	box-sizing: border-box;
	background-color: @_sap_m_Carousel_BulletInactiveBackground;
	border: 1px solid @_sap_m_Carousel_BulletInactiveBorderColor;
}

/* Page indicator at the top */
.sapMCrslTopOffset {

	.sapMCrslItem {
		padding-top: calc(@_sap_m_Carousel_ControlsNoArrowsHeight ~'+' 1px);

		&:focus {

			&::before,
			&::after {
				top: calc(@_sap_m_Carousel_ControlsNoArrowsHeight ~'+' 1px);
			}
		}
	}

	&.sapMCrslTopArrowsOffset .sapMCrslItem {
		padding-top: @_sap_m_Carousel_BottomArrowsOffset;

		&:focus {

			&::before,
			&::after {
				top: @_sap_m_Carousel_BottomArrowsOffset;
			}
		}
	}
}

/* Page indicator at the bottom */
.sapMCrslBottomOffset {

	.sapMCrslItem {
		padding-bottom: calc(@_sap_m_Carousel_ControlsNoArrowsHeight ~'+' 1px);

		&:focus {

			&::before,
			&::after {
				bottom: calc(@_sap_m_Carousel_ControlsNoArrowsHeight ~'+' 1px);
			}
		}
	}

	&.sapMCrslBottomArrowsOffset .sapMCrslItem {
		padding-bottom: @_sap_m_Carousel_BottomArrowsOffset;

		&:focus {

			&::before,
			&::after {
				bottom: @_sap_m_Carousel_BottomArrowsOffset;
			}
		}
	}
}

.sapMCrslControlsContainer {
	width: 100%;
	align-self: center;
	height: 2.5rem;
	display: flex;
	justify-content: center;
	margin-left: 1rem;
	margin-right: 1rem;
}

.sapMCrslControls.sapMCrslControlsTop,
.sapMCrslControlsNoArrows.sapMCrslControlsTop,
.sapMCrslControlsOverContentTop {
	position: absolute;
	top: 0;
	width: 100%;
	z-index: 1;
	align-self: center;
}

.sapMCrslControls.sapMCrslControlsBottom,
.sapMCrslControlsNoArrows.sapMCrslControlsBottom,
.sapMCrslControlsOverContentBottom {
	position: absolute;
	bottom: 0;
	width: 100%;
}

.sapMCrslControlsOverContentTop,
.sapMCrslControlsOverContentBottom {

	.sapMCrslBulleted,
	.sapMCrslNumeric {
		background-color: var(--sapPageFooter_Background);
		height: 1rem;
		border-radius: var(--sapElement_BorderCornerRadius);
		border: 1px solid var(--sapPageFooter_BorderColor);
		margin: 0 0.5rem;
	}
}

.sapMCrslControlsOverContentTop,
.sapMCrslControlsOverContentBottom {

	&.sapMCrslControlsNoArrows .sapMCrslBulleted,
	&.sapMCrslControlsNoArrows .sapMCrslNumeric {
		width: 9rem;
	}
}

.sapMCrsl .sapMCrslControlsOverContentTop,
.sapMCrsl .sapMCrslControlsOverContentBottom {
	background-color: transparent; /* the background should always be transparent since it is over the carousel's content */
}

.sapMCrslControlsNoArrowsTop {
	position: absolute;
	top: 0;
	width: 100%;
	z-index: 1;
	align-self: center;
}

.sapMCrslControlsNoArrowsBottom {
	position: absolute;
	bottom: 0;
	width: 100%;
}

.sapMCrslControlsBottom.sapMCrslControlsContainerBottom {
	width: 100%;
	align-self: center;
}

.sapMCrslControlsTop.sapMCrslControlsContainerTop {
	width: 100%;
	align-self: center;
}

.sapMCrslControls .sapMCrslPrev,
.sapMCrslControls .sapMCrslNext {
	display: inline-block;
	text-decoration: none;
	text-align: center;
	outline: none;
	position: relative;
	top: 3px;
}

.sapMCrslArrowInner {
	display: table;
	width: 100%;
	height: 100%;
	pointer-events: none;
}

.sapMCrslArrowInner .sapUiIcon {
	display: table-cell;
	text-align: center;
	vertical-align: middle;
	cursor: pointer;
	pointer-events: none;
}

.sapMCrslPrev .sapMCrslArrowInner .sapUiIcon {
	text-indent: -0.0625rem;
	font-size: 1rem;
}

.sapMCrslNext .sapMCrslArrowInner .sapUiIcon {
	text-indent: 0.0625rem;
	font-size: 1rem;
}

.sapMCrslBulleted,
.sapMCrslNumeric {
	padding-left: @_sap_m_Carousel_PageIndicatorContainerPadding;
	padding-right: @_sap_m_Carousel_PageIndicatorContainerPadding;
	height: 2rem;
	bottom: 1.0625rem;
	max-width: calc(~'100% - 6.5rem');
	display: flex;
	justify-content: center;
	align-self: center;

	&.sapMCrslPageIndicatorHidden {
		opacity: 0;
		width: 0;
		margin: 0;
		border: 0;
	}
}

.sapMCrslControlsNoArrows .sapMCrslBulleted,
.sapMCrslControlsNoArrows .sapMCrslNumeric {
	width: 100%;
	position: relative;
	bottom: 0;
	align-self: center;
}

.sapMCrslControlsOverContentTop {

	&.sapMCrslControlsNoArrows .sapMCrslBulleted,
	&.sapMCrslControlsNoArrows .sapMCrslNumeric {
		bottom: 2px;
	}
}

.sapMCrslControlsOverContentBottom {

	&.sapMCrslControlsNoArrows .sapMCrslBulleted,
	&.sapMCrslControlsNoArrows .sapMCrslNumeric {
		bottom: -2px;
	}
}

/* Styles for carousel dots on different screens */
/* screen width < 600px */
@media screen and (max-width: 599px) {

	 .sapMCrslBulleted,
	 .sapMCrslNumeric {
		width: @_sap_m_Carousel_PageIndicatorWidthSizeS;
	 }
}

/* 600px <= screen width < 1024px */
@media screen and (min-width : 600px) and (max-width: 1023px) {

	 .sapMCrslBulleted,
	 .sapMCrslNumeric {
		width: @_sap_m_Carousel_PageIndicatorWidthSizeM;
	 }
}

/* 1024px <= screen width < 1440px */
@media screen and (min-width : 1024px) and (max-width: 1439px) {

	 .sapMCrslBulleted,
	 .sapMCrslNumeric {
		width: @_sap_m_Carousel_PageIndicatorWidthSizeL;
	 }
}

/* screen width >= 1440px */
@media screen and (min-width: 1440px) {

	 .sapMCrslBulleted,
	 .sapMCrslNumeric {
		width: @_sap_m_Carousel_PageIndicatorWidthSizeXL;
	 }
}

/* Styles needed for keyboard handling
   ========================================================================== */
.sapMCrslItem {
	visibility: hidden;
	overflow: hidden;
	transition: visibility 0.5s;
}

@media all and (max-width: 1024px) {

	.sapMCrslItem {
		transition: visibility 0.3s;
	}
}

html[data-sap-ui-animation='off'] .sapMCrslItem {
	transition: visibility 0s;
}

.sapMCrslActive {
	visibility: visible;
	transition: visibility 0s;
}

/* Page Indicator Numeric
   ====================================================================== */
.sapMCrslNumeric span {
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	font-weight: normal;
	color: @_sap_m_Carousel_NumericIndicatorTextColor;
	white-space: nowrap;
	overflow: hidden;
	text-align: left;
	text-overflow: ellipsis;
	align-self: center;
}

/* Carousel controls: Bulleted */
.sapMCrslBulleted > span:hover,
.sapMCrslBulleted > span:focus {
	cursor: default;
	outline: none;
}

.sapMCrslBulleted > span.sapMCrslActive {
	background-color: @_sap_m_Carousel_BulletActiveBackground;
	border: 1px solid @_sap_m_Carousel_BulletActiveBorder;
	width: 0.5rem;
	height: 0.5rem;
	margin: 0 0.25rem;
}

/* Carousel controls: HUD */
.sapMCrslHud {
	padding-top: 0;
}

.sapMCrslItemTableCell.sapMCrslImg img {
	background: var(--sapBackgroundColor);
}

/* Arrows over content */
.sapMCrsl .sapMCrslHud.sapMCrslLeftmost .sapMCrslArrow.sapMCrslPrev,
.sapMCrsl .sapMCrslHud.sapMCrslRightmost .sapMCrslArrow.sapMCrslNext {
	opacity: 0;
	display: none;
}

/* Arrows in the Page Indicator area */
.sapMCrsl .sapMCrslArrow.sapMCrslLeftmost.sapMCrslPrev,
.sapMCrsl .sapMCrslArrow.sapMCrslRightmost.sapMCrslNext {
	opacity: 0;
}

.sapMCrslHud .sapMCrslPrev {
	left: @_sap_m_Carousel_PrevAndNextPosition;
}

.sapMCrslHud .sapMCrslNext {
	right: @_sap_m_Carousel_PrevAndNextPosition;
}

/* extra bits */
.sapMCrslControlsBackground-Solid {
	background: var(--sapPageFooter_Background);
}

.sapMCrslControlsBackground-Translucent {
	background: @_sap_m_Carousel_BackgroundColorTranslucent;
}

.sapMCrslControlsBackground-Transparent {
	background: @_sap_m_Carousel_BackgroundColorTransparent;
}

.sapMCrslControlsBorder-Solid.sapMCrslControlsTop {
	border-bottom: 1px solid var(--sapPageFooter_BorderColor);
}

.sapMCrslControlsBorder-Solid.sapMCrslControlsBottom {
	border-top: 1px solid var(--sapPageFooter_BorderColor);
}

/* Default Themes/Sytles for Slider */
.sapMCrslFluid .sapMCrslItem {
	margin-right: 1.25rem;
	width: 100%;
	height: 100%;
	overflow: hidden;
}

.sapMCrsl:hover,
.sapMCrslShowArrows {

	.sapMCrslHud .sapMCrslArrow {
		opacity: 1;
		display: block;
	}
}

html.sap-desktop {

	.sapMCrslHud {

		.sapMCrslPrev:focus,
		.sapMCrslNext:focus {

			&::before,
			&::after {
				content: '';
				position: absolute;
				top: @_sap_m_Carousel_FocusBorderOffsetArrow;
				right: @_sap_m_Carousel_FocusBorderOffsetArrow;
				bottom: @_sap_m_Carousel_FocusBorderOffsetArrow;
				left: @_sap_m_Carousel_FocusBorderOffsetArrow;
				pointer-events: none;
				border-radius: @_sap_m_Carousel_PrevAndNextFocusBorderRadius;
			}

			&::before {
				border: @_sap_m_Carousel_ContrastFocusBorder;
			}

			&::after {
				border: @_sap_m_Carousel_FocusBorder;
			}
		}
	}
}

/* fix for Safari */
.sapMCrslPrev:focus,
.sapMCrslNext:focus,
.sapMCrslPrev .sapUiIcon:focus,
.sapMCrslNext .sapUiIcon:focus {
	outline: none;
}

.sapMCrslDummyArea {
	position: fixed;
}
/* END "Carousel.less" */

/* START "CheckBox.less" */
/* =============================== */
/* CSS for control sap.m/CheckBox  */
/* Base theme                      */
/* =============================== */

.sapMCb {
	display: inline-block;
	position: relative;
	white-space: normal;
	box-sizing: border-box;
	height: 3rem;
	line-height: 3rem;
	vertical-align: top;
	text-align: left;
	padding: 0 0 0 3rem;

	&:not(.sapMCbBgDis):not(.sapMCbDisplayOnly) {

		&:hover,
		.sapMCbLabel:hover {
			cursor: pointer;
		}

		&.sapMCbHasLabel:focus::before {
			pointer-events: none;
		}
	}

	&.sapMCbDisplayOnly {

		&.sapMCbErr,
		&.sapMCbInfo,
		&.sapMCbWarn,
		&.sapMCbSucc {

			.sapMCbBg {
				border: none;

				&.sapMCbMarkChecked::before {
					color: var(--sapTextColor);
				}

				&.sapMCbMarkPartiallyChecked {
					border: solid 0.0625rem var(--sapField_BorderColor); /* was stated in #89919a */
				}
			}
		}
	}
}

.sapUiForm:not(.sapUiFormEdit-CTX) .sapMCb.sapMCbDisplayOnly {
	height: 1.125rem;
	line-height: 1.125rem;

	&:not(.sapMCbBgDis) {
		padding-left: 1.625rem;
	}

	.sapMCbBg {
		top: 0;
		margin: 0;
	}

	&.sapMCbWrapped.sapMCbHasLabel > .sapMLabel.sapMLabelWrapped {
		margin: 0;
		line-height: 1.125rem;
	}
}

.sapMCb.sapMCbWrapped.sapMCbHasLabel {
	height: auto;

	> .sapMLabel.sapMLabelWrapped {
		margin: 0.875rem 0;
		line-height: 1.4rem;
	}
}

.sapMCb.sapMCbDisplayOnly:not(.sapMCbBgDis) {
	padding-left: 2.375rem;
}

.sapUiFormEdit-CTX .sapMCb.sapMCbWrapped.sapMCbHasLabel > .sapMLabel.sapMLabelWrapped {
	margin: 0.75rem 0;
}

html[dir="rtl"] .sapMCb {
	text-align: right;
}

.sapMCb .sapMLabel {
	cursor: default;
	line-height: inherit;
	display: block;

	&.sapMLabelNoText {
		position: absolute;
		visibility: hidden;
		left: -10000px;
		top: -10000px;
	}
}

.sapMCb > label {
	vertical-align: top;
	overflow: hidden;
	text-overflow:ellipsis;
	white-space: nowrap;
	width : 100% /* If you put a CB in a form the with has to be 100% in rtl for text align */
}

.sapMCbBg { /* check box itself */
	background-color: var(--sapField_Background);
	border: solid 0.125rem var(--sapField_BorderColor);
	display: inline-block;
	position: absolute;
	left: 0.8125rem;
	top: 0.8125rem;
	height: 1.125rem;
	width: 1.125rem;
	line-height: 1.125rem;
	padding: 0;
	text-align: center;
	font-size: 1rem;

	/* note: some browsers like Samsung Galaxy S4 with Android 4.2 does not support the "border-radius" shorthand property
	but does support the long-hand properties for each corner like "border-top-left-radius" */
	border-top-left-radius: 0.125rem;
	border-top-right-radius: 0.125rem;
	border-bottom-right-radius: 0.125rem;
	border-bottom-left-radius: 0.125rem;
	-webkit-focus-ring-color: none;
}

.sapMCbBg > .sapMCbMark,
.sapMCbBg.sapMCbMark > input {
	font-size: inherit;
}

.sapMCbBg.sapMCbMarkChecked::before {
	content: '\e05b';
	font-family: var(--sapContent_IconFontFamily);
	display: inline-block;
}

.sapMCbBg.sapMCbMarkChecked.sapMCbMarkPartiallyChecked::before {
	content: '';
	width: 100%;
	height: 100%;
	background-color: currentColor;
	border: 0.1875rem solid transparent;
	box-sizing: border-box;
	background-clip: padding-box;
}

.sapUiSizeCompact .sapMCbBg.sapMCbMarkChecked.sapMCbMarkPartiallyChecked::before {
	border-width: 0.125rem;
}

.sapMCbBg > input {
	left: 0;
	-webkit-appearance: none;
	position: absolute;
	visibility: hidden;
}

html[data-sap-ui-browser^=ff] .sapMCbBg > input {
	/* in FF the input needs to be positioned exactly behind the visible checkbox for the focus outline */
	position: absolute;
}

/* Read only */
.sapMCbRo .sapMCbBg {
	background-color: var(--sapField_ReadOnly_Background);
	border-color: var(--sapField_ReadOnly_BorderColor);
}

.sapMCbRo > .sapMCbBg.sapMCbMarkChecked::before {
	color: var(--sapContent_NonInteractiveIconColor);
}

/* Display only */
.sapMCbDisplayOnly {
	font-family: var(--sapContent_IconFontFamily);

	.sapMCbBg.sapMCbMark.sapMCbMarkChecked.sapMCbMarkPartiallyChecked {
		border-radius: 0.0625rem;
		border: 1px solid var(--sapTextColor);

		&::before {
			background-color: var(--sapTextColor);
		}
	}

	.sapMCbBg:not(.sapMCbMarkPartiallyChecked) {
		background-color: initial;
		border: none;
		margin: 0.125rem;

		&::before {
			content: '\e19a';
			font-size: 1rem;
		}

		&.sapMCbMarkChecked::before {
			color: var(--sapTextColor);
			content: '\e03f';
		}
	}
}

/* Compact size */
.sapUiSizeCompact {

	.sapMCb {
		height: 2rem;
		line-height: 2rem;
		padding-left: 2rem;
	}

	.sapMCb.sapMCbWrapped.sapMCbHasLabel {
		height: auto;

		> .sapMLabel.sapMLabelWrapped {
			margin: 0.3rem 0;
		}
	}

	.sapUiFormEdit-CTX .sapMCb.sapMCbWrapped.sapMCbHasLabel > .sapMLabel.sapMLabelWrapped {
		margin: 0.25rem 0;
	}

	.sapMCbBg { /* check box itself */
		left: 0.5rem;
		top: 0.5rem;
		height: 0.75rem;
		width: 0.75rem;
		line-height: 0.75rem;
		font-size: 0.75rem;
		border-radius: 0;
	}

	.sapMLIBSelectM > .sapMCb {
		width: 2rem;
	}

	.sapUiForm:not(.sapUiFormEdit-CTX) .sapMCb.sapMCbDisplayOnly .sapMCbBg {
		top: 0.1875rem;
	}

	.sapMCb.sapMCbDisplayOnly:not(.sapMCbBgDis) {
		padding-left: 2rem;
	}
}

/* Condensed size */
.sapUiSizeCondensed.sapUiTable,
.sapUiSizeCondensed .sapUiTable {

	.sapMCb {
		height: 1.5rem;
		line-height: 1.5rem;
		padding-left: 1.5rem;
	}

	.sapUiTableDataCell > .sapUiTableCellInner > .sapMCb {
		margin-left: 0.25rem;
		padding-left: 1.75rem;
	}

	.sapMCbBg { /* check box itself */
		left: 0.25rem;
		top: 0.25rem;
		height: 0.75rem;
		width: 0.75rem;
		line-height: 0.75rem;
		font-size: 0.625rem;
		border-radius: 0;
	}
}

html[data-sap-ui-browser^="sf"] .sapMCbBg > .sapMCbMark, .sapMCbBg.sapMCbMark > input {
	margin: 0;
}
/* END "CheckBox.less" */

/* START "ColumnListItem.less" */
/* ===================================== */
/* CSS for control sap.m/ColumnListItem  */
/* Base theme                            */
/* ===================================== */

@_sap_m_table_SelectionControlWidth: 3rem;

.sapMListTblRow.sapMListTblRowTop > td {
	vertical-align: top;
}

.sapMListTblRow.sapMListTblRowMiddle > td {
	vertical-align: middle;
}

.sapMListTblRow.sapMListTblRowBottom > td {
	vertical-align: bottom;
}

.sapMListBGTransparent .sapMListTblSubRow {
	background: fade(@sapUiListBackground, 0%);
}

.sapMListTblSubRow {
	background: var(--sapList_Background);
}

/* adding alternating rows background color */
:nth-child(even of .sapMListTblRowAlternate),
:nth-child(even of .sapMListTblRowAlternate) + .sapMListTblSubRow {
	background-color: var(--sapList_AlternatingBackground);

	&.sapMLIBHoverable:hover {
		background: var(--sapList_Hover_Background);
	}

	&.sapMLIBSelected {
		background: var(--sapList_SelectionBackgroundColor);

		&.sapMLIBHoverable:hover {
			background: var(--sapList_Hover_SelectionBackground);
		}
	}
}

.sapMLIBHoverable:not(.sapMLIBSelected):hover + .sapMListTblSubRow,
.sapMLIBHoverable:not(.sapMLIBSelected) + .sapMListTblSubRow:hover,
.sapMLIBHoverable:not(.sapMLIBSelected).sapMPopinHovered {
	background: var(--sapList_Hover_Background);
}

.sapMLIBHoverable + .sapMListTblSubRow:hover {
	cursor: pointer;
}

.sapMLIBSelected + .sapMListTblSubRow,
.sapMLIBSelected.sapMListTblRowAlternate + .sapMListTblSubRow {
	background: var(--sapList_SelectionBackgroundColor);
}

.sapMLIB.sapMLIBActive + .sapMListTblSubRow {
	color: var(--sapList_Active_TextColor);
	background: var(--sapList_Active_Background);
}

.sapMLIBHoverable.sapMLIBSelected:hover + .sapMListTblSubRow,
.sapMLIBHoverable.sapMLIBSelected + .sapMListTblSubRow:hover,
.sapMLIBHoverable.sapMLIBSelected.sapMPopinHovered {
	background: @sapUiListSelectionHoverBackground;
}

.sapMLIBHoverable.sapMLIBSelected.sapMLIBActive:hover + .sapMListTblSubRow {
	background: var(--sapList_Active_Background);
}

/* table popin */
.sapMListTblSubCnt {
	padding: .5rem 1rem;
	box-sizing: border-box;
}

/* selectionControl width + cell padding */
.sapMListModeMultiSelect .sapMListTblSubCnt,
.sapMListModeSingleSelectLeft .sapMListTblSubCnt {
	padding-left: calc(@_sap_m_table_SelectionControlWidth + 0.5rem);
}

.sapMListTblSubCntRow {
	padding-bottom: 0;
	padding-top: 1px;
}

.sapMListTblSubCntRow + .sapMListTblSubCntRow {
	margin-top: .5rem;
}

.sapMListTblSubCntHdr {
	display: inline-flex;
}

.sapMListTblSubCntValBlock {
	margin-top: .25rem;
	display : block;
}

.sapMListTblSubCntValInline {
	display: inline-block;
	max-width: 100%;
}

.sapMListTblSubCntHdr > .sapMListTblSubCntSpr,
.sapMListTblSubCntRow > .sapMListTblSubCntHdr .sapMText,
.sapMListTblSubCntRow > .sapMListTblSubCntHdr .sapMLabel {
	color: var(--sapContent_LabelColor);
	padding-top: 0;
}

.sapMListTblSubCntRow > .sapMListTblSubCntVal .sapMText,
.sapMListTblSubCntRow > .sapMListTblSubCntVal .sapMLabel {
	color: var(--sapList_TextColor);
	padding-top: 0;
}

.sapMListTblSubCntSpr {
	font-size: @sapMFontMediumSize;
	padding-right: .5rem;

	&::before {
		content: attr(data-popin-colon);
		vertical-align: top;
	}
}

.sapMListTblSubCntRowInline {
	display: flex;
	align-items: center;
	flex-wrap: wrap;
}

 /* separators */
 .sapMLIBShowSeparator > td {
	border-top: 1px solid var(--sapList_BorderColor);
}

.sapMListShowSeparatorsNone .sapMLIBShowSeparator > td:not(.sapMListTblDummyCell),
.sapMListShowSeparatorsInner .sapMLIBShowSeparator:first-child > td:not(.sapMListTblDummyCell) {
	border-color: transparent;
}

.sapMListShowSeparatorsAll .sapMListTblSubRow:last-child > td:not(.sapMListTblDummyCell),
.sapMListShowSeparatorsAll .sapMLIBShowSeparator:last-child > td:not(.sapMListTblDummyCell) {
	border-bottom: 1px solid var(--sapList_BorderColor);
}

.sapMListShowSeparatorsAll .sapMLIBShowSeparator:last-child > td:not(.sapMListTblDummyCell),
.sapMListShowSeparatorsAll .sapMListTblSubRow:last-child > td:not(.sapMListTblDummyCell) {
	border-bottom: 1px solid var(--sapList_TableFooterBorder);
}

.sapMListShowSeparatorsInner .sapMLIBShowSeparator:last-child > td,
.sapMListShowSeparatorsInner .sapMListTblSubRow:last-child > td {
	border-bottom-color: transparent;
}

.sapMListShowSeparatorsNone .sapMListTblFooter > td {
	border-top-width: 2px;
}

.sapMTableRowCustomFocus {

	.sapMLIBIconDet,
	.sapMLIBIconDel {
		margin-left: 0;
	}
}

.sapMListTblHasPopin > .sapMTableRowCustomFocus:focus .sapMLIBHighlight,
.sapMListTblHasPopin > .sapMTableRowCustomFocus:focus .sapMLIBNavigated {
	bottom: 0;
}

.sapMListTblHasPopin > .sapMTableRowCustomFocus:focus + .sapMListTblSubRow .sapMLIBHighlight,
.sapMListTblHasPopin > .sapMTableRowCustomFocus:focus + .sapMListTblSubRow .sapMLIBNavigated {
	top: 0;
}

/********** COMPACT DESIGN ************/

/* decrease vertical popin paddings */
.sapUiSizeCompact .sapMListTblSubCnt {
	padding-top: .25rem;
	padding-bottom: .25rem;
}

/* popin padding */
.sapUiSizeCompact .sapMListModeMultiSelect .sapMListTblSubCnt,
.sapUiSizeCompact .sapMListModeSingleSelectLeft .sapMListTblSubCnt {
	padding-left: 2.5rem;
}
/* END "ColumnListItem.less" */

/* START "ColumnMenu.less" */
/* ======================================= */
/* CSS for control sap.m/table/ColumnMenu  */
/* Base theme                              */
/* ======================================= */

.sapMTCMenu {
	display: flex;
	flex-direction: column;
	height: 100%;
	min-width: 10rem;
	max-width: 31rem;

	.sapMTCMenuContainerWrapper {
		height: 100%;
		overflow: hidden;

		.sapUiRespGrid {
			width: auto;
		}
	}
}

.sapMTCMenu .sapMTCMenuQAListHidden {
	display: none;
}

.sapMTCMenuPopup.sapMPopover { /*Desktop, Tablet*/
	.sapMTCMenuQAForm {
		max-width: 25rem;
	}
}

.sapMTCMenuPopup.sapMDialog { /*Phone*/
	.sapMTCMenuQAForm,
	.sapMTCMenuContainerWrapper {
		width: 100%;
	}
}
/* END "ColumnMenu.less" */

/* START "ColorPalette.less" */
/* =================================== */
/* CSS for control sap.m/ColorPalette  */
/* Base theme                          */
/* ====================================*/
@_sap_m_ColorPalette_SwatchesPerRow: 5;

@_sap_m_ColorPalette_SwatchContainerPaddingLeftRightCozy: 0.6875rem;
@_sap_m_ColorPalette_SwatchContainerPaddingTopBottomCozy: 0.3125rem;
@_sap_m_ColorPalette_SwatchWidthHitAreaCozy: 2.375rem;
@_sap_m_ColorPalette_SwatchWidthCozy: 1.75rem;

@_sap_m_ColorPalette_SwatchContainerPaddingLeftRightCompact: 0.8125rem;
@_sap_m_ColorPalette_SwatchContainerPaddingTopBottomCompact: 0.1875rem;
@_sap_m_ColorPalette_SwatchWidthHitAreaCompact: 1.625rem;
@_sap_m_ColorPalette_SwatchWidthCompact: 1.25rem;

@_sap_m_ColorPalette_SwatchContainerPaddingLeftRightPhone: 0.625rem;
@_sap_m_ColorPalette_SwatchContainerPaddingTopBottomPhone: 0.375rem;
@_sap_m_ColorPalette_SwatchMaxWidthPhone: 4.5rem;
@_sap_m_ColorPalette_SwatchMarginPhone: 0.375rem;

@_sap_m_ColorPalette_SwatchWidthPhone: 20%;
@_sap_m_ColorPalette_SwatchSelectedMarginPhone: 0.0625rem;

.sapMColorPalette {
	height: 100%;
	/* overstyling the button to more like a menu item */
	.sapMBtnText {
		text-align: left;
	}

	.sapMBtn {
		/* Remove paddings and border of the inner element, so we defined them at root level */
		.sapMBtnInner {
			border: none;
			position: static; /* used to show the left and right border of the outline when focused */
			background-color: inherit;

			&.sapMBtnText {
				padding: 0 0 0 1rem;
			}
		}
		/* Define paddings, margins and border at root level */
		margin: 0;
		padding: 0;
		border: none;

		/* Remove hover & active styling of the inner button element */
		> .sapMBtnActive,
		&:hover > .sapMBtnActive,
		&:active > .sapMBtnActive {
			color: inherit;
			background: inherit;
		}

		.sapMBtnTransparent {
			color: inherit;
		}

		&:hover > .sapMBtnTransparent.sapMBtnHoverable {
			background-color: inherit;
		}

		/* define active & hover on the root button level */
		&:not(active) {
			color: var(--sapButton_TextColor);
			text-shadow: none;
		}

		/* active */
		&:active:not(:hover) {
			background-color: var(--sapButton_Background);
			color: var(--sapButton_TextColor);
			text-shadow: none;
		}

		&:active > .sapMBtnInner {
			text-shadow: none;
		}

		&:active {
			background-color: var(--sapList_Active_Background);
			color: var(--sapButton_Active_TextColor);
		}

		/* hover */
		&:hover:not(:active) {
			background: var(--sapList_Hover_Background);
		}

		&:focus {

			> .sapMFocusable { /* The focus should go at the outside container */

				.sap-desktop & {
					outline: none;
					outline-offset: 0;

					.sapMBtnFocusDiv { /* Button has special focus handling for IE, so remove it as well */
						box-sizing: inherit;
						position: inherit;
						left: 0;
						top: 0;
						right: 0;
						bottom: 0;
						border-radius: 0;
						border: none;
						pointer-events: none;
					}
				}

				.sap-desktop .sapContrast &,
				.sap-desktop .sapContrastPlus & {
					outline: none;
					outline-offset: 0;
				}
			}

			.sap-desktop & {

				&::after {
					content: ' ';
					position: absolute;
					box-sizing: border-box;
					left: 2px;
					top: 2px;
					right: 2px;
					bottom: 2px;
					border-radius: 0;
					border: 0.0625rem dotted var(--sapContent_FocusColor);
					pointer-events: none;
				}
			}
		}
	}

	.sapMColorPaletteContent .sapMRecentColorSquareDisabled {
		pointer-events: none;
		opacity: 0.5;
	}

	.sapMColorPaletteSeparator {
		position: relative;
		height: 0.0625rem;

		/* overstyling the hr to look like menu separator line */
		> hr {
			display: inline-block;
			position: absolute;
			margin: 0;
			left: 0;
			right: 0;
			border-width: 0;
			border-top: 1px solid var(--sapToolbar_SeparatorColor);
			color: var(--sapToolbar_SeparatorColor);
		}
	}

	:focus {
		outline: none;
	}

	.sapMColorPaletteSquare:focus {
		outline: none
	}
}

html:not(.sap-phone) {

	/* Dimension and paddings of the Palette */
	.sapMColorPalette {
		max-width: @_sap_m_ColorPalette_SwatchesPerRow * @_sap_m_ColorPalette_SwatchWidthHitAreaCozy + 2 * @_sap_m_ColorPalette_SwatchContainerPaddingLeftRightCozy;
		min-width: @_sap_m_ColorPalette_SwatchesPerRow * @_sap_m_ColorPalette_SwatchWidthHitAreaCozy + 2 * @_sap_m_ColorPalette_SwatchContainerPaddingLeftRightCozy;

		.sapMColorPaletteContent {
			padding: @_sap_m_ColorPalette_SwatchContainerPaddingTopBottomCozy @_sap_m_ColorPalette_SwatchContainerPaddingLeftRightCozy;
		}
	}
	&.sapUiSizeCompact .sapMColorPalette,
	& .sapUiSizeCompact .sapMColorPalette {
		max-width: @_sap_m_ColorPalette_SwatchesPerRow * @_sap_m_ColorPalette_SwatchWidthHitAreaCompact + 2 * @_sap_m_ColorPalette_SwatchContainerPaddingLeftRightCompact;
		min-width: @_sap_m_ColorPalette_SwatchesPerRow * @_sap_m_ColorPalette_SwatchWidthHitAreaCompact + 2 * @_sap_m_ColorPalette_SwatchContainerPaddingLeftRightCompact;

		.sapMColorPaletteContent {
			padding: @_sap_m_ColorPalette_SwatchContainerPaddingTopBottomCompact @_sap_m_ColorPalette_SwatchContainerPaddingLeftRightCompact;
		}
	}

	.sapMColorPalette {

		.sapMBtnInner {
			border-radius: 0.25rem 0.25rem 0 0;
		}

		.sapMColorPaletteSquare {
			box-sizing: border-box;
			position: relative;
			border: 0.0625rem solid var(--sapContent_ForegroundBorderColor);
			border-radius: 0.125rem;
			width: @_sap_m_ColorPalette_SwatchWidthCozy;
			height: @_sap_m_ColorPalette_SwatchWidthCozy;
			display: inline-block;
			margin: (@_sap_m_ColorPalette_SwatchWidthHitAreaCozy - @_sap_m_ColorPalette_SwatchWidthCozy)/2;

			&:hover,
			&.sapMColorPaletteSquareSelected { /* now the hit area (margin) gets occupied by the content */
				margin: 0;
				width: @_sap_m_ColorPalette_SwatchWidthHitAreaCozy;
				height: @_sap_m_ColorPalette_SwatchWidthHitAreaCozy;

				> div {
					border: 1px solid var(--sapGroup_ContentBackground);
				}
			}

			&:focus::before {
				content: '';
				box-sizing: border-box;
				position: absolute;
				left: 1px;
				top: 1px;
				right: 1px;
				bottom: 1px;
				border: 1px solid var(--sapContent_FocusColor);
				pointer-events: none;
			}

			&:focus::after {
				content: '';
				position: absolute;
				box-sizing: border-box;
				left: 1px;
				top: 1px;
				right: 1px;
				bottom: 1px;
				border: 1px dotted var(--sapContent_ContrastFocusColor);
				pointer-events: none;
			}

			> div {
				box-sizing: border-box;
			}
		}
	}
	&.sapUiSizeCompact .sapMColorPalette,
	& .sapUiSizeCompact .sapMColorPalette {

		.sapMColorPaletteSquare {
			width: @_sap_m_ColorPalette_SwatchWidthCompact;
			height: @_sap_m_ColorPalette_SwatchWidthCompact;
			display: inline-block;
			margin: (@_sap_m_ColorPalette_SwatchWidthHitAreaCompact - @_sap_m_ColorPalette_SwatchWidthCompact)/2;

			&:hover,
			&.sapMColorPaletteSquareSelected {
				margin: 0;
				width: @_sap_m_ColorPalette_SwatchWidthHitAreaCompact;
				height: @_sap_m_ColorPalette_SwatchWidthHitAreaCompact;
			}
		}
	}

	.sapMColorPaletteContent {
		display: flex;
		flex-wrap: wrap;
	}

	.sapMColorPaletteSquare {

		& > div {
			height: 100%;
			width: 100%;
		}
	}
}

.sap-phone {

	.sapMColorPalette {

		&html:not(.sap-phone) .sapMBtnInner {
			border-radius: 0.25rem 0.25rem 0 0;
		}
	}

	.sapMColorPaletteContent {
		display: flex;
		flex-wrap: wrap;
		padding: @_sap_m_ColorPalette_SwatchContainerPaddingTopBottomPhone @_sap_m_ColorPalette_SwatchContainerPaddingLeftRightPhone;
		max-width: 22.5rem;
		margin: auto;
	}

	.sapMColorPaletteContainer .sapMDialogScroll {
		height: 100%;

		.sapMDialogScrollCont.sapMDialogStretchContent {
			padding: 0;
		}
	}

	.sapMColorPaletteSquare {
		width: @_sap_m_ColorPalette_SwatchWidthPhone;
		max-width: @_sap_m_ColorPalette_SwatchMaxWidthPhone;
		position: relative;
		aspect-ratio: 1 / 1;

		& > div {
			border: 1px solid var(--sapContent_ForegroundBorderColor);
			border-radius: 0.125rem;
			position: absolute;
			top: @_sap_m_ColorPalette_SwatchMarginPhone;
			bottom: @_sap_m_ColorPalette_SwatchMarginPhone;
			left: @_sap_m_ColorPalette_SwatchMarginPhone;
			right: @_sap_m_ColorPalette_SwatchMarginPhone;
			aspect-ratio: 1 / 1;
		}

		&:active > div {
			box-shadow: inset 0 0 0 1px var(--sapGroup_ContentBackground);
			top: -0.0625rem;
			bottom: -0.0625rem;
			left: -0.0625rem;
			right: -0.0625rem;
		}

		&.sapMColorPaletteSquareSelected {
			width: ~"calc(@{_sap_m_ColorPalette_SwatchWidthPhone} + 0.625rem)";
			aspect-ratio: 1 / 1;

			& > div {
				aspect-ratio: 1 / 1;
				box-shadow: inset 0 0 0 1px var(--sapGroup_ContentBackground);
				top: @_sap_m_ColorPalette_SwatchSelectedMarginPhone;
				bottom: @_sap_m_ColorPalette_SwatchSelectedMarginPhone;
				left: @_sap_m_ColorPalette_SwatchSelectedMarginPhone;
				right: @_sap_m_ColorPalette_SwatchSelectedMarginPhone;
			}
		}
	}
}

@media (orientation: landscape) {

	.sap-phone {

		.sapMColorPaletteContent {
			max-width: initial;
		}
	}
}

html.sap-desktop .sapMDialog.CPDialog {
	min-width: 18rem; /* override the  Dialog's min-width for desktop */

	.sapUiColorPicker-ColorPickerMatrix {

		&.sapUiRespGrid.sapUiRespGridHSpace1 {
			padding-bottom: 0.5rem;
		}

		/* in a dialog ColorPicker doesn't have padding-top in order to be as compact as possible */
		&.sapUnifiedColorPicker.sapUiRespGrid.sapUiRespGridHSpace1 > div:first-child {
			height: auto;
			padding-top: 0;
		}
	}
}
/* END "ColorPalette.less" */

/* START "CustomTile.less" */
/* ================================= */
/* CSS for control sap.m/CustomTile  */
/* Base theme                        */
/* ================================= */

/* Style for Tablet/Desktop */
.sapMCustomTile {
	display: inline-block;
	box-sizing: border-box;
	-webkit-transition-property: transform3d;
	-webkit-transition-duration: 0.2s;
	transition-property: transform3d;
	transition-duration: 0.2s;
	width: 13em;
	height: 15em;
	margin: 0em;
}

.sapMCustomTileContent {
	overflow: hidden;
	width: 100%;
	height: 100%;
}

html.sap-phone .sapMCustomTile {
	width: 9.5em;
	height: 11em;
	margin: 0em;
}
/* END "CustomTile.less" */

/* START "DateRangeSelection.less" */
/* ========================================= */
/* CSS for control sap.m/DateRangeSelection  */
/* Base theme                                */
/* ========================================= */
/* END "DateRangeSelection.less" */

/* START "DatePicker.less" */
/* ===================================== */
/* CSS for control sap.m/DatePicker  */
/* Base theme                            */
/* ===================================== */

[data-sap-ui-popup].sapMRPCalendar {
	&.sapMDialog.sapUiPopupWithPadding:not(.sapUiNoContentPadding) .sapMDialogScrollCont {
		padding: 0;
	}
	.sapMDialogScroll,
	.sapMDialogScrollCont,
	.sapUiCal {
		min-height: 100%;
	}
	&.sapMDialog {
		padding-bottom: 0;
	}
}

html.sap-phone, html.sapUiMedia-Std-Phone {
	[data-sap-ui-popup] {
		.sapUiCal {
			width: 100%;
		}

		&.sapMLandscapePadding .sapUiCal {
			padding-bottom: 3rem;
		}
	}
}

.sapMInputBase {
	&.sapMDatePickerHiddenInput {
		display: none;
	}
	.sapMInputBaseContentWrapper{
		display: inline-flex;
	}
}
/* END "DatePicker.less" */

/* START "DateTimePicker.less" */
/* ===================================== */
/* CSS for control sap.m/DateTimePicker  */
/* Base theme                            */
/* ===================================== */

/* styles for popup */
.sapMDateTimePopupCont {
	display: flex;
	height: 25rem;
	min-width: 40.0625rem;
}

.sapMDateTimePopupCont > .sapUiCal,
.sapMDateTimePopupCont > .sapMTimePickerContainer {
	float: left;
	padding-top: 0.5rem;
}

.sapMDateTimePopupCont > .sapMTimePickerContainer {
	width: auto;
}

.sapMTimePickerSwitch {
	margin-top: 1rem;
	text-align: center;
}

.sapMTimePickerSep {
	height: ~'calc(100% - 2rem)';
	width: 0.0625rem;
	margin-top: 1rem;
	margin-bottom: 1rem;
	background-color: var(--sapGroup_ContentBorderColor);
	box-sizing: border-box;
	float: left;
}

.sapMDTP.sapMDTPWithTimezone {
	min-width: 6.875rem;
}

.sapMDTP.sapMDTPWithTimezone .sapMInputBaseContentWrapper {
	display: flex;
	justify-content: flex-start;
	position: relative;

	& > * {
		flex-basis: auto;
		flex-grow: 0;
		flex-shrink: 0;
	}

	& > input {
		padding-right: 0;
		width: ~'calc(100% - 4rem)';
		min-width: 2rem;
	}

	& > .sapMDTPTimezoneLabel {
		flex-grow: 1;
		flex-shrink: 1;
		min-width: 0;
	}
}

.sapMDTP.sapMDTPWithTimezone .sapMInputBaseContentWrapper .sapMDummyContent {
	visibility: hidden;
	position: absolute;
	left: 0;
	width: auto;
	padding-left: 0.625rem;
	font-size: 0.875rem;
}

.sapMDTP.sapMDTPWithTimezone .sapMInputBaseContentWrapper .sapMDTPTimezoneLabel {
	background: transparent;
	box-sizing: border-box;
	border: none;
	font-size: 0.875rem;
	font-style: italic;
	color: var(--sapField_PlaceholderTextColor);
	padding: 0 0.25rem 0 0.25rem;
	height: 100%;
	line-height: normal;
	vertical-align: inherit;
	display: flex;
	justify-content: start;
	align-items: center;
	cursor: pointer;

	& > span {
		white-space: nowrap;
		overflow: hidden;
		text-overflow: ellipsis;
		padding-right: 0.125rem;
	}
}

/* Compact mode */
.sapUiSizeCompact .sapMDateTimePopupCont {
	height: 21rem;
	min-width: 34.0625rem;
}

/* Phone mode use device and media selector to support small desktop and large phones*/
.sapMDateTimePopup.sapUiPopupWithPadding > .sapMDialogSection > .sapMDialogScroll > .sapMDialogScrollCont {
	padding: 0;
}

.sapMDateTimePopup.sapMDialogPhone > .sapMDialogSection > .sapMDialogScroll > .sapMDialogScrollCont {
	height: 100%;
}

html.sap-phone .sapMDateTimePopupCont,
html.sapUiMedia-Std-Phone .sapMDateTimePopupCont {
	height: 100%;
	min-width: 19.1rem;
	overflow: auto !important; /* allow scrolling in landscape mode */
	display: block;
}

html.sap-phone .sapMDialogScroll,
html.sap-phone .sapMDialog.sapUiPopupWithPadding:not(.sapUiNoContentPadding) .sapMDialogScrollCont.sapMDialogStretchContent {
	height: 100%;
}

html.sap-phone .sapUiSizeCompact .sapMDateTimePopupCont,
html.sapUiMedia-Std-Phone .sapUiSizeCompact .sapMDateTimePopupCont {
	min-width: 17rem;
}

html.sap-phone .sapMTimePickerSep,
html.sapUiMedia-Std-Phone .sapMTimePickerSep {
	display: none;
}

html.sap-phone .sapMDateTimePopupCont > .sapUiCal,
html.sapUiMedia-Std-Phone .sapMDateTimePopupCont > .sapUiCal {
	width: 100%;
}

html.sapUiMedia-Std-Phone .sapMDateTimePopupCont > div[class=sapUiCal] {
	height: 25.25rem;
	min-width: 19.1rem;
}

html.sapUiMedia-Std-Phone .sapMDateTimePopupCont.sapMTimePickerDropDown .sapMTimePickerContainer {
	height: 25.25rem;
	width: 100%;
}

html.sapUiMedia-Std-Phone.sapUiSizeCompact .sapMDateTimePopupCont.sapMTimePickerDropDown .sapMTimePickerContainer,
html.sapUiMedia-Std-Phone .sapUiSizeCompact .sapMDateTimePopupCont.sapMTimePickerDropDown .sapMTimePickerContainer {
	height: 15.25rem;
	width: 100%;
}

html.sap-phone .sapMDateTimePopupCont.sapMTimePickerDropDown .sapMTimePickerContainer {
	height: ~'calc(100% - 7rem)';
}

html.sap-phone .sapMDateTimePopupCont {
	background-color: var(--sapList_Background);

	.sapUiCalContent {

		& > .sapUiCalMonthPicker,
		& > .sapUiCalYearPicker {
			width: 100%;
			box-sizing: border-box;
			left: 0;
			margin: 0;
		}
	}
}

.sapUiSizeCompact .sapMTimePickerDropDown .sapMTimePickerContainer {
	min-width: 17rem;
}

html.sapUiMedia-Std-Tablet,
html.sapUiMedia-StdExt-Tablet {

	.sapMDateTimePopupCont > .sapUiCal:not(.sapUiCalInt) {
		width: 47%;
		min-width: 47%;
	}

	.sapMTimePickerContainer {
		min-width: 53%;
	}
}

html:not(.sapUiMedia-Std-Phone),
html:not(.sapUiMedia-StdExt-Phone) {

	.sapMDateTimePopup .sapMPopoverCont {
		overflow: hidden auto !important; /* disable horizontal scrolling when not phone size */
	}

	.sapMDateTimePopupCont {

		.sapMTPClocksContainer {
			padding: 0.5rem;
		}
	}
}

/* DateTimePicker in DynamicDateRange */

.sapMDynamicDateRangePopover {

	.sapUiCal:not(.sapUiCalInt) > .sapUiCalHead {
		max-width: 100%;
	}

	.sapMDateTimePopupCont {
		min-width: 20rem;
		overflow: hidden auto !important;
		display: block;
		height: 100%;
		padding: 0;

		.sapUiCal .sapUiCalMonthView,
		.sapUiCal .sapUiCalRow {
			padding-bottom: 0;
		}
	}

	.sapUiSizeCompact .sapMDateTimePopupCont {
		min-width: 17rem;
	}

	.sapMTimePickerSep {
		display: none;
	}

	.sapMDateTimePopupCont > .sapUiCal {
		width: 100%;
		height: auto;
	}

	.sapMDateTimePopupCont.sapMTimePickerDropDown .sapMTimePickerContainer {
		width: 100%;
		height: 22rem;
	}

	.sapUiSizeCompact .sapMDateTimePopupCont.sapMTimePickerDropDown .sapMTimePickerContainer {
		height: 15.25rem;
		width: 100%;
	}

	.sapMDateTimePopupCont {
		background-color: var(--sapList_Background);

		.sapUiCalContent {

			& > .sapUiCalMonthPicker,
			& > .sapUiCalYearPicker {
				width: 100%;
				box-sizing: border-box;
				left: 0;
				margin: 0;
			}
		}

		.sapMTPClocksContainer {
			width: 100%;
			height: auto;

			.sapMTPCButtons,
			.sapMTPCClocks {
				margin: 0 auto;
			}
		}
	}
}

html.sapUiMedia-Std-Tablet .sapMDynamicDateRangePopover .sapMDateTimePopupCont > .sapUiCal:not(.sapUiCalInt),
html.sapUiMedia-StdExt-Tablet .sapMDynamicDateRangePopover .sapMDateTimePopupCont > .sapUiCal:not(.sapUiCalInt) {
	width: 100%;
}

.sapUiSizeCompact .sapMDDRPopover .sapMPageWithFooter > section,
.sapMDDRPopover .sapMPageWithFooter > section {
	bottom: 2rem;
}

.sapUiSizeCompact .sapMDynamicDateRangePopover .sapUiCal:not(.sapUiCalInt) {
	width: 100%;
}

html.sap-desktop {

	.sapMDateTimePopup {

		.sapMValueStateHeaderRoot {
			width: 100% !important;
		}
	}
}
/* END "DateTimePicker.less" */

/* START "Dialog.less" */
/* ============================= */
/* CSS for control sap.m/Dialog  */
/* Base theme                    */
/* ============================= */

@_sap_m_Dialog_VerticalMargin: 3.5%;
@_sap_m_Dialog_BarHeight: 3rem;
@_sap_m_Dialog_BarCompactHeight: 2.5rem;
@_sap_m_Dialog_SubHeaderHeight: 3rem;
@_sap_m_Dialog_SubHeaderCompactHeight: 2rem;
@_sap_m_Dialog_InfoDesignBarHeight: 2rem;

@_sap_m_Dialog_Padding: 1rem;
@_sap_m_Dialog_NoStateHeaderShadow: inset 0 -0.0625rem var(--sapPageHeader_BorderColor);
@_sap_m_Dialog_SuccessHeaderShadow: inset 0 -0.125rem var(--sapSuccessBorderColor);
@_sap_m_Dialog_ErrorHeaderShadow: inset 0 -0.125rem var(--sapErrorBorderColor);
@_sap_m_Dialog_WarningHeaderShadow: inset 0 -0.125rem var(--sapWarningBorderColor);
@_sap_m_Dialog_InformationHeaderShadow: inset 0 -0.125rem var(--sapInformationBorderColor);
@_sap_m_Dialog_HeaderFocusBorderRadius: 0px;
@_sap_m_Dialog_HeaderFocusBorderBottomOffset: 3px;
@_sap_m_Dialog_HeaderFocusBorderTopOffset: 2px;
@_sap_m_Dialog_HeaderFocusBorderLeftOffset: 2px;
@_sap_m_Dialog_HeaderFocusBorderRightOffset: 2px;

.sapMDialog,
.sapMMessageDialog .sapMDialogScrollCont,
.sapMDialogSection {
	box-sizing: border-box;
}

.sapMDialog {
	/* scale(1) - Chrome does not clip children when transform is not applied resulting in non-visible border-radius */
	/* translateZ(0px) - Safari Problem with z-index - After dialog was opened it was inserted behind the dark overlay */
	-webkit-transform: scale(1) translateZ(0px);
	background-clip: padding-box;
	min-height: 6rem;
	outline: none;
	position: fixed !important;
	display: flex !important;
	flex-direction: column;
	opacity: 0;
	transition: opacity 0.15s;
	border-radius: var(--sapElement_BorderCornerRadius);
	box-shadow: var(--sapContent_Shadow3);

	&:not(.sapMDialogHighPixelDensity) {
		/* prevent blurring when the device pixel ration is 1 (not high density screens) */
		/* this is done with style and not with media query for wider support */
		filter: blur(0);
	}
}

.sapMDialog > .sapMDialogSection {
	max-width: 100%;
	background: var(--sapGroup_ContentBackground);
}

.sapMDialog .sapMDialogScrollCont > * {
	vertical-align: middle;
}

.sapMDialog.sapMDialogStretched {
	bottom: 0 !important;
	right: 0 !important;
	height: auto !important; /* this is for preventing inline size/width sizes set the width */
	width: auto !important; /* this is for preventing inline size/height sizes set the height */
	min-width: 0 !important; /* this is for preventing the dialog to hold it's width in small screens */
	min-height: 0 !important; /* this is for preventing the dialog to hold it's width in small screens */
}

/* Removing borders, shadow, and rounded corners when the dialog is on fullscreen */
html.sap-phone .sapMDialog.sapMDialogStretched {
	border: none;
	box-shadow: none;
	border-radius: 0;
}

html:not(.sap-phone) .sapMDialog {

	header .sapMHeader-CTX {
		border-radius: var(--sapElement_BorderCornerRadius) var(--sapElement_BorderCornerRadius) 0 0;
	}

	&.sapMDialog-NoHeader header .sapMSubHeader-CTX {
		border-radius: var(--sapElement_BorderCornerRadius) var(--sapElement_BorderCornerRadius) 0 0;
	}

	&.sapMDialog-NoHeader:not(.sapMDialogWithSubHeader) .sapMDialogSection {
		border-top-right-radius: var(--sapElement_BorderCornerRadius);
		border-top-left-radius: var(--sapElement_BorderCornerRadius);
	}

	footer .sapMFooter-CTX {
		border-radius: 0 0 var(--sapElement_BorderCornerRadius) var(--sapElement_BorderCornerRadius);
	}

	&.sapMDialog-NoFooter .sapMDialogSection {
		border-bottom-right-radius: var(--sapElement_BorderCornerRadius);
		border-bottom-left-radius: var(--sapElement_BorderCornerRadius);
	}
}

html.sap-phone .sapMDialog {
	min-width: 18rem;
	overflow: hidden;

	& > footer .sapMFooter-CTX {
		display: flex;
		padding: 0 0.25rem;
	}
}

html.sap-tablet .sapMDialog,
html.sap-desktop .sapMDialog {
	min-width: 20rem;
}

.sapMDialog .sapMIBar {
	position: relative;
	width: auto;
}

.sapMDialog .sapMDialogSection {
	overflow: hidden;
	position: relative;
}

.sapMDialogScroll {
	display: inline-block; /* this is for horizontal scrolling enablement */
	height: auto;
	width: 100%;
	vertical-align: middle;
	box-sizing: border-box;
}

.sapMDialog.sapMDialogVerticalScrollIncluded {

	.sapMDialogScroll {
		width: auto;
	}

	html[data-sap-ui-browser^='sf'] & .sapMDialogSection {
		/* For Safari - allow the vertical scroll bar to overlap the right padding which we use to compensate for its width.
		This happens by default for other browsers. */
		overflow-y: overlay !important;
		padding-right: 0px !important;
	}
}

/* disable scrolling automatically for sap.m.Page sap.m.NavContainer sap.m.ScrollingContainer inside Dialog */
.sapMDialogVerScrollDisabled .sapMDialogScroll {
	height: 100%;
}

/* disable scrolling automatically for sap.m.Page sap.m.NavContainer sap.m.ScrollingContainer inside Dialog */
.sapMDialogHorScrollDisabled .sapMDialogScroll {
	display: block;
	/* This div has position set to absolute when runs in android native browser with version less than 4.1. */
	/* Width needs to be set to 100% due to position absolute */
	width: 100%;
}

.sapMDialogHidden {
	/* display: none; */
	/* visibility need to be used instead of display none, because some control inside dialog may need to do calculation */
	visibility: hidden;
}

.sapMDialog.sapUiPopupWithPadding:not(.sapUiNoContentPadding) {

	.sapMDialogScrollCont {
		padding: @_sap_m_Dialog_Padding;

		&.sapMDialogStretchContent {
			height: calc(~"100% - 2 * " @_sap_m_Dialog_Padding);
		}
	}
}

.sapMDialogScrollCont.sapMDialogStretchContent {
	height: 100%;
}

/* messagedialog always has content padding */
.sapMMessageDialog .sapMDialogScrollCont {
	box-sizing: border-box;
	padding: 1rem;
}

.sapMActionSheetDialog .sapMDialogScrollCont {
	padding: 0; /* no content padding for action sheet */
}

.sapMDialogOpen {
	opacity: 1;
}

/* mixin call for creating container content padding classes (arguments: rootContainer, contentSelector) */
.sapUiContainerContentPadding(~".sapMDialog", ~"> section > .sapMDialogScroll > .sapMDialogScrollCont");

/* ====================================================== */

.sapMDialog.sapMMessageDialog {
	font-size: 0.875rem;
}

.sapMDialog > header .sapMDialogTitleGroup,
.sapMDialog > header .sapMDialogSubHeader {
	font-size: 1rem;
}

.sapMDialog:not(.sapMDialogWithSubHeader) .sapMDialogSubHeader {
	display: none;
}

.sapMDialogIcon ~ .sapMDialogTitle {
	display: inline;
}

/* remove the branding color of bar in page when page is in dialog*/
.sapMDialog .sapMIBar.sapMPageHeader::before {
	border-top: none;
}

.sapMMessageDialog.sapMPopup-CTX > header .sapMDialogTitleGroup {
	background-image: none;
	filter: none;
}

.sapMDialog > header .sapMDialogTitleGroup .sapMTitle {
	color: var(--sapPageHeader_TextColor);
	vertical-align: middle;
}

.sapMDialog > header .sapMDialogTitleGroup .sapMImg {
	height: 2.25rem;
	margin-right: 0.625rem;
	vertical-align: middle;
	width: 2.25rem;
}

.sapMDialog > header .sapMDialogTitleGroup .sapMDialogIcon.sapUiIcon {
	font-size: 1.25rem;
	vertical-align: middle;
}

.sapMTB.sapMTBNoBorders {
	border-bottom: none;
}

/* Compact size */
.sapUiSizeCompact {

	&.sapMDialog,
	.sapMDialog {
		min-height: @_sap_m_Dialog_BarCompactHeight;
		padding-bottom: @_sap_m_Dialog_BarCompactHeight;

		&.sapMDialogWithSubHeader {

			&.sapMDialog-NoHeader {

				& .sapMDialogSubHeader {
					top: 0;
				}
			}
		}

		.sapMDialogTitleGroup {
			min-height: @_sap_m_Dialog_BarCompactHeight;

			.sapMHeader-CTX,
			.sapMSubHeader-CTX {
				height: @_sap_m_Dialog_BarCompactHeight;
				padding-top: 0.25rem;
				padding-bottom: 0.25rem;
				box-sizing: border-box;
			}
		}

		& > .sapMDialogSection {
			min-height: @_sap_m_Dialog_BarCompactHeight;
		}

		& > footer {
			height: @_sap_m_Dialog_BarCompactHeight;

			.sapMFooter-CTX {
				height: @_sap_m_Dialog_BarCompactHeight;
			}
		}

		.sapMDialogSubHeader {
			height: @_sap_m_Dialog_SubHeaderCompactHeight;
			top: @_sap_m_Dialog_BarCompactHeight;
		}
	}
}

.sapMDialogDisableSelection {
	-webkit-user-select: none;
	user-select: none;
}

.sapMDialogResizeHandle {
	position: absolute;
	right: -0.5rem;
	bottom: -0.5rem;
	width: 1.5rem;
	height: 1.5rem;
	z-index: 2;
	line-height: 1rem;
	cursor: se-resize;
	border-radius: 50%;

	.sapMDialogResizeHandleIcon {
		position: relative;
		left: 0;
		top: 0;
		width: 1rem;
		height: 1rem;
		color: var(--sapButton_Lite_TextColor);
		cursor: inherit;
	}
}

.sapMDialog {
	box-sizing: content-box;
	min-height: @_sap_m_Dialog_BarHeight;
	padding-bottom: @_sap_m_Dialog_BarHeight;

	footer {

		.sapMTB.sapMFooter-CTX {
			border-bottom: none;
		}
	}

	&.sapMDialog-NoFooter {
		padding-bottom: 0;
	}

	&.sapMDialogWithSubHeader {

		&.sapMDialogSubHeaderInfoBar {

			.sapMDialogSubHeader {
				height: @_sap_m_Dialog_InfoDesignBarHeight;
			}
		}

		&.sapMDialog-NoHeader {

			.sapMDialogSubHeader {
				top: 0;
			}
		}
	}

	.sapMDialogTitleGroup {
		min-height: @_sap_m_Dialog_BarHeight;
		left: 0;
		position: absolute;
		right: 0;
		top: 0;
	}

	.sapMDialogTitleGroup:focus {
		outline: none;

		&::after {
			content: '';
			position: absolute;
			left: @_sap_m_Dialog_HeaderFocusBorderLeftOffset;
			bottom: @_sap_m_Dialog_HeaderFocusBorderBottomOffset;
			right: @_sap_m_Dialog_HeaderFocusBorderRightOffset;
			top: @_sap_m_Dialog_HeaderFocusBorderTopOffset;
			border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
			border-radius: @_sap_m_Dialog_HeaderFocusBorderRadius @_sap_m_Dialog_HeaderFocusBorderRadius 0 0;
			pointer-events: none;
		}
	}

	& > .sapMDialogSection {
		flex: 1 1 100%;
		min-height: @_sap_m_Dialog_BarHeight;
	}

	& > footer {
		bottom: 0;
		height: @_sap_m_Dialog_BarHeight;
		left: 0;
		position: absolute;
		right: 0;
	}

	&.sapMDialogDraggable > header {
		cursor: move;
	}

	.sapMDialogSubHeader {
		cursor: default;
		height: @_sap_m_Dialog_SubHeaderHeight;
		left: 0;
		position: absolute;
		right: 0;
		top: @_sap_m_Dialog_BarHeight;
		z-index: 2;
	}
}

/* remove transition when animation is off */
html[data-sap-ui-animation='off'] .sapMDialog {
	transition: none;
}

.sapMDialog.sapMDialogWithSubHeader .sapMDialogTitleGroup .sapMIBar {
	box-shadow: none;
}

.sapMDialog > header:last-of-type .sapMIBar {
	box-shadow: @_sap_m_Dialog_NoStateHeaderShadow;
}

.sapMDialogWarning > header {

	.sapMDialogTitleGroup .sapMDialogIcon.sapUiIcon {
		color: var(--sapCriticalElementColor);
	}

	& > div:last-of-type .sapMIBar {
		box-shadow: @_sap_m_Dialog_WarningHeaderShadow;
	}
}

.sapMDialogError > header {

	.sapMDialogTitleGroup .sapMDialogIcon.sapUiIcon {
		color: var(--sapNegativeElementColor);
	}

	& > div:last-of-type .sapMIBar {
		box-shadow: @_sap_m_Dialog_ErrorHeaderShadow;
	}
}

.sapMDialogSuccess > header {

	.sapMDialogTitleGroup .sapMDialogIcon.sapUiIcon {
		color: var(--sapPositiveElementColor);
	}

	& > div:last-of-type .sapMIBar {
		box-shadow: @_sap_m_Dialog_SuccessHeaderShadow;
	}
}

.sapMDialogInformation > header {

	.sapMDialogTitleGroup .sapMDialogIcon.sapUiIcon {
		color: var(--sapInformativeElementColor);
	}

	& > div:last-of-type .sapMIBar {
		box-shadow: @_sap_m_Dialog_InformationHeaderShadow;
	}
}

.sapMDialog .sapMDialogFirstFE,
.sapMDialog .sapMDialogLastFE {
	font-size: 0;
}

/* END "Dialog.less" */

/* START "DisplayListItem.less" */
/* ====================================== */
/* CSS for control sap.m/DisplayListItem  */
/* Base theme                             */
/* ====================================== */

.sapMDLI {
	display: flex;
	height: 3rem;
}

.sapMDLILabel {
	flex: auto;
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
	min-width: 3rem;
	display: block;
	font-size: var(--sapFontLargeSize);
	align-items: center;
	color: var(--sapList_TextColor);
}

.sapMLIBActionable .sapMDLILabel {
	cursor: pointer;
}

.sapMDLIValue {
	flex: auto;
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
	min-width: 3rem;
	font-size: @sapMFontMediumSize;
	margin-left: 1rem;
	text-align: right;
	color: var(--sapContent_LabelColor);
}

.sapMDLI > .sapMLIBContent {
	display: flex;
	white-space: nowrap;
	text-overflow: ellipsis;
	line-height: 3rem;
}

html[dir=rtl] .sapMDLIValue {
	text-align: left;
}

.sapMLIBActive .sapMDLILabel {
	color: var(--sapList_Active_TextColor);
}

.sapMLIBActive .sapMDLIValue {
	color: var(--sapList_Active_TextColor);
}

/* Compact size */
.sapUiSizeCompact .sapMDLI {
	height: 2rem;
}

.sapUiSizeCompact .sapMDLI > .sapMLIBContent {
	line-height: 2rem;
}

.sapUiSizeCompact .sapMDLILabel {
	font-size: @sapFontSize;
}
/* END "DisplayListItem.less" */

/* START "DraftIndicator.less" */
/* ===================================== */
/* CSS for control sap.m/DraftIndicator  */
/* Base theme                            */
/* ===================================== */

.sapMDraftIndicator {

	>.sapMLabel {
		font-style: italic;
		padding-left: 1rem;
		padding-right: 1rem;

		.sapMLabelTextWrapper {
			overflow: visible;
		}
	}

}
/* END "DraftIndicator.less" */

/* START "DynamicDateRange.less" */
/* ============================================= */
/* CSS for control sap.m/DynamicDateRange        */
/* Base theme                                    */
/* ============================================= */

.sapMDynamicDateRangePopover {

	section {

		& > * {
			box-sizing: border-box;
			padding: 0 1rem;
		}
		& > .sapMSlt {
			left: 1rem;
		}
		& > .sapMLabel {
			display: block;
			margin: 0.75rem 0 0.25rem 0;

			&.sapMDDRAdditionalLabel {
				display: inline-block;
				padding-left: 1.25rem;
				padding-bottom: 0.25rem
			}
		}

		& > .sapMLabel:first-child {
			margin-top: 1rem;
		}
	}

	.sapUiCal:not(.sapUiCalInt) {
		padding: 0;
		min-width: auto;
	}

	.sapUiCalMonthPicker {
		min-width: 19.5rem;
		width: 100%;
		margin: 0;
		padding: 1rem 0.25rem
	}

	&.sapMPage {
		background-color: var(--sapGroup_ContentBackground);
	}

	.sapMTB-Info-CTX.sapMIBar-CTX .sapMLabel {
		color: var(--sapPageFooter_TextColor);
		white-space: break-spaces;
	}
}

.sapUiSizeCompact .sapMDynamicDateRangePopover {

	.sapUiCalMonthPicker,
	.sapUiCalYearPicker {
		width: 100%;
		min-width: 16.5rem;
	}

	.sapMPageFooter .sapMIBar {
		height: 2rem;
	}

	section > .sapMLabel.sapMDDRAdditionalLabel {
		padding-left: 0;
	}
}

.sapUiIcon.sapMDDRDateOption {
	font-size: 1rem;
	color: var(--sapContent_IconColor);
}

.sapMDDRHiddenInput > .sapMInputBase {
	display: none;
}

.sapMDDRPopover .sapMTB-Info-CTX .sapMLabel .sapMLabelTextWrapper {
	padding-right: 0.125rem;
}
/* END "DynamicDateRange.less" */

/* START "ExpandableText.less" */
/* ===================================== */
/* CSS for control sap.m/ExpandableText  */
/* Base theme                            */
/* ===================================== */

.sapMExText {
	box-sizing: border-box;
	white-space: pre-line;
	word-wrap: break-word; /* works only if browser knows the width */
	cursor: text;
	font-size: @sapMFontMediumSize;
	font-family: var(--sapFontFamily);
	line-height: normal;
	color: var(--sapTextColor);
	max-width: 100%;

	.sapMLnk {
		vertical-align: bottom;
	}
}

.sapMExTextRenderWhitespaceWrap {
	white-space: pre-wrap;
}

.sapMExTextBreakWord {
	/* to break all words even browser does not know the width */
	word-break: break-all;
	word-break: break-word;  /* Non standard for old webkits */
}

.sapMPopover .sapMText.sapMExTextPopover {
	max-width: 30rem;
}

.sapMLIBActionable .sapMExText {
	cursor: pointer;
}

/*Necessary to align vertically to labels */
.sapUiForm .sapMExText {
	overflow: hidden;
}

.sapUiFormEdit .sapMExText {
	display: block;
	line-height: 1.375rem;
	min-height: 2.75rem; /* for empty text */
	padding-top: 0.6875rem;
	padding-bottom: 0.6875rem;

	& > a.sapMLnk {
		line-height: 1.375rem;
		padding: 0;
		margin: 0;
	}
}

html[data-sap-ui-browser^='cr'] .sapUiFormEdit .sapMExText {
	line-height: 1.4375rem;
}

.sapUiForm:not(.sapUiFormEdit) .sapMExText {
	min-height: 1rem; /* for empty text */
}

.sapMExTextEllipsis {
	white-space: pre-line;
	word-spacing: 0.125rem;
}

/* Compact size */
.sapUiSizeCompact .sapUiFormEdit .sapMExText {
	min-height: 2rem; /* for empty text */
	padding-top: 0.3125rem;
	padding-bottom: 0.3125rem;
}

html[data-sap-ui-browser^='cr'] .sapUiSizeCompact .sapUiFormEdit .sapMExText {
	line-height: 1.375rem;
}

.sapMExText .sapMEmptyIndicator.sapMEmptyIndicatorAuto {
	display: none;
}

.sapMShowEmpty-CTX .sapMExText .sapMEmptyIndicator.sapMEmptyIndicatorAuto {
	display: inline-block;
}
/* END "ExpandableText.less" */

/* START "AdditionalTextButton.less" */
.sapMBtn .sapMBtnAdditionalTextContent .sapMBtnText {
    flex-direction: column;
    font-size: 0.875rem;
}

.sapMBtnAdditionalTextContent .sapMBtnContentAddText:not(.sapMBtnActive) {
    width: 100%;
    float: left;
    color: @sapUiContentLabelColor;
    font-size: @sapMFontSmallSize;
}

 .sapMBtn:hover > .sapMBtnAdditionalTextContent.sapMBtnTransparent.sapMBtnHoverable:not(.sapMBtnActive):not(.sapMToggleBtnPressed) .sapMBtnContentAddText {
    color: @sapUiContentLabelColor;
}

.sapMBtnAdditionalTextContent .sapMBtnContent {
    line-height: 1.125rem;
}

:not(.sapMBtnDisabled) > .sapMBtnInner.sapMBtnActive .sapMBtnContentAddText {
	background-image: none;
	background-color: @sapUiButtonActiveBackground;
	border-color:     @sapUiButtonActiveBorderColor;
	color:            @sapUiButtonActiveTextColor;
	text-shadow:      none;
}

.sapMBtnAdditionalTextContent .sapMBtnContent {
    display: flex;
    flex-direction: column;
    justify-content: center;
    line-height: normal;
}

.sapUiSizeCompact .sapMBtnAdditionalTextContent .sapMBtnContent {
    line-height: 0.8125rem;
}
/* END "AdditionalTextButton.less" */

/* START "MessagePage.less" */
/* ================================== */
/* CSS for control sap.m/MessagePage  */
/* Base theme                         */
/* ================================== */

/* MessagePage general styles */
.sapMMessagePage {
	width: 100%;
	height: 100%;
	background: var(--sapBackgroundColor);
	position: relative;
}

.sapMMessagePageInner {
	box-sizing: border-box;
	display: flex;
	padding: 1rem;
	position: absolute;
	top: 0;
	right: 0;
	bottom: 0;
	left: 0;
	overflow-y: auto;
}

/* Allow MessagePage to be positioned inside ObjectPage */
.sapUxAPBlockContainer .sapMMessagePageInner {
	position: static;
}

.sapMMessagePageContentWrapper {
	margin: auto;
	display: flex;
	flex-direction: column;
	align-items: center;
	transform: translateY(-1rem);
	width: 30rem;
	max-width: 100%;
}

.sapMMessagePage > .sapMBar + .sapMMessagePageInner {
	top: 3rem;
}

/* MessagePage inner controls styles */
.sapMMessagePage {

	.sapMMessagePageInner {

		.sapMMessagePageIcon.sapUiIcon::before {
			font-size: 6rem;
			color: var(--sapContent_NonInteractiveIconColor);
		}

		& > .sapMMessagePageContentWrapper > * {
			max-width: 100%;
		}
	}

	.sapMMessagePageMainText {
		font-size: 1.5rem;
		line-height: 2rem;
		margin-top: 1rem;
	}

	.sapMText.sapMMessagePageMainText {
		color: var(--sapGroup_TitleTextColor);
		font-size: var(--sapFontHeader2Size);
	}

	.sapMMessagePageDescription {
		font-size: 1rem;
		margin-top: 1rem;

		.sapMLnk {
			font-size: 1rem;
		}
	}

	.sapMText.sapMMessagePageDescription {
		color: var(--sapContent_LabelColor);
		font-size: var(--sapFontHeader5Size);
	}

	.sapMMessagePageButtonsWrapper {
		margin-top: 2rem;
		text-align: center;
	}

	.sapMBtn {
		margin: 0 0.25rem;
	}
}

.sapUiSizeCompact .sapMMessagePage > .sapMBar {
	height: 2.5rem;

	.sapMBarPH,
	.sapMBarLeft {
		height: 2.5rem;
		padding-top: 0.25rem;
		padding-bottom: 0.25rem;
	}

	.sapMBarMiddle {
		height: 2.5rem;
	}

	& + .sapMMessagePageInner {
		top: 2.5rem;
	}
}
/* END "MessagePage.less" */

/* START "MultiUnitPopover.less" */
/* =================================================================================== */
/* CSS for control sap.m.table/Util.js#createOrUpdateMultiUnitPopover & it's contents  */
/* Base theme                                                                          */
/* =================================================================================== */
.sapMMultiUnitPopover .sapMCLI {
	padding: 0 2px 0 2px;
	height: 3rem;
}

.sapUiSizeCompact .sapMMultiUnitPopover .sapMCLI {
	height: 2rem;
}
/* END "MultiUnitPopover.less" */

/* START "FacetFilter.less" */
/* ================================== */
/* CSS for control sap.m/FacetFilter  */
/* Base theme                         */
/* ================================== */

.sapMFF > .sapMTB-Info-CTX.sapMTB,
.sapUiSizeCompact .sapMFF {
	height: 2rem;
}

.sapUiSizeCompact .sapMFFArrowScroll {
	padding: 0.5rem 0.4rem;
}

.sapUiSizeCompact .sapMFF .sapMFFLRemoveIcon {
	line-height: 0.8rem;
	padding: 0.6rem;
}

.sapMFF {
	position: relative;
}

.sapMFFHead {
	margin-left: 0rem;
	white-space: nowrap;
	position: relative;
	overflow: hidden;
}

.sapMFF .sapUiIcon,
.sapMFF .sapMTB .sapMBtnIcon .sapUiIcon {
	/* Override the default button icon size since it is too big. */
	font-size: 1rem;
}

html.sap-phone .sapMFFHead {
	margin: 1rem;
}

/* Scroll arrows */
html.sap-desktop .sapMFFScrolling > .sapMFFHead {
	margin-right: 2rem;
	margin-left: 3rem;
}

.sapMFFArrowScroll {
	font-size: 1rem;
	line-height: 1rem;
	position: absolute;
	top: 0;
	color: var(--sapContent_IconColor);
	padding: 1rem 0.4rem;
}

.sapMFFArrowScrollLeft {
	left: 1rem;
}

.sapMFFArrowScrollRight {
	right: 0;
}

.sapMFFNoScrolling > .sapMFFArrowScroll {
	visibility: hidden;
}

html.sap-desktop .sapMFFScrolling.sapMFFNoScrollBack > .sapMFFArrowScrollLeft,
html.sap-desktop .sapMFFScrolling.sapMFFNoScrollForward > .sapMFFArrowScrollRight {
	visibility: hidden;
}

/* Reset button */
.sapMFFResetDiv {
	position: absolute;
	width: 3rem;
	right: -3rem;
	top: 0;
}

.sapMFFResetSpacer {
	/* Space reserved at the end of the FacetFilter for placing the reset button */
	margin-right: 2.75rem;
	margin-left: 1rem;
}

.sapUiSizeCompact {

	.sapMFFResetSpacer {
		/* Space reserved at the end of the FacetFilter for placing the reset button */
		margin-right: 2.5rem;
		margin-left: 1rem;
	}
}

.sapMFF .sapMTB-Info-CTX .sapMBtnIcon .sapUiIcon {
	color: var(--sapContent_ContrastIconColor);
}

/* Move the reset button to the end of the toolbar */
.sapMFF .sapMTB-Info-CTX.sapMTB > .sapMBarChild.sapMBtn {
    margin: 0;
}

.sapMFF .sapMTB-Info-CTX.sapMTB {
    padding-right: 0;
}

.sapMFF .sapMTB-Info-CTX.sapMTB > .sapMBtn:hover {
    background-color: var(--sapInfobar_Hover_Background);
}

html.sap-desktop .sapMFFRefresh.sapMBtn:focus > .sapMFocusable {
	outline-color: var(--sapContent_ContrastFocusColor);
}

html.sap-desktop .sapMFF > .sapMTB-Info-CTX.sapMTB > .sapMFFRefresh.sapMBtn:focus .sapMBtnFocusDiv {
	border-color: var(--sapContent_ContrastFocusColor);
}

.sapMFF .sapMTB-Info-CTX.sapMTB > .sapMBtn:active {
    background-color: var(--sapInfobar_Active_Background);
}

.sapMFFCheckbar {
	cursor: pointer
}

.sapMFFDialog .sapMCbLabel {
	cursor: pointer
}

.sapMFF .sapMFFArrowScroll:hover {
	background-color: var(--sapButton_Lite_Hover_Background);
}

.sapMFF .sapMFFArrowScroll:active {
	background-color: var(--sapButton_Lite_Active_Background);
	color: var(--sapContent_ContrastIconColor);
}

/* specific selector for the light facet filter text to use padding instead of margin,
  otherwise the text gets cut off in Arabic because of the italic font
*/
.sapMFF > .sapMIBar.sapMTB.sapMTB-Info-CTX {
	padding-right: 0.5rem;
	padding-left: 0.75rem;

	.sapMText {
		padding: 0.5rem 0.5rem 0.5rem 0;
	}
}

.sapMFFDialog,
.sapMFFPop .sapMPopoverSubHeader {

	.sapMFFCheckbar .sapMBarLeft {
		padding-left: 0.1875rem;
	}
}

.sapUiSizeCompact {

	.sapMFFDialog .sapMFFCheckbar .sapMBarLeft,
	.sapMFFPop .sapMPopoverSubHeader .sapMFFCheckbar .sapMBarLeft {
		padding-left: 0.5rem;
	}
}
/* END "FacetFilter.less" */

/* START "FacetFilterItem.less" */
/* ====================================== */
/* CSS for control sap.m/FacetFilterItem  */
/* Base theme                             */
/* ====================================== */

/* Fix alignment of the list item counter and force a min height. */
.sapMFFLI {
	display: flex;
	min-height: 3rem;
}

/* Fix alignment of the list item counter. */
.sapMFFLI > .sapMLIBContent {
	text-overflow: ellipsis;
	align-items: center;
}

.sapMFFLITitle {
	font-size: var(--sapFontLargeSize);
	color: var(--sapList_TextColor);
	overflow: hidden;
	text-overflow: ellipsis;
}

.sapMFFLITitleWrap {
	word-wrap: break-word;
	overflow: hidden;
	text-overflow: ellipsis;
	flex: auto;
	align-items: center;
}

.sapUiSizeCompact .sapMFFLITitle {
	font-size: @sapMFontMediumSize;
}
/* END "FacetFilterItem.less" */

/* START "FacetFilterList.less" */
/* ====================================== */
/* CSS for control sap.m/FacetFilterList  */
/* Base theme                             */
/* ====================================== */

/* Facet Filter Popover */
.sapMFFPop.sapMPopover,
.sapMFFPop.sapMPopover .sapMPopoverCont {
	min-width: 300px;
}

.sapMFFPop .sapMCbLabel {
	cursor: pointer
}

.sapMFF .sapMFFLRemoveIcon {
	color: var(--sapButton_IconColor);
	font-size: 0.875rem;
	line-height: 2rem;
	padding-bottom: 0.25rem;
	padding-top: 0.25rem;
	padding-left: 0.65rem;
	padding-right: 0.65rem;
	margin-top: 0.2rem;
	margin-bottom: 0.2rem;
}

.sapMFF .sapMFFLRemoveIcon:hover {
	background-color: var(--sapButton_Lite_Hover_Background);
	border-radius: 0.2rem;
}

.sapUiSizeCompact .sapMFF .sapMFFLRemoveIcon {
	line-height: 1.5rem;
	padding-bottom: 0.05rem;
	padding-top: 0.05rem;
	padding-left: 0.25rem;
	padding-right: 0.25rem;
}

/* We need to toggle separate classes to show/hide the remove icon, otherwise
 clicking the icon does not remove the facet button in ie9 */
.sapMFF .sapMFFLVisibleRemoveIcon {
	visibility: visible;
}

.sapMFF .sapMFFLHiddenRemoveIcon {
	visibility: hidden;
}

.sapMFF .sapMFFLRemoveIcon:hover {
	background-color: var(--sapButton_Lite_Hover_Background);
}

.sapMFF .sapMFFLRemoveIcon:active {
	background-color: var(--sapButton_Lite_Active_Background);
	color: var(--sapContent_ContrastIconColor);
}
/* END "FacetFilterList.less" */

/* START "FeedContent.less" */
/* ================================== */
/* CSS for control sap.m/FeedContent  */
/* Base theme                         */
/* ================================== */

.sapMFC {
	outline: none;
	position: relative;
	width: 100%;
	height: 3.875rem;
	display: inline-block;
}

.sapMFCCTxt {
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
	height: 2.126rem;
	max-width: 20.5rem;
	margin-top: 0.3rem;

	.sapMText {
		display: inline;
		line-height: 1.063rem;
	}
}

.sapMFCSbh {
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
	max-width: 20.5rem;
	line-height: 1.063rem;
	font-size: var(--sapFontSmallSize);
	color: var(--sapTile_TextColor);
}

.sapMFCValue {
	position: relative;
	overflow: hidden;
	text-overflow: ellipsis;
	padding-left: 1rem;
	padding-top: 0.875rem;
	font-size: 2.25rem;
	line-height: 3rem;
	max-width: 10.25rem;
	min-width: 3rem;
	float: right;
	font-family: var(--sapFontHeaderFamily);
	font-weight: @sapUiFontHeaderWeight;
}

.sapMFCCTxt > .sapMText {
	cursor: inherit;
}

.sapMFC:focus:not([tabindex]) {
	outline: none;
}

.sapMFCCTxt .sapMText {
	font-size: @sapFontSize;
	color: var(--sapTile_TitleTextColor);
}

.sapMGT.TwoByOne.sapMTileSmallPhone {

	.sapMFCCTxt .sapMText {
		font-size: var(--sapFontSmallSize);
	}
}

.sapMFCValue.Neutral {
	color: var(--sapNeutralTextColor);
}

.sapMFCValue.Good {
	color: var(--sapPositiveTextColor);
}

.sapMFCValue.Critical {
	color: var(--sapCriticalTextColor);
}

.sapMFCValue.Error {
	color: var(--sapNegativeTextColor);
}

html.sap-desktop .sapMFC:focus {
	outline: var(--sapContent_FocusColor) var(--sapContent_FocusStyle) var(--sapContent_FocusWidth);
}

html.sap-desktop .sapMFC:focus:not([tabindex]) {
	outline: none;
}

.sapMTileSmallPhone {

	.sapMFC {
		width: 17.75rem;
		height: 3.438rem;
	}

	.sapMFCCTxt {
		max-width: 17.75rem;
		margin-top: 0.1rem;
	}

	.sapMFCSbh {
		max-width: 17.75rem;
	}

	.sapMFCValue {
		padding-top: 1.188rem;
		padding-left: 0.4rem;
		font-size: 2rem;
		line-height: 2.25rem;
		max-width: 8.25rem;
		min-width: 2.5rem
	}
}
/* END "FeedContent.less" */

/* START "FeedInput.less" */
/* ================================ */
/* CSS for control sap.m/FeedInput  */
/* Base theme                       */
/* ================================ */

.sapMFeedInBase {
	display: block;
	position: relative;
	height: auto;
	width: 100%;
	box-sizing: border-box;
}

.sapMFeedIn {
	display: block;
	position: relative;
	padding: 0.5rem 0.5rem 0.5rem 5rem;
	width: 100%;
	box-sizing: border-box;
}

.sapMFeedIn.sapMFeedInDisabled {
	opacity: 0.5;
}

/*hide the icon if the width of the screen <=25rem */
@media (max-width: 25rem) {

	.sapMFeedInFigure {
		display: none;
	}

	.sapMFeedIn {
		padding-left: 0.5rem;
	}
}

.sapMFeedInNoIcon {
	padding-left: 0.5rem;
}

.sapMFeedInFigure {
	position: absolute;
	height: 3rem;
	width: 3rem;
	margin: 0.5rem 1rem 0.5rem 1rem;
	background-color: none;
	box-sizing: border-box;
	left: 0;
	border-width: 0.125rem;
	border-style: none;
	border-color: var(--sapField_Background); /* the background parameter is used for to ensure the border theme is aligned with the text area background */
}

.sapMFeedInFigure .sapFAvatarSquare {
	border-radius: var(--sapElement_BorderCornerRadius);
}

.sapMFeedInFigure .sapFAvatarM {
	width: 3rem;
	height: 3rem;
	font-size: 1.125rem;
	box-sizing: border-box;
	border: 0.0625rem solid var(--sapGroup_ContentBorderColor);
}

.sapMFeedInFigure .sapFAvatarImageHolder {
	background: none;
	display: block;
	position: absolute;
	left: 0;
	top: 0;
	right: 0;
	bottom: 0;
	background-size: cover;
}

.sapMFeedInImage {
	width: 3.75rem;
	line-height: 3.75rem;
	font-size: 2.25rem;
	display: block;
	margin-left: auto;
	margin-right: auto;
	color: var(--sapContent_ImagePlaceholderForegroundColor);
}

.sapMFeedInImageBgColor {
	background-color: var(--sapContent_ImagePlaceholderBackground);
}

.sapMFeedInBase .sapMFeedInCounter {
	display: block;
	width: 100%;
	text-align: end;
}

.sapMFeedIn .sapMInputBase .sapMTextArea .sapMTextAreaInner {
	box-sizing: border-box;
	width: var(--sapField_BorderWidth);
	border-radius: var(--sapField_BorderCornerRadius);
	font-style: italic;
	font-weight: normal;
	color: var(--sapField_PlaceholderTextColor);
}

.sapMFeedInBase .sapMFeedInCounter .sapMTextAreaCounter {
	overflow: hidden;
	font-family: var(--sapFontFamily);
	font-size: var(--sapFontSmallSize);
	padding: 0.125rem;
}

.sapMFeedIn:not(.sapMFeedInDisabled) .sapMFeedInContainer {
	background-color: var(--sapGroup_ContentBackground);
}

.sapMFeedIn .sapMFeedInContainer {
	width: 100%;
	height: auto;
	position: relative;
	background-color: var(--sapGroup_ContentBackground);
}

.sapMFeedIn > .sapMFeedInContainer > .sapMInputBase {
	width: 100%;
	padding: 0 4.0rem 0 0;
	margin: 0;
	min-height: 4rem;
}

.sapMFeedIn > .sapMFeedInContainer > .sapMTextArea > .sapMInputBaseInner {
	border-bottom-style: hidden;
	border-top-style: hidden;
	border-left-style: hidden;
	border-right-style: hidden;
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	font-style: normal;
	font-weight: normal;
	color: var(--sapField_TextColor);
	line-height: 1.5;
	padding: 0.5rem 0.05rem 0.5rem 0.5rem;
	min-height: 4rem;
}

.sapMFeedInContainer .sapMInputBaseInner {
	border: none;
}

.sapMFeedIn .sapMBtn {
	width: 4.0rem;
	height: 3.75rem;
	padding: 0.25rem 0.625rem 0.25rem 0.625rem;
	margin: 0.125rem 0 0.125rem 0.5rem;
	position: absolute;
	right: 0;
	bottom: 0;
	border-width: var(--sapButton_BorderWidth);
	border-radius: var(--sapButton_BorderCornerRadius);
	background-color: var(--sapGroup_ContentBackground);
	font-size: @sapFontSize;
	font-family: var(--sapFontSemiboldDuplexFamily);
}

.sapMFeedIn .sapMBtnInner {
	width: 2.25rem;
	height: 2.25rem;
	padding: 0;
	border-radius: var(--sapButton_BorderCornerRadius);
	background-color: var(--sapButton_Background);
	border: 1px solid var(--sapButton_BorderColor);
	color: var(--sapButton_TextColor);
	border-width: var(--sapButton_BorderWidth);
}

.sapMFeedIn .sapMBtnIcon {
	width: 2.25rem;
	height: 2.25rem;
}

.sapMFeedIn .sapMInputBase.sapMTextArea .sapMTextAreaInner {
	min-height: 4rem;
	padding: 0.375rem 0.5rem;
}

/* Compact size */
.sapUiSizeCompact .sapMFeedIn .sapMBtnIcon {
	margin-left: 0;
	padding: 0.25rem 0.625rem 0.25rem 0.625rem;
}

.sapUiSizeCompact .sapMFeedIn > .sapMFeedInContainer > .sapMInputBase {
	margin: 0;
	min-height: 4rem;
}

.sapMFeedIn .sapMInputBase .sapMTextArea .sapMInputBaseContentWrapper {
	min-height: 4rem;
}

.sapMFeedIn .sapMInputBase .sapMTextArea {
	width: 100%;
	min-width: 6rem;
	min-height:2.25rem;
	padding: 0 0.625rem;
	margin: 0.25rem 0;
	padding-top: 0.5rem;
	box-sizing: border-box;
	background-color: var(--sapGroup_ContentBackground);
}
/* END "FeedInput.less" */

/* START "FeedListItem.less" */
/* =================================== */
/* CSS for control sap.m/FeedListItem  */
/* Base theme                          */
/* =================================== */

.sapMFeedListItem {
	display: flex;
}

.sapMFeedListItem .sapUiSelectable > * {
	-webkit-user-select: text;
	-moz-user-select: text;
	user-select: text;

	&::selection {
		background: var(--sapSelectedColor);
		color: var(--sapContent_ContrastTextColor);
	}

	&::-moz-selection {
		background: var(--sapSelectedColor);
		color: var(--sapContent_ContrastTextColor);
	}
}

.sapMFeedListItemFigure .sapFAvatarSquare {
	border-radius: var(--sapElement_BorderCornerRadius);

	.sapFAvatarImageHolder {
		border-radius: var(--sapElement_BorderCornerRadius);
	}
}

.sapMFeedListItemFigure .sapFAvatarS {
	height: 3rem;
	font-size: 2rem;
}

.sapMFeedListItemFigure .sapFAvatarImageHolder {
	background: none;
	display: block;
	position: absolute;
	left: 0;
	top: 0;
	right: 0;
	bottom: 0;
	background-size: cover;
}

.sapMFeedListItemImageInactive {
	cursor: default;
}

.sapMFeedListShowSeparatorsAll .sapMFeedListItem {
	padding-top: 1rem;
	padding-bottom: 1rem;
}

/* Added to remove unwanted padding when FeedListItem is used in table*/
.sapMListTblCell {

	.sapMFeedListShowSeparatorsAll .sapMFeedListItem {
		padding-top: 0;
		padding-bottom: 0;
	}
}

/* Makes sure that floating image is not displayed in the next list item. */
.sapMFeedListItem::after {
	content: ' ';
	display: block;
	height: 0;
	clear: both;
	overflow: hidden;
	visibility: hidden;
}

.sapMFeedListItem .sapMFeedListItemTextText a,
.sapMFeedListItem .sapMFeedListItemTextName a {
	font-family: var(--sapFontFamily);
	font-weight: bold;
	line-height: 1.4;
}

/*
 The following is necessary to make sure that image links
 do not show a border when run in IE.
 */
.sapMFeedListItemFigure img {
	border: 0;
}

.sapMFeedListItemText {
	vertical-align: top;
	margin: 0;
}

.sapMFeedListItemTextString {
	cursor: text;
}

.sapMFeedListItemText .sapMLnk {
	vertical-align: top;
	display: inline;
}

html.sap-tablet .sapMFeedListItemHasFigure,
html.sap-desktop .sapMFeedListItemHasFigure {
	margin-left: 1rem;
	width: 100%;
}

.sapMFeedListItemTextText {
	margin: 0;
	line-height: 1.4;
	font-family: var(--sapFontFamily);
	text-overflow: ellipsis;
}

.sapMFeedListItemFigure {
	float: left;
	height: 3rem;
	width: 3rem;
	overflow: visible;
	margin: 0;
}

.sapMFeedListItemImageInactive,
.sapMFeedListItemImage {
	width: 3rem;
	height: 3rem;
	line-height: 3rem;
	font-size: 2.5rem;
	margin-left: 0;
	margin-right: 0;
	overflow: hidden;
}

.sapMFeedListItemTimestamp {
	line-height: auto;
}

.sapMFeedListItemTimestampText,
.sapMFeedListItemInfoText {
	cursor: text;
}

.sapMFeedListItemInfo {
	line-height: auto;
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
}

.sapMFeedListItemTextName {
	line-height: 1.4;
	font-family: var(--sapFontFamily);
	font-weight: bold;
	font-size: @sapMFontMediumSize;
	color: var(--sapList_TextColor);
}

html.sap-desktop .sapMFeedListItemTextName,
html.sap-tablet .sapMFeedListItemTextName {
	margin-right: 0.3rem;
}

html[dir=rtl] .sapMFeedListItemTextName {
	/* In rtl mode, this will be converted to 'float: right;' which is the correct value */
	float: left;
}

html[dir=rtl].sap-phone .sapMFeedListItemTextName {
	/* Overwrite rtl values for phones */
	float: none;
}

.sapMFeedListItemFooter {
	margin-top: 0.375rem;
	margin-bottom: 0;
	overflow: hidden;
	white-space: pre-wrap;
	text-overflow: ellipsis;
	font-family: var(--sapFontFamily);
}

.sapMFeedListItemTitleDiv.sapMLIB {
	display: flex;
	flex-direction: row;
	align-items: center;
}

.sapMFeedListShowSeparatorsNone.sapMLIB {
	border-bottom: 0;
}

.sapMFeedListItemTitleDiv > .sapMLIBContent {
	flex-basis: auto;
}

/* phone adoption */
html.sap-phone .sapMFeedListItemHeader {
	white-space: nowrap;
	overflow: hidden;
}

html.sap-phone .sapMFeedListItemHeader.sapMFeedListItemFullHeight,
html.sap-phone .sapMFeedListItemHeader.sapMFeedListItemHasFigure {
	height: 3.4rem;
}

html.sap-phone .sapMFeedListItemImageInactive,
html.sap-phone .sapMFeedListItemImage {
	width: 3rem;
	height: 3rem;
	line-height: 3rem;
	font-size: 2.5rem;
}

html.sap-phone .sapMFeedListItemTextName {
	margin-top: 0.25rem;
	margin-bottom: 0.25rem;
}

html.sap-phone .sapMFeedListItemTextName a {
	font-family: var(--sapFontFamily);
	font-weight: normal;
	width: 100%;
}

html.sap-phone .sapMFeedListItemFigure {
	height: 3rem;
	width: 3rem;
	margin-right: 0.75rem
}

html.sap-phone .sapMFeedListItemFooter {
	margin-top: 0.375rem;
	margin-bottom: 0;
	overflow: hidden;
	text-overflow: ellipsis;
}

html.sap-phone .sapMFeedListItemTimestamp {
	margin-top: -0.25rem;
	margin-bottom: 0.25rem;
	overflow: hidden;
	text-overflow: ellipsis;
}

html.sap-phone .sapMFeedListItemText {
	clear: left;
	line-height: 1.4;
	margin-top: 0.75rem;
	margin-bottom: 0;
	overflow: hidden;
	text-overflow: ellipsis;
}

.sapMFeedListItemTextText {
	font-size: @sapMFontMediumSize;
	color: var(--sapList_TextColor);
	word-wrap: break-word;
}

.sapMFeedListItemImageInactive:focus,
.sapMFeedListItemImage:focus {
	outline: var(--sapContent_FocusColor) var(--sapContent_FocusStyle) var(--sapContent_FocusWidth);
}

html.sap-tablet .sapMFeedListItem .sapMLnk:focus {
	outline: var(--sapContent_FocusColor) var(--sapContent_FocusStyle) var(--sapContent_FocusWidth);
}

/* Allow image outline(-left) to be rendered completely */
.sapMFeedListShowSeparatorsAll .sapMFeedListItem {
	padding-left: 1rem;
	padding-right: 1rem;
}

.sapMFeedListItemTitleDiv.sapMLIB {
	padding-left: 0.9375rem;
}

html.sap-desktop .sapMFeedListItem .sapMLnk:focus {
	outline-color: var(--sapContent_FocusColor);
}

.sapMFeedListItemImage,
.sapMLIBActive .sapMFeedListItem span.sapMFeedListItemImage {
	color: var(--sapContent_IconColor);
}

.sapMFeedListItemImageInactive,
.sapMLIBActive .sapMFeedListItem span.sapMFeedListItemImageInactive {
	color: var(--sapContent_ContrastTextColor);
}

.sapMFeedListItemFooter {
	font-size: var(--sapFontSmallSize);
	color: var(--sapContent_LabelColor);
}

.sapMFeedListItem .sapMFeedListItemTextText a,
.sapMFeedListItem .sapMFeedListItemTextName a {
	font-size: @sapMFontMediumSize;
	color: var(--sapLinkColor);
}

.sapMFeedListItemText {
	color: var(--sapList_TextColor);
}

.sapMLIBActive .sapMFeedListItem a,
.sapMLIBActive .sapMFeedListItem span,
.sapMLIBActive .sapMFeedListItem p,
.sapMLIBActive .sapMFeedListItemFooter,
.sapMLIBActive .sapMFeedListItemText,
.sapMLIBActive .sapMFeedListItemTimestamp,
.sapMLIBActive .sapMFeedListItem .sapMFeedListItemTextText .sapMFeedListItemTextString,
.sapMLIBActive .sapMFeedListItem:hover .sapMFeedListItemTextText > .sapMFeedListItemLinkExpandCollapse,
.sapMLIBActive .sapMFeedListItemTextText,
.sapMLIBActive .sapMFeedListItemTextText > .sapMLnk,
.sapMLIBActive .sapMFeedListItemTextText > .sapMLnk.sapMLnkDsbl,
.sapMLIBActive .sapMFeedListItemTextName > .sapMLnk,
.sapMLIBActive .sapMFeedListItemTextName > .sapMLnk.sapMLnkDsbl {
	color: var(--sapList_Active_TextColor);
	opacity: 1;
	text-decoration: none;
}

.sapMFeedListItemTextText > .sapMLnk.sapMLnkDsbl,
.sapMFeedListItemTextName > .sapMLnk.sapMLnkDsbl,
html.sap-desktop .sapMFeedListItemTextName > .sapMLnk.sapMLnkDsbl:hover {
	opacity: 1;
	color: var(--sapList_TextColor);
}

.sapMFeedListItemTextText > .sapMLnk {
	display: inline;
}

.sapMFeedListItemTextText > .sapMLnk.sapMLnkDsbl {
	color: var(--sapList_TextColor);
}

html.sap-desktop .sapMFeedListItemTextText > .sapMLnk.sapMLnkDsbl:hover {
	color: var(--sapList_TextColor);
}

.sapMFeedListShowSeparatorsAll.sapMLIB {
	border-bottom: 0.063rem solid var(--sapList_BorderColor);
}

html.sap-phone .sapMFeedListItemTextName a {
	line-height: normal;
	font-size: var(--sapFontLargeSize);
}

html.sap-phone .sapMFeedListItemText,
html.sap-phone .sapMFeedListItemTimestamp {
	font-size: @sapMFontMediumSize;
}

html.sap-phone .sapMFeedListItemFooter {
	font-size: var(--sapFontSmallSize);
}

html.sap-phone .sapMFeedListItemTimestamp {
	color: var(--sapContent_LabelColor);
	line-height: normal;
}

.sapMFeedListItem .sapMFeedListItemTextText > .sapMFeedListItemLinkExpandCollapse {
	font-weight: normal;
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	font-weight: normal;
	color: var(--sapLinkColor);
}

.sapMFeedListItem:hover .sapMFeedListItemTextText > .sapMFeedListItemLinkExpandCollapse {
	color: var(--sapLinkColor);
}

.sapMFeedListItem .sapMFeedListItemTextText .sapMFeedListItemTextString {
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	font-style: normal;
	font-weight: normal;
	color: var(--sapList_TextColor);
	line-height: 1.4;
}

.sapMFeedListItemActionButton {
	float: right;
	position: relative;
	margin: -0.75rem 0.1rem 0.5rem 0.5rem;
}

.sapUiSizeCompact .sapMFeedListItemActionButton {
	margin-top: -0.6875rem;
}

html.sap-phone .sapMFeedListItem .sapMFeedListItemActionButton {
	margin: -0.1875rem 0 0 0.75rem;
}

/* added to show ellipsis for user name */
.sapMFeedListItemTextName {
	text-overflow: ellipsis;
	overflow: hidden;
	white-space: nowrap;
}

/* removed sapMFeedListItemActionButton so that the css is applied for every feed list item textnames */
.sapMFeedListItemText .sapMFeedListItemTextName,
.sapMFeedListItemHeader .sapMFeedListItemTextName {
	max-width: ~"calc(100% - 2.8rem)"; /* 2.5rem width of button in cozy mode + 0.3rem margin-right */
	display: inline-block;

	.sapUiSizeCompact & {
		max-width: ~"calc(100% - 2.3rem)"; /* 2rem width of button in compact mode */
	}

	html.sap-phone & {
		max-width: 100%;
	}
}
/* END "FeedListItem.less" */

/* START "FlexBox.less" */
/* ============================== */
/* CSS for control sap.m/FlexBox  */
/* Base theme                     */
/* ============================== */

/* mixin call for creating container content padding classes (arguments: rootContainer, contentSelector) */
.sapUiContainerContentPadding(~".sapMFlexItem", ~"");

.sapMFlexBoxBGSolid,
.sapMFlexBoxBGTranslucent {
	background-color: var(--sapGroup_ContentBackground);
}

.sapMFlexBoxBGTransparent {
	background-color: transparent;
}

.sapMFlexBox {
	display: flex;
	box-sizing: border-box;
}

.sapMFlexBoxInline:not(.sapMFlexBoxFit) {
	display: inline-flex;
}

.sapMFlexBoxFit,
.sapMFlexBoxScroll {
	height: 100%;
}

/* Nested FlexBox should have auto-height because of the wrappers */
.sapMFlexBox > .sapMFlexBoxFit {
	height: auto;
}

.sapMFlexItem {
	position: relative;
}

/* flex-direction */
.sapMHBox {
	flex-direction: row;
}

.sapMVBox {
	flex-direction: column;
}

.sapMHBox.sapMFlexBoxReverse {
	flex-direction: row-reverse;
}

.sapMVBox.sapMFlexBoxReverse {
	flex-direction: column-reverse;
}

/* justify-content */
.sapMFlexBoxJustifyStart {
	justify-content: flex-start;
}

.sapMFlexBoxJustifyEnd {
	justify-content: flex-end;
}

.sapMFlexBoxJustifyCenter {
	justify-content: center;
}

.sapMFlexBoxJustifySpaceBetween {
	justify-content: space-between;
}

.sapMFlexBoxJustifySpaceAround {
	justify-content: space-around;
}

.sapMFlexBoxJustifyInherit {
	justify-content: inherit;
}

/* align-items */
.sapMFlexBoxAlignItemsStart {
	align-items: flex-start;
}

.sapMFlexBoxAlignItemsEnd {
	align-items: flex-end;
}

.sapMFlexBoxAlignItemsCenter {
	align-items: center;
}

.sapMFlexBoxAlignItemsBaseline {
	align-items: baseline;
}

.sapMFlexBoxAlignItemsStretch {
	align-items: stretch;
}

.sapMFlexBoxAlignItemsInherit {
	align-items: inherit;
}

/* wrap */
.sapMFlexBoxWrapNoWrap {
	flex-wrap: nowrap;
}

.sapMFlexBoxWrapWrap {
	flex-wrap: wrap;
}

.sapMFlexBoxWrapWrapReverse {
	flex-wrap: wrap-reverse;
}

/* align-content */
.sapMFlexBoxAlignContentStart {
	align-content: flex-start;
}

.sapMFlexBoxAlignContentEnd {
	align-content: flex-end;
}

.sapMFlexBoxAlignContentCenter {
	align-content: center;
}

.sapMFlexBoxAlignContentSpaceBetween {
	align-content: space-between;
}

.sapMFlexBoxAlignContentSpaceAround {
	align-content: space-around;
}

.sapMFlexBoxAlignContentStretch {
	align-content: stretch;
}

/* align-self */
.sapMFlexItemAlignAuto {
	align-self: auto;
}

.sapMFlexItemAlignStart {
	align-self: flex-start;
}

.sapMFlexItemAlignEnd {
	align-self: flex-end;
}

.sapMFlexItemAlignCenter {
	align-self: center;
}

.sapMFlexItemAlignBaseline {
	align-self: baseline;
}

.sapMFlexItemAlignStretch {
	align-self: stretch;
}

.sapMFlexItemAlignInherit {
	align-self: inherit;
}
/* END "FlexBox.less" */

/* START "FormattedText.less" */
/* ==================================== */
/* CSS for control sap.m/FormattedText  */
/* Base theme                           */
/* ==================================== */

.sapMFT {
	font-family: var(--sapFontFamily);
	vertical-align: top;
	font-size: @sapMFontMediumSize;

	/* BCP: 1870022567 - Reset sapMLnk font-size and wrapping specifics */
	.sapMLnk {
		font-size: inherit;
		display: initial;
		white-space: initial;
		overflow: initial;
		text-overflow: initial;
		word-wrap: initial;
	}

	/* BCP: 002075129400010494822023 - preserve H* tags default 'display' property */
	.sapMTitle {
		display: inherit;
	}
}

.sapMFTOverflowWidth {
	overflow-x: auto;
}

.sapMFTOverflowHeight {
	overflow-y: auto;
}

/* If control is used on black/dark background */
.sapUiInverted-CTX .sapMFT {
	color: var(--sapContent_ContrastTextColor);
}

/* Do not invert for table */
.sapUiInverted-CTX .sapUiTable .sapMFT {
	color: var(--sapTextColor);
}
/* END "FormattedText.less" */

/* START "GenericTag.less" */
/* ================================= */
/* CSS for control sap.m/GenericTag  */
/* Base theme                        */
/* ================================= */

@_sap_m_GenericTag_BorderWidth: 0.25rem;
@_sap_m_GenericTag_PaddingLeft: 0.25rem;
@_sap_m_GenericTag_StatusIconWidth: 1rem;
@_sap_m_GenericTag_IconPaddingRight: 0.5rem;
@_sap_m_GenericTag_TextMinWidth: 1.5625rem;
@_sap_m_GenericTag_HeightCompact: 1.625rem;
@_sap_m_GenericTag_HeightCozy: 2.5rem;

.sapMGenericTag {
	margin-top: 0.1875rem;
	margin-bottom: 0.1875rem;
	display: inline-block;
	max-width: 100%;

	&:focus {
		outline: none;
	}

	&:-moz-focus-inner {
		padding: 0;
		border: 0
	}

	&.sapMGenericTagOverflowToolbar {
		margin-right: 0;
		max-width: 20rem;

		.sapMGenericTagWrap {
			width: 100%;
		}

		.sap-phone & {
			max-width: 100%;
		}
	}

	.sapMBarChild& {

		>.sapMGenericTagWrap {
			display: flex;
		}
	}
}

html.sap-desktop .sapMGenericTag:focus {
	position: relative;

	&::before {
		position: absolute;
		content: ' ';
		top: 1px;
		right: 1px;
		bottom: 1px;
		left: 1px;
		z-index: 2;
		pointer-events: none;
	}

	&::after {
		position: absolute;
		content: ' ';
		top: 1px;
		right: 1px;
		bottom: 1px;
		left: 1px;
		z-index: 2;
		pointer-events: none;
	}
}

.sapMGenericTagWrap {
	background: var(--sapList_Background);
	box-shadow: @sapUiShadowLevel0;
	box-sizing: border-box;
	border-radius: 0.125rem;
	border-left-width: @_sap_m_GenericTag_BorderWidth;
	border-left-style: solid;
	padding: 0 0.5rem 0 @_sap_m_GenericTag_PaddingLeft;
	display: inline-flex;
	max-width: 100%;
	flex-flow: row nowrap;
	align-items: center;
	page-break-after: auto;
	height: @_sap_m_GenericTag_HeightCompact;
	min-width: @_sap_m_GenericTag_BorderWidth + @_sap_m_GenericTag_PaddingLeft + @_sap_m_GenericTag_StatusIconWidth + @_sap_m_GenericTag_IconPaddingRight + @_sap_m_GenericTag_TextMinWidth;

	.sapMGenericTagIcon {
		width: 1rem;
		padding-right: @_sap_m_GenericTag_IconPaddingRight;
	}

	.sapMGenericTagError & {
		border-color: var(--sapErrorBorderColor);

		.sapMGenericTagIcon {
			color: var(--sapNegativeTextColor);
		}
	}

	.sapMGenericTagWarning & {
		border-color: var(--sapWarningBorderColor);

		.sapMGenericTagIcon {
			color: var(--sapCriticalTextColor);
		}
	}

	.sapMGenericTagSuccess & {
		border-color: var(--sapSuccessBorderColor);

		.sapMGenericTagIcon {
			color: var(--sapPositiveTextColor);
		}
	}

	.sapMGenericTagInformation & {
		border-color: var(--sapInformationBorderColor);

		.sapMGenericTagIcon {
			color: var(--sapInformativeTextColor);
		}
	}

	.sapMGenericTagNone & {
		border-color: var(--sapNeutralColor);

		.sapMGenericTagIcon {
			color: var(--sapNeutralTextColor);
		}
	}

	&:hover {
		box-shadow: 0 0 0 0.0625rem fade(@sapUiContentShadowColor, 30);
		cursor: pointer;
	}

	.sapMGenericTagActive & {
		background-color: var(--sapList_Active_Background);

		.sapMGenericTagIcon,
		.sapMGenericTagText,
		.sapMGenericTagValue,
		.sapMGenericTagErrorIcon {
			color: var(--sapList_Active_TextColor);
		}
	}

	.sapMGenericTagErrorIcon {
		overflow: hidden;
		color: var(--sapNegativeTextColor);
		padding-left: 0.5rem;

		&::before {
			line-height: normal;
		}
	}

	.sapMGenericTagText {
		font-family: var(--sapFontFamily);
		font-size: @sapMFontMediumSize;
		font-weight: normal;
		color: var(--sapTextColor);
		white-space: nowrap;
		overflow: hidden;
		text-overflow: ellipsis;
		box-sizing: border-box;
		flex: 1 10000 auto;
		min-width: @_sap_m_GenericTag_TextMinWidth;
	}

	.sapMGenericTagValue {
		display: flex;
		flex: 0 0.5 auto;
		white-space: nowrap;
		overflow: hidden;
		text-overflow: ellipsis;
		box-sizing: border-box;
		align-items: center;
		padding-left: 0.5rem;

		.sapMObjectNumberText {
			min-width: 0.75rem;

			&::after {
				/* simulate ObjectNumber's styling: The ObjectNumber value and unit are inline blocks and */
				/* have an empty space between them. That's because the browser interprets the line-break and spaces */
				/* in the HTML as content. Since we change the styling of the ObjectNumber we must have the same styling. */
				content: '\00a0';
			}
		}

		.sapMObjectNumberUnit {
			min-width: 0.75rem;
		}
	}
}

.sapUiSizeCozy {

	.sapMGenericTag {
		margin: 0.25rem 0;

		.sapMGenericTagWrap {
			height: @_sap_m_GenericTag_HeightCozy;
		}
	}
}
/* END "GenericTag.less" */

/* START "GenericTile.less" */
/* ================================== */
/* CSS for control sap.m/GenericTile  */
/* Base theme                         */
/* ================================== */

.sapMGT {
	display: inline-block;
	text-decoration: none;
	border-radius: var(--sapTile_BorderCornerRadius);
	position: relative;
	outline: none;
	box-sizing: border-box;
	background-position: 0;
	background: var(--sapTile_Background);
	border: 1px solid var(--sapTile_BorderColor);
	box-shadow: @sapUiShadowLevel0;
	vertical-align: top;

		& .sapMNCValue {
				justify-content: flex-start;
		}

		& .sapMNCValue.WithoutMargin {
			justify-content: flex-start;
		}
	}

.sapMGTCriticalBorder {
	height: 100%;
	width: 0.25rem;
	position: absolute;
	left: -1px;
	top: -1px;
	border-radius: 0.25rem 0 0 0.25rem;
	border: 1px solid;
}

.sapMGTCriticalBorder.Neutral {
	background-color: var(--sapNeutralTextColor);
	border-color: var(--sapNeutralTextColor);
}

.sapMGTCriticalBorder.Good {
	background-color: var(--sapPositiveTextColor);
	border-color: var(--sapPositiveTextColor);
}

.sapMGTCriticalBorder.Critical {
	background-color: var(--sapCriticalTextColor);
	border-color: var(--sapCriticalTextColor);
}

.sapMGTCriticalBorder.Error {
	background-color: var(--sapNegativeTextColor);
	border-color: var(--sapNegativeTextColor);
}

.sapMGTContentPlaceholder {
	outline: none;
	width: 300px;
	height: 100px;
	box-shadow: 0 2px 8px 0 rgba(0,0,0,0.10), 0 0 0 1px rgba(0,0,0,0.10);
	border-radius: 4px;
}

.sapMGTContentPlaceholder:focus::before {
	content: '';
	display: block;
	position: absolute;
	left: 1px;
	bottom: 1px;
	right: 1px;
	top: 1px;
	border: 1px dotted var(--sapContent_FocusColor);
	pointer-events: none;
}

.sapMGTContentShimmerPlaceholderItem,
.sapMTileCntContentShimmerPlaceholderItem {
	padding: 1rem 1rem 0 1rem;
	box-sizing: border-box;
	display: flex;
	position: relative;
	align-items: center;
	flex: 1 1 auto;

	&.sapMGTContentShimmerPlaceholderWithChart.sapMGTContentShimmerPlaceholderWithDescription,
	.sapMTileCntContentShimmerPlaceholderWithDescription {
		height: 6rem;
	}

	.sapMGTContentShimmerPlaceholderImg {
		margin: 0 0.75rem 0 0;
		height: 2rem;
		width: 2rem;
		display: inline-block;
	}

	.sapMGTContentShimmerPlaceholderRows,
	.sapMTileCntContentShimmerPlaceholderRows {
		flex-grow: 1;

		.sapMGTContentShimmerPlaceholderItemText {
			height: 0.5rem;
			margin-bottom: 0.5rem;
			width: 80%;
		}

		.sapMGTContentShimmerPlaceholderItemTextFooter,
		.sapMTileCntContentShimmerPlaceholderItemTextFooter {
			height: 0.5rem;
			margin-bottom: 0.5rem;
			margin-top: 0.5rem;
			width: 80%;
			top: 3rem;
		}

		.sapMGTContentShimmerPlaceholderItemBox,
		.sapMTileCntContentShimmerPlaceholderItemBox {
			height: 2rem;
			margin-bottom: 0.5rem;
			margin-top: 0.5rem;
			width: 20%;
			top: 3rem;
		}

		.sapMGTContentShimmerPlaceholderItemBoxSmall {
			height: 3.75rem;
			margin-bottom: 0.5rem;
			margin-top: 0.5rem;
		}

		.sapMGTContentShimmerPlaceholderItemHeader {
			height: 1rem;
			margin-bottom: 0.5rem;
			margin-top: 0.5rem;
		}
	}
}

.sapMGT.OneByOne.sapMTileSmallPhone.sapMGTStateLoading {

	.sapMGTContentShimmerPlaceholderItem {

		.sapMGTContentShimmerPlaceholderItemHeader {
			width: 7.25rem;
		}

		.sapMGTContentShimmerPlaceholderItemText {
			width: 5.875rem;
		}
	}
}

.sapMGTContentShimmerPlaceholderItemOneByOne {
	padding: 1rem 1rem 0 1rem;
	box-sizing: border-box;
	display: flex;
	position: relative;
	align-items: center;
	flex: 1 1 auto;

	&.sapMGTContentShimmerPlaceholderWithChart.sapMGTContentShimmerPlaceholderWithDescriptionOneByOne {
		height: 6rem;
	}

	.sapMGTContentShimmerPlaceholderImg {
		margin: 0 0.75rem 0 0;
		height: 2rem;
		width: 2rem;
		display: inline-block;
	}

	.sapMGTContentShimmerPlaceholderRowsOneByOne {
		flex-grow: 1;

		.sapMGTContentShimmerPlaceholderIconOneByOne {
			height: 2rem;
			width: 2rem;
			margin: auto;
			margin-bottom: 0.5rem;
			margin-top: 0.5rem;
		}

		.sapMGTContentShimmerPlaceholderItemTextOneByOne {
			height: 0.5rem;
			margin-bottom: 0.5rem;
		}
	}
}

.sapMGTContentShimmerPlaceholderItemTwoByHalf {
	padding: 0 1rem 0 0;
	box-sizing: border-box;
	display: flex;
	position: relative;
	align-items: center;
	flex: 1 1 auto;

	.sapMGTContentShimmerPlaceholderWithDescriptionTwoByHalfIconLoaded {
		padding: 0;
		flex: none;
	}

	&.sapMGTContentShimmerPlaceholderWithChart.sapMGTContentShimmerPlaceholderWithDescriptionTwoByHalf {
		height: 6rem;
	}

	.sapMGTContentShimmerPlaceholderImg {
		margin: 0 0.75rem 0 0;
		height: 2rem;
		width: 2rem;
		display: inline-block;
	}

	.sapMGTContentShimmerPlaceholderRowsTwoByHalf {
		flex-grow: 1;
		display: flex;
		align-items: center;

		.sapMGTContentShimmerPlaceholderIconTwoByHalf {
			height: 2rem;
			width: 2rem;
			margin-bottom: 0.5rem;
			margin-top: 0.5rem;
		}

		.sapMGTContentShimmerPlaceholderItemTextTwoByHalf {
			height: 0.5rem;
			width: 80%;
			margin-left: 0.75rem
		}
	}
}

.sapMGTContentShimmerPlaceholderItemTwoByHalf.sapMGTContentShimmerPlaceholderWithDescriptionTwoByHalfIconLoaded {
	padding: 0;
	flex: none;
}

.sapMGTLoadingShimmer,
.sapMTileCntLoadingShimmer{
	.sapMGTLoadingShimmerMixin();
}

.sapMGTLoadingShimmerMixin() {
	background-color: var(--sapContent_Placeholderloading_Background);
	background-image: var(--sapContent_Placeholderloading_Gradient);
	background-repeat: no-repeat;
	background-size: 800px 144px;
	position: relative;
	border-radius: 0.25rem;

	[data-sap-ui-animation-mode='full'] & {
		animation-duration: 2.5s;
		animation-fill-mode: forwards;
		animation-iteration-count: infinite;
		animation-name: placeHolderShimmer;
		animation-timing-function: linear;
	}

	[data-sap-ui-animation-mode='basic'] &,
	[data-sap-ui-animation-mode='minimal'] & {
		animation-duration: 8s;
		animation-fill-mode: forwards;
		animation-iteration-count: infinite;
		animation-name: placeHolderShimmer;
		animation-timing-function: linear;
	}
}

@keyframes placeHolderShimmer {

	0% {
		background-position: -468px 0;
	}

	100% {
		background-position: 468px 0;
	}
}

@keyframes placeHolderShimmer {

	0% {
		background-position: -468px 0;
	}

	100% {
		background-position: 468px 0;
	}
}

.sapMGTBackgroundImage {
	background-clip: padding-box;
	background-color: inherit;
	background-position: inherit;
	background-size: cover;
}

.sapMGT.sapMPointer * {
	cursor: pointer;
}

.sapMGT:focus {
	outline: none;
}

.sapMGT.OneByOne {
	width: 11rem;
	height: 11rem;
}

.sapMGT.TwoByOne {
	width: 22.5rem;
	height: 11rem;
}

.sapMGT.OneByHalf {
	width: 11rem;
	height: 5.25rem;
}

.sapMGT.TwoByHalf {
	width: 22.5rem;
	height: 5.25rem;
}

.sapMGT.sapMGTGridContainerOneRemGap.TwoByOne {
	width: 23rem;
}

.sapMGT.sapMGTGridContainerOneRemGap.TwoByHalf {
	width: 23rem;
	height: 5rem;

	.sapMGTHdrContent.TwoByHalf {
		line-height: normal;
	}

	.sapMGTContent {
		height: 2.6875rem;

		.sapMTileCnt {

			.sapMTileCntContent {
				margin-top: 0.125rem;

				.sapMImageContentImageIcon {
					font-size: 1.625rem;
					max-height: 1.625rem;
				}
			}

			.sapMTileCntFtrTxt {
				bottom: 0.75rem;
			}
		}
	}
}

.sapMGT.sapMGTGridContainerOneRemGap.OneByHalf {
	height: 5rem;

	.sapMGTContent {
		height: 2.6875rem;

		.sapMTileCnt {

			.sapMTileCntContent {
				margin-top: 0.125rem;

				.sapMImageContentImageIcon {
					font-size: 1.625rem;
					max-height: 1.625rem;
				}
			}

			.sapMTileCntFtrTxt {
				bottom: 0.75rem;
			}
		}
	}
}

.sapMTileSmallPhone.sapMGT.sapMGTGridContainerOneRemGap.TwoByOne {
	width: 19.5rem;
}

.sapMTileSmallPhone.sapMGT.sapMGTGridContainerOneRemGap.TwoByHalf {
	width: 19rem;
	height: 4.125rem;

	.sapMGTContent {
		height: 2.375rem;

		.sapMTileCnt {

			.sapMTileCntContent {
				margin-top: 0.25rem;

				.sapMImageContentImageIcon {
					max-height: 1.625rem;
				}
			}
		}
	}
}

.sapMTileSmallPhone.sapMGT.sapMGTGridContainerOneRemGap.OneByHalf {
	width: 9.25rem;
	height: 4.125rem;

	.sapMGTContent {
		height: 2.375rem;

		.sapMTileCntContent {
			margin-top: 0.25rem;
		}
	}
}

.tileWithAppInfo {

	.sapMGTTInfoContainer {
		display: flex;
		align-items: flex-end;
		justify-content: flex-end;
		min-width: 0;
	}

	.sapMGTTInfo {
		padding: 0.125rem 1rem 0.125rem 0.25rem;
		text-transform: uppercase;
		margin-bottom: 0.5rem;
		border-radius: 0.125rem 0 0 0.125rem;
		background-color: var(--sapInfobar_NonInteractive_Background);
		width: fit-content;

		.sapMText{
			font-size: var(--sapFontSmallSize) ;
			text-align: right !important;
		}

		.sapMGTSystemInfoText{
			display: flex;
			justify-content: flex-end;

			.sapMText {
				color: var(--sapTile_TitleTextColor);
				padding-left: 0.15rem;
			}
		}

		.sapMGTAppAndSystemInfoDivider{
			height: 0.25rem;
		}

		.sapMGTAppShortcutText {
			display: flex;
			justify-content: flex-end;

			.sapMText {
				color: var(--sapTile_TextColor);
			}
		}
	}
}

.OneByOne.tileWithAppInfo {

	.sapMGTTInfo {
		margin-bottom: 0.4rem;
		max-width: 7rem;
	}

	.sapMGTContent.appInfoWithFooter {

		.sapMTileCnt {
			height: 3.5rem;
			width: 100%;

			.sapMTileCntContent {

				.sapMNC {
					height: 1.75rem;

					.sapMNCLargeFontSize {
						font-size: 1.75rem;
						height: 1.75rem;

						&::before {
							padding-bottom: 0;
							margin-top: 0.5rem;
						}
					}

					.sapMNCIndScale {
						padding: 0;
						padding-left: 0.5rem;
					}
				}

				.sapMImageContent {
					height: 1.75rem;

					.sapMImageContentImageIcon {
						padding-bottom: 0;
						height: inherit;
						top: 0.1rem;
					}

					.sapMImg.sapMImageContentImageIcon {
						top: 0.2rem;
					}
				}
			}

			.sapMTileCntFtrTxt {
				bottom: -0.15rem;
			}
		}
	}

	.sapMGTContent.appInfoWithoutFooter {

		.sapMTileCnt {
			height: 3.5rem;
			width: 100%;

			.sapMTileCntContent {
				height: 3.25rem;

				.sapMNC {
					height: 3.25rem;
				}

				.sapMImageContent {
					height: 3.25rem;

					.sapMImageContentImageIcon.sapUiIcon {
						font-size: 1.75rem;
						padding-bottom: 0;
						top: 0;
					}

					.sapMImg {
						top: 0.3rem;
					}
				}
			}
		}
	}

	.sapMGTTInfoContainer {
		height: 2.75rem;
	}

	.appInfoWithoutTileCnt {
		display: flex;
		flex-direction: column-reverse;

		.sapMGTTInfo {
			margin-bottom: 0.3rem;
		}
	}
}

.OneByOne.sapMTileSmallPhone.tileWithAppInfo {

	.sapMGTHdrContent.OneByOne {
		height: 3rem;
	}

	.sapMGTContent {

		.sapMGTTInfoContainer{

			.sapMGTTInfo {
				margin-bottom: 0.5rem;
				padding-right: 0.5rem;
			}
		}
	}

	.sapMGTContent.appInfoWithFooter {

		.sapMTileCnt {
			height: 3rem;

			.sapMTileCntContent {

				.sapMImageContent {

					.sapMImageContentImageIcon {
						top: -0.1rem;
					}
				}
			}
		}

		.sapMNCValue {
			margin-bottom: 0.3rem;

			.sapMNCLargeFontSize {
				padding: 0;
			}

			.sapMNCLargeFontSize.sapMNCIconImage {
				padding-right: 0.375rem;
			}
		}
	}

	.sapMGTContent.appInfoWithoutFooter {

		.sapMTileCnt {
			height: 3rem;
		}

		.sapUiIcon {
			top: 0;
		}

		.sapMNCInner.WithoutMargin {
			top: 0;
		}
	}

	.appInfoWithoutTileCnt {

		.sapMGTTInfoContainer {

			.sapMGTTInfo {
				margin-bottom: 0.3rem;
			}
		}
	}

	.sapUiIcon.sapUiIconMirrorInRTL.sapMImageContentImageIcon {
		margin-bottom: 0.25rem;
	}
}

.TwoByHalf.tileWithAppInfo {

	.sapMGTWrapper {
		height: inherit;
		display: flex;
		justify-content: space-between;
		gap: 0.25rem;

		.sapMGTWrapperCnt {

			.sapMGTHdrContent {
				max-width: 13.5rem;
				padding-right: 0;
			}

			.sapMGTContent {

				.sapMTileCnt {

					.sapMTileCntContent {
						padding-right: 0;
					}
				}
			}
		}
	}

	.sapMGTTInfoContainer {
		max-width: 13.5rem;

		.sapMGTTInfo {

			.sapMGTAppShortcutText {

				.sapMText {
					max-width: 13ch;
				}
			}
			.sapMGTSystemInfoText {

				.sapMText {
					max-width: 13ch;
					padding-left: 0.2rem;
				}
			}
		}
	}
}

.TwoByOne.tileWithAppInfo {

	.sapMGTContent {

		.sapMGTTInfoContainer {

			.sapMGTTInfo {
				max-width: 10.875rem;
			}
		}
	}

	.sapMGTContent {
		display: flex;
		justify-content: space-between;
		margin-top: 0.5rem;

		.sapMTileCnt {

			.sapMTileCntContent {
				padding-right: 0;
			}

			.sapMTileCntFtrTxt {
				margin-right: 0;
			}
		}
	}

	.appInfoWithoutTileCnt {
		flex-direction: row-reverse;
	}

	.sapMGTTInfoContainer {
		max-width: 10.875rem;
		margin-left: 0.25rem;
		margin-bottom: 0.5rem;

		.sapMGTTInfo {

			.sapMGTAppShortcutText {
				text-align: right;
			}
		}
	}
}

.OneByHalf {

	.sapMGTHdrContent {
		height: 1.25rem;

		.sapMGTHdrTxt {

			.sapMText {
				display: block;
			}
		}
	}

	.sapMGTContent {
		height: 3rem;

		.sapMTileCnt {
			height: 100%;

			.sapMTileCntContent {
				height: 2.5rem;

				.sapMNCLargeFontSize {
					font-size: 1.5rem;
					padding-bottom: 0.55rem;
					height: auto;
					align-self: flex-end;
				}

				.sapMNCIndScale.sapMNCLargeFontSize {
					padding: 0.375rem 0 0.65rem 0.25rem;
				}

				.sapMImageContent {
					display: none;
				}
			}

			.sapMTileCntContent > div:first-child {
				height: 100%;
			}
		}
	}
}

.TwoByHalf {

	.sapMGTHdrContent {
		height: 1.25rem;

		.sapMGTHdrTxt {

			.sapMText {
				display: block;
			}
		}
	}

	.sapMGTContent {
		height: 3rem;

		.sapMTileCnt {
			height: 100%;

			.sapMTileCntContent {
				height: 2.5rem;

				.sapMNCLargeFontSize {
					font-size: 1.5rem;
					padding-bottom: 0.55rem;
					height: auto;
					align-self: flex-end;
				}

				.sapMNCIndScale.sapMNCLargeFontSize {
					padding: 0.375rem 0 0.65rem 0.25rem;
				}

				.sapMImageContentImageIcon {
					max-height: 1.75rem;
					bottom: auto;
				}
			}

			.sapMTileCntContent > div:first-child {
				height: 100%;
			}
		}
	}
}

.sapMGTHdrTxt,
.sapMGTHdrTxt > .sapMGTTitle,
.sapMGTSubHdrTxt,
.sapMGT .sapMGTContent {
	user-select: none;
	-webkit-user-select: none;
	-moz-user-select: -moz-none;
}

.sapMGTHdrTxt > .sapMGTTitle,
.sapMGenericTileFtrFldTxt>.sapMGTFailed,
.sapMTileCntFtrFldTxt>.sapMGTFailed {
	cursor: inherit;
}

.sapMGTInnrCnt {
	display: inline-block;
}

.sapMGTContent {
	height: 6.375rem;
	display: inline-block;
	white-space: nowrap;
	width: 100%;
	overflow: hidden;
}

.sapMGTHdrTxt {
	overflow: hidden;

	.sapMText {
		display: inline;
	}
}

/* added line height so that it is not overridden by other controls. */
.sapMGT {

	.sapMText {
		line-height: normal;
	}
}

.sapMGTHdrTxt > .sapMText {
	color: var(--sapTile_TitleTextColor);
	font-weight: normal;
	font-family: var(--sapFontFamily);
	font-size: var(--sapFontHeader5Size);
}

.sapMGTSubHdrTxt > .sapMText,
.sapMGTSubHdrTxt {
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	font-weight: normal;
	color: var(--sapTile_TextColor);
}

.sapMGTHdrContent {
	height: 3.625rem;
	padding-top: 1rem;
	padding-left: 1rem;
	padding-right: 1rem;
}

.OneByOne.tileWithAppInfo {

	.sapMGTHdrContent.OneByOne {
		height: 3.4rem;
	}
}

.sapMGTHdrContent.Stretch {
	height: 42%;
	padding-top: 0;
}

.sapMGT.sapMGTBackgroundImage.Stretch {

	.sapMGTContent {
		height: 58%;
		margin-top: 1px;
	}
}

.sapMGTHdrIconImage {
	float: right;
	width: 2.5rem;
	min-height: 2.5rem;
	font-size: 2.5rem;
	font-family: var(--sapFontHeaderFamily);
	font-weight: @sapUiFontHeaderWeight;
	color: var(--sapTile_IconColor);
}

.sapMGenericTileFtrFldTxt,
.sapMTileCntFtrFldTxt {
	font-family: var(--sapFontFamily);
	font-size: var(--sapFontSmallSize);
	color: var(--sapTile_TextColor);
}

.sapMGenericTileFtrFldIcn,
.sapMTileCntFtrFldIcn {
	color: var(--sapTile_TextColor);
}

.sapMGT:focus:not([tabindex]) {
	border-color: transparent;
}

.sapMGTOverlay {
	background-color: var(--sapTile_Background);
	border-radius: var(--sapTile_BorderCornerRadius);
}

.sapMGTOverlayMixin(@offset: 0px, @z-index: 0) {
	position: absolute !important;
	pointer-events: none;
	left: @offset;
	top: @offset;
	right: @offset;
	bottom: @offset;

	& when not (@z-index = 0) {
		z-index: @z-index;
	}
}

.sapMGT.sapMGTScopeActions,
.sapMGT.sapMGTScopeSingleAction {

	.sapMTileCntContent {
		position: relative;

		&::after {
			.sapMGTOverlayMixin();
			display: block;
			content: '';
			background-color: var(--sapTile_Background);
			opacity: 0.7;
		}
	}

	.sapMGTMoreIcon {
		height: 1rem;
		position: absolute;
		right: 0.75rem;
		bottom: 0.563rem;
		z-index: 1; /* Remark: If increased, this will break FLP footer visualization. */
		color: var(--sapButton_IconColor);

		&:focus {
			outline: none;
		}
	}

	.sapMGTMoreIcon {
		background-color: transparent;
		border: 0;

		.sapMBtnIcon {
			bottom: 1rem;
			left: 0.4rem;
			border: 0;
		}
	}

	.sapMGTRemoveButton {
		width: 1.375rem;
		height: 1.375rem;
		border-radius: 50%;
		padding: 0;
		position: absolute;
		top: -0.375rem;
		right: -0.375rem;
		z-index: 3; /* Remark: If increased, this will break FLP footer visualization. */
		background-color: var(--sapButton_Background);
		color: var(--sapButton_TextColor);
		overflow: visible;

		.sapMBtnFocusDiv {
			display: none;
		}

		.sapMBtnInner,
		.sapMBtnActive.sapMBtnInner {
			width: 100%;
			height: 100%;
			min-width: 0;
			outline-offset: -2px;

			&,
			&:hover,
			&:active,
			&:focus {
				border-color: var(--sapTile_Interactive_BorderColor); /* Explicitly needed here, because the border would otherwise be transparent in HCB on press */
				border-radius: 50% !important; /* Made important because the style is not to be overridden in any way */
			}
		}

		.sapMBtnIcon {
			font-size: 0.75rem;
			margin: 0;
			width: 100%;

			&::before {
				position: relative;
				top: -0.15rem;
				left: 0;
			}
		}

		&:focus > .sapMFocusable {
			outline: none;
			border-radius: 50%;
		}

		/* The touch area has a different size and is slightly moved to the left and bottom */
		&::before {
			content: '';
			color: var(--sapButton_IconColor);
			position: absolute;
			top: -1px;
			bottom: -0.6875rem;
			left: -0.6875rem;
			right: -0.0625rem;
		}
	}
}

.sapUiSizeCompact {

	.sapMGT.sapMGTScopeActions,
	.sapMGT.sapMGTScopeSingleAction {

		.sapMGTMoreIcon {

			.sapMBtnIcon {
				left: -0.3rem;
				bottom: 0.5rem
			}
		}
	}
}

.sapMGT.sapMGTIconMode {

	.sapUiIcon.sapMGTTileIcon {
		text-shadow: var(--sapContent_ContrastTextShadow);
	}
}

.sapMGT.sapMGTScopeActions.sapMGTIconMode {

	.sapMBtn.sapMGTMoreIcon {
		padding: 0;
		height: 0;
		bottom: auto;
		right: 0.5rem;

		&::before {
			content: '';
			position: absolute;
			top: -0.25rem;
			bottom: -2.5rem;
			left: -0.0125rem;
			right: -0.0125rem;
		}

		.sapMBtnIcon {
			bottom: 0;
			left: 0;
			padding-left: 0.625rem;
			padding-right: 0.625rem;
			width: 1rem;
		}
	}
}

.sapUiSizeCompact {

	.sapMGT.sapMGTScopeActions.sapMGTIconMode.TwoByHalf.sapMTileSmallPhone {

		.sapMBtn.sapMGTMoreIcon {
			top: 1.4rem;

			&::before {
				content: '';
				position: absolute;
				top: -0.1975rem;
				bottom: -1.8875rem;
				left: -0.0125rem;
				right: -0.0125rem
			}

			.sapMBtnInner {
				padding: 0;
				min-width: 0;
			}
		}
	}
}

.sapUiSizeCompact {

	.sapMGT.sapMGTScopeActions.sapMGTIconMode.TwoByHalf {

		.sapMBtn.sapMGTMoreIcon {
			top: 0.9rem;

			.sapMBtnIcon {
				padding-left: 0.5rem;
				padding-right: 0.5rem;
			}
		}
	}
}

.sapMGT.sapMGTScopeActions.sapMGTIconMode.TwoByHalf {

	.sapMBtn.sapMGTMoreIcon {
		top: 0.6rem;

		.sapMBtnInner {
			left: 0.2rem;
			padding: 0;
		}
	}
}

.sapMGT.sapMGTScopeActions.sapMGTIconMode.TwoByHalf.sapMTileSmallPhone {

	.sapMBtn.sapMGTMoreIcon {
		top: 1.1rem;

		.sapMBtnInner {
			left: 0;
		}
	}
}


.sapMGT.sapMGTScopeActions.sapMGTBackgroundImage,
.sapMGT.sapMGTScopeSingleActions.sapMGTBackgroundImage {

	.sapMTileCntContent {

		&::after {
			content: none;
		}
	}

	.sapMGTContent {

		&::before {
			.sapMGTOverlayMixin();
			display: block;
			content: '';
			background-color: var(--sapTile_Background);
			opacity: 0.7;
			border-radius: 0.125rem;
		}
	}
}

/* mixin for flex-box entries */
.sapMGTFlexEntry(@resizable: 0) {
	display: flex;
	align-items: center;
	-webkit-box-flex: @resizable;
	flex: @resizable @resizable auto;
	-webkit-flex: @resizable @resizable auto;
}

/* LINE MODE LIST VIEW */

.sapMGTLineModeListContainer {
	background-color: var(--sapGroup_ContentBackground);
	border: var(--sapTile_BorderColor);
	border-radius: 0.25rem;
	box-shadow: @sapUiShadowLevel0;
}

/* COZY */
.sapUiMedia-GenericTileDeviceSet-small {

	.sapMGTLineMode.sapMGT {
		align-items: center;
		padding: 0.5rem;
		min-height: 2rem;
		max-width: 100%;
		border-radius: 0.25rem;
		transition: background-color 0.2s;
		pointer-events: none;
	}

	.sapMGTLineMode .sapMGTTouchArea {
		display: flex;
		flex-direction: row;
		align-items: center;
		width: 100%;
		pointer-events: auto;
		padding: 0;
		margin: -0.1875rem 0;
		box-sizing: border-box;
	}

	.sapMGTLineMode .sapMGTLineModeHelpContainer {
		.sapMGTFlexEntry(1);
		flex-wrap: wrap;
		overflow: hidden;
	}

	.sapMGTLineMode .sapMGTHdrTxt {
		position: relative;
		font-size: @sapMFontMediumSize;
		display: inline-block;
		white-space: nowrap;
		text-overflow: ellipsis;
		max-width: 100%;
		line-height: 1.375rem;
	}

	.sapMGTLineMode .sapMGTHdrTxt::after {
		content: ' ';
		margin-right: 0.5rem;
	}

	.sapMGTLineMode .sapMGTSubHdrTxt {
		display: inline-block;
		font-size: @sapMFontMediumSize;
		line-height: 1.375rem;
		margin-top: 0;
		max-width: 100%;
	}

	.sapMGTLineMode .sapMGTLineModeFailedIcon {
		align-self: baseline;
		line-height: 1.375rem;
		padding-right: 0.375rem;
		text-align: left;
		font-size: 1.375rem !important;
	}
}

.sapMGTLineMode .sapMGTActionsContainer:not(.sapMGTScopeSingleActionContainer) {
	width: 2rem;

	.sapMGTMoreIcon {
		position: static;
		padding-right: 0.5rem;
	}

	.sapMGTRemoveButton {
		position: absolute;
		top: -1rem;
		left: 1.5rem;
	}
}

.sapMGTLineMode .sapMGTTouchArea .sapMGTScopeSingleActionContainer {

	.sapMGTMoreIcon {
		position: static;
	}

	.sapMGTRemoveButton {
		position: absolute;
		top: -1rem;
		left: 1.5rem;
	}
}

/* ActionsContainer Styles commonly shared with all screen sizes in compact and cozy */
.sapMGTLineMode .sapMGTActionsContainer,
.sapMGTLineMode .sapMGTScopeSingleActionContainer {
	.sapMGTFlexEntry;
	display: flex;
	flex-direction: row;
	align-items: center;
	line-height: 1.375rem;

	&,
	* {
		pointer-events: auto;
	}

	.sapMGTMoreIcon {
		display: block;
		right: -1.5rem;
		bottom: 0.25rem;
		width: 2rem;
	}

	.sapMGTRemoveButton {
		display: block;
		margin: 0 0.3125rem;
		top: -1rem;
		left: -0.5rem;
		z-index: 0; /* Remark: If increased, this will break FLP footer visualization. */

		&::before {
			content: none;
		}

		.sapMBtnIcon::before {
			width: 1.25rem;
			height: 1.5rem;
			padding: 0.75rem 0.3125rem;
			margin-top: -0.75rem;
			left: -0.3rem;
			display: inline-block;
		}
	}
}

.sapMGTLineMode.sapMGTScopeSingleAction {

	.sapMGTMoreIcon.sapMBtnBase {

		.sapMBtnIcon {
			left: 0;
			bottom: 0.6rem;
		}
	}
}

.sapUiSizeCompact {

	.sapMGTLineMode.sapMGTScopeSingleAction {

		.sapMGTMoreIcon.sapMBtnBase {

			.sapMBtnIcon {
				left: 0.5rem;
				bottom: 0.3rem;
			}
		}
	}
}

.sapMGTLineMode.sapMGTScopeActions {

	.sapMGTMoreIcon.sapMBtnBase {

		.sapMBtnIcon {
			bottom: 0.6rem;
			left: -1rem;
		}
	}
}

.sapUiSizeCompact {

	.sapMGTLineMode.sapMGTScopeActions {

		.sapMGTMoreIcon.sapMBtnBase {

			.sapMBtnIcon {
				bottom: 0.3rem;
				left: -0.5rem;
			}
		}
	}
}

.sapUiSizeCompact {

	.sapMGTLineMode .sapMGTActionsContainer {

		.sapMGTRemoveButton .sapMBtnIcon::before {
			padding: 0.25rem 0.3125rem;
		}
	}
}

.sapMGTLineMode .sapMGTLineModeFailedIcon {
	text-align: start;
}

.sapMGT.sapMGTScopeActions.sapMGTLineMode.sapMGTFailed,
.sapMGT.sapMGTScopeSingleAction.sapMGTLineMode.sapMGTFailed {

	.sapMBtn.sapMGTRemoveButton {
		opacity: 1 !important;
	}
}

/* COMPACT */
.sapUiMedia-GenericTileDeviceSet-small {

	&.sapUiSizeCompact,
	.sapUiSizeCompact {

		.sapMGTLineMode.sapMGT {
			min-height: 1.625rem;
		}

		.sapMGTLineMode .sapMGTTouchArea {
			padding: 0.125rem 0;
			margin: -0.1875rem 0;
		}

		.sapMGTLineMode .sapMGTLineModeFailedIcon {
			font-size: 0.875rem !important;
		}
	}
}

/* LINE MODE FLOATING VIEW */
.sapMGTLineModeFloatingContainer {
	background-color: var(--sapTile_Background);
	border: var(--sapTile_BorderColor);
	border-radius: 0.25rem;
	box-shadow: @sapUiShadowLevel0;
}

/* COZY */
.sapUiMedia-GenericTileDeviceSet-large {

	&.sapMGTLineModeContainer,
	.sapMGTLineModeContainer {
		padding-right: 1.0rem;
		box-sizing: border-box;
		position: relative; /* make the tile's parent <<positioned>>, in order to make JS get the correct offsetParent */
	}

	.sapMGT.sapMGTLineMode {
		user-select: none;
		-webkit-user-select: none;
		-moz-user-select: -moz-none;
		-webkit-text-size-adjust: 100%;
	}

	.sapMGT.sapMGTLineMode {
		float: none;
		clear: none;
		display: inline-block;
		min-width: 2rem;
		background-color: var(--sapTile_Background);
		border: var(--sapTile_BorderColor);
		border-radius: 0.25rem;
		box-shadow: @sapUiShadowLevel0;
		position: relative;
		padding: 0.5rem 0.5rem 0.5rem 0;
		@line-height: 2.75rem;
		@padding-top: 0.125rem;
		@padding-bottom: 0.125rem;
		min-height: 2rem; /* used as hover-style bar height */
		pointer-events: none;
	}

	.sapMGTLineMode .sapMGTHdrTxt,
	.sapMGTLineMode .sapMGTSubHdrTxt {
		word-break: normal;
		display: inline;
		white-space: normal;
		position: relative;
		left: 0.5rem;
		top: 1px;
		border-top: 2px solid transparent;
		border-bottom: 2px solid transparent;
		z-index: 2;
		padding-right: 0.5rem;

		&::after {
			content: none;
		}
	}

	.sapMGTLineMode .sapMGTSubHdrTxt {

		&::before { /* this is needed to enable line-wrapping between header end and subheader beginning */
			content: '';
			display: inline-block;
		}
	}

	.sapMGTLineMode .sapMGTLineModeFailedIcon {
		display: inline-block;
		vertical-align: middle;
		padding-right: 0;
		position: relative;
		left: 0.5rem;
		border-right: 0.375rem solid transparent;
		box-sizing: border-box;
		font-size: 0.875rem  !important;
	}

	.sapMGTLineMode .sapMGTHdrTxt {
		border-left: 0.5rem solid transparent;
		margin-left: -0.5rem;
	}

	.sapMGTLineMode .sapMGTHdrTxt:last-of-type {
		padding-right: 0.5rem;
		margin-right: 0.5rem;
	}

	.sapMGTLineMode.sapMGTScopeSingleAction {

		.sapMGTHdrTxt:last-of-type,
		.sapMGTSubHdrTxt {
			padding-right: 1.3rem;
		}
	}

	.sapMGTLineMode.sapMGTScopeActions {

		.sapMGTHdrTxt:last-of-type,
		.sapMGTSubHdrTxt {
			padding-right: 3.5rem;
		}
	}

	.sapMGTActionsContainer {
		position: absolute;
		right: 0.1875rem; /* together with RemoveButton's margin, this fulfills the right-padding of 0.5rem */
		bottom: 0;
		line-height: 1.375rem;
		z-index: 2; /* Remark: If increased, this will break FLP footer visualization. */
		padding: 0;
		height: 1.5rem;

		&,
		* {
			pointer-events: auto;
		}
	}

	.sapMGTStartMarker,
	.sapMGTEndMarker {
		display: inline-block;
		position: relative;
		top: 0.5rem;
		z-index: 2; /* Remark: If increased, this will break FLP footer visualization. Changed from 3 to 2 as it was breaking FLP footer visualization */
	}

	.sapMGTStyleHelper {
		position: absolute;
		top: 0;
		width: 100%;
	}

	.sapMGTLineStyleHelper {
		position: absolute;
		pointer-events: auto;

		.sapMGTLineStyleHelperInner {
			.sapMGTOverlayMixin();
		}

		&:not(:last-child) {
			border-bottom: 0.25rem solid transparent;
		}
	}

	.sapMGTLineMode .sapMGTHdrTxt {
		max-height: none !important;
	}
}

/* COMPACT */
.sapUiMedia-GenericTileDeviceSet-large {

	&.sapUiSizeCompact,
	.sapUiSizeCompact {

		.sapMGT.sapMGTLineMode {
			@line-height: 1.875rem;
			@padding-top: 0.125rem;
			@padding-bottom: 0.125rem;
			min-height: 2rem;
			padding: 0.5rem 0.5rem 0.5rem 0;
		}

		.sapMGTLineMode .sapMGTLineModeFailedIcon {
			display: inline-block;
			vertical-align: middle;
			padding-right: 0;
			margin-top: 0;
			position: relative;
			left: 0.5rem;
			border-right: 0.375rem solid transparent;
			box-sizing: border-box;
			font-size: 0.875rem  !important;
		}
	}
}

/* Fix for rendering order on tablets in RTL */
html[dir=rtl] .sapMGTLineMode {

	.sapMGTHdrTxt,
	.sapMGTSubHdrTxt {
		unicode-bidi: -webkit-isolate; /* needs to be prefixed for older webkit browsers */
	}
}

/* OVERLAYS AND HELPERS */
.sapMGTOverlay {
	.sapMGTOverlayMixin();
		background-color: var(--sapTile_Background);
		opacity: 0.7;
}

.sapMGT .sapMGTFocusDiv {
	.sapMGTOverlayMixin(0px, 2);
	border-radius: 0.25rem;
}
.sapMGT.sapMGTBackgroundImage .sapMGTFocusDiv {
	.sapMGTOverlayMixin(1px, 2);
}

.sapMGTLineMode .sapMGTFocusDiv {
	top: 1px;
	right: 1px;
	left: 1px;
	bottom: 1px;
	border-radius: 0.25rem !important;
}

.sapMGenericTileFtrFld,
.sapMTileCntFtrFld {
	left: 0.4375rem;
	right: 0.4375rem;
	bottom: 0.75rem;
	position: absolute;
}

.sapMGenericTileFtrFldTxt,
.sapMTileCntFtrFldTxt {
	font-weight: normal;
	display:table-cell;
	padding-left: 0.5rem;
	vertical-align: bottom;
	width: 100%;
}

html[dir=rtl] .sapMGenericTileFtrFldTxt,
.sapMTileCntFtrFldTxt {
	padding-right:0.5rem;
}

.sapMGenericTileFtrFldIcn,
.sapMTileCntFtrFldIcn {
	margin-right: 0.5rem;
	display: table-cell;
	vertical-align: bottom;
}

.sapMGenericTileFtrFldIcn > .sapMGTFtrFldIcnMrk,
.sapMTileCntFtrFldIcn > .sapMGTFtrFldIcnMrk {
	vertical-align: bottom;
}

/* the following is needed for non-webkit browsers that don't support line-clamping */

/* some places should be left for subheader when the subheader exists */
.sapMGT.sapMGTHeaderMode .sapMGTHdrTxt:nth-last-of-type(2) {
	max-height: 5.48rem;
}

.sapMGT:not(.sapMGTHeaderMode) .sapMGTHdrTxt:nth-last-of-type(2) {
	max-height: 2.74rem;
}

/* the max-height is calculated the same as webkit-line-clamping and supported to work when
the screen size changes between small and large (e.g. mobile devices auto rotation) without rerendering */
html[data-sap-ui-browser*='ff'] {

	.sapMGT.sapMGTHeaderMode .sapMGTHdrContent .sapMTextMaxLine {
		max-height: 6.85rem !important;
	}
}

.sapMTileSmallPhone,
.sapMTResponsiveIconMode {

	&.sapMGT.sapMGTHeaderMode .sapMGTHdrTxt:nth-last-of-type(2) {
		max-height: 4.72rem;
	}

	&.sapMGT:not(.sapMGTHeaderMode) .sapMGTHdrTxt:nth-last-of-type(2) {
		max-height: 2.36rem;
	}

	html[data-sap-ui-browser*='ff'] {

		&.sapMGT.sapMGTHeaderMode > .sapMGTHdrContent .sapMTextMaxLine {
			max-height: 5.9rem !important;
		}
	}

	html[data-sap-ui-browser*='ff'] {

		&.sapMGT:not(.sapMGTHeaderMode) > .sapMGTHdrContent .sapMTextMaxLine {
			max-height: 3.54rem !important;
		}
	}

	&.sapMGT.OneByOne {
		width: 9.25rem;
		height: 9.25rem;
	}

	&.sapMGT.TwoByOne {
		width: 19rem;
		height: 9.25rem;
	}

	&.sapMGT.OneByHalf {
		width: 9.25rem;
		height: 4.375rem;

		.sapMGTHdrContent {
			height: 1.25rem;
		}

		.sapMGTContent {
			height: 2.625rem;

			.sapMTileCnt {
				height: 2.125rem;

				.sapMTileCntContent {
					height: 100%;

					.sapMNCLargeFontSize {
						font-size: 1.5rem;
						height: auto;
						align-self: flex-end;
						padding-bottom: 0.25rem;
					}

					.sapMNCIndScale.sapMNCLargeFontSize {
						padding: 0.25rem 0 0.25rem 0.25rem;
					}

					.sapMImageContent {
						display: none;
					}
				}

				.sapMTileCntFtrTxt {
					margin-top: 0.25rem;
					bottom: 0;
				}
			}
		}
	}

	&.sapMGT.TwoByHalf {
		width: 19rem;
		height: 4.375rem;

		.sapMGTHdrContent {
			height: 1.25rem;
			width: calc(100% - 20%);
		}

		.sapMGTContent {
			height: 2.625rem;

			.sapMTileCnt {
				height: 2.125rem;

				.sapMTileCntContent {
					height: 100%;

					.sapMNCLargeFontSize {
						font-size: 1.5rem;
						align-self: flex-end;
						height: auto;
						padding-bottom: 0.25rem;
					}

					.sapMNCIndScale.sapMNCLargeFontSize {
						padding: 0.25rem 0 0.25rem 0.2	5rem;
					}

					.sapMImageContentImageIcon {
						height: 100%;
						max-height: 1.75rem;
					}
				}

				.sapMTileCntContent > div:first-child {
					height: 100%;
				}

				.sapMTileCntFtrTxt {
					margin-top: 0.25rem;
					bottom: 0;
				}
			}
		}
	}

	.sapMGTContent {
		height: 5.5rem;
	}

	.sapMGTHdrContent {
		height: 3.25rem;
		padding: 0.5rem 0.5rem 0 0.5rem;
	}

	.sapMGTSubHdrTxt {
		margin-top: 0;
	}

	.sapMGT.sapMGTScopeActions,
	.sapMGT.sapMGTScopeSingleAction {

		.sapMGTMoreIcon {
			right: 0.5rem;
			bottom: 0.313rem;
		}
	}

	.sapMGTHdrTxt > .sapMText {
		font-size: var(--sapFontHeader6Size);
	}
	.sapMGTSubHdrTxt,
	.sapMGTSubHdrTxt > .sapMText {
		font-size: var(--sapFontSmallSize);
	}

	.sapMGT.sapMGTBackgroundImage.Stretch {

		.sapMGTContent {
			height: 66%;
		}
	}
}

.sapTilePaddingTest .sapMTileCntContent {
	margin-top: 0.3125rem;
	margin-bottom: 0.3125rem;
	padding-left: 1rem;
	padding-right: 1rem;
}

.sapTilePaddingTest .sapMGTHdrContent {
	padding-top: 0.6875rem;
	padding-left: 1rem;
	padding-right: 1rem;
}

.sapTilePaddingTest .sapMTileCntFtrTxt {
	margin-left: 1rem;
	margin-right: 1rem;
	bottom: 1.0625rem;
}

.TwoByOne.sapMGTActionMode {
	height: 10.5rem;
	width: 20rem;
	box-shadow: var(--sapContent_Shadow0);

	.sapMGTTitle {
		font-weight: bold;
	}

	.sapMGTHdrContent.sapMGTHdrImage {
		display: flex;
		flex-direction: row;
		align-items: center;
		gap: 0.75rem;
		height: 2rem;

		.sapMGTHdrIconImage {
			display: flex;
			align-items: center;
		}

		.sapMGTHdrTxt {
			width: 14.25rem;
		}
	}

	.sapMGTHdrContent {
		padding: 0;
		margin: 1rem 1rem 0.75rem 1rem;
		height: fit-content;
		overflow: hidden;

		.sapMText {
			line-height: 1rem;
			font-size: var(--sapFontSize);
		}

		.sapMGTSubHdrTxt .sapMText {
			font-size: var(--sapFontSize);
		}

		.sapMGTHdrIconImage {
			float: left;
			width: 3rem;
		}

		.sapMGTHdrIconImage.sapUiIcon {
			width: 2rem;
			line-height: 1.5rem;
			min-height: 2.1875rem;
		}

		.sapMGTHdrIconImage.sapUiIcon::before {
			font-size: 2rem;
		}

		.sapMGTHdrIconImage.Neutral {
			color: var(--sapNeutralTextColor);
		}

		.sapMGTHdrIconImage.Good {
			color: var(--sapPositiveTextColor);
		}

		.sapMGTHdrIconImage.Critical {
			color: var(--sapCriticalElementColor);
		}

		.sapMGTHdrIconImage.Error {
			color: var(--sapNegativeTextColor);
		}

		.sapMGTHdrIconImage.None {
			color: var(--sapTile_IconColor);
		}
	}

	.sapMGTContent.sapMGTFtrMarginTop {
		height: 7.6rem;

		.sapMTileCnt {
			height: 7.6rem;
		}
	}

	.sapMGTContent {
		height: 6.6rem;

		.sapMTileCnt {
			height: 6.6rem;

			.sapMTileContainer {
				display: flex;
				align-items: center;

				.sapMTilePriority {
					margin: 0 0.5rem 0 1rem;

					.sapMTilePriorityCnt {
						display: flex;
						align-items: center;
						width: fit-content;
						overflow: hidden;

						.sapMTilePriorityBorder {
							width: 0.375rem;
							height: 1.625rem;
						}

						.sapMTilePriorityValue {
							font-size: 0.875rem;
							font-weight: 400;
							line-height: 1rem;
						}
					}
				}

				.High {

					.sapMTilePriorityCnt {
						color: var(--sapNegativeElementColor);
					}
				}

				.Medium {

					.sapMTilePriorityCnt {
						color: @sapWarningColor;
					}
				}

				.Low {

					.sapMTilePriorityCnt {
						color: var(--sapPositiveElementColor);
					}
				}

				.VeryHigh {

					.sapMTilePriorityCnt {
						color: var(--sapIndicationColor_1_BorderColor);
					}
				}

				.sapMTileCntContent {
					padding: 0;
					margin: 0 1rem 0 0;

					.sapMText {
						line-height: 1.5rem;
					}

					.sapMNC {

						.sapMNCInner {
							position: relative;
						}
					}
				}
			}

			.sapMTileCntContent {

				&:first-child {
					color: var(--sapTile_TextColor);
				}

				height: 3rem;
				overflow: hidden;
				padding: 0 1rem;
				margin: 0.75rem 0 1rem 0;

				p {
					margin: 0;
					line-height: 1rem;
					color: var(--sapTile_TextColor);
					white-space: nowrap;
					overflow: hidden;
					text-overflow: ellipsis;
				}

				.sapMText {
					line-height: 1rem;
					color: var(--sapTile_TextColor);
				}

				.sapMImageContent {
					height: 3rem;

					.sapMImageContentImageIcon {
						max-height: 3rem;
					}
				}
			}

			.sapMTileFtrCnt {
				height: 1rem;
				display: flex;
				position: absolute;
				left: 0;
				right: 0;
				bottom: 0;
				justify-content: space-between;
				align-items: center;
				padding-bottom: 1rem;

				.sapMTileCntFtrTxt {
					position: initial;
					margin-right: 0;
					margin-left: 1rem;
					font-size: var(--sapFontSize);
					margin-top: 1rem;
				}
			}

			.sapMTileCntFtrTxt {
				margin: 0 1rem 1rem 1rem;
				font-size: var(--sapFontSize);
				bottom: 0;
			}
		}

		.sapMTileCnt.sapMGTNoPriority {

			.sapMTileCntContent {
				margin: 0;
			}
		}

		.sapMTileCnt.sapMGTPriority {

			.sapMTileCntContent {
				height: 2rem;
			}
		}
	}

	&.sapMGTStateLoading {

		.sapMTileCnt {

			.sapMTileCntContentShimmerPlaceholderItem {
				padding-top: 0;
			}
		}

	}

	.sapMGTActionModeContainer {
		position: absolute;
		bottom: 0.9em;
		right: 1rem;
	}
}

.TwoByOne.sapMTileSmallPhone.sapMGTActionMode {
	height: 11.375rem;
	width: 19rem;

	.sapMGTContent {
		height: 7.6rem;

		.sapMTileCnt {
			height: 7.6rem;
		}
	}

	.sapMGTContent.sapMGTFtrMarginTop {
		height: 8.6rem;

		.sapMTileCnt {
			height: 8.6rem;
		}
	}

	&.sapMGTStateLoading {

		.sapMTileCnt {

			.sapMTileCntContentShimmerPlaceholderItem {
				padding-top: 2rem;
			}
		}

	}
}

.TwoByOne.sapMGTLinkTileContent {

	.sapMGTHdrContent {
		padding: 0;
		margin: 1rem 1rem 0.75rem 1rem;
		height: fit-content;

		.sapMGTHdrTxt {

			.sapMText {
				line-height: 1rem;
				font-size: var(--sapFontSize);
				font-weight: bold;
			}
		}
	}

	.sapMGTContent {
		height: 6.25rem;

		.sapMGTLinkTileContentWrapper {
			height: 7.15rem;
			display: flex;
			flex-direction: column;

			.sapMHBox {
				height: 1.5rem;

				.sapMFlexItem {
					height: 1.5rem;
					display: flex;
					justify-content: center;
					align-items: center;

					.sapMLnk {
						line-height: 1.5rem;
					}
				}
			}
		}

		&.sapMGTFtrMarginTop {
			height: 7.25rem;
		}

		&:not(.sapMGTFtrMarginTop) {

			.sapMGTLinkTileContentWrapper.saMGTLinkSubheaderNotPresent {
				gap: 2.5px;
			}
		}
	}
}

.TwoByOne.sapMGTLinkTileContent.sapMTileSmallPhone {

	.sapMGTHdrContent {
		margin: 0.5rem;
	}

	.sapMGTContent {
		height: 5.75rem;

		.sapMGTLinkTileContentWrapper {
			gap: 0 !important;
		}

		&.sapMGTFtrMarginTop {
			height: 6.6875rem;

			.sapMGTLinkTileContentWrapper {
				gap: 0.3125rem !important;
			}
		}
	}
}

.sapMAutoPointer {

	.sapMText {
		cursor: auto;
	}
}

.sapMGT.sapMGTOneByOne,
.sapMGT.sapMGTOneByOne.sapMTileSmallPhone {

	.sapMBtnBase.sapMGTMoreIcon {

		.sapMBtnIcon {
			bottom: 0.3rem;
		}
	}
}

@media (min-width:320px) and (max-width:600px) {

	.sapMGT.sapMGTOneByOne {
		width: 12.875rem;
		height: 7.8125rem;

		.sapMGTHideOverflow {
			bottom: 0.6875rem;
		}

		.sapMGTSubHdrTxt {

			.sapMTextMaxLine {
				-webkit-line-clamp: 1 !important;
			}
		}

		&.HighContrastTile {

			.sapMGTHideOverflow {
				bottom: 0.5rem;
			}
		}
	}
	.sapMBtnBase.sapMGTMoreIcon{
		.sapMBtnIcon{
			bottom: 0.7rem;
		}
	}
}

@media (min-width:600px) {

	.sapMGT.sapMGTOneByOne {
		width: 11.875rem;
		height: 8rem;

		.sapMBtnBase.sapMGTMoreIcon {

			.sapMBtnIcon {
				bottom: 0.3rem;
			}
		}

		.sapMGTHideOverflow {
			bottom: 1rem;
		}

		&.HighContrastTile {

			.sapMGTHideOverflow {
				bottom: 0.75rem;
			}
		}
	}
}

.sapMGT.sapMGTOneByOne {
	border-radius: var(--sapTile_BorderCornerRadius);
	border: none;

	.sapMGTHideOverflow {
		position: absolute;
		top: 0;
		right: 0;
		left: 0;
		overflow: hidden;
	}

	.sapMGTIconWrapper {
		display: flex;
		flex-direction: column;
		gap: 0.375rem;
		margin-bottom: 0.375rem;

		.sapMGTHdrContent {
			padding: 0 0.5rem;
			display: block;
			flex-direction: column;
			height: fit-content;
			align-items: center;

			.sapMGTHdrTxt {
				height: inherit;
				max-height: none;
				overflow: visible;
				justify-content: center;
				word-break: break-word;
			}
		}
	}

	.sapMGTSubHdrTxt {
		margin: 0 0.5rem 0 0.5rem;
		display: flex;
		align-items: center;
		flex-direction: column;
		white-space: initial;

		.sapMText {
			font-weight: 400;
			color: var(--sapContent_ContrastTextColor);
			text-align: center !important;
			white-space: pre-line;
			line-height: 1rem;
		}
	}

	&.sapMGTHeaderOneLine {

		.sapMGTHdrContent.sapMGTOneByOne {
			justify-content: end;

			.sapMGTHdrTxt {
				height: fit-content;
			}
		}
	}

	&.sapMGTHeaderThreeLine {

		.sapMGTSubHdrTxt {

			.sapMText {
				white-space: nowrap;
			}
		}
	}

	&:focus:not(:active) {
		outline: 0.125rem solid var(--sapTile_Background);

		.sapMGTFocusDiv {
			border: none;
			outline: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
			outline-offset: 0.125rem;
		}
	}
}

.sapMGTHdrContent.sapMGTOneByOne > .sapMGTHdrTxt > .sapMText {
	font-size: var(--sapFontSize);
	color: var(--sapContent_ContrastTextColor);
	text-align: center !important;
	white-space: initial;
	font-weight: 700;
}

.sapMGTArticleMode {

	.sapMGTContent {

		.sapMTileCnt {

			.sapMTileCntContent {
				height: 3.25rem;

				.sapMImageContent {
					height: 3rem;

					.sapMImageContentImageIcon {
						max-height: 3.25rem;
					}
				}
			}

			.sapMTileFtrCnt {
				height: 2.5rem;
				display: flex;
				position: absolute;
				left: 0;
				right: 0;
				bottom: 0.25rem;
				justify-content: space-between;
				align-items: center;

				.sapMTileCntFtrTxt {
					position: initial;
					margin-right: 0;
				}

				.sapMTileNavContainer {
					margin: 0 1rem 0 0.5rem;
				}
			}
		}
	}
}

.sapMGTArticleMode.sapMTileSmallPhone {

	.sapMGTContent {

		.sapMTileCnt {

			.sapMTileFtrCnt {

				.sapMTileNavContainer {
					margin-right: 0.5rem;
				}
			}
		}
	}
}

.sapMGT.sapMGTScopeDisplay.sapMGTLineMode.sapMGTInfoRendered {
	padding: 0;

	.sapMGTStartMarker {
		display: inline;
	}

	.sapMGTEndMarker {
		display: inline;
	}

	.sapMGTLineWrapper {
		display: flex;
		flex-wrap: wrap;
	}

	.sapMGTHeaderWrapper {
		display: inline-block;
		padding: 0.5rem 0.5rem 0.25rem 0;
		margin-bottom: 0.25rem;

		.sapMGTSubHdrTxt {
			margin-right: 0;
		}
	}

	.sapMGTTInfo {
		display: inline-block;
		margin: 0;
		flex-grow: 1;
		padding-left: 0.5rem;

		.sapMGTAppShortcutText {
			padding-left: 0;
		}
	}
}

.sapMGT.sapMGTScopeDisplay.sapMGTLineMode.sapMGTInfoRendered.sapMGTLineModeSmall {
	display: flex;
	flex-wrap: wrap;
	max-width: 94%;

	.sapMGTHeaderWrapper {
		padding-left:0.5rem;
		display: flex;
		text-overflow: ellipsis;
		overflow: hidden;
		flex-wrap: wrap;
		margin-bottom: 0;
	}

	.sapMGTTouchArea {
		padding: 0;
		margin: 0;
	}

	.sapMGTTInfo {

		.sapMGTAppShortcutText {
			padding-left: 0;
		}
	}
}

.sapMGT.sapMGTScopeDisplay.sapMGTLineMode.sapMGTInfoRendered {

	.sapMGTTInfoWrapper {
		padding: 0.5rem;
		flex-grow: 1;
		background-color: var(--sapInfobar_NonInteractive_Background);
		border-radius: 0 0.25rem 0.25rem 0;

		.sapMGTTInfo {
			margin: 0;
			vertical-align: top;
			text-transform: uppercase;
			flex-grow: 1;
			display: flex;
			overflow: hidden;
			align-items: baseline;
			flex-wrap: wrap;
			padding: 0;

			.sapMGTHdrTxt {
				padding-top: 0.3rem;
			}

			.sapMGTSubHdrTxt {
				margin-right: 0.25rem;
			}

			.sapMText {
				font-size: var(--sapFontSmallSize) ;
			}

			.sapMGTAppShortcutText {
				padding-right: 0.5rem;
				padding-left: 0;

				.sapMText {
					color: var(--sapTile_TextColor);
				}
			}

			.sapMGTSystemInfoText {
				font-size: var(--sapFontSmallSize);
				position: relative;
				display: inline-block;
				word-break: break-word;

				.sapMText{
					color: var(--sapTile_TitleTextColor);
				}
			}

			.sapMGTSystemInfoText.sapMGTSeperatorPresent {
				font-size: var(--sapFontSmallSize);
				position: relative;
				display: inline-block;

				&::before {
					content: '';
					position: absolute;
					display: inline-block;
					height: 1rem;
					width: 0.00623rem;
					left: 0;
					transform: translateX(-0.25rem);
					top: -0.125rem;
					background-color: var(--sapTile_TextColor);
				}
			}
		}
	}

	.sapMGTTInfo.sapMGTInfoNotContainsSeperator {

		.sapMGTAppShortcutText {
			padding-right: 0;
		}
	}
}

/* IconMode Styles */
.sapMGTIconMode {

	.sapMGTHdrTxt > .sapMText {
		font-weight: bold;
		font-size: var(--sapFontHeader6Size);
	}

	.sapMGTSubHdrTxt {
		line-height: 1rem;

		.sapMText {
			font-size:var(--sapFontSize);
			color: var(--sapContent_LabelColor);
		}
	}
}

.sapMGTOneByOne.sapMGTIconMode {
	min-width: 7rem;
	max-width: 15rem;

	.sapMGTHdrContent {
		padding: 0 0.5rem;
	}

	.sapMGTHdrTxt > .sapMText {
		text-shadow: none;
		line-height: 1rem;
		display: block;
	}

	.sapMGTOneByOneIconLoaded {
		margin-top: -1rem;
	}
}

.sapMGTOneByOne.sapMGTIconMode.sapMTResponsiveIconMode {

	.sapMGTHdrContent {
		height:auto;
		padding: 1rem 0.5rem 1rem 0;
	}
}

.sapMGT.sapMGTTwoByHalf {
	display: flex;
	border: none ;
	box-shadow: none ;
	background-color: transparent ;
	align-items: center;
	width: 18.75rem ;
	height: 3.438rem ;
	padding-left: 0.75rem;
	border-radius: var(--sapTile_BorderCornerRadius);
	box-sizing: border-box;
}

.sapMGTTwoByHalf  > .TwoByHalf > .sapMGTHdrTxt > .sapMText {
	font-size: var(--sapFontSize);
	color: var(--sapTile_TitleTextColor);
}

.TwoByHalf.sapMGT .sapMGTHdrContent {
	line-height: 2rem;
	vertical-align: middle;
}

.TwoByHalf.sapMGTIconMode {

	.sapMGTHdrContent {
		height: auto;
		padding-top: 0;
		padding-left: 1rem;
		max-width: 16rem;
		max-height: 2rem;
		align-items: center;
	}

	.sapMGTMoreIcon {
		visibility: hidden;
		bottom: auto;
		right: 1.5rem;
	}
}

.TwoByHalf.sapMGTIconMode.sapMGT:hover {

	.sapMGTMoreIcon {
		visibility: visible;
	}
}

.sapMGTOneByOneIcon {
	display: flex;
	align-items: flex-end;
	padding-top: 1rem;
	justify-content: center;

	.sapUiIcon {
		font-size: 1.25rem !important;
	}
}

.sapMGTTwoByHalfIcon {
	width: 2rem;
	height: 2rem;
	display: flex;
	justify-content: center;
	align-items: center;
	border-radius: 0.5rem;
	flex-shrink: 0;
}

.sapMGTTwoByHalfIcon.sapMGTIconBadge .sapMGTTileIcon {
	font-size: 2.25rem !important;
}

.sapMGTTwoByHalfIcon .sapMGTileBadge {
	color: var(--sapContent_ContrastTextColor);
	top: 1.46875rem;
	line-height: 0.75rem;
	font-weight: bold;
	position: absolute;
}

.sapMGTTileIcon {
	color: var(--sapContent_ContrastTextColor);
}

.TwoByHalf.sapMGTIconMode.sapMGT:focus {

	.sapMGTMoreIcon{
		visibility: visible !important;
	}
}
.sapMGTVisible {
	visibility: visible !important;
}

/* Mobile View-specific styles */
.sapMGTOneByOne.sapMGTIconMode.sapMTileSmallPhone {
	height: 4rem;
	max-width: none;

	.sapMGTHdrContent {
		height:auto;
		overflow: auto;
		padding-right: 0;
	}
}

.sapMGTOneByOne.sapMGTIconMode.sapMTileSmallPhone,
.sapMGTOneByOne.sapMGTIconMode.sapMTResponsiveIconMode {
	width: 14.5rem;
	height: 5.375rem;
	display: flex;
	flex-direction: column;

	.sapMGTHideOverflow {
		bottom: 1rem;
	}

	.sapMGTIconWrapper {
		margin: 1rem 0.5rem 0.375rem 0.5rem;
		align-items: center;
		flex-direction: row;
		gap: 0;

		.sapMGTOneByOneIcon {
			padding: 0;
			margin-bottom: auto;
			margin-top: 0.375rem;
		}

		.sapMGTHdrTxt {
			align-items: center;

			.sapMText {
				text-align: left !important;

				.sapMTextMaxLine {
					-webkit-line-clamp: 3 !important;
				}
			}
		}
	}

	.sapMGTSubHdrTxt {
		padding: 0 0.5rem;
		margin: 0;
		flex-direction: row;
		align-items: flex-start;

		.sapMText {
			font-weight: 400;
			white-space: initial;
			text-align: left !important;

			.sapMTextMaxLine {
				-webkit-line-clamp: 1 !important;
			}
		}
	}

	&.sapMGTHeaderOneLine {

		.sapMGTHdrContent.sapMGTOneByOne {
			justify-content: center;
		}
	}

	&.HighContrastTile {

		.sapMGTHideOverflow {
			bottom: 0.75rem;
		}
	}
}

.TwoByHalf.sapMGTIconMode.sapMTileSmallPhone {

	.sapMGTHdrContent {
		height: auto;
		max-width: 12.5rem;
	}

	.sapMGTTwoByHalfIcon .sapMGTileBadge {
		top: 1.90625rem;
	}
}

/* High Contrast Theme specific styles */
.sapMGT.sapMGTIconMode.HighContrastTile,
.sapMGT.sapMGTIconMode .HighContrastTile.sapMGTTwoByHalfIcon {
	background-color: var(--sapTile_Background);
	border: 0.125rem solid;
}

.sapMGT.sapMGTTwoByHalf.sapMGTIconMode.HighContrastTile {
	border: none;
}

.sapMGT.sapMGTTwoByHalf.sapMGTIconMode.HighContrastTile:not(.sapMGTStateDisabled):hover {
	box-shadow: 0 0 0 0.188rem var(--sapContent_ShadowColor) inset;
	border: none;
}

.sapMGT.sapMGTOneByOne.HighContrastTile:focus:not(:active) {
	outline-offset: 0.0625rem;

	.sapMGTFocusDiv{
		outline: none;
	}

	&::before {
		content: '';
		position: absolute;
		outline: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
		border-radius: 0.1rem;
		outline-offset: 0.25rem;
		inset: 0;
	}
}

.sapMGT.sapMGTOneByOne.HighContrastTile:hover {

	.sapMGTFocusDiv {
		outline-offset: 0 !important;
	}
}

.sapMGT:focus .sapMGTFocusDiv{
	border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
}

.sapMGT:focus .sapMGTFocusDiv,
.sapMGT:active .sapMGTFocusDiv,
.sapMGTLineModePress .sapMGTFocusDiv {
	border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
}

/*Badge Styles */

/* Styles when Icon and Text are present */

.sapMGT {

	.sapMGTBadge {
		position: absolute;
		top: -0.6rem;
		right: 0.5rem;
		border-radius: 0.5rem;
		font-weight: bold;
		font-size: 0.75rem;
		padding: 0 0.5rem;
		font-size: var(--sapFontSmallSize);
		height: 1rem;
		line-height: 1rem;
		z-index: 2;
		display: flex;
		flex-direction: row;
		align-items: center;
		justify-content: center;
		gap: 0.3125rem;
		border: 0.0625rem solid transparent;
		box-sizing: border-box;

		.sapMGTBadgeText {
			white-space: nowrap;
			overflow: hidden;
			text-overflow: ellipsis;
		}

		&.sapMGTBadgeOnlyIcon {
			border-radius: 1.375rem;
			padding: 0;
			height: 1.375rem;
			width: 1.375rem;
			top: -0.75rem;

			.sapUiIcon {
				top: 0.0625rem;
			}
		}

		.sapMGTBadgeOverlay {
			position: absolute;
			inset: -1px;
			border-radius: 0.5rem;
			opacity: 0.8;
		}

		&.sapMGTBadgeTextPresent {

			.sapUiIcon {
				line-height: 0;
			}
		}
	}

	&.OneByOne {

		.sapMGTBadge {

			.sapMGTBadgeText {
				max-width: 7.9rem;
			}
		}
	}

	&.TwoByOne {

		.sapMGTBadge {

			.sapMGTBadgeText {
				max-width: 19.55rem;
			}
		}
	}

	&.TwoByHalf {

		.sapMGTBadge {

			.sapMGTBadgeText {
				max-width: 19.55rem;
			}
		}
	}

	&.OneByHalf {

		.sapMGTBadge {

			.sapMGTBadgeText {
				max-width: 7.9rem;
			}
		}
	}
}

.sapMGT.sapMTileSmallPhone {

	&.OneByOne {

		.sapMGTBadge {

			.sapMGTBadgeText {
				max-width: 6.25rem;
			}
		}
	}

	&.TwoByOne {

		.sapMGTBadge {

			.sapMGTBadgeText {
				max-width: 15.85rem;
			}
		}
	}

	&.TwoByHalf {

		.sapMGTBadge {

			.sapMGTBadgeText {
				max-width: 15.85rem;
			}
		}
	}

	&.OneByHalf {

		.sapMGTBadge {

			.sapMGTBadgeText {
				max-width: 6.25rem;
			}
		}
	}
}

/* Styles when only text is present */
.sapMGT {

	&.OneByOne {

		.sapMGTBadge.sapMGTBadgeOnlyText {

			.sapMGTBadgeText {
				max-width: 9.0625rem;
			}
		}
	}

	&.TwoByOne {

		.sapMGTBadge.sapMGTBadgeOnlyText {

			.sapMGTBadgeText {
				max-width: 20.5625rem;
			}
		}
	}

	&.OneByHalf {

		.sapMGTBadge.sapMGTBadgeOnlyText {

			.sapMGTBadgeText {
				max-width: 9.0625rem;
			}
		}
	}

	&.TwoByHalf {

		.sapMGTBadge.sapMGTBadgeOnlyText {

			.sapMGTBadgeText {
				max-width: 20.5625rem;
			}
		}
	}
}

.sapMGT.sapMTileSmallPhone {

	&.OneByOne {

		.sapMGTBadge.sapMGTBadgeOnlyText {

			.sapMGTBadgeText {
				max-width: 7.3125rem;
			}
		}
	}

	&.TwoByOne {

		.sapMGTBadge.sapMGTBadgeOnlyText {

			.sapMGTBadgeText {
				max-width: 17.0625rem;
			}
		}
	}

	&.OneByHalf {

		.sapMGTBadge.sapMGTBadgeOnlyText {

			.sapMGTBadgeText {
				max-width: 7.3125rem;
			}
		}
	}

	&.TwoByHalf {

		.sapMGTBadge.sapMGTBadgeOnlyText {

			.sapMGTBadgeText {
				max-width: 17.0625rem;
			}
		}
	}
}

/* Styles in IconMode tile when size is TwoByHalf */

.sapMGT.sapMGTIconMode.TwoByHalf {

	.sapMGTBadge.sapMGTBadgeOnlyIcon {
		top: -0.125rem;
		left: 2.125rem;
	}

	&.sapMTileSmallPhone {

		.sapMGTBadge.sapMGTBadgeOnlyIcon {
			top: 0.3125rem;
		}
	}
}

.sapMGT {

	.sapMGTBadgeBackgroundColorIndication1 {
		background-color: var(--sapIndicationColor_1);
	}

	.sapMGTBadgeBackgroundColorIndication2 {
		background-color: var(--sapIndicationColor_2);
	}

	.sapMGTBadgeBackgroundColorIndication3 {
		background-color: var(--sapIndicationColor_3);
	}

	.sapMGTBadgeBackgroundColorIndication4 {
		background-color: var(--sapIndicationColor_4);
	}

	.sapMGTBadgeBackgroundColorIndication5 {
		background-color: var(--sapIndicationColor_5);
	}

	.sapMGTBadgeBackgroundColorIndication6 {
		background-color: var(--sapIndicationColor_6);
	}

	.sapMGTBadgeBackgroundColorIndication7 {
		background-color: var(--sapIndicationColor_7);
	}

	.sapMGTBadgeBackgroundColorIndication8 {
		background-color: var(--sapIndicationColor_8);
	}

	.sapMGTBadgeBackgroundColorIndication9 {
		background-color: var(--sapIndicationColor_9);
	}

	.sapMGTBadgeBackgroundColorIndication10 {
		background-color: var(--sapIndicationColor_10);
	}

	.sapMGTBadgeBackgroundColorCriticalTextColor {
		background-color: var(--sapCriticalTextColor);
	}

	.sapMGTBadgeBackgroundColorWarningBackground {
		background-color: var(--sapWarningBackground);
	}

	.sapMGTBadgeBackgroundColorWarningBorderColor {
		background-color: var(--sapWarningBorderColor);
	}

	.sapMGTBadgeBackgroundColorBrandColor {
		background-color: var(--sapBrandColor);
	}

	.sapMGTBadgeBackgroundColorInformationBorderColor {
		background-color: var(--sapInformationBorderColor);
	}

	.sapMGTBadgeBackgroundColorInformationBackgroundColor {
		background-color: var(--sapInformationBackground);
	}

	.sapMGTBadgeBackgroundColorNeutralElementColor {
		background-color: var(--sapNeutralElementColor);
	}

	.sapMGTBadgeBackgroundColorNeutralBackgroundColor {
		background-color: var(--sapNeutralBackground);
	}

	.sapMGTBadgeBackgroundColorNeutralBorderColor {
		background-color: var(--sapNeutralBorderColor);
	}

	.sapMGTBadgeColorIndication1 {
		color: var(--sapIndicationColor_1);
	}

	.sapMGTBadgeColorIndication2 {
		color: var(--sapIndicationColor_2);
	}

	.sapMGTBadgeColorIndication3 {
		color: var(--sapIndicationColor_3);
	}

	.sapMGTBadgeColorIndication4 {
		color: var(--sapIndicationColor_4);
	}

	.sapMGTBadgeColorIndication5 {
		color: var(--sapIndicationColor_5);
	}

	.sapMGTBadgeColorIndication6 {
		color: var(--sapIndicationColor_6);
	}

	.sapMGTBadgeColorIndication7 {
		color: var(--sapIndicationColor_7);
	}

	.sapMGTBadgeColorIndication8 {
		color: var(--sapIndicationColor_8);
	}

	.sapMGTBadgeColorIndication9 {
		color: var(--sapIndicationColor_9);
	}

	.sapMGTBadgeColorIndication10 {
		color: var(--sapIndicationColor_10);
	}

	.sapMGTBadgeColorCriticalTextColor {
		color: var(--sapCriticalTextColor);
	}

	.sapMGTBadgeColorWarningBackground {
		color: var(--sapWarningBackground);
	}

	.sapMGTBadgeColorWarningBorderColor {
		color: var(--sapWarningBorderColor);
	}

	.sapMGTBadgeColorBrandColor {
		color: var(--sapBrandColor);
	}

	.sapMGTBadgeColorInformationBorderColor {
		color: var(--sapInformationBorderColor);
	}

	.sapMGTBadgeColorInformationBackgroundColor {
		color: var(--sapInformationBackground)
	}

	.sapMGTBadgeColorNeutralElementColor {
		color: var(--sapNeutralElementColor)
	}

	.sapMGTBadgeColorNeutralBackgroundColor {
		color: var(--sapNeutralBackground)
	}

	.sapMGTBadgeColorNeutralBorderColor {
		color: var(--sapNeutralBorderColor)
	}

	.sapMGTBadgeBorderColorIndication1 {
		border-color: var(--sapIndicationColor_1);
	}

	.sapMGTBadgeBorderColorIndication2 {
		border-color: var(--sapIndicationColor_2);
	}

	.sapMGTBadgeBorderColorIndication3 {
		border-color: var(--sapIndicationColor_3);
	}

	.sapMGTBadgeBorderColorIndication4 {
		border-color: var(--sapIndicationColor_4);
	}

	.sapMGTBadgeBorderColorIndication5 {
		border-color: var(--sapIndicationColor_5);
	}

	.sapMGTBadgeBorderColorIndication6 {
		border-color: var(--sapIndicationColor_6);
	}

	.sapMGTBadgeBorderColorIndication7 {
		border-color: var(--sapIndicationColor_7);
	}

	.sapMGTBadgeBorderColorIndication8 {
		border-color: var(--sapIndicationColor_8);
	}

	.sapMGTBadgeBorderColorIndication9 {
		border-color: var(--sapIndicationColor_9);
	}

	.sapMGTBadgeBorderColorIndication10 {
		border-color: var(--sapIndicationColor_10);
	}

	.sapMGTBadgeBorderColorCriticalTextColor {
		border-color: var(--sapCriticalTextColor);
	}

	.sapMGTBadgeBorderColorWarningBackground {
		border-color: var(--sapWarningBackground);
	}

	.sapMGTBadgeBorderColorWarningBorderColor {
		border-color: var(--sapWarningBorderColor);
	}

	.sapMGTBadgeBorderColorBrandColor {
		border-color: var(--sapBrandColor);
	}

	.sapMGTBadgeBorderColorInformationBorderColor {
		border-color: var(--sapInformationBorderColor);
	}

	.sapMGTBadgeBorderColorInformationBackgroundColor {
		border-color: var(--sapInformationBackground)
	}

	.sapMGTBadgeBorderColorNeutralElementColor {
		border-color: var(--sapNeutralElementColor)
	}

	.sapMGTBadgeBorderColorNeutralBackgroundColor {
		border-color: var(--sapNeutralBackground)
	}

	.sapMGTBadgeBorderColorNeutralBorderColor {
		border-color: var(--sapNeutralBorderColor)
	}

	.sapMGTBadge {

		.sapMGTBadgeOverlay {

			&.sapMGTBadgeBackgroundColorIndication1 {
				background-color: var(--sapIndicationColor_1);
			}

			&.sapMGTBadgeBackgroundColorIndication2 {
				background-color: var(--sapIndicationColor_2);
			}

			&.sapMGTBadgeBackgroundColorIndication3 {
				background-color: var(--sapIndicationColor_3);
			}

			&.sapMGTBadgeBackgroundColorIndication4 {
				background-color: var(--sapIndicationColor_4);
			}

			&.sapMGTBadgeBackgroundColorIndication5 {
				background-color: var(--sapIndicationColor_5);
			}

			&.sapMGTBadgeBackgroundColorIndication6 {
				background-color: var(--sapIndicationColor_6);
			}

			&.sapMGTBadgeBackgroundColorIndication7 {
				background-color: var(--sapIndicationColor_7);
			}

			&.sapMGTBadgeBackgroundColorIndication8 {
				background-color: var(--sapIndicationColor_8);
			}

			&.sapMGTBadgeBackgroundColorIndication9 {
				background-color: var(--sapIndicationColor_9);
			}

			&.sapMGTBadgeBackgroundColorIndication10 {
				background-color: var(--sapIndicationColor_10);
			}

			&.sapMGTBadgeBackgroundColorCriticalTextColor {
				background-color: var(--sapCriticalTextColor);
			}

			&.sapMGTBadgeBackgroundColorWarningBackground {
				background-color: var(--sapWarningBackground);
			}

			&.sapMGTBadgeBackgroundColorWarningBorderColor {
				background-color: var(--sapWarningBorderColor);
			}

			&.sapMGTBadgeBackgroundColorBrandColor {
				background-color: var(--sapBrandColor);
			}

			&.sapMGTBadgeBackgroundColorInformationBorderColor {
				background-color: var(--sapInformationBorderColor);
			}

			&.sapMGTBadgeBackgroundColorInformationBackgroundColor {
				background-color: var(--sapInformationBackground);
			}

			&.sapMGTBadgeBackgroundColorNeutralElementColor {
				background-color: var(--sapNeutralElementColor);
			}

			&.sapMGTBadgeBackgroundColorNeutralBackgroundColor {
				background-color: var(--sapNeutralBackground);
			}

			&.sapMGTBadgeBackgroundColorNeutralBorderColor {
				background-color: var(--sapNeutralBorderColor);
			}
		}
	}
}
/* END "GenericTile.less" */

/* START "GroupHeaderListItem.less" */
/* ========================================== */
/* CSS for control sap.m/GroupHeaderListItem  */
/* Base theme                                 */
/* ========================================== */

.sapMGHLI .sapMLIBUnread,
.sapMGHLI .sapMLIBSelectM,
.sapMGHLI .sapMLIBSelectS,
.sapMGHLI .sapMLIBSelectD,
.sapMGHLI .sapMTblItemNav,
.sapMGHLI .sapMListTblSelCol,
.sapMGHLI .sapMListTblUnreadCol {
	display: none;
}

.sapMGHLIUpperCase {
	text-transform: uppercase;
}

.sapMGHLI > .sapMLIBContent,
.sapMGHLICell > .sapMLIBContent {
	display: flex;
	align-items: center;
}

.sapMGHLITitle,
.sapMGHLICounter {
	display: block;
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
}

.sapMGHLICounter {
	padding-left: 0.25rem;
	flex: none;
}

.sapMGHLI {
	align-items: flex-end;
}

.sapMListUl .sapMLIB.sapMGHLI {
	height: 2rem;
	line-height: 2rem;
	font-family: var(--sapFontHeaderFamily);
	font-size: var(--sapFontHeader6Size);
	font-weight: @sapUiFontHeaderWeight;
	color: var(--sapList_TableGroupHeaderTextColor);
	background: var(--sapList_GroupHeaderBackground);
	border-bottom: var(--sapElement_BorderWidth) solid var(--sapList_TableGroupHeaderBorderColor);
}

.sapMListUl > .sapMLIB.sapMGHLI {
	background: var(--sapList_GroupHeaderBackground);
	height: var(--sapElement_LineHeight);
	line-height: 2rem;
	padding-top: 1rem;
}

.sapMGHLI.sapMLIBActive.sapMLIB,
.sapMGHLI.sapMLIBActive.sapMLIB > td {
	color: var(--sapList_Active_TextColor);
	background: var(--sapList_Active_Background);
}

.sapMGHLICell {
	padding: 0 1rem;
}

.sapMGHLI > .sapMGHLICell > .sapMLIBContent:first-child {
	margin: 0;
}

/* group headers should have background color grey so that they are distinguishable from ListItems */
.sapMListTbl .sapMGHLI.sapMLIB:not(.sapMLIBActive):not(.sapMLIBSelected):not(:hover),
.sapMListTbl .sapMGHLI.sapMLIB.sapMLIBTypeInactive {
	background: var(--sapList_TableGroupHeaderBackground);
}
/* END "GroupHeaderListItem.less" */

/* START "HeaderContainer.less" */
/* ====================================== */
/* CSS for control sap.m/HeaderContainer  */
/* Base theme                             */
/* ====================================== */

.sapMHdrCntr {
	position: relative;
	display: inline-block;
	vertical-align: top;
}

.sapMHdrCntr.Horizontal {
	max-width: 100%;

	&.sapMHdrCntrSnapToRow {

		.sapMScrollContScroll {
			display: inline-flex;

			.sapMHdrCntrItemCntr > * {
				min-height: 100%;
			}
		}
	}
}

.sapMHdrCntr.Vertical {
	max-height: 100%;
}

/* initial condition for hide and display arrow button container*/
.sapMHdrCntrBtnCntr {
	display: none;
}

.sapMHdrCntrBtnCntr.Horizontal {
	position: absolute;
	top: 0;
	bottom: 0;
	width: 2.5rem;

	&.sapMHdrCntrLeft {
		left: 0.25rem;
	}

	&.sapMHdrCntrRight {
		right: 0.25rem;
	}
}

.sapUiSizeCompact .sapMHdrCntrBtnCntr.Horizontal {

	&.sapMHdrCntrLeft {
		left: 0;
	}

	&.sapMHdrCntrRight {
		right: 0;
	}
}

html.sap-phone .sapMHdrCntrBtnCntr.Horizontal,
html.sap-tablet:not(.sap-desktop) .sapMHdrCntrBtnCntr.Horizontal {
	width: 1.5rem;

	&.sapMHdrCntrLeft {
		left: 0;
	}

	&.sapMHdrCntrRight {
		right: 0;
	}
}

.sapMHdrCntrBtnCntr.Vertical {
	position: absolute;
	left: 0;
	right: 0;
	height: 2.5rem;

	&.sapMHdrCntrLeft {
		top: 0.25rem;
	}

	&.sapMHdrCntrRight {
		bottom: 0.25rem;
	}
}

html.sap-phone .sapMHdrCntrBtnCntr.Vertical,
html.sap-tablet:not(.sap-desktop) .sapMHdrCntrBtnCntr.Vertical {
	height: 1.5rem;

	&.sapMHdrCntrLeft {
		top: 0;
	}

	&.sapMHdrCntrRight {
		bottom: 0;
	}
}

.sapMHdrCntrBtnCntr.Horizontal > .sapMHdrCntrBtn {
	position: absolute;
	top: 50%;
	margin-top: -1.5rem;
	min-width: 1.375rem;
}

.sapUiSizeCompact .sapMHdrCntrBtnCntr.Horizontal > .sapMHdrCntrBtn {
	right: 0.25rem;
	margin-top: -1rem;
}

html.sap-phone .sapMHdrCntrBtnCntr.Horizontal > .sapMHdrCntrBtn,
html.sap-tablet:not(.sap-desktop) .sapMHdrCntrBtnCntr.Horizontal > .sapMHdrCntrBtn {
	margin-top: -0.6rem;
	font-size: 1rem;
}

.sapMHdrCntrBtnCntr.Vertical > .sapMHdrCntrBtn {
	position: absolute;
	height: 2.5rem;
	left: 50%;
	margin-left: -1.5rem;
	padding: 0;
}

html.sap-phone .sapMHdrCntrBtnCntr.Vertical > .sapMHdrCntrBtn,
html.sap-tablet:not(.sap-desktop) .sapMHdrCntrBtnCntr.Vertical > .sapMHdrCntrBtn {
	margin-left: -0.375rem;
	font-size: 1rem;
	height: 1.375rem;
}

.sapMHdrCntrBtnCntr.Vertical > .sapMHdrCntrBtn.sapMHdrCntrRight {
	bottom: 0;
}

.sapMHdrCntrCntr.Horizontal {
	white-space: nowrap;
}

.sapUiSizeCompact .sapMHrdrLeftPadding .sapMHdrCntrCntr.Horizontal {
	padding-left: 2.5rem;
}

.sapMHrdrLeftPadding .sapMHdrCntrCntr.Horizontal {
	padding-left: 3rem;
}

.sapUiSizeCompact .sapMHrdrRightPadding .sapMHdrCntrCntr.Horizontal {
	padding-right: 2.5rem;
}

.sapMHrdrRightPadding .sapMHdrCntrCntr.Horizontal {
	padding-right: 3rem;
}

html.sap-phone .sapMHrdrLeftPadding .sapMHdrCntrCntr.Horizontal,
html.sap-tablet:not(.sap-desktop) .sapMHrdrLeftPadding .sapMHdrCntrCntr.Horizontal {
	padding-left: 1.5rem;
}

html.sap-phone .sapMHrdrRightPadding .sapMHdrCntrCntr.Horizontal,
html.sap-tablet:not(.sap-desktop) .sapMHrdrRightPadding .sapMHdrCntrCntr.Horizontal {
	padding-right: 1.5rem;
}

.sapMHdrCntrCntr.Vertical {
	height: 100%;
	box-sizing: border-box;
}

.sapMHrdrTopPadding .sapMHdrCntrCntr.Vertical {
	padding-top: 3rem;
}

.sapUiSizeCompact .sapMHrdrTopPadding .sapMHdrCntrCntr.Vertical {
	padding-top: 2.5rem;
}

.sapMHrdrBottomPadding .sapMHdrCntrCntr.Vertical {
	padding-bottom: 3rem;
}

.sapUiSizeCompact .sapMHrdrBottomPadding .sapMHdrCntrCntr.Vertical {
	padding-bottom: 2.5rem;
}

html.sap-phone .sapMHrdrTopPadding .sapMHdrCntrCntr.Vertical,
html.sap-tablet:not(.sap-desktop) .sapMHrdrTopPadding .sapMHdrCntrCntr.Vertical {
	padding-top: 1.5rem;
}

html.sap-phone .sapMHrdrBottomPadding .sapMHdrCntrCntr.Vertical,
html.sap-tablet:not(.sap-desktop) .sapMHrdrBottomPadding .sapMHdrCntrCntr.Vertical {
	padding-bottom: 1.5rem;
}

.sapMHdrCntr .sapMScrollContScroll:focus {
	outline: none;
}

.sapMHrdrCntrInner {
	border: 1px solid transparent;
	position: relative;
	vertical-align: top;
	margin: 0.0625rem;
	padding: 0 0.375rem 0 0.375rem;
	overflow: visible;
}

.sapMHdrCntrCntr.Vertical .sapMHrdrCntrInner {
	float: left;
	clear: left;
}

.sapMHdrCntrCntr.Horizontal .sapMHrdrCntrInner:not(div:first-of-type) {
	margin-left: 1rem;
}

.sapMHdrCntrCntr.Vertical .sapMHrdrCntrInner:not(div:first-of-type) {
	margin-top: 1rem;
}

.sapMHrdrCntrDvdrs .sapMHrdrCntrInner:not(:first-child)::after {
	content: '';
	position: absolute;
	box-sizing: border-box;
	display: inline-block;
}

.sapMHrdrCntrDvdrs.Horizontal .sapMHrdrCntrInner::after {
	top: 0.375rem;
	bottom: 0.375rem;
	left: -0.6rem;
	border-left: 1px solid var(--sapList_BorderColor);
}

.sapMHrdrCntrDvdrs.Vertical .sapMHrdrCntrInner::after {
	left: 0.5rem;
	right: 0.5rem;
	top: -0.6rem;
	border-top: 1px solid var(--sapList_BorderColor);
}

.sapMKTileBorder {
	display: none;
}

.sapMHdrPddngRight2 {
	padding-right: 2rem;
}

.sapMHdrPddngRight0 {
	padding-right: 0;
}

.sapMHdrPddngLeft2 {
	padding-left: 2rem;
}

.sapMHdrPddngleft0 {
	padding-left: 0;
}

.sapMHdrCntrItemCntr {
	display: inline-block;
}

.sapMHdrCntrBGTransparent {
	background-color: var(--sapObjectHeader_Background);
}

.sapMHdrCntrBGSolid {
	background-color: var(--sapObjectHeader_Background);
}

.sapMHdrCntrBGTranslucent {
	background-color: var(--sapObjectHeader_Background);
}

html.sap-desktop .sapMHrdrCntrInner:focus {
	outline: none;
	border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
}

@media (min-width:320px) and (max-width:599px) {

	.sapMHdrCntrCntr.Horizontal .sapMHrdrCntrInner {
		margin-left: 0 !important;
		padding: 0 0.5rem 0 0.5rem !important;
	}

	.sapMHdrCntrCntr.Horizontal .sapMHrdrCntrInner:last-of-type {
		padding-right: 1rem !important;
	}

	.sapMHrdrCntrDvdrs.Horizontal .sapMHrdrCntrInner::after {
		left: 0.4rem;
		border-left: 1px solid var(--sapList_BorderColor);
	}

	html[dir=rtl] .sapMHdrCntrCntr.Horizontal .sapMHrdrCntrInner {
		margin-right: 0 !important;
		padding: 0 0.5rem 0 0.5rem !important;
	}

	html[dir=rtl] .sapMHdrCntrCntr.Horizontal .sapMHrdrCntrInner:last-of-type {
		padding-left: 1rem !important;
	}
}
/* END "HeaderContainer.less" */

/* START "IconTabBar.less" */
/* ================================= */
/* CSS for control sap.m/IconTabBar  */
/* Base theme                        */
/* ================================= */

/* ========================================================================== */
/* Variables */
/* ========================================================================== */

@_sap_m_IconTabBar_TextColor: var(--sapTab_TextColor);
@_sap_m_IconTabBar_TextShadow: @sapUiShadowText;
@_sap_m_IconTabBar_DisabledOpacity: var(--sapContent_DisabledOpacity);
@_sap_m_IconTabBar_Separator_Icon: var(--sapContent_NonInteractiveIconColor);
@_sap_m_IconTabBar_Separator_Font_Size: 1rem;
@_sap_m_IconTabBar_More_Button_Focus_Offset: 0.125rem;
@_sap_m_IconTabBar_Arrow_SubItem_Left_Padding: 0.375rem;
@_sap_m_IconTabBar_More_Button_Border_Radius: inherit;
@_sap_m_IconTabBar_HeaderMinHeight: 2.75rem;
@_sap_m_IconTabBar_TextHoverColor: var(--sapTab_Selected_TextColor);

/* ==================== Cozy size ==================== */

/* Content arrow */
@_sap_m_IconTabBar_ContentArrowHeight: 0.625rem;

/* Inline mode */
@_sap_m_IconTabBar_InLineFilterMargin: 1rem 0 0.1875rem 0;
@_sap_m_IconTabBar_InLineTextOnlyVerticalHeight: 3rem;
@_sap_m_IconTabBar_InLineTextOnlyContentArrowHeight: 0.8125rem;

/* TextOnly mode */
@_sap_m_IconTabBar_TextOnlyTabPadding: 0.5625rem 0 0.3125rem 0;
@_sap_m_IconTabBar_TextOnlyVerticalHeight: 3.5rem;

/* NoText mode */
@_sap_m_IconTabBar_NoTextFilterWidth: 3rem;
@_sap_m_IconTabBar_NoTextVerticalHeight: 4rem;
@_sap_m_IconTabBar_NoTextVerticalContentArrowHeight: 1rem;

/* All tabs */
@_sap_m_IconTabBar_AllTabHeight: 3rem;
@_sap_m_IconTabBar_AllTabLineHeight: 3rem;
@_sap_m_IconTabBar_AllTabMarginBottom: 1rem;

/* Horizontal tabs */
@_sap_m_IconTabBar_HorizontalFilterHeight: 4rem;
@_sap_m_IconTabBar_HorizontalTabHeight: 3rem;
@_sap_m_IconTabBar_HorizontalWrapperHeight: 3rem;
@_sap_m_IconTabBar_HorizontalTabWidth: 3rem;
@_sap_m_IconTabBar_HorizontalCountTop: 0.4375rem;
@_sap_m_IconTabBar_HorizontalTextPaddingTop: 0.625rem;
@_sap_m_IconTabBar_HorizontalTextPaddingLeft: 0.25rem;

/* Vertical tabs */
@_sap_m_IconTabBar_VerticalHeight: 5rem;
@_sap_m_IconTabBar_VerticalContentArrowHeight: 0.6875rem;

/* Filter icon */
@_sap_m_IconTabBar_FilterIconHeight: 3rem;
@_sap_m_IconTabBar_FilterIconWidth: 3rem;
@_sap_m_IconTabFilter_Icon_Border_Radius: 0.25rem;
@_sap_m_IconTabFilter_Icon_Border_Color: transparent;

/* Filter text */
@_sap_m_IconTabBar_FilterTextLineHeight: 0.9375rem;

/* Separator */
@_sap_m_IconTabBar_SeparatorHeight: 3rem;
@_sap_m_IconTabBar_SeparatorMargin: 0 0.25rem;
@_sap_m_IconTabBar_TextOnlySeparatorMargin: 0.375rem 0.25rem 0.375rem;
@_sap_m_IconTabBar_TextOnlySeparatorMarginCompact: @_sap_m_IconTabBar_SeparatorMargin;
@_sap_m_IconTabBar_SeparatorIconHeight: 3rem;
@__sap_m_IconTabBar_SeparatorIconLineHeight: 3rem;

/* Header */
@_sap_m_IconTabBar_HeaderBackground: var(--sapObjectHeader_Background);
@_sap_m_IconTabBar_HeaderBackgroundTranslucent: fade(@sapUiObjectHeaderBackground, 60);

@_sap_m_IconTabBar_HeaderBorderBottom: 0.125rem solid var(--sapObjectHeader_Background);
@_sap_m_IconTabBar_HeaderShadow: @sapUiShadowHeader;

/* Content */
@_sap_m_IconTabBar_ContentBackground: var(--sapGroup_ContentBackground);
@_sap_m_IconTabBar_ContentBackgroundTranslucent: fade(@sapUiGroupContentBackground, 60);

/* ==================== Compact size ==================== */

/* Content arrow */
@_sap_m_IconTabBar_Compact_NoTextContentArrowHeight: 0.625rem;
@_sap_m_IconTabBar_Compact_ContentArrowHeight: 0.75rem;
@_sap_m_IconTabBar_Compact_NoTextContentArrowWidth: 2rem;

/* Inline mode */
@_sap_m_IconTabBar_Compact_InLineVerticalTabHeight: 2rem;
@_sap_m_IconTabBar_Compact_InLineFilterTextMargin: 0 0 0.25rem 0;
@_sap_m_IconTabBar_Compact_TextOnlyContentArrowHeight: 0.5rem;

/* TextOnly */
@_sap_m_IconTabBar_Compact_TextOnlyVerticalHeight: 2.75rem;
@_sap_m_IconTabBar_Compact_TextOnlyTabPadding: 0;
@_sap_m_IconTabBar_Compact_TextOnlyHeadPadding: 0.25rem 0 0 0.1875rem;
@_sap_m_IconTabBar_Compact_TextOnlyTextLineHeight: 0.875rem;
@_sap_m_IconTabBar_Compact_TextOnlyTextPadding: 0.25rem 0 0;

/* NoText */
@_sap_m_IconTabBar_Compact_NoTextTabWidth: 2rem;
@_sap_m_IconTabBar_Compact_NoTextVerticalHeight: 2.625rem;

/* Horizontal */
@_sap_m_IconTabBar_Compact_HorizontalHeight: 2.75rem;
@_sap_m_IconTabBar_Compact_HorizontalContentArrowHeight: 0.75rem;

/* Vertical tabs */
@_sap_m_IconTabBar_Compact_VerticalHeight: 3.625rem;

/* Filter text */
@_sap_m_IconTabBar_Compact_FilterTextPadding: 0.25rem 0 0;

/* Focus border */
@_sap_m_IconTabBar_Compact_HorizontalFocusBottom: 0;

/* ========== Overflow ========= */
@_sap_m_IconTabBar_Overflow_BorderRadius: 0.75rem;
@_sap_m_IconTabBar_Overflow_BackgroundColor: var(--sapButton_Background);
@_sap_m_IconTabBar_Overflow_Border: 1px solid var(--sapButton_BorderColor);

@_sap_m_IconTabBar_Overflow_TextColor: var(--sapButton_TextColor);
@_sap_m_IconTabBar_Overflow_Icon_Color: var(--sapButton_IconColor);

@_sap_m_IconTabBar_Overflow_BorderColorHover: var(--sapButton_Hover_BorderColor);
@_sap_m_IconTabBar_Overflow_BackgroundColorHover: var(--sapButton_Hover_Background);
@_sap_m_IconTabBar_Overflow_TextColorHover: var(--sapButton_Hover_TextColor);
@_sap_m_IconTabBar_Overflow_Icon_ColorHover: var(--sapButton_Hover_TextColor);
@_sap_m_IconTabBar_Overflow_BoxShadowHover: none;

@_sap_m_IconTabBar_Overflow_BackgroundColorPressed: var(--sapButton_Active_Background);
@_sap_m_IconTabBar_Overflow_BorderColorPressed: var(--sapButton_Active_BorderColor);
@_sap_m_IconTabBar_Overflow_TextColorPressed: var(--sapButton_Active_TextColor);
@_sap_m_IconTabBar_Overflow_Icon_ColorPressed: var(--sapButton_Active_TextColor);

/* =========== Phone =========== */

@_sap_m_IconTabBar_Phone_TextOnlyContentArrowHeight: 0.5rem;
@_sap_m_IconTabBar_Phone_Compact_TextOnlyContentArrowHeight: @_sap_m_IconTabBar_Phone_TextOnlyContentArrowHeight;

/* =========== Shell =========== */
@_sap_m_IconTabBar_ShellHeaderShadow: var(--sapShell_Shadow);
@_sap_m_IconTabBar_ShellTextColor: var(--sapShell_Navigation_TextColor);
@_sap_m_IconTabBar_ShellSelectedTextColor: var(--sapShell_Navigation_Selected_TextColor);
@_sap_m_IconTabBar_ShellButtonOutlineColor: contrast(@sapUiShellColor, @sapUiContentFocusColor, @sapUiContentContrastFocusColor, @sapUiContentContrastTextThreshold);
@_sap_m_IconTabBar_ShellOverflowBorderColor: transparent;
@_sap_m_IconTabBar_ShellOverflowHoverBorderColor: transparent;
@_sap_m_IconTabBar_ShellOverflowPressedBorderColor: var(--sapShell_Navigation_Active_Background);
@_sap_m_IconTabBar_HeaderFocusBorderTopOffset: -1px;
@_sap_m_IconTabBar_HeaderFocusBorderBottomOffset: -1px;
@_sap_m_IconTabBar_HeaderFocusBorderRightOffset: -1px;
@_sap_m_IconTabBar_HeaderFocusBorderLeftOffset: -1px;
@_sap_m_IconTabBar_HeaderBorderRadius: 0px;
@_sap_m_IconTabBar_Filter_Expand_Shell_BackgroundColor_Hover: var(--sapShell_Hover_Background);
@_sap_m_IconTabBar_Filter_Expand_Shell_BorderColor_Hover: transparent;

/* ========== Badge ============ */
@_sap_m_IconTabBar_FilterExpandButtonBadgeTop: 1.125rem;
@_sap_m_IconTabBar_Compact_FilterExpandButtonBadgeTop: 1.125rem;
@_sap_m_IconTabBar_Badge_Neutral_Background: var(--sapNeutralElementColor);
@_sap_m_IconTabBar_Badge_Positive_Background: var(--sapPositiveElementColor);
@_sap_m_IconTabBar_Badge_Critical_Background: var(--sapCriticalElementColor);
@_sap_m_IconTabBar_Badge_Negative_Background: var(--sapNegativeElementColor);

/* ------------------------------------------------ */
/* IconTabBar main class */
/* ------------------------------------------------ */

.sapMITB {
	position: relative;
	width: 100%;

	&.sapMITBNoContentPadding > .sapMITBContainerContent > .sapMITBContent {
		padding: 0;
	}
}

/* ------------------------------------------------ */
/* IconTabBar tab class */
/* ------------------------------------------------ */

.sapMITBTab {
	width: 5rem;
	display: block;
}

/* ------------------------------------------------ */
/* IconTabBar content container */
/* ------------------------------------------------ */

.sapMITBContent {
	padding: 1rem;
	position: relative;
	background-color: @_sap_m_IconTabBar_ContentBackground;
}

.sapMITBContentClosed {
	background-color: transparent;
	border-bottom: none;
	margin-top: 0.75rem;
	padding: 0.125rem 0 0 0;
	position: relative;
}

.sapMITBContainerContent {
	margin-top: 0;
	padding: 0;
	position: relative;

	&.sapMITBContentClosed {
		background-color: transparent;
		border-bottom: none;
	}
}

/* ------------------------------------------------ */
/* IconTabBar filter class */
/* ------------------------------------------------ */

.sapMITBFilter {
	padding: 0 0.25rem;
	cursor: pointer;

	.sapMITBFilterIcon, .sapMITBInlineIcon {
		cursor: pointer;
	}

	&.sapMITBItem {
		display: inline-flex;
	}

	.sapMITBFilterWrapper {
		min-width: 0; /* fixes flex Chrome issue */
		max-width: 100%;
	}

	.sapMITBText {
		width: 5rem;
		line-height: @_sap_m_IconTabBar_FilterTextLineHeight;
		padding: 0.375rem 0 0;
		position: relative;
		color: @_sap_m_IconTabBar_TextColor;
		text-shadow: @_sap_m_IconTabBar_TextShadow;
		display: flex;
		align-items: center;
		justify-content: center;
		font-family: var(--sapFontHeaderFamily);

		.sapUiIcon {
			font-size: 1rem;
		}

		.sapMITHTextContent {
			display: inline-block;
			max-width: 100%;
			overflow: hidden;
			text-overflow: ellipsis;
			vertical-align: bottom;
		}
	}

	&.sapMITHUnselectable:not(.sapMITBDisabled) .sapMITHTextContent {
		position: relative;
	}

	.sapMITBCount {
		color: @_sap_m_IconTabBar_TextColor;
		text-shadow: @_sap_m_IconTabBar_TextShadow;
		max-width: 2rem;
		overflow: hidden;
		position: absolute;
		text-overflow: ellipsis;
	}

	.sapMITBInlineIcon {
		padding-right: 0.5rem;
	}
}

.sapMITHShowSubItemsIcon {
	pointer-events: none;
	padding-left: @_sap_m_IconTabBar_Arrow_SubItem_Left_Padding;

	.sapUiIcon {
		cursor: pointer;
	}
}

.sapMITH .sapMITBItem.sapMITBFilterHidden {
	display: none;
}

/* ------------------------------------------------ */
/* IconTabBar Tab Filter Show All Item class */
/* ------------------------------------------------ */

.sapMITBAll {
	margin-right: 1.25rem;
	cursor: pointer;


	&:first-child .sapMITBTab .sapMITBCount {
		padding-left: 0;
	}

	.sapMITBTab {
		margin-bottom: @_sap_m_IconTabBar_AllTabMarginBottom;
		display: inline-block;
		height: @_sap_m_IconTabBar_AllTabHeight;
		line-height: @_sap_m_IconTabBar_AllTabLineHeight;
		width: auto;
	}

	.sapMITBCount {
		display: inline-block;
		font-size: 1.5rem;
		padding: 0 0.5rem;
		vertical-align: middle;
		width: auto;
		color: @_sap_m_IconTabBar_TextColor;
		text-shadow: @_sap_m_IconTabBar_TextShadow;
	}

	.sapMITBText {
		display: inline-block;
		max-height: 3rem;
		overflow: hidden;
		text-align: left;
		text-overflow: ellipsis;
		vertical-align: middle;
		white-space: normal;
		width: auto;
		color: @_sap_m_IconTabBar_TextColor;
		text-shadow: @_sap_m_IconTabBar_TextShadow;
	}
}

/* ------------------------------------------------ */
/* IconTabBar Tab Text Only class */
/* ------------------------------------------------ */

.sapMITBTextOnly {

	&.sapMITBFilter:not(.sapMITBDisabled).sapMITBText {
		cursor: pointer;
	}

	.sapMITBFilter {
		padding: 0 1rem;

		.sapMITBText {
			height: 1rem;
			line-height: normal;
			margin: 0;
			padding: 0;
			text-align: left;
			width: auto;

			.sapMITHShowSubItemsIcon {
				line-height: 1rem;
			}
		}

		.sapMITBCount {
			font-size: @sapMFontMediumSize;
			max-width: none;
			position: relative;
		}
	}

	.sapMITBTab {
		height: 1rem;
		padding: @_sap_m_IconTabBar_TextOnlyTabPadding;
		text-align: left;
		width: auto;
	}

	.sapMITBVertical {
		height: @_sap_m_IconTabBar_TextOnlyVerticalHeight;

		&.sapMITBFilter .sapMITBText + .sapMITBContentArrow {
			height: @_sap_m_IconTabBar_ContentArrowHeight;
		}
	}

	.sapMITBText {
		font-size: @sapMFontMediumSize;
	}

	.sapMITBFilterDefault .sapMITBText {
		color: @_sap_m_IconTabBar_TextColor;
		text-shadow: @_sap_m_IconTabBar_TextShadow;
	}
}

/* ------------------------------------------------ */
/* IconTabBar Tab With No Text class */
/* ------------------------------------------------ */

.sapMITBNoText {

	.sapMITBFilter {

		.sapMITBTab {
			margin: 0 0.75rem;
			width: @_sap_m_IconTabBar_NoTextFilterWidth;
		}

		&:first-child .sapMITBTab {
			margin-left: 0;
		}

		.sapMITBContentArrow {
			margin: 0 auto;
			width: 3rem;
		}

		&:first-child .sapMITBContentArrow {
			margin: 0 auto 0 0;
		}

		&.sapMITBItemNoCount:last-child {
			padding-right: 0;
			margin-right: 0;

			.sapMITBTab {
				margin-right: 0;
			}

			.sapMITBContentArrow {
				margin-right: 0;
			}
		}
	}

	.sapMITBVertical {
		height: @_sap_m_IconTabBar_NoTextVerticalHeight;
	}
}

/* ------------------------------------------------ */
/* IconTabBar Stretch content height class */
/* ------------------------------------------------ */

.sapMITBStretch {
	display: flex;
	flex-direction: column;
	position: absolute;
	top: 0;
	bottom: 0;
	left: 0;
	right: 0;

	> .sapMITBContainerContent {
		flex-grow: 1;
	}

	> .sapMITBContainerContent > .sapMITBContent {
		bottom: 0;
		left: 0;
		position: absolute;
		right: 0;
		top: 0;
	}
}

/* ------------------------------------------------ */
/* IconTabBar Head container class */
/* ------------------------------------------------ */

.sapMITBHead {
	padding-top: 1rem;
	flex-grow: 1;
	position: relative;
	white-space: nowrap;
	overflow: hidden;
	vertical-align: top;
	box-sizing: border-box;
	padding: 0 0.1875rem;
}

/* ------------------------------------------------ */
/* IconTabBar Overflow tab class */
/* ------------------------------------------------ */

.sapMITH {

	.sapMITHEndOverflow,
	.sapMITHStartOverflow {
		align-self: stretch;
		flex-grow: 0;
		flex-shrink: 0;
		display: flex;
		align-items: center;

		.sapMITBFilterWrapper {
			display: inline-flex;
			align-items: center;
		}

		.sapMITBItem {
			visibility: hidden;
			margin: 0;
			padding: 0;
			height: 1.5rem;
			border: @_sap_m_IconTabBar_Overflow_Border;
			background-color: @_sap_m_IconTabBar_Overflow_BackgroundColor;
			border-radius: @_sap_m_IconTabBar_Overflow_BorderRadius;

			.sapMITBContentArrow {
				display: none;
			}

			.sapMITHShowSubItemsIcon {
				color: @_sap_m_IconTabBar_Overflow_Icon_Color;
			}

			&:hover {
				border-color: @_sap_m_IconTabBar_Overflow_BorderColorHover;
				background-color: @_sap_m_IconTabBar_Overflow_BackgroundColorHover;
				box-shadow: @_sap_m_IconTabBar_Overflow_BoxShadowHover;

				.sapMITBText {
					color: @_sap_m_IconTabBar_Overflow_TextColorHover;
				}

				.sapMITHShowSubItemsIcon {
					color: @_sap_m_IconTabBar_Overflow_Icon_ColorHover;
				}
			}

			&:active {
				border-color: @_sap_m_IconTabBar_Overflow_BorderColorPressed;
				background-color: @_sap_m_IconTabBar_Overflow_BackgroundColorPressed;

				.sapMITBText {
					color: @_sap_m_IconTabBar_Overflow_TextColorPressed;
					text-shadow: none;
				}

				.sapMITHShowSubItemsIcon {
					color: @_sap_m_IconTabBar_Overflow_Icon_ColorPressed;
				}
			}
		}

		.sapMITBText {
			font-size: @sapMFontMediumSize;
			padding: 0;
			margin: 0 0.5rem;
			text-align: left;
			width: auto;
			color: @_sap_m_IconTabBar_Overflow_TextColor;
		}

		.sapMITHOverflowVisible {
			visibility: visible;
		}

		.sapMITBTab {
			display: none;
		}
	}
}

.sapMITH:not(.sapMITHEndOverflowList) .sapMITHEndOverflow,
.sapMITH:not(.sapMITHStartOverflowList) .sapMITHStartOverflow {
	display: none;
}

/* ------------------------------------------------ */
/* IconTabBar Content arrow class */
/* ------------------------------------------------ */

.sapMITBContentArrow {
	box-sizing: border-box;
	height: 1rem;
	
	&::after {
		height: var(--sapTab_Selected_Indicator_Dimension);
	}
}

/* ------------------------------------------------ */
/* IconTabBar Tab item  class */
/* ------------------------------------------------ */

.sapMITBItem {
	display: inline-block;
	position: relative;
	text-align: center;
	vertical-align: top;
	color: var(--sapGroup_TitleTextColor);
	max-width: 100%;
	box-sizing: border-box;

	/* Dragged Filter Item */
	&.sapUiDnDDragging {
		background: var(--sapObjectHeader_Background);
	}
}

.sapMITBFilterExpandBtn {
	margin-left: 0.25rem;
	align-items: center;
	align-self: center;
	display: flex;
	height: 2rem;
	padding: 0.1875rem 0;
	width: 1.5rem;
}

.sapMITBFilterExpandBtn .sapMITBFilterExpandIcon {
	border:  0.0625rem solid @_sap_m_IconTabFilter_Icon_Border_Color;
	border-radius: @_sap_m_IconTabFilter_Icon_Border_Radius;
	box-sizing: content-box;
	overflow: visible;
	width: 1.5rem;
	min-width: 1.5rem;
	height: 1.5rem;
	line-height: 1.5rem;
	font-size: 1rem;
	color: @_sap_m_IconTabBar_TextColor;
	font-family: var(--sapContent_IconFontFamily);
}

.sapMITBFilterExpandBtn .sapMITBFilterExpandIcon {
	display: flex;
	justify-content: center;
	align-items: center;
}

.sapMITBFilterExpandBtn:hover .sapMITBFilterExpandIcon {
	border-color: var(--sapButton_Hover_BorderColor);
	background-color: var(--sapButton_Hover_Background);
	color: var(--sapButton_Lite_Hover_TextColor);
}

.sapMITBFilterExpandBtn:active .sapMITBFilterExpandIcon {
	background-color: var(--sapButton_Active_Background);
	border-color: var(--sapButton_Active_BorderColor);
	color: var(--sapButton_Active_TextColor);
}

.sapMITBDisabled {
	opacity: @_sap_m_IconTabBar_DisabledOpacity;
	cursor: default;

	.sapUiIcon {
		cursor: default;
	}

	.sapMITBFilterIcon {
		border-color: var(--sapGroup_TitleTextColor);
	}
}

.sapMITBFilterDefault {

	.sapMITBFilterWrapper:hover {

		.sapMITBText {
			color: @_sap_m_IconTabBar_TextHoverColor;
		}

		.sapMITBSelectItemIcon.sapUiIcon {
			color: @_sap_m_IconTabBar_TextHoverColor !important;
		}
	}

	.sapMITBFilterExpandBtn:hover {

		.sapMITBFilterExpandIcon {
			color: @_sap_m_IconTabBar_TextHoverColor;
		}
	}
}

/* ------------------------------------------------ */
/* Vertical Tab Filter Item */
/* ------------------------------------------------ */

.sapMITBVertical {
	height: @_sap_m_IconTabBar_VerticalHeight;

	&.sapMITBFilter .sapMITBContentArrow {
		height: @_sap_m_IconTabBar_VerticalContentArrowHeight;
	}
}

.sapMITH:not(.sapMITBTextOnly):not(.sapMITBNoText) .sapMITBHead .sapMITBVertical.sapMITHUnselectable {

	.sapMITBText {
		padding-top: 0.25rem;
	}

	.sapMITBContentArrow {
		height: 0.5rem;
	}
}

.sapMITBNoText .sapMITBVertical.sapMITBFilter .sapMITBContentArrow {
	height: @_sap_m_IconTabBar_NoTextVerticalContentArrowHeight;
}

/* ------------------------------------------------ */
/* Horizontal Tab Filter Item */
/* ------------------------------------------------ */

.sapMITBHorizontalWrapper {
	display: inline-block;
	height: @_sap_m_IconTabBar_HorizontalWrapperHeight;
	text-overflow: ellipsis;
	vertical-align: top;
	width: auto;
}

.sapMITBHorizontal {

	&.sapMITBFilter {
		height: @_sap_m_IconTabBar_HorizontalFilterHeight;

		&:first-child .sapMITBTab {
			padding-left: 0;
		}

		.sapMITBContentArrow {
			width: 3.375rem;
		}

		.sapMITBText {
			padding: @_sap_m_IconTabBar_HorizontalTextPaddingTop 0 0 @_sap_m_IconTabBar_HorizontalTextPaddingLeft;
			justify-content: flex-start;
			top: 0.3125rem;
			width: auto;
		}

		.sapMITBCount {
			display: block;
			max-width: none;
			padding-left: @_sap_m_IconTabBar_HorizontalTextPaddingLeft;
			position: relative;
			text-align: left;
			top: @_sap_m_IconTabBar_HorizontalCountTop;
		}

		.sapMITBTab {
			display: inline-block;
			height: @_sap_m_IconTabBar_HorizontalTabHeight;
			padding: 0;
			vertical-align: top;
			width: @_sap_m_IconTabBar_HorizontalTabWidth;
		}
	}

	&.sapMITBAll .sapMITBTab {
		margin-bottom: 0;
	}

	+ .sapMITBSep {
		width: auto;

		> .sapMITBSepIcon {
			padding: 0 0.5rem;
		}
	}
}

/* ------------------------------------------------ */
/* Tab Filter In Line */
/* ------------------------------------------------ */

.sapMITBInLine {

	.sapMITBHead .sapMITBFilter .sapMITBText {
		margin: @_sap_m_IconTabBar_InLineFilterMargin;
	}

	&.sapMITBTextOnly .sapMITBHead .sapMITBVertical.sapMITBFilter {
		height: @_sap_m_IconTabBar_InLineTextOnlyVerticalHeight;

		.sapMITBContentArrow {
			height: @_sap_m_IconTabBar_InLineTextOnlyContentArrowHeight;
		}
	}
}

/* ------------------------------------------------ */
/* Tab Filter Icon */
/* ------------------------------------------------ */

.sapMITBFilterIcon {
	border-bottom-left-radius: 3rem;
	border-bottom-right-radius: 3rem;
	-moz-border-radius: 3rem;
	border-top-left-radius: 3rem;
	/* some native browsers like Samsung Galaxy S4 with Android 4.2 do not support the "border-
	radius" shorthand property but does support the long-hand properties for each corner like
	"border-top-left-radius" */
	border-top-right-radius: 3rem;
	box-sizing: border-box;
	font-size: 1.5rem;
	display: inline-flex;
	justify-content: center;
	align-items: center;
	flex-direction: column;
	height: @_sap_m_IconTabBar_FilterIconHeight;
	overflow: hidden;
	width: @_sap_m_IconTabBar_FilterIconWidth;
	-moz-user-select: none;
	-webkit-user-select: none;
	user-select: none;
}

.sapMITBFilterNoIcon {
	display: inline-block;
	height: 3rem;
	vertical-align: middle;
	width: 3rem;
}

.sapMITBCount {
	color: var(--sapGroup_TitleTextColor);
}

/* ------------------------------------------------ */
/* IconTabBar Tab Separator Item */
/* ------------------------------------------------ */

.sapMITBSep {
	width: 1.5rem;
	pointer-events: none;
}

.sapMITBSepLine {
	height: @_sap_m_IconTabBar_SeparatorHeight;
	margin: @_sap_m_IconTabBar_SeparatorMargin;
	width: 0;
	border: 1px solid var(--sapList_BorderColor);
}

.sapMITBTextOnly .sapMITBSepLine,
.sapUiMedia-Std-Phone .sapMITBTextOnly .sapMITBSepLine,
.sapMITH.sapUiSizeCompact.sapMITBTextOnly .sapMITBSepLine {
	margin: @_sap_m_IconTabBar_TextOnlySeparatorMargin;
	height: 2rem;
}

.sapMITH.sapUiSizeCompact.sapMITBTextOnly .sapMITBSepLine {
	margin: @_sap_m_IconTabBar_TextOnlySeparatorMarginCompact;
}

.sapMITBSepIcon {
	font-size: @_sap_m_IconTabBar_Separator_Font_Size;
	height: @_sap_m_IconTabBar_SeparatorIconHeight;
	line-height: @__sap_m_IconTabBar_SeparatorIconLineHeight;
	padding: 0 0.25rem;
	color: @_sap_m_IconTabBar_Separator_Icon;

	&::before {
		text-shadow: @_sap_m_IconTabBar_TextShadow;
	}
}

.sapMITBTextUpperCase .sapMITBText {
	text-transform: uppercase;
}

.sapMITBBackgroundDesignTransparent > .sapMITBContainerContent > .sapMITBContent {
	background-color: transparent;
	border-bottom: none;
}

/* mixin call for creating container content padding classes (arguments: rootContainer, contentSelector) */
.sapUiContainerContentPadding(~".sapMITB", ~"> .sapMITBContainerContent > .sapMITBContent");

.sapMITH,
.sapMITBHead,
.sapMITBFilterIcon,
.sapMITBItem {

	&:focus {
		outline: none;
	}
}

.sapMITBItem .sapMITBFilterIcon {
	overflow: visible;
}

.sapMITH {
	vertical-align: top;
	font-size: var(--sapFontSmallSize);
	display: flex;
	align-items: flex-start;
	background-color: @_sap_m_IconTabBar_HeaderBackground;
	border-bottom: @_sap_m_IconTabBar_HeaderBorderBottom;
	box-shadow: @_sap_m_IconTabBar_HeaderShadow;
	position: relative;
	padding: 0 2rem;

	::-webkit-scrollbar {
		display: none;
	}

	.sapMITBFilter.sapMITHDragOver,
	.sapMITBFilterExpandBtn.sapMITHDragOver .sapMITBFilterExpandIcon {
		background: @sapUiDragAndDropActiveBackground;
		box-shadow: inset 0 0 0 0.125rem @sapUiDragAndDropActiveBorderColor;
	}

	.sapMITBHead .sapMITBFilter {

		&:first-child {
			padding-left: 0;
			margin-left: 0;
		}

		&:last-child {
			padding-right: 0;
			margin-right: 0;
		}
	}

	&.sapMITHBackgroundDesignTransparent {
		box-shadow: none;
		border-bottom: 0.0625rem solid var(--sapObjectHeader_BorderColor);
	}

}

/* Custom focus outline */
html.sap-desktop {

	.sapMITBItem:not(.sapUiDnDDragging):focus .sapMITBFilterIcon,
	.sapMITBTextOnly .sapMITBHead .sapMITBItem:not(.sapUiDnDDragging):focus .sapMITBText,
	.sapMITBItem:not(.sapUiDnDDragging):focus.sapMITBAll {

		&::after {
			content: '';
			position: absolute;
			top: @_sap_m_IconTabBar_HeaderFocusBorderTopOffset;
			bottom: @_sap_m_IconTabBar_HeaderFocusBorderBottomOffset;
			right: @_sap_m_IconTabBar_HeaderFocusBorderRightOffset;
			left: @_sap_m_IconTabBar_HeaderFocusBorderLeftOffset;
			border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
			border-radius: @_sap_m_IconTabBar_HeaderBorderRadius;
			pointer-events: none;
		}
	}

	.sapMITHStartOverflow,
	.sapMITHEndOverflow {

		.sapMITBItem:not(.sapUiDnDDragging):focus {

			&::after {
				content: '';
				position: absolute;
				top: @_sap_m_IconTabBar_More_Button_Focus_Offset;
				bottom: @_sap_m_IconTabBar_More_Button_Focus_Offset;
				right: @_sap_m_IconTabBar_More_Button_Focus_Offset;
				left: @_sap_m_IconTabBar_More_Button_Focus_Offset;
				border-radius: @_sap_m_IconTabBar_More_Button_Border_Radius;
				border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
				pointer-events: none;
			}
		}
	}
}

/* ------------------------------------------------ */
/* Phone device */
/* ------------------------------------------------ */

.sapUiMedia-Std-Phone {

	.sapMITH.sapMITBTextOnly:not(.sapMITBInLine) {

		.sapMITBHead .sapMITBTab {
			padding: 0.25rem 0 0.25rem 0;
		}

		.sapMITBHead .sapMITBVertical.sapMITBFilter {
			height: 3rem;

			.sapMITBContentArrow {
				height: @_sap_m_IconTabBar_Phone_TextOnlyContentArrowHeight;
			}
		}
	}

	.sapMITBHead .sapMITBFilter {
		padding: 0 0.25rem;
	}

	.sapMITBTextOnly .sapMITBHead .sapMITBFilter {
		margin: 0 0.5rem;
	}

	/* Compact size in phone size */
	.sapMITH.sapUiSizeCompact {

		/* This solves an issue when we are in compact mode on real device */
		/* We remove the bottom padding and leave only top */
		&.sapMITBTextOnly:not(.sapMITBInLine) {

			.sapMITBHead .sapMITBTab {
				padding: 0.25rem 0 0 0;
			}

			.sapMITBVertical.sapMITBFilter .sapMITBContentArrow {
				height: @_sap_m_IconTabBar_Phone_Compact_TextOnlyContentArrowHeight;
			}
		}

		&.sapMITBTextOnly .sapMITBFilter {
			padding: 0;
		}

		.sapMITBFilter {
			cursor: pointer;
		}
	}

	.sapMITBHead {
		vertical-align: top;
		padding: 0;
	}

	.sapMITH {
		padding: 0 1rem;
	}

	.sapMITHEndOverflow {
		margin-left: 1rem;
	}

	.sapMITHStartOverflow {
		margin-right: 1rem;
	}

	.sapMITBSepLine {
		margin: 0 0.125rem;
	}

	.sapMITBAll {
		margin-right: 0.25rem;

		.sapMITBText {
			padding-right: 0.5rem;
		}

		.sapMITBCount {
			padding: 0 0.25rem;
		}

		&:first-child {
			margin-left: 0.063rem;
		}
	}

	.sapMITBContent {
		padding: 0;
	}
}

.sapUiMedia-Std-Tablet,
.sapUiMedia-Std-Desktop {

	.sapMITH {
		padding: 0 2rem;
	}

	.sapMITHEndOverflow {
		margin-left: 2rem;
	}

	.sapMITHStartOverflow {
		margin-right: 2rem;
	}
}

.sapUiMedia-Std-Desktop {

	.sapUiResponsiveContentPadding.sapMITH {
		padding: 0 3rem;
	}

	/* --------------------------------------------------------- */
	/* Switch paddings when in a split container with side open  */
	/* --------------------------------------------------------- */
	.sapMSplitContainer.sapMSplitContainerShowHide {

		.sapMITH {
			padding: 0 2rem;
		}
	}

	.sapMSplitContainer.sapMSplitContainerHideMode {

		.sapMITH {
			padding: 0 3rem;
		}
	}
	/* ------------------------------------------------ */

	&.sapUiMedia-StdExt-LargeDesktop {

		.sapUiResponsiveContentPadding.sapMITH {
			padding: 0 3rem;
		}
	}
}

.sapMITBItem .sapMITBFilterIcon,
.sapMITBTextOnly .sapMITBItem .sapMITBText,
.sapMITBItem.sapMITBAll {
	position: relative;
}

/* ------------------------------------------------ */
/* Compact size */
/* ------------------------------------------------ */

.sapMITH.sapUiSizeCompact {

	.sapMITBHead {
		padding-top: 0.625rem;

		.sapMITBFilter {
			padding: 0 0.25rem;

			.sapMITBText {
				line-height: 0.875rem;
				padding: @_sap_m_IconTabBar_Compact_FilterTextPadding;
			}
		}
	}

	.sapMITHEndOverflow,
	.sapMITHStartOverflow {

		.sapMITBItem {
			height: 1.5rem;
		}
	}

	.sapMITBVertical {
		height: @_sap_m_IconTabBar_Compact_VerticalHeight;

		.sapMITBTab {
			height: 2rem;
		}

		&.sapMITBFilter .sapMITBContentArrow {
			height: @_sap_m_IconTabBar_Compact_TextOnlyContentArrowHeight;
		}
	}

	&.sapMITBNoText {

		.sapMITBHead .sapMITBVertical {
			height: @_sap_m_IconTabBar_Compact_NoTextVerticalHeight;
		}

		.sapMITBFilter .sapMITBContentArrow {
			width: @_sap_m_IconTabBar_Compact_NoTextContentArrowWidth;
			height: @_sap_m_IconTabBar_Compact_NoTextContentArrowHeight;
		}

		.sapMITBFilter .sapMITBTab {
			width: @_sap_m_IconTabBar_Compact_NoTextTabWidth;
		}

	}

	&.sapMITBInLine.sapMITBTextOnly .sapMITBHead {

		.sapMITBVertical.sapMITBFilter {
			height: @_sap_m_IconTabBar_Compact_InLineVerticalTabHeight;
		}

		.sapMITBText {
			margin: @_sap_m_IconTabBar_Compact_InLineFilterTextMargin;
		}

		.sapMITBContentArrow {
			height: @_sap_m_IconTabBar_Compact_TextOnlyContentArrowHeight;
		}
	}

	&.sapMITBTextOnly .sapMITBHead {
		padding: @_sap_m_IconTabBar_Compact_TextOnlyHeadPadding;

		.sapMITBFilter {
			padding: 0 1rem;

			&:first-child {
				padding-left: 0;
			}

			&:last-child {
				padding-right: 0;
			}

			.sapMITBTab {
				height: 1rem;
				padding: @_sap_m_IconTabBar_Compact_TextOnlyTabPadding;
			}

			.sapMITBText {
				line-height: normal;
				padding: @_sap_m_IconTabBar_Compact_TextOnlyTextPadding;

				.sapMITHShowSubItemsIcon {
					line-height: @_sap_m_IconTabBar_Compact_TextOnlyTextLineHeight;
				}
			}
		}

		.sapMITBVertical {
			height: @_sap_m_IconTabBar_Compact_TextOnlyVerticalHeight;
		}
	}

	.sapMITBFilterIcon {
		border-bottom-left-radius: 2rem;
		border-bottom-right-radius: 2rem;
		-moz-border-radius: 3rem;
		border-top-left-radius: 2rem;
		border-top-right-radius: 2rem;
		font-size: 1rem;
		height: 2rem;
		line-height: 1.9rem;
		width: 2rem;
	}

	.sapMITBSepLine {
		height: 2rem;
		margin: 0 0.125rem;
		width: 0;
		border: 1px solid #e5e5e5;
	}

	.sapMITBAll {
		margin-right: 1.375rem;

		.sapMITBContentArrow {
			height: @_sap_m_IconTabBar_ContentArrowHeight;
		}

		.sapMITBTab {
			line-height: 2rem;
		}

		&.sapMITBHorizontal .sapMITBContentArrow {
			height: @_sap_m_IconTabBar_Compact_HorizontalContentArrowHeight;
		}

		&.sapMITBHorizontal.sapMITBItem:focus {
			&::after {
				bottom: @_sap_m_IconTabBar_Compact_HorizontalFocusBottom;
			}
		}
	}

	.sapMITBHorizontalWrapper {
		height: 2rem;
	}

	.sapMITBContentArrow {
		height: @_sap_m_IconTabBar_Compact_ContentArrowHeight;
	}

	.sapMITBHorizontal {
		height: @_sap_m_IconTabBar_Compact_HorizontalHeight;

		&.sapMITBFilter:first-child .sapMITBFilterWrapper {
			padding-left: 0;
		}

		&.sapMITBFilter .sapMITBFilterWrapper {
			height: @_sap_m_IconTabBar_Compact_TextOnlyVerticalHeight;
			padding: 0 0.5rem;

			.sapMITBHorizontalWrapper {

				.sapMITBCount {
					top: 0;
					padding-left: 0.5rem;
				}

				.sapMITBText {
					top: 0;
					padding-left: 0.5rem;
				}
			}

			.sapMITBContentArrow {
				height: @_sap_m_IconTabBar_Compact_ContentArrowHeight;
				width: 2rem;
				margin: 0;
			}

			.sapMITBTab {
				height: 2rem;
				width: 2rem;
			}
		}

		.sapMITBTab {
			height: 2rem;
		}

	}

	.sapMITBSepIcon {
		height: 2rem;
		line-height: 2rem;
		padding: 0 0.5rem;
	}
}

.sapMITH.sapMITBTextOnly .sapMITBFilterWithItems:not(.sapMITHUnselectable) .sapMITBContentArrow {
	margin-right: -2.3125rem;
}

.sapMITH .sapMITBContentArrow {
	margin: 0 -0.1875rem;
}

/* container background design - translucent */
.sapMITBBackgroundDesignTranslucent .sapMITBContainerContent .sapMITBContent {
	background-color: @_sap_m_IconTabBar_ContentBackgroundTranslucent;
}

/* header background design - translucent */
.sapMITHBackgroundDesignTranslucent {
	background-color: @_sap_m_IconTabBar_HeaderBackgroundTranslucent;
}

/* header background design - transparent */
.sapMITHBackgroundDesignTransparent {
	background: transparent;
}

/* ------------------------------------------------ */
/* IconTabBar Right to left mode */
/* ------------------------------------------------ */

html[dir=rtl] {

	.sapMITBContentArrow {
		z-index: 1;
	}

	.sapMITBTextOnly .sapMITBTab {
		text-align: right;
	}

	.sapMITBHorizontal.sapMITBFilter .sapMITBHorizontalWrapper .sapMITBCount {
		text-align: right;
	}
}

/* ------------------------------------------------ */
/* Shell overrides */
/* ------------------------------------------------ */

.sapUshellShellTabBar .sapMITH,
.sapUshellShellTabBar.sapMITH {

	box-shadow: @_sap_m_IconTabBar_ShellHeaderShadow;
	background-color: var(--sapShell_Navigation_Background);
	min-height: @_sap_m_IconTabBar_HeaderMinHeight;

	.sapMITBSelected .sapMITBContentArrow::after {
		background: var(--sapShell_Navigation_SelectedColor);
		height: var(--sapTab_Selected_Indicator_Dimension);
	}

	.sapMITBFilterDefault {

		&:hover .sapMITBText {
			color: @_sap_m_IconTabBar_ShellSelectedTextColor;
		}

		&.sapMITBSelected {

			&:hover .sapMITBText {
				color: @_sap_m_IconTabBar_ShellSelectedTextColor;
			}

			.sapMITBText {
				color: @_sap_m_IconTabBar_ShellSelectedTextColor;
			}

			.sapMITBFilterIcon {
				background: @_sap_m_IconTabBar_ShellSelectedTextColor;
				color: var(--sapShell_Navigation_Background);
			}
		}

		.sapMITBFilterIcon {
			border-color: @_sap_m_IconTabBar_ShellSelectedTextColor;
			color: @_sap_m_IconTabBar_ShellSelectedTextColor;
		}
	}
	.sapMITBFilter .sapMITBText .sapMITHTextContent {
		vertical-align: inherit;
	}

	.sapMITBFilter.sapMITHUnselectable:not(.sapMITBDisabled) .sapMITHTextContent {
		position: initial;
	}

	.sapMITBFilter {

		.sapMITBText,
		.sapMITBCount {
			color: @_sap_m_IconTabBar_ShellTextColor;
			font-family: var(--sapFontHeaderFamily);
			font-size: @sapMFontMediumSize;
		}

		&.sapMITBSelected .sapMITBCount {
			color: @_sap_m_IconTabBar_ShellSelectedTextColor;
		}
	}

	&:not(.sapMITBTextOnly) {

		.sapMITBFilter {

			.sapMITBText {
				line-height: 0.9375rem;
				padding: 0.3125rem 0 0;
			}
		}

		.sapMITHStartOverflow,
		.sapMITHStartOverflow {

			.sapMITBItem .sapMITBText {
				padding-top: 0.25rem;
			}
		}
	}

	.sapMITHUnselectable {

		&:hover .sapUiIcon {
			color: @_sap_m_IconTabBar_ShellSelectedTextColor;
		}

		.sapUiIcon {
			color: @_sap_m_IconTabBar_ShellTextColor;
		}

		&.sapMITBSelected {

			.sapUiIcon {
				color: @_sap_m_IconTabBar_ShellSelectedTextColor;
			}
		}
	}

	.sapMITBSelected .sapMITBFilterExpandBtn .sapMITBFilterExpandIcon {
		color: @_sap_m_IconTabBar_ShellSelectedTextColor;
	}

	.sapMITBFilterExpandBtn .sapMITBFilterExpandIcon {
		color: @_sap_m_IconTabBar_ShellTextColor;
		background-color: transparent;
		border-color: transparent;
	}

	.sapMITBFilterExpandBtn:hover .sapMITBFilterExpandIcon {
		background: @_sap_m_IconTabBar_Filter_Expand_Shell_BackgroundColor_Hover;
		border-color: @_sap_m_IconTabBar_Filter_Expand_Shell_BorderColor_Hover;
	}

	.sapMITBFilterExpandBtn:focus {
		outline-color: @_sap_m_IconTabBar_ShellButtonOutlineColor;
		background: var(--sapShell_Active_Background);
	}

	.sapMITHStartOverflow,
	.sapMITHEndOverflow {

		.sapMITBItem {
			color: @_sap_m_IconTabBar_ShellTextColor;
			background-color: transparent;
			border-color: @_sap_m_IconTabBar_ShellOverflowBorderColor;

			.sapMITHShowSubItemsIcon {
				color: @_sap_m_IconTabBar_ShellTextColor;
			}

			&:hover {
				background: var(--sapShell_Hover_Background);
				border-color: @_sap_m_IconTabBar_ShellOverflowHoverBorderColor;
			}

			&:active {
				background: @sapUiShellNavigationActiveBackground;
				border-color: @_sap_m_IconTabBar_ShellOverflowPressedBorderColor;
			}
		}
	}

	html.sap-desktop & {

		.sapMITBItem:not(.sapUiDnDDragging):focus .sapMITBFilterIcon,
		.sapMITBTextOnly .sapMITBItem:not(.sapUiDnDDragging):focus .sapMITBText,
		.sapMITBItem:not(.sapUiDnDDragging):focus.sapMITBAll,
		.sapMITBItem:not(.sapUiDnDDragging):focus .sapMITBText,
		.sapMITHStartOverflow .sapMITBItem:not(.sapUiDnDDragging):focus,
		.sapMITHEndOverflow .sapMITBItem:not(.sapUiDndDragging):focus {

			&::after {
				border-color: @_sap_m_IconTabBar_ShellTextColor;
			}
		}
	}
}

/* ------------------------------------------------ */
/* IconTabBar Badge */
/* ------------------------------------------------ */

.sapMITH {

	.sapMITBFilterWithItems .sapMITBFilterExpandBtn {
		margin-left: 0.625rem; /* extra margin to create space for badge */
	}
}

.sapMITH,
.sapMITBFilterPopover {

	.sapMITBFilterBadgeMotion .sapMBadgeAttention.sapMBadgeAnimationAdd {
		animation-name: sapMBadgeAppearMotion;
		animation-duration: 1s;
		animation-timing-function: linear;
	}

	/* small badge */
	&.sapUiSizeCompact,
	.sapMITBText,
	.sapMITFExpandButtonBadge {

		.sapMBadgeAttention {
			width: 0.375rem;
			height: 0.375rem;
			border-width: 0.063rem;

			&.sapMITBFilterBadgeMotion.sapMBadgeAnimationAdd {
				animation-name: sapMBadgeAppearMotionSmall;
			}
		}
	}

	.sapMITBText .sapMBadgeAttention {
		top: 0;
		right: -0.6rem;
	}

	.sapMITBSelectList .sapMITBText .sapMBadgeAttention {
		right: 0.4375rem;
	}

	.sapMITBFilterIcon .sapMBadgeAttention,
	.sapMITBCount .sapMBadgeAttention {
		top: 0;
		right: 0;
	}

	.sapMITBAll {

		.sapMITBCount .sapMBadgeAttention {
			top: 0.2rem;

			.sapUiMedia-Std-Phone & {
				right: -0.25rem;
			}
		}
	}

	&.sapUiSizeCompact {

		.sapMITBFilterIcon .sapMBadgeAttention {
			top: -0.063rem;
			right: -0.063rem;
		}

		.sapMITBCount .sapMBadgeAttention {
			top: -0.125rem;
			right: -0.125rem;
		}

		.sapMITFExpandButtonBadge {
			top: @_sap_m_IconTabBar_Compact_FilterExpandButtonBadgeTop;
		}
	}

	.sapMITFExpandButtonBadge {
		top: @_sap_m_IconTabBar_FilterExpandButtonBadgeTop;
		left: 0.375rem;
	}

	.sapMITHOverflowVisible .sapMITFExpandButtonBadge {
		top: 0;
	}

	.sapMITBFilterNeutral .sapMBadgeAttention {
		background-color: @_sap_m_IconTabBar_Badge_Neutral_Background;
	}

	.sapMITBFilterPositive .sapMBadgeAttention {
		background-color: @_sap_m_IconTabBar_Badge_Positive_Background;
	}

	.sapMITBFilterCritical .sapMBadgeAttention {
		background-color: @_sap_m_IconTabBar_Badge_Critical_Background;
	}

	.sapMITBFilterNegative .sapMBadgeAttention {
		background-color: @_sap_m_IconTabBar_Badge_Negative_Background;
	}
}

html[data-sap-ui-animation='off'] {

	.sapMITH,
	.sapMITBFilterPopover {

		.sapMITBFilterBadgeMotion .sapMBadgeAttention.sapMBadgeAnimationAdd {
			animation: none;
		}
	}
}

.sapMITH {

	.sapMITHStartOverflow,
	.sapMITHEndOverflow {

		.sapMITFExpandButtonBadge {
			left: 0;

			.sapMBadgeAttention {
				top: -1px;
				right: -4px;
				width: 0.5rem;
				height: 0.5rem;
				border-width: 0.125rem;
			}
		}
	}
}

/* Responsive paddings when the IconTabBar is inside a DynamicPage */

.sapFDynamicPage-Std-Phone {

	.sapUiResponsiveContentPadding {

		.sapMITH,
		&.sapMITH {
			padding: 0 1rem !important;
		}

		.sapMITBContent {
			padding: 0 1rem !important;
		}
	}
}

.sapFDynamicPage-Std-Tablet,
.sapFDynamicPage-Std-Desktop {

	.sapUiResponsiveContentPadding {

		.sapMITH,
		&.sapMITH {
			padding: 0 2rem !important;
		}

		.sapMITBContent {
			padding: 1rem 2rem !important;
		}
	}
}

.sapFDynamicPage-Std-Desktop-XL {

	.sapUiResponsiveContentPadding {

		.sapMITH,
		&.sapMITH {
			padding: 0 3rem !important;
		}

		.sapMITBContent {
			padding: 1rem 3rem !important;
		}
	}
}
/* END "IconTabBar.less" */

/* START "IconTabBarSelectList.less" */
/* =========================================== */
/* CSS for control sap.m/IconTabBarSelectList  */
/* Base theme                                  */
/* =========================================== */

@_sap_m_IconTabBar_SelectListItem_PaddingLeft: 0.5rem;
@_sap_m_IconTabBar_SelectListItem_PaddingLeftAdditional: 0;
@_sap_m_IconTabBar_ListBorderRadius: 0px;
@_sap_m_IconTabBar_List_Focus_Offset: 1px;

.sapMITBSelectList {
	margin: 0;
	padding: 0;
	list-style: none;

	&:focus {
		outline: none;
	}
}

.sapMITBSelectList .sapMITBSelectItem {
	position: relative;
	width: 100%;
	box-sizing: border-box;
	display: flex;
	align-items: center;
	cursor: pointer;

	.sapUiIcon {
		cursor: pointer;
	}

	&.sapMITHUnselectable {
		cursor: default;
	}
}

.sapMITBSelectList .sapMITBSelectItem:not(.sapMITBSep) {
	padding: 0.75rem 0 0.75rem;
}

.sapMITBSelectList .sapMITBSep {
	margin: 0;
	height: 1rem;
	justify-content: center;
	padding-right: 0;

	&:first-child {
		margin-top: 0.5rem;
	}

	&.sapMITBSepLine {
		margin-left: 0;
		border: 0;
		position: relative;
		height: 0.25rem;
		padding-right: 17px; /* big enough to cover the case when a scrollbar appears */

		&::before {
			width: 1rem;
			content: '';
		}

		&::after {
			content: '';
			width: 100%;
			border-top: 0.0625rem solid var(--sapGroup_TitleBorderColor);
		}
	}

	.sapMITBSepIcon {
		height: 1rem;
		line-height: 1rem;
		overflow: hidden;
	}
}

.sapMITBSelectItem .sapMITBSelectItemIcon {
	outline: none;
	-webkit-user-select: none; /* Chrome all / Safari all */
	-moz-user-select: none; /* Firefox all */
	user-select: none;
}

.sapMITBSelectItem .sapMITBText {
	display: block;
	padding: 0 1rem 0 0;
	pointer-events: none;
	min-width: 0; /* chrome flex overflow issue */
	overflow: visible;
	white-space: normal;
	font-size: @sapMFontMediumSize;
}

.sapMITBSelectListTextOnly .sapMITBSelectItem .sapMITBText {
	padding-left: 1rem;
}

.sapMITBSelectItem:not(.sapMITBSep) .sapUiIcon {
	font-size: 1.125rem;
	min-width: 2.75rem;
}

/* Compact size */
.sapUiSizeCompact {

	.sapMITBSelectList .sapMITBSelectItem:not(.sapMITBSep) {
		padding: 0.5rem 0 0.5rem;
	}

	.sapMITBSelectItem:not(.sapMITBSep) .sapUiIcon {
		font-size: 1rem;
		min-width: 2rem;
	}

	.sapMITBSelectList .sapMITBSep {

		&.sapMITBSepLine {
			padding-right: 0.5rem;

			&::before {
				width: 0.5rem;
			}
		}
	}
}

.sapMITBSelectItem {
	background-color: var(--sapList_Background);
}

.sapMITBSelectItem {

	&.sapMITBFilterNeutral,
	&.sapMITBFilterNegative,
	&.sapMITBFilterPositive,
	&.sapMITBFilterCritical {

		.sapMITBText {
			color: inherit;
		}
	}
}

.sapMITBSelectItem {

	&.sapMITBFilterDefault {

		.sapMITBInlineIcon {
			color: var(--sapList_TextColor);
		}
	}
}

.sapMITBSelectItem:not(.sapMITHUnselectable):hover {
	background-color: var(--sapList_Hover_Background);
}

.sapMITBSelectList .sapMITBSelectItem:not(.sapMITHUnselectable) {

	&:not(.sapUiDnDDragging):active,
	&.sapMITBSelectItemSelected:not(.sapUiDnDDragging):active {
		background-color: var(--sapList_Active_Background);

		& > .sapMITBText {
			color: var(--sapList_Active_TextColor);
		}

		& > .sapUiIcon {
			color: var(--sapList_Active_TextColor);
		}
	}
}

.sapMITBSelectItem:focus {
	outline: none;
}

.sapMITBSelectItem:focus::before {
	border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
	position: absolute;
	content: ' ';
	top: @_sap_m_IconTabBar_List_Focus_Offset;
	right:  @_sap_m_IconTabBar_List_Focus_Offset;
	bottom:  @_sap_m_IconTabBar_List_Focus_Offset;
	left:  @_sap_m_IconTabBar_List_Focus_Offset;
	z-index: 2;
	border-radius: @_sap_m_IconTabBar_ListBorderRadius;
	pointer-events: none;
}

.sapMITBSelectItem.sapMITBSelectItemSelected {
	background-color: var(--sapList_SelectionBackgroundColor);
}

.sapMITBSelectItem.sapMITBSelectItemSelected:hover {
	background-color: @sapUiListSelectionHoverBackground;
}

.sapUiDnDIndicator {
	z-index: 100;
}
/* END "IconTabBarSelectList.less" */

/* START "Illustration.less" */
/* =================================== */
/* CSS for control sap.m/Illustration  */
/* Base theme                          */
/* ============================= ===== */

:root {
	--sapIllus_BrandColorPrimary: var(--sapContent_Illustrative_Color1);
	--sapIllus_BrandColorSecondary: var(--sapContent_Illustrative_Color2);
	--sapIllus_StrokeDetailColor: var(--sapContent_Illustrative_Color4);
	--sapIllus_Layering1: var(--sapContent_Illustrative_Color5);
	--sapIllus_Layering2: var(--sapContent_Illustrative_Color6);
	--sapIllus_BackgroundColor: var(--sapContent_Illustrative_Color7);
	--sapIllus_ObjectFillColor: var(--sapContent_Illustrative_Color8);
	--sapIllus_AccentColor: var(--sapContent_Illustrative_Color3);
	--sapIllus_NoColor: none;
	--sapIllus_PatternShadow: ~"url(#sapIllus_PatternShadow)";
	--sapIllus_PatternHighlight: ~"url(#sapIllus_PatternHighlight)";

	.generate-illustrative-colors (20);
}

.generate-illustrative-colors (@colorsCount, @i: 1) when (@i =< @colorsCount) {
	@illusColorLESSvar: 'sapContent_Illustrative_Color@{i}';
	--sapContent_Illustrative_Color@{i}: @@illusColorLESSvar;
	.generate-illustrative-colors(@colorsCount, (@i + 1));
}

.sapIllus_MaskTypeAlpha {
	mask-type: alpha;
}

.sapIllus_BrandColorPrimary {
	fill: var(--sapIllus_BrandColorPrimary);
}

.sapIllus_BrandColorSecondary {
	fill: var(--sapIllus_BrandColorSecondary);
}

.sapIllus_StrokeDetailColor {
	fill: var(--sapIllus_StrokeDetailColor);
}

.sapIllus_Layering1 {
	fill: var(--sapIllus_Layering1);
}

.sapIllus_Layering2 {
	fill: var(--sapIllus_Layering2);
}

.sapIllus_BackgroundColor {
	fill: var(--sapIllus_BackgroundColor);
}

.sapIllus_ObjectFillColor {
	fill: var(--sapIllus_ObjectFillColor);
}

.sapIllus_AccentColor {
	fill: var(--sapIllus_AccentColor);
}

.sapIllus_NoColor {
	fill: var(--sapIllus_NoColor);
}

.sapIllus_PatternShadow {
	fill: var(--sapIllus_PatternShadow);
}

.sapIllus_PatternHighlight {
	fill: var(--sapIllus_PatternHighlight);
}
/* END "Illustration.less" */

/* START "IllustratedMessage.less" */
/* ======================================== */
/* CSS for control sap.f/IllustratedMessage */
/* Base theme                               */
/* ======================================== */

.sapMIllustratedMessage {
	width: 100%;
	margin: 0 auto;
	text-align: center;
	padding: 1rem;
	box-sizing: border-box;
	justify-content: center;

	.sapMIllustratedMessageMainContent {
		display: flex;
		align-items: center;
		flex-direction: column;
		min-height: 0;
	}

	figcaption {
		margin: inherit;
	}

	.sapMTitle {
		font-family: var(--sapFontHeaderFamily);
		color: var(--sapGroup_TitleTextColor);
		display: block;
	}

	.sapMIllustratedMessageDescription {
		font-size: @sapFontSize;
		font-family: var(--sapFontFamily);
		color: var(--sapTextColor);
	}

	&.sapMIllustratedMessageScalable {
		height: 100%;
		display: flex;
		align-items: center;
		flex-direction: column;
	}

	&.sapMIllustratedMessage-Base {

		figcaption {
			max-width: 10rem;
		}

		.sapMIllustratedMessageAdditionalContent,
		.sapMIllustration {
			display: none;
		}
	}

	&.sapMIllustratedMessage-Dot {
		padding: 0.25rem;

		.sapMIllustratedMessageMainContent {
			display: inline-flex;
			justify-content: center;
			flex-direction: row;
		}

		figcaption {
			display: grid;
			max-width: 18rem;
		}

		.sapMIllustration {
			flex-shrink: 0;
			width: 2.813rem;
			height: 2.813rem;
			margin-right: 0.25rem;
			align-self: baseline;
		}

		.sapMTitle {
			font-size: var(--sapFontHeader5Size);
			margin-bottom: 0.25rem;
		}

		.sapMIllustratedMessageDescription {
			margin-bottom: 0.25rem;
		}

		.sapMIllustratedMessageAdditionalContent {
			margin: 0.25rem 0;
			padding-left: 3.063rem; /* width of dot svg + its margin right */
		}
	}

	&.sapMIllustratedMessage-Spot {
		padding: 0.5rem;

		figcaption {
			max-width: 16.25rem;
		}

		.sapMIllustration {
			width: 8rem;
			height: 8rem;
			margin-bottom: 0.5rem;
		}

		.sapMTitle {
			font-size: var(--sapFontHeader4Size);
			margin-bottom: 0.5rem;
		}

		.sapMIllustratedMessageDescription {
			margin-bottom: 0.5rem;
		}

		.sapMIllustratedMessageAdditionalContent {
			margin: 0.5rem 0;
		}
	}

	&.sapMIllustratedMessage-Dialog {

		figcaption {
			max-width: 20rem;
		}

		.sapMIllustration {
			width: 10rem;
			height: 10rem;
			margin: 1rem 0;
		}

		.sapMTitle {
			margin-bottom: 0.5rem;
			font-size: var(--sapFontHeader3Size);
		}

		.sapMIllustratedMessageDescription {
			margin-bottom: 0.5rem;
		}

		.sapMIllustratedMessageAdditionalContent {
			margin-top: 0.5rem;
			margin-bottom: 1rem;
		}
	}

	&.sapMIllustratedMessage-Scene {

		figcaption {
			max-width: 30rem;
		}

		.sapMIllustration {
			width: 20rem;
			height: 15rem;
			margin: 2rem 0;
		}

		.sapMTitle {
			margin-bottom: 1rem;
			font-size: var(--sapMFontHeader2Size);
		}

		.sapMIllustratedMessageDescription {
			margin-bottom: 0.5rem;
			font-size: var(--sapMFontLargeSize);
		}

		.sapMIllustratedMessageAdditionalContent {
			margin: 1rem 0;
		}
	}
}
/* END "IllustratedMessage.less" */

/* START "Image.less" */
/* ============================ */
/* CSS for control sap.m/Image  */
/* Base theme                   */
/* ============================ */

.sapMImg {
	border: none;

	/* fix the highlight color on tap*/
	-webkit-tap-highlight-color: rgba(255, 255, 255, 0);
	-webkit-user-select: none;
	-moz-user-select: none;
	-webkit-touch-callout: none;
}

span.sapMImg {
	display: inline-block;
}

.sapMImg.sapMNoImg {
	visibility: hidden;
}

/* fix chrome browser yellow border on focus problem*/
.sapMImg:focus {
	outline: none;
}

.sapMImgFocusable:focus {
	outline: 0.0625rem dotted @sapUiContentFocusColor;
    outline-offset: 0.0625rem;
	.sapUiRespGrid & {
		z-index: 100;
	}
}

html:not(.sap-desktop) .sapMImgFocusable:focus {
	outline: none;
}


/* ==========================================================================
   Image opening a LightBox
  ========================================================================== */

.sapMLightBoxImage {
	position: relative;
	display: inline-block;

	.sapMImg {
		display: block;
	}

	.sapMLightBoxMagnifyingGlass {
		line-height: normal;
	}
}

// Apply LightBox Magnifying glass mixin to control
.lightBoxMagnifyingGlassMixing(~".sapMLightBoxImage", ~".sapMLightBoxMagnifyingGlass");
/* END "Image.less" */

/* START "ImageContent.less" */
/* =================================== */
/* CSS for control sap.m/ImageContent  */
/* Base theme                          */
/* =================================== */

.sapMImageContent {
	position: relative;
	height: 3.875rem;
	width: 9rem;
	outline: none;
}

.sapMImageContentImageIcon {
	display: block;
	position: absolute;
	right: 0;
	bottom: 0;
	max-height: 3.5rem;
}

.sapMGT .sapMImageContentImageIcon {
	left: 0;
	right: auto;
}

html.sap-desktop .sapMImageContent:focus {
	outline-color: var(--sapContent_FocusColor);
	outline-style: var(--sapContent_FocusStyle);
	outline-width: var(--sapContent_FocusWidth);
}

html.sap-desktop .sapMImageContent:focus:not([tabindex]) {
	outline: none;
}

.sapMImageContentImageIcon.sapUiIcon {
	font-size: 1.75rem;
	color: var(--sapTile_IconColor);
	padding-bottom: 0.675rem;
}

.sapMTileSmallPhone {

	.sapMImageContent {
		height: 3.438rem;
		width: 8rem;
	}

	.sapMImageContentImageIcon {
		max-height: 3rem;
	}

	.sapMImageContentImageIcon.sapUiIcon {
		font-size: var(--sapFontHeader3Size);;
		padding-bottom: 0.25rem;
	}
}
/* END "ImageContent.less" */

/* START "Input.less" */
/* ============================ */
/* CSS for control sap.m/Input  */
/* Base theme                   */
/* ============================ */

:root {
	--sPopoverMaxWidth: 640;
}

.sapMListFocus .sapMLIBFocused {
	outline: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
	outline-offset: -0.125rem;
}

.sapMInputDescriptionText {
	padding-left: 0.5rem;
	font-size: var(--sapFontSize);
	line-height: normal;
	color: var(--sapField_TextColor);
	font-family: var(--sapFontFamily);
	font-style: normal;
	font-weight: normal;
	white-space: nowrap;
	text-overflow: ellipsis;
	overflow: hidden;
}

.sapMInputWithDescription {
	white-space: nowrap;
}

.sapMInputDescriptionWrapper {
	display: inline-flex;
}

/* Input with description */
.sapMInputBaseHasEndIcons,
.sapMInputBaseHasBeginIcons {

	.sapMInputDescriptionWrapper {
		max-width: ~"calc(100% - (@{_InputBase_Control_OneIcon_MinWidth}))";

		.sapMInputBaseState:not(.sapMInputBaseContentWrapperSuccess) & {
			max-width: ~"calc(100% - (@{_InputBase_Control_OneIcon_State_MinWidth}))";
		}
	}
}

.sapMInputBaseHasEndIcons.sapMInputBaseHasBeginIcons {

	.sapMInputDescriptionWrapper {
		max-width: ~"calc(100% - (@{_InputBase_Control_TwoIcons_MinWidth}))";

		.sapMInputBaseState:not(.sapMInputBaseContentWrapperSuccess) & {
			max-width: ~"calc(100% - (@{_InputBase_Control_TwoIcons_State_MinWidth}))";
		}
	}
}

.sapMInputBase:not(.sapMInputBaseHasEndIcons):not(.sapMInputBaseHasBeginIcons) {

	.sapMInputDescriptionWrapper {
		max-width: ~"calc(100% - (@{_InputBase_Control_MinWidth}))";

		.sapMInputBaseState:not(.sapMInputBaseContentWrapperSuccess) & {
			max-width: ~"calc(100% - (@{_InputBase_Control_State_MinWidth}*2))";
		}
	}
}

.sapUiSizeCompact {

	.sapMInputBaseHasEndIcons,
	.sapMInputBaseHasBeginIcons {

		.sapMInputDescriptionWrapper {
			max-width: ~"calc(100% - (@{_InputBase_Control_OneIcon_MinWidth_Compact}))";

			.sapMInputBaseState:not(.sapMInputBaseContentWrapperSuccess) & {
				max-width: ~"calc(100% - (@{_InputBase_Control_OneIcon_State_MinWidth_Compact}))";
			}
		}
	}

	.sapMInputBaseHasEndIcons.sapMInputBaseHasBeginIcons {

		.sapMInputDescriptionWrapper {
			max-width: ~"calc(100% - (@{_InputBase_Control_TwoIcons_MinWidth_Compact}))";

			.sapMInputBaseState:not(.sapMInputBaseContentWrapperSuccess) & {
				max-width: ~"calc(100% - (@{_InputBase_Control_TwoIcons_State_MinWidth_Compact}))";
			}
		}
	}

	.sapMInputBase:not(.sapMInputBaseHasEndIcons):not(.sapMInputBaseHasBeginIcons) {

		.sapMInputDescriptionWrapper {
			max-width: ~"calc(100% - (@{_InputBase_Control_MinWidth_Compact}))";

			.sapMInputBaseState:not(.sapMInputBaseContentWrapperSuccess) & {
				max-width: ~"calc(100% - (@{_InputBase_Control_State_MinWidth_Compact}*2))";
			}
		}
	}
}

.sapMInputHighlight {
	font-weight: bold;
}

/* Overrides due to experimental flex layouting after IE drop */

.sapMInput.sapMInputBaseHasEndIcons .sapMInputBaseInner,
.sapMInput.sapMInputBaseHasBeginIcons .sapMInputBaseInner {
	width: auto;
	flex: 1
}

.sapMInput .sapMInputBaseContentWrapper {
	display: inline-flex;
}
/* END "Input.less" */

/* START "InputBase.less" */
/* ================================ */
/* CSS for control sap.m/InputBase  */
/* Base theme                       */
/* ================================ */

@_sap_m_InputBase_InnerMinWidth: 3rem;
@_sap_m_InputBase_InnerPadding: 0 0.75rem;
@_sap_m_InputBase_DisabledOpacity: 0.5;
@_sap_m_InputBase_IconLineHeight: 2.375rem;
@_sap_m_InputBase_IconWidth: 2.375rem;
@_sap_m_InputBase_Compact_IconWidth: 2rem;
@_sap_m_InputBase_IconMinWidth: 2.375rem;
@_sap_m_InputBaseWrapper_Sum_Border: 0.125rem;
@_sap_m_InputBase_InnerMinWidth_Compact: 2rem;
@_sap_m_InputBase_Compact_InnerPadding: 0 0.5rem;
@_sap_m_InputBase_FocusOffset: 1px;
@_sap_m_InputBase_FocusOffset_Busy: @_sap_m_InputBase_FocusOffset + 1px;
@_sap_m_InputBase_FocusBorderRadius: 0;


/* MIN-WIDTH variables */
/* rounding values to 2 symbols, because of IE and Edge */
@_InputBase_Control_MinWidth: round((@_sap_m_InputBase_InnerMinWidth + @_sap_m_InputBaseWrapper_Sum_Border), 2);
@_InputBase_Control_OneIcon_MinWidth: round((@_InputBase_Control_MinWidth + @_sap_m_InputBase_IconWidth), 2);
@_InputBase_Control_TwoIcons_MinWidth: round((@_InputBase_Control_MinWidth + @_sap_m_InputBase_IconWidth * 2), 2);
@_InputBase_Control_State_MinWidth: round((@_InputBase_Control_MinWidth + @_sap_m_InputBaseWrapper_Sum_Border), 2);
@_InputBase_Control_TwoIcons_State_MinWidth: round((@_InputBase_Control_TwoIcons_MinWidth + @_sap_m_InputBaseWrapper_Sum_Border), 2);
@_InputBase_Control_OneIcon_State_MinWidth: round((@_InputBase_Control_OneIcon_MinWidth + @_sap_m_InputBaseWrapper_Sum_Border), 2);

@_InputBase_Control_MinWidth_Compact: round((@_sap_m_InputBase_InnerMinWidth_Compact + @_sap_m_InputBaseWrapper_Sum_Border), 2);
@_InputBase_Control_OneIcon_MinWidth_Compact: round((@_InputBase_Control_MinWidth_Compact + @_sap_m_InputBase_Compact_IconWidth), 2);
@_InputBase_Control_TwoIcons_MinWidth_Compact: round((@_InputBase_Control_MinWidth_Compact + @_sap_m_InputBase_Compact_IconWidth * 2), 2);
@_InputBase_Control_State_MinWidth_Compact: round((@_InputBase_Control_MinWidth_Compact + @_sap_m_InputBaseWrapper_Sum_Border), 2);
@_InputBase_Control_TwoIcons_State_MinWidth_Compact: round((@_InputBase_Control_TwoIcons_MinWidth_Compact + @_sap_m_InputBaseWrapper_Sum_Border), 2);
@_InputBase_Control_OneIcon_State_MinWidth_Compact: round((@_InputBase_Control_OneIcon_MinWidth_Compact + @_sap_m_InputBaseWrapper_Sum_Border), 2);

.sapMInputBase {
	height: var(--sapElement_Height);
	background: transparent;
	position: relative;
	display: inline-block;
	vertical-align: top;
	outline: none;
	box-sizing: border-box;
	line-height: 0;
}

.sapMInputBaseHeightMargin {
	margin: 0.25rem 0;
}

/* handle min-width calculation */
.sapMInputBaseHasEndIcons,
.sapMInputBaseHasBeginIcons {

	.sapMInputBaseContentWrapper {
		min-width: @_InputBase_Control_OneIcon_MinWidth;

		&.sapMInputBaseContentWrapperState:not(.sapMInputBaseContentWrapperSuccess) {
			min-width: @_InputBase_Control_OneIcon_State_MinWidth;
		}
	}
}

.sapMInputBaseHasEndIcons.sapMInputBaseHasBeginIcons {

	.sapMInputBaseContentWrapper {
		min-width: @_InputBase_Control_TwoIcons_MinWidth;

		&.sapMInputBaseContentWrapperState:not(.sapMInputBaseContentWrapperSuccess) {
			min-width: @_InputBase_Control_TwoIcons_State_MinWidth;
		}
	}
}

.sapMInputBase:not(.sapMInputBaseHasEndIcons):not(.sapMInputBaseHasBeginIcons) {

	.sapMInputBaseContentWrapper {
		min-width: @_InputBase_Control_MinWidth;

		&.sapMInputBaseContentWrapperState:not(.sapMInputBaseContentWrapperSuccess) {
			min-width: @_InputBase_Control_State_MinWidth;
		}
	}
}

.sapMInputBaseInner {
	background: transparent;
	border: none;
	font-style: normal;
	-webkit-appearance: none;
	-moz-appearance: textfield;
	font-size: var(--sapFontSize);
	font-family: var(--sapFontFamily);
	color: var(--sapField_TextColor);
	line-height: normal;
	padding: @_sap_m_InputBase_InnerPadding;
	height: 100%;
	box-sizing: border-box;
	min-width: @_sap_m_InputBase_InnerMinWidth;
	overflow: hidden;
	text-overflow: ellipsis;
	vertical-align: inherit;
}

.sapMInputBaseReadonly .sapMInputBaseIconContainer {
	height: 0;
}

.sapMInputBase:not(.sapMInputBaseHasEndIcons):not(.sapMInputBaseHasBeginIcons) .sapMInputBaseInner,
.sapMInputBaseNoWidth .sapMInputBaseInner {
	width: 100%;
}

.sapMInputBaseInner {

	.sapMInputBaseHasEndIcons &,
	.sapMInputBaseHasBeginIcons & {
		width: calc(~'100% - @{_sap_m_InputBase_IconWidth}');
	}

	.sapMInputBaseHasEndIcons.sapMInputBaseHasBeginIcons & {
		width: calc(~'100% - @{_sap_m_InputBase_IconWidth}*2');
	}
}

.sapMInputBaseContentWrapper {
	height: 100%;
	box-sizing: border-box;
	display: inline-block;
	position: relative;
	overflow: hidden;
	border-radius: 0;
	vertical-align: middle;
	margin: 0;
	background: var(--sapField_BackgroundStyle);
	background-color: var(--sapField_Background);
	border: var(--sapField_BorderWidth) var(--sapField_BorderStyle) var(--sapField_BorderColor);
	border-radius: var(--sapField_BorderCornerRadius);
}

.sapMInputBaseContentWrapper:not(.sapMInputBaseReadonlyWrapper):not(.sapMInputBaseContentWrapperState) {
	box-shadow: var(--sapField_Shadow);
}

.sapMILI-CTX .sapMInputBaseContentWrapperError,
.sapMInputBaseContentWrapperError {
	background: var(--sapField_InvalidBackgroundStyle);
	background-color: var(--sapField_InvalidBackground);
	border: var(--sapField_InvalidBorderWidth) var(--sapField_InvalidBorderStyle) var(--sapField_InvalidColor);
	box-shadow: var(--sapField_InvalidShadow);
}

.sapMInputBaseContentWrapperWarning {
	background: var(--sapField_WarningBackgroundStyle);
	background-color: var(--sapField_WarningBackground);
	border: var(--sapField_WarningBorderWidth) var(--sapField_WarningBorderStyle) var(--sapField_WarningColor);
	box-shadow: var(--sapField_WarningShadow);
}

.sapMInputBaseContentWrapperSuccess {
	background: var(--sapField_SuccessBackgroundStyle);
	background-color: var(--sapField_SuccessBackground);
	border: var(--sapField_SuccessBorderWidth) var(--sapField_SuccessBorderStyle) var(--sapField_SuccessColor);
	box-shadow: var(--sapField_SuccessShadow);
}

.sapMInputBaseContentWrapperInformation {
	background: var(--sapField_InformationBackgroundStyle);
	background-color: var(--sapField_InformationBackground);
	border: var(--sapField_InformationBorderWidth) var(--sapField_InformationBorderStyle) var(--sapField_InformationColor);
	box-shadow: var(--sapField_InformationShadow);
}

.sapMInputBaseIconContainer {
	display: inline-block;
	vertical-align: inherit;
	height: 100%;
	width: auto;
	outline: none;
}

.sapMInputBaseHasBeginIcons .sapMInputBaseInner {
	padding-left: 0.25rem;
}

.sapMInputBaseHasEndIcons .sapMInputBaseInner {
	padding-right: 0.25rem;
}

.sapMInputBaseIcon {
	width: @_sap_m_InputBase_IconWidth;
	min-width: @_sap_m_InputBase_IconMinWidth;
	height: 100%;
	cursor: pointer;
	background: transparent;
	-webkit-user-select: none;
	-moz-user-select: none;
	user-select: none;
}

.sapMInputBaseIconContainer .sapUiIcon.sapMInputBaseIcon {
	line-height: @_sap_m_InputBase_IconLineHeight;
}

.sapMFocus.sapMInputBase .sapMInputBaseContentWrapper:not(.sapMInputBaseDisabledWrapper) {

	&::before {
		border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
		border-radius: @_sap_m_InputBase_FocusBorderRadius;
		content: '';
		position: absolute;
		top: @_sap_m_InputBase_FocusOffset;
		left: @_sap_m_InputBase_FocusOffset;
		right: @_sap_m_InputBase_FocusOffset;
		bottom: @_sap_m_InputBase_FocusOffset;
		z-index: 2;
		pointer-events: none;
	}
}

.sapMInputBase .sapUiLocalBusyIndicator:focus::after {
	top: @_sap_m_InputBase_FocusOffset_Busy;
	left: @_sap_m_InputBase_FocusOffset_Busy;
	right: @_sap_m_InputBase_FocusOffset_Busy;
	bottom: @_sap_m_InputBase_FocusOffset_Busy;
	border-radius: @_sap_m_InputBase_FocusBorderRadius;
}

.sapMInputBaseInner::-webkit-input-placeholder {
	font-size: @sapMFontMediumSize;
	font-style: italic;
	padding: 0 .125rem;
}

.sapMInputBaseInner::-moz-placeholder {
	font-size: @sapMFontMediumSize;
	font-style: italic;
	padding: 0 .125rem;
}

.sapMInputBaseInner::-webkit-outer-spin-button,
.sapMInputBaseInner::-webkit-inner-spin-button {
	-webkit-appearance: none;
	margin: 0;
}

.sapMInputBaseInner:focus {
	outline: none;
	outline-width: 0;
}

.sapMPointer>.sapMInputBaseInner {
	cursor: pointer;
}

.sapMInputBaseDisabled {
	opacity: @_sap_m_InputBase_DisabledOpacity;

	.sapMInputBaseIcon {
		cursor: default;
	}
}

.sapMInputBaseInner[placeholder] {
	text-overflow: ellipsis;
}

.sapMInputBaseInner::-webkit-input-placeholder {
	color: var(--sapField_PlaceholderTextColor);
	text-overflow: ellipsis !important;
	/* important is necessary to overwrite inline style of shadow dom */
}

.sapMInputBaseInner::-moz-placeholder {
	color: var(--sapField_PlaceholderTextColor);
}

input.sapMInputBaseInner:invalid {
	/* disable native browser input validation visualization e.g. for type email or number */
	box-shadow: none;
}

/*read only*/

.sapMInputBaseReadonly:not(.sapMInputBaseDisabled) .sapMInputBaseContentWrapper {
	border-color: var(--sapField_ReadOnly_BorderColor);
	border-style: var(--sapField_ReadOnly_BorderStyle);
	background: var(--sapField_ReadOnly_BackgroundStyle);
	background-color: var(--sapField_ReadOnly_Background);
}

.sapMInputBaseInner:not(.sapMInputBaseReadonlyWrapper):not(.sapMInputBaseContentWrapperState):focus {
	border-color: var(--sapField_Focus_BorderColor);
}

.sapMInputBase:not(.sapMFocus) .sapMInputBaseContentWrapper:not(.sapMInputBaseReadonlyWrapper):not(.sapMInputBaseContentWrapperState):not(.sapMInputBaseDisabledWrapper) {

	&:hover {
		background: var(--sapField_Hover_BackgroundStyle);
		background-color: var(--sapField_Hover_Background);
		border-color: var(--sapField_Hover_BorderColor);
		box-shadow: var(--sapField_Hover_Shadow);
	}
}

.sapMInputBaseContentWrapper:not(.sapMInputBaseReadonlyWrapper):not(.sapMInputBaseContentWrapperState):not(.sapMInputBaseDisabledWrapper) {

	&:hover {
		border-color: var(--sapField_Hover_BorderColor);
	}

	&:active {
		background: @sapUiFieldFocusBackground;
		border-color: var(--sapField_Hover_BorderColor);
	}
}

.sapMInputBase:not(.sapMFocus) .sapMInputBaseContentWrapperState:hover,
.sapMInputBase:not(.sapMFocus).sapMInputBaseIconPressed .sapMInputBaseContentWrapperState {

	&.sapMInputBaseContentWrapperWarning:hover,
	&.sapMInputBaseContentWrapperWarning {
		box-shadow: var(--sapField_Hover_WarningShadow);
	}

	&.sapMInputBaseContentWrapperSuccess:hover,
	&.sapMInputBaseContentWrapperSuccess {
		box-shadow: var(--sapField_Hover_SuccessShadow);
	}

	&.sapMInputBaseContentWrapperError:hover,
	&.sapMInputBaseContentWrapperError {
		box-shadow: var(--sapField_Hover_InvalidShadow);
	}

	&.sapMInputBaseContentWrapperInformation:hover,
	&.sapMInputBaseContentWrapperInformation {
		box-shadow: var(--sapField_Hover_InformationShadow);
	}
}

.sapMInputBase:not(.sapMInputBaseDisabled) {

	&.sapMInputBaseIconPressed .sapMInputBaseIcon,
	.sapMInputBaseIcon:active,
	.sapMInputBaseIcon:hover {
		box-shadow: var(--sapField_Hover_Shadow);
	}

	&.sapMInputBaseIconPressed .sapMInputBaseContentWrapperError .sapMInputBaseIconContainer .sapMInputBaseIcon,
	.sapMInputBaseContentWrapperError .sapMInputBaseIconContainer .sapMInputBaseIcon:active,
	.sapMInputBaseContentWrapperError .sapMInputBaseIconContainer .sapMInputBaseIcon:hover {
		box-shadow: var(--sapField_Hover_InvalidShadow);
	}

	&.sapMInputBaseIconPressed .sapMInputBaseContentWrapperWarning .sapMInputBaseIconContainer .sapMInputBaseIcon,
	.sapMInputBaseContentWrapperWarning .sapMInputBaseIconContainer .sapMInputBaseIcon:active,
	.sapMInputBaseContentWrapperWarning .sapMInputBaseIconContainer .sapMInputBaseIcon:hover {
		box-shadow: var(--sapField_Hover_WarningShadow);
	}

	&.sapMInputBaseIconPressed .sapMInputBaseContentWrapperSuccess .sapMInputBaseIconContainer .sapMInputBaseIcon,
	.sapMInputBaseContentWrapperSuccess .sapMInputBaseIconContainer .sapMInputBaseIcon:active,
	.sapMInputBaseContentWrapperSuccess .sapMInputBaseIconContainer .sapMInputBaseIcon:hover {
		box-shadow: var(--sapField_Hover_SuccessShadow);
	}

	&.sapMInputBaseIconPressed .sapMInputBaseContentWrapperInformation .sapMInputBaseIconContainer .sapMInputBaseIcon,
	.sapMInputBaseContentWrapperInformation .sapMInputBaseIconContainer .sapMInputBaseIcon:active,
	.sapMInputBaseContentWrapperInformation .sapMInputBaseIconContainer .sapMInputBaseIcon:hover {
		box-shadow: var(--sapField_Hover_InformationShadow);
	}
}

/* Compact size */

.sapUiSizeCompact {

	.sapMInputBase {
		line-height: 1rem;
		height: var(--sapElement_Compact_Height);
	}

	.sapMInputBaseContentWrapper {
		height: var(--sapElement_Compact_Height);
	}

	.sapMInputBaseInner {
		padding: @_sap_m_InputBase_Compact_InnerPadding;
		min-width: @_sap_m_InputBase_InnerMinWidth_Compact;
	}

	.sapMInputBaseHasEndIcons .sapMInputBaseInner,
	.sapMInputBaseHasBeginIcons .sapMInputBaseInner {
		width: calc(~'100% - @{_sap_m_InputBase_Compact_IconWidth}');
	}

	.sapMInputBaseHasEndIcons.sapMInputBaseHasBeginIcons .sapMInputBaseInner {
		width: calc(~'100% - @{_sap_m_InputBase_Compact_IconWidth}*2');
	}

	.sapMInputBaseIcon {
		width: @_sap_m_InputBase_Compact_IconWidth;
		min-width: 2rem;
	}

	.sapMInputBaseIconContainer .sapUiIcon.sapMInputBaseIcon {
		line-height: 1.5rem;
	}

	.sapMInputBaseContentWrapper.sapMInputBaseContentWrapperState:not(.sapMInputBaseContentWrapperSuccess) .sapMInputBaseIcon {
		line-height: 1.375rem;
	}

	.sapMInputBaseHasBeginIcons .sapMInputBaseInner {
		padding-left: 0.25rem;
	}

	.sapMInputBaseHasEndIcons .sapMInputBaseInner {
		padding-right: 0.25rem;
	}

	/* handle min-width calculation */
	.sapMInputBaseHasEndIcons,
	.sapMInputBaseHasBeginIcons {

		.sapMInputBaseContentWrapper {
			min-width: @_InputBase_Control_OneIcon_MinWidth_Compact;

			&.sapMInputBaseContentWrapperState:not(.sapMInputBaseContentWrapperSuccess) {
				min-width: @_InputBase_Control_OneIcon_State_MinWidth_Compact;
			}
		}
	}

	.sapMInputBaseHasEndIcons.sapMInputBaseHasBeginIcons {

		.sapMInputBaseContentWrapper {
			min-width: @_InputBase_Control_TwoIcons_MinWidth_Compact;

			&.sapMInputBaseContentWrapperState:not(.sapMInputBaseContentWrapperSuccess) {
				min-width: @_InputBase_Control_TwoIcons_State_MinWidth_Compact;
			}
		}
	}

	.sapMInputBase:not(.sapMInputBaseHasEndIcons):not(.sapMInputBaseHasBeginIcons) {

		.sapMInputBaseContentWrapper {
			min-width: @_InputBase_Control_MinWidth_Compact;

			&.sapMInputBaseContentWrapperState:not(.sapMInputBaseContentWrapperSuccess) {
				min-width: @_InputBase_Control_State_MinWidth_Compact;
			}
		}
	}

	.sapMInputBaseHeightMargin {
		margin: 0.1875rem 0;
	}
}

/* Condensed size */

.sapUiSizeCondensed .sapUiTableCell {

	.sapMInputBase {
		height: 1.375rem;
		/* + 2*1px padding = 1.5rem */
		padding: 0 0.0625rem;
		/* 1px padding only */
	}

	.sapMInputBaseHeightMargin {
		margin: 0;
	}

	.sapMInputBaseInner:not(.sapMInputBaseContentWrapperState) {
		border-color: transparent;
	}

	.sapMInputBaseContentWrapper:not(.sapMInputBaseContentWrapperSuccess) {
		height: 1.375rem;
	}
}

/* Input selection fix -
 * Due to the fact that the CSS parsing rules require dropping the whole rule when encountering an invalid
 * pseudo-element, two separate rules must be written */

.sapMInputBaseInner {

	&::selection,
	&::-moz-selection {
		background: var(--sapSelectedColor);
		color: var(--sapContent_ContrastTextColor);
	}
}

/* Google chrome and Safari input selection fix:
 * The browser renders the selection background with 50% opacity by default.
 * The solution is to use fade with a value for alpha 99 if it's set 100 the browser ignores it.
 */

html[data-sap-ui-browser^='sf'],
html[data-sap-ui-browser^='cr'] {

	.sapMInputBaseInner::selection {
		background: fade(@sapUiSelected, 99);
		color: fade(@sapUiContentContrastTextColor, 99);
	}
}

/* When disabled or read-only hide placeholder by setting opacity to 0 */
.sapMInputBaseDisabled {

	.sapMInputBaseInner::-webkit-input-placeholder {
		opacity: 0;
	}

	.sapMInputBaseInner::-moz-placeholder {
		opacity: 0;
	}
}

.sapMInputBaseReadonly {

	.sapMInputBaseInner::-webkit-input-placeholder {
		opacity: 0;
	}

	.sapMInputBaseInner::-moz-placeholder {
		opacity: 0;
	}
}

.sapMInputBaseDisabledWrapper .sapMInputBaseIcon {
	color: var(--sapContent_DisabledTextColor);
}

/* pressed */

.sapMInputBaseIconPressed .sapMInputBaseIcon,
.sapMInputBaseIconPressed .sapMInputBaseIcon:hover {
	background: @sapUiToggleButtonPressedBackground;
}

.sapMInputBaseIconPressed .sapMInputBaseIcon::before {
	color: @sapUiToggleButtonPressedTextColor;
}

.sapMInputBaseIconPressed .sapMInputBaseContentWrapper:not(.sapMInputBaseReadonlyWrapper):not(.sapMInputBaseContentWrapperState):not(.sapMInputBaseDisabledWrapper) {
	border-color: var(--sapField_Active_BorderColor);
}

.sapUiSizeCondensed .sapUiTableDataCell {

	.sapMInputBaseContentWrapper {
		height: 1.375rem;
	}

	.sapMInputBaseContentWrapper:not(.sapMInputBaseContentWrapperState) {
		border: none;
	}

	.sapMInputBase .sapMInputBaseIcon {
		height: 1.375rem;
		width: 1.375rem;
		line-height: 1.375rem;
	}
}

.sapMInputBase .sapUiLocalBusyIndicator:focus {
	outline: none;
}
/* END "InputBase.less" */

/* START "LinkTileContent.less" */
.sapLTC {
    margin-left: 3rem;
    height: 1rem;
    .sapMLnk {
        margin-left: 1rem;
        max-width: 16.6rem;
    }
}

//When the linktilecontent is present inside the GenericTile
.sapMGT.sapMGTLinkTileContent.sapMTileSmallPhone {
    .sapLTC {
        .sapMLnk {
            max-width: 13.1rem;
        }
    }
}
/* END "LinkTileContent.less" */


/* note: the following three files are not in alphabetical order and should be imported after InputBase.less */
/* START "ComboBoxTextField.less" */
/* ======================================== */
/* CSS for control sap.m/ComboBoxTextField  */
/* Base theme                               */
/* ======================================== */
/* END "ComboBoxTextField.less" */

/* START "ComboBoxBase.less" */
/* =================================== */
/* CSS for control sap.m/ComboBoxBase  */
/* Base theme                          */
/* =================================== */

.sapMComboBoxBase.sapMInputBaseHasEndIcons .sapMInputBaseInner {
	flex: 1;
}

.sapMComboBoxBase .sapMInputBaseContentWrapper {
	display: inline-flex;
}

/* ------------------------------ */
/* arrow                          */
/* ------------------------------ */

.sapMComboBoxBase.sapMInputBaseReadonly .sapMInputBaseIcon {
	opacity: 0;
	cursor: default;
}

.sapMInputBaseIconPressed .sapMComboBoxBaseClearIcon {
	color: var(--sapContent_IconColor);
	background: transparent;

	&:hover {
		background: var(--sapButton_Lite_Hover_Background);
	}

	&:active {
		background-color: var(--sapButton_Lite_Active_Background);
		color: var(--sapButton_Active_TextColor);

		&::before {
			color: var(--sapButton_Active_TextColor);
		}
	}
}

.sapMInputBaseIconPressed .sapMComboBoxBaseClearIcon::before {
	color: var(--sapContent_IconColor);
}

/* ------------------------------ */
/* dialog                         */
/* ------------------------------ */

div.sapMComboBoxBasePicker .sapMDialogScrollCont {
	padding: 0;
}

/* ------------------------------ */
/* list                           */
/* ------------------------------ */

.sapMComboBoxBaseList.sapUiLocalBusy {
	min-height: 2rem;
}

.sapMComboBoxBaseList .sapMSLITitleOnly {
	font-size: 0.875rem;
}

.sapMComboBoxBaseList .sapMSLI {
	height: 2.5rem;
}
/* END "ComboBoxBase.less" */

/* START "ComboBox.less" */
/* =============================== */
/* CSS for control sap.m/ComboBox  */
/* Base theme                      */
/* =============================== */

.sapMComboBoxBasePicker-CTX .sapMComboBoxList.sapMListFocus .sapMLIBSelected,
.sapMComboBoxBasePicker-CTX .sapMComboBoxList.sapMListFocus .sapMComboBoxBaseNonInteractiveItem.sapMLIBFocused {
	outline: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
	outline-offset: -0.125rem;
}
/* END "ComboBox.less" */


/* START "InputListItem.less" */
/* ==================================== */
/* CSS for control sap.m/InputListItem  */
/* Base theme                           */
/* ==================================== */

.sapMILI {
	min-height: 3rem;
}

.sapMILILabel {
	font-size: 1.125rem;
	color: var(--sapList_TextColor);
}

.sapMLIBActionable .sapMILILabel {
	cursor: pointer;
}

.sapMILIContentWrapper {
	display: flex;
	gap: 0 1rem;
	align-items: center;
}

.sapMILIContentWrapperL {
	flex-wrap: wrap;

	& > .sapMILILabel {
		flex: 1 1 auto;
	}
}

.sapMILILabel {
	padding: 0.25rem 0;
}

.sapMILIDiv {
	flex: 1 1 auto;
	text-align: right;
}

html[dir='rtl'] .sapMILIDiv {
	text-align: left;
}

.sapMLIBActive .sapMILILabel {
	color: var(--sapList_Active_TextColor);
}

/* Compact size */
.sapUiSizeCompact .sapMILI {
	min-height: 2rem;
}

.sapUiSizeCompact .sapMILILabel {
	font-size: @sapMFontMediumSize;
}
/* END "InputListItem.less" */

/* START "Label.less" */
/* ============================ */
/* CSS for control sap.m/Label  */
/* Base theme                   */
/* ============================ */

@_sap_m_Label_InsideInactiveInfoToolbarColor: var(--sapList_TextColor);

.sapMLabel {
	color: var(--sapContent_LabelColor);
	font-size: @sapMFontMediumSize;
	font-family: var(--sapFontFamily);
	font-weight: normal;
	display: inline-block;
	white-space: nowrap;
	cursor: text;
	overflow: hidden;
	box-sizing: border-box;
}

.sapMLabelInner {
	display: inline-flex;
	width: 100%;
}

.sapMLabel.sapMLabelWrapped .sapMLabelInner {
	display: contents;
}

.sapMLabel .sapMLabelTextWrapper {
	text-overflow: ellipsis;
	overflow: hidden;
}

.sapMLabelNoText.sapMLabelRequired {
	line-height: @sapMFontMediumSize;
}

.sapMLabelBold,
.sapMLabel.sapMLabelBold {
	font-weight: bold;
}

.sapMLabelMaxWidth {
	max-width: 100%;
}

.sapMLIBActionable .sapMLabel {
	cursor: pointer;
}

.sapMLabel.sapMLabelRequired .sapMLabelColonAndRequired::after {
	content: '*';
	color: var(--sapField_RequiredColor);
	font-family: var(--sapFontFamily);
	font-size: var(--sapFontLargeSize);
	font-weight: bold;
	vertical-align: middle;
	line-height: 0;
	margin-left: 0.125rem;
}

html[dir='rtl'] .sapMLabel[dir='ltr'].sapMLabelRequired .sapMLabelColonAndRequired::after,
html[dir='ltr'] .sapMLabel[dir='rtl'].sapMLabelRequired .sapMLabelColonAndRequired::after {
	margin-right: 0.125rem;
	margin-left: 0;
}

.sapMIBar-CTX .sapMLabel {
	color: var(--sapContent_LabelColor);
}

.sapMIBar-CTX.sapMFooter-CTX .sapMLabel {
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	color: var(--sapPageFooter_TextColor);
}

.sapMTB-Info-CTX.sapMTB.sapMTBInactive .sapMLabel {
	color: @_sap_m_Label_InsideInactiveInfoToolbarColor;
}

/* Toolbar overflow context */
.sapMOverflowToolbarMenu-CTX {

	.sapMLabel {
		max-width: 20rem;
		margin-top: 0.375rem;
		margin-bottom: 0.375rem;
		white-space: normal;

		&.sapMLabelMediumMarginTop {
			margin-top: 0.625rem;
			margin-bottom: 0.125rem;
		}

		& + .sapMLabel {
			margin-top: 0.75rem;
		}
	}

	.sapMCb .sapMLabel {
		margin-top: 0;

		&.sapMLabelMediumMarginTop,
		& + .sapMLabel {
			margin-top: 0;
		}
	}
}

/* ============================ */
/* Overwrites for Label in Form */
/* ============================ */

/* GridLayout */
td.sapUiFormElementLbl:not(.sapUiGridLabelFull) > .sapMLabel {
	text-align: end !important; /* to overwrite setting on label */
	width: 100%; /* to react on touch/click on whole label cell */
}

/* ResponsiveGridLayout */
/* make labels right aligned if left from fields */
.sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanS12) > .sapMLabel,
.sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanM12) > .sapMLabel,
.sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanL12) > .sapMLabel,
.sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanXL12) > .sapMLabel {
	float: right;
}

/* ResponsiveLayout */
/* If label is above the fields it's left aligned, if it's left of the fields it's right aligned */
.sapUiFormResLayout .sapUiRFLRow:first-child > .sapUiRFLContainer:first-child > .sapUiRFLContainerContent > .sapMLabel {
	text-align: end !important; /* to overwrite setting on label */
	width: 100%; /* to react on touch/click on whole label cell */
}

/* ================================== */
/* Label in Form - Colon and Asterisk */
/* ================================== */
.sapUiFormLblColon {

	/* Label side by side with a field */
	/* In this case we need to align all : and * in the form */
	/* GridLayout */
	td:not(.sapUiGridLabelFull),
	/* ResponsiveGridLayout */
	.sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > :not(.sapUiRespGridSpanS12),
	.sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > :not(.sapUiRespGridSpanM12),
	.sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > :not(.sapUiRespGridSpanL12),
	.sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > :not(.sapUiRespGridSpanXL12),
	/* ResponsiveLayout */
	.sapUiFormResLayout .sapUiRFLRow:first-child > .sapUiRFLContainer:first-child:not(.sapUiRFLFullLength) > .sapUiRFLContainerContent,
	/* ColumnLayout*/
	.sapUiFormCL.sapUiFormCLSmallColumns :not(.sapUiFormCLCellsS12),
	.sapUiFormCL.sapUiFormCLWideColumns :not(.sapUiFormCLCellsL12) {

		&.sapUiFormElementLbl > .sapMLabel {

			.sapMLabelInner {
				display: inline-flex;
				align-items: flex-end;
				justify-content: flex-end !important;
			}

			.sapMLabelColonAndRequired {
				display: inline-flex;
				align-items: center;
				min-width: 0.85em; /* Accommodate space for colon + asterisk. The use of em is intentional since it has to relate to the current font size applied to the label */
			}

			/* Accommodate space for colon + asterisk for specific languages */
			&:lang(fr) .sapMLabelColonAndRequired {
				min-width: 1.125em;
			}

			&:lang(zh-TW) .sapMLabelColonAndRequired {
				min-width: 1.59em;
			}

			&:lang(zh-CN) .sapMLabelColonAndRequired{
				min-width: 1.59em;
			}

			&:lang(zh-Hans) .sapMLabelColonAndRequired {
				min-width: 1.59em;
			}

			&:lang(zh-Hant) .sapMLabelColonAndRequired {
				min-width: 1.59em;
			}
		}
	}
}

.sapUiFormResLayout .sapUiRFLRow:first-child > .sapUiRFLContainer.sapUiRFLFullLength > .sapUiRFLContainerContent > .sapMLabel {
	text-align: start !important; /* to overwrite setting on label */
	padding-bottom: 2px;
}

/* ResponsiveGridLayout */
.sapUiFormResGridCont > .sapUiFormElementLbl > .sapMLabel {
	width: 100%;
}

/* collapse lines in a form */
.sapUiForm .sapUiFormElementLbl > .sapMLabel {
	height: auto;
}

.sapUiForm.sapUiFormEdit .sapUiFormElementLbl > .sapMLabel {
	vertical-align: top; /* align vertically to multiline texts */
}

.sapMLabel.sapMLabelShowColon,
.sapUiFormLblColon .sapUiFormElementLbl > .sapMLabel {

	.sapMLabelColonAndRequired::before {
		content: attr(data-colon);
		white-space: pre;
	}
}

/* Wrapping of the Label's text */
.sapMLabel.sapMLabelWrapped {
	white-space: pre-line;
	overflow: hidden;
	word-wrap: break-word;
}

.sapUiFormLblColon .sapUiFormElementLbl > .sapMLabel.sapMLabelNoText .sapMLabelColonAndRequired::before {
	content: none;
}

.sapMLIBUnread .sapMLabel,
.sapMLIBUnread + .sapMListTblSubRow .sapMLabel {
	font-family: var(--sapFontFamily);
	font-weight: bold;
}

.sapMListTbl .sapMLIBActive .sapMLabel,
.sapMLIBActive .sapMLabel {
	color: var(--sapList_Active_TextColor);
}

/* Transparent Toolbar Context*/
.sapMTB-Transparent-CTX .sapMLabel {
	font-weight: @sapUiFontHeaderWeight;
	font-family: var(--sapFontHeaderFamily);
	color: var(--sapGroup_TitleTextColor);
}

/* HorizontalLayout */
.sapUiHLayoutChildWrapper > .sapMLabel {
	vertical-align: text-bottom;
}

/* ================================ */
/* SMALL,  MEDIUM and LARGE DEVICES */
/* ================================ */
.sapUiRespGridMedia-Std-Phone.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanS12) > .sapMLabel,
.sapUiRespGridMedia-Std-Tablet.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanM12) > .sapMLabel,
.sapUiRespGridMedia-Std-Desktop.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanL12) > .sapMLabel,
.sapUiRespGridMedia-Std-LargeDesktop.sapUiFormResGridCont > .sapUiFormElementLbl:not(.sapUiRespGridSpanXL12) > .sapMLabel {
	text-align: end !important; /* to overwrite style set on element */
	float: none; /* as label has 100% width not needed (will bring wrong alignments on iPad) */
}

/* ========================================================== */
/* Form - ColumnLayout                                        */
/* ========================================================== */
.sapUiFormCL .sapUiFormElementLbl > .sapMLabel {
	width: 100%;
}

.sapUiFormCL.sapUiFormCLSmallColumns .sapUiFormElementLbl:not(.sapUiFormCLCellsS12) > .sapMLabel,
.sapUiFormCL.sapUiFormCLWideColumns .sapUiFormElementLbl:not(.sapUiFormCLCellsL12) > .sapMLabel {
	text-align: end !important; /* to overwrite style set on element */
	float: none; /* otherwise ":" is inside of Elements padding */
}

/* ========================================================== */
/* COMPACT + CONDENSED SIZE + SPECIALS FOR SAP.UI.TABLE.TABLE */
/* ========================================================== */

.sapUiSizeCompact,
.sapUiSizeCondensed.sapUiTable,
.sapUiSizeCondensed .sapUiTable {

	.sapMIBar-CTX.sapMPageHeader .sapMLabel,
	.sapMDialog-CTX .sapMIBar-CTX .sapMLabel {
		font-size: var(--sapFontHeader6Size);
	}
}

.sapUiSizeCompact,
.sapUiSizeCondensed {

	.sapUiTableCell:not(.sapUiTableHeaderDataCell) .sapMLabel {
		font-weight: normal;
		font-size: var(--sapFontHeader6Size);
	}
}

.sapUiSizeCondensed {

	.sapUiTableDataCell {

		:not(.sapMRbBLabel):not(.sapMCbLabel) .sapMLabel {
			line-height: 1.5rem;
		}
	}
}

.sapUiTableCell .sapMLabel {
	color: var(--sapList_TextColor);
	vertical-align: middle;
}

.sapUiAnalyticalTable {

	.sapUiTableCell .sapMLabel {
		width: 100%;
	}
}

.sapUiTableHeaderDataCell .sapMLabel,
.sapMListTbl .sapMColumnHeader .sapMLabel {
	font-weight: @sapUiColumnHeaderFontWeight;
	font-size: @sapUiColumnHeaderFontSize;
	color: var(--sapList_HeaderTextColor);
}
/* END "Label.less" */

/* START "LightBox.less" */
/* =============================== */
/* CSS for control sap.m/LightBox  */
/* Base theme                      */
/* =============================== */

@_sap_m_LightBox_MinHeight: 18rem;
@_sap_m_LightBox_FooterOneLineHeightCompact: 2.5rem;
@_sap_m_LightBox_FooterTwoLinesHeightCompact: 3.5rem;
@_sap_m_LightBox_FooterOneLineHeightCozy: 3rem;
@_sap_m_LightBox_FooterTwoLinesHeightCozy: 3.5rem;
@_sap_m_LightBox_TitleSectionPadding: 1rem;
@_sap_m_LightBox_TitleSectionPaddingTwoLines: 0.5rem;
@_sap_m_LightBox_BusyIndicatorHeight: 1rem;
@_sap_m_LightBox_SubtitleMarginTop: 0.5rem;
@_sap_m_LightBox_OverlayBackground: var(--sapBlockLayer_Background);
@_sap_m_LightBox_ContainerBackground: var(--sapBackgroundColor);
@_sap_m_LightBox_OverlayOpacity: 0.9;
@_sap_m_LightBox_FontFamily: var(--sapFontFamily);

/* ========================================================================== */
/* Container */
/* ========================================================================== */

.sapMLightBox {
	min-width: 20rem;
	max-width: 100%;
	min-height: 18rem;
	max-height: 100%;
	border-radius: var(--sapElement_BorderCornerRadius);
	background-color: @_sap_m_LightBox_ContainerBackground;
	box-shadow: @sapUiShadowLevel3;
	outline: none;
}

html[data-sap-ui-browser^='sf'] .sapMLightBox {
	transform: translateZ(0px); /* fixes displaying the control behind the block layer in Safari */
}

.sap-desktop .sapMLightBox {
	max-width: calc(~"100% - 4rem");
	max-height: calc(~"100% - 4rem");
}

.sap-tablet .sapMLightBox {
	max-width: calc(~"100% - 2rem");
	max-height: calc(~"100% - 2rem");
}

.sap-phone .sapMLightBox {
	max-width: calc(~"100% - 1rem");
	max-height: calc(~"100% - 1rem");
}

.sapMLightBoxFooter {
	background-color: var(--sapPageFooter_Background);
	border-top: 1px solid var(--sapPageFooter_BorderColor);
}

/* ========================================================================== */
/* Overlay */
/* ========================================================================== */

.sapMLightBox + .sapUiBLy {
	background: @_sap_m_LightBox_OverlayBackground;
	opacity: @_sap_m_LightBox_OverlayOpacity;
}

/* ========================================================================== */
/* Image */
/* ========================================================================== */

.sapMLightBox .sapMImg {
	margin: auto;
	position: absolute;
	top: 0;
	right: 0;
	bottom: 0;
	left: 0;
}

.sapMLightBox.sapMLightBoxTopCornersRadius .sapMImg {
	border-radius: var(--sapElement_BorderCornerRadius) var(--sapElement_BorderCornerRadius) 0 0;
}

/* ========================================================================== */
/* Footer */
/* ========================================================================== */

.sapMLightBoxFooter {
	display: flex;
	flex-direction: row;
	width: 100%;
	position: absolute;
	bottom: 0;
	justify-content: space-between;
	height: @_sap_m_LightBox_FooterOneLineHeightCozy;
	align-items: flex-end;
	border-radius: 0 0 var(--sapElement_BorderCornerRadius) var(--sapElement_BorderCornerRadius);

	&.sapMLightBoxFooterTwoLines {
		height: @_sap_m_LightBox_FooterTwoLinesHeightCozy;
	}
}

.sapMLabel.sapMLightBoxSubtitle  {
	margin-top: @_sap_m_LightBox_SubtitleMarginTop;
}

.sapMLightBoxTitleSection {
	margin: 0 1rem;
	padding: @_sap_m_LightBox_TitleSectionPadding 0;
	display: flex;
	flex-direction: column;
	overflow: hidden;
	flex: 1;
}

.sapMLightBoxTwoLines .sapMLightBoxTitleSection {
	padding: @_sap_m_LightBox_TitleSectionPaddingTwoLines 0;
}

.sapMLightBoxFooter .sapMBtn {
	padding-right: 0.25rem;
}

.sapMTitle.sapMLightBoxTitle,
.sapMLabel.sapMLightBoxSubtitle {
	white-space: nowrap;
	overflow: hidden;
	text-align: left;
	text-overflow: ellipsis;
}

/* ========================================================================== */
/* Footer title */
/* ========================================================================== */

.sapMLightBoxImageContainer {
	min-height: calc(@_sap_m_LightBox_MinHeight - 3.5rem);
	position: absolute;
	top: 0;
	bottom: @_sap_m_LightBox_FooterOneLineHeightCozy;
	width: 100%;
}

.sapMLightBoxImageContainerTwoLines {
	height: calc(~"100% - " @_sap_m_LightBox_FooterTwoLinesHeightCozy);
	position: absolute;
	top: 0;
	width: 100%;
}

/* ========================================================================== */
/* Busy Indicator */
/* ========================================================================== */

.sapMLightBox {

	.sapUiSizeCompact {

		.sapMBusyIndicator {
			display: block;
			margin-top: (@_sap_m_LightBox_MinHeight - @_sap_m_LightBox_BusyIndicatorHeight - @_sap_m_LightBox_FooterOneLineHeightCompact) / 2;
		}

		.sapMLightBoxTwoLines {

			.sapMBusyIndicator {
				height: 14rem;
				margin-top: (@_sap_m_LightBox_MinHeight - @_sap_m_LightBox_BusyIndicatorHeight - @_sap_m_LightBox_FooterTwoLinesHeightCompact) / 2;
			}
		}
	}
}

.sapMLightBox {

	.sapMBusyIndicator {
		display: block;
		margin-top: (@_sap_m_LightBox_MinHeight - @_sap_m_LightBox_BusyIndicatorHeight - @_sap_m_LightBox_FooterOneLineHeightCozy) / 2;
	}

	.sapMLightBoxTwoLines {

		.sapMBusyIndicator {
			height: 14rem;
			margin-top: (@_sap_m_LightBox_MinHeight - @_sap_m_LightBox_BusyIndicatorHeight - @_sap_m_LightBox_FooterTwoLinesHeightCozy) / 2;
		}
	}
}

/* ========================================================================== */
/* Error State */
/* ========================================================================== */

.sapMLightBox.sapMLightBoxError .sapMLightBoxFooter {
	position: initial;
}

/* ========================================================================== */
/* Compact */
/* ========================================================================== */

.sapUiSizeCompact {

	.sapMLightBoxImageContainer {
		bottom: @_sap_m_LightBox_FooterOneLineHeightCompact;
	}

	.sapMLightBoxImageContainerTwoLines {
		bottom: @_sap_m_LightBox_FooterTwoLinesHeightCompact;
		height: calc(~"100% - " @_sap_m_LightBox_FooterTwoLinesHeightCompact);
	}

	.sapMLightBoxFooter {
		height: @_sap_m_LightBox_FooterOneLineHeightCompact;

		&.sapMLightBoxFooterTwoLines {
			height: @_sap_m_LightBox_FooterTwoLinesHeightCompact;
		}
	}

	.sapMLightBoxTitleSection {
		padding: 0.75rem 0;
	}

	.sapMLightBoxTwoLines .sapMLightBoxTitleSection {
		padding: @_sap_m_LightBox_TitleSectionPaddingTwoLines 0;
	}

	.sapMLightBoxFooter .sapMBtn {
		padding-right: 0.5rem;
		margin-bottom: 0.25rem;
	}
}
/* END "LightBox.less" */

/* START "Link.less" */
/* =========================== */
/* CSS for control sap.m/Link  */
/* Base theme                  */
/* =========================== */

.sapMLnk {
	color: var(--sapLinkColor);
	text-decoration: var(--sapLink_TextDecoration);
	display: inline-block;
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	cursor: pointer;

	&:visited {
		color: var(--sapLink_Visited_Color);
	}

	&:not(.sapMLnkDsbl) {

		&:active {
			color: var(--sapLink_Active_Color);
			text-decoration: var(--sapLink_Active_TextDecoration);
		}
	}

	&:not(.sapMLnkWrapping) {
		display: inline-flex;
		align-items: baseline;

		.sapMLnkText {
			white-space: nowrap;
			overflow: hidden;
			text-overflow: ellipsis;
		}

		.sapMLnkEndIcon {
			float: inline-end;
		}
	}
}

.sapMLnkIcon {
	float: inline-start;
	margin-inline-end: 0.125rem;
	cursor: pointer;
}

.sapMLnkEndIcon {
	margin-inline-start: 0.125rem;
	cursor: pointer;
}

html.sap-desktop .sapMLnk:not(.sapMLnkDsbl) {

	&:focus {
		outline: 1px dotted var(--sapContent_FocusColor);
		outline-offset: -1px;
		text-decoration: underline;
	}

	&:hover:not(:active):not(:focus) {
		color: var(--sapLink_Hover_Color);
		text-decoration: var(--sapLink_Hover_TextDecoration);
	}
}

.sapUiInverted-CTX .sapUiLnk {
	color: var(--sapLink_InvertedColor);
}

.sapMLnk.sapMLnkWrapping {
	white-space: normal;
	word-wrap: break-word;
}

.sapMLnk.sapMLinkContainsEmptyIdicator,
.sapMLnk.sapMLinkContainsEmptyIdicator > * {
	pointer-events: none;
	text-decoration: none;
}

.sapMLnkMaxWidth {
	max-width: 100%;
}

/* Link in sap.ui.table.* tables */

.sapUiTableCell .sapMLnk {
	vertical-align: middle;
}

.sapUiSizeCondensed .sapUiTableCell .sapMLnk {
	line-height: 1.5rem;
}

.sapUiHLayoutChildWrapper > .sapMLnk {
	vertical-align: text-bottom;
}

.sapMFT .sapMLnk,
.sapMFT .sapMTitle .sapMLnk {
	vertical-align: top;
}

/* Link focus offset and visualization in sap.ui.table.* tables */
html.sap-desktop .sapUiSizeCondensed .sapUiTableCell .sapMLnk:focus:not(.sapMLnkDsbl) {
	outline-offset: -2px;
}

.sapUiForm .sapMLnk {
	min-height: 1rem; /* do give empty links a size too. Keep the Form stable */
	vertical-align: top;
}

.sapUiFormEdit .sapMLnk {
	line-height: normal;
	margin-top: 1rem; /* don't use padding as this would stretch the focus outline */
	margin-bottom: 0.75rem; /* for some reason browser adds 4px, display: block would lead to stretched focus outline */
}

.sapUiFormToolbar .sapMTB .sapMLnk {
	line-height: normal;
	margin-top: auto;
	margin-bottom: auto;
}

.sapMLnkEmphasized {
	font-family: var(--sapFontBoldFamily);
}

.sapMLnk .sapMEmptyIndicator.sapMEmptyIndicatorAuto {
	display: none;
}

.sapMShowEmpty-CTX .sapMLnk .sapMEmptyIndicator.sapMEmptyIndicatorAuto {
	display: inline-block;
}

.sapMShowEmpty-CTX .sapMLnk .sapMEmptyIndicator.sapMEmptyIndicatorAuto,
.sapMLnk .sapMEmptyIndicator {
	color: var(--sapTextColor);
}

.sapMListTblRow.sapMLIBActive .sapMLnk,
.sapMListTblSubRow.sapMLIBActive .sapMLnk,
.sapMLIB.sapMLIBActive .sapMLink {
	text-decoration: none;
	text-shadow: none;
 }

/* Compact size */
.sapUiSizeCompact .sapUiFormEdit .sapMLnk {
	margin-top: 0.5rem;
	margin-bottom: 0.25rem;
}

.sapMLnk:-webkit-any-link:focus {
	outline: none;
}
/* END "Link.less" */

/* START "ListBase.less" */
/* =============================== */
/* CSS for control sap.m/ListBase  */
/* Base theme                      */
/* =============================== */

.sapMList {
	position: relative; /* This should not break the list but help swipe list content position calculation */
	box-sizing: border-box;
	margin: 0;
	padding: 0;
}

.sapMListUl {
	list-style-type: none;
	padding: 0;
	margin: 0;
}

.sapMListUl:focus {
	outline: none;
}

.sapMListPreventFocus .sapMListDummyArea {
	display: none;
}

.sapMListSwp {
	overflow: hidden;
	position: absolute;
	right: 0.5em;
	z-index: 1;
	opacity: 0;
	pointer-events: auto;
	display: flex;
	align-items: center;
	margin-top: -1px;
}

.sapMListDummyArea {
	position: fixed; /* keep it in the visible viewport, so that IE does not scroll on focus */
}

.sapMListDummyAreaSticky {
	position: sticky; /* keep it in the scrollable area, so that browser does not scroll on focus */
	height: 0.1px; /* webkit needs height for the sticky elements */
	bottom: 0;
}

/* swipe animations */
@keyframes sapMListSwpInAnim {

	0% {
		opacity: 0;
	}

	100% {
		opacity: 1;
	}
}

@keyframes sapMListSwpOutAnim {

	0% {
		opacity: 1;
	}

	100% {
		opacity: 0;
	}
}

.sapMListSwpInAnim {
	animation: sapMListSwpInAnim 0.2s ease-in forwards;
}

.sapMListSwpOutAnim {
	animation: sapMListSwpOutAnim 0.2s ease-in forwards;
}

.sapMListNoData {
	list-style-type: none;
	display: flex;
	align-items: center;
	justify-content: center;
	color: var(--sapList_TextColor);
	background-color: var(--sapList_Background);
	border-bottom: 1px solid var(--sapList_BorderColor);
	padding: 0.5rem 1rem !important;
	min-height: 3rem;
	line-height: 1.4;
}

.sapMListNoDataText {
	overflow: hidden;
	overflow: clip;
	white-space: pre-wrap;
	text-align: center;
}

.sapMListNoDataContent {
	flex: 1 1 auto;
}

.sapMListShowSeparatorsAll {
	/* class will be added for others, just a reminder */
}

.sapMListShowSeparatorsInner {
	/* class will be added for others, just a reminder */
}

.sapMListShowSeparatorsNone {
	/* class will be added for others, just a reminder */
}

/* Growing feature */

.sapMGrowingList {
	border-bottom: 1px solid @sapUiButtonActionSelectBorderColor;
}

.sapMGrowingListTrigger {
	display: block;
	text-align: center;
	overflow: hidden;
	border-top: 1px solid var(--sapList_HeaderBorderColor);
}

.sapMGrowingListDummyColumn {
	border-right: 1px solid var(--sapList_TableFooterBorder);
}

/* .sapMSLIDescription also has padding defined, hence making the style rule stronger */
.sapMGrowingListDescription.sapMSLIDescription {
	display: none;
	padding: 0.375rem 2rem 0.875rem 2rem;
	text-align: center;
}

.sapMGrowingList > .sapMCLI.sapMLIB:not(.sapMLIBActive) {
	background: @sapUiButtonActionSelectBackground;
	border-bottom: none;
}

.sapMGrowingList > .sapMCLI.sapMLIBHoverable:hover {
	background: @sapUiButtonLiteActionSelectHoverBackground;
}

.sapMGrowingListBusyIndicatorVisible .sapMGrowingListTriggerText,
.sapMGrowingListBusyIndicatorVisible .sapMGrowingListDescription {
	visibility: hidden;
}

/* mixin call for creating container content padding classes (arguments: rootContainer, contentSelector) */
.sapUiContainerContentPadding(~".sapMList");

.sapUiFioriObjectPage > section > .sapMList:not(.sapMListInsetBG) {
	margin: 1rem 0;
	padding: 0 1rem;
	box-sizing: border-box;
}

.sapUiFioriObjectPage > section > .sapMITB + .sapMList:not(.sapMListInsetBG) {
	margin-top: 1rem;
}

html.sap-phone .sapUiFioriObjectPage > section > .sapMList:not(.sapMListInsetBG) {
	padding: 0;
}

.sapUIMeTabContainerContent .sapMList {
	padding: 1rem;
	box-sizing: border-box;
}

html.sap-phone .sapUIMeTabContainerContent .sapMList {
	padding: 0;
	box-sizing: border-box;
}

.sapMListInsetBG {
	padding: 2rem;
}

.sapMListInsetBG > .sapMListSwp {
	margin-right: 2rem;
}

html.sap-phone .sapMListInsetBG {
	padding: 1rem;
}

html.sap-phone .sapMListInsetBG > .sapMListSwp {
	margin-right: 1rem;
}

.sapMListHdrText {
	overflow: hidden;
	white-space: nowrap;
	text-overflow: ellipsis;
	box-sizing: border-box;
	font-size: var(--sapFontHeader4Size);
	font-family: var(--sapFontHeaderFamily);
	color: var(--sapGroup_TitleTextColor);
	height: 3rem;
	line-height: 3rem;
	padding: 0 1rem;
	background-color: var(--sapGroup_TitleBackground);
	border-bottom: 1px solid var(--sapGroup_TitleBorderColor);
}

.sapMListFtr {
	height: 2rem;
	box-sizing: border-box;
	-webkit-text-size-adjust: none; /* To improve readability Mobile Safari automatically increases the size of small text so let's disable this */
	font-size: @sapMFontMediumSize;
	line-height: 2rem;
	background-color: var(--sapList_FooterBackground);
	color: var(--sapList_FooterTextColor);
	padding: 0 1rem;
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
}

.sapMListTopBorder {
	border-top: 1px solid var(--sapList_BorderColor);
}

.sapMListShowSeparatorsNone .sapMListNoData {
	border-bottom: 0;
}

.sapUIMeTabContainerContent .sapMFeedListItem {
	display: block;
	padding: .5rem 0;
}

/* Growing feature */
.sapMGrowingListBusyIndicator {
	margin: 1.5rem 1.0rem;
	position: absolute;
}

.sapMGrowingListTriggerText {
	padding: 0 1rem;
}

.sapMGrowingListTriggerText > .sapMSLITitle {
	margin: 0.813rem 0; /* overwrites any user-agent default margin values */
	padding-bottom: 0;
	color: var(--sapButton_TextColor);
	font-weight: bold;
}

.sapMGrowingListDescription.sapMSLIDescription {
	color: var(--sapButton_TextColor);
}

/* sticky headers */
.sapMSticky > .sapMListHdr,
.sapMSticky > .sapMListInfoTBarContainer,
.sapMSticky .sapMGHLI {
	position: sticky;
}

/* sticky headerToolbar/headerText background when transparent */
.sapMSticky > .sapMListHdrText,
.sapMSticky > .sapMListHdrTBar.sapMTB-Transparent-CTX {
	background: var(--sapList_HeaderBackground);
}

.sapMSticky > .sapMListHdr,
.sapMSticky > .sapMTableOverlay,
.sapMSticky > .sapUiLocalBusyIndicator {
	z-index: 3;
}

/* .sapMSticky3 denotes that the headerToolbar/headerText and the infoToolbar are sticky */
/* .sapMSticky2 denotes that only the infoToolbar is sticky */
/* .sapMSticky1 denotes that only the headerToolbar/headerText is sticky */
/* top: 0 sticky conditions : 1st sticky elements */
.sapMSticky1 > .sapMListHdr,
.sapMSticky2 > .sapMListInfoTBarContainer,
.sapMSticky3 > .sapMListHdr {
	z-index: 2;
	top: 0;
}

/* top: 3rem sticky conditions : 2nd sticky elements*/
.sapMSticky3 > .sapMListInfoTBarContainer {
	z-index: 2;
	top: var(--sapElement_LineHeight);
}

/* handle transparent background on rounded corners to not show underlaying elements on scrolling */
._sap_m_ListBase_Rounded_HeaderToolbarMixin() {

	.sapMSticky:has(> .sapMListHdr):not(:has(> .sapMMsgStrip))::before {
		position: sticky;
		content: '';
		display: block;
		background: var(--sapGroup_ContentBackground);
		height: var(--sapElement_BorderCornerRadius);
		margin-bottom: ~"calc(var(--sapElement_BorderCornerRadius) * -1)"; /* prevent new empty space above sticky element */
		top: 0;
		z-index: 2;

		/* handle transparent background on ObjectPageSection */
		.sapUiTableOnObjectPageAdjustmentsForSection.sapUxAPObjectPageSection .sapUxAPObjectPageSectionContainer .sapUxAPObjectPageSubSection .sapUxAPBlockContainer & {
			background: transparent;
		}
	}
}

/* Compact size */
.sapUiSizeCompact .sapMListNoData {
	min-height: 2rem;
	font-size: @sapMFontMediumSize;
}

.sapUiSizeCompact .sapMGrowingListDescription {
	padding-top: 0;
}
/* END "ListBase.less" */

/* START "ListItemBase.less" */
/* =================================== */
/* CSS for control sap.m/ListItemBase  */
/* Base theme                          */
/* =================================== */

@_sap_m_ListItemBase_DeleteIcon: sys-cancel;
@_sap_m_ListItemBase_ButtonMarginTop: auto;
@_sap_m_ListItemBase_Highlight_InformationColor: var(--sapInformationBorderColor);
@_sap_m_ListItemBase_Navigated: var(--sapList_SelectionBorderColor);
@_sap_m_ListItemBase_Detail_Delete_MarginLeft: 0.125rem;
@_sap_m_ListItemBase_Focus_Outline: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
@_sap_m_ListItemBase_Focus_OffsetWithGap: calc(-@sapUiContentFocusWidth ~"- 0.0625rem");

.sapMLIB,
.sapMLIBImgNav {
	box-sizing: border-box;
}

.sapMLIB {
	display: flex;
	align-items: center;
	position: relative;
	background: var(--sapList_Background);
	border-bottom: 1px solid var(--sapList_BorderColor);
	padding: 0 1rem 0 1rem;
}

.sapMLIBIconDet,
.sapMLIBIconDel {
	margin-left: @_sap_m_ListItemBase_Detail_Delete_MarginLeft;
}

.sapMLIBContent {
	flex: 1 1 auto;
	max-width: 100%;
	overflow: hidden;
}

.sapMLIBFocusable:focus,
.sapMTblCellFocusable:focus {
	outline: @_sap_m_ListItemBase_Focus_Outline;
	outline-offset: @_sap_m_ListItemBase_Focus_OffsetWithGap;
}

/* makes the focus outline visible when hightlight is used */
.sapMLIBFocusable:focus .sapMLIBHighlight,
.sapMLIBFocusable:focus + .sapMListTblSubRow .sapMLIBHighlight {
	width: 0.25rem;
	left: 2px;
	bottom: 2px;
	top: 2px;
}

.sapMListTblRow:not(.sapMLIBFocusable):focus,
.sapMLIB:not(.sapMLIBFocusable):focus,
.sapMListTblSubRow:focus {
	outline: none;
}

.sapMLIBActionable,
.sapMLIBActionable > .sapMLIBImgNav,
.sapMListTblNavCol > .sapMLIBImgNav,
.sapMLIBActionable .sapMSLIThumbnail,
.sapMLIBActionable .sapMSLIImgIcon {
	cursor: pointer;
}

.sapMLIBImgNav,
.sapMLIBImgDet,
.sapMLIBImgDel,
.sapMLIBIconDet,
.sapMLIBSelectS,
.sapMLIBSelectM,
.sapMLIBSelectD,
.sapMLIBCounter {
	flex: none;
}

.sapMLIBHighlight {
	position: absolute;
	width: 0.375rem;
	bottom: 0;
	left: 0;
	top: 0;
	border-right: 0.0625rem solid var(--sapList_Background);
	box-sizing: border-box;
}

.sapMLIBHighlight + .sapMLIBContent,
.sapMListHighlight .sapMLIBContent:first-child {
	margin-left: 0.375rem;
}

.sapMLIBSelectS,
.sapMLIBSelectM {
	margin-right: 0.75rem;
}

.sapMLIBHighlight + .sapMLIBSelectS,
.sapMLIBHighlight + .sapMLIBSelectM,
.sapMListHighlight > .sapMLIB > .sapMLIBSelectS:first-child,
.sapMListHighlight > .sapMLIB > .sapMLIBSelectM:first-child {
	margin-left: -0.625rem;
}

.sapMLIB > .sapMLIBSelectS:first-child,
.sapMLIB > .sapMLIBSelectM:first-child {
	margin-left: -0.8125rem;
}

.sapMLIBHighlightInformation {
	background: @_sap_m_ListItemBase_Highlight_InformationColor;
}

.sapMLIBHighlightSuccess {
	background: var(--sapSuccessBorderColor);
}

.sapMLIBHighlightWarning {
	background: var(--sapWarningBorderColor);
}

.sapMLIBHighlightError {
	background: var(--sapErrorBorderColor);
}

.sapMLIBHighlightIndication01 {
	background: var(--sapIndicationColor_1);
}

.sapMLIBHighlightIndication02 {
	background: var(--sapIndicationColor_2);
}

.sapMLIBHighlightIndication03 {
	background: var(--sapIndicationColor_3);
}

.sapMLIBHighlightIndication04 {
	background: var(--sapIndicationColor_4);
}

.sapMLIBHighlightIndication05 {
	background: var(--sapIndicationColor_5);
}

.sapMLIBHighlightIndication06 {
	background: var(--sapIndicationColor_6);
}

.sapMLIBHighlightIndication07 {
	background: var(--sapIndicationColor_7);
}

.sapMLIBHighlightIndication08 {
	background: var(--sapIndicationColor_8);
}

.sapMLIBHighlightIndication09 {
	background: var(--sapIndicationColor_9);
}

.sapMLIBHighlightIndication10 {
	background: var(--sapIndicationColor_10);
}

:not(.sapMListModeDelete) > .sapMLIB.sapMLIBTypeNavigation {
	padding-right: 0;
}

.sapMListShowSeparatorsNone .sapMLIBShowSeparator {
	border-bottom: 1px solid transparent;
}

.sapMListShowSeparatorsInner .sapMLIBShowSeparator:last-child {
	border-bottom-color: transparent;
}

.sapMLIB.sapMListTblSupRow {
	border-bottom: 0;
}

.sapMLIBSelectAnimation {
	animation-duration: 0.25s;
	animation-timing-function: ease-in;
	animation-name: resize;
}

@keyframes resize {

	0% {
		width: 0;
		opacity: 0;
	}

	100% {
		width: 2.5rem;
		opacity: 1;
	}
}

.sapUiSizeCompact .sapMLIBSelectAnimation {
	animation-duration: 0.25s;
	animation-timing-function: ease-in;
	animation-name: resizeCompact;
}

@keyframes resizeCompact {

	0% {
		width: 0;
		opacity: 0;
	}

	100% {
		width: 2rem;
		opacity: 1;
	}
}

.sapMLIBUnselectAnimation {
	animation-duration: 0.25s;
	animation-timing-function: ease-in;
	animation-name: close;
}

@keyframes close {

	0% {
		width: 3rem;
	}

	100% {
		width: 0;
	}
}

.sapMLIBUnread,
.sapMLIBUnread > .sapMListTblCell,
.sapMLIBUnread + .sapMListTblSubRow {
	font-family: var(--sapFontFamily);
	font-weight: bold;
}

.sapMLIBCounter {
	font-size: 0.875rem;
	color: var(--sapContent_MarkerTextColor);
	padding-left: 1rem;
}

.sapMLIBImgNav {
	width: var(--sapElement_LineHeight);
	font-size: 0.75rem;
	color: var(--sapContent_LabelColor);
}

.sapMLIBActive .sapMLIBImgNav,
.sapMLIBActive .sapMLIBIconDet .sapMBtnIcon,
.sapMLIBActive .sapMLIBIconDel .sapMBtnIcon {
	color: var(--sapList_Active_TextColor);
}

.sapMListBGTransparent .sapMLIB {
	background: fade(@sapUiListBackground, 0);
}

.sapMLIBHoverable:hover {
	background: var(--sapList_Hover_Background);
}

.sapMLIB.sapMLIBSelected {
	background: var(--sapList_SelectionBackgroundColor);
}

.sapMLIB.sapMLIBActive {
	color: var(--sapList_Active_TextColor);
	background: var(--sapList_Active_Background);
}

.sapMLIBHoverable.sapMLIBSelected:hover {
	background : @sapUiListSelectionHoverBackground;
}

.sapMLIBHoverable.sapMLIBSelected.sapMLIBActive:hover {
	background: var(--sapList_Active_Background);
}

.sapMLIBActive div,
.sapMLIBActive .sapMLIBCounter {
	color: var(--sapList_Active_TextColor);
}

.sapMLIBNavigated {
	width: 0.1875rem;
	position: absolute;
	right: 0;
	top: 0;
	bottom: 0;
	background-color: @_sap_m_ListItemBase_Navigated;
}

/* to make the focus outline visible when navigated=true */
.sapMLIBFocusable:focus .sapMLIBNavigated,
.sapMLIBFocusable:focus + .sapMListTblSubRow .sapMLIBNavigated {
	width: 0.125rem;
	right: 2px;
	bottom: 2px;
	top: 2px;
}

/* Compact size */
.sapUiSizeCompact .sapMLIB > .sapMLIBSelectS:first-child,
.sapUiSizeCompact .sapMLIB > .sapMLIBSelectM:first-child {
	margin-left: -0.5rem;
}

/* Correct alignment of the selection control according to the specs. */
/* 1rem padding-left (.sapMLIB) - 0.5rem margin-left (.sapMLIBSelectM) + 3rem width (.sapMLIBSelectM) - 0.5rem margin-left (.sapMLIBSelectM + .sapMLIBContent) = 3rem */
.sapUiSizeCompact .sapMLIB > .sapMLIBSelectM + .sapMLIBContent,
.sapUiSizeCompact .sapMLIB > .sapMLIBSelectS + .sapMLIBContent {
	margin-left: -0.5rem;
}
/* END "ListItemBase.less" */


/* Note: The following file is not in alphabetical order and should be imported after ListItemBase.less */
/* START "CustomListItem.less" */
/* ===================================== */
/* CSS for control sap.m/CustomListItem  */
/* Base theme                            */
/* ===================================== */

.sapMCLI {
	padding: 0;
}

.sapMListModeSingleSelectLeft > .sapMCLI,
.sapMListModeMultiSelect > .sapMCLI {
	padding-left: 1rem;
}

.sapMListModeDelete > .sapMCLI {
	padding-right: 1rem;
}
/* END "CustomListItem.less" */


/* START "Menu.less" */
/* =========================== */
/* CSS for control sap.m/Menu  */
/* Base theme                  */
/* =========================== */

.sapMRespMenuDialog .sapMLIB {
	border-bottom: none;
}

/* compact */
.sapUiSizeCompact .sapMMenu.sapUiMnu {
	min-width: 50px;
}

.sapUiSizeCompact .sapMMenu .sapUiMnuItmIco > .sapUiIcon {
	font-size: 1rem;
}

/* end compact */
/* END "Menu.less" */

/* START "MenuButton.less" */
/* ================================= */
/* CSS for control sap.m/MenuButton  */
/* Base theme                        */
/* ================================= */

.sapMMenuBtn {
	display: inline-block;
	width: auto;
	max-width: 100%;
}

.sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner {
	text-align: left;
	padding-right: 2rem;
	position: relative;
	padding-left: 0.625rem;
}

.sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner:not(.sapMBtnText) {
	padding-right: 1.4375rem;
}

.sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner::after {
	position: absolute;
	top: 0;
	right: 0;
	pointer-events: none;
	font-family: var(--sapContent_IconFontFamily);
	content: '\e1ef';
	display: inline-block;
	font-size: 0.75rem;
	height: 2.125rem;
	line-height: 2.125rem;
	padding-right: 0.75rem;
	padding-left: 0.5625rem;
	color: var(--sapButton_IconColor);
}

.sapMMenuBtn.sapMMenuBtnRegular:hover .sapMBtnInner::after {
	color: var(--sapButton_Hover_TextColor);
}

.sapMMenuBtn.sapMMenuBtnSplit .sapMSBActive:not(.sapMBtnDisabled) > .sapMBtnInner {
	background-image: none;
	text-shadow: none;
	background-color: var(--sapButton_Active_Background);
	border-color: var(--sapButton_Active_BorderColor);
}

.sapUiSizeCompact {

	.sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner {
		padding-right: 1.625rem;
		padding-left: 0.5rem;
	}

	.sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner:not(.sapMBtnText) {
		padding-right: 1.25rem;
	}

	.sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner::after {
		height: 1.5rem;
		line-height: 1.5rem;
		padding-right: 0.5rem;
		padding-left: 0.375rem;
	}
}

/* type Emphasized */
.sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner.sapMBtnEmphasized::after {
	color: var(--sapButton_Emphasized_TextColor);
	text-shadow: var(--sapButton_Emphasized_TextShadow);
}

.sapMMenuBtn.sapMMenuBtnRegular:hover .sapMBtnInner.sapMBtnEmphasized::after {
	color: var(--sapButton_Emphasized_Hover_TextColor);
}

.sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner.sapMBtnEmphasized.sapMBtnActive::after {
	text-shadow: none;
}

/* type Accept */
.sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner.sapMBtnAccept::after {
	color: var(--sapButton_Accept_TextColor);
}

.sapMMenuBtn.sapMMenuBtnRegular:hover .sapMBtnInner.sapMBtnAccept::after {
	color: var(--sapButton_Accept_Hover_TextColor);
}

/* type Attention */
.sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner.sapMBtnAttention::after {
	color: var(--sapButton_Attention_TextColor);
}

.sapMMenuBtn.sapMMenuBtnRegular:hover .sapMBtnInner.sapMBtnAttention::after {
	color: var(--sapButton_Attention_Hover_TextColor);
}

/* type Reject */
.sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner.sapMBtnReject::after {
	color: var(--sapButton_Reject_TextColor);
}

.sapMMenuBtn.sapMMenuBtnRegular:hover .sapMBtnInner.sapMBtnReject::after {
	color: var(--sapButton_Reject_Hover_TextColor);
}

/* type Default, Transparent(header) */
.sapMMenuBtn.sapMMenuBtnRegular .sapMBtn:not(.sapMBtnDisabled) .sapMBtnInner.sapMBtnActive::after {
	color: var(--sapButton_Active_TextColor);
}

/* Transparent(footer) */
.sapMIBar-CTX.sapMFooter-CTX .sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner::after {
	text-shadow: var(--sapButton_Emphasized_TextShadow);
}

.sapMFooter-CTX .sapMMenuBtn .sapMBtn:hover > .sapMBtnHoverable:not(.sapMBtnActive) {
	text-shadow: none;
}

.sapMIBar-CTX.sapMFooter-CTX .sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner.sapMBtnActive::after {
	text-shadow: none;
}

.sapMMenuBtn.sapMMenuBtnSplit .sapMSBActive:not(.sapMBtnDisabled) > .sapMBtnInner.sapMBtnReject {
	background-color: var(--sapButton_Reject_Active_Background);
	border-color: var(--sapButton_Reject_Active_BorderColor);
}

.sapMMenuBtn.sapMMenuBtnSplit .sapMSBActive:not(.sapMBtnDisabled) > .sapMBtnInner.sapMBtnAccept {
	background-color: var(--sapButton_Accept_Active_Background);
	border-color: var(--sapButton_Accept_Active_BorderColor);
}

.sapMMenuBtn.sapMMenuBtnSplit .sapMSBActive:not(.sapMBtnDisabled) > .sapMBtnInner.sapMBtnAttention {
	background-color: var(--sapButton_Attention_Active_Background);
	border-color: var(--sapButton_Attention_Active_BorderColor);
}

.sapMMenuBtn.sapMMenuBtnSplit .sapMSBActive:not(.sapMBtnDisabled) > .sapMBtnInner.sapMBtnEmphasized {
	background-color: var(--sapButton_Emphasized_Active_Background);
	border-color: var(--sapButton_Emphasized_Active_BorderColor);
}

.sapMMenuBtn.sapMMenuBtnSplit .sapMSBActive:not(.sapMBtnDisabled) > .sapMBtnInner .sapMBtnIcon {
	color: var(--sapButton_Active_TextColor);
}

.sapMMenuBtn .sapMBtn .sapMBtnInner {
	display: flex;
}

.sapMMenuBtn .sapMBtn:not(.sapMSBArrow) .sapMBtnInner.sapMBtnIconFirst .sapMBtnIcon {
	padding-right: 0.375rem;
}

.sapMMenuBtn .sapMBtn .sapMBtnInner.sapMBtnText:not(.sapMBtnIconFirst) .sapMBtnContent {
	width: 100%;
}

.sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner.sapMBtnCritical::after {
	color: var(--sapButton_Critical_TextColor);
}

.sapMMenuBtn.sapMMenuBtnRegular:hover .sapMBtnInner.sapMBtnCritical::after {
	color: var(--sapButton_Critical_Hover_TextColor);
}

.sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner.sapMBtnNegative::after {
	color: var(--sapButton_Negative_TextColor);
}

.sapMMenuBtn.sapMMenuBtnRegular:hover .sapMBtnInner.sapMBtnNegative::after {
	color: var(--sapButton_Negative_Hover_TextColor);
}

.sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner.sapMBtnSuccess::after {
	color: var(--sapButton_Success_TextColor);
}

.sapMMenuBtn.sapMMenuBtnRegular:hover .sapMBtnInner.sapMBtnSuccess::after {
	color: var(--sapButton_Success_Hover_TextColor);
}

.sapMMenuBtn.sapMMenuBtnRegular .sapMBtnInner.sapMBtnNeutral::after {
	color: var(--sapButton_Neutral_TextColor);
}

.sapMMenuBtn.sapMMenuBtnRegular:hover .sapMBtnInner.sapMBtnNeutral::after {
	color: var(--sapButton_Neutral_Hover_TextColor);
}
/* END "MenuButton.less" */

/* START "MenuListItem.less" */
/* =================================== */
/* CSS for control sap.m/MenuListItem  */
/* Base theme                          */
/* =================================== */

/* phone */
.sapMRespMenuDialog .sapMLIB {
	position: relative;
	padding: 0 0.875rem;
}

.sapMRespMenuDialog .sapMSLITitleOnly {
	font-style: normal;
	font-weight: normal;
	font-size: @sapMFontMediumSize;
}

.sapMMenuLIHasChildren .sapMSLITitleDiv {
	text-overflow: ellipsis;
}

.sapMMenuLIImgThumbWrapper {
	font-size: 1.375rem;
	min-width: 1rem;
	max-width: 1rem;
	padding-inline-end: 0.875rem;
	text-align: center;
}

.sapMMenuLIImgThumbIcon {
	font-size: 1.375rem;
}

.sapMMenuLIArrowRightIcon {
	font-size: .75rem;
	width: 1.875rem;
	height: 3rem;
	line-height: 3rem;
}

.sapMRespMenuDialog .sapUiMnuDiv {
	height: 1px;
	width: 100%;
	position: absolute;
	left: 0;
	top: 0;
	display: block;
}

.sapMRespMenuDialog .sapUiMnuDiv > hr {
	height: 0;
	margin: 0;
	left: 1rem;
	right: 1rem;
}

.sapMMLIDisabled .sapMSLITitle,
.sapMMLIDisabled .sapMSLITitleOnly {
	color: var(--sapContent_DisabledTextColor);
}

html.sap-phone .sapMMLIDisabled .sapMMenuLIImgThumbIcon,
html.sap-tablet .sapMMLIDisabled .sapMMenuLIImgThumbIcon {
	color: var(--sapContent_DisabledTextColor);
}

.sapMMenuLISel {
	padding-inline-start: 0.5rem;
	padding-inline-end: 0;
	font-weight: normal;
	text-align: center;

	&::after {
		color: var(--sapSelectedColor);
		content: '\e05b';
		font-family: var(--sapContent_IconFontFamily);
		outline: none;
		line-height: inherit;
		padding-inline-end: 0.125rem;
		width: 2rem;
		height: 2rem;
	}
}

/* desktop and tablet */
.sapMMenu .sapUiMnuDiv > hr {
	height: 0;
	margin: 0;
}

/* really strong selector in the sapUiMnu demands this */
.sapMMenu.sapUiMnu.sapUiMnuCozySupport .sapUiMnuItmTxt {
	text-overflow: ellipsis;
	max-width: 100%;
	max-width: -moz-available; /* WebKit-based browsers will ignore this. */
	max-width: -webkit-fill-available; /* Mozilla-based browsers will ignore this. */
	max-width: fill-available;
}

/* compact */
.sapUiSizeCompact .sapMMenuLIArrowRightIcon {
	width: 2rem;
	height: 2rem;
	line-height: 2rem;
}
/* END "MenuListItem.less" */

/* START "MessageBox.less" */
/* ================================= */
/* CSS for control sap.m/MessageBox  */
/* Base theme                        */
/* ================================= */

/* header shadow - different states */
@_sap_m_MessageBox_HeaderShadow: inset 0 -0.125rem;
@_sap_m_MessageBox_InformationShadow: @_sap_m_MessageBox_HeaderShadow var(--sapInformationBorderColor);
@_sap_m_MessageBox_WarningShadow: @_sap_m_MessageBox_HeaderShadow var(--sapWarningBorderColor);
@_sap_m_MessageBox_ErrorShadow: @_sap_m_MessageBox_HeaderShadow var(--sapErrorBorderColor);
@_sap_m_MessageBox_SuccessShadow: @_sap_m_MessageBox_HeaderShadow var(--sapSuccessBorderColor);
@_sap_m_MessageBox_QuestionShadow: @_sap_m_MessageBox_HeaderShadow var(--sapNeutralBorderColor);

.sapMMessageBoxLinkText {
	vertical-align: bottom;
}

.sapMMessageBoxLinkText,
.sapMMessageBoxErrorText,
.sapMMessageBoxDetails {
	margin-top: 1rem;
}

.sapMDialog.sapMMessageBoxInfo .sapMDialogIcon.sapUiIcon {
	color: var(--sapNeutralElementColor);
}

.sapMDialog.sapMMessageBoxWarning .sapMDialogIcon.sapUiIcon {
	color: var(--sapCriticalElementColor);
}

.sapMDialog.sapMMessageBoxError .sapMDialogIcon.sapUiIcon {
	color: var(--sapNegativeElementColor);
}

.sapMDialog.sapMMessageBoxSuccess .sapMDialogIcon.sapUiIcon {
	color: var(--sapPositiveElementColor);
}

.sapMDialog.sapMMessageBoxQuestion .sapMDialogIcon.sapUiIcon {
	color: var(--sapNeutralElementColor);
}

.sapMDialog.sapMMessageDialog {

	&.sapMMessageBoxInfo .sapMDialogTitleGroup > .sapMBar {
		box-shadow: @_sap_m_MessageBox_InformationShadow;
	}

	&.sapMMessageBoxWarning .sapMDialogTitleGroup > .sapMBar {
		box-shadow: @_sap_m_MessageBox_WarningShadow;
	}

	&.sapMMessageBoxError .sapMDialogTitleGroup > .sapMBar {
		box-shadow: @_sap_m_MessageBox_ErrorShadow;
	}

	&.sapMMessageBoxSuccess .sapMDialogTitleGroup > .sapMBar {
		box-shadow: @_sap_m_MessageBox_SuccessShadow;
	}

	&.sapMMessageBoxQuestion .sapMDialogTitleGroup > .sapMBar {
		box-shadow: @_sap_m_MessageBox_QuestionShadow;
	}
}
/* END "MessageBox.less" */

/* START "MessageStrip.less" */
/* =================================== */
/* CSS for control sap.m/MessageStrip  */
/* Base theme                          */
/* =================================== */

.sapMMsgStrip {
	border: var(--sapMessage_BorderWidth) solid;
	border-radius: var(--sapPopover_BorderCornerRadius);
	box-sizing: border-box;
	font-size: @sapMFontMediumSize;
	font-weight: normal;
	padding: 0.4375rem 1rem;
	position: relative;
	min-height: 2rem;

	&[data-sap-ui-ms-closable='true'] > .sapMMsgStripMessage {
		margin-right: 1.25rem;
	}
}

html[data-sap-ui-animation='on'] {

	.sapMMsgStrip {
		transition: opacity 0.2s;
	}

	.sapMMsgStripCloseButton {
		transition: color 0.1s;
	}
}

.sapMMsgStripClosing {
	opacity: 0;
}

.sapMMsgStripIcon {
	position: absolute;
	left: 0;
	width: 2.5rem;
	font-size: 1rem;
	text-align: center;

	& ~ .sapMMsgStripMessage {
		margin-left: 1.5rem;
	}
}

.sapMMsgStripMessage {
	max-height: 10rem;
	overflow: auto;

	& > .sapMText,
	& > .sapMFT,
	& > .sapMLnk {
		display: inline;
	}

	& > .sapMLnk {
		margin-left: 0.25rem;

		&:not(.sapMLnkDsbl) {
			text-shadow: var(--sapContent_TextShadow);
		}
	}

	& > .sapMText {
		line-height: 1.2;
	}
}

.sapMMsgStripCloseButton {
	position: absolute;
	top: -0.5625rem;
	right: 0.125rem;
}

.sapUiSizeCompact .sapMMsgStripCloseButton {
	top: -0.0625rem;
}

.sapMMsgStripCloseButton {
	position: absolute;
	top: -0.5625rem;
	right: 0.125rem;
}

.sapMMsgStripInformation {
	background-color: var(--sapNeutralBackground);
	border-color: var(--sapNeutralBorderColor);
	color: var(--sapTextColor);

	& .sapMMsgStripIcon {
		color: var(--sapNeutralElementColor);
	}
}

.sapMMsgStripSuccess {
	background-color: var(--sapSuccessBackground);
	border-color: var(--sapSuccessBorderColor);
	color: var(--sapTextColor);

	& .sapMMsgStripIcon {
		color: var(--sapPositiveElementColor);
	}
}

.sapMMsgStripWarning {
	background-color: var(--sapWarningBackground);
	border-color: var(--sapWarningBorderColor);
	color: var(--sapTextColor);

	& .sapMMsgStripIcon {
		color: var(--sapCriticalElementColor);
	}
}

.sapMMsgStripError {
	background-color: var(--sapErrorBackground);
	border-color: var(--sapErrorBorderColor);
	color: var(--sapTextColor);

	& .sapMMsgStripIcon {
		color: var(--sapNegativeElementColor);
	}
}
/* END "MessageStrip.less" */

/* START "MessagePopover.less" */
/* ===================================== */
/* CSS for control sap.m/MessagePopover  */
/* Base theme                            */
/* ===================================== */

.sapMMsgPopover.sapMMsgPopover-init {

	.sapMNavItem.sapMPage {
		position: relative;
	}

	.sapMIBar {
		border-bottom: 0;
	}
}

.sapMMsgPopoverCloseBtn {

	.sapMBtnIcon {
		width: 2.5rem;
		height: 2.5rem;
		font-size: 1rem;
	}
}

/* Compact size */
.sapUiSizeCompact.sapMMsgPopover {

	.sapMMsgPopoverCloseBtn .sapMBtnIcon {
		width: 1.626rem;
		height: 1.626rem;
		font-size: 0.75rem;
	}

	.sapMMsgPopoverCloseBtn .sapMBtnIcon {
		margin-right: 0;
	}

	.sapMBtnCustomIcon {
		margin-left: 0;
	}
}

.sapMPopover.sapMMsgPopover.sapMMsgPopover-init {

	.sapMPopoverArr::after {
		border-top-color: var(--sapPageHeader_Background);
	}
}

.sapMMsgPopoverCloseBtn .sapMBtnIcon {
	color: var(--sapButton_IconColor);
}

/* style links in the description as Link control */
.sapMMsgPopoverDescriptionText a {
	text-decoration: none;
	color: var(--sapLinkColor);

	&:hover {
		text-decoration: underline;
		color: var(--sapLink_Hover_Color);
	}

	&:visited {
		text-decoration: none;
		color: var(--sapLink_Visited_Color);
	}

	&:active {
		color: var(--sapLink_Active_Color);
	}
}
/* END "MessagePopover.less" */

/* START "MessageView.less" */
/* ===================================== */
/* CSS for control sap.m/MessagePopover  */
/* Base theme                            */
/* ===================================== */

@_sap_m_MessageView_DescIcon_Width: 3rem;

.sapMMsgView {
	height: 100%;

	.sapMSegBBtnMixed .sapMSegBBtnInner {
		padding-left: 0;
		padding-right: 0;
	}

	.sapMDialogScrollCont {
		padding: 0;
	}

	.sapMIBar .sapMBarChild {
		margin-left: .25rem;
		margin-right: .25rem;
	}

	.sapMMsgViewTitleText,
	.sapMMsgViewSubtitleText,
	.sapMMsgViewDescriptionText {
		margin: 1rem 1rem 0 3rem;

		a.sapMLnkMaxWidth,
		&.sapMLnkMaxWidth {
			white-space: normal;
		}
	}

	.sapMMsgViewDescriptionText {
		padding-bottom: 1rem;
		font-size: @sapMFontMediumSize;
		display: block;
	}

	.sapMMsgViewDescriptionLink {
		display: block;
		margin: 1rem 0 0 3rem;
	}

	.sapMMsgViewDescriptionText:empty {
		display: none;
	}

	.sapMMsgViewTitleText {
		vertical-align: middle;
		font-size: var(--sapFontHeader5Size);
	}

	.sapMMsgViewSubtitleText {
		display: block;
		margin: 0 0 1rem 2.5rem;
		font-family: var(--sapFontFamily);
		color: var(--sapContent_LabelColor);
		font-size: var(--sapFontSize);
	}

	.sapMMsgViewItem {
		padding-left: 0;

		.sapMSLIImgIcon {
			margin-left: 0;
		}
	}

	.sapMMsgViewItem.sapMSLI.sapMSLIWrapping .sapMLIBContent .sapUiIcon {
		align-self: center;
		margin-top: 0;
	}

	.sapMSLIDescription {
		margin-top: 0.125rem;
		font-family: var(--sapFontFamily);
		font-size:  @sapMFontMediumSize;
		color: var(--sapContent_LabelColor);
	}

	.sapMSLITitle {
		font-family: var(--sapFontHeaderFamily);
		font-size: var(--sapFontHeader5Size);
		font-weight: @sapUiFontHeaderWeight;
		color: var(--sapGroup_TitleTextColor);
	}

	.sapMSLITitleOnly {
		font-size: var(--sapFontHeader5Size);
	}

	.sapMLIB.sapMLIBActive {

		.sapMSLIImgIcon {
			color: var(--sapList_Active_TextColor);
		}
	}

	.sapMSLIDescIcon .sapMSLIDescription {
		margin-top: 0.125rem;
	}

	.sapMSLIInfo {
		margin: 0;
	}

	.sapMTextMaxWidth {
		max-width: ~"calc(100% - @{_sap_m_MessageView_DescIcon_Width})";
	}

	.sapMSLITitle,
	.sapMSLITitleOnly,
	.sapMSLIWithDescription > .sapMLIBContent > .sapMSLIDiv > .sapMSLIDescription {
		padding: 0;
	}

	.sapMMsgViewHiddenContainer {
		display: none;
	}

	/* remove the truncation behavior, when a list item does not need it */
	/* (if it needs to have a truncation we change the item type to navigation) */
	.sapMMsgViewItem:not(.sapMLIBTypeNavigation) .sapMSLITitleOnly {
		text-overflow: clip;
	}
}

.sapMMsgViewDescIcon {
	position: absolute;
	left: 0;
	width: @_sap_m_MessageView_DescIcon_Width;
	top: 0.7rem;
	font-size: 1.35rem;
}

.sapMMsgViewItemActive {

	& .sapMLnk {
		font-size: var(--sapFontHeader5Size);
	}

	& .sapMSLITitleOnly {
		display: flex;
		align-items: center;
	}
}

/* Compact size */
.sapUiSizeCompact .sapMMsgView {

	&.sapMPopoverArrUp::after,
	&.sapMPopoverWithoutBar.sapMPopoverNav.sapMPopoverWithHeaderCont > .sapMPopoverArrUp::after,
	&.sapMPopoverWithoutBar.sapMPopoverPage.sapMPopoverWithHeaderCont > .sapMPopoverArrUp::after {
		bottom: -9px;
		left: -8px;
	}

	.sapMMsgViewItem.sapMSLIWrapping > .sapMLIBContent > .sapMSLIDiv {
		padding: 0;
	}

	.sapMMsgViewItemActive .sapMLnk {
		font-size: var(--sapFontHeader6Size);
	}

	.sapMMsgViewItem .sapMLIBImgNav.sapUiIcon {
		height: 3.4375rem;
		line-height: 3.4375rem;
	}

	.sapMMsgViewCloseBtn .sapMBtnIcon {
		width: 1.626rem;
		height: 1.626rem;
		font-size: 0.75rem;
	}

	.sapMList .sapMLIBContent .sapUiIcon {
		font-size: 1rem;
	}

	.sapMBtnInner {
		padding: 0;
	}

	.sapMMsgViewBackBtn .sapMBtnIcon {
		margin-right: 0;
	}

	.sapMBtnCustomIcon  {
		margin-left: 0;
	}

	.sapMMsgViewDescIcon {
		width: @_sap_m_MessageView_DescIcon_Width;
		height: 1.626rem;
		line-height: 1.626rem;
		font-size: 1rem;
	}

	.sapMSLIDescription {
		margin-top: 0;
	}

	.sapMSLITitle,
	.sapMSLITitleOnly {
		font-size: var(--sapFontHeader6Size);
	}

	.sapMSLIDescIcon .sapMSLITitle,
	.sapMSLIDescIcon .sapMSLIDescription {
		margin-top: 0.125rem;
		margin-bottom: 0.125rem;
	}
}

.sapMMsgViewCloseBtn .sapMBtnIcon {
	color: var(--sapButton_IconColor);
}

.sapMMsgViewItemSuccess .sapMSLIImgIcon,
.sapMMsgViewBtnSuccess.sapMSegBBtn .sapUiIcon,
.sapMMsgViewDescIconSuccess {
	color: var(--sapPositiveElementColor);
}

.sapMMsgViewItemWarning .sapMSLIImgIcon,
.sapMMsgViewBtnWarning.sapMSegBBtn .sapUiIcon,
.sapMMsgViewDescIconWarning {
	color: var(--sapCriticalElementColor);
}

.sapMMsgViewItemError .sapMSLIImgIcon,
.sapMMsgViewBtnError.sapMSegBBtn .sapUiIcon,
.sapMMsgViewDescIconError {
	color: var(--sapNegativeElementColor);
}

.sapMMsgViewItemInformation .sapMSLIImgIcon,
.sapMMsgViewBtnInformation.sapMSegBBtn .sapUiIcon,
.sapMMsgViewDescIconInformation {
	color: var(--sapInformativeElementColor);
}

.sapMMsgViewItem .sapMSLIIconActive {
	color: var(--sapContent_ContrastIconColor);
}

.sapMMsgViewDescriptionText a.sapMLnk {
	overflow: visible;
}

.sapMMsgViewDetailsPage .sapMMsgViewTitleText .sapMLnkText {
	white-space: normal;
}
/* END "MessageView.less" */

/* START "MessageToast.less" */
/* =================================== */
/* CSS for control sap.m/MessageToast  */
/* Base theme                          */
/* =================================== */

.sapMMessageToast {
	box-sizing: border-box;
	position: absolute;
	max-width: 15rem;
	word-wrap: break-word;
	text-align: center;
	text-decoration: none;
	text-overflow: ellipsis;
	overflow: hidden;
	white-space: pre-line;
	font-size: var(--sapFontSize);
	font-family: var(--sapFontFamily);
	font-weight: normal;
	padding: 1rem;
	background: var(--sapIndicationColor_9_Background);
	color: var(--sapContent_ContrastTextColor);
	box-shadow: var(--sapContent_Lite_Shadow);
	border-radius: var(--sapElement_BorderCornerRadius);
}

.sapMMessageToast.sapMFocus {
	outline-width: var(--sapContent_FocusWidth);
	outline-style: var(--sapContent_FocusStyle);
	outline-color: var(--sapContent_FocusColor);
	outline-offset: ~"calc(var(--sapContent_FocusWidth) * -1)";
}
/* END "MessageToast.less" */

/* START "MultiComboBox.less" */
/* ==================================== */
/* CSS for control sap.m/MultiComboBox  */
/* Base theme                           */
/* ==================================== */

@_sap_m_InputWithTokens_TokenizerMinWidth: 3.25rem;

/*MIN-WIDTH variables*/
/* rounding values to 2 symbols, because of IE and Edge */
@_InputBase_Control_Tokens_MinWidth: round((@_InputBase_Control_OneIcon_MinWidth + @_sap_m_InputWithTokens_TokenizerMinWidth), 2);
@_InputBase_Control_Tokens_ReadOnly_MinWidth: round((@_InputBase_Control_MinWidth + @_sap_m_InputWithTokens_TokenizerMinWidth), 2);
@_InputBase_Control_Tokens_State_MinWidth: round((@_InputBase_Control_Tokens_MinWidth + @_sap_m_InputBaseWrapper_Sum_Border), 2);

@_InputBase_Control_Tokens_MinWidth_Compact: round((@_InputBase_Control_OneIcon_MinWidth_Compact + @_sap_m_InputWithTokens_TokenizerMinWidth), 2);
@_InputBase_Control_Tokens_ReadOnly_MinWidth_Compact: round((@_InputBase_Control_MinWidth_Compact + @_sap_m_InputWithTokens_TokenizerMinWidth), 2);
@_InputBase_Control_Tokens_State_MinWidth_Compact: round((@_InputBase_Control_Tokens_MinWidth_Compact + @_sap_m_InputBaseWrapper_Sum_Border), 2);

/* hande min-width calculation */
.sapMMultiComboBox.sapMMultiComboBoxHasToken {

	.sapMTokenizer {
		min-width: @_sap_m_InputWithTokens_TokenizerMinWidth;
	}

	&.sapMFocus .sapMTokenizer {
		left: 0.125rem;
	}

	.sapMInputBaseContentWrapper {
		min-width: @_InputBase_Control_Tokens_MinWidth;

		&.sapMInputBaseContentWrapperState:not(.sapMInputBaseContentWrapperSuccess) {
			min-width: @_InputBase_Control_Tokens_State_MinWidth;
		}
	}

	&.sapMInputBaseReadonly .sapMInputBaseContentWrapper {
		min-width: @_InputBase_Control_Tokens_ReadOnly_MinWidth;
	}
}

.sapMMultiComboBox:not(.sapMMultiComboBoxHasToken) .sapMTokenizer {
	padding-left: 0;
}

.sapMMultiComboBox .sapMTokenizer {
	max-height: 100%;
}

.sapMMultiComboBoxHasToken .sapMInputBaseInner {
	padding-left: 0;
}

.sapMMultiComboBox.sapMInputBaseNoWidth {
	width: 100%;
}

li.sapMMultiComboBoxItem .sapMCbBg {
	top: 0;
	bottom: 0;
	margin: auto;
}

/* Keep placeholder invisible, when at least one token is chosen */
.sapMMultiComboBoxHasToken .sapMInputBaseInner::-moz-placeholder {
	color: transparent;
}

.sapMMultiComboBoxHasToken .sapMInputBaseInner::-webkit-input-placeholder {
	color: transparent;
}

.sapMMultiComboBoxHasToken .sapMInputBaseInner:-moz-placeholder {
	color: transparent;
}

/* ------------------------------ */
/* ListItem                       */
/* ------------------------------ */

.sapMMultiComboBoxItem.sapMSLIWrapping .sapMCb {
	height: 2.4375rem;
	line-height: 2.4375rem;
	margin-top: 0;
}

.sapMMultiComboBoxPicker {

	.sapMFocusable:focus {
		outline: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
		outline-offset: -0.125rem;
	}

	.sapMSLITitleOnly {
		font-size: 0.875rem;
	}
}

/* Compact size */
.sapUiSizeCompact {

	.sapMLIB.sapMMultiComboBoxItem.sapMSLIWrapping > .sapMCb {
		height: 2rem;
		line-height: 2rem;
		margin-top: 0;
	}

	/* handle min-width calculation */
	.sapMMultiComboBox.sapMMultiComboBoxHasToken {

		.sapMInputBaseContentWrapper {
			min-width: @_InputBase_Control_Tokens_MinWidth_Compact;

			&.sapMInputBaseContentWrapperState:not(.sapMInputBaseContentWrapperSuccess) {
				min-width: @_InputBase_Control_Tokens_State_MinWidth_Compact;
				outline-offset: -3px;

				.sapMToken {
					margin-top: 0.0625rem;
					margin-bottom: 0.0625rem;
				}

				.sapMTokenizerIndicator {
					line-height: 1rem;
				}
			}
		}

		&.sapMInputBaseReadonly .sapMInputBaseContentWrapper {
			min-width: @_InputBase_Control_Tokens_ReadOnly_MinWidth_Compact;
		}
	}

	.sapMMultiComboBoxSelectAll.sapMTB.sapMIBar,
	.sapMMultiComboBoxSelectAll.sapMTB-Transparent-CTX.sapMTB.sapMListHdrTBar {
		padding: 0;

		.sapMCb {
			margin-left: 0.5rem;
			padding-left: 2.25rem;
		}
	}
}

.sapMMultiComboBoxSelectAll.sapMTB.sapMIBar,
.sapMMultiComboBoxSelectAll.sapMTB-Transparent-CTX.sapMTB.sapMListHdrTBar {
	padding: 0.125rem;

	&.sapMMultiComboBoxSelectAllFocused {
		outline: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
		outline-offset: -0.125rem;
	}

	& .sapMBarChild.sapMCb.sapMCbHasLabel:focus::before {
		border: none;
	}
}
/* END "MultiComboBox.less" */

/* START "MultiInput.less" */
/* ================================= */
/* CSS for control sap.m/MultiInput  */
/* Base theme                        */
/* ================================= */

@_sap_m_InputWithTokens_TokenizerMinWidth: 3.25rem;

/*MIN-WIDTH variables*/
/* rounding values to 2 symbols, because of IE and Edge */
@_InputBase_Control_Tokens_MinWidth: round((@_InputBase_Control_OneIcon_MinWidth + @_sap_m_InputWithTokens_TokenizerMinWidth), 2);
@_InputBase_Control_Tokens_ReadOnly_MinWidth: round((@_InputBase_Control_MinWidth + @_sap_m_InputWithTokens_TokenizerMinWidth), 2);
@_InputBase_Control_Tokens_State_MinWidth: round((@_InputBase_Control_Tokens_MinWidth + @_sap_m_InputBaseWrapper_Sum_Border), 2);

@_InputBase_Control_Tokens_MinWidth_Compact: round((@_InputBase_Control_OneIcon_MinWidth_Compact + @_sap_m_InputWithTokens_TokenizerMinWidth), 2);
@_InputBase_Control_Tokens_ReadOnly_MinWidth_Compact: round((@_InputBase_Control_MinWidth_Compact + @_sap_m_InputWithTokens_TokenizerMinWidth), 2);
@_InputBase_Control_Tokens_State_MinWidth_Compact: round((@_InputBase_Control_Tokens_MinWidth_Compact + @_sap_m_InputBaseWrapper_Sum_Border), 2);



.sapMMultiInput:not(.sapMMultiInputHasTokens) {
	& .sapMTokenizer {
		padding-left: 0;
 	}
}

.sapMMultiInput.sapMMultiInputHasTokens .sapMTokenizer.sapMTokenizerNoNMore {
	min-width: 2rem;
}

/* handle min-width calculation */
.sapMMultiInput.sapMMultiInputHasTokens {
	.sapMTokenizer {
		min-width: @_sap_m_InputWithTokens_TokenizerMinWidth;
	}

	.sapMInputBaseContentWrapper {
		min-width: @_InputBase_Control_Tokens_MinWidth;
		&.sapMInputBaseContentWrapperState:not(.sapMInputBaseContentWrapperSuccess) {
			min-width: @_InputBase_Control_Tokens_State_MinWidth;
		}
	}

	&.sapMInputBaseReadonly .sapMInputBaseContentWrapper {
		min-width: @_InputBase_Control_Tokens_ReadOnly_MinWidth;
	}
}

.sapMMultiInput.sapMMultiInputHasTokens {
	& .sapMInputBaseInner {
		padding-left: 0;
	}
}

.sapMMultiInput .sapMTokenizer {
	max-height: 100%;
}

// prevent placing the multiinput over a busy indicator overlay
.sapUiLocalBusy .sapMMultiInput .sapMTokenizer {
	z-index: 0;
}

/* Keep placeholder invisible, when at least one token is chosen */
.sapMMultiInputHasTokens .sapMInputBaseInner::-moz-placeholder {
	color: transparent;
}

.sapMMultiInputHasTokens .sapMInputBaseInner::-webkit-input-placeholder {
	color: transparent;
}

.sapMMultiInputHasTokens .sapMInputBaseInner:-moz-placeholder {
	color: transparent;
}

.sapUiSizeCompact {
	/* min-width calculations */
	.sapMMultiInput.sapMMultiInputHasTokens {
		.sapMInputBaseContentWrapper {
			min-width: @_InputBase_Control_Tokens_MinWidth_Compact;
			&.sapMInputBaseContentWrapperState:not(.sapMInputBaseContentWrapperSuccess) {
				min-width: @_InputBase_Control_Tokens_State_MinWidth_Compact;
				outline-offset: -3px;
				.sapMToken {
					margin-top: 0.0625rem;
					margin-bottom: 0.0625rem;
				}
				.sapMTokenizerIndicator {
					line-height: 1rem;
				}
			}
		}

		&.sapMInputBaseReadonly .sapMInputBaseContentWrapper {
			min-width: @_InputBase_Control_Tokens_ReadOnly_MinWidth_Compact;
		}
	}
}

.sapUiSizeCondensed .sapUiTableDataCell {

	.sapMTokenizer .sapMToken {
		height: 1.125rem;
		margin-top: 0.125rem;
		margin-bottom: 0.125rem;
	}

	.sapMTokenText {
		height: 100%;
		line-height: 1rem;
	}

	.sapMToken .sapMTokenIcon {
		line-height: 1rem;
	}

	.sapMMultiInput .sapMInputBaseInner {
		height: 1.25rem;
	}

	.sapMMultiInputIndicator {
		line-height: 1.25rem;
	}
}

/* MultiInput with description */
.sapMMultiInput.sapMMultiInputHasTokens {
	.sapMInputDescriptionWrapper {
		max-width: ~"calc(100% - (@{_InputBase_Control_Tokens_MinWidth}))";
		.sapMInputBaseState:not(.sapMInputBaseContentWrapperSuccess) & {
			max-width: ~"calc(100% - (@{_InputBase_Control_Tokens_State_MinWidth}))";
		}

		.sapMInputBaseReadonly & {
			max-width: ~"calc(100% - (@{_InputBase_Control_Tokens_ReadOnly_MinWidth}))";
		}
	}

	&.sapMInputFocused .sapMTokenizer {
		left: 0.125rem;
	}

	.sapUiSizeCompact & {
		.sapMInputDescriptionWrapper {
			max-width: ~"calc(100% - (@{_InputBase_Control_Tokens_MinWidth_Compact}))";
			.sapMInputBaseState:not(.sapMInputBaseContentWrapperSuccess) & {
				max-width: ~"calc(100% - (@{_InputBase_Control_Tokens_State_MinWidth_Compact}))";
			}

			.sapMInputBaseReadonly & {
				max-width: ~"calc(100% - (@{_InputBase_Control_Tokens_ReadOnly_MinWidth_Compact}))";
			}
		}
	}
}
/* END "MultiInput.less" */

/* START "NavContainer.less" */
/* =================================== */
/* CSS for control sap.m/NavContainer  */
/* Base theme                          */
/* =================================== */

.sapMNav {
	overflow: hidden;
	position: relative;
	height: 100%;
}

.sapMNavFlip {
	-webkit-perspective: 1000;
	perspective: 1000px;
}

.sapMNavItem,
.sapMNavItem.sapMPage {
	position: absolute;
	top: 0;
	left: 0;
	display: block;
	height: 100%;
}

.sapMNavItem.sapMNavItemRendering {
	visibility: hidden;
	left: -4000px;
}

.sapMNavItem.sapMNavItemHidden {
	display: none;
}

/* animations */
html[data-sap-ui-animation='on'] {

	.sapMNavItem,
	.sapMNavItem.sapMPage {
		-webkit-transition-property: -webkit-transform;
		transition-property: transform;
	}

	/* for slide animation */
	.sapMNavItem.sapMNavItemCenter {
		-webkit-transform: translate3d(0px, 0px, 0px);
		transform: translate3d(0px, 0px, 0px);
	}

	.sapMNavItem.sapMNavItemSlideLeft {
		-webkit-transform: translate3d(1rem, 0px, 0px);
		transform: translate3d(1rem, 0px, 0px);
	}

	.sapMNavItem.sapMNavItemSlideRight {
		-webkit-transform: translate3d(-1rem, 0px, 0px);
		transform: translate3d(-1rem, 0px, 0px);
	}

    /* for base slide animation */

	.sapMNavItem.sapMNavItemSlideRightToCenter {
		-webkit-animation: sapMNavItemAnimationRightToCenter .3s;
		animation: sapMNavItemAnimationRightToCenter .3s;
	}

    .sapMNavItem.sapMNavItemSlideCenterToLeft {
        -webkit-animation: sapMNavItemAnimationCenterToLeft .3s;
        animation: sapMNavItemAnimationCenterToLeft .3s;
    }

    @-webkit-keyframes sapMNavItemAnimationRightToCenter {
        0% {
            -webkit-transform: translate3d(100%, 0px, 0px);
        }
        100% {
            -webkit-transform: translate3d(0px, 0px, 0px);
        }
    }

    @-webkit-keyframes sapMNavItemAnimationCenterToLeft {
        0% {
            -webkit-transform: translate3d(0px, 0px, 0px);
        }
        100% {
            -webkit-transform: translate3d(-100%, 0px, 0px);
        }
    }

    @keyframes sapMNavItemAnimationRightToCenter {
        0% {
            transform: translate3d(100%, 0px, 0px);
        }
        100% {
            transform: translate3d(0px, 0px, 0px);
        }
    }

    @keyframes sapMNavItemAnimationCenterToLeft {
        0% {
            transform: translate3d(0px, 0px, 0px);
        }
        100% {
            transform: translate3d(-100%, 0px, 0px);
        }
    }

	/* for fade animation */

	.sapMNavItem.sapMNavItemTransparent {
		-webkit-transition-property: opacity, transform;
		transition-property: opacity, transform;
		opacity: 0;
	}

	.sapMNavItem.sapMNavItemOpaque {
		-webkit-transition-property: opacity, transform;
		transition-property: opacity, transform;
		opacity: 1;
		-webkit-transform: translate3d(0px, 0px, 0px);
		transform: translate3d(0px, 0px, 0px);
	}

	.sapMNavItem.sapMNavItemFading {
		-webkit-transition-duration: .15s;
		transition-duration: .15s;
	}

	/* for flip animation */

	.sapMNavItem.sapMNavItemFlipNext {
		-webkit-transform: rotateY(180deg);
		transform: rotateY(180deg);
		-webkit-backface-visibility: hidden;
		backface-visibility: hidden;
	}

	.sapMNavItem.sapMNavItemFlipPrevious {
		-webkit-transform: rotateY(-180deg);
		transform: rotateY(-180deg);
		backface-visibility: hidden;
	}

	.sapMNavItem.sapMNavItemFlipping {
		-webkit-transition-duration: .5s;
		transition-duration: .5s;
		-webkit-backface-visibility: hidden;
		backface-visibility: hidden;
	}

	/* for door animation */

	@-webkit-keyframes sapMNavItemDoorInPrevious {
		from {
			-webkit-transform: rotate3d(0, 1, 0, 0deg);
			visibility: visible;
			opacity: 1;
		}
		40% {
			-webkit-transform: rotate3d(0, 1, 0, -70deg);
			visibility: visible;
			opacity: 0;
		}
		41% {
			-webkit-transform: rotate3d(0, 1, 0, -70deg);
			visibility: hidden;
		}

		to {
			-webkit-transform: rotate3d(0, 1, 0, -70deg);
		}
	}

	@keyframes sapMNavItemDoorInPrevious {
		from {
			transform: rotate3d(0, 1, 0, 0deg);
			visibility: visible;
			opacity: 1;
		}
		40% {
			transform: rotate3d(0, 1, 0, -70deg);
			visibility: visible;
			opacity: 0;
		}
		41% {
			transform: rotate3d(0, 1, 0, -70deg);
			visibility: hidden;
		}

		to {
			transform: rotate3d(0, 1, 0, -70deg);
		}
	}

	@-webkit-keyframes sapMNavItemDoorInNext {
		from {
			-webkit-transform: rotate3d(0, 1, 0, 80deg);
			opacity: 0;
		}
		46% {
			-webkit-transform: rotate3d(0, 1, 0, 80deg);
			opacity: 0;
		}
		to {
			-webkit-transform: rotate3d(0, 1, 0, 0deg);
			opacity: 1;
		}
	}

	@keyframes sapMNavItemDoorInNext {
		from {
			transform: rotate3d(0, 1, 0, 80deg);
			visibility: hidden;
			opacity: 0;
		}
		45% {
			transform: rotate3d(0, 1, 0, 80deg);
			visibility: hidden;
			opacity: 0;
		}
		46% {
			transform: rotate3d(0, 1, 0, 80deg);
			visibility: visible;
			opacity: 0;
		}
		to {
			transform: rotate3d(0, 1, 0, 0deg);
			opacity: 1;
		}
	}

	@-webkit-keyframes sapMNavItemDoorOutPrevious {
		from {
			-webkit-transform: rotate3d(0, 1, 0, 0deg);
			visibility: visible;
			opacity: 1;
		}
		40% {
			-webkit-transform: rotate3d(0, 1, 0, 80deg);
			visibility: visible;
			opacity: 0;
		}
		41% {
			-webkit-transform: rotate3d(0, 1, 0, 80deg);
			visibility: hidden;
		}
		to {
			-webkit-transform: rotate3d(0, 1, 0, 80deg);
		}
	}

	@keyframes sapMNavItemDoorOutPrevious {
		from {
			transform: rotate3d(0, 1, 0, 0deg);
			visibility: visible;
			opacity: 1;
		}
		40% {
			transform: rotate3d(0, 1, 0, 80deg);
			visibility: visible;
			opacity: 0;
		}
		41% {
			transform: rotate3d(0, 1, 0, 80deg);
			visibility: hidden;
		}
		to {
			transform: rotate3d(0, 1, 0, 80deg);
		}
	}

	@-webkit-keyframes sapMNavItemDoorOutNext {
		from {
			-webkit-transform: rotate3d(0, 1, 0, -70deg);
			opacity: 0;
		}
		46% {
			-webkit-transform: rotate3d(0, 1, 0, -70deg);
			opacity: 0;
		}

		to {
			-webkit-transform: rotate3d(0, 1, 0, 0deg);
			opacity: 1;
		}
	}

	@keyframes sapMNavItemDoorOutNext {
		from {
			transform: rotate3d(0, 1, 0, -70deg);
			visibility: hidden;
			opacity: 0;
		}
		45% {
			transform: rotate3d(0, 1, 0, -70deg);
			visibility: hidden;
			opacity: 0;
		}
		46% {
			transform: rotate3d(0, 1, 0, -70deg);
			visibility: visible;
			opacity: 0;
		}

		to {
			transform: rotate3d(0, 1, 0, 0deg);
			opacity: 1;
		}
	}

	.sapMNavDoor {
		-webkit-perspective: 1000;
		perspective: 1000px;
		-webkit-perspective-origin: 0% 50%;
		perspective-origin: 0% 50%;
	}

	.sapMNavItem.sapMNavItemDoorInPrevious {
		-webkit-animation: sapMNavItemDoorInPrevious .8s;
		animation: sapMNavItemDoorInPrevious .8s;
		visibility: hidden;
		opacity: 0;
	}

	.sapMNavItem.sapMNavItemDoorInNext {
		-webkit-animation: sapMNavItemDoorInNext .8s;
		animation: sapMNavItemDoorInNext .8s;
		visibility: visible;
	}

	.sapMNavItem.sapMNavItemDoorOutPrevious {
		-webkit-animation: sapMNavItemDoorOutPrevious .8s;
		animation: sapMNavItemDoorOutPrevious .8s;
		visibility: hidden;
		opacity: 0;
	}

	.sapMNavItem.sapMNavItemDoorOutNext {
		-webkit-animation: sapMNavItemDoorOutNext .8s;
		animation: sapMNavItemDoorOutNext .8s;
		visibility: visible;
	}

	.sapMNavItem.sapMNavItemDooring {
		-webkit-animation-timing-function: ease-out;
		animation-timing-function: ease-out;
		-webkit-transform-origin: 0% 50%;
		transform-origin: 0% 50%;
	}

}
/* END "NavContainer.less" */

/* START "NewsContent.less" */
/* ================================== */
/* CSS for control sap.m/NewsContent  */
/* Base theme                         */
/* ================================== */

.sapMNwC {
	display: inline-block;
	outline: none;
	width: 100%;
	height: 3.5625rem;
	color: var(--sapTile_TextColor);
	background-color: var(--sapTile_Background);
}

.sapMNwCCTxt {
	min-height: fit-content;
	text-overflow: ellipsis;
	white-space: nowrap;
	font-family: var(--sapFontFamily);
	font-weight: normal;
	margin-top: 0.1875rem;
	max-width: 20.5rem;

	.sapMText {
		display: inline;
		line-height: 1.1875rem;
	}

	.sapMText.sapMPointer * {
		cursor: pointer;
	}
}

.Stretch .sapMNwCCTxt {
	max-width: none;

	.sapMFT {
		display: -webkit-box;
		-webkit-line-clamp: 2;
		-webkit-box-orient: vertical;
		font-weight: bold;
		overflow: hidden;
	}
}

.TwoByOne .sapMNwCCTxt {
	display: -webkit-box;
	-webkit-line-clamp: 2;
	-webkit-box-orient: vertical;
}

.Stretch {

	.sapMTileCnt.News .sapMNwC {
		height: 100%;
		display: flex;
		flex-direction: column;
		justify-content: space-between;

		.sapMNwCSbh {
			max-width: none;
			padding-top: 0.25rem;

			.sapMFT {
				word-wrap: break-word;
				white-space: normal;
			}
		}
	}

	.sapMNwCExtend {
		height: 100%;

		.sapMFT {
			display: -webkit-box;
			-webkit-line-clamp: 1;
			-webkit-box-orient: vertical;
			overflow: hidden;
		}
	}
}

.sapMNwCSbh {
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
	font-family: var(--sapFontFamily);
	font-weight: normal;
	max-width: 20.5rem;
	width: 100%;
	display: inline-block;
	line-height: 1.1875rem;
	color: var(--sapTile_TextColor);
	font-size: @sapMFontMediumSize;
}

.sapMNwCCTxt .sapMText,
.sapMNwCCTxt .sapMFT {
	font-size: @sapMFontMediumSize;
	color: var(--sapContent_ContrastTextColor);
	overflow-wrap: break-word;
	white-space: pre-line;
}

.sapMNwC:focus:not([tabindex]) {
	outline: none;
}

.sapMNwC:focus {
	outline: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
	outline-offset: 1px;
}

.sapMTileSmallPhone {

	.sapMNwC {
		height: 3.188rem;
	}

	.sapMNwCCTxt {
		max-width: 17.75rem;
		padding: 0;

		.sapMText {
			line-height: 1.063rem;
		}
	}

	&.sapMGTStretch {

		.sapMNwCCTxt {
			max-width: none;
		}
	}

	.sapMNwCSbh {
		max-width: 17.75rem;
		line-height: 1.063rem;
	}

	.sapMNwCCTxt .sapMText,
	.sapMNwCSbh {
		font-size: var(--sapFontSmallSize);
	}

	.sapMTileSmallPhone.Stretch {

		.sapMNwCSbh {
			padding-top: 0.5rem;
		}

		.sapMNwCSbh > .sapMFT {
			white-space: nowrap;
			overflow: hidden;
			text-overflow: ellipsis;
		}
	}
}
/* END "NewsContent.less" */

/* START "NotificationListBase.less" */
/* =========================================== */
/* CSS for control sap.m/NotificationListBase  */
/* Base theme                                  */
/* =========================================== */

.sapMNLIB {
	position: relative;

	&:focus {
		outline: none;
	}

	.sapMNLICloseBtn:not(:empty) {
		margin: 0 0.5rem;
	}
}

/* Priority icons */

.sapMNLIBPriority {
	padding-right: 0.625rem;

	.sapUiIcon {
		font-size: 1rem;
		line-height: 1rem;
	}
}

.sapMNLIBPriorityLow .sapUiIcon {
	color: var(--sapSuccessBorderColor);

	&::before {
		content: '\e1c1';
	}
}

.sapMNLIBPriorityMedium .sapUiIcon {
	color: var(--sapWarningBorderColor);

	&::before {
		content: '\e053';
	}
}

.sapMNLIBPriorityHigh .sapUiIcon {
	color: var(--sapErrorBorderColor);

	&::before {
		content: '\e1ec';
	}
}

.sapMNLIBHiddenButton {
	display: none;
}

.sapUiSizeCompact {

	.sapMNLIBPriority {
		padding-right: 0.5rem;
	}
}
/* END "NotificationListBase.less" */

/* START "NotificationListGroup.less" */
/* ============================================ */
/* CSS for control sap.m/NotificationListGroup  */
/* Base theme                                   */
/* ============================================ */

@_sap_m_NotificationListGroup_Background: var(--sapList_GroupHeaderBackground);
@_sap_m_NotificationListGroup_HeaderColor: var(--sapList_TableGroupHeaderTextColor);

.sapMNLGroup {

	&.sapMLIB {
		display: block;
		padding: 0;
		margin: 0;
		overflow: hidden;
		border-bottom: none;
	}

	&.sapMNLGroupCollapsed .sapMNLGroupChildren {
		display: none;
	}

	.sapMNLGroupHeader {
		position: relative;
		min-height: 2.75rem;
		background: @_sap_m_NotificationListGroup_Background;
		border-bottom: 0.0625rem solid var(--sapList_GroupHeaderBorderColor);
		box-sizing: border-box;
		padding: 0.75rem 0.5rem 0 1rem;
		display: flex;
		align-items: flex-start;
		outline: none;

		.sapMNLIBPriority {
			padding-top: 0.75rem;
		}

		.sapMNLGroupTitle {
			padding: 0.75rem 0;
			color: @_sap_m_NotificationListGroup_HeaderColor;
			font-size: var(--sapFontHeader6Size);
			font-family: var(--sapFontHeaderFamily);
			font-weight: bold;
			overflow-wrap: break-word;
			flex: 1;
			min-width: 0;
		}
	}

	&:focus .sapMNLGroupHeader::before {
		content: ' ';
		border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
		position: absolute;
		top: 1px;
		right: 1px;
		bottom: 1px;
		left: 1px;
		z-index: 2;
		pointer-events: none;
	}

	.sapMNLGroupChildren {
		margin: 0;
		padding: 0;
	}

	.sapMNLGroupCollapseButton {
		margin-right: 0.75rem;
		width: 2.25rem;
	}

	.sapMNLGroupMaxNotifications {
		padding: 1rem;
		background: var(--sapList_Background);
		text-align: center;

		.sapMNLGroupMNTitle {
			font-family: var(--sapFontHeaderFamily);
			font-size: var(--sapFontHeader5Size);
			font-weight: @sapUiFontHeaderWeight;
			color: var(--sapGroup_TitleTextColor);
			margin-bottom: 0.5rem;
		}

		.sapMNLGroupMNDescription {
			font-family: var(--sapFontFamily);
			font-size: @sapMFontMediumSize;
			color: var(--sapContent_LabelColor);
		}
	}
}

.sapMNLIActionsHidden {
	display: none;
}

.sapUiSizeCompact {

	.sapMNLGroup {

		.sapMNLGroupHeader {
			padding: 0.5rem 0.5rem 0 0.75rem;

			.sapMNLIBPriority {
				padding-top: 0.5rem;
			}

			.sapMNLGroupTitle {
				padding: 0.5rem 0;
			}
		}
	}
}
/* END "NotificationListGroup.less" */

/* START "NotificationListItem.less" */
/* =========================================== */
/* CSS for control sap.m/NotificationListItem  */
/* Base theme                                  */
/* =========================================== */

@_sap_m_NotificationListBase_ItemBackground: var(--sapList_Background);
@_sap_m_NotificationListBase_ItemGroupedBackground: var(--sapList_Background);

.sapMNLI {
	background: @_sap_m_NotificationListBase_ItemBackground;
	cursor: pointer;
	display: flex;
	flex-direction: column;
	padding: 1rem 0.5rem 1rem 1rem;

	.sapMMsgStrip  {
		margin-bottom: 1rem;
	}

	.sapMNLIMain {
		display: flex;
		flex-direction: row-reverse;
		width: 100%;
	}

	.sapMNLIItemAC {
		display: flex;
	}

	.sapMNLIImage {
		margin-right: 0.75rem;
	}

	.sapMNLIContent {
		display: flex;
		flex-direction: column;
		min-width: 0; /* chrome flex overflow issue */
		flex: 1;
		padding-right: 1rem;

		.sapMNLITitle {
			display: flex;
			align-items: flex-start;
			font-size: var(--sapFontHeader6Size);
			font-weight: @sapUiFontHeaderWeight;
			line-height: 1.4;
			color: var(--sapGroup_TitleTextColor);
			padding-bottom: 0.25rem;
		}

		.sapMNLIDescription {
			font-size: @sapMFontMediumSize;
			color: @sapUiBaseText;
			line-height: 1.4;
			margin-top: 0.5rem;
			word-wrap: break-word;

			&.sapMNLIDescriptionNoText {
				margin-top: 0;
			}
		}
	}

	.sapMNLIFooter {
		display: flex;
		align-items: center;
		padding-top: 0.5rem;

		.sapMNLIFooterItem {
			font-size: @sapMFontMediumSize;
			color: var(--sapContent_LabelColor);
			padding-top: 2px;
			white-space: nowrap;
		}

		/* the author name should truncate */
		.sapMNLIFooterItem:first-child {
			overflow: hidden;
			text-overflow: ellipsis;
			white-space: nowrap;
		}

		.sapMNLIFooterBullet {
			margin: 0 0.5rem;
		}

		.sapMBtn {
			height: 2rem;
			box-sizing: border-box;
		}
	}

	&.sapMNLIUnread {

		.sapMNLIContent {

			.sapMNLITitle {
				font-weight: bold;
			}
		}
	}

	&.sapMNLINoAvatar .sapMNLIImage,
	&.sapMNLIB-LayoutS .sapMNLIImage {
		display: none;
	}

	&:focus::before {
		content: ' ';
		border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
		position: absolute;
		top: 1px;
		right: 1px;
		bottom: 1px;
		left: 1px;
		z-index: 2;
		pointer-events: none;
	}
}

.sapMNLGroup .sapMNLI {
	background: @_sap_m_NotificationListBase_ItemGroupedBackground;
}

.sapMNLIItemTextLineClamp {
	display: -webkit-box;
	-webkit-line-clamp: 2;
	-webkit-box-orient: vertical;
	overflow: hidden;
	max-height: 40px; /* font-size * line-height (value 1.4) * 2 (number of rows) */
}

.sapMNLIShowMore .sapMLnk {
	vertical-align: middle;
	margin-left: 1rem;
}

.sapMNLITitleText {
	font-size: var(--sapFontHeader6Size);
}

.sapMLIB.sapMNLI {
	align-items: normal;
}

.sapUiSizeCompact {

	.sapMNLI {
		padding: 1rem 0.5rem 1rem 0.75rem;
	}
}
/* END "NotificationListItem.less" */

/* START "NumericContent.less" */
/* ============================= */
/* CSS for sap.m/NumericContent  */
/* Base theme                    */
/* ============================= */

.sapMNC {
	position: relative;
	box-sizing: border-box;
	outline: none;

	height: 3.875rem;
	width: 9rem;

	&.WithoutMargin {
		width: 100%;
	}
}

.sapMNCAnimation {
	animation-name: sapMCOpacityAnimation;
	animation-duration: 1s;
}

@keyframes sapMCOpacityAnimation {

	0% {
		opacity: 0.25;
	}

	100% {
		opacity: 1;
	}
}

.sapMNCInner {
	position: absolute;
	bottom: 0;
	width: 100%;

	&.WithoutMargin {
		overflow: visible;
		white-space: nowrap;
		display: flex;
	}
}

td .sapMNCValue.WithoutMargin {
	justify-content: flex-end;
}

.sapMNCIconImage {
	display: flex;
	flex-direction: column;
	justify-content: center;
	padding-right: 0.375rem;
	font-family: var(--sapFontHeaderFamily);
	font-size: 1.75rem;
	font-weight: @sapUiFontHeaderWeight;
	color: var(--sapTile_IconColor);

	&.sapMNCLargeFontSize::before {
		padding-bottom: 0.25rem;
	}

	&.sapMNCMediumFontSize::before {
		padding-bottom: 0.25rem;
	}

	&.sapMNCSmallFontSize::before {
		padding-bottom: 0.25rem;
	}
}

.sapMNCValue {
	display: inline-flex;
	justify-content: flex-end;
	position: relative;
	width: 100%;
	white-space: nowrap;
	text-align: right;
	text-overflow: ellipsis;
	font-size: 2.25rem;
	font-family: var(--sapFontLightFamily);
	font-weight: @sapUiFontHeaderWeight;

	&.WithoutMargin {
		justify-content: flex-start;
	}
}

.sapMNCLargeFontSize {
	height: 2.875rem;
	padding-top: 0.125rem;

	&:not(.sapMNCIconImage) {
		font-size: 2.25rem;
	}
}

.sapMNCMediumFontSize {
	height: 2.5rem;
	padding-top: 0;

	&:not(.sapMNCIconImage) {
		font-size: 2rem;
	}
}

.sapMNCSmallFontSize {
	height: 2.25rem;
	padding-top: 0;

	&:not(.sapMNCIconImage) {
		font-size: 1.75rem;
	}
}

html[dir=rtl] .sapMNCValue {
	text-align: left;
}

.sapMNCIndIcon {
	grid-column: 1;
	grid-row: 1;
}

.sapMNCIndScale {
	display: grid;
	text-overflow: ellipsis;
	text-align: center;

	&.sapMNCLargeFontSize {
		height: 2.125rem;
		padding: 0.5rem 0 0.375rem 0.375rem;

		.sapMNCScale {
			margin-top: 0.9375rem;
		}
	}

	&.sapMNCMediumFontSize {
		height: 1.938rem;
		padding: 0.25rem 0 0.5rem 0.5rem;

		.sapMNCScale {
			margin-top: 0.875rem;
		}
	}

	&.sapMNCSmallFontSize {
		height: 1.938rem;
		padding: 0 0 0.313rem 0.5rem;

		.sapMNCScale {
			margin-top: 0.875rem;
		}
	}

	&.WithoutMargin {
		float: left;
	}
}

.sapMNCScale {
	font-size: var(--sapFontSize);
	font-family: var(--sapFontHeaderFamily);
	font-weight: @sapUiFontHeaderWeight;
	grid-column: 1;
	grid-row: 1;
}

.sapMListTblSubRowCell .sapMNCValue {
	display: contents;
}

.sapMNCValue.Neutral,
.sapMNCIndScale.Neutral,
.sapMNCScale.Neutral {
	color: var(--sapNeutralTextColor);
}

.sapMNCValue.Good,
.sapMNCIndScale.Good,
.sapMNCScale.Good {
	color: var(--sapPositiveTextColor);
}

.sapMNCValue.Error,
.sapMNCIndScale.Error,
.sapMNCScale.Error {
	color: var(--sapNegativeTextColor);
}

.sapMNCValue.Critical,
.sapMNCIndScale.Critical,
.sapMNCScale.Critical {
	color: var(--sapCriticalTextColor);
}

.sapMNCIndScale.Loading,
.sapMNCScale.Loading,
.sapMNCIconImage.Loading,
.sapMNCValue.Failed {
	color: var(--sapContent_LabelColor);
	opacity: 0.25;
}

.sapMNC:focus {
	outline: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
}

.sapMNC:focus:not([tabindex]) {
	outline: none;
}

.sapMTileSmallPhone {

	.sapMNC {
		height: 3.438rem;
		width: 8rem;
	}

	.sapMNCLargeFontSize {
		font-size: var(--sapFontHeader3Size);
		height: 1.875rem;
		padding-top: 0.5rem;
	}

	.sapMNCMediumFontSize {
		font-size: var(--sapFontHeader3Size);
		height: 1.875rem;
		padding-top: 0.5rem;
	}

	.sapMNCSmallFontSize {
		font-size: var(--sapFontHeader3Size);
		height: 1.875rem;
		padding-top: 0.5rem;
	}

	.sapMNCIndScale {
		padding: 0.25rem 0 0.25rem 0.375rem;

		&.sapMNCSmallFontSize {
			height: 2.25rem;
			padding-bottom: 0;
			padding-top: 0;

			.sapMNCIndIcon {
				padding-top: 0.3125rem;
			}

			.sapMNCScale {
				margin-top: 1.0625rem;
			}
		}

		.sapMNCIndIcon:only-child { /* when there's no scale */
			margin-top: 0.5rem;
		}

		.sapMNCScale {
			margin-top: 0.8125rem;
		}
	}

	.sapMGTHdrTxt > .sapMText {
		font-size: var(--sapFontHeader6Size);
	}
}

.sapMNCContentShimmerPlaceholderItem {
	padding: 1rem 1rem 0 1rem;
	box-sizing: border-box;
	display: flex;
	position: relative;
	align-items: center;
	flex: 1 1 auto;

	.sapMNCContentShimmerPlaceholderRows {
		flex-grow: 1;

		.sapMNCContentShimmerPlaceholderItemHeader {
			height: 1rem;
			margin-bottom: 0.5rem;
			margin-top: 0.5rem;
		}
	}
}

.sapMNCLoadingShimmer {
	.sapMNCLoadingShimmerMixin();
}

.sapMNCLoadingShimmerMixin() {
	background-color: var(--sapContent_Placeholderloading_Background);
	background-image: var(--sapContent_Placeholderloading_Gradient);
	background-repeat: no-repeat;
	background-size: 800px 144px;
	position: relative;
	border-radius: 0.25rem;

	[data-sap-ui-animation-mode='full'] & {
		animation-duration: 2.5s;
		animation-fill-mode: forwards;
		animation-iteration-count: infinite;
		animation-name: placeHolderShimmer;
		animation-timing-function: linear;
	}

	[data-sap-ui-animation-mode='basic'] &,
	[data-sap-ui-animation-mode='minimal'] & {
		animation-duration: 8s;
		animation-fill-mode: forwards;
		animation-iteration-count: infinite;
		animation-name: placeHolderShimmer;
		animation-timing-function: linear;
	}
}

@keyframes placeHolderShimmer {

	0% {
		background-position: -468px 0;
	}

	100% {
		background-position: 468px 0;
	}
}

@keyframes placeHolderShimmer {

	0% {
		background-position: -468px 0;
	}

	100% {
		background-position: 468px 0;
	}
}
/* END "NumericContent.less" */

/* START "ObjectAttribute.less" */
/* ====================================== */
/* CSS for control sap.m/ObjectAttribute  */
/* Base theme                             */
/* ====================================== */

.sapMObjectAttributeDiv .sapMText,
.sapMObjectAttributeDiv .sapMObjectAttributeTitle,
.sapMObjectAttributeDiv .sapMObjectAttributeColon,
.sapMObjectAttributeDiv .sapMObjectAttributeText {
	font-size: .875rem;
	color: var(--sapContent_LabelColor);
}

.sapMObjectAttributeDiv .sapMText {
	vertical-align: top;
}

.sapMObjectAttributeActive {
	pointer-events: none;

	.sapMObjectAttributeColon {
		vertical-align: top;
	}
}

.sapMListTbl .sapMObjectAttributeActive .sapMText,
.sapMObjectAttributeActive .sapMText,
.sapMObjectAttributeActive .sapMObjectAttributeText {
	color: var(--sapLinkColor);
	cursor: pointer;
	pointer-events: auto;
}

.sapMObjectAttributeActive .sapMObjectAttributeTitle,
.sapMObjectAttributeActive .sapMObjectAttributeText {
	pointer-events: auto;
}

.sapMObjectAttributeActive .sapMObjectAttributeText:active,
.sapMObjectAttributeActive .sapMObjectAttributeText:hover,
.sapMObjectAttributeActive .sapMObjectAttributeText:focus,
.sapMObjectAttributeActive .sapMText:active,
.sapMObjectAttributeActive .sapMText:focus,
.sapMObjectAttributeActive .sapMText:hover {
	text-decoration: underline;
	outline: none;
}

.sapMObjectAttributeActive {
	display: block;
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
}

.sapMObjectAttributeActive {
	white-space: normal;
	word-break: break-word;

	.sapMObjectAttributeTitle,
	.sapMObjectAttributeText {
		white-space: normal;
		vertical-align: top;
		/* we need this otherwise the focus get broken when the link is on more lines */
		display: inline-block;
	}
}

.sapMObjectAttributeText > .sapMLnk {
	display: inline;
}

.sapMObjectAttributeActive.sapMLnkLargeReactiveArea .sapMObjectAttributeText > .sapMLnk {
	display: inline-block;
}

.sapMOHR .sapMObjectAttributeText > .sapMText {
	display: inline;
}

.sapMOH .sapMObjectAttributeText > .sapMText {
	vertical-align: top;
}

.sapMObjectAttributeDiv {
	font-size: @sapMFontMediumSize;
}

.sapMLIBActive .sapMObjectAttributeText,
.sapMLIBActive .sapMObjectAttributeDiv {
	color: var(--sapList_Active_TextColor);
}

.sapMObjectAttributeActive > span:only-of-type {
	max-width: 100%;
}

/*=========================================== */
/* Custom design for Responsive Object Header */
/*=========================================== */

.sapMOHR .sapMOHRAttr .sapMObjectAttributeTitle,
.sapMOHR .sapMOHRAttr .sapMObjectAttributeText {
	display: inline-block;
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	font-weight: normal;
	color: var(--sapContent_LabelColor);
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
	max-width: 50%;
	vertical-align: top;
}

.sapMOHR .sapMOHRAttr .sapMObjectAttributeTitle {
	padding-right: 0.5rem;
	max-width: 50%;
	box-sizing: border-box;
}

.sapMOHR .sapMOHRAttr .sapMObjectAttributeDiv > span:only-of-type {
	max-width: 100%;
}

.sapMOHR .sapMOHRAttr .sapMObjectAttributeColon {
	display: inline-block;
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	font-weight: normal;
	color: var(--sapContent_LabelColor);
	white-space: pre;
	margin-left: -0.5rem;
	overflow: auto;
	vertical-align: top;
}

.sapMOHR .sapMObjectAttributeActive .sapMObjectAttributeText {
	color: var(--sapLinkColor);
	cursor: pointer;
}

.sapMOHR .sapMObjectAttributeActive .sapMObjectAttributeText:active,
.sapMOHR .sapMObjectAttributeActive .sapMObjectAttributeText:hover,
.sapMOHR .sapMObjectAttributeActive .sapMObjectAttributeText:focus {
	text-decoration: none;
}

.sapMOHR .sapMOHRAttr .sapMObjectAttributeDiv {
	display: block;
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
}

/*=========================================== */
/* Custom design for Static Object Header     */
/*=========================================== */

.sapMOH .sapMOHAttr .sapMObjectAttributeDiv {
	max-height: 2rem;
	overflow: hidden;
}

.sapMOH .sapMOHAttr .sapMObjectAttributeTitle,
.sapMOH .sapMOHAttr .sapMObjectAttributeText,
.sapMOH .sapMOHAttr .sapMObjectAttributeColon {
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	font-weight: normal;
	color: var(--sapContent_LabelColor);
}

.sapMOH .sapMObjectAttributeActive .sapMObjectAttributeText {
	color: var(--sapLinkColor);
	cursor: pointer;
}

.sapMOH .sapMOHAttr .sapMObjectAttributeDiv > span:only-of-type {
	max-width: 100%;
}

.sapMOH .sapMObjectAttributeActive .sapMObjectAttributeText:active,
.sapMOH .sapMObjectAttributeActive .sapMObjectAttributeText:hover,
.sapMOH .sapMObjectAttributeActive .sapMObjectAttributeText:focus {
	text-decoration: none;
}
/* END "ObjectAttribute.less" */

/* START "ObjectHeader.less" */
/* =================================== */
/* CSS for control sap.m/ObjectHeader  */
/* Base theme                          */
/* =================================== */

.sapMOH h6, .sapMOHR h6,
.sapMOH h5, .sapMOHR h5,
.sapMOH h4, .sapMOHR h4,
.sapMOH h3, .sapMOHR h3,
.sapMOH h2, .sapMOHR h2,
.sapMOH h1, .sapMOHR h1 {
	margin: 0;
	font-weight: normal;
}

.sapMOH {
	padding: 3rem;
}

.sapMOH .sapMOHTitleActive:focus {
	outline: none;
}

html.sap-tablet.sapUiMedia-Std-Tablet .sapMOH,
html:not(.sapUiMedia-Std-Phone) .sapMSplitContainer:not(.sapMSplitContainerHideMode) .sapMOH {
	padding: 1.5rem 2rem;
}

html .sapUiContainer-Narrow .sapMOH,
html.sap-desktop.sapUiMedia-Std-Phone .sapMOH,
html.sap-phone .sapMOH {
	padding: 2rem 1rem;
}

.sapMPopover .sapMOH {
	padding: 2rem 1rem;
}

.sapMOH.sapMOHC {
	margin: 0;
	padding: 1.125rem 2rem 1rem 2rem;
}

html.sap-tablet.sapUiMedia-Std-Tablet .sapMOH.sapMOHC,
html:not(.sapUiMedia-Std-Phone) .sapMSplitContainer:not(.sapMSplitContainerHideMode) .sapMOH.sapMOHC {
	margin: 0;
	padding: 1.125rem 2rem 1rem 2rem;
}

html.sap-phone .sapMOH.sapMOHC {
	margin: 0;
	padding: 1rem;
}

.sapMDialog .sapMOH.sapMOHC {
	padding: 1rem;
}

.sapMPopover .sapMOH.sapMOHC {
	margin: 0;
	padding: 1.125rem 0 1rem 0;
}

.sapMOHIntro {
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
	font-size: @sapMFontMediumSize;
	margin-top: 0.375rem;
	margin-bottom: 0.625rem;
	color: var(--sapContent_LabelColor);
}

.sapMOHIntroActive > a:active,
html.sap-desktop .sapMOHIntroActive > a:hover,
.sapMOHTitleActive h6 .sapMText:active,
html.sap-desktop .sapMOHTitleActive h6 .sapMText:hover,
.sapMOHTitleActive h5 .sapMText:active,
html.sap-desktop .sapMOHTitleActive h5 .sapMText:hover,
.sapMOHTitleActive h4 .sapMText:active,
html.sap-desktop .sapMOHTitleActive h4 .sapMText:hover,
.sapMOHTitleActive h3 .sapMText:active,
html.sap-desktop .sapMOHTitleActive h3 .sapMText:hover,
.sapMOHTitleActive h2 .sapMText:active,
html.sap-desktop .sapMOHTitleActive h2 .sapMText:hover,
.sapMOHTitleActive h1 .sapMText:active,
html.sap-desktop .sapMOHTitleActive h1 .sapMText:hover {
	text-decoration: underline;
}

.sapMOHTitleDiv {
	box-sizing: border-box;
	max-height: 6.3rem;
	width: 65%;
	overflow: hidden;
	float: left;
}

.sapMOHC > .sapMOHTitleDiv {
	padding-right: 0.5rem;
}

.sapMOHC > .sapMOHTitleDivFull,
.sapMOHTitleDivFull {
	width: 100%;
}

.sapMOHTitleDiv.sapMOHNumberWrap {
	max-height: 4.2rem;
}

html.sap-phone .sapMOHTitleDiv {
	max-height: 5rem;
}

html.sap-phone .sapMOHTitleDiv.sapMOHNumberWrap {
	max-height: 3.4rem;
}

.sapMOHIcon {
	position: absolute;
	margin-top: 0.375rem;
}

.sapMOHIconCircle .sapMImg {
	border-radius: 100%;
}

.sapMOHIcon .sapMImg {
	max-height: 3rem;
	max-width: 3rem;
}

.sapMOHTitle {
	margin-top: 0.375rem;
	display: inline-block;
	position: relative;
}

html.sap-phone .sapMOHTitle .sapMText,
.sapUiMedia-Std-Phone .sapMOHTitle .sapMText,
html .sapUiContainer-Narrow .sapMOHTitle .sapMText {
	display: table-cell;
	word-wrap: break-word;
	word-break: break-word;
}

.sapMText.sapMOHTitle {
	position: relative;
}

.sapMOHTitle h6 .sapMText,
.sapMOHTitle h5 .sapMText,
.sapMOHTitle h4 .sapMText,
.sapMOHTitle h3 .sapMText,
.sapMOHTitle h2 .sapMText,
.sapMOHTitle h1 .sapMText {
	font-size: 1.375rem;
}

.sapMListTbl .sapMOHTitle h6 .sapMText,
.sapMListTbl .sapMOHTitle h5 .sapMText,
.sapMListTbl .sapMOHTitle h4 .sapMText,
.sapMListTbl .sapMOHTitle h3 .sapMText,
.sapMListTbl .sapMOHTitle h2 .sapMText,
.sapMListTbl .sapMOHTitle h1 .sapMText {
	font-size: 0.875rem;
}

.sapMOHTitleFollowArrow {
	margin-right: 3rem;
}

.sapMOHC .sapMOHTitleFollowArrow {
	margin-right: 2.5rem;
}

.sapMOHTitleAndArrow {
	display: inline-block;
	position: relative;
	min-height: 3rem;
}

.sapMOHTitleIcon {
	min-height: 3rem;
}

html.sap-phone .sapMOHTitleIcon {
	min-height: 2.5rem;
}

html.sap-phone .sapMText.sapMOHTitle {
	font-size: 1.188rem;
}

.sapMOHIcon + .sapMOHTitle {
	margin-left: 4rem;
}

.sapMOHIcon + .sapMOHTitleAndArrow {
	margin-left: 3.5rem;
}

.sapMOHTitleArrow {
	display: inline-block;
	vertical-align: top;
	position: absolute;
	right: 0;
	color: var(--sapContent_IconColor);
}

.sapMOHC .sapMOHTitleArrow {
	margin-top: 0.3rem;
}

.sapMOHTitleArrow .sapUiIcon {
	margin-top: 1px;
	vertical-align: top;
	display: inline-block;
	padding: 0.4rem 0.5rem 0.4rem 0.5rem;
}

html.sap-desktop.sapUiMedia-Std-Phone .sapMOHTitleArrow .sapUiIcon,
html.sap-phone .sapMOHTitleArrow .sapUiIcon,
html .sapUiContainer-Narrow .sapMOHTitleArrow .sapUiIcon {
	padding: 0.3rem 0.4rem 0.3rem 0.4rem;
}

.sapMOHTitle + .sapMOHTitleArrow {
	margin-left: 0.5rem;
}

.sapMOHNumberDiv {
	margin-top: 0.375rem;
	display: inline-block;
	float: right;
	text-align: end;
	overflow: hidden;
	white-space: nowrap;
}

.sapMOHC > .sapMOHNumberDiv {
	padding-left: 0.5rem;
}

.sapMOHC .sapMOHTitle h6 >.sapMText,
html.sap-phone .sapMOHC .sapMOHTitle h6 >.sapMText,
.sapMOHC .sapMOHTitle h5 >.sapMText,
html.sap-phone .sapMOHC .sapMOHTitle h5 >.sapMText,
.sapMOHC .sapMOHTitle h4 >.sapMText,
html.sap-phone .sapMOHC .sapMOHTitle h4 >.sapMText,
.sapMOHC .sapMOHTitle h3 >.sapMText,
html.sap-phone .sapMOHC .sapMOHTitle h3 >.sapMText,
.sapMOHC .sapMOHTitle h2 >.sapMText,
html.sap-phone .sapMOHC .sapMOHTitle h2 >.sapMText,
.sapMOHC .sapMOHTitle h1 >.sapMText,
html.sap-phone .sapMOHC .sapMOHTitle h1 >.sapMText {
	line-height: 1.8rem;
	font-size: 1.125rem;
}

.sapMOHNumberWrap {
	width: 100%;
}

html.sap-phone .sapMOHNumberWrap {
	margin-top: 0.25rem;
}

.sapMOHNumber {
	display: block;
	font-size: 1.75rem;
	margin-top: -0.25rem;
}

.sapMOHNumberUnit {
	display: block;
	font-size: 0.75rem;
	margin-top: -0.25rem;
}

.sapMOHBottomRow {
	margin-top: 1rem;
	font-size: @sapMFontMediumSize;
}

.sapMOHAttrRow {
	margin-top: 0.625rem;
	overflow: hidden;
}

.sapMOHAttrRow:first-child {
	margin-top: 0;
}

.sapMOHC > .sapMOHAttr {
	margin-top: 0.125rem;
}

.sapMOHAttr {
	box-sizing: border-box;
	display: inline-block;
	width: 65%;
	text-align: start;
	vertical-align: top;
	overflow: hidden;
	float: left;
}

.sapMOH .sapMObjectAttributeDiv:not(.sapMObjectAttributeActive ) {
	display: inline-block;
	text-overflow: inherit;
	white-space: inherit;
	color: var(--sapContent_LabelColor);
	vertical-align: middle;
}

.sapMOHStatus,
.sapMOHStatusFixedWidth {
	padding: 0.125rem 0;
	box-sizing: border-box;
	display: inline-block;
	text-align: end;
	vertical-align: top;
	overflow: hidden;
	float: right;
	white-space: nowrap;

	.sapMObjStatusText {
		word-break: normal;
	}
}

.sapMOHStatusFixedWidth {
	width: 35%;
}

.sapMOHStatusFixedWidth > .sapMPI {
	float: right;
}

.sapMOHDivider,
.sapMOHLastDivider {
	clear: both;
}

.additionalOHNumberDiv.sapMOHNumberDiv {
	padding-right: 1em;
}

html.sapUiMedia-Std-Phone .additionalOHNumberSeparatorDiv,
html.sapUiMedia-Std-Tablet .additionalOHNumberSeparatorDiv,
html.sapUiMedia-Std-Phone .additionalOHNumberDiv.sapMOHNumberDiv,
html.sapUiMedia-Std-Tablet .additionalOHNumberDiv.sapMOHNumberDiv,
html .sapUiContainer-Narrow .additionalOHNumberSeparatorDiv,
html .sapUiContainer-Medium .additionalOHNumberSeparatorDiv,
html .sapUiContainer-Narrow .additionalOHNumberDiv.sapMOHNumberDiv,
html .sapUiContainer-Medium .additionalOHNumberDiv.sapMOHNumberDiv {
	display: none;
}

.additionalOHNumberDiv.sapMOHNumberDiv .sapMObjectNumber:not(.sapMObjectNumberStatusError):not(.sapMObjectNumberStatusWarning):not(.sapMObjectNumberStatusSuccess) {
	color: var(--sapContent_LabelColor);
}

.additionalOHNumberDiv.sapMOHNumberDiv .sapMObjectNumber .sapMObjectNumberText {
	font-weight: normal;
}

.additionalOHNumberSeparatorDiv {
	margin-top: 0.375rem;
	margin-right: 0.5rem;
	display: inline-block;
	float: right;
	white-space: nowrap;
	width: 1px;
	background: var(--sapToolbar_SeparatorColor);
	height: 1.375rem;
}

.additionalOHNumberDiv.sapMOHNumberDiv.sapMOHOnlyANumber {
	padding-right: 0.5em;
}

.sapMOH .sapMObjStatusMarker .sapMObjectMarkerIcon .sapUiIcon {
	margin-left: 0.5rem;
	font-size: 0.875rem;
}

.sapMOH .sapMObjStatusMarker .sapUiIcon::before {
	vertical-align: top;
}

/*
 * Do NOT clear if ObjectHeader is placed in a responsive grid layout.
 * The layout will clear after the ObjectHeader by itself. This caused problems
 * on webkit browser on phones (CSN 0120031469 0000034838 2014)
 */

div[class*='sapUiRespGrid'] > .sapMOH > .sapMOHLastDivider {
	clear: none;
}

/* Styles for Responsive Object Header */

.sapMOHR {
	width: 100%;
	height: 100%;
	padding: 1.1rem 3rem 1.5rem 3rem;
	box-sizing: border-box;
}

html.sap-desktop.sapUiMedia-Std-Tablet .sapMOHR,
html.sap-tablet.sapUiMedia-Std-Tablet .sapMOHR,
html .sapUiContainer-Medium .sapMOHR,
html:not(.sapUiMedia-Std-Phone) .sapMSplitContainer:not(.sapMSplitContainerHideMode) .sapMOHR {
	padding-top: 1.1rem;
	padding-left: 2rem;
	padding-right: 2rem;
}

html.sap-desktop.sapUiMedia-Std-Phone .sapMOHR,
html.sap-phone .sapMOHR,
html .sapUiContainer-Narrow .sapMOHR {
	padding: 0.625rem 1rem 1rem 1rem;
}

/* sapMOHRNoBorder this class is set when the ObjectHeader has headerContainer in that case the bottom padding should be 0 */
.sapMOHRNoBorder.sapMOHR,
html.sap-desktop.sapUiMedia-Std-Phone .sapMOHRNoBorder.sapMOHR,
html.sap-phone .sapMOHRNoBorder.sapMOHR,
html .sapUiContainer-Narrow .sapMOHRNoBorder.sapMOHR,
html.sap-desktop.sapUiMedia-Std-Tablet .sapMOHRNoBorder.sapMOHR,
html.sap-tablet.sapUiMedia-Std-Tablet .sapMOHRNoBorder.sapMOHR,
html .sapUiContainer-Medium .sapMOHRNoBorder.sapMOHR,
html:not(.sapUiMedia-Std-Phone) .sapMSplitContainer:not(.sapMSplitContainerHideMode) .sapMOHRNoBorder.sapMOHR {
	padding-bottom: 0;
}

/* Group1 (title, flags and number) */
.sapMOHRTitleNumberDiv {
	display: inline-block;
	max-width: 95rem; /* set max width in order the content not to be stretched too much on a very wide screen */
	width: 100%;
}

.sapMOHRMaxWidth {
	max-width: 95rem; /* set max width in order the content not to be stretched too much on a very wide screen */
}

html.sap-desktop .sapMOHRStatesOneOrThree .sapMOHRTitleNumberDiv {
	width: 70%;
	max-width: none;
}

/* icon */
.sapMOHRIcon {
	position: absolute;
	top: 0.4375rem;
	margin-right: 1rem;
	background-size: cover;
	display: block;
}

.sapMOHRIcon.sapMPointer .sapUiIcon.sapUiIconPointer {

	&:active,
	&:focus {
		outline: none;
	}
}

.sapMOHRIconCircle .sapMImg {
	border-radius: 100%;
}

.sapMOHRIcon .sapMImg {
	max-height: 3rem;
	max-width: 3rem;
}

.sapMOHRHideIcon {
	display: none;
	visibility: hidden;
	margin-left: 0;
}

.sapMOHRTitleIcon > .sapMOHRTitle {
	margin-left: 4rem;
	min-height: 3.375rem;
	overflow: hidden;
}

/* 1) title block */
.sapMOHRTitleDiv {
	display: inline-block;
	box-sizing: border-box;
	vertical-align: top;
	position: relative;
	width: 100%;
}

.sapMOHRTitleTextWrappable {
	white-space: normal;
}

html.sap-phone .sapMOHRTitleTextWrappable,
.sapUiMedia-Std-Phone .sapMOHRTitleTextWrappable,
html .sapUiContainer-Narrow .sapMOHRTitleTextWrappable {
	display: table-cell;
	word-wrap: break-word;
	word-break: break-all;
}

.sapMOHRTitleText {
	margin-top: 0.4rem;
	text-decoration:none;
}

.sapMOHRTitleTextContainer {
	margin: 0.4rem 0.5rem 0 0;
	display: inline-block;
	position: relative;
}

.sapMOHR .sapMObjStatusMarker {
	white-space: nowrap;
}

.sapMOHR .sapMObjStatusMarker .sapUiIcon {
	display: inline;
	margin-left: 0.5rem;
}

.sapMOHR .sapMObjStatusMarker .sapMObjectMarker,
.sapMOHR .sapMObjStatusMarker .sapMObjectMarker .sapMText {
	display: inline;
}

.sapMOHR .sapMObjStatusMarker .sapUiIcon::before {
	vertical-align: inherit;
}

.sapMOHR .sapMObjStatusMarker.sapMObjStatusMarkerOpposite .sapUiIcon {
	margin-right: 0.5rem;
	margin-left: 0;
}

/* overwrite transform for the markers favorite and flag icon */
html[dir=rtl] .sapMObjStatusMarker > .sapUiIcon.sapUiIconMirrorInRTL {
	transform: none;
}

.sapMOHRIntro {
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
	margin-top: 0.5rem;
	line-height: normal;
}

.sapMOHRIntroMargin {
	margin-top: 0;
}

html.sap-desktop.sapUiMedia-Std-Phone .sapMOHRIntroMargin,
html.sap-phone .sapMOHRIntroMargin,
html .sapUiContainer-Narrow .sapMOHRIntroMargin {
	margin-top: -0.3rem;
}

.sapMOHRIntroActive > a:active,
html.sap-desktop .sapMOHRIntroActive > a:hover,
.sapMOHRTitleActive .sapMOHRTitleText:active,
html.sap-desktop .sapMOHRTitleActive .sapMOHRTitleText:hover {
	text-decoration: underline;
}

.sapMOHRTitleArrow {
	display: inline-block;
	vertical-align: top;
	min-height: 2.56rem;
}

.sapMOHRTitleArrow .sapUiIcon {
	margin-top: 1px;
	vertical-align: top;
	position: absolute;
	display: inline-block;
	padding: 0.4rem 0.5rem 0.4rem 0.5rem;
}

html.sap-desktop.sapUiMedia-Std-Phone .sapMOHRTitleArrow .sapUiIcon,
html.sap-phone .sapMOHRTitleArrow .sapUiIcon,
html .sapUiContainer-Narrow .sapMOHRTitleArrow .sapUiIcon {
	padding: 0.3rem 0.4rem 0.3rem 0.4rem;
}

.sapMOHRTitleFollowArrow {
	margin-right: 2.5rem;
	padding-right: 0;
}

/* 3) states block */
.sapMOHRStates {
	display: inline-block;
	box-sizing: border-box;
	vertical-align: top;
	padding-top: 1rem;
	width: 100%;
	max-width: 95rem; /* set max width in order the content not to be stretched too much on a very wide screen */
}

html.sap-desktop .sapMOHRStatesOneOrThree > .sapMOHRStates {
	width: 30%;
	box-sizing: border-box;
	padding: 0.85rem 0 0 1rem;
	text-align: end;
}

html.sap-desktop .sapMOHRStatesOneOrThree > .sapMOHRStates .sapMObjectAttributeDiv > span[class*='sapMObjectAttribute'] {
	float: none;
}

.sapMOHR .sapMObjectNumber {
	display: inline-block;
	padding: 0.4rem 0 0 1rem;
}

.sapMOHR .sapMObjectNumberUnit {
	padding-top: 0.125rem;
}

/* Android fix */
html[data-sap-ui-os^='Android'] {

	.sapMOHR .sapMObjectNumberUnit {
		padding-top: 0.25rem;
	}
}

html.sap-desktop .sapMOHR .sapMObjectNumber {
	width: 30%;
	box-sizing: border-box;
}

html.sap-desktop.sapUiMedia-Std-Tablet .sapMOHR .sapMObjectNumber,
html.sap-tablet .sapMOHR .sapMObjectNumber,
html .sapUiContainer-Medium .sapMOHR .sapMObjectNumber {
	width: 35%;
	box-sizing: border-box;
}

html.sap-desktop.sapUiMedia-Std-Phone .sapMOHR .sapMObjectNumber,
html.sap-phone .sapMOHR .sapMObjectNumber,
html .sapUiContainer-Narrow .sapMOHR .sapMObjectNumber {
	box-sizing: border-box;
	float: right;
}

html.sap-desktop.sapUiMedia-Std-Phone .sapMOHR .sapMObjectNumber.sapMObjectNumberBelowTitle,
html.sap-phone .sapMOHR .sapMObjectNumber.sapMObjectNumberBelowTitle,
html .sapUiContainer-Narrow .sapMOHR .sapMObjectNumber.sapMObjectNumberBelowTitle {
	width: 100%;
	padding: 0.5rem 0 0 0;
}

html.sap-desktop.sapUiMedia-Std-Phone .sapMOHRNumberWrap,
html.sap-phone .sapMOHRNumberWrap,
html .sapUiContainer-Narrow .sapMOHRNumberWrap {
	width: 100%;
	text-align: left;
	margin-left: 0;
}

html.sap-desktop .sapMOHRTitleDiv:not(.sapMOHRTitleDivFull),
html.sap-desktop .sapMOHRStatesOneOrThree > .sapMOHRTitleNumberDiv > .sapMOHRTitleDiv:not(.sapMOHRTitleDivFull) {
	width: 70%;
}

html.sap-desktop.sapUiMedia-Std-Tablet .sapMOHRTitleDiv:not(.sapMOHRTitleDivFull),
html.sap-desktop.sapUiMedia-Std-Tablet .sapMOHRStatesOneOrThree > .sapMOHRTitleNumberDiv > .sapMOHRTitleDiv:not(.sapMOHRTitleDivFull),
html.sap-tablet .sapMOHRTitleDiv:not(.sapMOHRTitleDivFull),
html.sap-tablet .sapMOHRStatesOneOrThree > .sapMOHRTitleNumberDiv > .sapMOHRTitleDiv:not(.sapMOHRTitleDivFull),
html .sapUiContainer-Medium .sapMOHRTitleDiv:not(.sapMOHRTitleDivFull),
html .sapUiContainer-Medium .sapMOHRStatesOneOrThree > .sapMOHRTitleNumberDiv > .sapMOHRTitleDiv:not(.sapMOHRTitleDivFull) {
	width: 65%;
}

html.sap-desktop.sapUiMedia-Std-Phone .sapMOHRTitleDiv:not(.sapMOHRTitleDivFull),
html.sap-phone .sapMOHRTitleDiv:not(.sapMOHRTitleDivFull),
html .sapUiContainer-Narrow .sapMOHRTitleDiv:not(.sapMOHRTitleDivFull) {
	width: 60%;
}

html.sap-desktop.sapUiMedia-Std-Phone .sapMOHRTitleDivFull.sapMOHRTitleDiv,
html.sap-phone .sapMOHRTitleDivFull.sapMOHRTitleDiv,
html .sapUiContainer-Narrow .sapMOHRTitleDivFull.sapMOHRTitleDiv,
.sapMOHRTitleDivFull.sapMOHRTitleDiv {
	width: 100%;
}

.sapMOHRStatesCont1,
.sapMOHRStatesCont2,
.sapMOHRStatesCont3,
.sapMOHRStatesCont4 {
	float: left;
}

.sapMOHROneCols {
	width: 100%;
}

.sapMOHRTwoCols {
	width: 50%;
	box-sizing: border-box;
}

.sapMOHRTwoCols.sapMOHRStatesCont1 {
	padding-right: 0.5rem;
}

.sapMOHRTwoCols.sapMOHRStatesCont2 {
	text-align: end;
	padding-left: 0.5rem;
}

.sapMOHRTwoCols.sapMOHRStatesCont2 .sapMOHRAttr .sapMObjectAttributeDiv > span[class*='sapMObjectAttribute'] {
	float: none;
}

.sapMOHRThreeCols {
	width: 33%;
	box-sizing: border-box;
}

.sapMOHRThreeCols.sapMOHRStatesCont1 {
	padding-right: 0.66rem;
}

.sapMOHRThreeCols.sapMOHRStatesCont3 {
	padding-left: 0.66rem;
}

.sapMOHRThreeCols.sapMOHRStatesCont2 {
	padding-left: 0.33rem;
	padding-right: 0.33rem;
}

.sapMOHRFourCols {
	width: 25%;
	box-sizing: border-box;
}

.sapMOHRFourCols.sapMOHRStatesCont1 {
	padding-right: 0.75rem;
}

.sapMOHRFourCols.sapMOHRStatesCont4 {
	padding-left: 0.75rem;
}

.sapMOHRFourCols.sapMOHRStatesCont2 {
	padding-right: 0.5rem;
	padding-left: 0.25rem;
}

.sapMOHRFourCols.sapMOHRStatesCont3 {
	padding-left: 0.5rem;
	padding-right: 0.25rem;
}

div[class*='sapMOHRStatesCont'] > div:last-child {
	padding-bottom: 0;
}

/* styles for the IconTabBar when inside ObjectHeader */

.sapMOHR .sapMITH {
	position: relative;
	margin-left: -3rem;
	margin-right: -3rem;
}

html.sap-tablet.sapUiMedia-Std-Tablet .sapMOHRTabs .sapMITH,
html.sap-desktop.sapUiMedia-Std-Tablet .sapMOHRTabs .sapMITH,
html .sapUiContainer-Medium .sapMOHRTabs .sapMITH,
html:not(.sapUiMedia-Std-Phone) .sapMSplitContainer:not(.sapMSplitContainerHideMode) .sapMOHRTabs .sapMITH {
	margin-left: -2rem;
	margin-right: -2rem;
	padding: 0 2rem;
}

html.sap-tablet.sapUiMedia-Std-Tablet .sapUi-Std-PaddingXL .sapMOHRTabs .sapMITH,
html.sap-desktop.sapUiMedia-Std-Tablet .sapUi-Std-PaddingXL .sapMOHRTabs .sapMITH,
html .sapUiContainer-Medium .sapUi-Std-PaddingXL .sapMOHRTabs .sapMITH,
html:not(.sapUiMedia-Std-Phone) .sapMSplitContainer:not(.sapMSplitContainerHideMode) .sapUi-Std-PaddingXL .sapMOHRTabs .sapMITH {
	margin-left: -3rem;
	margin-right: -3rem;
	padding: 0 3rem;
}

html.sap-phone .sapMOHRTabs .sapMITH {
	margin-left: -1.5rem;
	margin-right: -1.5rem;
}

html.sap-desktop.sapUiMedia-Std-Phone .sapMOHRTabs .sapMITH,
html .sapUiContainer-Narrow .sapMOHRTabs .sapMITH {
	margin-left: -1rem;
	margin-right: -1rem;
	padding: 0 1rem;
}

.sapUiMedia-Std-Desktop {

	&.sapUiMedia-StdExt-LargeDesktop {

		.sapUiResponsiveContentPadding .sapMOHR {

			.sapMITH {
				margin-left: -3rem;
				margin-right: -3rem;
			}
		}

		.sapFFCLColumnBegin.sapUiContainer-Narrow,
		.sapFFCLColumnMid.sapUiContainer-Narrow,
		.sapFFCLColumnEnd.sapUiContainer-Narrow {

			.sapUiResponsiveContentPadding .sapMOHR {

				.sapMITH {
					margin-left: -1rem;
					margin-right: -1rem;
				}
			}
		}

		.sapFFCLColumnBegin.sapUiContainer-Medium,
		.sapFFCLColumnMid.sapUiContainer-Medium,
		.sapFFCLColumnEnd.sapUiContainer-Medium {

			.sapUiResponsiveContentPadding .sapMOHR {

				.sapMITH {
					margin-left: -2rem;
					margin-right: -2rem;
				}
			}
		}
	}
}

/* end styles for the IconTabBar when inside ObjectHeader */

.sapMOHR .sapMPI {
	margin: 0;
}

.sapMOHTitle.sapMOHTitleActive > h6 .sapMText,
.sapMOHTitle.sapMOHTitleActive > h5 .sapMText,
.sapMOHTitle.sapMOHTitleActive > h4 .sapMText,
.sapMOHTitle.sapMOHTitleActive > h3 .sapMText,
.sapMOHTitle.sapMOHTitleActive > h2 .sapMText,
.sapMOHTitle.sapMOHTitleActive > h1 .sapMText {
	color: var(--sapLinkColor);
	cursor: pointer;
}

.sapMOHTitle h6 .sapMText,
.sapMOHTitle h5 .sapMText,
.sapMOHTitle h4 .sapMText,
.sapMOHTitle h3 .sapMText,
.sapMOHTitle h2 .sapMText,
.sapMOHTitle h1 .sapMText {
	color: var(--sapGroup_TitleTextColor);
}

html.sap-phone .sapMOHTitle h1 > .sapMText {
	font-size: var(--sapFontHeader4Size);
}

.sapMOHNumberDiv .sapMObjectNumber {
	margin: 0;
	padding-bottom: 0.125rem;
	line-height: initial;
}

.sapMOHNumberDiv .sapMObjectNumberText {
	font-family: var(--sapFontHeaderFamily);
	font-size: 1.375rem;
	font-weight: @sapUiFontHeaderWeight;
	white-space: nowrap;
	line-height: initial;
}

.sapMOHNumberDiv .sapMObjectNumberUnit {
	padding: 0;
	display: block;
}

.sapMOHRTitleNumberDiv .sapMObjectNumber {
	padding-top: 0.5625rem;
	line-height: initial;

	.sapMObjectNumberText {
		vertical-align: top;
	}
}

.sapMOHC .sapMOHNumberDiv .sapMObjectNumberText {
	font-family: var(--sapFontHeaderFamily);
	font-size: 1.375rem;
}

.sapMOH .sapMObjectAttributeActive {
	color: var(--sapLinkColor);
}

.sapMOHNumberStateNone {
	color: var(--sapGroup_TitleTextColor);
}

.sapMOHNumberStateError {
	color: var(--sapNegativeTextColor);
}

.sapMOHNumberStateWarning {
	color: var(--sapCriticalTextColor);
}

.sapMOHNumberStateSuccess {
	color: var(--sapPositiveTextColor);
}

.sapMOHIcon .sapUiIcon {
	color: var(--sapContent_IconColor);
}

.sapMOHIntro.sapMOHIntroActive >.sapMLnk:active {
	background: none;
}

html.sap-desktop .sapMOHTitleArrow .sapUiIcon:hover {
	background-color: var(--sapButton_Lite_Hover_Background);
}

.sapMOHC .sapMOHTitleArrow .sapUiIcon:active,
.sapMOH .sapMOHTitleArrow .sapUiIcon:active {
	background-color: var(--sapButton_Lite_Active_Background);
	color: var(--sapContent_ContrastTextColor);
}

/* styles for responsive object header */
.sapMOHRTitle h1 {
	font-family: var(--sapFontHeaderFamily);
	font-size: var(--sapFontHeader2Size);
	font-weight: @sapUiFontHeaderWeight;
	color: var(--sapGroup_TitleTextColor);
}

.sapMOHRTitle h6,
.sapMOHRTitle h5,
.sapMOHRTitle h4,
.sapMOHRTitle h3,
.sapMOHRTitle h2 {
	font-family: var(--sapFontHeaderFamily);
	font-weight: @sapUiFontHeaderWeight;
	color: var(--sapGroup_TitleTextColor);
}

html.sap-phone .sapMOHRTitle h1,
html.sap-desktop.sapUiMedia-Std-Phone .sapMOHRTitle h1,
html .sapUiContainer-Narrow .sapMOHRTitle h1 {
	font-size: var(--sapFontHeader4Size);
}

.sapMOHRIcon .sapUiIcon {
	font-family: var(--sapContent_IconFontFamily);
	font-size: 2.5rem;
	font-weight: normal;
	color: var(--sapContent_IconColor);
}

.sapMOHRTitleArrow .sapUiIcon {
	color: var(--sapContent_IconColor);
}

html.sap-desktop .sapMOHRTitleArrow .sapUiIcon:hover {
	background-color: var(--sapButton_Lite_Hover_Background);
	border-color: var(--sapButton_Lite_Hover_BorderColor);
}

.sapMOHRTitleIcon .sapMOHRTitleFollowArrow .sapMOHRTitleArrow .sapUiIcon:active {
	background-color: var(--sapButton_Lite_Active_Background);
	color: var(--sapButton_Active_TextColor);
}

.sapMOHR .sapMObjStatusMarker{
	color: var(--sapContent_MarkerIconColor);
}

.sapMOHRIntro {
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	font-weight: normal;
	color: var(--sapContent_LabelColor);
}

.sapMOHRTitleActive .sapMOHRTitleText {
	cursor: pointer;
	color: var(--sapLinkColor);
	text-decoration: none;
}

.sapMOHRIntro.sapMOHRIntroActive >.sapMLnk:active {
	background: none;
}

.sapMOHR .sapMObjectNumberText {
	font-family: var(--sapFontHeaderFamily);
	font-size: 1.375rem;
	font-weight: @sapUiFontHeaderWeight;
}

.sapMOHR .sapMObjectNumberUnit {
	font-family: var(--sapFontFamily);
	font-weight: normal;
}

.sapMOHR .sapMOHRAttr,
.sapMOHR .sapMOHRStatus,
.sapMOHR .sapMObjStatus {
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	font-weight: normal;
	color: var(--sapContent_LabelColor);
}

.sapMOHR .sapMOHRAttr,
.sapMOHR .sapMOHRStatus {
	padding-bottom: 0.5rem;
}

.sapMOHR .sapMOHRStatus {

	.sapMObjStatusText {
		word-break: normal;
	}
}

.sapMOHRStatus .sapMObjStatusNone .sapMObjStatusText {
	color: var(--sapNeutralTextColor);
}

.sapMOHRStatus .sapMObjStatusSuccess .sapMObjStatusText {
	color: var(--sapPositiveTextColor);
}

.sapMOHRStatus .sapMObjStatusError .sapMObjStatusText {
	color: var(--sapNegativeTextColor);
}

.sapMOHRStatus .sapMObjStatusWarning .sapMObjStatusText {
	color: var(--sapCriticalTextColor);
}
/* END "ObjectHeader.less" */

/* START "ObjectIdentifier.less" */
/* ======================================= */
/* CSS for control sap.m/ObjectIdentifier  */
/* Base theme                              */
/* ======================================= */

.sapMObjectIdentifierIcons {
	overflow: hidden;
	text-align: right;
	font-size: 1rem;
	color: @sapUiContentNonInteractiveIconColor;
	display: inline-block;
	vertical-align: middle;
}

.sapMObjectIdentifierIconSpan {
	padding-top: 0;
	padding-bottom: 0;
	padding-left: 0.5rem;
}

.sapMObjectIdentifierTitle .sapMLnk .sapMLnkText,
.sapMObjectIdentifierTitle .sapMText {
	overflow: visible;
	font-weight: bold;
	font-size: 1rem;
	display: inline;
	word-wrap: break-word;
	word-break: break-word;
	white-space: normal;
}

.sapMObjectIdentifierTitle .sapMLnk.sapMLnkLargeReactiveArea {
	display: inline-block;
}

html[data-sap-ui-browser^='ff'] {

	.sapMObjectIdentifierTitle .sapMLnk .sapMLnkText,
	.sapMObjectIdentifierTitle .sapMText {
		display: inline-block;
	}
}

.sapMObjectIdentifierTitle .sapMText {
	color: @sapUiGroupTitleTextColor;
}

.sapMObjectIdentifierTitle .sapMLnk,
.sapMOITitleActive .sapMLnk {
	color: @sapUiLink;
}

.sapMObjectIdentifierText .sapMText,
.sapMObjectIdentifierText,
.sapMObjectIdentifier {
	color: @sapUiContentLabelColor;
}

.sapMObjectIdentifierText {
	font-size: @sapMFontMediumSize;
	padding-top: 0;
}

.sapMObjectIdentifierTextBellow{
	margin-top: 0.5rem;
}

.sapMLIBActive .sapMObjectIdentifier {
	color: @sapUiListActiveTextColor;
}

/* ObjectIdentifier inside a table */
.sapMListTbl .sapMObjectIdentifierTitle .sapMLnk .sapMLnkText,
.sapMListTbl .sapMObjectIdentifierTitle .sapMText {
	font-size: @sapMFontMediumSize;
	padding-top: 0;
}

.sapMListTbl .sapMObjectIdentifierTitle .sapMText {
	font-weight: bold;
	color: @sapUiListTextColor;
}

.sapMListTbl .sapMObjectIdentifierText .sapMText {
	line-height: 1.4;
	padding-top: 0;
}

.sapMObjectIdentifierTitle {
	display: inline;
}

.sapMObjectIdentifier .sapMEmptyIndicator.sapMEmptyIndicatorAuto {
	display: none;
}

.sapMShowEmpty-CTX .sapMObjectIdentifier .sapMEmptyIndicator.sapMEmptyIndicatorAuto {
	display: inline-block;
}
/* END "ObjectIdentifier.less" */

/* START "ObjectListItem.less" */
/* ===================================== */
/* CSS for control sap.m/ObjectListItem  */
/* Base theme                            */
/* ===================================== */

.sapMObjLItem {
	box-sizing: border-box;
	vertical-align: baseline;
	/* set this otherwise if one of the parents has "nowrap" style this
	affects the whole arrangement of the ObjectListItem */
	white-space: normal;
}

.sapMLIB.sapMObjLItem {
	padding: 1rem;
}

.sapMObjLItem .sapMLIBImgNav:active,
.sapMObjLItem .sapMLIBIconDet:active,
.sapMObjLItem .sapMLIBIconDel:active {
	border-bottom: none;
}

.sapMObjLTopRow {
	float: left;
	width: 100%;
}

.sapMObjLIntro {
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
	font-family: var(--sapFontFamily);
	color: var(--sapContent_LabelColor);
	font-size: @sapMFontMediumSize;
	padding-bottom: 0.75rem;
}

.sapMText.sapMObjLTitle {
	vertical-align: top;
	word-break: break-word;
	color: var(--sapGroup_TitleTextColor);
	font-size: var(--sapFontHeader4Size);
	font-weight: @sapUiFontHeaderWeight;
	line-height: 1.5rem;
}

html.sap-phone .sapMText.sapMObjLTitle,
.sapUiMedia-Std-Phone .sapMText.sapMObjLTitle,
html .sapUiContainer-Narrow .sapMText.sapMObjLTitle {
	word-break: break-all;
}

/*Object header title must break words, otherwise it may be to wide if used within
  a layout control with fixed width. */
.sapMOHTitle:not(.sapMOHRTitleNoWordBreak) > .sapMText {
	/* to break all words even browser does not know the width */
	word-break: break-word;
	hyphens: auto;
}

.sapMObjLNumberDiv {
	float: right;
	padding-left: 1rem;

	.sapMObjectNumber {
		line-height: initial;
	}
}

.sapMObjLNumberDiv .sapMObjectNumber.sapMObjectNumberStatusNone {
	color: var(--sapGroup_TitleTextColor);
}

.sapMObjLNumberDiv .sapMObjectNumber .sapMObjectNumberText {
	font-family: var(--sapFontHeaderFamily);
	font-size: 1.375rem;
	line-height: 1.375rem;
	font-weight: @sapUiFontHeaderWeight;
	white-space: nowrap;
}

.sapMObjLNumberDiv .sapMObjectNumber .sapMObjectNumberUnit {
	padding: 0;
	display: block;
}

.sapMObjLBottomRow {
	margin-top: 0.25rem;
}

.sapMObjLAttrRow {
	padding-top: 0.313rem;
}

.sapMObjLAttrDiv {
	box-sizing: border-box;
	display: inline-block;
	width: 50%;
	text-align: left;
	vertical-align: top;
}

html[dir=rtl] .sapMObjLAttrDiv {
	text-align: right;
}

.sapMObjLAttrDiv .sapMObjectAttributeDiv {
	text-overflow: ellipsis;
	white-space: nowrap;
	overflow: hidden;
	width: 100%;

	&.sapMObjectAttributeActive {

		.sapMObjectAttributeTitle,
		.sapMObjectAttributeText {
			text-overflow: ellipsis;
			white-space: nowrap;
			overflow: hidden;
			max-width: 50%;
		}

		&.sapMObjectAttributeTextOnly {

			.sapMObjectAttributeTitle,
			.sapMObjectAttributeText {
				max-width: 100%;
			}
		}

	}
}

/* First and second attributes get one line with ellipsis */
.sapMObjLAttrRow:nth-child(1) > .sapMObjLAttrDiv .sapMObjectAttributeDiv,
.sapMObjLAttrRow:nth-child(2) > .sapMObjLAttrDiv .sapMObjectAttributeDiv {
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
	width: 100%;
}

.sapMObjLAttrDiv .sapMObjectAttributeDiv {
	display: inline-block;
}

.sapMObjLStatusDiv {
	box-sizing: border-box;
	display: inline-block;
	width: 50%;
	text-align: right;
	vertical-align: top;
	padding-left: 0.5rem;
}

html[dir=rtl] .sapMObjLStatusDiv {
	text-align: left;
	padding-left: 0;
	padding-right: 0.5rem;
}

.sapMObjLAttrRow .sapMObjStatus {
	overflow: visible;
	white-space: nowrap;
	text-overflow: ellipsis;
	width: 100%;

	.sapMObjStatusText {
		word-break: normal;
	}
}

/* invertedStatus in ObjectListItem */
.sapMObjLAttrRow .sapMObjStatusInverted {
	min-height: 1rem;
	line-height: 0.75rem;
}

.sapMObjLBottomRow .sapMObjStatusIcon {
	font-size: 1rem;
}

.sapMObjLItem > .sapMLIBContent {
	flex: 1 1 auto;
}

.sapMObjLListModeDiv {
	display: flex;
	flex-direction: row;
	align-items: center;
}

/* overwrite transform for the markers favorite and flag icon */
html[dir=rtl] .sapMObjStatusMarker > .sapUiIcon.sapUiIconMirrorInRTL {
	transform: none;
}

.sapMObjLIconDiv > .sapMImg {
	max-height: 3rem;
	max-width: 3rem;
}

.sapMObjLItem .sapMObjStatusMarker .sapMObjectMarkerText,
.sapMObjLItem .sapMObjStatusMarker .sapMObjectMarkerIcon {
	margin-left: 0.5rem;
}

.sapMObjLItem .sapMObjStatusMarker .sapMText,
.sapMObjLItem .sapMObjStatusMarker .sapUiIcon {
	font-size: 0.875rem;
}

.sapMObjLItem .sapMObjStatusMarker .sapUiIcon::before {
	vertical-align: top;
}

.sapMObjLIcon {
	margin-top: 2px;
	color: var(--sapContent_IconColor);
}

.sapMLIBActive .sapMObjLIcon {
	color: var(--sapContent_ContrastIconColor);
}

.sapMLIBActive .sapMObjLNumberDiv .sapMObjectNumber,
.sapMLIBActive .sapMObjStatusMarker .sapMObjectMarker .sapUiIcon {
	color: var(--sapList_Active_TextColor);
}

.sapMObjLIconDiv {
	float: left;
	margin-right: 0.75rem;
	width: 3rem;
	height: 3rem;
	text-align: center;
	box-sizing: border-box;
}

.sapMObjLItem .sapMObjLAttrDiv {
	color: var(--sapContent_LabelColor);
}

.sapMObjLAdditAttribs {
	color: var(--sapContent_LabelColor);
}

.sapMObjLAttrDiv.sapMObjAttrWithMarker {
	padding-top: 0.1875rem
}
/* END "ObjectListItem.less" */

/* START "ObjectMarker.less" */
/* =================================== */
/* CSS for control sap.m/ObjectMarker  */
/* Base theme                          */
/* =================================== */

.sapMObjectMarker {
	display: inline-block;

	.sapUiIcon {
		font-size: 1rem;
		color: var(--sapContent_MarkerIconColor);

		&::before {
			vertical-align: middle;
		}

		&:not([data-sap-ui-icon-content]) {
			display: none;
		}
	}

	.sapMLnk {
		overflow: visible;
		padding-bottom: 1px;
	}

	.sapMText {
		color: var(--sapContent_LabelColor);
	}

	&.sapMObjectMarkerIcon.sapMObjectMarkerText .sapUiIcon {
		margin-right: 0.25rem;
	}
}

.sapUiTableCellInner .sapMObjectMarkerIcon,
.sapMListTbl .sapMObjectMarkerIcon {

	.sapMText {
		padding-top: 0; /* Icon in table should not be padded */
	}

	.sapUiIcon::before {
		vertical-align: inherit; /* Icon in table should not be vertically aligned */
	}
}

.sapMListTbl .sapMObjectMarker .sapMText {
	color: var(--sapContent_LabelColor);
}

.sapMListTbl .sapMObjectMarker {
	word-break: break-all;
}

.sapUiTableCellInner .sapMObjectMarkerIcon .sapUiIcon {
	margin-top: 0.0625rem;
}
/* END "ObjectMarker.less" */

/* START "ObjectNumber.less" */
/* =================================== */
/* CSS for control sap.m/ObjectNumber  */
/* Base theme                          */
/* =================================== */

@sapMONInformativeElement: var(--sapInformativeElementColor);

.sapMObjectNumberLarge .sapMObjectNumberText,
.sapMObjectNumberLarge .sapMObjectNumberUnit {
	font-size: 1.5rem;
}

.sapMObjectNumber {
	display: block;
	white-space: normal;
	max-width: 100%;
	text-overflow: ellipsis;
	word-wrap: normal;
	overflow: visible;
	min-height: 1rem;
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;

	&.sapMObjectNumberLarge {
		vertical-align: middle;
		height: 2rem;

		&.sapMObjectNumberInverted .sapMObjectNumberInner {
			padding: 0 0.25rem;
		}

		& .sapMObjectNumberInner {
			height: 2rem;
		}

		.sapMObjectNumberText,
		.sapMObjectNumberUnit {
			line-height: 2rem;
			vertical-align: middle;
			height: 2rem;
		}
	}

	&.sapMObjectNumberLongText {

		.sapMObjectNumberInner {
			height: initial;
		}

		.sapMObjectNumberText {
			display: inline;
			word-break: break-word;
			text-overflow: unset;
			white-space: break-spaces;
			-webkit-box-orient: unset;
		}
	}
}

.sapMObjectNumberText {
	display: inline-block;
	white-space: pre;
	overflow: hidden;
	text-overflow: ellipsis;
	max-width: 100%;
	text-decoration: none;
	min-height: 1rem;
	vertical-align: middle;
	line-height: 1rem;
	font-family: var(--sapFontFamily);
	text-shadow: var(--sapContent_TextShadow);
}

.sapMObjectNumberEmph .sapMObjectNumberText {
	font-weight: bold;
}

.sapMObjectNumberUnit {
	display: inline-block;
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
	max-width: 100%;
	text-decoration: none;
	height: 1rem;
	line-height: 1rem;
	vertical-align: middle;
	font-family: var(--sapFontLightFamily);
 	text-shadow: var(--sapContent_TextShadow);
}

/* Android fix */
html[data-sap-ui-os^='Android'] {

	.sapMObjectNumberUnit {
		padding-top: 0;
	}
}

.sapUiForm .sapMObjectNumberUnit,
.sapUiForm .sapMObjectNumberText {
	vertical-align: top;
}

.sapUiForm.sapUiFormEdit .sapMObjectNumber {
	margin-top: 0.875rem;

	&.sapMObjectNumberLarge {
		line-height: initial;
		margin-top: 0.375rem;
	}
}

.sapUiSizeCompact .sapUiForm.sapUiFormEdit .sapMObjectNumber {
	margin-top: 0.5rem;

	&.sapMObjectNumberLarge {
		line-height: initial;
		margin-top: 0;
	}
}

.sapMLIBActive {

	.sapMObjectNumberText,
	.sapMObjectNumberUnit {
		color: var(--sapList_Active_TextColor);
		text-shadow: none;
	}
}

.sapMObjectNumberStatusNone {
	color: var(--sapNeutralTextColor);
}

.sapMObjectNumberStatusError {
	color: var(--sapNegativeTextColor);
}

.sapMObjectNumberStatusWarning {
	color: var(--sapCriticalTextColor);
}

.sapMObjectNumberStatusSuccess {
	color: var(--sapPositiveTextColor);
}

.sapMObjectNumberStatusInformation {
	color: var(--sapInformativeTextColor);
}

/* focus */

.sapMObjectNumberActive .sapMObjectNumberInner {
	cursor: pointer;
}

.sapMObjectNumberInner {
	position: relative;
	box-sizing: border-box;
	display: inline-block;
	min-height: 1rem;
	line-height: 0.875rem;
	vertical-align: text-top;
}

html.sap-desktop .sapMObjectNumberActive:focus {
	outline: none;

	.sapMObjectNumberInner::after {
		pointer-events: none;
		display: block;
		content: '';
		position: absolute;
		top: -1px;
		left: 1px;
		bottom: 0;
		right: 1px;
		height: 1.125rem;
		outline: 1px dotted var(--sapContent_FocusColor);
	}

	&.sapMObjectNumberInverted .sapMObjectNumberInner::after {
		top: -1px;
		left: 1px;
		bottom: -1px;
		right: 1px;
		height: 1.125rem;
		outline: 1px dotted var(--sapContent_FocusColor);
	}

	&.sapMObjectNumberLarge {

		.sapMObjectNumberInner::after {
			top: 0;
			left: 1px;
			bottom: 0;
			right: 1px;
			height: 2rem;
		}

		&.sapMObjectNumberInverted .sapMObjectNumberInner::after {
			top: 3px;
			left: 3px;
			bottom: 3px;
			right: 3px;
			height: 1.625rem;
			outline: 1px dotted var(--sapContent_ContrastFocusColor);
		}
	}
}

.sapMObjectNumberActive:not(.sapMObjectNumberInverted) .sapMObjectNumberInner {

	* {
		text-decoration: underline;
	}
}

.sapMObjectNumberActive:not(.sapMObjectNumberInverted) {

	&.sapMObjectNumberPressed .sapMObjectNumberInner,
	& .sapMObjectNumberInner:hover,
	& .sapMObjectNumberInner:active {

		* {
			text-decoration: none;
		}
	}
}

/* inverted classes */

.sapMObjectNumberInverted {
	vertical-align: initial;

	.sapMObjectNumberInner {
		height: 1rem;
		width: auto;
		min-width: 1.25rem;
		padding: 0 0.25rem;
		border-radius: var(--sapElement_BorderCornerRadius);
		line-height: 0.75rem;
	}

	.sapMObjectNumberInner::after {
		top: -1px;
		left: 1px;
		bottom: 1px;
		right: 1px;
		height: 1.25rem;
		outline: 1px dotted var(--sapContent_FocusColor);
	}

	&:not(.sapMObjectNumberLarge) {
		font-weight: bold;
	}

	.sapMObjectNumberText,
	.sapMObjectNumberUnit {
		font-family: var(--sapFontFamily);
		font-size: var(--sapFontSmallSize);
		text-shadow: none;
	}

	&.sapMObjectNumberLarge {
		height: 2rem;

		.sapMObjectNumberText,
		.sapMObjectNumberUnit {
			font-size: 1.25rem;
			line-height: 2rem;
		}
	}
}

.objectNumberStatusMixIn(@class, @innerBackgroundColor, @innerColor, @activeInnerBackgroundColor, @activeInnerColor, @pressedInnerBackgroundColor) {

	.sapMObjectNumberInverted.@{class} {

		.sapMObjectNumberInner {
			background-color: @innerBackgroundColor;
			color: @innerColor;
		}

		&.sapMObjectNumberActive {

			.sapMObjectNumberInner:hover {
				background-color: @activeInnerBackgroundColor;
			}

			.sapMObjectNumberInner:active,
			&.sapMObjectNumberPressed .sapMObjectNumberInner {
				background-color: @pressedInnerBackgroundColor;
				color: @activeInnerColor;
			}
		}
	}
}

.sapMObjectNumber .sapMEmptyIndicator.sapMEmptyIndicatorAuto {
	display: none;
}

.sapMShowEmpty-CTX .sapMObjectNumber .sapMEmptyIndicator.sapMEmptyIndicatorAuto {
	display: inline-block;
}

.objectNumberStatusMixIn(sapMObjectNumberStatusError,
	@sapUiButtonNegativeBackground,
	@sapUiButtonNegativeTextColor,
	@sapUiButtonNegativeHoverBackground,
	@sapUiButtonNegativeHoverTextColor,
	@sapUiButtonNegativeActiveBackground
);

.objectNumberStatusMixIn(sapMObjectNumberStatusWarning,
	@sapUiButtonCriticalBackground,
	@sapUiButtonCriticalTextColor,
	@sapUiButtonCriticalHoverBackground,
	@sapUiButtonCriticalHoverTextColor,
	@sapUiButtonCriticalActiveBackground
);

.objectNumberStatusMixIn(sapMObjectNumberStatusSuccess,
	@sapUiButtonSuccessBackground,
	@sapUiButtonSuccessTextColor,
	@sapUiButtonSuccessHoverBackground,
	@sapUiButtonSuccessHoverTextColor,
	@sapUiButtonSuccessActiveBackground
);

.objectNumberStatusMixIn(sapMObjectNumberStatusInformation,
	@sapMONInformativeElement,
	@sapUiButtonInformationTextColor,
	@sapUiButtonInformationHoverBackground,
	@sapUiButtonInformationHoverTextColor,
	@sapUiButtonInformationActiveBackground
);

.objectNumberStatusMixIn(sapMObjectNumberStatusNone,
	@sapUiButtonNeutralBackground,
	@sapUiButtonNeutralTextColor,
	@sapUiButtonNeutralHoverBackground,
	@sapUiButtonNeutralHoverTextColor,
	@sapUiButtonNeutralActiveBackground
);
/* END "ObjectNumber.less" */

/* START "ObjectStatus.less" */
/* =================================== */
/* CSS for control sap.m/ObjectStatus  */
/* Base theme                          */
/* =================================== */

.sapMObjStatus {
	font-size: @sapMFontMediumSize;
	color: var(--sapNeutralTextColor);
	display: inline-block;
	overflow: visible;
	min-height: 1rem;
	vertical-align: top;
	line-height: normal;

	&.sapMObjectStatusLarge {
		font-size: 1rem;
		min-height: 2rem;
	}

	.sapMObjStatusIcon {
		width: 1rem;
		text-align: center;
		height: 1rem;
		line-height: normal;
		display: inline-flex;
		justify-content: center;
		align-items: center;

		& > .sapMImg {
			max-width: 1rem;
			max-height: 1rem;
		}
	}

	.sapMObjStatusTitle,
	.sapMObjStatusText {
		line-height: normal;
		display: inline-block;
		height: inherit;
	}

	&.sapMObjectStatusLongText{

		&.sapMObjectStatusLarge {

			.sapMObjStatusTitle,
			.sapMObjStatusText {
				height: inherit;
			}
		}

		.sapMObjStatusText {
			word-break: break-all;
		}
	}

	.sapMObjStatusTitle::after {
		content: attr(data-colon);
		white-space: pre;
	}

	.sapMEmptyIndicator{
		line-height: normal;
	}
}

.sapMObjStatus:not(.sapMObjStatusInverted) .sapMObjStatusTitle {
	vertical-align: top;
}

.sapMLIBActive {

	.sapMObjStatus:not(.sapMObjStatusInverted) .sapMObjStatusText,
	.sapMObjStatus:not(.sapMObjStatusInverted) .sapMObjStatusIcon {
		text-shadow: none;
	}
}

.sapMObjStatus:not(.sapMObjStatusInverted) .sapMObjStatusLink {
	min-height: 1rem;
	vertical-align: top;
}

.sapUiFormCLElement .sapMObjStatus .sapMObjStatusIcon,
.sapUiFormCLElement .sapMObjStatus .sapMObjStatusTitle,
.sapUiFormCLElement .sapMObjStatus .sapMObjStatusText {
	padding-bottom: 2px;
}

.sapUiHLayout .sapMObjStatus .sapMObjStatusIcon,
.sapUiHLayout .sapMObjStatus .sapMObjStatusTitle,
.sapUiHLayout .sapMObjStatus .sapMObjStatusText {
	padding-bottom: 2px;
}

.sapMObjectStatusLarge {

	.sapMObjStatusTitle,
	.sapMObjStatusText,
	.sapMObjStatusIcon {
		font-size: 1.5rem;
		font-family: var(--sapFontLightFamily);
		height: 2rem;
	}

	.sapMObjStatusIcon {
		width: 2rem;

		& > .sapMImg {
			max-width: 1.5rem;
			max-height: 1.5rem;
		}
	}

	.sapMObjStatusIcon > .sapUiIcon {
		vertical-align: baseline;
	}
}

.sapMObjStatusText {
	word-break: normal;
	overflow-wrap: break-word;
}

.sapMObjStatus > .sapMObjStatusTitle {
	color: var(--sapContent_LabelColor);
}

.sapMObjStatusNone .sapMObjStatusText {
	color: var(--sapNeutralTextColor);
}

.sapMObjStatusError .sapMObjStatusText {
	color: var(--sapNegativeTextColor);
}

.sapMObjStatusWarning .sapMObjStatusText {
	color: var(--sapCriticalTextColor);
}

.sapMObjStatusSuccess .sapMObjStatusText{
	color: var(--sapPositiveTextColor);
}

.sapMObjStatusInformation .sapMObjStatusText {
	color: var(--sapInformativeTextColor);
}

.sapMObjStatusNone .sapMObjStatusIcon {
	color: var(--sapNeutralElementColor);
}

.sapMObjStatusError .sapMObjStatusIcon {
	color: var(--sapNegativeElementColor);
}

.sapMObjStatusWarning .sapMObjStatusIcon {
	color: var(--sapCriticalElementColor);
}

.sapMObjStatusSuccess .sapMObjStatusIcon {
	color: var(--sapPositiveElementColor);
}

.sapMObjStatusInformation .sapMObjStatusIcon {
	color: var(--sapInformativeTextColor);
}

.sapMObjStatusIndication01 .sapMObjStatusText,
.sapMObjStatusIndication01 .sapMObjStatusIcon {
	color: var(--sapIndicationColor_1);
}

.sapMObjStatusIndication02 .sapMObjStatusText,
.sapMObjStatusIndication02 .sapMObjStatusIcon {
	color: var(--sapIndicationColor_2);
}

.sapMObjStatusIndication03 .sapMObjStatusText,
.sapMObjStatusIndication03 .sapMObjStatusIcon {
	color: var(--sapIndicationColor_3);
}

.sapMObjStatusIndication04 .sapMObjStatusText,
.sapMObjStatusIndication04 .sapMObjStatusIcon {
	color: var(--sapIndicationColor_4);
}

.sapMObjStatusIndication05 .sapMObjStatusText,
.sapMObjStatusIndication05 .sapMObjStatusIcon {
	color: var(--sapIndicationColor_5);
}

.sapMObjStatusIndication06 .sapMObjStatusText,
.sapMObjStatusIndication06 .sapMObjStatusIcon {
	color: var(--sapIndicationColor_6);
}

.sapMObjStatusIndication07 .sapMObjStatusText,
.sapMObjStatusIndication07 .sapMObjStatusIcon {
	color: var(--sapIndicationColor_7);
}

.sapMObjStatusIndication08 .sapMObjStatusText,
.sapMObjStatusIndication08 .sapMObjStatusIcon {
	color: var(--sapIndicationColor_8);
}

.sapMObjStatus:not(.sapMObjStatusInverted) .sapMObjStatusText,
.sapMObjStatus:not(.sapMObjStatusInverted) .sapMObjStatusIcon,
.sapMObjStatusInverted .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted .sapMObjStatusLink:active .sapMObjStatusIcon {
	text-shadow: var(--sapContent_TextShadow);
}

/* inverted classes */

.sapMObjStatusInverted .sapMObjStatusText,
.sapMObjStatusInverted .sapMObjStatusIcon {
	text-shadow: var(--sapContent_ContrastTextShadow);
	font-family: var(--sapFontFamily);
	font-size: var(--sapFontSmallSize);
	font-weight: bold;
}

.sapMObjStatusWrapper {
	display: inline-flex;
}

.sapMObjStatusInverted .sapMObjStatusWrapper {
	border-radius: 0.25rem;
}

.sapMListTbl .sapMObjStatusInverted .sapMObjStatusIcon > .sapUiIcon:not(.sapMLIBImgNav):not(.sapMBtnIcon) {
	font-size: var(--sapFontSmallSize);
	vertical-align: top;
}

.sapMObjStatusInverted .sapMObjStatusIcon > .sapUiIcon {
	font-size: var(--sapFontSmallSize);
	vertical-align: top;
}

.sapMObjStatusInverted {

	.sapMObjStatusIcon {
		height: 0.875rem;

		& > .sapUiIcon {
			vertical-align: top;
		}
	}

	&.sapMObjectStatusLarge {
		vertical-align: top;

		.sapMObjStatusText,
		.sapMObjStatusIcon {
			font-family: var(--sapFontLightFamily);
			font-size: 1.25rem;
			font-weight: normal;
			color: var(--sapContent_ContrastTextColor);

			& > .sapUiIcon {
				font-size: 1.25rem;
			}
		}
	}
}

.sapMObjStatusInverted .sapMObjStatusText,
.sapMObjStatusInverted .sapMObjStatusIcon .sapMObjStatusIconOnly {
	padding: 0 0.25rem;
	line-height: 1rem;
}

.sapMObjStatusInverted .sapMObjStatusIcon {
	padding: 0 0 0 0.25rem;
}

/*  First set of inverted indication colors */
.sapMObjStatusInverted.sapMObjStatusIndication01 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_1_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication02 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_2_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication03 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_3_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication04 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_4_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication05 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_5_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication06 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_6_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication07 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_7_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication08 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_8_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication09 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_9_Background);
	border-color: var(--sapIndicationColor_9_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication10 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_10_Background);
	border-color: var(--sapIndicationColor_10_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusError .sapMObjStatusWrapper {
	background-color: var(--sapButton_Negative_Background);
}

.sapMObjStatusInverted.sapMObjStatusError .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusError .sapMObjStatusIcon {
	color: var(--sapButton_Negative_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusWarning .sapMObjStatusWrapper {
	background-color: var(--sapButton_Critical_Background);
}

.sapMObjStatusInverted.sapMObjStatusWarning .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusWarning .sapMObjStatusIcon {
	color: var(--sapButton_Critical_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusSuccess .sapMObjStatusWrapper {
	background-color: var(--sapButton_Success_Background);
}

.sapMObjStatusInverted.sapMObjStatusSuccess .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusSuccess .sapMObjStatusIcon {
	color: var(--sapButton_Success_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusInformation .sapMObjStatusWrapper {
	background-color: var(--sapInformativeElementColor);
}

.sapMObjStatusInverted.sapMObjStatusInformation .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusInformation .sapMObjStatusIcon {
	color: var(--sapButton_Information_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusNone .sapMObjStatusWrapper {
	background-color: var(--sapButton_Neutral_Background);
}

.sapMObjStatusInverted.sapMObjStatusNone .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusNone .sapMObjStatusIcon {
	color: var(--sapButton_Neutral_TextColor);
}

.sapMObjStatusIcon > .sapUiIcon {
	font-size: 1rem;
}

.sapMObjStatusInverted .sapMObjStatusIcon > .sapUiIcon {
	font-size: 0.75rem;
	line-height: 1rem;
}

.sapUiTableCell {

	.sapMObjStatus {
		line-height: inherit;
	}

	.sapMObjStatusIcon {
		line-height: inherit;
		vertical-align: text-bottom;

		& > .sapUiIcon {
			vertical-align: bottom;
		}
	}

	.sapMObjStatusText {
		vertical-align: middle;
	}

	.sapMObjStatusText,
	.sapMObjStatusTitle {
		display: inline;
	}
}

.sapMObjectStatusLarge .sapMObjStatusIcon > .sapUiIcon {
	font-size: 1.5rem;
	line-height: 2rem;
}

.sapMObjStatusInverted .sapMObjectStatusLarge .sapMObjStatusText {
	vertical-align: top;
}

.sapMObjStatusIcon {
	padding-right: .25rem;
}

.sapMObjStatus > .sapMObjStatusTitle {
	padding-right: .25rem;
}

.sapUiTableCell .sapMObjStatusWrapper,
.sapUiTableCell .sapMObjStatus {
	font-size: @sapMFontMediumSize;
	text-overflow: ellipsis;
	white-space: nowrap;
	overflow: hidden;
	display: block;
}

.sapUiTableCell .sapMObjStatus:not(.sapMObjStatusInverted) .sapMObjStatusIcon {
	margin-left: 0.03rem;
}

html[data-sap-ui-browser*='ff'] {

	.sapUiTableCell .sapMObjStatus:not(.sapMObjStatusInverted) .sapMObjStatusIcon > .sapUiIcon {
		padding-bottom: 0.025rem
	}
}

.sapUiTableCell .sapMObjStatusInverted {
	display: inline-flex;
}

.sapUiForm .sapMObjStatusIcon,
.sapUiForm .sapMObjStatusTitle,
.sapUiForm .sapMObjStatusText {
	display: inline-block;
	line-height: normal;
	user-select: text;
}

.sapUiForm .sapMObjStatusInverted .sapMObjStatusIcon,
.sapUiForm .sapMObjStatusInverted .sapMObjStatusText {
	line-height: 1rem;
}

/*
	FIX for 1580155978 and 1680267289.
	Fiori design guidelines say that statuses should be specially styled for forms
	and share looks with other text only controls.
*/
.sapUiForm.sapUiFormEdit .sapMObjStatus {
	padding-top: 0.875rem;
	padding-bottom: 0.875rem;
	line-height: normal;
}

.sapUiSizeCompact .sapUiForm.sapUiFormEdit .sapMObjStatus {
	padding-top: 0.5rem;
	padding-bottom: 0.5rem;
	line-height: normal;
}

/*
	FIX for 2715207
	Fiori design guidelines say that statuses should be specially styled for ObjectPage
	and share looks with other text only controls.
*/
.sapMObjStatusNone:not(.sapMObjStatusInverted) {

	.sapUxAPObjectPageContainer &,
	.sapUxAPObjectPageHeaderContentItem &,
	&.sapUxAPObjectPageHeaderContentItem,
	.sapUxAPObjectPageHeaderDetails .sapFDynamicPageHeaderContent & {

		.sapMObjStatusText,
		.sapMObjStatusIcon {
			color: var(--sapGroup_TitleTextColor);
		}
	}
}

.sapUxAPObjectPageHeaderContent .sapUxAPObjectPageHeaderContentItem {

	.sapMObjStatus {
		vertical-align: text-bottom;
	}

	.sapMObjStatusIcon {
		height: 1.2rem;
	}
}

/* ============================================= */
/* FIX: CSS 0120061532 0001316230 2014 */
/* ============================================= */
.sapMLIBActive .sapMObjStatusNone .sapMObjStatusText,
.sapMLIBActive .sapMObjStatusNone .sapMObjStatusIcon,
.sapMLIBActive .sapMObjStatusError .sapMObjStatusText,
.sapMLIBActive .sapMObjStatusError .sapMObjStatusIcon,
.sapMLIBActive .sapMObjStatusWarning .sapMObjStatusText,
.sapMLIBActive .sapMObjStatusWarning .sapMObjStatusIcon,
.sapMLIBActive .sapMObjStatusSuccess .sapMObjStatusText,
.sapMLIBActive .sapMObjStatusSuccess .sapMObjStatusIcon {
	color: var(--sapList_Active_TextColor);
}

/* Condensed size */
.sapUiSizeCondensed .sapUiTableCell .sapMObjStatus > .sapMObjStatusText {
	vertical-align: middle;
	font-size: @sapMFontMediumSize;
}

html.sap-desktop .sapMObjStatusActive:focus {
	outline: none;
}

.sapMObjStatusActive .sapMObjStatusText {
	pointer-events: auto;
}

.sapMObjStatusActive:hover > .sapMObjStatusText,
.sapMObjStatusActive:hover > .sapMObjStatusIcon > .sapUiIcon {
	text-decoration: underline;
	cursor: pointer;
}

.sapMObjStatusActive:focus .sapMObjStatusTitle {
	text-decoration: none;
}

.sapMObjStatusLink:hover .sapMObjStatusIcon > .sapUiIcon {
	cursor: pointer;
}

.sapMObjStatusLink:hover .sapMObjStatusText,
.sapMObjStatusLink:hover .sapMObjStatusIcon:only-child > .sapUiIcon {
	text-decoration: underline;
	cursor: pointer;
}

.sapMObjStatusLink .sapMObjStatusIcon:only-child {
	padding-right: 0;
}

.sapMObjStatusInverted .sapMObjStatusIcon.sapMObjStatusIconOnly,
.sapMObjStatusInverted .sapMObjStatusLink .sapMObjStatusIcon.sapMObjStatusIconOnly {
	padding: 0 0.25rem;
	border-radius: 0.25rem 0.25rem;
}

.sapMObjStatusInverted .sapMObjStatusLink:hover .sapMObjStatusText,
.sapMObjStatusInverted .sapMObjStatusLink:hover .sapMObjStatusIcon.sapMObjStatusIconOnly > .sapUiIcon {
	text-decoration: none;
}

.sapMObjStatusInverted.sapMObjStatusIndication01 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_1_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication01 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_1_Active_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication02 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_2_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication02 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_2_Active_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication03 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_3_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication03 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_3_Active_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication04 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_4_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication04 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_4_Active_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication05 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_5_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication05 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_5_Active_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication06 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_6_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication06 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_6_Active_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication07 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_7_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication07 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_7_Active_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication08 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_8_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication08 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_8_Active_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication09 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_9_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication09 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_9_Active_Background);
	border-color: var(--sapIndicationColor_9_Active_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication09 .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication09 .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapIndicationColor_9_Active_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication10 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_10_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication10 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_10_Active_Background);
	border-color: var(--sapIndicationColor_10_Active_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication10 .sapMObjStatusLink:active .sapMObjStatusTex,
.sapMObjStatusInverted.sapMObjStatusIndication10 .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapIndicationColor_10_Active_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusError .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapButton_Negative_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusError .sapMObjStatusLink:hover .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusError .sapMObjStatusLink:hover .sapMObjStatusIcon {
	color: var(--sapButton_Negative_Hover_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusError .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapButton_Negative_Active_Background);
}

.sapMObjStatusInverted.sapMObjStatusError .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusError .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapButton_Active_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusWarning .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapButton_Critical_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusWarning .sapMObjStatusLink:hover .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusWarning .sapMObjStatusLink:hover .sapMObjStatusIcon {
	color: var(--sapButton_Critical_Hover_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusWarning .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapButton_Critical_Active_Background);
}

.sapMObjStatusInverted.sapMObjStatusWarning .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusWarning .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapButton_Active_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusSuccess .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapButton_Success_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusSuccess .sapMObjStatusLink:hover .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusSuccess .sapMObjStatusLink:hover .sapMObjStatusIcon {
	color: var(--sapButton_Success_Hover_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusSuccess .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapButton_Success_Active_Background);
}

.sapMObjStatusInverted.sapMObjStatusSuccess .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusSuccess .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapButton_Active_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusInformation .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapButton_Information_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusInformation .sapMObjStatusLink:hover .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusInformation .sapMObjStatusLink:hover .sapMObjStatusIcon {
	color: var(--sapButton_Information_Hover_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusInformation .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapButton_Information_Active_Background);
}

.sapMObjStatusInverted.sapMObjStatusInformation .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusInformation .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapButton_Active_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusNone .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapButton_Neutral_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusNone .sapMObjStatusLink:hover .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusNone .sapMObjStatusLink:hover .sapMObjStatusIcon {
	color: var(--sapButton_Neutral_Hover_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusNone .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapButton_Neutral_Active_Background);
}

.sapMObjStatusInverted.sapMObjStatusNone .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusNone .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapButton_Active_TextColor);
}

/* Second set of inverted indication colors regular state */
.sapMObjStatusInverted.sapMObjStatusIndication11 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_1b_Background);
	border-color: var(--sapIndicationColor_1b_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication11 .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication11 .sapMObjStatusIcon {
	text-shadow: var(--sapContent_ContrastTextShadow);
	color: var(--sapIndicationColor_1b_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication12 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_2b_Background);
	border-color: var(--sapIndicationColor_2b_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication12 .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication12 .sapMObjStatusIcon {
	text-shadow: var(--sapContent_ContrastTextShadow);
	color: var(--sapIndicationColor_2b_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication13 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_3b_Background);
	border-color: var(--sapIndicationColor_3b_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication13 .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication13 .sapMObjStatusIcon {
	text-shadow: var(--sapContent_ContrastTextShadow);
	color: var(--sapIndicationColor_3b_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication14 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_4b_Background);
	border-color: var(--sapIndicationColor_4b_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication14 .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication14 .sapMObjStatusIcon {
	text-shadow: var(--sapContent_ContrastTextShadow);
	color: var(--sapIndicationColor_4b_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication15 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_5b_Background);
	border-color: var(--sapIndicationColor_5b_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication15 .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication15 .sapMObjStatusIcon {
	text-shadow: var(--sapContent_ContrastTextShadow);
	color: var(--sapIndicationColor_5b_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication16 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_6b_Background);
	border-color: var(--sapIndicationColor_6b_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication16 .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication16 .sapMObjStatusIcon {
	text-shadow: var(--sapContent_ContrastTextShadow);
	color: var(--sapIndicationColor_6b_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication17 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_7b_Background);
	border-color: var(--sapIndicationColor_7b_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication17 .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication17 .sapMObjStatusIcon {
	text-shadow: var(--sapContent_ContrastTextShadow);
	color: var(--sapIndicationColor_7b_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication18 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_8b_Background);
	border-color: var(--sapIndicationColor_8b_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication18 .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication18 .sapMObjStatusIcon {
	text-shadow: var(--sapContent_ContrastTextShadow);
	color: var(--sapIndicationColor_8b_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication19 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_9b_Background);
	border-color: var(--sapIndicationColor_9b_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication19 .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication19 .sapMObjStatusIcon {
	text-shadow: var(--sapContent_ContrastTextShadow);
	color: var(--sapIndicationColor_9b_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication20 .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_10b_Background);
	border-color: var(--sapIndicationColor_10b_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication20 .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication20 .sapMObjStatusIcon {
	text-shadow: var(--sapContent_ContrastTextShadow);
	color: var(--sapIndicationColor_10b_TextColor);
}

/* Second set of inverted indication colors hovered state */
.sapMObjStatusInverted.sapMObjStatusIndication11 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_1b_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication12 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_2b_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication13 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_3b_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication14 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_4b_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication15 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_5b_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication16 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_6b_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication17 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_7b_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication18 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_8b_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication19 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_9b_Hover_Background);
}

.sapMObjStatusInverted.sapMObjStatusIndication20 .sapMObjStatusLink:hover > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_10b_Hover_Background);
}

/* Second set of inverted indication colors active state */
.sapMObjStatusInverted.sapMObjStatusIndication11 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_1_Active_Background);
	border-color: var(--sapIndicationColor_1_Active_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication11 .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication11 .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapIndicationColor_1_Active_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication12 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_2_Active_Background);
	border-color: var(--sapIndicationColor_2_Active_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication12 .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication12 .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapIndicationColor_2_Active_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication13 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_3_Active_Background);
	border-color: var(--sapIndicationColor_3_Active_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication13 .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication13 .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapIndicationColor_3_Active_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication14 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_4_Active_Background);
	border-color: var(--sapIndicationColor_4_Active_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication14 .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication14 .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapIndicationColor_4_Active_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication15 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_5_Active_Background);
	border-color: var(--sapIndicationColor_5_Active_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication15 .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication15 .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapIndicationColor_5_Active_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication16 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_6_Active_Background);
	border-color: var(--sapIndicationColor_6_Active_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication16 .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication16 .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapIndicationColor_6_Active_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication17 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_7_Active_Background);
	border-color: var(--sapIndicationColor_7_Active_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication17 .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication17 .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapIndicationColor_7_Active_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication18 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_8_Active_Background);
	border-color: var(--sapIndicationColor_8_Active_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication18 .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication18 .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapIndicationColor_8_Active_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication19 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_9_Active_Background);
	border-color: var(--sapIndicationColor_9_Active_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication19 .sapMObjStatusLink:active .sapMObjStatusText,
.sapMObjStatusInverted.sapMObjStatusIndication19 .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapIndicationColor_9_Active_TextColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication20 .sapMObjStatusLink:active > .sapMObjStatusWrapper {
	background-color: var(--sapIndicationColor_10_Active_Background);
	border-color: var(--sapIndicationColor_10_Active_BorderColor);
}

.sapMObjStatusInverted.sapMObjStatusIndication20 .sapMObjStatusLink:active .sapMObjStatusTex,
.sapMObjStatusInverted.sapMObjStatusIndication20 .sapMObjStatusLink:active .sapMObjStatusIcon {
	color: var(--sapIndicationColor_10_Active_TextColor);
}

html.sap-desktop .sapMObjStatusActive:focus .sapMObjStatusLink {
	position: relative;
	display: inline-block;
}

.sapMObjStatusActive {

	&.sapMObjStatusInverted .sapMObjStatusText {
		vertical-align: top;
	}

	&.sapMObjStatusInverted .sapMObjStatusIcon > .sapUiIcon {
		vertical-align: top;
	}
}

html.sap-desktop .sapMObjStatusActive:focus {

	.sapMObjStatusLink::after {
		pointer-events: none;
		display: block;
		content: '';
		position: absolute;
		top: -1px;
		left: -1px;
		bottom: -1px;
		right: -1px;
		outline: 1px dotted var(--sapContent_FocusColor);
	}

	&.sapMObjectStatusLarge {

		.sapMObjStatusLink::after {
			position: absolute;
			top: 0;
			left: 1px;
			bottom: 0;
			right: 1px;
			height: 2rem;
		}

		&.sapMObjStatusInverted .sapMObjStatusLink::after {
			position: absolute;
			top: 3px;
			left: 3px;
			bottom: 3px;
			right: 3px;
			height: 1.625rem;
			outline: 1px dotted var(--sapContent_ContrastFocusColor);
		}
	}
}

html.sap-desktop .sapMOHStatus .sapMObjStatusActive:focus .sapMObjStatusLink::after {
	left: 1px;
	right: 1px;
}

html.sap-desktop .sapUiTableCell .sapMObjStatusActive:focus .sapMObjStatusLink::after {
	left: 1px;
	right: 1px;
	top: 1px;
	bottom: 1px;
}

.sapMObjStatus .sapUiPseudoInvisibleText {
	font-size: 0;
}

.sapMObjStatus .sapMEmptyIndicator.sapMEmptyIndicatorAuto {
	display: none;
}

.sapMShowEmpty-CTX .sapMObjStatus .sapMEmptyIndicator.sapMEmptyIndicatorAuto {
	display: inline-block;
}

html.sap-desktop .sapMListTblHasPopin .sapMObjStatusActive:focus {

	.sapMObjStatusLink::after {
		pointer-events: none;
		display: block;
		content: '';
		position: absolute;
		top: 0;
		left: 1px;
		bottom: 1px;
		right: 1px;
		outline: 1px dotted var(--sapContent_FocusColor);
	}
}

.sapMListTblCell {

	.sapMObjStatus.sapMObjStatusInverted {
		display: inline-table;

		&:not(.sapMObjectStatusLongText):not(:has(.sapMObjStatusIcon)) {
			width: min-content;
		}

		.sapMObjStatusText {
			line-height: 1rem;
			vertical-align: middle;
		}
	}
}

/* BCP: 2370130730 - when there is a text in Thai languiage, the text decorations are cut, that's why there is */
/* CSS class provided - sapUiHigherText - that can be applied on control level or on parent element/BODY level */
/* in general this class should fix the cutting on most of the languages that have characters with decorations */
.sapUiHigherText.sapMObjStatus,
.sapUiHigherText .sapMObjStatus {

	.sapMObjStatusTitle {
		line-height: inherit;
		vertical-align: initial;
	}

	.sapMObjStatusIcon,
	.sapMObjStatusText {
		line-height: 1.6;
	}

	.sapMObjStatusWrapper {
		align-items: baseline;
	}

	.sapMObjStatusIcon {

		.sapUiIcon {
			line-height: inherit;
			top: 0.0625rem;
		}
	}
}
/* END "ObjectStatus.less" */

/* START "OverflowToolbarAssociativePopover.less" */
/* ======================================================== */
/* CSS for control sap.m/OverflowToolbarAssociativePopover  */
/* Base theme                                               */
/* ======================================================== */

.sapMOTAPopover {

	&:focus {
		outline: none;
	}

	.sapMBtn .sapMBtnInner {
		justify-content: flex-start;
	}

	&.sapMPopover .sapMPopoverCont .sapMPopoverScroll {
		padding: 0.25rem 0.5rem;

		/* Disappear */
		& > .sapMOTAPHidden {
			display: none;
		}

		/* All elements inside must have display block */
		& > :not(.sapMOTAPHidden) {
			display: block;
		}

		& > .sapMTBSeparator.sapMTBSeparatorOverflowToolbar {

			&:first-child,
			&:last-child {
				display: none;
			}
		}

		/* Button, ToggleButton, OverflowToolbarButton */
		& > button.sapMBtn {
			width: 100% !important;
			max-width: 100% !important;

			/* Button alignment */
			& > .sapMBtnInner {
				text-align: start;
			}
		}

		/* Select */
		& > div.sapMSlt {
			width: auto !important;
		}

		/* SegmentedButton - select form */
		& > div.sapMSegBSelectWrapper {
			width: 100% !important;

			& > div.sapMSlt {
				width: 100% !important;
				max-width: 100% !important;
			}
		}

		/* ComboBox */
		& > div.sapMComboBox {
			width: auto !important;
			max-width: auto !important;
		}

		/* SearchField */
		& > div.sapMSF {
			width: 100% !important;
			max-width: 100% !important;
		}

		/* Input */
		& > div.sapMInput {
			width: 100% !important;
			max-width: 100% !important;
		}

		/* DateTimeInput */
		& > div.sapMDTI {
			width: 100% !important;
			max-width: 100% !important;
		}

		/* Menu Button */
		& > .sapMMenuBtn {
			max-width: 100% !important;
		}
	}
}

/* Compact mode overrides */
.sapUiSizeCompact {

	.sapMOTAPopover {

		&.sapMPopover .sapMPopoverCont .sapMPopoverScroll {
			padding: 0.1875rem 0.375rem;
		}

		.sapMBtnInner.sapMBtnIconFirst:not(.sapMBtnBack) {
			padding-left: 0.5rem;
			padding-right: 0.5rem;
		}
	}
}

/* Phone mode overrides */
html.sap-phone {

	.sapMOTAPopover {
		overflow: hidden !important;
	}
}
/* END "OverflowToolbarAssociativePopover.less" */

/* START "P13nColumnsPanel.less" */
/* ======================================= */
/* CSS for control sap.m/P13nColumnsPanel  */
/* Base theme                              */
/* ======================================= */

.sapMP13nColumnsPanel {
	color: var(--sapTextColor);
}

.sapMP13nColumnsPanel .sapMLIB.sapMLIBSelected,
.sapMP13nColumnsPanel .sapMLIB.sapMLIBSelected > td {
	background-color: transparent;
}

.sapMP13nColumnsPanel .sapMListBGTranslucent .sapMLIB.sapMListTblRow.sapMP13nColumnsPanelItemSelected,
.sapMP13nColumnsPanel .sapMListBGTranslucent .sapMLIB.sapMListTblRow.sapMP13nColumnsPanelItemSelected:focus,
.sapMP13nColumnsPanel .sapMListBGTranslucent .sapMLIB:not(.sapMLIBActive):not(.sapMLIBSelected):not(:hover).sapMListTblRow.sapMP13nColumnsPanelItemSelected,
.sapMP13nColumnsPanel .sapMListBGTranslucent .sapMLIB:not(.sapMLIBActive):not(.sapMLIBSelected):not(:hover).sapMListTblRow.sapMP13nColumnsPanelItemSelected:focus {
	background-color: var(--sapList_SelectionBackgroundColor);
}

.sapMP13nColumnsPanel .sapMLIBActive .sapMText,
.sapMP13nColumnsPanel .sapMLIBActive .sapMListTblCell .sapMText {
	color: var(--sapList_TextColor);
}

.sapMP13nColumnsPanel .sapMLIBActive .sapMCbBg,
.sapMP13nColumnsPanel .sapMLIBActive.sapMLIB,
.sapMP13nColumnsPanel .sapMLIBActive.sapMLIB > td {
	background-color: transparent;
}

.sapMP13nColumnsPanel .sapMLIBActive .sapMCbBg,
.sapMP13nColumnsPanel .sapMLIBActive .sapMCbBg.sapMCbMarkChecked,
.sapMP13nColumnsPanel .sapMLIBActive .sapMCbBg.sapMCbBgDis,
.sapMP13nColumnsPanel .sapMLIBActive .sapMCbBg.sapMCbBgDis.sapMCbMarkChecked {
	border-color: var(--sapField_BorderColor);
}

.sapMP13nColumnsPanel .sapMLIBActive .sapMCbBg.sapMCbMarkChecked::before {
	color: var(--sapSelectedColor);
}

html.sap-desktop .sapMP13nColumnsPanel .sapMLIBSelected:not(.sapMLIBActive):not(.sapMALI):hover,
html.sap-desktop .sapMP13nColumnsPanel .sapMLIBSelected:not(.sapMLIBActive):not(.sapMALI):hover > td {
	background-color: var(--sapList_Hover_Background);
}
/* END "P13nColumnsPanel.less" */

/* START "P13nConditionPanel.less" */
/* ========================================= */
/* CSS for control sap.m/P13nConditionPanel  */
/* Base theme                                */
/* ========================================= */

.sapMConditionPanel .sapUiRespGridMedia-Std-Phone .floatRight {
	float: right;
}

.sapMConditionPanel .displayNone {
	display: none;
}

.sapMConditionPanel .conditionRootGrid > div + div .sapUiRespGridMedia-Std-Tablet {
	margin-top: 1rem;
}

.sapMConditionPanel .conditionRootGrid > div + div .sapUiRespGridMedia-Std-Phone {
	margin-top: 1rem;
}

.sapMConditionPanel .conditionAddBtnFloatRight {
	float: right;
}

.sapUiSizeCompact .sapMConditionPanel .conditionLabel {
	margin-top: 0.5rem;
}

.sapMConditionPanel .conditionLabel {
	margin-top: 1rem;
}

.sapMConditionPanel .sapMCb {
	display: block;
}

.sapMConditionPanel .sapMTB {
	border-bottom: 0;
}

.sapMFilterPanel .sapMPanelWrappingDivTb {
	border-bottom: 0;
}

.sapMFilterPanel .sapMTB {
	border-bottom: 0;
}

/* WORKAROUND BCP:1770523047 : set min-height back to 0px */
.sapMConditionPanel div[class*='sapUiRespGrid'] > div[class*='sapUiRespGridSpan'] {
	min-height: 0;
}
/* END "P13nConditionPanel.less" */

/* START "P13nDialog.less" */
/* ================================= */
/* CSS for control sap.m/P13nDialog  */
/* Base theme                        */
/* ================================= */

.sapMP13nDialog {
	color: var(--sapTextColor);
}

.sapMP13nDialog .sapMITBHead {
	margin: 0;
}

.sapMP13nDialog.sapMDialog > .sapMDialogSection {
	background-color: var(--sapBackgroundColor);
}

.sapMP13nDialog .sapMBarLeft,
.sapMP13nDialog.sapMDialog .sapMBarLeft {
	padding-left: 0;
}

.sapMP13nDialog .sapMITH.sapMITBInLine .sapMITBHead {
	padding-top: 1.875rem;
}

.sapUiSizeCompact .sapMP13nDialog .sapMITH.sapMITBInLine .sapMITBHead {
	padding-top: 1.5rem;
}
/* END "P13nDialog.less" */

/* START "p13n.Container.less" */
/* ===================================== */
/* CSS for control sap.m/p13n.Container  */
/* Base theme                            */
/* ===================================== */

/* Change background color for the container control */
.sapMP13nContainer .sapMITB {
	line-height: 3rem;
}

/* Remove padding to align the ITB with the rest of the p13n related controls (left aligned) */
.sapMP13nContainer .sapMITH {
	padding: 0;
}
/* END "p13n.Container.less" */

/* START "p13n.QueryPanel.less" */
/* ====================================== */
/* CSS for control sap.m/p13n.QueryPanel  */
/* Base theme                             */
/* ====================================== */

.sapMP13nQueryPanel .sapUiRespGrid {
	display: flex;
	align-items: center;
}

.sapMP13nQueryPanel .sapUiRespGrid.sapUiRespGridVSpace1 > div {
	margin-bottom: 0;
}
/* END "p13n.QueryPanel.less" */

/* START "p13n.SelectionPanel.less" */
/* ========================================== */
/* CSS for control sap.m/p13n.SelectionPanel  */
/* Base theme                                 */
/* ========================================== */

/* Display ':' for Labels in the 'list' view */
.sapMSelectionPanel.sapUiMDCAFLabelMarkingList tbody .sapMLabel > .sapMLabelColonAndRequired::before {
	content: attr(data-colon);
}

/* Display checkbox at the top for items in the ListView */
.sapMSelectionPanel tbody .sapMListTblSelCol {
	vertical-align: top;
}

/* Remove padding for items in list view */
.sapMSelectionPanel.SelectionPanelHover .sapMListTblCell {
	padding-top: 0;
	padding-bottom: 0;
}

.sapMSelectionPanel .sapMPanelContent li {
	align-items: start; /* Checkbox should not be centered */
	display: flex; /* Is the default for Lists - only needs to be set as the parent is a Table */
}

/* Align inner rows with the outer controls */
.sapMSelectionPanel table {

	tr > td {
		padding-left: 0.5rem;
	}
}

/* 'Select All' should also be aligned correctly with the other controls in the panel */
.sapMSelectionPanel .sapMTableTH {
	padding-left: 0.5rem;
}

/* filters popover should have correct margins */
.sapMSelectionPanelFiltersPopover {
	// right margin is smaller due to margin of switch
	margin: 1rem 0.5rem 1rem 1rem;
};

/* distance between two switch filtes should be correct */
.sapMSelectionPanelFiltersContainer:first-child {
	margin-bottom: 0.375rem;
}

/* text should be centered vertically */
.sapMSelectionPanelFilters {
	display: flex;
}
/* END "p13n.SelectionPanel.less" */

/* START "P13nFilterPanel.less" */
/* ====================================== */
/* CSS for control sap.m/P13nFilterPanel  */
/* Base theme                             */
/* ====================================== */

.sapMFilterPanel .compPadding {
	padding-left: 1rem;
	padding-right: 1rem;
	padding-bottom: 1rem;
}

.sapMFilterPanel .sapMPanelContent {
	padding: 0;
	border-bottom-width: 0;
}

.sapMFilterPanel .sapMPanelBGTranslucent {
	background-color: Transparent;
}

.sapMFilterPanel .sapUiRespGrid.sapUiRespGridVSpace1 > div {
	margin-bottom: 0;
	margin-left: 0;
}

.sapMFilterPanel .sapMPanelWrappingDiv {
	border-bottom: 0;
}

.sapMFilterPanel .panelTopMargin {
	margin-top: 1rem;
}

.sapMFilterPanel .panelNoHeader .sapMPanelHdr {
	height: 0;
	border-bottom: 0;
}

.sapMFilterPanel .sapMPanelContent:not(.sapMPanelBGTransparent) {
	border-bottom-width: 0;
}
/* END "P13nFilterPanel.less" */

/* START "P13nSortPanel.less" */
/* ==================================== */
/* CSS for control sap.m/P13nSortPanel  */
/* Base theme                           */
/* ==================================== */

.sapMSortPanel {
	width: 100%;
}

.sapMSortPanelContent {
	padding-top: 1rem;
	padding-bottom: 1rem;
}

.sapMGroupPanelContent {
	padding-top: 1rem;
	padding-bottom: 1rem;
}
/* END "P13nSortPanel.less" */

/* START "Page.less" */
/* =========================== */
/* CSS for control sap.m/Page  */
/* Base theme                  */
/* =========================== */

@_sap_m_Page_AppHeight: 3333px; /* Ux decision: never show the header, so make this value large enough even for zoom-out cases */
@_sap_m_Page_HeaderHeight: 68px; /* should be 4.25rem */
@_sap_m_Page_MinHeightForHeader: (@_sap_m_Page_AppHeight + @_sap_m_Page_HeaderHeight);
@_sap_m_Page_FloatinFooterHeight: 2.75rem;
@_sap_m_Page_FloatinFooterHeight_Compact: 2.5rem;
@_sap_m_Page_FloatingFooterMarginBottom: 0.5rem;

.sapMPage,
.sapMPage > section {
	width: 100%;
	box-sizing: border-box;
}

.sapMPage {
	height: 100%;
	position: relative;
	isolation: isolate; /* makes sure the z-index stuff for headers/footer has no effect outside the Page; only works in the very newest browsers as of 03/2015 */
	z-index: 0; /* workaround for missing "isolation" support in most browsers; TODO: check when this can be removed */
}

.sapMPage > section {
	position: absolute;
	overflow-y: hidden;
	top: 0;
	left: 0;
	right: 0;
	bottom: 0;
}

.sapMPage > section.sapMPageEnableScrolling {
	overflow-y: auto;
	will-change: scroll-position;
}

/* If there are subpages with disabled scrolling, then its content should overflow. I.e. reset overflow-y to defaults. */
.sapMPage > section.sapMPageEnableScrolling .sapMPage > section {
	overflow-y: visible;
}

html[data-sap-ui-os^='iOS'] {

	.sapMPage > section.sapMPageEnableScrolling {
		overflow-y: scroll;
	}
}

/* Fix for a ToolBar set as headerContent */
html[data-sap-ui-browser^='ff'].sap-desktop {

	.sapMPage > .sapMBar.sapMPageHeader .sapMBarContainer {
		display: initial;
	}

	.sapUiSizeCompact .sapMPage > .sapMBar.sapMPageHeader .sapMBarContainer {
		vertical-align: -moz-middle-with-baseline;
	}

	.sapMPage > .sapMTitle {
		font-size: 1rem;
	}
}

/*
* Workaround for IOS 7 - on orientation change to lanscape mode:
* on the Ipad the footer is cut off.
* on the phone, the header is cut off and if you open the nav Bar(clicking in the footer), the header will be cut off even more.
* setting the body to fixed seems to work. We also tried it with absolute, but it does not work (tested only on ipad).
*/
@media (orientation: landscape) {

	html[data-sap-ui-os^='iOS7'][data-sap-ui-browser^='msf'] {
		padding: 0;
		margin: 0;
		bottom: 0;
	}

	html[data-sap-ui-os^='iOS7'][data-sap-ui-browser^='msf'] body {
		position: fixed;
		top: 0;
		bottom: 0;
	}
}

/* CSS for displaying the Footer/Header controls at the correct position */
.sapMPageFooter {
	position: absolute;
	bottom: 0;
	left: 0;
	/* footer has to overlap the header if the page is very small (phone with open keyboard)*/
	z-index: 2;
	width: 100%;
}

.sapMPageFooter.sapMPageFloatingFooter {
	left: 0;
	right: 0;
}

.sapMPageFooter.sapUiHidden {
	bottom: auto;
}

.sapMPageFooter .sapMTB.sapMIBar {
	right: 0;
	border-bottom-width: 0;
}

.sapMPageWithFloatingFooter .sapMPageFooter .sapMIBar-CTX.sapMFooter-CTX.sapMTB {
	border-top-width: 0;
}

.sapMPageSubHeader,
.sapMPageHeader {
	/* header has to be on top of the Scroll container */
	z-index: 1;
}

.sapMPage.sapMPageBusyCoversAll > .sapUiLocalBusyIndicator {
	/* busyIndicator within page needs a z-index to render on top of header/footer */
	z-index: 3;
}

/* mixin call for creating container content padding classes (arguments: rootContainer, contentSelector) */
.sapUiContainerContentPadding(~".sapMPage", ~"> section");

/* Fiori 2.0 adaptation */
.sapF2CollapsedHeader > .sapMPageHeader {
	display: none;
}

.sapMPage.sapF2CollapsedHeader > .sapMPageHeader + section,
.sapMPage.sapF2CollapsedHeader > .sapMPageHeader + .sapUiHiddenPlaceholder + section {
	top: 0;
}

.sapMPage.sapF2CollapsedHeader > .sapMPageHeader + .sapMPageSubHeader + section {
	top: 3rem;
}

.sapUiSizeCompact .sapF2CollapsedHeader.sapMPageWithHeader > section {
	top: 0;
}

.sapUiSizeCompact .sapF2CollapsedHeader.sapMPageWithHeader.sapMPageWithSubHeader > section {
	top: 2rem;
}

.sapUiSizeCompact .sapF2CollapsedHeader > .sapMPageHeader .sapMIBar {
	height: 0;
}

.sapMIBar .sapF2AdaptedTitle,
.sapMIBar .sapF2AdaptedNavigation {
	display: none;
}

/* page with footer */
.sapMPageWithFooter > section {
	bottom: 3rem;
}

.sapMPageWithFloatingFooter {

	> .sapMPageFooter {
		border-radius: 0.25rem;
		margin: 0.5rem;
		width: ~"calc(100% - 1rem)";
		opacity: 0.9;

		.sapMIBar {
			border-radius: 0.25rem;
		}
	}
}

.sapMPageWithFloatingFooter.sapMPageWithFooter > section {
	bottom: 0;
	scroll-padding-bottom: ~"calc(@{_sap_m_Page_FloatinFooterHeight} + @{_sap_m_Page_FloatingFooterMarginBottom} + 2px)";

	&::after {
		content: '';
		display: block;
		height: 0;
		width: 100%;
		margin-bottom: 4rem;
	}
}

.sapUiSizeCompact .sapMPageWithFooter.sapMPageWithFloatingFooter > section {
	bottom: 0;
	scroll-padding-bottom: ~"calc(@{_sap_m_Page_FloatinFooterHeight_Compact} + @{_sap_m_Page_FloatingFooterMarginBottom} + 2px)";
	padding-bottom: 3.5rem;
}

html[data-sap-ui-animation='on'] .sapMPageWithFloatingFooter {

	.sapMPageFooterControlShow {
		animation: bounceShow 0.35s forwards ease-in-out;
	}

	.sapMPageFooterControlHide {
		animation: bounceHide 0.35s forwards ease-in-out;
	}

	@keyframes bounceShow {

		0% {
			transform: translateY(100%);
			opacity: 0;
		}

		100% {
			opacity: 0.9;
		}
	}

	@keyframes bounceHide {

		0% {
			transform: translateY(-5%);
			opacity: 0.9;
		}

		100% {
			transform: translateY(100%);
			opacity: 0;
		}
	}
}

.sapMPageBgSolid {
	background-color: var(--sapBackgroundColor);
}

/* page with header and subHeader */
.sapMPage > .sapMPageHeader + .sapMPageSubHeader + section {
	top: 6rem;
}

.sapMPage > .sapMPageHeader + .sapMPageSubHeader.sapMPageSubHeaderInfoBar + section {
	top: 5rem;
}

/* page with header */
.sapMPage > .sapMPageHeader + section,
/* page with subheader */
.sapMPage > .sapMPageSubHeader + section,
/* page with header and invisible subheader */
.sapMPage > .sapMPageHeader + .sapUiHiddenPlaceholder + section {
	top: 3rem;
}

/* page app icon */
.sapMBar-CTX .sapMBarLeft > .sapMImg.sapMPageAppIcon {
	max-height: 1.375rem;
	max-width: 1.375rem;
	margin: 0.8125rem;
}

/* vertical scroll bar */
.sapMPage .sapMScrollbarV > div {
	background-color: var(--sapScrollBar_FaceColor);
	border-color: var(--sapScrollBar_BorderColor);
}

/* sap.m.Shell + Tiles Page specific style */
@media (min-height: @_sap_m_Page_MinHeightForHeader) {

	.sap-desktop .sapMShell .sapMPage.sapMPageHideHeaderWhenShellHeader header.sapMIBar {
		display: none;
	}
}

html.sap-tablet .sapUiFioriObjectPage > section,
html.sap-desktop .sapUiFioriObjectPage > section {
	padding-bottom: 1rem;
}

/* Compact size */
.sapUiSizeCompact {

	/* Header and footer have 2.5 rem */
	.sapMPageWithHeader > section {
		top: 2.5rem;
	}

	.sapMPageHeader .sapMIBar,
	.sapMPageFooter .sapMIBar {
		height : 2.5rem;

		.sapMBarLeft,
		.sapMBarRight,
		.sapMBarPH {
			height: 2.5rem;
			padding-top: 0.25rem;
			padding-bottom: 0.25rem;
		}
	}

	.sapMPageWithFooter > section {
		bottom: 2.5rem;
	}

	/* Subheader only 2 rem */
	.sapMPageWithSubHeader > section {
		top: 2rem;
	}

	.sapMPageWithHeader {

		&.sapFShellBar-CTX > section {
			top: 3rem;
		}

		&.sapMPageWithSubHeader.sapFShellBar-CTX > section {
			top: 5rem;
		}
	}

	&.sapMPage.sapMPageWithHeader.sapMPageWithSubHeader,
	.sapMPage.sapMPageWithHeader.sapMPageWithSubHeader {

		> section {
			top: 4.5rem;
		}

		.sapMPageSubHeader.sapMPageSubHeaderInfoBar + section {
			top: 4.5rem;
		}
	}

	.sapMBar-CTX .sapMBarLeft > .sapMImg.sapMPageAppIcon {
		max-height: 1rem;
		max-width: 1rem;
		margin: 0.5rem;
	}
}

/*
In order to skip adding that to each control we decided to move that on Popup level
will-change property has been aded for performance improvements when having a huge scroll of the page (painting improvement)
Usually the Popover/Dialog does not have that much of a content and the scrolling performance is good
*/
.sapMPopup-CTX .sapMPage > section.sapMPageEnableScrolling {
	will-change: unset; /* fixes iOS and OS X issue - the content and the header sometimes get misplaced */
}
/* END "Page.less" */

/* START "PagingButton.less" */
/* =================================== */
/* CSS for control sap.m/PagingButton  */
/* Base theme                          */
/* =================================== */

.sapMPagingButton {
	display: inline-block;
}
/* END "PagingButton.less" */

/* START "Panel.less" */
/* ============================ */
/* CSS for control sap.m/Panel  */
/* Base theme                   */
/* ============================ */

.sapMPanel {
	overflow: hidden;
	box-sizing: border-box;
	/* The following needed for calculation of height for content child */
	position: relative;
}

.sapMPanel.sapMPanelHasStickyHeader {
	overflow: visible;
}

.sapMPanelHdr {
	box-sizing: border-box;
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
	text-transform: none;
	height: 3rem;
	line-height: 3rem;
	margin: 0;
	-webkit-text-size-adjust: none; /* To improve readability Mobile Safari automatically increases the size of small text so let's disable this */
	font-weight: @sapUiFontHeaderWeight;
	font-size: var(--sapGroup_Title_FontSize);
	font-family: var(--sapFontHeaderFamily);
	color: var(--sapGroup_TitleTextColor);
}

.sapMPanel {

	.sapMPanelHdr,
	.sapMTB.sapMPanelHeaderTB {
		padding-left: 1rem;
		padding-right: 0.5rem;
		background-color: var(--sapGroup_TitleBackground);
	}

	&.sapMPanelExpandable {

		.sapMPanelHdr,
		.sapMTB.sapMPanelHeaderTB {
			padding-left: 0;
		}
	}
}

.sapUiSizeCompact {

	.sapMPanel {
		min-height: 2rem;

		.sapMPanelHdr {
			line-height: 2rem;
			height: 2rem;
		}

		.sapMPanelWrappingDiv .sapUiIcon {
			width: 2rem;
			height: 1.5rem;
			line-height: 1.5rem;
		}
	}
}

.sapMPanelStickyHeadingDiv {
	position: sticky;
	top: 0;
	z-index: 4; /* The z-index of the Table sticky header is 3, so we need bigger one to make the Panel header on top */
	background-color: var(--sapBackgroundColor);
}

.sapMPanelContent {
	padding: 0.625rem 1rem 1.375rem 1rem;
	box-sizing: border-box;
	overflow: auto;
	white-space: normal;

	/* This is needed in order to remove the additional outline added by tabindex=-1 */
	&:focus {
		outline: none;
	}
}

.sapMPanelExpandablePart {
	clear: both;
}

.sapMPanelWrappingDiv,
.sapMPanelWrappingDivTb {
	position: relative;
	background-color: var(--sapGroup_TitleBackground);

	> .sapMBtn {
		position: absolute;
		top: 50%;
		transform: translateY(-50%);
		padding-left: 0.25rem;
		padding-right: 0.25rem;
	}
}

.sapMPanelWrappingDiv .sapUiIcon {
	width: 2.25rem;
	height: 2.25rem;
	line-height: 2.25rem;
	position: absolute;
	top: 50%;
	transform: translateY(-50%);
	padding: 0.25rem;
	font-size: 1rem;
	cursor: inherit;
	color: var(--sapContent_IconColor);
}

/* if Panel is expandable, we do not wish toolbar to come with own border */
.sapMPanelWrappingDivTb > .sapMTB {
	border-width: 0;
}

.sapMPanelWrappingDiv.sapMPanelWrappingDivExpanded {
	border-bottom-width: 0;
}

.sapMPanelWrappingDivTb.sapMPanelWrappingDivTbExpanded {
	border-bottom-width: 0;
}

html.sap-tablet .sapUiFioriObjectPage > section > .sapMPanel,
html.sap-desktop .sapUiFioriObjectPage > section > .sapMPanel {
	margin: 1rem 0;
	padding: 0 1rem;
	box-sizing: border-box;
}

.sapMPanel .sapMPanelWrappingDiv .sapMPanelHdr,
.sapMPanel .sapMPanelWrappingDivTb .sapMIBar.sapMTB {
	margin-left: 2.75rem;
	padding-left: 0.25rem;

	> :first-child {
		margin-left: 0;
	}
}

/* mixin call for creating container content padding classes (arguments: rootContainer, contentSelector) */
.sapUiContainerContentPadding(~".sapMPanel", ~"> .sapMPanelContent");

.sapMPanelExpandable .sapMPanelWrappingDiv {

	&:focus {
		outline: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
		outline-offset: -0.1875rem;
	}

	&:hover {
		cursor: pointer;
	}
}
/* END "Panel.less" */

/* START "PDFViewer.less" */

.sapMPDFViewerLoading {
	visibility: hidden;
}

.sapMPDFViewerWrapper {
	width: 100%;
	height: 100%;
	position: relative;
}

.sapMPDFViewerContent {
	width: 100%;
	height: 100%;
	border: 0 none;
	outline: 0 none;
}

.sapMPDFViewerContent.sapMPDFViewerReducedContent {
	height: calc(~'100% - 3.5rem');
}

.sapMPDFViewerError {
	width: 100%;
	height: 100%;
    .sapMMessagePage {
        min-height: 300px;
    }
}

.sapMPDFViewerError.sapMPDFViewerReducedContent {
	height: calc(~'100% - 3.5rem');
}

.sapMPDFViewerNonTrustedIllustratedMessage {
	width: 100%;
	height: 100%;
}
/* END "PDFViewer.less" */

/* START "PlanningCalendar.less" */
/* ======================================= */
/* CSS for control sap.m/PlanningCalendar  */
/* Base theme                              */
/* ======================================= */

.sapMPlanCal {
	background-color: var(--sapList_Background);
}

.sapMPlanCal > .sapMList .sapMListInfoTBar.sapMTB-Transparent-CTX.sapMTB {
	padding: 0;
}

.sapMPlanCal > .sapMList .sapMListInfoTBar.sapMTB-Transparent-CTX.sapMTB >:first-child {
	margin-left: 0;
}

.sapMPlanCalHead {
	width: 20%;
	height: 3rem;
}

.sapMPlanCalWithWeekNumbers .sapMPlanCalHead {
	height: 6rem;
}

.sapPCRelative.sapMPlanCalWithWeekNumbers .sapMPlanCalHead {
	height: 3rem;
}

.sapMPlanCalWithDayNamesLine .sapMPlanCalHead {
	height: 5rem;
}

.sapMPlanCalWithWeekNumbers.sapMPlanCalWithDayNamesLine .sapMPlanCalHead {
	height: 6rem;
}

.sapMPlanCalHead > .sapMTB.sapMIBar {
	padding-left: 0.875rem;
}

.sapMPlanCal .sapUiCalItem.sapUiCalItemWithSecondaryType {
	display: flex;
	flex-direction: column;
	justify-content: center;
	font-size: 0.75rem;

	.sapUiCalItemText {
		height: 50%;
		line-height: 1.175rem;
	}

	.sapUiCalItemAddText {
		line-height: 1.175rem;
	}
}

.sapUiSizeCompact {

	.sapMPlanCal .sapUiCalNoNameLine .sapUiCalItem {
		height: 2rem;
	}

	.sapMPlanCal .sapUiCalOneMonthDatesRow.sapUiCalNoNameLine .sapUiCalItem {
		height: 3rem;
	}

	.sapMPlanCalHead {
		height: 2rem;
	}

	.sapMPlanCalWithWeekNumbers .sapMPlanCalHead,
	.sapMPlanCalWithDayNamesLine .sapMPlanCalHead {
		height: 3rem;
	}

	.sapMPlanCalWithWeekNumbers.sapMPlanCalWithDayNamesLine .sapMPlanCalHead {
		height: 6rem;
	}

	.sapMPlanCal .sapUiCalNoNameLine .sapUiCalItem > .sapUiCalItemText {
		padding-top: 0;
		padding-left: 53%;
		line-height: inherit;
		text-align: start;
	}

	.sapMPlanCal .sapUiCalOneMonthDatesRow.sapUiCalNoNameLine .sapUiCalItem > .sapUiCalItemText {
		padding-top: 1rem;
		padding-left: 0;
		text-align: center;
	}

	.sapMPlanCal .sapUiCalItemNow > .sapUiCalItemText,
	.sapMPlanCal .sapUiCalNoNameLine .sapUiCalItem.sapUiCalItemNow > .sapUiCalItemText,
	.sapMPlanCal.sapUiCalNoNameLine .sapUiCalItem.sapUiCalItemNow > .sapUiCalItemText,
	.sapMPlanCal .sapUiCalNoNameLine .sapUiCalItem.sapUiCalItemNow > .sapUiCalItemText {
		line-height: 1.725rem;
	}

	.sapMPlanCal .sapUiCalNoNameLine .sapUiCalItem > .sapUiCalDayName {
		left: 0;
		height: 100%;
		width: 53%;
		line-height: inherit;
		padding-top: 0;
		padding-right: 0.25rem;
		text-align: end;
		box-sizing: border-box;
	}

	.sapMPlanCal .sapUiCalOneMonthDatesRow.sapUiCalNoNameLine .sapUiCalItem > .sapUiCalDayName {
		left: 0;
		height: 100%;
		width: 100%;
		line-height: inherit;
		padding-top: 0;
		padding-right: 0;
		text-align: center;
		box-sizing: border-box;
	}

	.sapMPlanCal .sapMLIB.sapMLIBShowSeparator.sapMSLI {
		height: auto;
	}
}

.sapMPlanCal > .sapMList .sapMPlanCalRowHead > .sapMLIB,
.sapMPlanCal > .sapMList .sapMPlanCalAppRow > .sapUiCalendarRow,
.sapMPlanCal > .sapMList .sapMPlanCalAppRow > div > .sapUiCalendarRow {
	background-color: transparent;
	display: block;
	border: none;
}

/* if Head is displayed next to CalendarRow, use head size as minimum */
.sapMPlanCal:not(.sapMPlanCalNoHead) > .sapMList > table > tbody > tr > td > .sapUiCalendarRow:not(.sapMPlanCalRowLarge) > .sapUiCalendarRowApps {
	min-height: 4rem;
}

/* if Head has 2 lines, CalendarRow must be large */
.sapMPlanCal:not(.sapMPlanCalNoHead) > .sapMList > table > tbody > tr > td > .sapUiCalendarRow.sapMPlanCalRowLarge > .sapUiCalendarRowApps {
	min-height: 4rem;
}

.sapMPlanCalHead > .sapMTB {
	border-bottom: none;
}

.sapMPlanCalHead > .sapMTB > .sapMSlt {
	flex: auto; /* to make the select as big as possible */
}

.sapMPlanCal > .sapMList .sapMListInfoTBar > .sapMBarChild.sapUiCalRow {
	width: 80%;
	padding: 0;
	margin-left: 0;
	margin-right: 0;
}

.sapMPlanCal > .sapMList .sapMListInfoTBar > .sapMBarChild.sapMPlanCalHead {
	margin-right: 0;
}

.sapMPlanCal > .sapMList .sapMListInfoTBar > .sapUiCalRow {
	padding-left: 0;
	padding-right: 0;
	padding-bottom: 0;
}

.sapMPlanCal > .sapMList .sapMListInfoTBar > .sapUiCalRow > .sapUiCalContent >.sapUiCalContentOver {
	bottom: 0;
}

.sapMPlanCal > .sapMList .sapMListTblCell.sapMPlanCalRowHead,
.sapMPlanCal > .sapMList .sapMListTblCell.sapMPlanCalAppRow {
	padding: 0;
}

/* Responsiveness */
.sapMPlanCal > .sapMList > table > tbody > tr > td > .sapMListTblSubCnt,
.sapMPlanCal > .sapMList > table > tbody > .sapMListTblSubRow > td {
	padding: 0;
}

.sapMPlanCal .sapMListTblSubCntRow {
	display: block;
}

.sapMPlanCal > .sapMList > table > tbody > tr > td > .sapMListTblSubCnt > .sapMListTblSubCntRow >.sapMListTblSubCntVal {
	margin: 0;
}

.sapMPlanCal .sapMSticky > .sapMTB-Transparent-CTX,
.sapMPlanCal .sapMSticky > .sapMListInfoTBarContainer {
	background-color: var(--sapList_Background);
}

.sapMPlanCal.sapMPlanCalNoHead > .sapMList .sapMListInfoTBar,
.sapMSize0.sapMPlanCal > .sapMList .sapMListInfoTBar,
.sapMSize1.sapMPlanCal > .sapMList .sapMListInfoTBar {
	display: block;
}

.sapMSize0.sapMPlanCal.sapPCRelative > .sapMList .sapMListInfoTBar,
.sapMSize1.sapMPlanCal.sapPCRelative > .sapMList .sapMListInfoTBar {
	display: flex;
}

.sapMPlanCal.sapMPlanCalNoHead > .sapMList .sapMListInfoTBar > .sapMPlanCalHead,
.sapMPlanCal.sapMPlanCalNoHead > .sapMList .sapMListInfoTBar > .sapUiCalRow,
.sapMSize0.sapMPlanCal > .sapMList .sapMListInfoTBar > .sapMPlanCalHead,
.sapMSize0.sapMPlanCal > .sapMList .sapMListInfoTBar > .sapUiCalRow,
.sapMSize1.sapMPlanCal > .sapMList .sapMListInfoTBar > .sapMPlanCalHead,
.sapMSize1.sapMPlanCal > .sapMList .sapMListInfoTBar > .sapUiCalRow {
	width: 100%;
}

.sapMSize0.sapMPlanCal.sapPCRelative > .sapMList .sapMListInfoTBar > .sapMPlanCalHead,
.sapMSize1.sapMPlanCal.sapPCRelative > .sapMList .sapMListInfoTBar > .sapMPlanCalHead {
	width: 20%;
}

.sapMSize0.sapMPlanCal.sapPCRelative > .sapMList .sapMListInfoTBar > .sapUiCalRow,
.sapMSize1.sapMPlanCal.sapPCRelative > .sapMList .sapMListInfoTBar > .sapUiCalRow {
	width: 80%;
}

.sapMPlanCal.sapMPlanCalNoHead > .sapMList .sapMListInfoTBar > .sapMPlanCalHead,
.sapMSize0.sapMPlanCal > .sapMList .sapMListInfoTBar > .sapMPlanCalHead,
.sapMSize1.sapMPlanCal > .sapMList .sapMListInfoTBar > .sapMPlanCalHead {
	height: auto;
}

.sapMPlanCal.sapMPlanCalNoHead.sapMPlanCalMultiSel > .sapMList .sapMListInfoTBar > .sapUiCalRow {
	width: ~'calc(100% - 3rem)';
}

.sapMPlanCal.sapMPlanCalNoHead.sapMPlanCalMultiSel > .sapMList .sapMListInfoTBar > .sapMCb {
	width: 3rem;
	margin: 0;
	vertical-align: bottom;
}

.sapMPlanCal.sapMPlanCalNoHead.sapMPlanCalMultiSel > .sapMList .sapMListInfoTBar > .sapMCb > .sapMCbLabel {
	visibility: hidden;
}

.sapUiSizeCompact .sapMPlanCal.sapMPlanCalNoHead.sapMPlanCalMultiSel > .sapMList .sapMListInfoTBar > .sapMCb {
	width: 2.25rem;
	margin-left: 0.25rem;
}

.sapMSize0.sapMPlanCal.sapMPlanCalMultiSel > .sapMList .sapMListInfoTBar > .sapMCb,
.sapMSize1.sapMPlanCal.sapMPlanCalMultiSel > .sapMList .sapMListInfoTBar > .sapMCb {
	width: auto;
	margin-left: 0;
}

.sapMSize0.sapMPlanCal.sapMPlanCalMultiSel > .sapMList .sapMListInfoTBar > .sapMCb > .sapMCbLabel,
.sapMSize1.sapMPlanCal.sapMPlanCalMultiSel > .sapMList .sapMListInfoTBar > .sapMCb > .sapMCbLabel {
	visibility: visible;
}

.sapMSize0.sapMPlanCal.sapMPlanCalMultiSel > .sapMList .sapMListInfoTBar > .sapUiCalRow,
.sapMSize1.sapMPlanCal.sapMPlanCalMultiSel > .sapMList .sapMListInfoTBar > .sapUiCalRow {
	width: 100%;
	display: block;
}
.sapMPlanCal .sapMListBGTransparent .sapMLIB:not(.sapMLIBActive):not(.sapMLIBSelected):not(:hover),
.sapMPlanCal .sapMListBGTranslucent .sapMLIB:not(.sapMLIBActive):not(.sapMLIBSelected):not(:hover),
.sapMPlanCal .sapMListBGSolid .sapMLIB:not(.sapMLIBActive):not(.sapMLIBSelected):not(:hover) {
	background: transparent;
}

.sapMPlanCal {

	.sapMListTbl .sapMSLIImgIcon.sapUiIcon:not(.sapMLIBImgNav):not(.sapMBtnIcon) {
		font-size: 1.375rem;
		height: 3rem;
		width: 3rem;
		line-height: 3rem;
		margin: 0 0.75rem 0 0;
	}

	.sapMPlanCalRowHead {

		.sapMLIBContent {
			position: relative;
		}

		:not(li) .sapMLIBContent {
			padding: .5rem 0;
		}

		.sapMSLIImg {
			max-width: 3rem;
			max-height: 3rem;
			width: auto;
			height: auto;
			margin: 0;
		}

		.sapMSLIDiv {

			.sapMSLITitleOnly,
			.sapMSLITitle {
				white-space: normal;
			}
		}

		.sapMSLIThumbnail:not(.sapMSLIWrapping) > .sapMLIBContent > .sapMSLIDiv,
		.sapMSLIImgIcon + .sapMSLIDiv {
			box-sizing: border-box;
			position: absolute;
			padding-left: calc(3rem + 0.75rem); /* Width of the image + right margin */
			width: 100%;
		}

		.sapMCLI .sapMObjLItem { /* row header in custom content scenario */
			display: revert;
			padding: 0;
			border-bottom: 0;
		}
	}

	.sapUiCalTimesRow,
	.sapUiCalDatesRow,
	.sapUiCalMonthsRow {
		flex: 1;
	}

	.sapUiCalendarAppArrowIconLeft,
	.sapUiCalendarAppArrowIconRight,
	.sapUiCalendarRowAppsIntHeadIcon {
		position: relative;
	}

	.sapUiCalendarAppContainer {
		display: flex;
		align-items: center;
	}

	.sapUiCalendarAppContainerRight {
		flex: 1;
	}
}

.sapMPlanCal .sapUiCalHead {
	display: flex;
	justify-content: center;

	.sapUiCalHeadPrev,
	.sapUiCalHeadNext {
		position: static;
	}
}

.sapUiCalAppResizeGhost {
	position: absolute;
	top: 0;
	left: 0;
	display: inline-block;
	width: 1px;
	height: 1px;
	visibility: hidden;
	opacity: 0.01;
}

.sapUiDnDIndicatorHide {
	display: none !important;
}

.sapMPCSticky {

	.sapMPCHead {
		top: 0;
	}
}

.sapMPlanCal .sapUiCalMonthView .sapUiCalWeekNum {
	text-align: center;
}

.sapMPlanCal .sapMListTblCell.sapMPlanCalRowHead {
	vertical-align: top;
}

.sapMPlanCal .sapMIBar.sapMTB .sapMBarChild ~ .sapUiCalMonthView.sapMBarChild {
	margin: 0;
}

.sapMPlanCal.sapMPlanCalMultiSel {

	&.sapMSize0,
	&.sapMSize1 {

		.sapMListInfoTBarContainer .sapMCb {
			display: none;
		}
	}
}
/* END "PlanningCalendar.less" */

/* START "PlanningCalendarHeader.less" */
/* ============================================= */
/* CSS for control sap.m/PlanningCalendarHeader  */
/* Base theme                                    */
/* ============================================= */

@_sap_m_PlanningCalendarHeader_Background: var(--sapList_Background);

.sapMPCHead .sapMPCHeadActionsToolbar,
.sapMPCHead .sapMPCHeadNavToolbar,
.sapMPCHead .sapMTB.sapMTB-Transparent-CTX {
	padding-left: 1rem;
	background-color: @_sap_m_PlanningCalendarHeader_Background;
}

.sapMPCHead .sapMPCHeadNavToolbar {
	border-bottom: none;
	flex-wrap: wrap;
	white-space: normal;
	min-height: 3rem;

	&.sapMIBar {
		height: auto;
	}
}

.sapUiSizeCompact .sapMPCHead {

	.sapMPCHeadActionsToolbar {
		min-height: 3rem;
	}

	.sapMPCHeadNavToolbar {
		min-height: 2rem;
	}
}

.sapMIBar.sapMTB .sapMBarChild.sapUiCalMonthView {
	margin-left: 0;
}

html.sap-phone:not(.sapUiMedia-Std-Tablet),
html.sapUiMedia-Std-Phone:not(.sapUiMedia-Std-Tablet) {

	.sapMPCHeadNavToolbar {
		padding: 0 0 0 0.125rem;

		.sapMBarChild,
		.sapMBarChild ~ .sapMBarChild {
			margin: 0 0.5rem 0 0;

			&:last-of-type {
				margin: 0;
			}
		}

		.sapMBtn {
			max-width: 100%;

			bdi {
				overflow: inherit;
				text-overflow: inherit;
			}
		}
	}
}
/* END "PlanningCalendarHeader.less" */

/* START "PlanningCalendarLegend.less" */
/* ============================================= */
/* CSS for control sap.m/PlanningCalendarLegend  */
/* Base theme                                    */
/* ============================================= */

.sapMPlanCalLegend {

	&.sapUiUnifiedLegend hr {
		border-color: var(--sapList_TableGroupHeaderBorderColor);
	}

	.sapUiUnifiedLegendSquareColor.sapMPlanCalLegendAppCircle {
		border-radius: 1rem;
	}
}

.sapMPlanCalLegendHeader {
	color: var(--sapList_TableGroupHeaderTextColor);
	font-size: @sapMFontMediumSize;
	padding-top: 0.5rem;
}
/* END "PlanningCalendarLegend.less" */

/* START "Popover.less" */
/* ============================== */
/* CSS for control sap.m/Popover  */
/* Base theme                     */
/* ============================== */

 /* Buffer width for the scroll bar */
@_sap_m_Popover_Scroll_Width_Buffer: 20px;
@_sap_m_Popover_ArrowOffset: 0.5rem; /* Arrow offset from the Popover's corners */
@_sap_m_Popover_CompactArrowOffset: @_sap_m_Popover_ArrowOffset; /* Arrow offset from the Popover's corners in compact mode */
@_sap_m_Popover_ThickShadowSize: 0.0625rem;
@_sap_m_Popover_OpacityTransitionDuration: 0.2s;

/* shared mixin for arrow shadow*/
.sapMPopoverArrAfterShadow(@horizontalShadow, @verticalShadow) {
	box-shadow: @horizontalShadow @verticalShadow 0.750rem 0 fade(@sapUiContentShadowColor, 30), 0 0 0.125rem 0 fade(@sapUiContentShadowColor, 30);
}

.sapMPopover {

	.sapContrastPlus.sapMBar,
	.sapContrastPlus .sapMBar,
	.sapContrast.sapMBar,
	.sapContrast .sapMBar {
		box-shadow: none;
	}
}

.sapMPopover.sapMPopoverVerticalScrollIncluded {

	.sapMPopoverScroll {
		width: calc(~'100% + 20px');
	}

	html[data-sap-ui-browser^='sf'] & .sapMDialogSection {
		/* For Safari - allow the vertical scroll bar to overlap the right padding which we use to compensate for its width. This happens by default for other browsers. */
		overflow-y: overlay !important;
	}
}

.sapMPopoverWrapper {
	overflow: hidden;
	border-radius: inherit;
	display: flex;
	flex-direction: column;
	height: 100%;
}

.sapMPopoverCont::-webkit-scrollbar {
	border-bottom-right-radius: var(--sapPopover_BorderCornerRadius);
	border-top-right-radius: var(--sapPopover_BorderCornerRadius);
}

.sapMPopover .sapMIBar.sapMFooter-CTX ,
.sapMPopover.sapMPopoverWithoutFooter .sapMPopoverCont {
	border-bottom-right-radius: 0.25rem;
	border-bottom-left-radius: 0.25rem;
}

.sapMPopover .sapMPopoverCont {
	box-sizing: border-box;
}

.sapMPopoverWithoutSubHeader .sapMPopoverHeader .sapMIBar.sapMHeader-CTX,
.sapMPopoverSubHeader .sapMIBar.sapMSubHeader-CTX {
	background-color: var(--sapGroup_ContentBackground);
	box-shadow: var(--sapContent_HeaderShadow);
}

.sapMPopoverWithSubHeader .sapMPopoverHeader .sapMIBar.sapMHeader-CTX {
	box-shadow: none;
}

.sapMPopover .sapMIBar.sapMSubHeader-CTX {
	background-color: var(--sapGroup_ContentBackground);
}

.sapMPopover.sapMPopoverWithoutBar .sapMPopoverCont,
.sapMPopover .sapMIBar.sapMHeader-CTX {
	border-top-right-radius: 0.25rem;
	border-top-left-radius: 0.25rem;
}

.sapMPopoverHidden {
	visibility: hidden;
}

.sapMPopover {
	position: absolute;
	color: var(--sapContent_ForegroundTextColor);
	min-width: 6.25rem;
	box-sizing: border-box;
	outline: none;
	max-width: 100%;
	max-height: 100%;
	background: var(--sapGroup_ContentBackground);
	border: none;
	box-shadow: var(--sapContent_Shadow2);
	border-radius: 0.25rem;
	min-height: 2rem;
}

.sapMPopoverHeader .sapMTitle {
	font-size: var(--sapFontHeader5Size);
}

.sapMPopover .sapMPopoverCont {
	overflow: hidden;
	position: relative;
	/* set margin to every side */
	margin: 0.4375em;
	background-color: var(--sapGroup_ContentBackground);
}

.sapMPopover .sapMPopoverScroll {
	/* width: 100%;  when set width to 100%, the horizontal scrolling will be disabled and this is done in the end of setArrowPosition method */
	display: inline-block;
	vertical-align: middle;
	min-width: 100%;
	box-sizing: border-box;
}

.sapMPopover.sapUiPopupWithPadding .sapMPopoverScroll {
	padding: 1rem;
}

/* disable scrolling automatically for sap.m.Page sap.m.NavContainer sap.m.ScrollingContainer inside Popover */
.sapMPopoverVerScrollDisabled .sapMPopoverScroll {
	max-height: inherit;
	height: 100%;
}

/* disable scrolling automatically for sap.m.Page sap.m.NavContainer sap.m.ScrollingContainer inside Popover */
.sapMPopoverHorScrollDisabled .sapMPopoverScroll {
	display: block;
}

.sapMPopoverCont:focus {
	outline: none;
}

html[data-sap-ui-animation='on'] .sapMPopover.sapMPopoverOpacityTransition {
	transition-property: opacity;
	transition-timing-function: linear;
	transition-duration: @_sap_m_Popover_OpacityTransitionDuration;
}

/* disable margin according to the header and footer availability */
.sapMPopoverWithBar.sapMPopoverWithFooter > .sapMPopoverCont {
	margin-bottom: 0;
	margin-top: 0;
}

/* disable margin according to the header and footer availability */
.sapMPopoverWithoutBar.sapMPopoverWithFooter > .sapMPopoverCont {
	margin-bottom: 0;
}

/* disable margin according to the header and footer availability */
.sapMPopoverWithBar.sapMPopoverWithoutFooter > .sapMPopoverCont {
	margin-top: 0;
}

.sapMPopoverWithoutBar.sapMPopoverNav > .sapMPopoverCont,
.sapMPopoverWithoutBar.sapMPopoverPage > .sapMPopoverCont,
.sapMActionSheetPopover > .sapMPopoverCont {
	margin: 0;
	background-color: transparent;
}

.sapMPopoverHeader > .sapMBar,
.sapMPopoverSubHeader > .sapMBar,
.sapMPopoverFooter > .sapMBar {
	width: auto;
}

.sapMPopoverWithoutBar.sapMPopoverNav .sapMPopoverCont .sapMPage {
	background-color: transparent;
}

.sapMPopover > .sapMPopoverCont .sapMPage > section {
	box-sizing: border-box;
	width: auto;
}

.sapMPopoverWithoutBar .sapMPage .sapMPopoverHeader,
.sapMPopoverWithoutBar .sapMPage .sapMPopoverSubHeader,
.sapMPopoverWithoutBar .sapMPage .sapMPopoverFooter {
	position: absolute;
}

.sapMPopoverHiddenFocusable {
	width: 0;
	height: 0;
	font-size: 0;
	line-height: 0;
}

/* Adds a hidden layer below the Popover arrow to prevent closing the Popover when clicking on the arrow. In the past */
/* it was working but the area was bigger you could not click near to the arrow */
.sapMPopoverArr {
	pointer-events: none;
	display: block;
	width: 1rem;
	height: 1rem;
	position: absolute;
	overflow: hidden;

	&::after {
		content: ' ';
		display: block;
		width: 0.7rem;
		height: 0.7rem;
		background-color: var(--sapGroup_ContentBackground);
		transform: rotate(-45deg);
	}
}

html[data-sap-ui-browser^='sf'] .sapMPopover {
	/* scale(1) - Chrome does not clip children when transform is not applied resulting in non-visible border-radius */
	/* translateZ(0px) - Safari Problem with z-index - After dialog was opened it was inserted behind the dark overlay */
	-webkit-transform: scale(1) translateZ(0px);
}

.sapMPopoverWithFooter .sapMPopoverArrDown::after {
	background-color: var(--sapPageFooter_Background);
}

/* pointing upward arrow */
.sapMPopoverArrUp {
	/* offset */
	left: 20px;
	/* position */
	top: -@_sap_m_Popover_ArrowOffset;
	height: @_sap_m_Popover_ArrowOffset;

	&::after {
		margin: 0.1875rem 0 0 0.1875rem;

		.sapMPopoverArrAfterShadow(-0.375rem, 0.375rem);
	}
}

/* pointing right arrow */
.sapMPopoverArrRight {
	/* offset */
	top: 1rem;
	/* position */
	right: -@_sap_m_Popover_ArrowOffset;
	width: @_sap_m_Popover_ArrowOffset;

	&::after {
		margin: 0.1875rem 0 0 -0.375rem;

		.sapMPopoverArrAfterShadow(-0.375rem, -0.375rem);
	}
}

/* pointing downward arrow */
.sapMPopoverArrDown {
	/* offset */
	left: 1rem;
	/* position */
	height: @_sap_m_Popover_ArrowOffset;
	top: 100%;

	&::after {
		margin: -0.375rem 0 0 0.125rem;

		.sapMPopoverArrAfterShadow(0.375rem, -0.375rem);
	}
}

/* pointing left arrow */
.sapMPopoverArrLeft {
	/* position */
	left: -@_sap_m_Popover_ArrowOffset;
	top: -2rem;
	width: @_sap_m_Popover_ArrowOffset;
	height: 1rem;

	&::after {
		margin: 0.125rem 0 0 0.25rem;

		.sapMPopoverArrAfterShadow(0.375rem, 0.375rem);
	}
}

.sapMPopover.sapUiSizeCompact {

	.sapMPopoverArrLeft::before,
	.sapMPopoverArrRight::before {
		margin-top: -.5rem;
		margin-left: .25rem;
		width: 1rem;
		height: 1rem;
	}

	.sapMPopoverArrRight::before {
		margin-left: -1.25rem;
	}

	.sapMPopoverArrUp::before,
	.sapMPopoverArrDown::before {
		width: 1rem;
		height: 1rem;
		margin-left: -.5rem;
	}

	.sapMPopoverArrDown::before {
		margin-top: -1.25rem;
	}
}

.sapMPopover .sapUiRespGrid {
	width: 100%;
}

/* mixin call for creating container content padding classes (arguments: rootContainer, contentSelector) */
.sapUiContainerContentPadding(~".sapMPopover", ~".sapMPopoverCont > .sapMPopoverScroll");

.sapMPopoverResizeHandle {
	position: absolute;
	width: 1.5rem;
	height: 1.5rem;
	border-radius: 50%;
	z-index: 10;

	.sapMPopoverResizeHandleIcon {
		position: absolute;
		width: 1rem;
		height: 1rem;
		cursor: inherit;
	}
}

.sapMPopoverResizeHandleTopRight {

	.sapMPopoverResizeHandle {
		top: -0.5rem;
		right: -0.5rem;
		cursor: ne-resize;

		.sapMPopoverResizeHandleIcon {
			bottom: 0;
			left: 0;
			transform: rotate(270deg);
		}
	}
}

.sapMPopoverResizeHandleTopLeft {

	.sapMPopoverResizeHandle {
		top: -0.5rem;
		left: -0.5rem;
		cursor: nw-resize;

		.sapMPopoverResizeHandleIcon {
			bottom: 0;
			right: 0;
			transform: rotate(180deg);
		}
	}
}

.sapMPopoverResizeHandleBottomLeft {

	.sapMPopoverResizeHandle {
		bottom: -0.5rem;
		left: -0.5rem;
		cursor: ne-resize;

		.sapMPopoverResizeHandleIcon {
			top: 0;
			right: 0;
			transform: rotate(90deg);
		}
	}
}

.sapMPopoverResizeHandleBottomRight {

	.sapMPopoverResizeHandle {
		bottom: -0.5rem;
		right: -0.5rem;
		cursor: nw-resize;

		.sapMPopoverResizeHandleIcon {
			top: 0;
			left: 0;
		}
	}
}

.sapMPopoverResizing,
.sapMPopoverResizing * {
	-moz-user-select: none !important;
	-webkit-user-select: none !important;
	user-select: none !important;
}

.sapMPopover .sapMPopoverCont {
	background-color: var(--sapGroup_ContentBackground);
	margin: 0;
}

/* padding for title in header is removed because of ticket 0120031469 0004432742 2013 */
.sapMPopoverHeader .sapMBarMiddle > .sapMBarPH {
	box-sizing: border-box;
}

.sapMPopoverHeader .sapMBarMiddle > .sapMBarPH > .sapMLabel {
	display: inline-block;
}

/* for only two buttons in footer's contentMiddle aggregation */
.sapMPopoverSpecialFooter .sapMBarMiddle {
	display: block;
	height: 100%;
	/*padding: 0 0.25rem; */
	box-sizing: border-box;
}

.sapMPopoverSpecialFooter .sapMBarMiddle > .sapMBarPH {
	display: block;
	padding: 0;
}

.sapMPopoverSpecialFooter .sapMBarLeft,
.sapMPopoverSpecialFooter .sapMBarRight {
	display: none;
}

.sapMPopoverSpecialFooter .sapMBarMiddle > .sapMBarPH > .sapMBtn {
	width: 50%;
	padding-left: 0.25rem;
}

.sapMPopoverSpecialFooter .sapMBarMiddle > .sapMBarPH {

	.sapMBtn:last-child {
		margin-left: 0;
	}

	.sapMBtn:first-child {
		padding-right: 0.25rem;
		padding-left: 0;
	}
}

.sapMPopover.sapMPopoverVerticalScrollIncluded .sapMPopoverCont {
	padding-right: @_sap_m_Popover_Scroll_Width_Buffer;
}
/* END "Popover.less" */

/* START "ProgressIndicator.less" */
/* ======================================== */
/* CSS for control sap.m/ProgressIndicator  */
/* Base theme                               */
/* ======================================== */

@_sap_m_ProgressIndicator_PopoverCloseIconWidth: 1.4375rem;
@_sap_m_ProgressIndicator_PopoverCloseIconHeight: 1.375rem;
@_sap_m_ProgressIndicator_PopoverCloseIconMargin: 0.125rem;
@_sap_m_ProgressIndicator_TextLeftTextThreshold: @sapUiContentContrastTextThreshold;

.sapMPI {

	&.sapMPIHoverable:hover {
		cursor: pointer;
	}

	display: flex;
	flex-direction: row;
	box-sizing: border-box;
	outline: none;
	background: var(--sapField_Background);
	width: 100%;
	height: 1.5rem;
	min-width: 6rem;
	min-height: 1rem;
	padding: 0;
	margin: 0.75rem 0;
	border: none;
	border-radius: 0.5rem;
	overflow: hidden;
}

.sapMPIPopover {
	min-height: @_sap_m_ProgressIndicator_PopoverCloseIconHeight;

	.sapMPopoverScroll {
		padding-left: 0.5rem;
		display: flex !important;
		justify-content: space-between;
	}

	.sapMText {
		align-self: center;
		margin: 0.25rem 0;
		word-break: break-all;
	}

	.sapUiIcon {
		width: @_sap_m_ProgressIndicator_PopoverCloseIconWidth;
		height: @_sap_m_ProgressIndicator_PopoverCloseIconHeight;
		line-height: @_sap_m_ProgressIndicator_PopoverCloseIconHeight;
		font-size: var(--sapFontSmallSize);
		color: var(--sapContent_IconColor);
		flex-shrink: 0;
		margin: @_sap_m_ProgressIndicator_PopoverCloseIconMargin;
	}

	/* Additional touch area for the close icon/button required by ACC standards */
	.sapUiIcon::after {
		content: '';
		position: absolute;
		right: (@_sap_m_ProgressIndicator_PopoverCloseIconMargin * -1);
		top: (@_sap_m_ProgressIndicator_PopoverCloseIconMargin * -1);
		width: @_sap_m_ProgressIndicator_PopoverCloseIconWidth + (@_sap_m_ProgressIndicator_PopoverCloseIconMargin * 2);
		height: @_sap_m_ProgressIndicator_PopoverCloseIconHeight + (@_sap_m_ProgressIndicator_PopoverCloseIconMargin * 2);
	}
}

.sapMPIBar,
.sapMPIBarRemaining {
	display: flex;
	min-width: 0;
	align-items: center;
}

.sapMPIBar {
	height: 100%;
	justify-content: flex-end;
	flex-shrink: 0;
	border-top-left-radius: 0.5rem;
	border-bottom-left-radius: 0.5rem;
}

.sapMPIValueMax .sapMPIBarRemaining {
	display: none;
}

.sapMPIBarRemaining {
	flex-grow: 1;
	border-top-right-radius: 0.5rem;
	border-bottom-right-radius: 0.5rem;
	border-top: 1px solid var(--sapField_BorderColor);
	border-right: 1px solid var(--sapField_BorderColor);
	border-bottom: 1px solid var(--sapField_BorderColor);
}

.sapMPIBarDisabled {
	opacity: 0.5;
}

.sapMPI.sapMPIDisplayOnly {
	height: 1rem;
	min-width: 4rem;
	margin: 0;
}

.sapMPIText {
	font-family: var(--sapFontFamily);
	font-size: var(--sapFontSmallSize);
	font-weight: normal;
	text-overflow: ellipsis;
	overflow: hidden;
	white-space: nowrap;
	color: var(--sapContent_ForegroundTextColor);
}

.sapMPIDisplayOnly .sapMPIText {
	color: var(--sapTextColor);
	font-size: 0.6875rem;
}

.sapMPITextLeft {
	color: var(--sapContent_ContrastTextColor);
	padding-right: 0.5rem;
	display: none;
}

.sapMPITextRight {
	padding-left: 0.5rem;
}

.sapMPIValueGreaterHalf .sapMPITextRight {
	display: none;
}

.sapMPIValueGreaterHalf .sapMPITextLeft {
	display: inline-block;
}

/* Progress Indicator Bar */
.sapMPIValueMax .sapMPIBar {
	border-top-right-radius: 0.5rem;
	border-bottom-right-radius: 0.5rem;
}

.sapMPIValueNormal .sapMPIBar {
	max-width: calc(~"100% - 0.5rem");
	min-width: 0.5rem;
}

.sapMPIBarNeutral {
	background: var(--sapNeutralElementColor);
}

.sapMPIBarPositive {
	background: var(--sapPositiveElementColor);
}

.sapMPIBarNegative {
	background: var(--sapNegativeElementColor);
}

.sapMPIBarCritical {
	background: var(--sapCriticalElementColor);
}

.sapMPIBarInformation {
	background: var(--sapInformativeElementColor);
}

.sapMPIDisplayOnly:not(.sapMPIBarDisabled) .sapMPIBarNeutral {
	background: var(--sapTextColor);
}

/* Progress Indicator Remaining Bar */
.sapMPIValueMin .sapMPIBarRemaining {
	border-radius: 0.5rem;
	border: 1px solid var(--sapField_BorderColor);
}

/* Progress Indicator Value Text */
.sapMPIBarNeutral .sapMPIText.sapMPITextLeft {
	color: contrast(@sapUiNeutralElement, @sapUiContentContrastTextColor, @sapUiBaseText, @_sap_m_ProgressIndicator_TextLeftTextThreshold);
}

.sapMPIBarPositive .sapMPIText.sapMPITextLeft {
	color: contrast(@sapUiPositiveElement, @sapUiContentContrastTextColor, @sapUiBaseText, @_sap_m_ProgressIndicator_TextLeftTextThreshold);
}

.sapMPIBarNegative .sapMPIText.sapMPITextLeft {
	color: contrast(@sapUiNegativeElement, @sapUiContentContrastTextColor, @sapUiBaseText, @_sap_m_ProgressIndicator_TextLeftTextThreshold);
}

.sapMPIBarCritical .sapMPIText.sapMPITextLeft {
	color: contrast(@sapUiCriticalElement, @sapUiContentContrastTextColor, @sapUiBaseText, @_sap_m_ProgressIndicator_TextLeftTextThreshold);
}

.sapMPIBarInformation .sapMPIText.sapMPITextLeft {
	color: contrast(@sapUiHighlight, @sapUiContentContrastTextColor, @sapUiBaseText, @_sap_m_ProgressIndicator_TextLeftTextThreshold);
}

.sapMPIBarInformation.sapMPIDisplayOnly .sapMPIText.sapMPITextLeft {
	color: contrast(@sapUiBaseText, @sapUiContentContrastTextColor, @sapUiBaseText, @_sap_m_ProgressIndicator_TextLeftTextThreshold);
}

/* Compact */
.sapUiSizeCompact .sapMPI:not(.sapMPIDisplayOnly) {
	height: 1.125rem;
	margin: 0.4375rem 0;
}

/* Condensed */
.sapUiSizeCondensed .sapUiTableDataCell .sapMPI:not(.sapMPIDisplayOnly) {
	height: 1.125rem;
	margin: 0;
}
/* END "ProgressIndicator.less" */

/* START "PullToRefresh.less" */
/* ==================================== */
/* CSS for control sap.m/PullToRefresh  */
/* Base theme                           */
/* ==================================== */

.sapMPullDown {
	position: relative;
	padding: 0.6rem 1rem 0;
	font-size: 0.875rem;
	overflow: hidden;
	cursor: pointer;
	color: var(--sapContent_LabelColor);
	background-color: var(--sapGroup_ContentBackground);
}

.sapMPullDownNontouch {
	min-height: 2rem;
	padding-bottom: 0.1875rem;
	padding-left: 0.8125rem;
	white-space: nowrap;
	text-overflow: ellipsis;
}

.sapMPullDownNontouch:focus {
	outline: 0.0625rem dotted var(--sapContent_FocusColor);
	outline-offset: -0.1875rem;
}

.sapMPullDownTouch {
	min-height: 3rem;
	border-bottom: 1px solid var(--sapGroup_ContentBorderColor);
}

/* make place for the busy indicator */
.sapMPullDownTouch.sapMLoading:not(.sapMPullDownLogo) {
	padding-left: 4rem;
}

.sapMPullDownTouch.sapMPullDownLogo {
	padding-top: 4.125rem;
	height: 3.25rem;
	line-height: 3.75rem;
	background-image: none;
	background-repeat: no-repeat;
	background-position: center 1rem;
	background-size: 5.5rem 2.75rem;
}

.sapMPullDownCI {
	display: block;
	width: 100%;
	height: 3.125rem;
	overflow: hidden;
	margin: 0 auto;
	text-align: center;
}

.sapMPullDownCIImg {
	max-height: 3.125rem;
}

/* Text */

.sapMPullDownTouch > .sapMPullDownText {
	font-weight: normal;
	font-size: @sapMFontMediumSize;
	color: var(--sapList_TextColor);
}

.sapMPullDownTouch > .sapMPullDownText,
.sapMPullDownTouch > .sapMPullDownInfo {
	width: 100%;
	display: block;
	text-align: center;
	line-height: 1.3rem;
	box-sizing: border-box;
	font-family: var(--sapFontFamily);
}

.sapMPullDownNontouch > .sapMPullDownText {
	margin: 0 0.5rem;
	text-transform: uppercase;
	vertical-align: middle;
}

.sapMPullDownInfo {
	vertical-align: middle;
	font-size: var(--sapFontSmallSize);
	color: var(--sapContent_LabelColor);
}

.sapMPullDownNontouch > .sapMPullDownIcon { /* circle arrows */
	vertical-align: middle;
}

.sapMPullDownNontouch > .sapMPullDownIcon::before {
	width: 1.5rem;
	height: 1.5rem;
	font-size: 1.125rem;
	content: '\e010';
}

/* Icon */

.sapMPullDown > .sapMPullDownIcon::before {
	display: inline-block;
	font-family: var(--sapContent_IconFontFamily);
	speak: none;
	float: left;
}

.sapMPullDown.sapMLoading .sapMPullDownIcon::before {
	visibility: hidden;
}

.sapMPullDownTouch > .sapMPullDownIcon {
	position: absolute;
	left: 1rem;
	line-height: 1rem;
	bottom: 1.5rem;
}

.sapMPullDownTouch > .sapMPullDownIcon::before { /* animated arrow */
	font-size: 2em;
	color: var(--sapHighlightColor);
	content: '\e089';
	transform-origin: center center;
	transition-duration: 250ms;
	transition-property: transform;
	transform: rotate(0deg);
}

.sapMPullDownTouch.sapMFlip .sapMPullDownIcon::before {
	transform: rotate(-180deg);
}

/* BusyIndicator */

.sapMPullDown .sapMPullDownBusy {
	display: none;
	position: absolute;
	left: 1rem;
	line-height: 1rem;
	bottom: 1rem;
}

.sapMPullDownNontouch .sapMPullDownBusy {
	bottom: 0.5rem;
}

.sapMPullDown.sapMLoading .sapMPullDownBusy {
	display: block;
}
/* END "PullToRefresh.less" */

/* START "QuickView.less" */
/* ================================ */
/* CSS for control sap.m/QuickView  */
/* Base theme                       */
/* ================================ */

@_sap_m_QuickView_HorizontalLayoutMarginBottom: 2rem;
@_sap_m_QuickView_HorizontalLayoutIconFontSize: 2.25rem;
@_sap_m_QuickView_PagePaddingTop: 3rem;
@_sap_m_QuickView_TitleMaxHeight: 2.25rem;

@_sap_m_QuickViewCard_PagePadding: 0;

@_sap_m_QuickView_TitleMarginBottom: 0.5rem;
@_sap_m_QuickView_TitleFontSize: var(--sapFontHeader5Size);
@_sap_m_QuickView_FormTitleFontSize: var(--sapFontHeader5Size);

.sapMQuickView .sapMNav,
.sapMQuickViewCard .sapMNav {
	white-space: nowrap;
}

.sapMQuickViewCard .sapUiHLayoutChildWrapper a {
	margin-right: 1px; /* this is needed for the focus to be visible on the right side (BCP 1980423560) */
}

.sapMQuickViewCard.sapMQuickViewCardNoScroll {
	height: 100%;
}

.sapMQuickViewPage {

	.sapUiHLayout.sapUiHLayoutNoWrap .sapUiHLayoutChildWrapper {
		max-width: 100%;
	}

	&.sapMNavItem {
		vertical-align: top;
		display: inline-block;

		&.sapMNavItemHidden {
			display: none;
		}

		&.sapMNavItemOffset {
			left: -100%;
		}
	}
}

.sapMQuickViewCard .sapMQuickViewPage.sapMNavItem {
	width: 100% !important;
}

html.sap-desktop .sapMQuickViewPage .sapUiForm .sapUiFormResGrid > div,
html.sap-tablet .sapMQuickViewPage .sapUiForm .sapUiFormResGrid > div {
	padding: 0;
	margin: 0;
}

.sapMQuickView.sapMPopover .sapMQuickViewPage.sapMScrollContScroll,
.sapMQuickView.sapMPopover .sapMQuickViewPage.sapMPage {
	position: relative;
	height: 100%;
	box-sizing: border-box;
	padding-top: @_sap_m_QuickView_PagePaddingTop;
}

.sapMQuickView.sapMPopover .sapMQuickViewPage.sapMScrollContScroll,
.sapMQuickView.sapMPopover .sapMQuickViewPage.sapMPage.sapMQuickViewPageWithoutHeader,
.sapUiSizeCompact .sapMQuickView.sapMPopover .sapMQuickViewPage.sapMScrollContScroll,
.sapUiSizeCompact .sapMQuickView.sapMPopover .sapMQuickViewPage.sapMPage.sapMQuickViewPageWithoutHeader {
	padding-top: 0;
}

.sapUiSizeCompact .sapMQuickView.sapMPopover .sapMQuickViewPage.sapMScrollContScroll,
.sapUiSizeCompact .sapMQuickView.sapMPopover .sapMQuickViewPage.sapMPage {
	padding-top: 2.5rem;
}

html.sap-phone .sapMQuickViewPage.sapMNavItem.sapMPage {
	position: relative;
}

html.sap-desktop .sapMQuickViewPage.sapMPage > header,
html.sap-tablet .sapMQuickViewPage.sapMPage > header {
	position: absolute;
	height: @_sap_m_QuickView_PagePaddingTop;
	top: 0;
	left: 0;
	right: 0;
	bottom: 0;
	font-size: 16px;
}

.sapUiSizeCompact .sapMQuickView.sapMPopover header.sapMBar {
	height: 2.5rem;
}

.sapMQuickViewPage.sapMPage > header .sapMBarRight .sapUiIcon {
	font-size: 1rem;
}

html.sap-desktop .sapMQuickViewPage > section,
html.sap-tablet .sapMQuickViewPage > section {
	position: static;
	height: 100%;
	padding: 1rem;
}

.sapMQuickViewCard .sapMQuickViewPage {
	padding: @_sap_m_QuickViewCard_PagePadding;
	box-sizing: border-box;

	.sapMTitle {
		font-size: @_sap_m_QuickView_TitleFontSize;
	}
}

.sapMQuickViewPage .sapUiVltCell>.sapMTitle,
.sapMQuickViewPage .sapUiVltCell .sapMLnk:not(.sapMLnkWrapping) .sapMLnkText {
	display: -webkit-box;
	-webkit-box-orient: vertical;
	-webkit-line-clamp: 3;
	white-space: normal;
	word-wrap: break-word;
}

.sapMQuickViewPage .sapUiFormResGridCont > div {
	padding: 0;
}

.sapMQuickViewPage .sapUiVlt .sapMText {
	font-weight: normal;
}

.sapMQuickViewPage .sapUiFormResGrid.sapUiFormBackgrTranslucent > div {
	background-color: transparent;
}

.sapMQuickViewPage .sapUiSimpleForm .sapMTitle {
	margin-top: 1rem;
}

.sapMQuickViewPage .sapMFlexItem:first-child {
	width: 70%;
	display: inline-block;
}

.sapMQuickViewPage .sapMFlexItem:last-child {
	width: 30%;
	display: inline-block;
	font-family: SAP-icons;
	min-width: 3rem;
	text-align: right;
	left: -1px;
}

.sapMQuickViewPage .sapMQuickViewThumbnail {
	display: block;
	margin-right: 0.75rem;
}

.sapMQuickViewPage .sapUiHLayout {
	margin-bottom: @_sap_m_QuickView_HorizontalLayoutMarginBottom;
}

.sapMQuickViewPage .sapUiHLayout,
.sapMQuickViewPage .sapUiVlt {
	display: block;
}

.sapMQuickViewCard .sapUiHLayoutChildWrapper:only-child {
	width: 100%;
}

html.sap-phone .sapMQuickViewPage .sapUiHLayout {
	margin: 1rem 0 0 0.75rem;
}

.sapMQuickViewPage .sapUiHLayoutChildWrapper + .sapUiHLayoutChildWrapper:last-child {
	width: ~'calc(100% - 3.75rem)';
}

.sapMQuickViewPage .sapUiVltCell {
	display: flex;
}

.sapMQuickViewPage .sapUiVltCell:first-child {
	margin-bottom: @_sap_m_QuickView_TitleMarginBottom;
}

.sapMQuickViewPage .sapUiVltCell + .sapUiVltCell span {
	width: 100%;
}

.sapMQuickViewPage .sapUiSimpleForm .sapUiIcon {
	width: 100%;
	text-align: right;
	outline: none;
}

.sapMQuickViewPage .sapUiSimpleForm .sapUiIcon:before {
	margin-right: 3px;
}

.sapMQuickViewPage div[class*='sapUiRespGrid'] > div[class*='sapUiRespGridSpan'] {
	overflow: visible;
}

.sapMQuickViewPage .sapMIBar-CTX .sapMTitle {
	text-shadow: @sapUiShadowText;
}

html.sap-desktop .sapMQuickViewHeader,
html.sap-tablet .sapMQuickViewHeader {
	font-size: var(--sapFontHeader5Size);
}

.sapUiSizeCompact .sapMQuickView.sapMPopover .sapMQuickViewTitle {
	font-size: var(--sapFontHeader6Size);
}

.sapMQuickViewPage h3,
.sapMQuickViewPage .sapUiForm .sapUiFormTitle {
	font-family: var(--sapFontHeaderFamily);
	font-size: @_sap_m_QuickView_FormTitleFontSize;
	font-weight: @sapUiFontHeaderWeight;
	line-height: normal;
	color: var(--sapGroup_TitleTextColor);
	padding-left: 0;
}

.sapMQuickViewPage .sapUiVltCell .sapMLnk {
	font-family: var(--sapFontHeaderFamily);
	font-size: var(--sapFontHeader5Size);
	font-weight: @sapUiFontHeaderWeight;
	line-height: normal;
	white-space: normal;
	color: var(--sapLinkColor);

	&:active {
		color: var(--sapLink_Active_Color);
	}

	&:visited {
		color: var(--sapLink_Visited_Color);
	}

	&hover:not(.sapMLnkDsbl) {
		color: var(--sapLink_Hover_Color);
	}
}

.sapMQuickViewPage .sapUiHLayout h3 span {
	white-space: normal;
}

.sapMQuickViewPage .sapUiVlt .sapMText {
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	color: var(--sapContent_LabelColor);
}

.sapMQuickViewPage .sapMFlexItem:last-child {
	color: var(--sapContent_IconColor);
}

.sapMQuickViewPage .sapUiIcon:focus,
html.sap-phone .sapMQuickViewPage .sapMLnk:focus {
	outline: none;
}

html.sap-phone .sapMPanel .sapMQuickViewCard {
	overflow-y: scroll;
}

html.sap-tablet.sap-ios .sapMPanel .sapMQuickViewCard {
	-webkit-overflow-scrolling: auto;
}

.sapMQuickViewPage .sapMPageEnableScrolling {
	outline: none;
}

.sapMQuickView .sapMBar {
	padding-left: 0.75rem;
}

/* Compact size */
.sapUiSizeCompact {

	.sapMQuickViewPage.sapMPage > header.sapMPageHeader {
		height: 2.5rem;
	}
}
/* END "QuickView.less" */

/* START "RadioButton.less" */
/* ================================== */
/* CSS for control sap.m/RadioButton  */
/* Base theme                         */
/* ================================== */

@_sap_m_Radio_Button_Read_Only_fill: var(--sapContent_NonInteractiveIconColor);

.sapMRb {
	height: 3rem;
	float: left;
	clear: left;
	outline: none;
	overflow: hidden;
	text-overflow: ellipsis;
	position: relative;
	white-space: nowrap;
	max-width: 100%;
	display: flex;

	> .sapMRbBLabel:not(.sapMLabelNoText) {
		padding-right: 0.125rem; /* if this is changed, please change in RadioButtonGroup too for usage in Form */
	}
}

.sapMRb .sapMLabel.sapMRbBLabel {
	align-items: center;
	display: inline-flex;
}

.sapMRb.sapMRbWrapped {
	height: auto;
}

.sapMILI-CTX .sapMRb {
	float: right;
}
.sapMListTblCell > .sapMRb {
	float: inherit;
	display: inline-block;
	vertical-align: top;
}

.sapMRb .sapMRbB {
	height: 3rem;
	width: 3rem;
	display: inline-block;
	font-size: 1rem; /* override font size of the message dialog */

	&:focus {
		outline: none
	}

	.sapMRbBOut {
		height: 100%;
		width: 100%;
		transform: translate(0.675rem, 0.675rem);
		margin: 0.825rem;
		padding: 1px; /* using padding here so that margin won't collide */
		stroke: var(--sapField_BorderColor);
		fill: var(--sapField_Background);
	}
}

.sapUiSizeCompact {

	.sapMRb .sapMRbB {

		.sapMRbBOut {
			transform: translate(0.5rem, 0.5rem);
		}
	}
}

.sapMRbSvg {
	height: 1.375rem;
	width: 1.375rem;
	margin: 0.825rem;
	overflow: visible;
}

.sapMRbBLabel {
	vertical-align: top;
	height: 3rem;
	line-height: 3rem;
	cursor: default;
}

.sapMRb:not(.sapMRbDis) {

	&:hover {
		cursor: pointer;
	}

	.sapMRbBLabel {

		&:hover {
			cursor: pointer;
		}
	}
}

.sapMRbB input {
	margin: 0; /* FF puts margin */
	-webkit-appearance: none;
	visibility: hidden;
}

.sapMRb:focus::before {
	content: '';
	display: block;
	position: absolute;
	top: .625rem;
	bottom: .625rem;
	left: .625rem;
	right: .6875rem;
	pointer-events: none;
}

.sapMRbHasLabel.sapMRb.sapMRbWrapped:focus::before {
	bottom: 0.375rem;
	top: 0.375rem;
	left: 0.375rem;
	right: 0;
}

.sapMRbB .sapMRbBInn {
	transform: translate(0.675rem, 0.675rem);
	stroke: transparent;
	fill: none;
	-webkit-user-select: none;
	align-self: center;
	r: 22%;
}

.sapUiSizeCompact {

	.sapMRbBInn {
		transform: translate(0.5rem, 0.5rem);
	}
}

.sapMRbHasLabel.sapMRb:focus::before {
	right: 0;
}

/* Right to left */
.sapMRbG[dir=rtl] .sapMRbHasLabel.sapMRb:focus::before {
	left: 0;
	right: .6875rem;
}

.sapMRbDis:focus::before {
	display: none;
}

.sapMRbSel .sapMRbBInn {
	fill: var(--sapSelectedColor);
	stroke: none;
}

.sapMLIBActive .sapMRbSel .sapMRbBInn {
	fill: var(--sapList_Active_TextColor);
}

/* Read only */
.sapMRb.sapMRbRo {

	.sapMRbBOut {
		stroke: var(--sapField_ReadOnly_BorderColor);
		fill: var(--sapField_ReadOnly_Background);
	}

	&.sapMRbSel .sapMRbBInn {
		fill: @_sap_m_Radio_Button_Read_Only_fill;
		stroke: none;
	}
}

/* Align RadioButton in Form with other controls */
.sapUiForm.sapUiFormEdit-CTX {

	.sapMRb > .sapMRbB {
		width: 2.3rem;
	}

	.sapMRb > .sapMRbBLabel:not(.sapMLabelNoText) {
		padding-right: 0; /* space between controls is handled by Form */
	}

	.sapMRbSvg {
		margin-left: 0.125rem;
	}

	.sapMRb:focus::before {
		left: 0;
	}
}

/* Compact size */
.sapUiSizeCompact {

	.sapMRb {
		height: 2rem;
	}

	.sapMRb.sapMRbWrapped {
		height: auto;
	}

	.sapMRbSvg {
		height: 1rem;
		width: 1rem;
		display: inline-block;
		overflow: visible;
		margin: 0.5rem;
	}

	.sapMRb:focus::before {
		top: .325rem;
		bottom: .325rem;
		left: .325rem;
		right: .3875rem;
	}

	.sapMRbB {
		height: 2rem;
		width: 2rem;
		font-size: 1rem; /* override font size of the message dialog */

		.sapMRbBOut {
			height: 100%;
			width: 100%;
			border-width: 0.125rem;
			margin: 0.5rem;
			box-sizing: border-box;
		}
	}

	.sapMRbB .sapMRbBInn {
		height: 0.375rem;
		width: 0.375rem;
	}

	.sapMRbBLabel {
		line-height: 2rem;
		height: 2rem;
	}

	.sapMRb > .sapMRbBLabel.sapMLabelWrapped {
		line-height: normal;
		height: auto;
		padding-block: 0.5rem;
		padding-inline-end: 1px;
	}

	.sapMRbHasLabel.sapMRb:focus::before {
		right: 0;
	}

	.sapMRbHasLabel.sapMRb.sapMRbWrapped:focus::before {
		bottom: 0;
		top: 0.25rem;
		left: 0.25rem;
	}

	/* Right to left */
	.sapMRbG[dir=rtl] .sapMRbHasLabel.sapMRb:focus::before  {
		left: 0;
		right: .3875rem;
	}

	.sapUiForm.sapUiFormEdit-CTX {

		.sapMRb > .sapMRbB {
			width: 1.625rem;
		}

		.sapMRbSvg {
			margin-left: 0.125rem;
		}

		.sapMRb:focus::before {
			left: 0;
		}
	}
}

html[dir='rtl'] {

	.sapMRbB .sapMRbBInn,
	.sapMRb .sapMRbB .sapMRbBOut {
		transform: translate(-0.675rem, 0.675rem);
	}

	.sapUiSizeCompact {

		.sapMRbB .sapMRbBInn,
		.sapMRb .sapMRbB .sapMRbBOut {
			transform: translate(-0.5rem, 0.5rem);
		}
	}
}

/* Condensed size */
.sapUiSizeCondensed.sapUiTable,
.sapUiSizeCondensed .sapUiTable {

	.sapMRb {
		height: 1.5rem;
		vertical-align: top;
	}

	.sapMRbB {
		font-size: 1rem; /* override font size of the message dialog */
		height: 1.375rem;
		width: 1.375rem;

		.sapMRbBOut {
			height: 0.75rem;
			width: 0.75rem;
			border-width: 1px;
			margin: 0.375rem;
		}

		.sapMRbBInn {
			height: 0.25rem;
			width: 0.25rem;
			margin: 0.0625rem;
		}
	}

	.sapUiTableDataCell > .sapUiTableCellInner > .sapMRb > .sapMRbB {
		margin: 0 0.25rem; /* extra space in the condensed ULV */
	}

	.sapMRbBLabel {
		line-height: 1.5rem;
		height: 1.5rem;
	}

	.sapMRb > .sapMRbBLabel.sapMLabelWrapped {
		line-height: normal;
		height: auto;
	}
}

.sapMRb > .sapMRbBLabel.sapMLabelWrapped {
	line-height: normal;
	display: flex;
	height: auto;
	padding-block: 0.75rem;
	padding-inline-end: 1px;
}
/* END "RadioButton.less" */

/* START "RadioButtonGroup.less" */
/* ======================================= */
/* CSS for control sap.m/RadioButtonGroup  */
/* Base theme                              */
/* ======================================= */

@_sap_m_RadioButtonGroup_WidthWithLabel: 3rem;
@_sap_m_RadioButtonGroup_PaddingLeftLabel: 3rem;
@_sap_m_RadioButtonGroup_PaddingLeftLabel_Form: 2.3rem;

.sapMRbG{
	display: inline-block;
}

.sapMRbG:focus {
	outline: none;
}

.sapMRbGTab{
	display: -webkit-box;
	display: -webkit-flex;
	display: flex;
}

.sapMRbGCol{
	vertical-align: top;
	min-width: 10px;
	margin-right: 0;
	overflow: hidden; /* needed in Safari */
}

.sapMRbG > .sapMRb,
.sapMRbGCol > .sapMRb{
	float: left;
	clear: left;
	text-overflow: ellipsis;
}

.sapMRbG.sapMRbG1Row > .sapMRb{
	clear: none;
}

.sapMRbGCol > .sapMRb{
	width: 100%;
	outline-offset: -1px;
}

.sapMRbG .sapMRbB {
	position: absolute;
	left: 0;
}

.sapMRbG  .sapMRbHasLabel .sapMRbB {
	width: @_sap_m_RadioButtonGroup_WidthWithLabel;
}

.sapMRbG .sapMRbBLabel {
	margin-left: 0;
	padding-left: @_sap_m_RadioButtonGroup_PaddingLeftLabel;
}

.sapMRbG .sapMRbHasLabel .sapMRbBOut {
	margin: 0.8125rem;
}

/* Right to left */
.sapMRbG[dir=rtl] .sapMRbB {
	right: 0;
}

.sapMRbG[dir=rtl] .sapMRbBLabel {
	padding-left: 0.125rem;
	padding-right: 2.75rem;
}

/* Align RadioButton in Form with other controls */
.sapUiForm.sapUiFormEdit-CTX{
	.sapMRbG{
		.sapMRbBLabel {
			padding-left: @_sap_m_RadioButtonGroup_PaddingLeftLabel_Form;
		}

		.sapMRbBLabel:not(.sapMLabelNoText) {
			padding-right: 0.125rem;
		}

		> .sapMRbGCol{
			padding-left: 0.5rem; // to align to spaced between Form cells
		}

		> .sapMRbGCol:first-child{
			padding-left: 0;
		}
	}

	.sapMRbG[dir=rtl] .sapMRbBLabel {
		padding-left: 0;
		padding-right: @_sap_m_RadioButtonGroup_PaddingLeftLabel_Form;
	}

	.sapMRbG.sapMRbG1Row{
		.sapMRb{
			padding-left: 0;
			margin-left: 0.5rem; // to align to spaced between Form cells
		}

		.sapMRb:first-child{
			margin-left: 0;
		}
	}
}

/* Compact Size */
.sapUiSizeCompact {
	.sapMRbG .sapMRbHasLabel .sapMRbB {
		width: 2rem;
	}

	.sapMRbG .sapMRbHasLabel .sapMRbBOut {
		margin: 0.5rem;
	}

	.sapMRbG .sapMRbBLabel {
		padding-left: 2rem;
	}

	.sapMRbG[dir=rtl] .sapMRbBLabel {
		padding-left: 0.125rem;
		padding-right: 2rem;
	}

	.sapUiForm.sapUiFormEdit-CTX{
		.sapMRbG .sapMRbBLabel {
			padding-left: 1.625rem;
		}

		.sapMRbG[dir=rtl] .sapMRbBLabel {
			padding-left: 0;
			padding-right: 1.625rem;
		}
	}
}
/* END "RadioButtonGroup.less" */

/* START "RangeSlider.less" */
/* ================================== */
/* CSS for control sap.m/RangeSlider  */
/* Base theme                         */
/* ================================== */

.sapMSliderProgress {
	width: auto;
}

.sapMSliderProgress:focus {
	outline: none;
}

html.sap-desktop .sapMSliderProgress:focus::before {
	border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
	content: '';
	display: block;
	position: absolute;
	bottom: -2px;
	right: -2px;
	pointer-events: none;
}

/* RangeSlider Labels */
.sapMSliderLabels {
	margin: 0.875rem -0.875rem 0 -0.875rem;
	height: 1rem;
	display: flex;
	flex-direction: row;
	flex-wrap: nowrap;
	justify-content: space-between;
	margin-left: -1.0625rem;
	margin-right: -1.406rem;
}

.sapMSliderRangeLabel {
	display: block;
	width: 1.75rem;
	font-size: 0.75rem;
	align-self: flex-end;
	text-align: center;
	direction: ltr;
}

/* overlap */
.sapMSliderHandle.sapMSliderHandleOverlap {
	background-color: var(--sapField_Active_BorderColor);
	border: var(--sapField_Active_BorderColor);
}

/* pressed */
.sapMSliderInner > .sapMSliderHandle.sapMSliderHandlePressed {
	background: var(--sapHighlightColor);
	border: 0.125rem solid var(--sapField_Active_BorderColor);
}

/* compact size */
.sapUiSizeCompact {

	.sapMSliderLabels {
		margin: 0.5rem -0.6875rem 0 -0.6875rem;
	}

	.sapMSliderRangeLabel {
		width: 1.375rem;
	}
}
/* END "RangeSlider.less" */

/* START "RatingIndicator.less" */
/* ====================================== */
/* CSS for control sap.m/RatingIndicator  */
/* Base theme                             */
/* ====================================== */

.sapMRI {
	position: relative;
	display: inline-block;
	overflow: hidden;
	vertical-align: top;
	padding: 0;
	margin: 0.8125rem 0;
}

html.sap-phone .sapMRI:focus,
html.sap-tablet .sapMRI:focus {
	outline: none;
}

.sapMRISelector {
	position: absolute;
	background: transparent;
	width: 100%;
	height: 100%;
}

.sapMRISel {
	float: left;
	display: block;
	overflow: hidden;
	padding: 0;
	margin: 0;
	color: var(--sapAccentColor1);
	word-wrap: normal;
	white-space: nowrap;
}

.sapMRIUnsel {
	position: relative;
	float: right;
	color: darken(@sapUiContentForegroundColor, 25);
	word-wrap: normal;
	white-space: nowrap;
}

.sapMRIUnselWrapper {
	float: right;
	overflow: hidden;
}

.sapMRIHov {
	float: left;
	display: none;
	overflow: hidden;
	word-wrap: normal;
	white-space: nowrap;
	color: darken(@sapUiAccent1, 5);
}

.sapMRIIconSel,
.sapMRIIconUnsel,
.sapMRIIconHov,
.sapMRIIconReadOnly {
	font-family: var(--sapContent_IconFontFamily);
}

.sapMListTblCell > .sapMRI {
	margin: 0;
}

.sapUiSizeCondensed .sapUiTableDataCell > .sapUiTableCellInner > .sapMRI {
	box-sizing: content-box;
	padding: 0;
	margin: 0 0.5rem;
}

.sapMRI.sapUiRatingIndicatorXS .sapUiIcon:last-child,
.sapMRI.sapUiRatingIndicatorXS .sapMImg:last-child,
.sapMRI.sapUiRatingIndicatorS  .sapUiIcon:last-child,
.sapMRI.sapUiRatingIndicatorS .sapMImg:last-child,
.sapMRI.sapUiRatingIndicatorM .sapUiIcon:last-child,
.sapMRI.sapUiRatingIndicatorM .sapMImg:last-child,
.sapMRI.sapUiRatingIndicatorL .sapUiIcon:last-child,
.sapMRI.sapUiRatingIndicatorL .sapMImg:last-child {
	padding-right: 0;
}

.sapMRIDisabled {
	outline: none;
	cursor: default;
}

.sapMRIDisplayOnly {
	outline: none;
	cursor: default;
	margin: 0;
}

.sapMRI.sapMRIDisplayOnly .sapUiIcon,
.sapMRI.sapMRIDisplayOnly .sapMImg {
	padding-right: @sapUiRIIconPaddingDisplayOnly;
	height: @sapUiRIIconSizeDisplayOnly;
}

.sapMRI .sapUiIcon,
.sapMRI .sapMImg {
	padding-right: @sapUiRIIconPaddingCozy;
	height: @sapUiRIIconSizeCozy;
	overflow: hidden;
	vertical-align:top;
}

.sapMRI .sapUiIcon:last-child,
.sapMRI .sapMImg:last-child {
	padding-right: 0;
}

/* Compact mode */
.sapUiSizeCompact {

	.sapMRI {
		margin: 0.5rem 0;
	}

	.sapMRI.sapMRIDisplayOnly {
		margin: 0;
	}

	.sapMRI .sapUiIcon,
	.sapMRI .sapMImg {
		padding-right: @sapUiRIIconPaddingCompact;
		height: @sapUiRIIconSizeCompact;
	}

	.sapMRI.sapMRIDisplayOnly.sapMRINoCustomIconSize .sapUiIcon,
	.sapMRI.sapMRIDisplayOnly.sapMRINoCustomIconSize .sapMImg {
		padding-right: @sapUiRIIconPaddingDisplayOnly;
		height: @sapUiRIIconSizeDisplayOnly;
	}

	.sapMRI .sapUiIcon:last-child,
	.sapMRI .sapMImg:last-child {
		padding-right: 0;
	}
}

/* Condensed mode */
.sapUiSizeCondensed .sapUiTableDataCell .sapMRI .sapUiIcon,
.sapUiSizeCondensed .sapUiTableDataCell .sapMRI .sapMImg {
	vertical-align: top;
}

.sapUiSizeCondensed .sapMRI .sapUiIcon,
.sapUiSizeCondensed .sapMRI .sapMImg {
	padding-right: 0;
}

.sapMRI.sapUiRatingIndicatorXS {

	.sapUiIcon,
	.sapMImg {
		padding-right: @sapUiRIIconPaddingXS;
		vertical-align: top;
	}
}

.sapMRI.sapUiRatingIndicatorS {

	.sapUiIcon,
	.sapMImg {
		padding-right: @sapUiRIIconPaddingS;
		vertical-align: top;
	}
}

.sapMRI.sapUiRatingIndicatorM {

	.sapUiIcon,
	.sapMImg {
		padding-right: @sapUiRIIconPaddingM;
		vertical-align: top;
	}
}

.sapMRI.sapUiRatingIndicatorL {

	.sapUiIcon,
	.sapMImg {
		padding-right: @sapUiRIIconPaddingL;
		vertical-align: top;
	}
}

.sapMRI.sapUiRatingIndicatorXS,
.sapMRI.sapUiRatingIndicatorS,
.sapMRI.sapUiRatingIndicatorM,
.sapMRI.sapUiRatingIndicatorL {

	.sapUiIcon:last-child,
	.sapMImg:last-child {
		padding-right: 0;
	}
}

.sapMRI.sapMRIDisplayOnly {

	.sapMRIIconSel {
		color: var(--sapTextColor);
	}
}
/* END "RatingIndicator.less" */

/* START "ScrollBar.less" */
/* ====================================== */
/* CSS for control sap.m/ScrollBar  */
/* Base theme                             */
/* ====================================== */

.sapMScrollBarTouch {
	-webkit-overflow-scrolling: auto;
}

.sapMScrollBarTouch ::-webkit-scrollbar {
	height: 7px;
	width: 7px;
	-webkit-appearance: none;
}

.sapMScrollBarTouch ::-webkit-scrollbar-thumb {
	background-color: rgba(0,0,0,.5);
	-webkit-box-shadow: 0 0 1px rgba(255,255,255,.5);
}

.sapMScrollBarOuterDiv {
	overflow: hidden;
	height: 100%;
	right: 0;
	position: absolute;
	z-index: 100;
}

html.sap-desktop.sapUiNativeScrollbars .sapMScrollBarOuterDiv {
	display: none;
}

.sapMScrollBarInnerDiv {
	overflow-y: scroll;
	overflow-x: hidden;
	height: 100%;
}

.sapMScrollBarDistantSpan {
	position: absolute;
	top: -9000px;
	left: -9000px;
	visibility: hidden;
	line-height: normal;
}
/* END "ScrollBar.less" */

/* START "ScrollContainer.less" */
/* ====================================== */
/* CSS for control sap.m/ScrollContainer  */
/* Base theme                             */
/* ====================================== */

.sapMScrollCont {
	overflow: hidden;
	position: relative; /* to make the iScroll scrollbars appear at the correct position, if enabled */
	outline: none;
}

.sapMScrollContV .sapMScrollContScroll {
	display: block; /* when there is only vertical scrolling, the whole available width should be consumed */
}

/* mixin call for creating container content padding classes (arguments: rootContainer, contentSelector) */
.sapUiContainerContentPadding(~".sapMScrollCont", ~"> .sapMScrollContScroll");

.sapMScrollCont:focus {
	outline: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
	outline-offset: 0.0625rem;
}
/* END "ScrollContainer.less" */

/* START "SearchField.less" */
/* ================================== */
/* CSS for control sap.m/SearchField  */
/* Base theme                         */
/* ================================== */

@_sap_m_SearchField_Height: 3rem;
@_sap_m_SearchField_ButtonSize: 2.375rem;
@_sap_m_SearchField_ButtonFontSize: 1.125rem;
@_sap_m_SearchField_CompactButtonLineHeight: 1.5rem;
@_sap_m_SearchField_Compact_FormPaddingLeft: 0.375rem;
@_sap_m_SearchField_ResetIcon: '\e1c7';
@_sap_m_SearchField_SearchIcon: '\e00d';
@_sap_m_SearchField_PlaceholderColor: var(--sapField_PlaceholderTextColor);
@_sap_m_SearchField_Border: var(--sapField_BorderWidth) var(--sapField_BorderStyle) var(--sapField_BorderColor);
@_sap_m_SearchField_FocusOffset: 1px;
@_sap_m_SearchField_FocusBorderRadius: unset;
@_sap_m_SearchField_PlaceholderFontStyle: italic;
@_sap_m_SearchField_ActiveBackgroundStyle: var(--sapField_Focus_Background);

.sapMSF {
	position: relative;
	-moz-user-modify: read-only;
	-webkit-user-modify: read-only;
	-webkit-user-select: none;
	user-select: none;
	box-sizing: border-box;
	width: 100%;
	min-width: 7rem;
	height: @_sap_m_SearchField_Height;
	background: transparent;
	padding: 0.25rem 0;
}

.sapUiUfdShellHead .sapMSF {
	padding: 0;
}

.sapUiSizeCompact .sapUiUfdShellHead .sapMSF {
	padding: 0.25rem 0;
}

html.sap-desktop .sapMFocus > .sapMSFF::before {
	border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapField_Active_BorderColor);
	position: absolute;
	content: ' ';
	top: @_sap_m_SearchField_FocusOffset;
	right: @_sap_m_SearchField_FocusOffset;
	left: @_sap_m_SearchField_FocusOffset;
	bottom: @_sap_m_SearchField_FocusOffset;
	border-radius: @_sap_m_SearchField_FocusBorderRadius;
	z-index: 2;
	pointer-events: none;
}

.sapMSFI[type='search'] {
	font-size: 0.88em;
	-webkit-appearance: none;
	box-sizing: border-box;
	width: 100%;
}

.sapMSFI[type='search']::-webkit-search-decoration,
.sapMSFI[type='search']::-webkit-search-results-decoration,
.sapMSFI[type='search']::-webkit-search-results-button {
	-webkit-appearance: none;
}

.sapMSFI[type='search']:focus {
	-webkit-appearance: none;
}

input.sapMSFI[type='search']:focus {
	outline: none;
	outline-width: 0;
}

.sapMSFR {
	visibility: hidden;
	background: transparent;
}

.sapMSplitContainerMaster .sapMSF {
	padding-right: 1px;
}

.sapMSFF {
	display: flex;
	box-sizing: border-box;
	height: var(--sapElement_Height);
	box-shadow: var(--sapField_Shadow);
	border: @_sap_m_SearchField_Border;
	background: var(--sapField_BackgroundStyle);
	background-color: var(--sapField_Background);
	vertical-align: top;
	width: 100%;
	position: relative;
	padding-left: 0.625rem;
	overflow: hidden;
}

/* Block elements do not work with bar layout */
.sapMBar .sapMSF {
	display: inline-block;
}

.sapMFocus>.sapMSFF {
	border-color: var(--sapField_Focus_BorderColor);
}

:not(.sapMSFDisabled) > .sapMSFF:hover {
	border-color: var(--sapField_Hover_BorderColor);
	box-shadow: var(--sapField_Hover_Shadow);
}

:not(.sapMSFDisabled) > .sapMSFF:active {
	background: @_sap_m_SearchField_ActiveBackgroundStyle;
}

.sapMSFI {
	color: var(--sapField_TextColor);
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	background: transparent;
	height: 100%;
	border: none;
	outline: none;
	padding: 0;
	margin: 0;
	text-indent: 0.125rem;
	vertical-align: top;
	text-overflow: ellipsis;
	flex: 1;
}

.sapMSFI[type='search'] {
	font-size: 0.875rem;
}

/* remove clear x button in webkit */
.sapMSFI[type='search']::-webkit-search-cancel-button {
	-webkit-appearance: none;
	display: none;
}

.sapMSFB {
	width: @_sap_m_SearchField_ButtonSize;
	height: 100%;
	line-height: @_sap_m_SearchField_ButtonSize;
	font-size: @_sap_m_SearchField_ButtonFontSize;
	color: var(--sapContent_IconColor);
	text-align: center;
	cursor: pointer;
	padding:0;
	box-sizing: border-box;
}

.sapMSFB::after {
	speak: none;
	font-weight: normal;
	font-family: var(--sapContent_IconFontFamily);
	-webkit-font-smoothing: antialiased;
}

.sapMSFR::after {
	content: @_sap_m_SearchField_ResetIcon;
}

.sapMSFVal > .sapMSFF .sapMSFR,
:not(.sapMSFVal):not(.sapMFocus) > .sapMSFReload .sapMSFR {
	visibility: visible;
}

/* search icon in place of the reset button */
:not(.sapMSFVal) > .sapMSFReload .sapMSFR::after {
	content: @_sap_m_SearchField_SearchIcon;
}

/* search button */
.sapMSFS::after {
	content: @_sap_m_SearchField_SearchIcon;
}

/* refresh icon in some cases */
:not(.sapMFocus) > .sapMSFReload .sapMSFS::after {
	content: '\e010';
}

/* grouping is not allowed here */
.sapMSFI::-webkit-input-placeholder {
	color: @_sap_m_SearchField_PlaceholderColor;
	font-size: @sapMFontMediumSize;
	font-style: @_sap_m_SearchField_PlaceholderFontStyle;
}

.sapMSFI::-moz-placeholder {
	color: @_sap_m_SearchField_PlaceholderColor;
	font-size: @sapMFontMediumSize;
	font-style: @_sap_m_SearchField_PlaceholderFontStyle;
	opacity: 1; /* in FF by default the opacity is 60% */
}

.sapMSF.sapMSFDisabled form,
.sapMSF:not(.sapMSFVal) form.sapMSFNS:not(.sapMSFReload) {
	padding-right: 0.625rem;
}

/* Suggestion Item Description */
.sapMSFSItemDescr {
	padding-inline-start: 0.5rem;
}

/* Compact size */
.sapUiSizeCompact {

	.sapMSF {
		height: 2rem;
		padding: 0.1875rem 0;
		min-width: 6rem;
	}

	.sapMSFF {
		height: var(--sapElement_Compact_Height);
		padding-left: @_sap_m_SearchField_Compact_FormPaddingLeft;
	}

	.sapMSFB {
		width: 2rem;
		font-size: 1rem;
		text-align: center;
		padding:0;
		line-height: @_sap_m_SearchField_CompactButtonLineHeight;
	}

	.sapMSF.sapMSFDisabled form,
	.sapMSF:not(.sapMSFVal) form.sapMSFNS:not(.sapMSFReload) {
		padding-right: @_sap_m_SearchField_Compact_FormPaddingLeft;
	}
}
/* END "SearchField.less" */

/* START "SegmentedButton.less" */
/* ====================================== */
/* CSS for control sap.m/SegmentedButton  */
/* Base theme                             */
/* ====================================== */

.sapMSegB {
	margin: 0; /* To override the native browser margin */
	padding: 0;
	-webkit-tap-highlight-color: rgba(255, 255, 255, 0);
	border: none;
	white-space: nowrap;
	display: inline-block;
	box-sizing: border-box;
	height: 3rem;
	vertical-align: top;
	position: relative;
}

.sapMSegB:focus {
	outline: none;
}

.sapMSegB .sapMSegBBtn {
	list-style: none;
	cursor: pointer;
	display: inline-block;
	padding: 0.250rem 0;
	outline: none;
}

.sapMSegB .sapMSegBBtn {

	.sapMSegBBtnInner {
		text-overflow: ellipsis;
		overflow: hidden;
		-webkit-tap-highlight-color: rgba(255, 255, 255, 0);
		text-align: center;
		vertical-align: text-bottom;
		box-sizing: border-box;
		font-size: 0.875rem;
		height: 2.5rem;
		line-height: 2.375rem;
		padding-left: 0.75rem;
		padding-right: 0.75rem;
		text-shadow: none;
		color: @sapUiSegmentedButtonTextColor;
		border: 1px solid @sapUiSegmentedButtonBorderColor;
		background-color: @sapUiSegmentedButtonBackground;
	}

	&:not(.sapMSegBBtnMixed) .sapMImg {
		padding-left: 0.75rem;
		padding-right: 0.75rem;
	}

	&:not(.sapMSegBBtnMixed) .sapMBtnIcon {
		margin-right: 0;
	}
}

/* Adding margin to be displayed correct in sap.m.Bar */
.sapMBar .sapMSegB {
	margin-left: 0.250rem;
}

html.sap-desktop .sapMSegBBtnFocusable:hover .sapMSegBBtnInner {
	background: @sapUiSegmentedButtonHoverBackground;
}

.sapMSegBBtn:focus .sapMSegBBtnInner {
	outline: none;
}

.sapMSegBBtn .sapUiIcon:focus {
	outline: none;
}

.sapMSegBBtn .sapMBtnIconLeft {
	position: relative;
}

/* style for icon and text usage */
.sapMSegB.sapMSegBIcons {

	.sapMSegBBtn:not(.sapMSegBBtnMixed) .sapMSegBBtnInner {
		padding: 0; /*BCP: 1580200804*/
		margin-right: 0;
	}
	.sapMSegBBtn.sapMSegBBtnMixed .sapMImg,
	.sapMSegBBtn.sapMSegBBtnMixed .sapUiIcon {
		padding-left: 0;
		padding-right: 0.5rem;
	}
}

.sapMSegB img {
	margin: 0;
	pointer-events: none; /* To disable the save image menu on ios */
	line-height: 2.375rem;
	max-height:  2.375rem;
	vertical-align: middle;
}

.sapMSegBBtn.sapMSegBBtnDis {
	cursor: default;
}

.sapMSegBBtn.sapMSegBBtnDis .sapMSegBBtnInner {
	text-shadow: none;
	opacity: var(--sapContent_DisabledOpacity);
}

.sapMSegBBtn:not(.sapMSegBBtnDis) .sapUiIcon {
	cursor: pointer;
}

.sapMSegBBtn.sapMSegBBtnSel .sapMSegBBtnInner {
	background: @sapUiSegmentedButtonSelectedBackground;
	color: @sapUiSegmentedButtonSelectedTextColor;
	border-color: @sapUiSegmentedButtonSelectedHoverBorderColor;

	.sapUiIcon {
		color: var(--sapButton_Selected_TextColor);
	}
}

.sapMSegBBtnSel:active .sapMSegBBtnInner,
.sapMSegBBtnFocusable.sapMBtnActive .sapMSegBBtnInner {
	background: @sapUiSegmentedButtonActiveBackground;
	color: @sapUiSegmentedButtonActiveTextColor;

	.sapUiIcon {
		color: @sapUiSegmentedButtonActiveIconColor;
	}
}

html.sap-desktop .sapMSegBBtn.sapMSegBBtnFocusable.sapMSegBBtnSel:hover .sapMSegBBtnInner,
html.sap-desktop .sapMSegBBtnFocusable.sapMBtnActive:hover .sapMSegBBtnInner {
	background: @sapUiSegmentedButtonSelectedHoverBackground;
}

.sapMSegBBtn.sapMSegBBtnFocusable.sapMSegBBtnSel .sapMSegBBtnInner {
	border-right: 1px solid @sapUiSegmentedButtonSelectedHoverBorderColor;
}

.sapMSegBBtn .sapUiIcon {
	font-size: 1.375rem;
	line-height: 2.375rem;
	width: 2.375rem;
	color: @sapUiSegmentedButtonIconColor;
	vertical-align: middle;
	text-align: center;
}

.sapMSegBFit {
	width: 100%;
}

.sapMFlexItem .sapMSegBFit,
.sapMOTB > .sapMSegBFit {
	width: auto;
}

html[data-sap-ui-os^='iOS'] {

	header,
	footer,
	.sapMBar {

		.sapMSegB {
			line-height: 1.5rem;
			display: inline-block;
		}
	}
}

/* Compact size */
.sapUiSizeCompact {

	.sapMSegB {
		height: 2rem;
		padding: 0;
	}

	.sapMSegB .sapMSegBBtn {
		padding: 0.1875rem 0;
	}

	.sapMSegB img {
		line-height: 1.5rem;
		height: 1.3rem;
		width: 1.3rem;
	}

	.sapMSegBBtn .sapUiIcon {
		font-size: 1rem;
		line-height: 1.5rem;
		width: 1rem;
		vertical-align: top;
	}

	.sapMSegBIcons:not(.sapMSegBBtnMixed) {

		.sapMImg,
		.sapUiIcon {
			padding-left: 0.4375rem;
			padding-right: 0.4375rem;
		}
	}

	.sapMSegBIcons {

		.sapMSegBBtn.sapMSegBBtnMixed {

			.sapMImg,
			.sapUiIcon {
				padding-left: 0;
				padding-right: 0;
			}
		}
	}

	.sapMSegBBtn .sapMSegBBtnInner {
		height: 1.625rem;
		line-height: 1.5rem;
		padding-left: 0.4375rem;
		padding-right: 0.4375rem;
	}

	.sapMSegBBtn:first-child .sapMSegBBtnInner {
		border-top-left-radius: 0.2rem;
		border-bottom-left-radius: 0.2rem;
	}

	.sapMSegBBtn:last-child .sapMSegBBtnInner {
		border-top-right-radius: 0.2rem;
		border-bottom-right-radius: 0.2rem;
	}

	.sapMSegB {

		.sapMBtnIconLeft {
			margin-left: 0;
		}

		&.sapMSegBIcons .sapMSegBBtnMixed {

			img {
				padding: 0;
			}
		}
	}
}

/* Android size compact fix */
html[data-sap-ui-os^='Android'] {

	.sapUiSizeCompact .sapMSegBBtn .sapUiIcon {
		line-height: 1.6rem;
	}
}
/* END "SegmentedButton.less" */

/* START "Select.less" */
/* ============================= */
/* CSS for control sap.m/Select  */
/* Base theme                    */
/* ============================= */

/* commons styles                 */
.sapMSltPicker-CTX {

	.sapMSelectList {

		.sapMSelectListRow {
			height: auto;
			display: flex;
			box-sizing: border-box;
			word-break: break-all;

			.sapMSelectListFirstCell {
				height: auto;
				overflow: hidden;
				text-overflow: ellipsis;
				white-space: nowrap;
				display: block;
				box-sizing: border-box;
			}

			.sapMSelectListLastCell {
				display: block;
				box-sizing: border-box;
			}

			.sapMSelectListFirstCell,
			.sapMSelectListLastCell {
				padding-top: 0.75rem;
				padding-bottom: 0.75rem;
				word-break: break-word;
				&:empty {
					height: 2.5rem;
				}
			}
		}

		&.sapMSelectListWrappedItems {

			.sapMSelectListItem.sapMSelectListItemWithIcon,
			.sapMSelectListRow.sapMSelectListItemWithIcon .sapMSelectListFirstCell {
				position: relative;
				padding-left: 2.25rem;

				.sapMSelectListItemIcon {
					position: absolute;
					left: 1rem;
				}
			}

			.sapMSelectListItem {
				white-space: normal;
				padding-top: 0.75rem;
				padding-bottom: 0.75rem;
				line-height: 1rem;
				height: auto;

				&:empty {
					height: 2.5rem;
				}
			}

			.sapMSelectListRow {
				display: table-row;

				.sapMSelectListFirstCell {
					white-space: normal;
				}

				.sapMSelectListLastCell {
					vertical-align: top;
				}

				.sapMSelectListCell {
					display: table-cell;
				}
			}
		}

		.sapMSelectListItem {
			display: flex;
			align-items: center;
		}
	}
}

/* sets max-width of 600px to selectList, only when wrapItemsText property is false */
.sapMSltPicker-CTX:not(.sapMPickerWrappedItems) {
	max-width: 600px;

	.sapMSelectList:not(.sapMSelectListWrappedItems) {
		max-width: 600px;

		.sapMSelectListRow {
			max-width: 600px;
		}
	}
}

/* defines the ratio of columns inside selectList, when text is truncated */
.sapMSltPicker-CTX .sapMSelectList:not(.sapMSelectListWrappedItems) {

	.sapMSelectListFirstCell {
		width: 60%;
	}

	.sapMSelectListLastCell {
		width: 40%;
	}
}

/* sets max-width of 100% to selectList, only when wrapItemsText property is true */
.sapMSltPicker-CTX.sapMDialogPhone {
	max-width: 100%;

	.sapMSelectList:not(.sapMSelectListWrappedItems) {
		max-width: 100%;

		.sapMSelectListRow {
			max-width: 100%;
		}
	}
}

.sapUiSizeCompact .sapMSltPicker-CTX .sapMSelectList {

	.sapMSelectListRow {

		.sapMSelectListFirstCell,
		.sapMSelectListLastCell {
			padding-top: 0.5rem;
			padding-bottom: 0.5rem;

			&:empty {
				height: 2rem;
			}
		}
	}

	&.sapMSelectListWrappedItems .sapMSelectListItem {
		padding-top: 0.5rem;
		padding-bottom: 0.5rem;

		&:empty {
			height: 2rem;
		}
	}
}

.sapMSlt,
.sapMSlt > .sapMSltLabel {
	box-sizing: border-box;
}

.sapMSlt,
.sapMSltLabel,
.sapMSltArrow {
	display: inline-block;
}

.sapMSlt,
.sapMSltLabel {
	overflow: hidden;
}

.sapMSlt:not(.sapMSltDisabled),
.sapMSlt:not(.sapMSltDisabled) > .sapMSltLabel,
.sapMSltIconOnly:not(.sapMSltDisabled) .sapMSltIcon {
	cursor: pointer;
}

.sapMSlt.sapMSltReadonly {
	border-color: var(--sapField_ReadOnly_BorderColor);
	background: var(--sapField_ReadOnly_Background);

	.sapMSltArrow {
		display: none;
	}
}

.sapMSlt > .sapMSltLabel,
.sapMSltArrow {
	height: 100%;
}

.sapMSlt > .sapMSltLabel,
.sapMSlt .sapMSelectListItemBase {
	padding: 0 0.625rem;
}

.sapMSltWithArrow > .sapMSltLabel,
.sapMSltWithArrow .sapMSelectListItemBase {
	padding: 0 3rem 0 0.625rem;
}

/* field                          */
.sapMSlt {
	position: relative;
	vertical-align: top;
	outline: none;
	height: 2.5rem;
	margin: 0.25rem 0 0.25rem 0;
	border: 1px solid var(--sapField_BorderColor);
	background: var(--sapField_Background);
	line-height: initial;
}

.sapMSltMinWidth {
	min-width: 5.125rem; /* enough for one letter, the ellipsis and the Select`s arrow */
}

/* pressed */

.sapMSltPressed:not(.sapMSltDisabled):not(.sapMSltState) {
	border-color: var(--sapField_Active_BorderColor);
}

.sapMSltPressed.sapMSltIconOnly:not(.sapMSltDisabled) {
	background: @sapUiToggleButtonPressedBackground;
	text-shadow: none;
}

/* value state */

.sapMSltError {
	background: var(--sapField_InvalidBackground);
	border-color: var(--sapField_InvalidColor);
}

.sapMSltWarning {
	background: var(--sapField_WarningBackground);
	border-color: var(--sapField_WarningColor);
}

.sapMSltSuccess {
	background: var(--sapField_SuccessBackground);
	border-color: var(--sapField_SuccessColor);
}

.sapMSltInformation {
	background: var(--sapField_Background);
	border-color: var(--sapHighlightColor);
}

/* label                          */
.sapMSlt > .sapMSltLabel {
	position: absolute;
	width: 100%;
	left: 0;
	right: 0;
	text-overflow: ellipsis;
	white-space: nowrap;
	text-decoration: none;
	text-indent: 0;
	text-align: start;
	-webkit-user-select: none;
	-moz-user-select: none;
	user-select: none;
	color: var(--sapField_TextColor);
	font-style: normal;
	font-size: @sapMFontMediumSize;
	vertical-align: middle;

	&::after {
		content: '';
		height: 100%;
		display: inline-block;
		vertical-align: middle;
		width: 0;
	}

	.sapMSelectListItemText {
		vertical-align: middle;
	}

	.sapMSelectListItemIcon {
		vertical-align: middle;
	}
}

.sapMSltAutoAdjustedWidth > .sapMSltLabel {
	position: relative;
	vertical-align: top;
}

/* icon                           */
.sapMSltIcon {
	position: absolute;
	display: block;
	font-size: 1.375rem;
	line-height: 2.375rem;
	padding: 0 0.4375rem;
	color: var(--sapContent_IconColor);
}

.sapMSltAutoAdjustedWidth .sapMSltIcon {
	position: relative;
}

/* pressed */

.sapMSltPressed > .sapMSltIcon {
	color: var(--sapContent_ContrastIconColor);
}

/* arrow                          */
.sapMSltArrow {
	position: absolute;
	text-align: center;
	width: 2.5rem;
	line-height: 2.5rem;
	top: 0;
	right: 0;
	color: var(--sapContent_IconColor);
	font-size: var(--sapFontLargeSize);
}

.sapMSltArrow::before {
	font-family: var(--sapContent_IconFontFamily);
	speak: none;
	content: '\e1ef';
}

/* hover */
.sapMSltHoverable:not(.sapMSltPressed):hover > .sapMSltArrow {
	background: var(--sapButton_Lite_Hover_Background);
}

/* pressed */
.sapMSltPressed > .sapMSltArrow {
	background: @sapUiToggleButtonPressedBackground;
}

.sapMSltPressed > .sapMSltArrow::before {
	color: @sapUiToggleButtonPressedTextColor;
}

/* dropdown list                  */
.sapMSlt > .sapMSelectList {
	visibility: hidden;
	position: relative;
	cursor: inherit;
	border: none;
	margin: 0;
	padding: 0;
	min-width: 6rem;
}

/* table context                  */
.sapUiSizeCondensed .sapUiTableCell .sapMSlt {
	margin: 0;
}

.sapUiSizeCondensed .sapUiTableCell .sapMSlt:not(.sapMSltState) {
	border-color: transparent;
}

.sapMSelectListTableLayout {
	border-collapse: collapse;
}

/* value states commons styles adaptations  */
.sapMSltLabelState,
.sapMSltArrowState {
	line-height: 2.25rem;
}


/* Select inside a Bar: header and sub-header contexts */
.sapMSubHeader-CTX .sapMSltPressed > .sapMSltIcon {
	color: var(--sapContent_ContrastIconColor);
}

.sapMHeader-CTX .sapMSltPressed,
.sapMSubHeader-CTX .sapMSltPressed {

	/* pressed selects arrow and icon look wrong with text-shadow */
	text-shadow: none;
}

.sapMHeader-CTX .sapMSltIconOnly.sapMSltHoverable:hover,
.sapMSubHeader-CTX .sapMSltIconOnly.sapMSltHoverable:hover {
	border-color: var(--sapButton_Lite_Hover_BorderColor);
	background: var(--sapButton_Lite_Hover_Background);
}

.sapMHeader-CTX .sapMSltIconOnly:not(.sapMSltPressed),
.sapMSubHeader-CTX .sapMSltIconOnly:not(.sapMSltPressed) {
	border-color: var(--sapButton_Lite_BorderColor);
	background-color: var(--sapButton_Lite_Background);
}

/* icon                           */
.sapMSubHeader-CTX .sapMSltPressed > .sapMSltIcon {
	color: var(--sapContent_ContrastIconColor);
}

/* Select inside a Bar: footer context */
.sapMFooter-CTX .sapMSltIconOnly.sapMSltHoverable:not(.sapMSltPressed):hover {
	background: @sapUiButtonFooterHoverBackground;
	border-color: var(--sapButton_Lite_BorderColor);
}

/* icon                           */
.sapMFooter-CTX .sapMSltIconOnly {
	background: var(--sapButton_Lite_Background);
	border-color: var(--sapButton_Lite_BorderColor);
}

/* label                          */
.sapMIBar-CTX.sapMFooter-CTX .sapMSltArrow {
	text-shadow: none;
}

.sapMFooter-CTX .sapMSltLabel {
	text-shadow: none;
}

/* picker                         */
div.sapMSltPicker {
	border: none;
	box-shadow: @sapUiShadowLevel1;

	&.sapUiPopupWithPadding .sapMDialogScrollCont.sapMDialogStretchContent {
		height: 100%;
	}

	.sapMSltPickerValueState {
		padding: 0.5625rem 1rem;
		color: var(--sapTextColor);
		font-size: var(--sapFontSmallSize);
		box-shadow: inset 0 -0.0625rem var(--sapPageHeader_BorderColor);

		&.sapMSltPickerErrorState {
			background: var(--sapErrorBackground);
		}

		&.sapMSltPickerSuccessState {
			background: var(--sapSuccessBackground);
		}

		&.sapMSltPickerWarningState {
			background: var(--sapWarningBackground);
		}

		&.sapMSltPickerInformationState {
			background: var(--sapInformationBackground);
		}

		&.sapMSltPickerNoneState {
			display: none;
		}
	}
}

/* dialog                         */
div.sapMSltPicker .sapMDialogScrollCont {
	padding: 0;
}

.sap-phone {
	.sapMSelectListTableLayout {
		display: inline;
	}
	.sapMSelectListWrappedItems.sapMSelectListTableLayout {
		display: table;
	}
}
/* CSS for control sap.m/Select.control Belize theme smaller sizes - common part */

/* commons styles                 */
.sapUiSizeCompact .sapMSltLabel,
.sapUiSizeCompact .sapMSlt .sapMSelectListItemBase,
.sapUiSizeCondensed .sapUiTableCell .sapMSltLabel,
.sapUiSizeCondensed .sapUiTableCell .sapMSlt .sapMSelectListItemBase {
	padding-right: 2.5rem;
	padding-left: 0.5rem;
}

.sapUiSizeCompact .sapMSlt > .sapMSltArrow {
	line-height: 1.5rem;
}

/* arrow                          */
.sapUiSizeCompact .sapMSltArrow {
	width: 2rem;
}

/* label and arrow commons styles */
.sapUiSizeCompact .sapMSltLabelState,
.sapUiSizeCompact .sapMSltArrowState {
	line-height: 1.375rem;
}

/* icon                           */
.sapUiSizeCompact .sapMSltIcon {
	font-size: 1rem;
	line-height: 1.5rem;
}

.sapUiSizeCondensed .sapUiTableCell .sapMSltIcon {
	font-size: 1rem;
	line-height: 1.375rem; /* + 2*1px padding = 1.5rem */
}

/* compact size              */

/* field                          */
.sapUiSizeCompact .sapMSlt {
	height: 1.625rem;
	margin-top: 0.1875rem;
	margin-bottom: 0.1875rem;
}

/* condensed size (table context) */


/* field                          */
.sapUiSizeCondensed .sapUiTableCell .sapMSlt {
	height: 1.375rem; /* + 2*1px padding = 1.5rem */
}

/* label                          */
.sapUiSizeCondensed .sapUiTableCell .sapMSlt .sapMSltLabel {
	line-height: 1.375rem;
}

/* arrow                          */
.sapUiSizeCondensed .sapUiTableCell .sapMSltArrow {
	line-height: 1.375rem;
}

.sapUiSizeCondensed .sapUiTableCell .sapMSltHoverable:not(.sapMSltState):hover {
	border-color: var(--sapField_Hover_BorderColor);
}

/* commons styles                 */
.sapUiSizeCondensed .sapUiTableCell .sapMSltLabelState,
.sapUiSizeCondensed .sapUiTableCell .sapMSltArrowState {
	line-height: 1.25rem;
}

/* hidden focus element */

.sapMSltHiddenSelect {
	width: 100%;
	height: 100%;
	border: none;
}

/* when the Busy Indicator is inside Select it should be with different positions - BCP 1870362962 */
.sapMSlt .sapUiLocalBusyIndicator:focus::after {
	left: 1px;
	top: 1px;
	right: 1px;
	bottom: 1px;
}
/* END "Select.less" */

/* START "SelectDialog.less" */
/* =================================== */
/* CSS for control sap.m/SelectDialog  */
/* Base theme                          */
/* =================================== */

.sapMSelectDialog.sapMDialog .sapMDialogScrollCont {
	padding: 0px;
}

.sapMBusyIndicator.sapMSelectDialogBusyIndicator {
	position: absolute;
	top: 50%;
	left: 50%;
	margin-top: -1rem;
	margin-left: -1rem;
	display: none;
}

.sapMSelectDialogListHide {
	display: none;
}

.sapMSelectDialog.sapMDialog .sapMDialogScroll {
	width: 100% !important;
}

.sapMSelectDialog.sapUiPopupWithPadding:not(.sapUiNoContentPadding) .sapMDialogScrollCont{
	padding: 0;
}
/* END "SelectDialog.less" */

/* START "SelectList.less" */
/* ================================= */
/* CSS for control sap.m/SelectList  */
/* Base theme                        */
/* ================================= */

@_sap_m_SelectList_ItemHeight: 2.5rem;
@_sap_m_SelectList_ItemCompactHeight: 2rem;

/* commons styles */

.sapMSelectList,
.sapMSelectListItemBase {
	box-sizing: border-box;
}

/* disabled */

.sapMSelectListDisabled {
	pointer-events: none;
}

/* list */

.sapMSelectList {
	margin: 0;
	padding: 0;
	outline: none;
	list-style-type: none;
	background: var(--sapList_Background);
}

/* item */

.sapMSelectListItemBase {
	outline: none;
	-webkit-user-select: none;
	-moz-user-select: none;
	user-select: none;
	color: var(--sapList_TextColor);
	font-size: @sapMFontMediumSize;
	height: @_sap_m_SelectList_ItemHeight;
	cursor: pointer;
}

.sapMSelectListItem {
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
	padding: 0 1rem 0 1rem;
	line-height: @_sap_m_SelectList_ItemHeight;
}

.sapMSelectListItemIcon {
	padding-right: 0.4375rem;
}

li.sapMSelectListItemBaseInvisible {
	display: none;
}

/* elements inside the select effects the size of the browsers default dragging ghost element */
.sapUiDnDDragging .sapMSelectListItemBase + .sapMSelectListItemBase {
	display: none;
}

/* disabled */
.sapMSelectListItemBaseDisabled {
	cursor: unset;
}

/* hover */
.sapMSelectListItemBaseHoverable:hover {
	background: var(--sapList_Hover_Background);
}

/* hover selected */
.sapMSelectListItemBaseSelected.sapMSelectListItemBaseHoverable:hover {
	background-color: @sapUiListSelectionHoverBackground;
}

/* pressed and pressed selected */
:not(.sapMSelectListItemBaseDisabled).sapMSelectListItemBase.sapMSelectListItemBasePressed,
:not(.sapMSelectListItemBaseDisabled).sapMSelectListItemBase.sapMSelectListItemBasePressed.sapMSelectListItemBaseSelected {
	color: var(--sapList_Active_TextColor);
	background: var(--sapList_Active_Background);
}

/* focus */
html.sap-desktop .sapMSelectListItemBase:focus {
	outline: dotted 1px;
	outline-offset: -2px;
}

/* row */

.sapMSelectListRow {
	display: table-row;
}

/* cell */

.sapMSelectListCell {
	display: table-cell;
	vertical-align: middle;
	padding-left: 1rem;
}

.sapMSelectListLastCell {
	text-align: right;
	padding-right: 1rem;
}

/* separator */

.sapMSelectListSeparatorItem {
	border: 1px solid var(--sapList_BorderColor);
}

/* compact size  */

/* commons styles */

.sapUiSizeCompact .sapMSelectListItemBase {
	height: @_sap_m_SelectList_ItemCompactHeight;
}

.sapUiSizeCompact .sapMSelectListItem {
	line-height: @_sap_m_SelectList_ItemCompactHeight;
}
/* END "SelectList.less" */

/* START "SelectionDetails.less" */
/* ======================================= */
/* CSS for control sap.m/SelectionDetails  */
/* Base theme                              */
/* ======================================= */

.sapMSD .sapMDialogStretchContent {
	padding: 0px;

	&.sapMDialogScrollCont {
		height: 100%;
	}
}

.sapMSD .sapMSDFirstActionGroup {
	border-top: 1px solid var(--sapList_BorderColor);
	box-sizing: border-box;
}

.sapMSD .sapMTB.sapMSDPageHeader {
	padding: 0px;
}
/* END "SelectionDetails.less" */

/* START "SelectionDetailsItem.less" */
/* ========================================== */
/* CSS for control sap.m/SelectionDetailsItem */
/* Base theme                                 */
/* ========================================== */

.sapMSDItem {
	display: block;
	padding: 0;
}

.sapMSDItem > .sapMLIBContent {
	padding: 0.5rem 0;
	box-sizing: border-box;
	width: 100%;
	display: flex;
	align-items: center;
}

.sapMSDItem .sapMSDItemLines {
	flex: 1 1;
	max-width: ~"calc(100% - 3rem)";
}

.sapMSDItem .sapMSDItemActions {
	width: 100%;
	flex: 0 0 100%;

	.sapMTB {
		border-bottom: none;
	}
}

.sapMSDItem:not(.sapMLIBTypeNavigation) {

	.sapMSDItemActions {
		flex-basis: ~"calc(100% + 2rem)";
	}
}

.sapMSDItem .sapMLIBImgNav {
	cursor: pointer;
}

.sapMSDItemLine {
	display: flex;
	overflow: hidden;
}

.sapMSDItem .sapMSDItemLineMarkerContainer {
	flex: 0 0 3rem;
	box-sizing: border-box;
	display: flex;
	justify-content: center;
	align-items: center;
	padding-left: 2px;
	padding-right: 2px;
}

.sapMSDItem .sapMSDItemLineLabel,
.sapMSDItem .sapMSDItemLineValue {
	flex: 1 1;
	padding: 0.25rem;
	box-sizing: border-box;
	font-size: smaller;
	min-width: 0; /* explicitly allow items to shrink to 0 */
	word-wrap: break-word;
}

.sapMSDItem .sapMSDItemLineLabel {
	margin-right: 1rem;
	color: var(--sapContent_LabelColor);
	white-space: normal;
	overflow: hidden;
	text-overflow: ellipsis;
}

.sapMSDWrapLabels .sapMSDItem .sapMSDItemLineLabel {
	white-space: normal;
	text-overflow: clip;
}

.sapMSDItem .sapMSDItemLineValue {
	color: var(--sapList_TextColor);

	&.sapMSDItemLineBold {
		font-weight: bold;
	}
}

.sapMSDItem .sapMSDItemLineUnit {
	font-weight: normal;
}

.sapMSDItem.sapMLIBActive {

	.sapMSDItemLineLabel,
	.sapMSDItemLineValue,
	.sapMBtnText,
	.sapMBtnTransparent > .sapMBtnIcon {
		color: var(--sapList_Active_TextColor);
		text-shadow: none;
	}
}
/* END "SelectionDetailsItem.less" */

/* START "SemanticPage.less" */
/* =================================== */
/* CSS for control sap.m/SemanticPage  */
/* Base theme                          */
/* =================================== */

.sapMSemanticPage {
	height: 100%;
	width: 100%;
}
/* END "SemanticPage.less" */

/* START "SimpleFixFlex.less" */
.sapUiSimpleFixFlex{
	display: flex;
	flex-direction: column;
	height: 100%;
	box-sizing: border-box;

	.sapUiSimpleFixFlexFixed {
		width: 100%;
		overflow:auto;

		&.sapUiSimpleFixFlexFixedWrap {
			position: absolute;
			top: 0;
		}
	}

	.sapUiSimpleFixFlexFlexContentContainer {
		flex: 1 1 auto;
		overflow: auto;
		width: 100%;

		.sapUiSimpleFixFlexFlexContent{
			height: 100%;
			overflow: auto;
		}
	}
}
/* END "SimpleFixFlex.less" */

/* START "SuggestionsPopover.less" */
/* ========================================= */
/* CSS for control sap.m/SuggestionsPopover  */
/* Base theme                                */
/* ========================================= */

@_sap_m_SuggestionsPopover_ItemMinHeight: 2.5rem;
@_sap_m_SuggestionsPopover_ItemMinHeightCompact: 2rem;
@_sap_m_SuggestionsPopover_GroupItemHeight: 2.75rem;
@_sap_m_SuggestionsPopover_GroupItemHeightCompact: 2.75rem;

@_sap_m_SuggestionsPopover_TabularItemHeight: 2.5rem;
@_sap_m_SuggestionsPopover_TabularItemHeightCompact: 2rem;
@_sap_m_SuggestionsPopover_TabularGroupItemHeight: 2.5rem;
@_sap_m_SuggestionsPopover_TabularGroupItemHeightCompact: 2rem;

.sapMSuggestionsPopover {

	.sapMListUl > .sapMLIB.sapMSLIWrapping {
		min-height: @_sap_m_SuggestionsPopover_ItemMinHeight;
	}

	.sapMSLIWrapping > .sapMLIBContent > .sapMSLIDiv {
		padding: 0.25rem 0;
	}

	.sapMListUl > .sapMLIB.sapMGHLI {
		height: @_sap_m_SuggestionsPopover_GroupItemHeight;
	}

	.sapMListTbl .sapMLIB {
		height: @_sap_m_SuggestionsPopover_TabularItemHeight;
	}

	.sapMListTbl .sapMListTblHeader,
	.sapMListTbl .sapMLIB.sapMGHLI {
		height: @_sap_m_SuggestionsPopover_TabularGroupItemHeight;
	}

	.sapMListTbl .sapMListTblHeader .sapMColumnHeader,
	.sapMListTbl .sapMLIB .sapMListTblCell {
		padding-top: 0.625rem;
		padding-bottom: 0.625rem;
	}
}

div.sapMSuggestionsPopover {
	border: none;
	box-shadow: var(--sapContent_Shadow1);
}

.sapMSuggestionsPopover.sapMPopover.sapMSuggestionPopoverDefaultWidth {
	max-width: 40rem;
}

.sapMSuggestionsPopover.sapMPopover:not(.sapMSuggestionPopoverDefaultWidth):not(.sapMSuggestionPopoverInputWidth) {
	max-width: ~"calc(100% - 2rem)" !important;
}

.sapMSuggestionsPopover.sapMPopover .sapMPopoverCont {
	max-width: 100% !important;
}

.sapUiSizeCompact {

	.sapMSuggestionsPopover,
	&.sapMSuggestionsPopover {

		.sapMListUl > .sapMLIB.sapMSLIWrapping {
			min-height: @_sap_m_SuggestionsPopover_ItemMinHeightCompact;
		}

		.sapMSLIWrapping > .sapMLIBContent > .sapMSLIDiv {
			padding: 0.25rem 0;
		}

		.sapMListUl > .sapMLIB.sapMGHLI {
			height: @_sap_m_SuggestionsPopover_GroupItemHeightCompact;
		}

		.sapMListTbl .sapMLIB {
			height: @_sap_m_SuggestionsPopover_TabularItemHeightCompact;
		}

		.sapMListTbl .sapMListTblHeader,
		.sapMListTbl .sapMLIB.sapMGHLI {
			height: @_sap_m_SuggestionsPopover_TabularGroupItemHeightCompact;
		}

		.sapMListTbl .sapMListTblHeader .sapMColumnHeader,
		.sapMListTbl .sapMLIB .sapMListTblCell {
			padding-top: 0.25rem;
			padding-bottom: 0.25rem;
		}
	}
}

.sapMDialog.sapMSuggestionsPopover .sapMDialogScroll .sapMDialogScrollCont {
	padding: 0;
}

.sapMInputSuggestionTableHidden {
	display: none;
}

.sapMSuggestionsPopover .sapMDLIValue {
	font-size: @sapMFontMediumSize;
	color: var(--sapList_TextColor);
	padding-right: 1rem;
}

.sapMSuggestionsPopover .sapMLIBActive .sapMDLIValue {
	color: var(--sapList_Active_TextColor);
}

.sapMSuggestionsPopover .sapMDLILabel {
	font-size: @sapMFontMediumSize;
}

.sapMSuggestionsPopover .sapMSLITitleOnly {
	font-size: @sapMFontMediumSize;
}

div.sapMSuggestionsPopover .sapMSuggestionsPopoverValueState {
	font-size: var(--sapFontSmallSize);
	line-height: normal;
	padding: 0.5625rem 1rem;

	&.sapMSuggestionsPopoverErrorState {
		background: var(--sapErrorBackground);
		color: var(--sapTextColor);
	}

	&.sapMSuggestionsPopoverInformationState {
		background: var(--sapInformationBackground);
	}

	&.sapMSuggestionsPopoverSuccessState {
		background: var(--sapSuccessBackground);
		color: var(--sapTextColor);
	}

	&.sapMSuggestionsPopoverWarningState {
		background: var(--sapWarningBackground);
		color: var(--sapTextColor);
	}

	.sapMSuggestionsPopoverTitle.sapMTitle {
		padding-left: 0.75rem;
	}
}

.sapMSuggestionsPopover .sapMSuggestionsPopoverNoneState {
	display: none;
}
/* END "SuggestionsPopover.less" */

/* START "ValueStateHeader.less" */
.sapMValueStateHeaderRoot {
	width: auto;
	padding: 0.5625rem 1rem;
	box-sizing: border-box;
	position: sticky;
	top: 0;
	z-index: 42;
}

// When not on mobile (in sap.m.Dialog) the ValueStateHeader's width
// must be 0 to get calculated correctly and set by the sap.m.Popover
html.sap-desktop .sapMValueStateHeaderRoot {
	width: 0;
}

.sapMPseudoFocus {
	outline: @sapUiContentFocusWidth @sapUiContentFocusStyle @sapUiContentFocusColor;
	outline-offset: -0.125rem;
}

.sapMValueStateHeaderText {
	display: inline-block;
	line-height: normal;
	white-space: pre-line;
	word-wrap: break-word;
	font-size: @sapMFontSmallSize;
	color: @sapUiListTextColor;

	.sapMFT {
		font-size: @sapMFontSmallSize;
	}
}

.sapMValueStateHeaderError,
.sapMSuggestionsPopoverErrorState .sapMPopoverHeader {
	background: @sapUiErrorBG;
	color: @sapUiBaseText;
}

.sapMValueStateHeaderInformation,
.sapMSuggestionsPopoverInformationState .sapMPopoverHeader {
	background: @sapUiInformationBG;
}

.sapMValueStateHeaderSuccess,
.sapMSuggestionsPopoverSuccessState .sapMPopoverHeader {
	background: @sapUiSuccessBG;
	color: @sapUiBaseText;
}

.sapMValueStateHeaderWarning,
.sapMSuggestionsPopoverWarningState .sapMPopoverHeader {
	background: @sapUiWarningBG;
	color: @sapUiBaseText;
}

.sapMValueStateHeaderNone {
	display: none;
}

/* END "ValueStateHeader.less" */

/* START "Shell.less" */
/* ============================ */
/* CSS for control sap.m/Shell  */
/* Base theme                   */
/* ============================ */

@_sap_m_Shell_AppWidthLimit: 1280px;
@_sap_m_Shell_AppWidthLimitBy2: @_sap_m_Shell_AppWidthLimit / 2;
@_sap_m_Shell_AppWidthLimitPlusPadding: @_sap_m_Shell_AppWidthLimit + 16px;

@_sap_m_Shell_AppHeight: 3333px; /* Ux decision: never show the header, so make this value large enough even for zoom-out cases */
@_sap_m_Shell_HeaderHeight: 68px; /* should be 4.25rem */
@_sap_m_Shell_MinHeightForHeader: (@_sap_m_Shell_AppHeight + @_sap_m_Shell_HeaderHeight);

@_sap_m_Shell_MinWidthForLogoAside: 1480px; /* = @_sap_m_Shell_AppWidthLimit + 200; but additions do not work in LESS for media query arguments */
@_sap_m_Shell_MinWidthForBigLogoAside: 1680px; /* = @_sap_m_Shell_AppWidthLimit + 400; but additions do not work in LESS for media query arguments */

.sapMShell {
	width: 100%;
	height: 100%;
	position: relative;
	box-sizing: border-box;
}

.sapMShellBrandingBar {
	position: absolute;
	left: 0;
	top: 0;
	width: 100%;
	height: 4px;
	z-index: 2; /* to lift it above the CenterBox shadow and SplitApp */
	background-color: var(--sapBrandColor);
}

/* this "sapMBrandingBar-CTX" CSS class can be set by any container control that comes with its own branding bar, so any shell inside will not display its branding bar */
.sapMBrandingBar-CTX .sapMShellBrandingBar,
.sapUiUfdShell .sapMShellBrandingBar {
	display: none;
}

.sapMShellBG {
	position: absolute;
	left: 0;
	top: 0;
	width: 100%;
	height: 100%;
}

.sapMShellCentralBox {
	position: relative;
	height: 100%;
}

.sapMShellHeader {
	position: relative;
	width: 100%;
	text-align: center;
	box-sizing: border-box;
	padding: 0 16px; /* must be px because of calculations further below */
	-webkit-user-select: none;
	user-select: none;
}

.sapMShellHeaderText {
	display: none;
	cursor: default;
	position: relative;
}

.sapMShellLogo {
	display: none;
	position: relative;
	z-index: 1; /* above background image */
}

.sapMShellLogoImgAligner {
	display: none;
}

.sapMShellHeaderRight {
	display: none;
	position: relative;
}

.sapMShellContent {
	box-sizing: border-box;
	height: 100% !important; /* important because overwritten by SplitApp/App control */
	width: 100%;
	position: relative;
	margin: 0 auto;
}

/* modifications for taller screens */

@media (min-height: @_sap_m_Shell_MinHeightForHeader) { /* 1500px plus 96px for header */

	html.sap-desktop .sapMShellLogo {
		position: absolute;
		float: left;
		display: flex;
		height: 40px;
		margin: 0.75em;
		margin-left: 0;
	}

	html.sap-desktop .sapMShellLogoImg {
		margin: auto;
		max-width: 6rem;
		max-height: 40px;
	}

	html.sap-desktop .sapMShellHeader {
		display: block;
		position: relative;
		height: 4.25rem;
		color: var(--sapPageHeader_TextColor);
		border-bottom: 1px solid var(--sapShell_BorderColor);
	}

	html.sap-desktop .sapMShellHeaderText {
		display: inline-block;
		margin-top: 1.25rem;
		font-size: 1.75em;
		font-weight: lighter;
	}

	html.sap-desktop .sapMShellHeaderRight {
		display: inline-block;
		float: right;
		font-size: 0.875em;
		margin-top: 30px;
	}

	html.sap-desktop .sapMShellHeaderRightText {
		padding: 0.75em 0.57em; /* text size is 14px here... */
	}

	html.sap-desktop .sapMShellHeaderLogout {
		cursor: pointer;
		padding: 0.75em 0.57em; /* text size is 14px here... */
	}

	html.sap-desktop .sapMShellHeaderLogout:focus {
		outline: none;
	}

	html.sap-desktop .sapMShellHeaderLogout:active {
		background-color: var(--sapButton_Active_Background);
		color: var(--sapButton_Active_TextColor);
		outline: none;
	}

	html.sap-desktop .sapMShellContent {
		position: absolute;
		top: @_sap_m_Shell_HeaderHeight;
		bottom: 0;
		height: auto !important; /* important because overwritten by SplitApp/App control */
	}

	.sapMShellHeader {
		border-top: 4px solid transparent;
	}
}

@media (min-height: 769px) and (max-width: 640px) { /* logo would appear above but the window is really slim */

	html.sap-desktop .sapMShellLogo {
		display: none;
	}
}

/* modifications for wider screens */

@media (min-width: @_sap_m_Shell_AppWidthLimit) {

	html.sap-desktop .sapMShellLogo {
		margin-left: 1rem;
	}

	html.sap-desktop .sapMShellAppWidthLimited .sapMShellCentralBox {
		width: @_sap_m_Shell_AppWidthLimit;
		margin-left: -@_sap_m_Shell_AppWidthLimitBy2; /* @_sap_m_Shell_AppWidthLimit divided by 2 */
		left: 50%;
		background-color: fade(lighten(@sapUiBaseBG, 5), 30);
	}
}

@media (min-width: @_sap_m_Shell_AppWidthLimit) {

	html.sap-desktop .sapMShellAppWidthLimited .sapMShellHeader {
		padding: 0;
	}
}

@media (min-width: @_sap_m_Shell_MinWidthForLogoAside) {

	html.sap-desktop .sapMShellAppWidthLimited .sapMShellLogo {
		position: absolute;
		opacity: 1;
		height: auto;
		width: 80px;
		text-align: center;
		display: block;
		right: @_sap_m_Shell_AppWidthLimit;
		top: 0;
		margin: 0.75em;
		margin-left: 0;
	}

	html.sap-desktop .sapMShellAppWidthLimited .sapMShellLogoImg {
		max-width: 80px;
		max-height: 80px;
	}
}

@media (min-width: @_sap_m_Shell_MinWidthForBigLogoAside) {

	html.sap-desktop .sapMShellAppWidthLimited .sapMShellLogo {
		width: 160px;
		margin: 2rem;
	}

	html.sap-desktop .sapMShellAppWidthLimited .sapMShellLogoImg {
		max-width: 160px;
		max-height: 160px;
	}
}

.sapUiUfdShell .sapMShellHeader {
	border-top: none;
}

html.sap-desktop .sapMShellHeaderText {
	font-size: var(--sapFontHeader2Size);
	color: var(--sapShell_TextColor);
}
/* END "Shell.less" */

/* START "SinglePlanningCalendar.less" */
/* ============================================= */
/* CSS for control sap.m/SinglePlanningCalendar  */
/* Base theme                                    */
/* ============================================= */

@_sap_m_PlanningCalendarHeader_ActionsToolbarHeight_Compact: 3rem;
@_sap_m_PlanningCalendarHeader_ActionsToolbarHeight: 2.75rem;
@_sap_m_SinglePlanningCalendarGrid_HoursBackground: var(--sapList_Background);

.sapMPCHead,
.sapMSinglePCColumnHeader {
	position: sticky;
	z-index: 4;
}

.sapMSinglePCStickyAll {

	.sapMPCHead {
		top: 0;
	}
}

.sapMSinglePCStickyNavBarAndColHeaders {

	.sapMPCHead {
		top: -@_sap_m_PlanningCalendarHeader_ActionsToolbarHeight;
	}

	/* If the actions toolbar are hidden, then the navigation toolbar will */
	/* be on top by default. */
	&.sapMSinglePCActionsHidden {

		.sapMPCHead {
			top: 0;
		}
	}
}

.sapUiSizeCompact {

	.sapMSinglePCStickyNavBarAndColHeaders {

		.sapMPCHead {
			top: -@_sap_m_PlanningCalendarHeader_ActionsToolbarHeight_Compact;
		}
	}
}

html.sap-desktop .sapMSinglePC .sapUiCalItem:not(.sapUiCalItemSel):hover {
	background-color: darken(@sapUiContentForegroundColor,5);
}

html.sap-desktop .sapMSinglePC .sapUiCalItem:not(.sapUiCalItemSel) :not(.sapUiCalWeekNum):hover {
	background-color: transparent;
}

.sapMSinglePCGridContent {
	background-color: @_sap_m_SinglePlanningCalendarGrid_HoursBackground;
}
/* END "SinglePlanningCalendar.less" */

/* START "SinglePlanningCalendarGrid.less" */
/* ================================================= */
/* CSS for control sap.m/SinglePlanningCalendarGrid  */
/* Base theme                                        */
/* ================================================= */

@_sap_m_SinglePlanningCalendarGrid_RowHeadersWidth: 4rem;
@_sap_m_SinglePlanningCalendarGrid_RowHeight: 4.3125rem;
@_sap_m_SinglePlanningCalendarGrid_CompactRowHeight: 3rem;

@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType01: @sapLegendColor1;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType02: @sapLegendColor2;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType03: @sapLegendColor3;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType04: @sapLegendColor4;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType05: @sapLegendColor5;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType06: @sapLegendColor6;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType07: @sapLegendColor7;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType08: @sapLegendColor8;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType09: @sapLegendColor9;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType10: @sapLegendColor10;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType11: @sapLegendColor11;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType12: @sapLegendColor12;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType13: @sapLegendColor13;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType14: @sapLegendColor14;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType15: @sapLegendColor15;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType16: @sapLegendColor16;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType17: @sapLegendColor17;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType18: @sapLegendColor18;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType19: @sapLegendColor19;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppType20: @sapLegendColor20;

@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType01: @sapLegendBackgroundColor1;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType02: @sapLegendBackgroundColor2;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType03: @sapLegendBackgroundColor3;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType04: @sapLegendBackgroundColor4;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType05: @sapLegendBackgroundColor5;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType06: @sapLegendBackgroundColor6;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType07: @sapLegendBackgroundColor7;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType08: @sapLegendBackgroundColor8;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType09: @sapLegendBackgroundColor9;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType10: @sapLegendBackgroundColor10;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType11: @sapLegendBackgroundColor11;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType12: @sapLegendBackgroundColor12;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType13: @sapLegendBackgroundColor13;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType14: @sapLegendBackgroundColor14;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType15: @sapLegendBackgroundColor15;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType16: @sapLegendBackgroundColor16;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType17: @sapLegendBackgroundColor17;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType18: @sapLegendBackgroundColor18;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType19: @sapLegendBackgroundColor19;
@_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType20: @sapLegendBackgroundColor20;

/*****************
	COLUMN HEADERS
*****************/
.sapMSinglePCColumnHeader {
	padding-left: @_sap_m_SinglePlanningCalendarGrid_RowHeadersWidth;

	overflow: hidden;

	/* TODO: Consider refactoring of the CSS of PlanningCalendar DatesRow CSS and fix the styles in one place */
	.sapUiCalItem:focus {
		outline: 0.0625rem dotted;
		outline-offset: -0.1875rem;

		.sapUiCalItemText {
			outline: none;
		}
	}

	.sapUiCalItem.sapUiCalItemNow {
		border: 0.125rem solid;

		&:not(:focus) {
			outline-offset: -0.1875rem;
		}

		.sapUiCalItemText,
		.sapUiCalItemSel.sapUiItemText {
			border: none;
			box-shadow: none;
		}
	}
}

.sapMSinglePCGrid.sapUiDnDDragging {
	opacity: 1;
}

/*****************
	GRID CONTENT
*****************/

/* ROW HEADERS */
.sapMSinglePCGridContent {
	display: flex;
	position: relative;
}

.sapMSinglePCGrid {
	cursor: default;
}

.sapMSinglePCRowHeaders {
	display: flex;
	flex-direction: column;
	font-size: 0.875rem;
}

.sapMSinglePCRowHeader {
	flex: 1;
	transform: translateY(-0.5rem);
}

.sapMSinglePCRowHeaderHidden {
	visibility: hidden;
}

.sapMSinglePCRowHeaderAMPM,
.sapMSinglePCNowMarkerAMPM {
	font-size: 0.625rem;
}

.sapMSinglePCRowHeaderAMPM {
	top: -0.1875rem;
	position: relative;
}

/* GRID COLUMNS */
.sapMSinglePCColumns,
.sapMSinglePCBlockersColumns {
	flex: 1;
	display: flex;
}

.sapMSinglePCBlockersColumns {
	margin-left: @_sap_m_SinglePlanningCalendarGrid_RowHeadersWidth;
	min-height: 1.75rem;
	position: relative;
}

.sapMSinglePCColumn,
.sapMSinglePCBlockersColumn {
	position: relative;
	flex: 1;
}

.sapMSinglePCRow,
.sapMSinglePCBlockersColumn {
	box-sizing: border-box;
	border: 0.0625rem solid;
	border-top-width: 0;
	height: @_sap_m_SinglePlanningCalendarGrid_RowHeight;
	position: relative;
	outline: none;
}

.sapMSinglePCBlockersColumn {
	border-top-width: 0.0625rem;
	border-bottom-width: 0.125rem;
	height: auto;
}

/* NOW MARKER */
.sapMSinglePCNowMarker {
	display: flex;
	align-items: center;
	font-size: 0.75rem;
	margin-top: -0.5rem;
	position: absolute;
	z-index: 1;
	right: 0;
	left: 0;
	pointer-events: none;
}

.sapMSinglePCRowHeader,
.sapMSinglePCNowMarkerText {
	box-sizing: border-box;
	display: block;
	padding-right: 0.5rem;
	text-align: end;
	width: @_sap_m_SinglePlanningCalendarGrid_RowHeadersWidth;
}

.sapMSinglePCNowMarker::after {
	content: '';
	border-top: 0.125rem solid;
	flex: 1;
}

.sapMSinglePCColumnToday .sapMSinglePCRow::before {
	content: '';
	box-sizing: border-box;
	border: 0.063rem solid;
	border-bottom: none;
	border-top: none;
	position: absolute;
	height: calc(~"100% + 0.0625rem");
	width: 100%;
}

/* SELECTED COLUMNS */
.sapMSinglePCGrid .sapUiCalColumnSelected,
.sapMSinglePCGrid .sapUiCalColumnSelected.sapMSinglePCColumnWeekend {
	background: var(--sapList_SelectionBackgroundColor);
}

.sapMSinglePCGrid .sapUiCalColumnSelected,
.sapMSinglePCGrid .sapUiCalColumnSelected.sapMSinglePCColumnWeekend {

	.sapMSinglePCRow::before {
		top: -0.625rem;
		position: absolute;
		bottom: 0;
		left: 0;
		right: 0;
		border: 0.0625rem solid var(--sapList_SelectionBorderColor);
		content: '';
		pointer-events: none;
		border-top: none;
		border-bottom: none;
	}

	.sapMSinglePCRow:last-child::before{
		border-bottom: 0.0625rem solid var(--sapList_SelectionBorderColor);
	}

	&.sapMSinglePCBlockersColumn::before {
		position: absolute;
		content: '';
		pointer-events: none;
		top: 0;
		bottom: 0;
		left: 0;
		right: 0;
		border: 0.0625rem solid var(--sapList_SelectionBorderColor);
		bottom: -0.0625rem;
		border-bottom: none;
	}
}

.sapMSinglePCGrid .sapUiCalColumnSelected.sapUiCalColumnSelectedStartBlocker::before {
	right: -0.0625rem;
	border-right: none;
}

.sapMSinglePCGrid .sapUiCalColumnSelected.sapUiCalColumnSelectedStartColumn {

	.sapMSinglePCRow::before {
		border-right: none;
	}
}

.sapMSinglePCGrid .sapUiCalColumnSelected.sapUiCalColumnSelectedEndBlocker::before {
	border-left: none;
	left: -0.0625rem;
}

.sapMSinglePCGrid .sapUiCalColumnSelected.sapUiCalColumnSelectedEndColumn .sapMSinglePCRow::before {
	border-left: none;
}

.sapMSinglePCGrid .sapUiCalColumnSelected.sapUiCalColumnSelectedBetweenBlocker::before {
	border-left: none;
	border-right: none;
	left: -0.1875rem;
	right: -0.1875rem;
}

.sapMSinglePCGrid .sapUiCalColumnSelected.sapUiCalColumnSelectedBetweenColumn .sapMSinglePCRow::before {
	border-left: none;
	border-right: none;
}

/* APPOINTMENTS */
.sapMSinglePCAppointments,
.sapMSinglePCBlockers {
	position: absolute;
	top: 0;
	right: 0.125rem;
	left: 0.125rem;
	bottom: 0;
	pointer-events: none;

	& > .sapMSinglePCAppointmentWrap.sapUiCalendarRowApps {
		z-index: 2;
	}

	.sapUiCalendarApp {

		&.sapUiCalendarAppSel::after {
			content: '';
			position: absolute;
			display: block;
			top: 0;
			bottom: 0;
			right: 0;
			left: 0;
			z-index: 0;
			color: var(--sapList_Active_TextColor);
			background-color: fade(@sapUiSelected, 90%);
			border-radius: 0 0.125rem 0.125rem 0.125rem;
		}

		.sapUiCalendarAppCont {
			.sapUiCalendarAppointmentWrapper,
			.sapUiIcon,
			.sapUiCalendarAppIcon,
			.sapUiCalendarAppTitle,
			.sapUiCalendarAppText {
				z-index: 1;
			}
		}
	}

	.sapMSinglePCAppointmentWrap:focus {

		.sapUiCalendarAppCont::after {
			z-index: 1;
		}
	}
}

/* space is needed for the day marker in day view when special dates */
.sapMSpecialDaysInDayView .sapMSinglePCBlockers {
	top: 0.3125rem;
}

html.sap-desktop {

	.sapMSinglePCBlockersRow > .sapMSinglePCBlockersColumns > .sapMSinglePCBlockersColumn:focus::after,
	.sapMSinglePCGridContent .sapMSinglePCColumns .sapMSinglePCRow:focus::after {
		content: '';
		position: absolute;
		top: 0.0625rem;
		left: 0.0625rem;
		right: 0.0625rem;
		bottom: 0.0625rem;
		border: 0.0625rem dotted var(--sapContent_FocusColor);
		pointer-events: none;
		z-index: 3;
	}

	.sapMSinglePCBlockers,
	.sapMSinglePCAppointments {

		.sapUiCalendarApp {

			&:hover::after {
				background-color: var(--sapList_Hover_Background);
			}
		}
	}
}

.sapMSinglePCGrid {
	.sapUiCalendarAppointmentWrapper,
	.sapUiCalendarAppTitle,
	.sapUiCalendarAppText {
		width: 100%;
		margin-right: 0.125rem;

		& + .sapUiCalendarAppArrowIconRight {
			margin-left: 0.25rem;
		}
	}
}

.sapMSinglePCAppointments,
.sapMSinglePCBlockersColumns {

	.sapUiCalendarAppCont {
		display: flex;
	}

	.sapUiCalendarAppArrowIconRight {
		top: 0;
		right: 0;
		padding-right: 0.125rem;
	}

	.sapUiCalendarAppArrowIconLeft {
		top: 0;
		left: 0;
		padding-left: 0.25rem;
	}

	.sapUiCalendarAppIcon {
		left: 0;
		margin-right: 0;
		padding-left: 0.375rem;
	}
}

.sapMSinglePCBlockersColumns {

	.sapUiCalendarAppCont {
		align-items: center;
	}

	.sapUiCalendarApp > .sapUiCalendarAppCont > .sapUiCalendarAppIcon {
		top: 0;
	}

	.sapUiCalendarAppIcon + .sapUiCalendarAppTitle {
		padding-left: 0.375rem;
	}
}

.sapMSinglePCAppointments {

	.sapUiCalendarAppArrowIconLeft,
	.sapUiCalendarAppArrowIconRight {
		padding-top: 0.25rem;
	}
}

.sapMSinglePCAppointmentWrap.sapUiCalendarRowApps {
	/* TODO: this styles will be removed when Appointments refactoring occured */
	box-sizing: border-box;
	padding: 0.0625rem;
	position: absolute;
	pointer-events: all;

	&:focus {
		outline: none;

		.sapUiCalendarAppCont::after {
			position: absolute;
			top: 0.0625rem;
			left: 0.0625rem;
			right: 0.0625rem;
			bottom: 0.0625rem;
			content: '';
			border: 0.0625rem dotted var(--sapContent_FocusColor);
			pointer-events: none;
		}
	}

	.sapUiCalendarApp {
		border-left: 0;
		position: relative;
		min-width: auto;
		margin-left: 0.25rem;
		height: calc(~"100% - 0.0625rem");
		-webkit-user-select: none;
		-moz-user-select: none;
		user-select: none;

		& > .sapUiCalendarAppCont,
		&:active > .sapUiCalendarAppCont,
		&.sapUiCalendarAppSel > .sapUiCalendarAppCont {
			pointer-events: none;
		}

		.sapUiCalendarAppTitle {
			padding-top: 0.2rem;
		}

		.sapUiCalendarAppointmentWrapper {
			> .sapUiCalendarAppText,
			> .sapUiCalendarAppTitle {
				white-space: pre-line;
				word-break: break-all;
				overflow: hidden;
				line-height: 1rem;
			}
		}
	}
}

.sapUiSizeCompact {

	.sapMSinglePCBlockersColumns .sapMSinglePCAppointmentWrap.sapUiCalendarRowApps .sapUiCalendarApp {
		height: 1.438rem;
	}

	.sapMSinglePCAppointmentWrap.sapUiCalendarRowApps .sapUiCalendarApp .sapUiCalendarAppTitle {
		padding-top: 0.1rem;
	}

	.sapMSinglePCGrid .sapUiCalendarAppIcon {
		top: 0.1rem;
	}
}

.sapMSinglePCBlockersColumns .sapMSinglePCAppointmentWrap.sapUiCalendarRowApps .sapUiCalendarApp {
	height: 2rem;

	.sapUiCalendarAppTitle {
		padding-top: 0;
	}
}

.sapUiCalendarAppCont .sapUiIcon .sapUiCalendarAppArrowIcon {
	font-size: 0.75rem;
	color: var(--sapContent_LabelColor);
}

.sapMSinglePCGrid .sapUiCalendarRowApps {
	/* TODO: this styles will be removed when Appointments refactoring occured */
	position: absolute;
	min-height: auto;
	height: auto;
	overflow: visible;
}

.sapMSinglePCGrid .sapUiAppCreate {
	border-top: 0.125rem solid var(--sapContent_ForegroundBorderColor);
	border-bottom: 0.125rem solid var(--sapContent_ForegroundBorderColor);
	border-right: 0.125rem solid var(--sapContent_ForegroundBorderColor);
}

.sapMSinglePCColumnHeader.sapUiCalRow,
.sapMSinglePCColumnHeader.sapUiCalRow.sapUiCalNoNameLine .sapUiCalItem {
	height: 2.75rem;
}

.sapMSinglePC .sapUiCalNoNameLine .sapUiCalItem > .sapUiCalItemText {
	padding-top: 0.75rem;
}

.sapUiSizeCompact {

	.sapMSinglePCColumnHeader.sapUiCalRow,
	.sapMSinglePCColumnHeader.sapUiCalRow .sapUiCalItem {
		height:2rem;
	}

	/* TODO: Consider refactoring of the CSS of PlanningCalendar DatesRow CSS and fix the styles in one place */
	.sapMSinglePCColumnHeader {

		.sapUiCalItem {
			display: flex;
			flex-direction: row-reverse;
			align-items: center;
			text-align: start;

			.sapUiCalItemText,
			.sapUiCalDayName {
				padding: 0;
				line-height: 2rem;
				height: auto;
				position: static;
			}

			.sapUiCalDayName {
				text-align: end;
				padding-right: 0.5rem;
			}
		}
	}

	.sapMSinglePCRow {
		height: @_sap_m_SinglePlanningCalendarGrid_CompactRowHeight;
	}

	.sapMSinglePCColumnHeader .sapUiCalItem:focus > .sapUiCalItemText {
		outline: none;
	}
}

.sapMSinglePCGrid .sapUiCalendarAppIcon {
	min-width: 1rem;
	position: relative;
	top: 0.2rem;
}

.sapMSinglePCOverlay {
	position: absolute;
	height: 100%;
	width: 100%;
	visibility: hidden;
}

.sapMSinglePCOverlayDragging {
	z-index: 3;
	visibility: visible;
}

.sapMSinglePCPlaceholder {
	height: 2.15625rem;
}

.sapUiSizeCompact .sapMSinglePCPlaceholder {
	height: 1.5rem;
}

.sapMSinglePCBlockersColumns .sapMSinglePCOverlay {
	display: flex;
}

.sapMSinglePCBlockersColumns .sapMSinglePCPlaceholder {
	flex: 1;
	height: auto;
}

.sapMSinglePCAppointmentWrap::before {
	content: '';
	height: calc(~"100% - 0.1875rem");
	position: absolute;
	border-radius: 0.125rem 0 0 0.125rem;
	border-left: 0.3125rem solid;
	border-left-color: inherit;
	box-sizing: border-box;
	z-index: 1;
}

.sapMSinglePCAppointments .sapMSinglePCAppointmentWrap::before {
	height: calc(~"100% - 0.1875rem");
	min-height: 0.125rem;
}

.sapMSinglePCBlockers .sapMSinglePCAppointmentWrap::before {
	height: calc(~"100% - 0.3875rem");
}

.sapMSinglePCAppResizeHandleBottom,
.sapMSinglePCAppResizeHandleTop {
	width: 100%;
	height: 0.3125rem;
	cursor: row-resize;
	display: block;
	position: absolute;
	z-index: 1;
}

.sapMSinglePCAppResizeHandleBottom {
	bottom: 0;
}

.sapMSinglePCAppResizeHandleTop {
	top: 0;
}

.calendarRowTypeColorMixIn(@class, @iitemtypeclass, @color, @bcolor){

	.sapMSinglePCAppointments, .sapMSinglePCBlockers {

		.@{class}.sapMSinglePCAppointmentWrap {
			border-left-color: @color;
		}
	}

	.sapMSinglePCBlockers, .sapMSinglePCAppointments {

		&.sapUiCalendarRowVisFilled > .@{class}.sapUiCalendarRowApps > .sapUiCalendarApp:not(.sapUiCalendarAppSel) > .sapUiCalendarAppCont {
			background-color: fade(@bcolor, 90%);
		}
	}
	/* Special Dates */
	.sapMSinglePCGrid .@{iitemtypeclass} {
		/* day view */
		.sapMSinglePCBlockersColumn {
			box-shadow: inset 0 0.25rem 0 @color;
		}
	}
}

.calendarRowTypeColorMixIn(sapUiCalendarAppType01, sapUiCalItemType01, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType01, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType01);
.calendarRowTypeColorMixIn(sapUiCalendarAppType02, sapUiCalItemType02, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType02, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType02);
.calendarRowTypeColorMixIn(sapUiCalendarAppType03, sapUiCalItemType03, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType03, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType03);
.calendarRowTypeColorMixIn(sapUiCalendarAppType04, sapUiCalItemType04, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType04, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType04);
.calendarRowTypeColorMixIn(sapUiCalendarAppType05, sapUiCalItemType05, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType05, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType05);
.calendarRowTypeColorMixIn(sapUiCalendarAppType06, sapUiCalItemType06, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType06, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType06);
.calendarRowTypeColorMixIn(sapUiCalendarAppType07, sapUiCalItemType07, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType07, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType07);
.calendarRowTypeColorMixIn(sapUiCalendarAppType08, sapUiCalItemType08, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType08, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType08);
.calendarRowTypeColorMixIn(sapUiCalendarAppType09, sapUiCalItemType09, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType09, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType09);
.calendarRowTypeColorMixIn(sapUiCalendarAppType10, sapUiCalItemType10, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType10, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType10);
.calendarRowTypeColorMixIn(sapUiCalendarAppType11, sapUiCalItemType11, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType11, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType11);
.calendarRowTypeColorMixIn(sapUiCalendarAppType12, sapUiCalItemType12, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType12, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType12);
.calendarRowTypeColorMixIn(sapUiCalendarAppType13, sapUiCalItemType13, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType13, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType13);
.calendarRowTypeColorMixIn(sapUiCalendarAppType14, sapUiCalItemType14, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType14, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType14);
.calendarRowTypeColorMixIn(sapUiCalendarAppType15, sapUiCalItemType15, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType15, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType15);
.calendarRowTypeColorMixIn(sapUiCalendarAppType16, sapUiCalItemType16, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType16, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType16);
.calendarRowTypeColorMixIn(sapUiCalendarAppType17, sapUiCalItemType17, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType17, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType17);
.calendarRowTypeColorMixIn(sapUiCalendarAppType18, sapUiCalItemType18, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType18, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType18);
.calendarRowTypeColorMixIn(sapUiCalendarAppType19, sapUiCalItemType19, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType19, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType19);
.calendarRowTypeColorMixIn(sapUiCalendarAppType20, sapUiCalItemType20, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppType20, @_sap_m_SinglePlanningCalendar_sapUiCalendarAppBackgroundType20);
/* END "SinglePlanningCalendarGrid.less" */

/* START "SinglePlanningCalendarMonthGrid.less" */
/* ====================================================== */
/* CSS for control sap.m/SinglePlanningCalendarMonthGrid  */
/* Base theme                                             */
/* ====================================================== */

.sapMSPCMonthGrid > :not(.sapMSinglePCGridContent) {
	margin-left: 4rem;
}

.sapMSPCMonthGrid .sapMSinglePCGridContent {
	flex-direction: column;
}

.sapMSPCMonthWeek {
	display: flex;
}

.sapMSPCMonthWeek .sapMSPCMonthDays {
	display: flex;
	flex-direction: row;
	flex-grow: 1;
	position: relative;
}

.sapMSPCMonthGrid .sapMSPCMonthWeek .sapMSPCMonthDays > .sapMSinglePCBlockers {
	left: 0;
	right: 0;
}

.sapMSPCMonthWeek > .sapMSPCMonthWeekNumber {
	text-align: right;
	box-sizing: border-box;
	color: var(--sapTextColor);
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	background-color: lighten(@sapUiContentForegroundColor,7);

	&:hover {
		background-color: var(--sapList_Hover_Background);
		cursor: pointer;
	}
}

.sapMSPCMonthDay {
	flex: 1 1;
	display: flex;
	flex-direction: column;
	border: 0.0625rem solid transparent;
	border-left-color: var(--sapList_BorderColor);
	border-top-color: var(--sapList_BorderColor);
	box-sizing: border-box;
	background-color: var(--sapList_Background);
	position: relative;
	outline: none;
}

.specialDateIndicator {
	width: 1.75rem;
	height: 0.188rem;
	border-radius: 0.125rem;
	position: absolute;
	top: 0;
	right: 0;
	margin: 0.063rem;
}

.sapMSPCMonthGrid .sapMSPCMonthDayNames {
	display: flex;
}

.sapMSPCMonthGrid .sapMSPCMonthDayNames > .sapUiCalWH {
	flex: 1 1;
}

.sapMSPCMonthDay.nonWorkingTimeframe {
	background-color: var(--sapList_AlternatingBackground);
}

.sapMSPCMonthNowMarker {
	border: 0.125rem solid var(--sapLegend_CurrentDateTime);
	pointer-events: none;
	height: inherit;

	.sapMSPCMonthDayNumber {
		padding-top: 0.4rem;
	}
}

html.sap-desktop .sapMSPCMonthDay.sapMSPCMonthDayToday:focus .sapMSPCMonthNowMarker::after {
	content: '';
	position: absolute;
	top: 0.0625rem;
	left: 0.0625rem;
	right: 0.0625rem;
	bottom: 0.0625rem;
	border: 0.0625rem dotted var(--sapContent_FocusColor);
	pointer-events: none;
	z-index: 3;
}

.sapMSPCMonthDay.sapMSPCMonthDayToday .specialDateIndicator {
	top: 0.125rem;
	right: 0.125rem;
}

.sapMSPCMonthDayNumber {
	text-align: end;
	height: 1.75rem;
	padding-top: 0.5rem;
	padding-right: 0.375rem;
	box-sizing: border-box;
	color: var(--sapTextColor);
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
}

.sapMSPCMonthGrid .sapMSPCMonthLnkMore {
	position: absolute;
	bottom: 0;
	right: 0;
	padding-right: 0.125rem;
	text-align: end;
	height: 2.125rem;
	line-height: 2.125rem;
	pointer-events: auto;
}

.sapMSPCMonthGrid .sapMSinglePCPlaceholder {
	height: 100%;
}

.sapMSPCMonthDay {
	height: 8.25rem;
	min-width: 2.3125rem;
}

.sapMSPCMonthWeek > .sapMSPCMonthWeekNumber {
	width: 4rem;
	max-width: 4rem;
	padding-top: 0.25rem;
	padding-right: 0.375rem;
}

.specialDateIndicator {
	width: 1.75rem;
	height: 0.188rem;
}

.sapMSPCMonthGrid .sapMSinglePCAppointmentWrap.sapUiCalendarRowApps .sapUiCalendarApp {
	height: 2rem;
}

.sapMSPCMonthGrid .sapMSinglePCBlockers .sapUiCalendarAppCont {
	display: flex;
	align-items: center;
}

.sapMSPCMonthGrid .sapUiCalendarAppIcon + .sapUiCalendarAppTitle,
.sapMSPCMonthGrid .sapUiCalendarAppIcon + .sapUiCalendarAppTitle + .sapUiCalendarAppText {
	padding-left: 0.375rem;
}

.sapMSPCMonthGrid .sapUiCalendarAppIcon {
	top: initial;
}

html.sap-desktop .sapMSPCMonthDay:not(.sapMSPCMonthDayToday):focus::after {
	content: '';
	position: absolute;
	top: 0.0625rem;
	left: 0.0625rem;
	right: 0.0625rem;
	bottom: 0.0625rem;
	border: 0.0625rem dotted var(--sapContent_FocusColor);
	pointer-events: none;
	z-index: 3;
}

.sapMSPCMonthDay.sapMSPCMonthDaySelected.sapMSPCMonthDayToday .sapMSPCMonthNowMarker::before {
	content: '';
	position: absolute;
	top: 0.165rem;
	left: 0.165rem;
	right: 0.165rem;
	bottom: 0.165rem;
	border: 0.0625rem solid var(--sapList_SelectionBorderColor);
	pointer-events: none;
	z-index: 3;
}

.sapMSPCMonthDay.sapMSPCMonthDaySelected:not(.sapMSPCMonthDayToday) {
	border: 0.0625rem solid var(--sapList_SelectionBorderColor);
}

.sapMSPCMonthDay.sapMSPCMonthDaySelected {
	background: var(--sapList_SelectionBackgroundColor);
}

.sapMSPCMonthDay.sapMSPCMonthDaySelected:hover {
	background: var(--sapList_Hover_SelectionBackground);
}

.sapUiSizeCompact {

	.sapMSPCMonthDay {
		height: 7.875rem;
		min-width: 2.3125rem;
	}

	.sapMSPCMonthDayNumber {
		height: 1.5rem;
		padding-top: 0.313rem;
	}

	.sapMSPCMonthGrid .sapMSinglePCAppointmentWrap.sapUiCalendarRowApps .sapUiCalendarApp {
		height: 1.438rem;
	}

	.sapMSPCMonthGrid .sapMSinglePCAppointmentWrap.sapUiCalendarRowApps .sapUiCalendarApp .sapUiCalendarAppTitle {
		padding-top: 0.1rem;
		padding-top: 0;
	}

	.sapMSPCMonthGrid {

		.sapUiCalendarAppCont {

			.sapUiCalendarAppArrowIcon {
				top: 0;
				left: 0;
				padding-left: 0.25rem;
			}
		}
	}

	.sapMSPCMonthGrid .sapUiCalendarAppIcon {
		top: initial;
	}

	.sapMSPCMonthGrid .sapMSPCMonthLnkMore {
		height: 1.563rem;
		line-height: 1.563rem;
	}
}

.sapMSPCMonthGrid {

	.sapUiCalendarAppCont {

		.sapUiCalendarAppArrowIcon {
			top: 0;
			left: 0;
			padding-left: 0.25rem;
		}
	}
}

html.sap-desktop .sapMSPCMonthDaysapMSPCMonthGrid .sapMSPCMonthLnkMore {
	right: 0.25rem;
}

html.sap-desktop .sapMSPCMonthDaysapMSPCMonthGrid .specialDateIndicator {
	top:  0.25rem;
	right:  0.25rem;
}

html.sap-phone {

	.sapMSPCMonthGrid > :not(.sapMSinglePCGridContent) {
		margin-left: 0;
	}

	.sapMSPCMonthGrid .sapMSPCMonthWeek > .sapMSPCMonthWeekNumber {
		width: 0;
		max-width: 0;
		padding-left: 0;
		padding-right: 0;
	}

	.sapMSPCMonthDays.sapMSPCMonthDaysMax0,
	.sapMSPCMonthDays.sapMSPCMonthDaysMax1,
	.sapMSPCMonthDays.sapMSPCMonthDaysMax2 {
		height: 6.125rem;

		.sapMSPCMonthDay {
			height: 100%;
		}
	}

	.sapUiSizeCompact {

		.sapMSPCMonthDays.sapMSPCMonthDaysMax0,
		.sapMSPCMonthDays.sapMSPCMonthDaysMax1,
		.sapMSPCMonthDays.sapMSPCMonthDaysMax2 {
			height: 4.675rem;

			.sapMSPCMonthDay {
				height: 100%;
			}
		}

		.sapMSPCMonthDays.sapMSPCMonthDaysMax3 {
			height: 6.275rem;

			.sapMSPCMonthDay {
				height: 100%;
			}
		}
	}
}

@media (max-width: 42rem) {

	.sapMSPCMonthWeek > .sapMSPCMonthWeekNumber {
		width: 10%;
	}

	.sapMSPCMonthDay {
		width: 12.8571%;
	}

	.sapMSPCMonthGrid .sapMSPCMonthDayNames > .sapUiCalWH {
		min-width: 12.8571%;
		width: 12.8571%;
	}

	.sapMSPCMonthGrid > :not(.sapMSinglePCGridContent) {
		margin-left: 10%;
	}
}

/* Disable text selection while dragging the cursor */
.sapMSinglePCGrid * {
	-webkit-user-select: none; /* Safari */
	-ms-user-select: none; /* IE 10 and IE 11 */
	user-select: none; /* Standard syntax */
}
/* END "SinglePlanningCalendarMonthGrid.less" */

/* START "Slider.less" */
/* ============================= */
/* CSS for control sap.m/Slider  */
/* Base theme                    */
/* ============================= */

/* Slider commons styles */

@_sap_m_Slider_Height: 3rem;
@_sap_m_Slider_Height_Compact: 2rem;
@_sap_m_Slider_Progress: .1875rem;
@_sap_m_Slider_Progress_Border: 0px;
@_sap_m_Slider_Top_Bottom_Padding: calc(@_sap_m_Slider_Height - @_sap_m_Slider_Progress - (2 * @_sap_m_Slider_Progress_Border));
@_sap_m_Slider_Top_Bottom_Padding_Compact: calc(@_sap_m_Slider_Height_Compact - @_sap_m_Slider_Progress - (2 * @_sap_m_Slider_Progress_Border));

.sapMSlider {
	display: inline-block;
	vertical-align: top;
	box-sizing: border-box;
	position: relative;
	height: @_sap_m_Slider_Height;
	padding: calc(@_sap_m_Slider_Top_Bottom_Padding ~"/" 2) 1.0625rem;
	z-index: 0;
}

.sapMSliderInner {
	width: 100%;
	height: @_sap_m_Slider_Progress;
	min-width: 4rem;
	border-radius: 0.25rem;
	position: relative;
	background-repeat: no-repeat;
	z-index: 1;
	box-sizing: border-box;
}

.sapUiLocalBusy .sapMSliderInner {
	z-index: 0;
}

.sapMSliderProgress {
	height: 100%;
	border-radius: 0.25rem;
}

.sapMSliderProgress,
.sapMSliderHandle {
	position: absolute;
}

.sapMSliderHandle {
	box-sizing: border-box;
	display: block;
	outline: none;
}

.sapMSliderInput {
	display: none;
}

.sapMSliderTickmarks {
	margin: 0;
	padding: 0;
	list-style: none;
	position: absolute;
	top: -0.75rem;
	left: 0;
	right: 0;
	display: flex;
	height: 1.75rem;
	z-index: 0;
}

.sapMSliderTick {
	display: inline-block;
	height: 1rem;
	border-left: 0.0625rem solid var(--sapField_BorderColor);
	margin: 0.4375rem 0 0 -0.0625rem;
}

.sapMSliderTickLabel {
	display: inline-block;
	position: absolute;
	width: 0.0625rem;
	background-color: var(--sapField_BorderColor);
	margin: 0 0 0 -0.0625rem;

	&:last-child {
		right: 0;
	}
}

.sapMSliderLabel {
	font-size: 0.75rem;
	padding-top: 0.5rem;
	display: block;
	width: 5rem;
	text-align: center;
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
	position: absolute;
	top: 1.75rem;
	left: -2.5rem;
	direction: ltr;
}

/* slider progress bar */
.sapMSliderProgress {
	background: var(--sapField_Active_BorderColor);

	&::before {
		content: '';
		position: absolute;
		width: 100%;
		height: 500%;
		left: 50%;
		top: 50%;
		transform: translate(-50%, -50%);
	}
}

/* slider handle */
.sapMSliderHandle {
	margin-left: -1rem;
	top: -0.875rem;
	height: 2rem;
	width: 2rem;
	direction: ltr;
	cursor: pointer;

	/* some browsers like Samsung Galaxy S4 with Android 4.2 does not support the "border-radius" shorthand property
	but does support the long-hand properties for each corner like "border-top-left-radius" */
	border-top-left-radius: 1rem;
	border-top-right-radius: 1rem;
	border-bottom-right-radius: 1rem;
	border-bottom-left-radius: 1rem;

	&:focus::before {
		content: '';
		position: absolute;
		border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
		border-radius: 50%;
		pointer-events: none;
		top: -0.25rem;
		bottom: -0.25rem;
		left: -0.25rem;
		right: -0.25rem;
	}
}

/* compact size */

/* slider */
.sapUiSizeCompact {

	.sapMSlider {
		height: @_sap_m_Slider_Height_Compact;
		padding: calc(@_sap_m_Slider_Top_Bottom_Padding_Compact ~"/" 2) .875rem;
	}

	/* slider handle */
	.sapMSliderHandle {
		height: 1.625rem;
		width: 1.625rem;
		margin-left: -0.8125rem;
		top: -0.75rem;
	}

	.sapMSliderTickmarks {
		height: 1rem;
		top: -0.375rem;
	}

	.sapMSliderLabel {
		padding-top: 0.3125rem;
		top: 1rem;
	}

	.sapMSliderTick {
		margin-top: 0.125rem;
		height: 0.75rem;
	}
}
/* END "Slider.less" */

/* START "SliderTooltip.less" */
/* ==================================== */
/* CSS for control sap.m/SliderTooltip  */
/* Base theme                           */
/* ==================================== */

/* Slider Tooltip styles          */
.sapMSliderTooltip {
	position: relative;
	display: block;
	flex-shrink: 0;
	height: 1.625rem;
	padding-left: 0.5rem;
	padding-right: 0.5rem;
	text-align: center;
	visibility: visible;
	box-shadow: @sapUiShadowLevel1;
	background-color: var(--sapGroup_ContentBackground);
}

.sapMSliderTooltipInput {
	height: 100%;
	width: 100%;
	border: none;
	padding: 0;
	margin: 0;
	box-sizing: border-box;
	text-align: center;
	outline: none;
	-moz-appearance: textfield;
}

.sapMSliderTooltipInput::-webkit-outer-spin-button,
.sapMSliderTooltipInput::-webkit-inner-spin-button {
	-webkit-appearance: none;
	margin: 0;
}

.sapMSliderTooltipNonEditable {
	pointer-events: none;
}

.sapMSliderTooltipInput:-webkit-inner-spin-button,
.sapMSliderTooltipInput:-webkit-outer-spin-button {
	-webkit-appearance: none;
	margin: 0;
}

.sapMSliderTooltipErrorState {
	border: 2px solid;
}
/* END "SliderTooltip.less" */

/* START "SliderTooltipContainer.less" */
/* ============================================= */
/* CSS for control sap.m/SliderTooltipContainer  */
/* Base theme                                    */
/* ============================================= */

.sapMSliderTooltipContainer {
	display: flex;
	position: absolute;
	justify-content: space-between;
	transition-property: left, right;
}

.sapMSliderTooltipContainerDisabled {
	opacity: 0.5;
	pointer-events: none;
}
/* END "SliderTooltipContainer.less" */

/* START "SlideTile.less" */
/* ================================ */
/* CSS for control sap.m/SlideTile  */
/* Base theme                       */
/* ================================ */

.sapMST {
	background: transparent;
	position: relative;
	display: inline-block;
	font-size: 0;
	border-radius: 0.25rem;
	outline: none;
	box-shadow: @sapUiShadowLevel0;

	.sapMSTOverflowHidden {
		overflow: hidden;
		position: relative;
		height: 100%;
		width: 100%;
		border-radius: 0.23rem 0.23rem 0.25rem 0.25rem;
	}

	&.sapMSTScopeActions {

		.sapMSTMoreIcon {
			height: 1rem;
			position: absolute;
			right: 0.75rem;
			bottom: 0.563rem;
			z-index: 1;
			color: var(--sapButton_IconColor);
			pointer-events: none;
			background-color: transparent;
			border: 0;

			.sapMBtnIcon {
				bottom: 0.6rem;
				left: 0.9rem;
				border: 0;
				color: var(--sapButton_Lite_TextColor);
			}

			&:focus {
				outline: none;
			}
		}

		.sapMSTRemoveButton {
			width: 1.375rem;
			height: 1.375rem;
			border-radius: 50%;
			padding: 0;
			position: absolute;
			top: -0.5rem;
			right: -0.5rem;
			z-index: 3;
			border: 1px solid var(--sapButton_BorderColor);
			background-color: var(--sapButton_Background);
			color: var(--sapButton_TextColor);
			overflow: visible;

			.sapMBtnFocusDiv {
				display: none;
			}

			.sapMBtnInner {
				width: 100%;
				height: 100%;
				min-width: 0;
				border-radius: 50%;
				border: none;
				outline-offset: -2px;
			}

			.sapMBtnIcon {
				font-size: 0.75rem;
				margin: 0;
				width: 100%;

				&::before {
					position: relative;
					top: -0.125rem;
				}
			}

			&:focus > .sapMFocusable {
				outline: none;
				border-radius: 50%;
			}

			/* The touch area has a different size and is slightly moved to the left and bottom */
			&::before {
				content: '';
				color: var(--sapButton_IconColor);
				position: absolute;
				top: -1px;
				bottom: -0.6875rem;
				left: -0.6875rem;
				right: -0.0625rem;
			}
		}
	}
}

.sapMST {

	.sapMGT.sapMGTStateLoading {

		.sapMTileCnt.News {
			background-color: transparent;
		}
	}
}

.sapMST.sapMSTGridContainerOneRemGap.TwoByOne {
	width: 23rem;
}

.sapMST.sapMSTGridContainerOneRemGap.TwoByHalf {
	width: 23rem;
}

.sapMTileSmallPhone.sapMST.sapMSTGridContainerOneRemGap.TwoByOne {
	width: 19.5rem;
}

.sapMTileSmallPhone.sapMST.sapMSTGridContainerOneRemGap.TwoByHalf {
	width: 19.5rem;
}

.sapMST.sapMTileSmallPhone {

	.sapMGT.sapMGTStateLoading.TwoByOne {

		.sapMTileCntContentShimmerPlaceholderItemTextFooter {
			width: 5.875rem;
		}
	}
}

.sapUiSizeCompact{

	.sapMST {

		.sapMSTMoreIcon {

			.sapMBtnIcon {
				left: 0.3rem;
				bottom: 0.4rem;
			}
		}
	}
}

.sapMSTWrapper {
	position: absolute;
	left: 100%;
}

.sapMSTWrapper>.sapMGT,
.sapMSTWrapper>.sapMGT:hover,
html.sap-phone .sapMGTHvrOutln:active,
html.sap-tablet .sapMGTHvrOutln:active {
	box-shadow: none !important;
}

.sapMST.OneByOne {
	width: 11rem;
	height: 11rem;
}

.sapMST.TwoByOne {
	width: 22.5rem;
	height: 11rem;
}

.sapMST.Stretch {
	width: 11rem;
	height: 11rem;
}

.sapMGTTileStretch {
	height: 100%;
	width: 100%;
}

.sapMGTTileStretch .sapMGT {
	height: 100%;
	width: 100%;
}

.sapMSTFocusDiv {
	position: absolute;
	pointer-events: none;
	left: 1px;
	top: 1px;
	right: 1px;
	bottom: 1px;
	border-radius: 0.125rem;
}

.sapMST:focus .sapMSTFocusDiv {
	border: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
}

/* Bulleted indicator of multiple tiles*/
.sapMSTBulleted {
	position: absolute;
	text-align: center;
	bottom: 0.0625rem;
	width: 100%;
	z-index: 1;
	cursor: pointer;
	display: flex;
	justify-content: center;
	align-items: center;
	height: 1.5rem;
	bottom: 0.25rem;

	.sapMSTIndicatorWrapper {
		display: flex;
		max-width: 7.5rem;
		overflow: hidden;

		.sapMSTIndicatorTapArea {
			min-width: 24px;
			min-height: 24px;
			display: flex;
			align-items: center;
			justify-content: center;
			transition: transform 0.5s ease
		}
	}
}

.sapMSTBulleted .sapMSTIndicator {
	display: inline-block;
	height: 0.5rem;
	width: 0.5rem;
	margin: 0 0.25rem;
	border-radius: 50%;
}

.sapMST .sapMSTIconClickTapArea {
	position: absolute;
	right: 0.75rem;
	top: 0.75rem;
	height: 2rem;
	width: 2rem;
	z-index: 3;
	background-color: transparent;
	border-radius: 50%;
	cursor: pointer;
}

.sapMST .sapMSTIconDisplayArea {
	position: absolute;
	right: 0.75rem;
	top: 0.75rem;
	height: 2rem;
	width: 2rem;
	z-index: 1;
	opacity: 0.6;
	overflow: hidden;
	border-radius: 50%;
}

.sapMST .sapMSTIconClickTapArea:hover ~.sapMSTIconDisplayArea {
	opacity: 0.8;
}

.sapMST.sapMSTIconPressed .sapMSTIconClickTapArea:hover ~.sapMSTIconDisplayArea {
	opacity: 1.0;
}

.sapMST .sapMSTIconNestedArea {
	right: 0.75rem;
	top: 0.75rem;
	line-height: 2rem;
	height: 2rem;
	z-index: 1;
	position: absolute;

	.sapUiIcon {
		width: 2rem;
		line-height: inherit;
	}
}

html.sap-phone .sapMST .sapMSTIconClickTapArea,
html.sap-tablet .sapMST .sapMSTIconClickTapArea {
	right: 0.25rem;
	top: 0.25rem;
	height: 3rem;
	width: 3rem;
}

.sapMTileSmallPhone {

	&.sapMST.OneByOne {
		width: 9.25rem;
		height: 9.25rem;
	}

	&.sapMST.TwoByOne {
		width: 19rem;
		height: 9.25rem;
	}

	&.sapMSTBulleted {
		bottom: 0.188rem;
	}
}

.sapMST {

	.sapMTileCnt.News {

		.sapMNwCCTxt {
			margin-top: 0;

			.sapMFT {
				display: -webkit-box;
				-webkit-box-orient: vertical;
				-webkit-line-clamp: 2;
				overflow: hidden;
				text-overflow: ellipsis;
			}
		}

		.sapMNwCSbh {

			.sapMFT {
				text-overflow: ellipsis;
				white-space: nowrap;
				overflow: hidden;
			}
		}
	}

	&.sapMSTIndicatorVisible {

		.sapMGenericTileFtrFld {
			top: 0.5rem;
		}
	}
}

.sapMST.sapMSTIndicatorVisible.Stretch {

	.sapMGenericTileFtrFldTxt {

		.sapMText {
			width: -webkit-calc(~'100% - 40px');
			font-family: var(--sapFontBoldFamily);
			font-size: var(--sapFontSize);
			line-height: normal;
			text-shadow: var(--sapContent_TextShadow);
		}
	}
}

.sapMST.sapMSTIndicatorVisible.Stretch.sapMTileSmallPhone {

	.sapMGenericTileFtrFldTxt {

		.sapMText {
			font-size: var(--sapFontSmallSize);
		}
	}
}

.sapMST.sapMSTIndicatorVisible.TwoByOne {

	.sapMGT.TwoByOne {

		.sapMTileCnt.News {
			height: 6.375rem;
			margin-top: 0;

			.sapMTileCntContent{
				height: 3rem;

				.sapMNwC {
					max-height: 3rem;

					.sapMNwCCTxt {
						max-height: 2rem;

						.sapMText {
							font-size: var(--sapFontSize);
							font-family: var(--sapFontBoldFamily);
						}

						.sapMFT {
							font-size: var(--sapFontSize);
							font-family: var(--sapFontBoldFamily);
						}
					}
				}
			}

			.sapMTileCntFtrTxt {
				bottom: 1.875rem;
				font-family: var(--sapFontFamily);
				font-size: var(--sapFontSmallSize);
				line-height: normal;
			}
		}
	}

	.sapMSTBulleted {
		bottom: 0.25rem;
	}

	.sapMGenericTileFtrFldTxt {

		.sapMText {
			width: 17.125rem;
			font-family: var(--sapFontBoldFamily);
			font-size: var(--sapFontSize);
			line-height: normal;
			text-shadow: var(--sapContent_TextShadow);
		}
	}
}

.sapMST.sapMSTIndicatorVisible.OneByOne {

	.sapMGT.OneByOne {

		.sapMGTHdrContent {
			height: 3.375rem;
			padding: 0;
		}

		.sapMGTContent {
			height: 7.625rem;

			.sapMTileCnt.News {
				height: 100%;
				margin: 0;

				.sapMTileCntContent {
					margin: 0.5rem 0 0.5rem 0;
					height: 2.625rem;

					.sapMFT {
						font-size: var(--sapFontSmallSize);
					}
				}

				.sapMTileCntFtrTxt {
					bottom: 3rem;
					font-family: var(--sapFontFamily);
					font-size: var(--sapFontSmallSize);
					line-height: normal;
				}
			}
		}
	}

	.sapMSTBulleted {
		bottom: 1rem;

		.sapMSTIndicatorWrapper {
			max-width: 6rem;
		}
	}

	.sapMGenericTileFtrFld {
		display: flex;
		flex-direction: column;
		gap: 0.25rem;

		.sapMGenericTileFtrFldTxt {
			padding: 0;

			.sapMText {
				width: 7.8125rem;
				font-family: var(--sapFontBoldFamily);
				font-size: var(--sapFontSmallSize);
				line-height: normal;
				text-shadow: var(--sapContent_TextShadow);
			}
		}
	}
}

.sapMST.sapMSTIndicatorVisible.sapMTileSmallPhone.TwoByOne {

	.sapMGT.TwoByOne {

		.sapMGTHdrContent {
			height: 3.1875rem;
			padding: 0;
		}

		.sapMGTContent {
			height: 6.125rem;

			.sapMTileCnt.News {
				height: 100%;

				.sapMTileCntContent {
					height: 2.5rem;

					.sapMNwC {
						height: 2.5rem;

						.sapMNwCCTxt {
							height: 1.75rem;
						}

						.sapMNwCSbh {
							height: 14px;
						}

						.sapMFT {
							font-size: var(--sapFontSmallSize);
						}
					}
				}
			}

			.sapMTileCntFtrTxt {
				bottom: 2.25rem;
			}
		}
	}

	.sapMSTBulleted {
		bottom: 0.5rem;
	}

	.sapMGenericTileFtrFldTxt {

		.sapMText {
			width: 13.6875rem;
			font-size: var(--sapFontSmallSize);
		}
	}
}

.sapMST.sapMSTIndicatorVisible.sapMTileSmallPhone.OneByOne {

	.sapMGT.OneByOne {

		.sapMGTContent {
			height: 5.875rem;

			.sapMTileCnt.News {
				height: inherit;
				margin-top: 0;

				.sapMTileCntContent {
					height: 2.625rem;
					margin-bottom: 0.125rem;

					.sapMNwC {
						height: inherit;
					}
				}

				.sapMTileCntFtrTxt {
					bottom: 1.6875rem
				}
			}
		}
	}

	.sapMSTBulleted {
		bottom: 0.25rem;

		.sapMSTIndicatorWrapper {
			max-width: 4.5rem;
		}
	}

	.sapMGenericTileFtrFldTxt {

		.sapMText {
			width: 6.125rem;
		}
	}
}
/* END "SlideTile.less" */

/* START "SplitButton.less" */
/* ================================== */
/* CSS for control sap.m/SplitButton  */
/* Base theme                         */
/* ================================== */

.sapMSB {
	display: inline-block;
	position: relative;
	vertical-align: top;
}

.sapMSB .sapMSBInner .sapMSBText:not(.sapMSBTextIE) .sapMBtnInner {
	margin-right: 2.5rem;
}

.sapMSB .sapMSBArrow {
	position: absolute;
	top: 0;
	right: 0;
}

.sapMSB .sapMSBArrow .sapMBtnIcon {
	font-size: 0.75rem;
	line-height: 2.25rem;
}

.sapUiSizeCompact .sapMSB .sapMSBArrow .sapMBtnIcon {
	line-height: 1.625rem;
}

.sapMSB .sapMSBText .sapMBtnInner {
	text-align: left;
	border-top-right-radius: 0;
	border-bottom-right-radius: 0;
}

.sapMSB.sapMBtnTransparent .sapMSBText:hover .sapMBtnInner {
	border-right-width: 0.0625rem;
	padding-right: 0.5rem;
}

.sapMTB-Transparent-CTX .sapMSB .sapMSBText:hover .sapMBtnDefault.sapMBtnInner {
	border-right-width: 0.0625rem;

	&.sapMBtnText {
		padding-right: 0.5rem;
	}
}

.sapMSB .sapMSBArrow .sapMBtnInner {
	border-top-left-radius: 0;
	border-bottom-left-radius: 0;
}

.sapMSB:focus {
	outline: none;
}

.sapMSB:focus .sapMSBInner::after {
	content: '';
	pointer-events: none;
	position: absolute;
}

/* ie */
.sapMSBTextIE.sapMSBText {
	padding-right: 2.5rem;
}

.sapUiSizeCompact {

	.sapMSB .sapMSBInner .sapMSBText:not(.sapMSBTextIE) .sapMBtnInner {
		margin-right: 2rem;
	}

	/* ie */
	.sapMSBTextIE.sapMSBText {
		padding-right: 2rem;
	}

	.sapMSB .sapMSBInner .sapMSBArrow .sapMBtnIconFirst {
		padding-left: 0;
		padding-right: 0;
	}

	.sapMTB-Transparent-CTX .sapMSB .sapMSBText:hover .sapMBtnDefault.sapMBtnInner {
		padding-right: 0.375rem;
	}
}

.sapMSB:focus .sapMSBInner:not(.sapMSBInnerDisabled)::after {
	top: 0.3125rem;
	bottom: 0.3125rem;
	left: 0.0625rem;
	right: 0.0625rem;
	border: 0.0625rem dotted var(--sapContent_FocusColor);
}

.sapUiSizeCompact {

	.sapMSB:focus .sapMSBInner:not(.sapMSBInnerDisabled)::after {
		top: 0.25rem;
		bottom: 0.25rem;
	}
}
/* END "SplitButton.less" */

/* START "SplitContainer.less" */
/* ===================================== */
/* CSS for control sap.m/SplitContainer  */
/* Base theme                            */
/* ===================================== */

@_sap_m_SplitContainer_AppHeight: 3333px; /* Ux decision: never show the header, so make this value large enough even for zoom-out cases */
@_sap_m_SplitContainer_HeaderHeight: 68px; /* should be 4.25rem */
@_sap_m_SplitContainer_MinHeightForHeader: (@_sap_m_SplitContainer_AppHeight + @_sap_m_SplitContainer_HeaderHeight);

.sapMSplitContainer {
	width: 100%;
	height: 100%;
	display: block;
	position: relative;
	overflow: hidden;
}

.sapMSplitContainerMaster {
	box-sizing: border-box;
	width: 20rem;
	float: left;
	left: 0;
	z-index: 1;
	background-color: var(--sapGroup_ContentBackground);
	border-right: 0.0625rem solid var(--sapGroup_ContentBorderColor);
}

.sapMSplitContainerDetail {
	box-sizing: border-box;
	height: 100%;
}

/* Default Master Animation Behavior */
.sapMSplitContainerStretchCompress > .sapMSplitContainerMaster,
.sapMSplitContainerShowHide:not(.sapMSplitContainerPortrait) > .sapMSplitContainerMaster,
.sapMSplitContainerPopover:not(.sapMSplitContainerPortrait) > .sapMSplitContainerMaster {
	border-radius: 0;
}

.sapMSplitContainerPortrait.sapMSplitContainerShowHide > .sapMSplitContainerMaster,
.sapMSplitContainerHideMode > .sapMSplitContainerMaster {
	display: inline-block;
	position: absolute;
	transform: translate3d(-330px,0,0);
	transition: all 0ms;
	z-index: 5;
	top: 0;
	height: 100%;
	box-shadow: @sapUiShadowLevel2;
	border-right: var(--sapGroup_ContentBorderColor);
	background-clip: padding-box;
}

.sapMSplitContainerPopover > .sapMPopoverCont > .sapMPopoverScroll > .sapMSplitContainerMaster {
	border-right: none;
}

.sapMSplitContainerPortrait.sapMSplitContainerShowHide > .sapMSplitContainerDetail,
.sapMSplitContainerHideMode > .sapMSplitContainerDetail,
.sapMSplitContainerPopover .sapMSplitContainerDetail {
	margin-left: 0;
}

.sapMSplitContainerPortrait.sapMSplitContainerShowHide > .sapMSplitContainerMasterVisible,
.sapMSplitContainerHideMode > .sapMSplitContainerMasterVisible {
	transform: translate3d(0,0,0);
	transition: all 300ms;
}

.sapMSplitContainerPortrait.sapMSplitContainerShowHide > .sapMSplitContainerMasterHidden,
.sapMSplitContainerHideMode > .sapMSplitContainerMasterHidden {
	transform: translate3d(-330px,0,0);
	transition: all 300ms;
	box-shadow: @sapUiShadowLevel2;
	visibility: hidden;
}

.sapMSplitContainerPortrait.sapMSplitContainerShowHide > .sapMSplitContainerMasterHidden.sapMSplitContainerNoTransition,
.sapMSplitContainerHideMode > .sapMSplitContainerMasterHidden.sapMSplitContainerNoTransition {
	transition: none;
}


.sapMSplitContainerPortrait.sapMSplitContainerStretchCompress > .sapMSplitContainerMaster {
	width: 320px !important;
	left: 0;
}

/* Keyframe Animations */
@keyframes sapMSplitContainerSlidein {

	from { width: 0px; }

	to { width: 320px; }
}

@keyframes sapMSplitContainerSlideout {

	from { width: 320px; }

	to { width: 0px; }
}

@keyframes fadeIn {

	from { opacity: 0; }

	to { opacity: 1; }
}

@keyframes fadeOut {

	from { opacity: 1; }

	to { opacity: 0.7; }
}

.sapMSplitContainerPortrait .sapMSplitContainerMasterBtnShow {
	display: block;
	animation: fadeIn 300ms;
}

.sapMSplitContainerPortrait .sapMSplitContainerMasterBtnHide {
	animation: fadeOut 150ms;
}

.sapMSplitContainerMasterBtnShow {
	display: block;
	animation: fadeIn 1ms;
}

.sapMSplitContainerMasterBtnHide {
	animation: fadeOut 5ms;
}

.sapMSplitContainerMasterBtnHidden {
	display: none;
}

.sapMSplitContainerMobile {
	width: 100%;
	height: 100%;
}

.sapMSplitContainer > .sapMSplitContainerDetail .sapMSplitContainerHiddenChild {
	display: none;
}

/* decrease to minimum transition and animation when animation is off */
/* there are 1ms and 5ms values, because the js code depend of the end of the animation */
html[data-sap-ui-animation='off'] {

	.sapMSplitContainerPortrait.sapMSplitContainerShowHide > .sapMSplitContainerMaster,
	.sapMSplitContainerHideMode > .sapMSplitContainerMaster {
		transition: all 1ms;
	}

	.sapMSplitContainerPortrait.sapMSplitContainerShowHide > .sapMSplitContainerMasterVisible,
	.sapMSplitContainerHideMode > .sapMSplitContainerMasterVisible {
		transition: all 1ms;
	}

	.sapMSplitContainerPortrait.sapMSplitContainerShowHide > .sapMSplitContainerMasterHidden,
	.sapMSplitContainerHideMode > .sapMSplitContainerMasterHidden {
		transition: all 1ms;
	}

	.sapMSplitContainerPortrait .sapMSplitContainerMasterBtnShow {
		animation: fadeIn 5ms;
	}

	.sapMSplitContainerPortrait .sapMSplitContainerMasterBtnHide {
		animation: fadeOut 5ms;
	}
}

.sapMSplitContainerMaster .sapMPageHeader {
	background-color: var(--sapGroup_ContentBackground);
	box-shadow: inset 0 -1px var(--sapGroup_ContentBorderColor);
}

/* footer separator has priority */
.sapMSplitContainerMaster footer {
	z-index: 6;
}

.sapMPopoverCont .sapMSplitContainerMaster::after {
	/* popover has its own border, do not show separator */
	display: none;
}
/* END "SplitContainer.less" */

/* START "StandardListItem.less" */
/* ======================================= */
/* CSS for control sap.m/StandardListItem  */
/* Base theme                              */
/* ======================================= */

.sapMSLI {
	height: 3rem;
}

.sapMSLI.sapMSLIThumbnail,
.sapMSLI.sapMSLINoIconInset,
.sapMSLI.sapMSLIWithDescription,
.sapMSLI.sapMSLINoTitleAdapt {
	height: 5rem;
}

.sapMSLI > .sapMLIBContent {
	white-space: nowrap;
	text-overflow: ellipsis;
	display: flex;
	align-items: center;
}

.sapMSLIDescriptionAndInfo {
	display: flex;
	overflow: hidden;
}

.sapMSLIDiv {
	display: flex;
	flex: auto;
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
}

.sapMSLIWithDescription > .sapMLIBContent > .sapMSLIDiv {
	flex-direction: column;
}

.sapMSLINoTitleAdapt:not(.sapMSLIWithDescription) .sapMSLIDiv {
	height: 2.5625rem;
}

.sapMSLITitle,
.sapMSLITitleOnly,
.sapMSLIDescription,
.sapMSLIDescriptionText {
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
	flex: 1 1 auto;
}

.sapMSLIThumbnail:not(.sapMSLIWrapping) > .sapMLIBContent > .sapMSLIDiv,
.sapMSLIDescriptionText {
	padding-bottom: 0.125rem;
}

.sapMSLITitle,
.sapMSLITitleOnly {
	font-size: var(--sapFontLargeSize);
	color: var(--sapList_TextColor);
}

.sapMSLIDescription,
.sapMSLIDescriptionText {
	font-size: @sapMFontMediumSize;
	color: var(--sapContent_LabelColor);
}

.sapMSLIWithDescription .sapMSLIDescription {
	padding-top: 0.5rem;
}

.sapMSLIImgIcon,
.sapMSLIImg,
.sapMSLIImgNoInsetIcon,
.sapMSLIImgNoInset {
	width: 3rem;
	height: 3rem;
	line-height: 3rem;
	flex: none;
}

.sapMSLIImgIcon,
.sapMSLIImgNoInsetIcon {
	font-family: var(----sapContent_IconFontFamily);
	font-size: 1.375rem;
	color: var(--sapContent_NonInteractiveIconColor);
}

.sapMSLIImgNoInsetIcon,
.sapMSLIImgNoInset,
.sapMSLIImg {
	font-size: 2.5rem;
	margin-right: 0.75rem;
}

.sapMLIBActive .sapMSLIImgNoInsetIcon,
.sapMLIBActive .sapMSLIImgIcon,
.sapMLIBActive .sapMSLIExpandCollapse {
	color: var(--sapList_Active_TextColor);
	text-shadow: none;
}

.sapMLIBActive .sapMSLIInfo:not(.sapMSLIInfoStateInverted) {
	text-shadow: none;
}

.sapMSLIDiv.sapMSLIInfoMiddle {
	flex-direction: row;
}

.sapMSLIInfo {
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
	text-align: right;
	margin: 0 0 0 1rem;
	font-size: @sapMFontMediumSize;
	min-width: 2rem;
}

.sapMSLIWrapping .sapMSLIInfo {
	white-space: normal;
	word-break: break-word;
}

.sapMSLIInfo:not(.sapMSLIInfoStateInverted) {
	text-shadow: var(--sapContent_TextShadow);
}

.sapMSLIInfoMiddle > .sapMSLIInfo {
	padding: 0;
}

/* The info is aligned right and therefore in RTL mode should be aligned on the left */
html[dir=rtl] .sapMSLIInfo {
	text-align: left;
}

.sapMSLIInfoNone {
	color: var(--sapContent_LabelColor);
}

.sapMSLIInfoError {
	color: var(--sapNegativeTextColor);
}

.sapMSLIInfoWarning {
	color: var(--sapCriticalTextColor);
}

.sapMSLIInfoSuccess {
	color: var(--sapPositiveTextColor);
}

.sapMSLIInfoInformation {
	color: var(--sapInformativeTextColor);
}

/* inverted info text */
.sapMSLIInfoStateInverted {
	font-size: var(--sapFontSmallSize);
	font-weight: bold;
	color: @sapUiContentContrastTextColor;
	border-radius: 0.25rem;
	box-sizing: border-box;
	height: 0.875rem;
	line-height: ~"calc(var(--sapFontSmallSize) - 0.0625rem);"
}

.sapMSLIInfo.sapMSLIInfoStateInverted {
	padding: 0.0625rem 0.25rem;
	text-align: center;
	text-shadow: var(--sapContent_ContrastTextShadow);
}

.sapMSLIInfoStateInverted.sapMSLIInfoNone {
	background: var(--sapNeutralElementColor);
}

.sapMSLIInfoStateInverted.sapMSLIInfoError {
	background: var(--sapNegativeElementColor);
}

.sapMSLIInfoStateInverted.sapMSLIInfoWarning {
	background: var(--sapCriticalElementColor);
}

.sapMSLIInfoStateInverted.sapMSLIInfoSuccess {
	background: var(--sapPositiveElementColor);
}

.sapMSLIInfoStateInverted.sapMSLIInfoInformation {
	background: var(--sapInformativeElementColor);
}

/* Wrapping */
.sapMSLI.sapMSLIWrapping {
	height: auto;
	min-height: 3rem;
}

.sapMSLIWithDescription.sapMSLIWrapping,
.sapMSLINoIconInset.sapMSLIWrapping,
.sapMSLINoTitleAdapt.sapMSLIWrapping {
	min-height: 5rem;
}

.sapMSLIWrapping .sapMSLITitle,
.sapMSLIWrapping .sapMSLITitleOnly,
.sapMSLIWrapping .sapMSLIDescription,
.sapMSLIWrapping .sapMSLIDescriptionText {
	white-space: normal;
	word-wrap: break-word;
}

.sapMSLINoTitleAdapt > .sapMLIBSelectS,
.sapMSLINoTitleAdapt > .sapMLIBSelectM,
.sapMSLIWithDescription > .sapMLIBSelectS,
.sapMSLIWithDescription > .sapMLIBSelectM,
.sapMSLIWrapping > .sapMLIBSelectS,
.sapMSLIWrapping > .sapMLIBSelectM,
.sapMSLIWrapping > .sapMLIBContent > .sapMSLIImgIcon,
.sapMSLIWrapping > .sapMLIBContent > .sapMSLIImg,
.sapMSLIWrapping > .sapMLIBContent > .sapMSLIImgNoInsetIcon,
.sapMSLIWrapping > .sapMLIBContent > .sapMSLIImgNoInset {
	align-self: flex-start;
	margin-top: 0.275rem;
}

.sapMSLIWrapping.sapMSLINoIconInset > .sapMLIBContent > .sapMSLIImgNoInsetIcon,
.sapMSLIWrapping.sapMSLINoIconInset > .sapMLIBContent > .sapMSLIImgNoInset,
.sapMSLIWrapping > .sapMLIBContent > .sapMSLIImg {
	margin-top: 0.725rem;
}

.sapMSLIWrapping > .sapMLIBContent > .sapMSLIDiv {
	padding: 1rem 0;

	&:has(.sapMSLIDescription) {
		padding-bottom: 0.5rem;

		.sapMSLITitle {
			padding-bottom: 0.25rem;
		}
	}

	.sapMSLIDescription {
		padding: 0.5rem 0;
	}
}

.sapMSLINoTitleAdapt.sapMSLIWrapping > .sapMLIBContent > .sapMSLIDiv > .sapMSLITitleOnly {
	padding: 0
}

.sapMSLIWrapping .sapMSLITitleOnly > .sapMSLIInfo,
.sapMSLIWrapping .sapMSLIDescriptionText > .sapMSLIInfo {
	float: right;
}

.sapMSLIExpandCollapse {
	text-wrap: nowrap;
	position: relative;
	color: var(--sapLinkColor);
	cursor: pointer;
	padding: 0 1px;
}

.sapMSLIExpandCollapse:hover {
	color: var(--sapLink_Hover_Color);
	text-decoration: underline;
}

.sapMSLIExpandCollapse:focus {
	outline: @_sap_m_ListItemBase_Focus_Outline;
}

.sapMSLIExpandCollapse::before {
	position: absolute;
	content: '';
	height: 1.5rem;
	top: -0.25rem;
	width: 100%;
}

.sapMSLIImgIcon {
	margin-inline-start: -0.75rem;
}

.sapMSLIAvatar {
	margin: 0.25rem;
	margin-right: 0.75rem;
}

/* Compact size */
.sapUiSizeCompact .sapMSLI {
	height: 2rem;
}

.sapUiSizeCompact .sapMSLI.sapMSLIThumbnail,
.sapUiSizeCompact .sapMSLI.sapMSLINoIconInset,
.sapUiSizeCompact .sapMSLI.sapMSLIWithDescription,
.sapUiSizeCompact .sapMSLI.sapMSLINoTitleAdapt {
	height: 5rem;
}

.sapUiSizeCompact .sapMSLI.sapMSLIWrapping {
	height: auto;
	min-height: 2rem;
}

.sapUiSizeCompact .sapMSLI.sapMSLIWithDescription.sapMSLIWrapping,
.sapUiSizeCompact .sapMSLI.sapMSLINoIconInset.sapMSLIWrapping,
.sapUiSizeCompact .sapMSLI.sapMSLINoTitleAdapt.sapMSLIWrapping {
	min-height: 5rem;
}

.sapUiSizeCompact .sapMSLIThumbnail:not(.sapMSLIWrapping) > .sapMLIBContent > .sapMSLIDiv {
	padding: 0.25rem 0;
}

.sapUiSizeCompact .sapMSLITitle,
.sapUiSizeCompact .sapMSLITitleOnly {
	font-size: @sapMFontMediumSize;
}

.sapUiSizeCompact .sapMSLIImgIcon {
	height: 2rem;
	line-height: 2rem;
	font-size: 1rem;
}

.sapUiSizeCompact .sapMSLI.sapMSLIWrapping > .sapMLIBSelectS,
.sapUiSizeCompact .sapMSLI.sapMSLIWrapping > .sapMLIBSelectM,
.sapUiSizeCompact .sapMSLI.sapMSLIWithDescription > .sapMLIBSelectS,
.sapUiSizeCompact .sapMSLI.sapMSLIWithDescription > .sapMLIBSelectM,
.sapUiSizeCompact .sapMSLI.sapMSLIWrapping > .sapMLIBContent > .sapMSLIImgIcon {
	margin-top: 0.725rem;
}
/* END "StandardListItem.less" */

/* START "StandardTreeItem.less" */
/* ======================================= */
/* CSS for control sap.m/StandardTreeItem  */
/* Base theme                              */
/* ======================================= */

.sapMSTIIcon {
	margin: 0.5rem 0.75rem 0.5rem 0rem;
	color: var(--sapContent_NonInteractiveIconColor);
	font-size: 1rem;
	height: 1rem;
	width: 1rem;
	vertical-align: middle;
	flex-shrink: 0;
}

.sapMSTI > .sapMLIBContent {
	word-break: break-word;
}
/* END "StandardTreeItem.less" */

/* START "StandardTile.less" */
/* =================================== */
/* CSS for control sap.m/StandardTile  */
/* Base theme                          */
/* =================================== */

.sapMStdTileTopRow {
	height: 4rem;
	padding: 1rem 1rem 0 1rem;
	margin-bottom: 1rem;
}

.sapMStdTileIconDiv {
	display: inline-block;
	float: left;
}

.sapMStdTileNumDiv {
	display: inline-block;
	float: right;
	width: 7.5rem;
	height: 4rem;
}

.sapMStdTileNum,
.sapMStdTileNumM,
.sapMStdTileNumS {
	white-space: nowrap;
	overflow: hidden;
	text-align: right;
	line-height: 1em;
	font-family: @sapUiFontCondensedFamily;
	font-stretch: condensed; /* for Firefox */
	color: var(--sapContent_LabelColor);
}

.sapMStdTileNum {
	font-size: 3rem;
	margin-top: -.437rem;
}

.sapMStdTileNumM {
	font-size: 2.25rem;
	margin-top: -.25rem;
}

.sapMStdTileNumS {
	font-size: 1.5rem;
	margin-top: -.125rem;
}

.sapMStdTileNumUnit {
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
	text-align: right;
	text-transform: uppercase;
	font-size: 0.875rem;
}

.sapMStdTileTitle {
	line-height: 1.4rem;
	vertical-align: top;
	text-align: left;
	font-size: 1rem;
	height: 4rem;
	width: 100%;
	overflow: hidden;
	word-break: break-word;
}

html[dir=rtl] .sapMStdTileTitle {
	text-align: right;
}

.sapMStdTileInfo {
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
	text-align: right;
	font-size: 0.875rem;
	margin-top: 2rem;
}

html[dir=rtl] .sapMStdTileNum,
html[dir=rtl] .sapMStdTileNumM,
html[dir=rtl] .sapMStdTileNumS,
html[dir=rtl] .sapMStdTileNumUnit,
html[dir=rtl] .sapMStdTileInfo {
	text-align: left;
}

.sapMStdTileBottomRow {
	height: 7rem;
	padding: 0 1rem 1rem 1rem;
}

.sapMStdTileMonitorType {
	margin-top: -2rem;
	padding-top: 2rem;
	background-color: var(--sapTile_Background);
}

.sapMStdTileBottomRow .sapMStdTileMonitorType {
	height: 10rem;
}

html.sap-phone .sapMStdTileTopRow {
	height: 3rem;
	margin-bottom: 0;
}

html.sap-phone .sapMStdTileNumDiv {
	width: 5rem;
	height: 3rem;
}

html.sap-phone .sapMStdTileNumUnit {
	font-size: 0.75rem;
}

html.sap-phone .sapMStdTileBottomRow {
	height: 5rem;
}

html.sap-phone .sapMStdTileMonitorType {
	margin-top: -1rem;
	padding-top: 1rem;
}

html.sap-phone .sapMStdTileTitle {
	line-height: 1.2rem;
	font-size: .875rem;
	height: 4rem;
	margin-bottom: 0;
}

html.sap-phone .sapMStdTileInfo {
	height: 1rem;
	font-size: 0.75rem;
	width: 100%;
	margin-top: 0;
}

html.sap-phone .sapMStdTileNum {
	font-size: 2.375rem;
}

html.sap-phone .sapMStdTileNumM {
	font-size: 1.5rem;
}

html.sap-phone .sapMStdTileNumS {
	font-size: 1rem;
}

.sapMStdTileIconDiv,
.sapMStdTileTitle {
	color: var(--sapTile_TitleTextColor);
}

.sapMStdIconMonitor {
	color: var(--sapContent_NonInteractiveIconColor);
}

.sapMStdIconCreate {
	color: var(--sapContent_NonInteractiveIconColor);
}

.sapMStdTileNumUnit {
	color: var(--sapContent_LabelColor);
	text-transform: uppercase;
}

.sapMStdTileInfo {
	color: var(--sapContent_LabelColor);
}

.sapMStdTileInfoError {
	color: var(--sapNegativeTextColor);
}

.sapMStdTileInfoWarning {
	color: var(--sapCriticalTextColor);
}

.sapMStdTileInfoSuccess {
	color: var(--sapPositiveTextColor);
}

.sapMTileActive-CTX .sapMStdIconCreate {
	opacity: 1;
}

.sapMTileActive-CTX .sapMStdTileInfoWarning,
.sapMTileActive-CTX .sapMStdTileInfoSuccess,
.sapMTileActive-CTX .sapMStdTileInfoError,
.sapMTileActive-CTX .sapMStdTileTitle,
.sapMTileActive-CTX .sapMStdTileNumUnit,
.sapMTileActive-CTX .sapMStdTileNum,
.sapMTileActive-CTX .sapMStdTileNumM,
.sapMTileActive-CTX .sapMStdTileNumS,
.sapMTileActive-CTX .sapMStdTileInfo,
.sapMTileActive-CTX .sapMStdTileMonitorType {
	color: var(--sapContent_ContrastTextColor);
}

.sapMTileActive-CTX .sapMStdTileIconDiv,
.sapMTileActive-CTX .sapMStdIconMonitor,
.sapMTileActive-CTX .sapMStdIconCreate {
	color: var(--sapContent_ContrastIconColor);
}
/* END "StandardTile.less" */

/* START "StepInput.less" */
/* ================================ */
/* CSS for control sap.m/StepInput  */
/* Base theme                       */
/* ================================ */

.sapMStepInput {
	display: inline-block;
	width: 100%;
	outline: none;
}

.sapMStepInput .sapUiIcon:focus {
	outline: none;
}

.sapMStepInput {

	.sapMStepInputIconDisabled {
		opacity: 0.5;
		pointer-events: none;
	}

	& > .sapMInputBase.sapMInputBaseHasBeginIcons > .sapMInputBaseContentWrapper {
		padding-left: 0;
	}

	&:focus {

		.sapMInputBaseContentWrapper:not(.sapMInputBaseReadonlyWrapper):not(.sapMStepInputReadOnly) {

			&::before {
				content: '';
				position: absolute;
				top: 1px;
				left: 1px;
				right: 1px;
				bottom: 1px;
				border: 1px dotted var(--sapContent_FocusColor);
				pointer-events: none;
			}
		}
	}
}
/* END "StepInput.less" */

/* START "SuggestionItem.less" */
/* ===================================== */
/* CSS for control sap.m/SuggestionItem  */
/* Base theme                            */
/* ===================================== */

.sapMSuL.sapMSelectList {
	.sapMSelectListItem {
		display: block;
	}
}

.sapMSuggestionItemIcon {
	margin-right: 0.5rem;
	margin-left: -0.5rem;
}
/* END "SuggestionItem.less" */

/* START "Support.less" */
/* ============================== */
/* CSS for control sap.m/Support  */
/* Base theme                     */
/* ============================== */

html.sap-phone .sapMSupport .sapMDialogScrollCont {
	padding: 0;
}

.sapMSupport .sapUiSupportLabel{
	font-weight: bold;
}

.sapMSupport .sapUiSupportTechInfoBorder {
	border: 1px solid var(--sapList_BorderColor);
}

.sapMSupport .sapUiSupportTxtFld,
.sapMSupport .sapUiSupportBtn {
	border: 1px solid var(--sapField_BorderColor);
	font-size: 14px;
	height: 22px;
	box-sizing: border-box;
	margin-left:10px;
}

.sapMSupport .sapUiSupportTxtArea {
	width: 100%;
	height: 50px;
	margin-top: 5px;
	resize: none;
	box-sizing: border-box;
}

.sapMSupport .sapUiSupportPnlLbl {
	display: block;
}

.sapMSupport .sapUiSupportDiv {
	margin-top: 5px;
}

.sapMSupport .sapUiSupportTxtFld {
	background-color: var(--sapField_Background);
	padding: 1px;
}

.sapMSupport .sapUiSupportTxtFld:focus {
	outline: 1px dashed var(--sapContent_FocusColor);
	outline-offset: 2px;
}

.sapMSupport .sapUiSupportBtn {
	background-color: var(--sapButton_Background);
	border-color: var(--sapButton_BorderColor);
	padding-bottom: 3px;
	cursor: pointer;
}

.sapMSupport .sapUiSupportBtn:focus,
.sapMSupport .sapUiSupportBtn.sapUiSupportRunningTrace {
	outline: 1px dashed var(--sapContent_FocusColor);
	outline-offset: 2px;
}

.sapMSupport .sapUiSupportBtn.sapUiSupportRunningTrace {
	cursor: default;
}
/* END "Support.less" */

/* START "Switch.less" */
/* ============================= */
/* CSS for control sap.m/Switch  */
/* Base theme                    */
/* ============================= */

/* Switch required variables */
@_sap_m_Switch_OnPosition: -32;
@_sap_m_Switch_OffPosition: 0;
@_sap_m_Switch_TransitionTime: 175;

/* Switch common styles */
.sapMSwtText,
.sapMSwtHandle,
.sapMSwtInner {
	position: absolute;
}

.sapMSwtCont,
.sapMSwt {
	position: relative;
}

.sapMSwtText,
.sapMSwtAcceptReject .sapMSwtHandle {
	box-sizing: border-box;
}

/* Switch container */
.sapMSwtCont {
	display: inline-block;
	vertical-align: top;
	outline: none;
	-webkit-user-select: none;
	-moz-user-select: none;
	user-select: none;
	padding: 0.5rem 0 0.5rem 0;
}

.sapMSwtCont:not(.sapMSwtContDisabled) {
	cursor: pointer;
}

.sapMSwtCont > input {
	display: none;
}

/* Switch */
.sapMSwt {
	outline: none;
	width: 4rem;
	height: 2rem;
	/* use px instead of rem, borders sized in "rem" disappear when the page is zoomed out in Chrome. */
	border: 1px solid;
	/* 1px margin from left and right, so the outline doesn't get cropped when focuses */
	margin: 0 1px;
	/* note: some browsers like Samsung Galaxy S4 with Android 4.2 does not support the "border-radius" shorthand property
	but does support the long-hand properties for each corner like "border-top-left-radius" */
	border-top-left-radius: 1rem;
	border-top-right-radius: 1rem;
	border-bottom-right-radius: 1rem;
	border-bottom-left-radius: 1rem;
	overflow: hidden;
}

/* background */
.sapMSwtOn:not(.sapMSwtAcceptReject) {
	background: @sapUiToggleButtonPressedBackground;
	border-color: @sapUiToggleButtonPressedBorderColor;
}

.sapMSwtOff:not(.sapMSwtAcceptReject) {
	background: var(--sapButton_Background);
	border-color: var(--sapButton_BorderColor);
}

/* hover */
.sapMSwt.sapMSwtHoverable:hover {
	background: var(--sapButton_Hover_Background);
}

.sapMSwt.sapMSwtHoverable:not(.sapMSwtAcceptReject):hover {
	border-color: var(--sapButton_Hover_BorderColor);
}

.sapMSwtHoverable.sapMSwtOn:not(.sapMSwtAcceptReject):hover {
	background: @sapUiToggleButtonPressedHoverBackground;
	border-color: @sapUiToggleButtonPressedHoverBorderColor;
}

/* focus */
.sapMSwtDisabled:focus {
	outline: none;
}

/* Switch inner */
.sapMSwtInner {
	width: 100%;
	height: 100%;
}

.sapMSwtOff > .sapMSwtInner {
	left: -2rem;
}

.sapMSwtOn > .sapMSwtInner {
	left: 0;
}

/* CSS transitions */

.sapMSwtTrans > .sapMSwtInner {
	transition: left 0.125s ease-in-out;
}

html[dir=rtl] .sapMSwtTrans > .sapMSwtInner {
	transition-property: right;
}

/* Switch text */
.sapMSwtText {
	text-align: center;
	font-size: var(--sapFontSmallSize);
	width: 100%;
	line-height: 2rem;
}

.sapMSwtLabel {
	display: block;
	white-space: nowrap;
	text-decoration: none;
	overflow: hidden;
	width: 1.875rem;
	padding: 0 0.1875rem;
	text-transform: uppercase;
	line-height: inherit;
}

.sapMSwtOn .sapMSwtText {
	color: @sapUiToggleButtonPressedTextColor;
}

.sapMSwtOff .sapMSwtText {
	color: var(--sapContent_LabelColor);
}

.sapMSwtTextOff {
	left: 2rem;
}

.sapMSwtLabelOff {
	float: right;
}

.sapMSwt.sapMSwtDefault {

	&.sapMSwtOff .sapMSwtTextOn,
	&.sapMSwtOn .sapMSwtTextOff {

		.sapMSwtLabel {
			/* Special case - the currently invisible label should be smaller */
			/* so the text won't get in the visible part of the control */
			width: 1.6rem;
		}
	}
}

/* Switch handler */
.sapMSwtHandle {
	display: block;
	top: 0;
	bottom: 0;
	margin: auto;
	width: 1.5rem;
	height: 1.5rem;
	left: 2.25rem;

	/* note: some browsers like Samsung Galaxy S4 with Android 4.2 does not support the "border-radius" shorthand property */
	/* but does support the long-hand properties for each corner like "border-top-left-radius" */
	border-top-left-radius: 0.75rem;
	border-top-right-radius: 0.75rem;
	border-bottom-right-radius: 0.75rem;
	border-bottom-left-radius: 0.75rem;
	-webkit-background-clip: padding-box;
	background-clip: padding-box;
}

/* background */
.sapMSwtOn .sapMSwtHandle {
	background: transparent;
	box-sizing: border-box;
	border: 2px solid contrast(@sapUiToggleButtonPressedBackground, @sapUiContentIconColor, @sapUiContentContrastIconColor, @sapUiContentContrastTextThreshold);
}

.sapMSwtOff .sapMSwtHandle {
	background: transparent;
	box-sizing: border-box;
	border: 2px solid var(--sapButton_BorderColor);
}


/* Switch type "AcceptReject" */
/* Switch type "AcceptReject" common styles */

/* background */
.sapMSwtAcceptReject.sapMSwtOn {
	border-color: var(--sapPositiveElementColor);
}

.sapMSwtAcceptReject.sapMSwtOff {
	border-color: var(--sapNegativeElementColor);
}

/* Switch */

/* background */
.sapMSwtAcceptReject.sapMSwt {
	background: var(--sapButton_Background);
}

/* Switch text */
.sapMSwtAcceptReject .sapMSwtLabelOn::before,
.sapMSwtAcceptReject .sapMSwtLabelOff::before {
	font-family: var(--sapContent_IconFontFamily);
	speak: none;
}

/* icon fonts */
.sapMSwtAcceptReject .sapMSwtLabel {
	font-size: var(--sapFontLargeSize);
}

.sapMSwtAcceptReject .sapMSwtLabelOn::before {
	content: '\e05b';
	color: var(--sapPositiveElementColor);
}

.sapMSwtAcceptReject .sapMSwtLabelOff::before {
	content: '\e03e';
	color: var(--sapNegativeElementColor);
}

/* Switch handler */
.sapMSwtAcceptReject .sapMSwtHandle {
	background: transparent;
	border-style: solid;
	border-width: 0.125rem;
}

.sapMSwtAcceptReject.sapMSwtOn .sapMSwtHandle {
	border-color: var(--sapPositiveElementColor);
}

.sapMSwtAcceptReject.sapMSwtOff .sapMSwtHandle {
	border-color: var(--sapNegativeElementColor);
}


/* compact size */
/* Switch container */

.sapUiSizeCompact .sapMSwtCont {
	padding-top: 0.125rem;
	padding-bottom: 0.125rem;
}

/* Switch */
.sapUiSizeCompact .sapMSwt {
	width: 3.5rem;
	height: 1.625rem;
}

/* Switch text */
.sapUiSizeCompact .sapMSwtText {
	line-height: 1.625rem;
}

/* Switch handler */
.sapUiSizeCompact .sapMSwtHandle {
	height: 1rem;
	width: 1rem;
}
/* END "Switch.less" */

/* START "TabContainer.less" */
/* =================================== */
/* CSS for control sap.m/TabContainer  */
/* Base theme                          */
/* =================================== */

.sapMTabContainer {
	display: flex;
	flex-direction: column;
	height: 100%;

	.sapMTabContainerContent {
		position: relative;
		flex-grow: 1;

		.sapMTabContainerInnerContent {
			position: absolute;
			top: 0;
			bottom: 0;
			left: 0;
			right: 0;
			overflow-x: auto;
		}
	}
}

/* mixin call for creating container content padding classes (arguments: rootContainer, contentSelector) */
.sapUiContainerContentPadding(~".sapMTabContainer", ~"> .sapMTabContainerContent > .sapMTabContainerInnerContent");

.sapMTabContainer {

	.sapMTabContainerContentTransparent {
		background-color: transparent;
	}

	.sapMTabContainerContentList {
		background-color: var(--sapGroup_ContentBackground);
	}

	.sapMTabContainerContentSolid {
		background-color: var(--sapShell_Background);
	}

	.sapMTabContainerContentStandard {
		background-color: var(--sapBackgroundColor);
	}
}
/* END "TabContainer.less" */

/* START "Table.less" */
/* ============================ */
/* CSS for control sap.m/Table  */
/* Base theme                   */
/* ============================ */

.sapMListTbl {
	width: 100%;
	border-collapse: separate;
	border-spacing: 0;
	table-layout: fixed;
}

.sapMTableDisableClearAll {
	opacity: 0.3;
	pointer-events: none;
}

.sapMListTbl .sapMLIB {
	display: table-row;
}

.sapMListTblCellNoData {
	text-align: center;
	white-space: pre-wrap;
}

.sapMTablePopinColumn {
	border: 0;
	height: 0;
	padding: 0;
	overflow: hidden;
}

.sapMListTblHeaderNone {
	height: 0;
}

.sapMListTblCellDup > .sapMListTblCellDupCnt {
	/* hide content visually but have it available for screen readers */
	position: absolute;
	clip: rect(0, 0, 0, 0);
	height: 0;
}

tr.sapMListTblHeaderNone > .sapMTableTH {
	border: 0;
	margin-top: 0;
	padding-top: 0;
	margin-bottom: 0;
	padding-bottom: 0;
}

.sapMListTblHeaderNone > .sapMListTblSelCol > .sapMCb,
.sapMListTblHeaderNone > .sapMListTblCell > * {
	display: none;
}

/**
 * For compatibility reasons text alignment should inherit from the table headers & cells.
 * Default value of textAlign property is Begin(not Initial) for the following controls
 * therefore they always set text-align as a style attribute in DOM, this blocks
 * alignment inheritance from the table headers & cells.
 */
.sapMColumnHeader > .sapMLabel,
.sapMColumnHeader > .sapMText,
.sapMListTblCell > .sapMSlt > .sapMSltLabel,
.sapMListTblCell > .sapMObjectNumber,
.sapMListTblCell > .sapMLabel,
.sapMListTblCell > .sapMText {
	text-align: inherit !important;
}

.sapMListTblCell {
	padding: .25rem .5rem;
	font-size: @sapUiListTableTextSize;
	overflow: hidden;
	overflow: clip;
}

.sapMListTblSelCol,
.sapMListTblCell,
.sapMListTblNavCol {
	vertical-align: middle;
}

.sapMListTblNavCol {
	width: 0;
	line-height: 3rem;
}

.sapMListTblUnreadCol {
	width: 1.74rem;
}

.sapMListTblSubCnt {
	display: block;
}

.sapMListTblSubCntRow {
	overflow: hidden;
}

.sapMListTblSubCntSpr,
.sapMListTblSubCntVal {
	display: inline-flex;
}

.sapMListTblSubCntValBlock {
	display: block;
}

.sapMListTblHighlightCol,
.sapMListTblNavigatedCol {
	width: 0;
	padding: 0;
}

.sapMListTblHighlightCol + .sapMListTblHeaderCell,
.sapMListTblHighlightCell + .sapMListTblCell {
	padding-left: 1rem;
}

/* 0.0625rem -> border-right width for highlight */
.sapMListHighlight .sapMListTblHighlightCol {
	width: 0.375rem + 0.0625rem;
	min-width: 0.375rem + 0.0625rem;
}

.sapMListNavigated .sapMListTblNavigatedCol {
	width: 0.1875rem;
}

.sapMListTblHighlightCell,
.sapMListTblNavigatedCell {
	padding: 0;
	position: relative;
	overflow: hidden;
}

/* mixin call for creating container content padding classes (arguments: rootContainer, contentSelector) */
.sapUiContainerContentPadding(~".sapMListTbl");

.sapMListTblNavCol,
.sapMListTblSelCol {
	padding: 0;
}

.sapMListTblUnreadCol {
	display: none;
}

.sapMListTbl {
	font-size: @sapUiListTableTextSize;
}

.sapMListTbl .sapUiIcon:not(.sapMLIBImgNav):not(.sapMBtnIcon):not(.sapFAvatarTypeIcon) {
	font-size: @sapUiListTableIconSize;
}

/* Ensuring that the row height is not smaller that 3rem in Cozy mode */
.sapMListTblRow {
	height: var(--sapElement_LineHeight);
	position: static; /* override the position: relative rule of the list item */
}

.sapMListTblCell.sapMListTblCellNoData {
	font-size: 1rem;
	padding-top: .75rem;
	padding-right: 1rem;
	padding-bottom: .75rem;
	border-bottom: 1px solid var(--sapList_BorderColor);

	&.sapMListTblCellNoIllustratedMessage {
		line-height: 1.6;
	}
}

.sapMListTblSelCol,
.sapMListTblHasNav .sapMListTblNavCol {
	width: 3rem;
}

.sapMLIB.sapMListTblRow {
	border-bottom: 0;
}

.sapMListTblCell.sapMListTblCellDup {
	border-color: transparent;
}

/* table column header */
.sapMTableTH {
	background: var(--sapList_HeaderBackground);
	border-bottom: 1px solid var(--sapList_BorderColor);
	vertical-align: middle;
}

.sapMTableClearAll {
	cursor: pointer;
}

/* backgound-clip is required in other browsers, so that focus outline is visible on all sides of the header row */
.sapMSticky .sapMListTblHeader:focus > .sapMTableTH {
	background-clip: padding-box;
}

/* sticky headerToolbar, infoToolbar and column headers */
.sapMSticky .sapMListTblHeader {
	position: sticky;
}

/* sapMSticky15 is the result of sticky groupHeaders, headerToolbar, infoToolbar and column headers */
/* sapMSticky14 is the result of sticky groupHeaders, infoToolbar and column headers */
/* sapMSticky13 is the result of sticky groupHeaders, headerToolbar and column headers */
/* sapMSticky12 is the result of sticky groupHeaders and column headers */
/* sapMSticky11 is the result of sticky groupHeaders and headerToolbar and infoToolbar */
/* sapMSticky10 is the result of sticky groupHeaders and InfoToolbar */
/* sapMSticky9 is the result of sticky groupHeaders and headerToolbar */
/* .sapMSticky7 denotes that the headerToolbar, infoToolbar and column headers are sticky */
/* .sapMSticky6 denotes that the infoToolbar and column headers are sticky */
/* .sapMSticky5 denotes that the headerToolbar and column headers are sticky */
/* .sapMSticky4 denotes that only the column headers are sticky */
/* top: 0 sticky conditions : 1st sticky elements */
.sapMSticky4 .sapMListTblHeader,
.sapMSticky5 > .sapMListHdr,
.sapMSticky6 > .sapMListInfoTBarContainer,
.sapMSticky7 > .sapMListHdr,
.sapMSticky8 .sapMGHLI,
.sapMSticky9 > .sapMListHdr,
.sapMSticky10 > .sapMListInfoTBarContainer,
.sapMSticky11 > .sapMListHdr,
.sapMSticky12 .sapMListTblHeader,
.sapMSticky13 > .sapMListHdr,
.sapMSticky14 > .sapMListInfoTBarContainer,
.sapMSticky15 > .sapMListHdr {
	z-index: 3; /* allow controls to have own z-index: 1 */
	top: 0;
}

/* top: 2rem sticky conditions : 2nd sticky elements */
.sapMSticky6 .sapMListTblHeader,
.sapMSticky14 .sapMListTblHeader,
.sapMSticky10 .sapMGHLI {
	z-index: 3;
	top: 2rem;
}

/* top: 3rem sticky conditions : 2nd sticky elements */
.sapMSticky5 .sapMListTblHeader,
.sapMSticky7 > .sapMListInfoTBarContainer,
.sapMSticky9 .sapMGHLI,
.sapMSticky11 > .sapMListInfoTBarContainer,
.sapMSticky12 .sapMGHLI,
.sapMSticky13 .sapMListTblHeader,
.sapMSticky15 > .sapMListInfoTBarContainer {
	z-index: 3;
	top: var(--sapElement_LineHeight);
}

/* top: 5rem conditions : 3rd sticky elements */
.sapMSticky7 .sapMListTblHeader,
.sapMSticky11 .sapMGHLI,
.sapMSticky14 .sapMGHLI,
.sapMSticky15 .sapMListTblHeader {
	z-index: 3;
	top: ~"calc(var(--sapElement_LineHeight) + 2rem)";
}

.sapMSticky13 .sapMGHLI {
	z-index: 3;
	top: ~"calc(var(--sapElement_LineHeight) * 2)";
}

/* busyIndicator should not block the toolbars if <table> is set busy */
.sapMSticky > .sapMListTbl > .sapUiLocalBusyIndicator {
	z-index: 3;
}

.sapMSticky15 .sapMGHLI {
	z-index: 3;
	top:  ~"calc(var(--sapElement_LineHeight) * 2 + 2rem)";
}

.sapUiSizeCompact {

	& .sapMSticky12 .sapMGHLI {
		top: 2rem;

		/* only first child needs 1px more which prevents (tiny) double borders through stacking the following groupHeaders */
		&:first-child {
			top: 2rem + 0.0625rem;
		}
	}

	& .sapMSticky13 .sapMGHLI {
		top:  ~"calc(var(--sapElement_LineHeight) + 1.95rem)";

		/* only first child needs 2px more which prevents (tiny) double borders through stacking the following groupHeaders */
		&:first-child {
			top: ~"calc(var(--sapElement_LineHeight) + 1.95rem + 0.125rem )";
		}
	}

	& .sapMSticky14 .sapMGHLI {
		top:  ~"calc(var(--sapElement_LineHeight) + 1.25rem)";

		/* only first child needs 1px more which prevents (tiny) double borders through stacking the following groupHeaders */
		&:first-child {
			top: ~"calc(var(--sapElement_LineHeight) + 1.25rem + 0.0625rem)" ;
		}
	}

	& .sapMSticky15 .sapMGHLI {
		top: ~"calc(var(--sapElement_LineHeight) * 2 + 1.2rem)";

		/* only first child needs 2px more which prevents (tiny) double borders through stacking the following groupHeaders */
		&:first-child {
			top:  ~"calc(var(--sapElement_LineHeight) * 2 + 1.2rem + 0.125rem)";
		}
	}
}

/* define outline for the block layer */
.sapMListTblCnt > .sapMTableOverlay:focus {
	outline: @_sap_m_ListItemBase_Focus_Outline;
	outline-offset: @_sap_m_ListItemBase_Focus_OffsetWithGap;
}

/* table footer */
.sapMListTblFooter > td {
	border-top: 1px solid var(--sapList_TableFooterBorder);
}

/* footer text should be bold */
.sapMListTblFooter .sapMText,
.sapMListTblFooter .sapMLabel {
	font-weight: bold;
}

.sapMListTblDummyCell {
	padding: 0;
}

.sapMTableLastColumn ~ .sapMListTblDummyCell {
	width: 0;
}

.sapMListTblDummyCell:last-child {
	background-color: var(--sapBackgroundColor);
	border-left: 1px solid var(--sapList_TableFooterBorder);
	border-bottom: none;
}

:not(.sapMListHasGrowing):not(.sapMListShowSeparatorsInner):not(.sapMListShowSeparatorsNone) > .sapMTableTBody .sapMListTblRow:last-child > .sapMListTblDummyCell:last-child {
	border-bottom: 1px solid var(--sapList_BorderColor);
}

.sapMListTblHeader > .sapMListTblDummyCell:last-child {
	background-color: var(--sapList_HeaderBackground);
}

.sapMListTblFooter > .sapMListTblDummyCell:last-child,
.sapMTableTBody > :not(:first-child) .sapMListTblDummyCell:last-child {
	border-top-color: transparent;
}

/* grid layout for popin row */
.sapMListTblSubCntGridSmall {
	display: grid;
	grid-template-columns: repeat(auto-fill, minmax(13rem, 1fr));
	grid-column-gap: 1rem;
}

.sapMListTblSubCntGridLarge {
	display: grid;
	grid-template-columns: repeat(auto-fill, minmax(26rem, 1fr));
	grid-column-gap: 1rem;
}

.sapMListTblSubCntGridSmall > .sapMListTblSubCntRow,
.sapMListTblSubCntGridLarge > .sapMListTblSubCntRow {
	margin-top: .5rem;
}

.sapMTableTH > .sapMLnk,
.sapMTableTH > .sapMText,
.sapMTableTH > .sapMLabel,
.sapMTableTH > .sapMObjectNumber {
	vertical-align: middle;
}

.sapMListTblCell > .sapMLnk {
	vertical-align: bottom;
}

/* Column header and column header content */
.sapMListTblCell.sapMListTblHeaderCell {
	padding: 0;
}

.sapMColumnHeader {
	display: flex;
	flex: 1 1 auto;
	padding: .875rem .5rem 1rem;
}

.sapMColumnHeaderText {
	width: 100%;
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
	font-weight: normal;
	font-size: @sapMFontMediumSize;
}

.sapMColumnHeaderIcons {
	white-space: nowrap;
}

.sapMColumnHeaderIcons > .sapUiIcon {
	padding-left: 0.375rem;
}

.sapMListTblHighlightCol + .sapMListTblCell.sapMListTblHeaderCell {
	padding-left: .5rem;
}

.sapMColumnHeaderActive:hover {
	background-color: var(--sapList_Hover_Background);
}

.sapMColumnHeaderActive:focus {
	outline: @_sap_m_ListItemBase_Focus_Outline;
	outline-offset: @_sap_m_ListItemBase_Focus_OffsetWithGap;
}

.sapMColumnHeaderActive:active {
	color: var(--sapList_Active_TextColor);
	background-color: var(--sapList_Active_Background);
	outline-color: var(--sapContent_ContrastFocusColor);
}

/* Allows truncation of text within children of a flex item. */
.sapMColumnHeaderContent {
	min-width: 0;
}

.sapMColumnHeaderActive:active > .sapMColumnHeaderContent {
	color: var(--sapList_Active_TextColor);
}

.sapMColumnHeaderActive, .sapMColumnHeaderActive > .sapMColumnHeaderContent {
	cursor: pointer;
}

.sapMTableTH:focus {
	outline: @_sap_m_ListItemBase_Focus_Outline;
	outline-offset: @_sap_m_ListItemBase_Focus_OffsetWithGap;
}

/* sort icons */
[aria-sort=ascending] > .sapMColumnHeader::after,
[aria-sort=descending] > .sapMColumnHeader::after {
	font-family: var(--sapContent_IconFontFamily);
	font-size: @sapMFontMediumSize;
	font-weight: normal;
	align-self: center;
	padding-left: 0.5rem;
	box-sizing: border-box;
}

[aria-sort=ascending] > .sapMColumnHeader::after {
	content: '\e1fd';
}

[aria-sort=descending] > .sapMColumnHeader::after {
	content: '\e1fc';
}

/* Focus border workaround for dummy column */
.sapMListTblDummyCell:last-child {
	pointer-events: none;
}

.sapMTableRowCustomFocus:focus {
	outline: none;

	.sapMListTblCell:not(.sapMListTblCellDup) > :first-child {
		position: relative;
	}

	& > :not(.sapMListTblDummyCell:last-child),
	& + .sapMListTblSubRow > .sapMListTblSubRowCell {
		position: relative;
	}

	& + .sapMListTblSubRow > td::before,
	& > :not(.sapMListTblDummyCell:last-child)::before {
		content: '';
		position: absolute;
		left: 0;
		right: 0;
		z-index: 1;
		pointer-events: none;
		top: 1px; /* mimics outline-offset: -1px */
		bottom: 1px; /* mimics outline-offset: -1px */
		border-top: @_sap_m_ListItemBase_Focus_Outline;
		border-bottom: @_sap_m_ListItemBase_Focus_Outline;
		margin-right: 1px; /* make the focus border clearly visible on all sides */
	}

	.sapMListTblHasPopin > &:not(.sapMGHLI) > :not(.sapMListTblCellNoData, .sapMListTblDummyCell:last-child, .sapMListTblNavigatedCell)::before {
		border-bottom: 0;
		bottom: 0;
	}

	& + .sapMListTblSubRow > td::before {
		border-top: 0;
		top: 0;
	}

	& > .sapMTblFirstVisibleCell::before,
	& + .sapMListTblSubRow > .sapMTblFirstVisibleCell::before {
		border-left: @_sap_m_ListItemBase_Focus_Outline;
		margin-left: 1px; /* mimics outline-offset */
	}

	& > .sapMTblLastVisibleCell::before,
	& + .sapMListTblSubRow > .sapMTblLastVisibleCell::before {
		border-right: @_sap_m_ListItemBase_Focus_Outline;
		margin-right: 1px; /* mimics outline-offset */
	}
}

html[data-sap-ui-browser^='ff'] .sapMTableRowCustomFocus:focus > :not(.sapMListTblDummyCell)::before {
	margin-right: 0; /* firefox seems fine with rendering borders clearly without additional margin */
}

.sapMTablePopinHeaders {
	display: none;
}

/********** COMPACT DESIGN ************/

.sapUiSizeCompact .sapMListTblCell.sapMListTblCellNoData {
	font-size: @sapUiListTableTextSize;
}

/* special class name to add margin to multiline contents */
/* deprecated - kept for compatibility reasons, but apps should use sapUiTinyMarginTopBottom instead */
.sapUiSizeCompact .sapMTableContentMargin {
	margin: .5rem 0;
}

/* navigation and selection columns are in the middle */
.sapUiSizeCompact .sapMListTblNavCol,
.sapUiSizeCompact .sapMListTblSelCol {
	line-height: 2rem;
	text-align: center;
}

/* decrease the navigation icon size */
.sapUiSizeCompact .sapMListTblNavCol > .sapMLIBImgNav {
	width: auto;
}

.sapUiSizeCompact .sapMListTbl:not(.sapMListModeDelete) {

	.sapMListTblSelCol,
	.sapMListTblSelCol > .sapMLIBSelectM,
	.sapMListTblSelCol > .sapMLIBSelectS {
		width: 2rem;
	}
}

/* min height for the table rows */
.sapUiSizeCompact .sapMListTblRow {
	height: 2rem;
}

/* reset height of cells when no header is defined */
.sapMListTblHeaderNone > .sapMListTblCell {
	height: 0;
	padding-top: 0;
	padding-bottom: 0;
}

/* Column header and column header content */
.sapUiSizeCompact .sapMColumnHeader {
	padding-top: .5rem;
	padding-bottom: .5rem;
}

.sapUiSizeCompact .sapMListTblCell.sapMListTblHeaderCell {
	padding: 0;
}

.sapUiSizeCompact .sapMListTblHighlightCol + .sapMListTblCell.sapMListTblHeaderCell {
	padding-left: .5rem;
}
/* END "Table.less" */

/* START "TablePersoDialog.less" */
/* ======================================= */
/* CSS for control sap.m/TablePersoDialog  */
/* Base theme                              */
/* ======================================= */

.sapMPersoDialogFixedBar {
	position: relative;

	.sapMPersoDialogResetBtn {
		position: absolute;
		top: 0;
		right: 0;
	}
}

.sapMPersoDialog {
	color: var(--sapTextColor);
}

.sapMPersoDialog .sapMLIB.sapMLIBSelected,
.sapMPersoDialog .sapMLIB.sapMLIBSelected > td {
	background-color: transparent;
}

.sapMPersoDialog .sapMTableTBody:not(.sapMListTblHasPopin) > .sapMLIBSelected,
.sapMPersoDialog .sapMTableTBody.sapMListTblHasPopin > .sapMLIBSelected + .sapMListTblSubRow,
.sapMPersoDialog .sapMTableTBody:not(.sapMListTblHasPopin) > .sapMLIBSelected:last-child {
	/* Commented as of 1.119 --> causes issues in hcb theme, rather stick to the Table's default styling */
	/* box-shadow: inset 0px -0.0625rem @sapUiListHoverBackground; */
}

.sapMPersoDialog .sapMListBGTranslucent .sapMLIB.sapMListTblRow.sapMPersoDialogItemSelected,
.sapMPersoDialog .sapMListBGTranslucent .sapMLIB.sapMListTblRow.sapMPersoDialogItemSelected:focus,
.sapMPersoDialog .sapMListBGTranslucent .sapMLIB:not(.sapMLIBActive):not(.sapMLIBSelected):not(:hover).sapMListTblRow.sapMPersoDialogItemSelected,
.sapMPersoDialog .sapMListBGTranslucent .sapMLIB:not(.sapMLIBActive):not(.sapMLIBSelected):not(:hover).sapMListTblRow.sapMPersoDialogItemSelected:focus {
	background-color: @sapUiListSelectionHoverBackground;
	/* Commented as of 1.119 --> causes issues in hcb theme, rather stick to the Table's default styling */
	/* box-shadow: inset 0px -0.0625rem @sapUiListSelectionBorderColor; */
}

.sapMPersoDialog .sapMLIBActive .sapMText,
.sapMPersoDialog .sapMLIBActive .sapMListTblCell .sapMText {
	color: var(--sapList_TextColor);
}

.sapMPersoDialog .sapMLIBActive .sapMCbBg,
.sapMPersoDialog .sapMLIBActive.sapMLIB,
.sapMPersoDialog .sapMLIBActive.sapMLIB > td {
	background-color: transparent;
}

.sapMPersoDialog .sapMLIBActive .sapMCbBg.sapMCbMarkChecked::before {
	color: var(--sapSelectedColor);
}

html.sap-desktop .sapMPersoDialog .sapMLIBSelected:not(.sapMLIBActive):not(.sapMALI):hover,
html.sap-desktop .sapMPersoDialog .sapMLIBSelected:not(.sapMLIBActive):not(.sapMALI):hover > td {
	background-color: @sapUiListSelectionHoverBackground;
}

.sapMPersoDialogFixedBar {

	.sapMPersoDialogSelectAllCb > .sapMCbLabel {
		font-size: 1.125rem;
	}
}

.sapUiSizeCompact .sapMPersoDialogFixedBar {

	.sapMPersoDialogSelectAllCb > .sapMCbLabel {
		font-size: 0.975rem;
	}
}

.sapUiSizeCompact .sapMTB-Transparent-CTX.sapMTB.sapMPersoDialogFixedBar > :first-child {
	margin-left: 1rem;
}

.sapMPersoDialogLI .sapMILILabel {
	max-width: none;
}

.sapMPersoDialogLI .sapMILIDiv {
	min-width: 5.875rem;
}

.sapMTB-Transparent-CTX.sapMTB.sapMPersoDialogFixedBar {
	padding-left: 0;
	border-bottom: 2px solid var(--sapList_HeaderBorderColor);
}

.sapMTB-Transparent-CTX.sapMTB.sapMPersoDialogFixedBar > :first-child {
	margin-left: 1rem;
}

.sapMPersoDialogLIHeader {
	border-bottom: 0;
}

html.sap-tablet:not(.sap-combi) .sapMPersoDialog {
	min-width: 20rem;
}
/* END "TablePersoDialog.less" */

/* START "TableSelectDialog.less" */
/* ======================================== */
/* CSS for control sap.m/TableSelectDialog  */
/* Base theme                               */
/* ======================================== */

.sapMTableSelectDialog.sapMDialog .sapMDialogScrollCont {
	padding: 0;
}

.sapMBusyIndicator.sapMTableSelectDialogBusyIndicator {
	position: absolute;
	top: 50%;
	left: 50%;
	margin-top: -1rem;
	margin-left: -1rem;
	display: none;
}

.sapMTableSelectDialogTableHide {
	display: none;
}
/* END "TableSelectDialog.less" */

/* START "TabStrip.less" */
/* =============================== */
/* CSS for control sap.m/TabStrip  */
/* Base theme                      */
/* =============================== */

/* Common style */
.sapMTabStrip {
	position: relative;
	height: 3rem;
	background: var(--sapPageHeader_Background);
	box-shadow: @sapUiShadowHeader;
	box-sizing: border-box;

	.sapMTSLeftOverflowButtons {
		position: absolute;
		margin-left: 0.25rem;
		visibility: hidden;
		width: 3rem;
		z-index: 5;
		box-sizing: border-box;
		height: 100%;
		border-bottom: 2px solid transparent;
	}

	.sapMTSLeftOverflowButtons::after {
		content: '';
		position: absolute;
		height: 2px;
		left: 0;
		right: 0;
		bottom: -2px;
	}

	.sapMTSTabsContainer {
		height: 100%;
		white-space: nowrap;
		position: absolute;
		left: 2.5rem;
		right: 7.5rem;
		overflow: hidden;
		outline: none;

		.sapMTSTabs {
			float: left;
			height: 100%;
			margin-right: -999em;

			.sapMTabStripItem {
				position: relative;
				box-sizing: border-box;
				height: 100%;
				padding: 0 0 0 0.5rem;
				float: left;
				cursor: pointer;
				-webkit-user-select: none;
				-moz-user-select: none;
				user-select: none;
				background: transparent;
				color: var(--sapGroup_TitleTextColor);
				border-bottom: solid 0.125rem transparent;

				&.sapMTabStripItemSelected {
					color: var(--sapGroup_TitleTextColor);
					border-bottom: solid 0.125rem var(--sapSelectedColor);
				}

				&:focus {
					outline: none;
				}

				&:focus::before {
					pointer-events: none;
					content: '';
					position: absolute;
					top: 1px;
					left: 1px;
					right: 1px;
					bottom: 1px;
					border: 1px dotted var(--sapContent_FocusColor);
				}

				.sapMTabStripItemLabel {
					vertical-align: middle;
					font-size: @sapMFontMediumSize;
				}

				.sapMTSItemCloseBtnCnt {
					display: inline-block;
					width: 2rem;
					text-align: center;
					vertical-align: middle;

					.sapMBtn {
						opacity: 1;
					}
				}

				&.sapMTabStripItemModified {

					.sapMTabStripItemModifiedSymbol::after {
						content: '*';
						vertical-align: middle;
					}
				}
			}
		}
	}

	.sapMTSRightOverflowButtons {
		position: absolute;
		right: 5.5rem;
		visibility: hidden;
		width: 3rem;
		z-index: 5;
		box-sizing: border-box;
		height: 100%;
		border-bottom: 2px solid transparent;

		& .sapMBtn.sapMBtnBase {
			padding-left: 1rem;
		}
	}

	.sapMTSRightOverflowButtons::after {
		content: '';
		position: absolute;
		height: 2px;
		left: 0;
		right: 0;
		bottom: -2px;
	}

	.sapMTSTouchArea {
		height: 100%;
		position: absolute;
		right: 0.25rem;

		.sapMTSAddNewTabBtn {
			margin-left: 0.5rem;
		}
	}
}

.sapMTSOverflowSelectListItem {
	vertical-align: middle;
}

.sapMTabStripItemAddText,
.sapMTabStripItemLabel {
	line-height: normal;
}

.sapMTSOverflowSelectListItem .sapMTabStripItemAddText,
.sapMTabStripItem .sapMTabStripItemAddText {
	color: var(--sapContent_LabelColor);
	font-family: var(--sapFontFamily);
	font-size: var(--sapFontSmallSize);
}
.sapMSelectListItemBase.sapMSelectListItemBasePressed.sapMTSOverflowSelectListItem .sapMTabStripSelectListItemCloseBtn .sapMBtnIcon,
.sapMSelectListItemBase.sapMSelectListItemBasePressed.sapMTSOverflowSelectListItem .sapMSelectListItemText .sapMTabContIcon.sapUiIcon,
.sapMSelectListItemBase.sapMSelectListItemBasePressed.sapMTSOverflowSelectListItem .sapMTabStripItemAddText {
	color: var(--sapList_Active_TextColor);
}

.sapMTSTexts {
	display: inline-block;
	vertical-align: middle;
}

.sapMTabContIcon.sapUiIcon {
	font-size: 2rem;
	color: var(--sapSelectedColor);
	vertical-align: middle;
	padding-right: 0.5rem;
	cursor: pointer;
}

.sapMTabContIcon.sapMImg {
	height: 2rem;
	width: 2rem;
	vertical-align: middle;
	padding-right: 0.5rem;
}

/* Phone */

html.sap-phone {

	.sapMTabStrip {

		.sapMSltWithArrow {
			visibility: visible;
			display: none;
		}

		.sapMTabStripItem {
			line-height: 2.5rem;

			.sapMTSItemCloseBtnCnt {
				position: absolute;
				right: 0;
			}
		}

		.sapMTSTabs,
		.sapMTabStripItem {
			width: 100%;
		}
	}

	.sapMTabStrip.sapUiSelectable .sapMSltWithArrow {
		display: inline-block;
	}

	.sapMTabContainer.sapUiShowAddNewButton .sapMTabStrip.sapUiSelectable .sapMTSTabsContainer {
		right: 5.8rem;
	}

	.sapMTabContainer.sapUiShowAddNewButton .sapMTabStrip .sapMTSTabsContainer,
	.sapMTabContainer .sapMTabStrip.sapUiSelectable .sapMTSTabsContainer {
		right: 2.75rem;
	}

	.sapMTabContainer .sapMTabStrip .sapMTSTabsContainer {
		right: 0;
	}
}

/* Other styles */

html[data-sap-ui-animation='on'] .sapMTabStrip .sapMTSTabsContainer .sapMTSTabs .sapMTabStripItem {
	transition: padding 0.3s;
}

html[data-sap-ui-animation='on'] .sapMTabStrip .sapMTSTabsContainer .sapMTSTabs .sapMTabStripItem .sapMTSItemCloseBtnCnt .sapMBtn {
	transition: opacity 0.3s;
}

html[dir=ltr] .sapMTabStrip .sapMTSLeftOverflowButtons,
html[dir=rtl] .sapMTabStrip .sapMTSRightOverflowButtons {
	background: linear-gradient(90deg, @sapUiPageHeaderBackground 0%, @sapUiPageHeaderBackground 75%, fade(@sapUiPageHeaderBackground, 0) 100%);
}

html[dir=ltr] .sapMTabStrip .sapMTSLeftOverflowButtons::after,
html[dir=rtl] .sapMTabStrip .sapMTSRightOverflowButtons::after {
	background: linear-gradient(90deg, @sapUiObjectHeaderBorderColor 0%, @sapUiObjectHeaderBorderColor 75%, fade(@sapUiObjectHeaderBorderColor, 0) 100%);
}

html[dir=ltr] .sapMTabStrip .sapMTSRightOverflowButtons,
html[dir=rtl] .sapMTabStrip .sapMTSLeftOverflowButtons {
	background: linear-gradient(90deg, fade(@sapUiPageHeaderBackground, 0) 0%, @sapUiPageHeaderBackground 25%, @sapUiPageHeaderBackground 100%);
}

html[dir=ltr] .sapMTabStrip .sapMTSRightOverflowButtons::after,
html[dir=rtl] .sapMTabStrip .sapMTSLeftOverflowButtons::after {
	background: linear-gradient(90deg, fade(@sapUiObjectHeaderBorderColor, 0) 0%, @sapUiObjectHeaderBorderColor 25%, @sapUiObjectHeaderBorderColor 100%);
}

html .sapMTabStripSelectListItemCloseBtn .sapMBtnInner {
	display: block;
	height: 1.625rem;
	min-width: 1.625rem;
	width: 1.625rem;
	padding: 0;
	margin: 0 auto;
}

.sapMTabStripSelectListItemCloseBtn .sapMBtnIcon {
	line-height: 1.45rem;
	width: 100%;
	height: 100%;
	font-size: 1rem;
}

/* Form factor cozy */

.sapUiSizeCozy .sapMTabStrip {
	height: 3rem;

	.sapMTSLeftOverflowButtons {
		width: 3.5rem;
	}

	.sapMTSRightOverflowButtons {
		right: 5.75rem;
		width: 4rem;
	}
}

/* Form factor compact */

.sapUiSizeCompact .sapMTabStrip {
	height: 2.5rem;

	.sapMTSTabsContainer {

		.sapMTSTabs {

			.sapMTabStripItem {
				padding: 0 1rem;
				line-height: 2.5rem;

				.sapMTSItemCloseBtnCnt {
					width: 1.5rem;
					position: absolute;
					right: 0;

					.sapMBtn {
						opacity: 0;
					}
				}

				&.sapMTabStripItemSelected,
				&:hover {
					padding: 0 1.5rem 0 0.5rem;

					.sapMTSItemCloseBtnCnt {

						.sapMBtn {
							opacity: 1;
						}
					}
				}
			}
		}
	}

	.sapMTSTouchArea .sapMBtn,
	.sapMTSLeftOverflowButtons .sapMBtn,
	.sapMTSRightOverflowButtons .sapMBtn {
		height: 2.5rem;
	}

	.sapMTSTouchArea .sapMSlt {
		margin-top: 0.49375rem;
	}
}

.sapUiSizeCompact {

	.sapMTabContIcon.sapUiIcon {
		font-size: 1.5rem;
	}

	.sapMTabContIcon.sapMImg {
		height: 1.5rem;
		width: 1.5rem;
	}
}

/* Visibility */

.sapMTSOverflowSelect,
.sapUiSizeCompact .sapMTSOverflowSelectListItem .sapMTabStripSelectListItemCloseBtn {
	visibility: hidden;
}

.sapMTSScrollable .sapMTSOverflowSelect,
.sapMTabStrip.sapMTSScrollForward .sapMTSRightOverflowButtons,
.sapMTabStrip.sapMTSScrollBack .sapMTSLeftOverflowButtons,
.sapUiSizeCompact .sapMTSOverflowSelectListItem.sapMSelectListItemBaseSelected .sapMTabStripSelectListItemCloseBtn,
.sapUiSizeCompact .sapMTSOverflowSelectListItem:hover .sapMTabStripSelectListItemCloseBtn {
	visibility: visible;
}

/* Overflow Select */

.sapMTSOverflowSelectLabelModified::after {
	content: '*';
	display: inherit;
}

.sapMTSOverflowSelectListItem {
	padding-right: 0.25rem;

	.sapMSelectListItemText {
		height: 100%;
		float: left;
		margin: 0;
	}

	&.sapMTSOverflowSelectListItemModified {

		.sapMSelectListItemText::after {
			content: '*';
		}
	}

	.sapMTabStripSelectListItemCloseBtn {
		height: 100%;
		width: 2.25rem;
		padding: 0;
		float: right;
	}
}

.sapMTSOverflowSelectListItem .sapMTabStripSelectListItemCloseBtn {

	.sapMBtnInner {
		display: block;
		height: 1.625rem;
		min-width: 1.625rem;
		width: 1.625rem;
		padding: 0;
		margin: 0 auto;
	}

	.sapMBtnIcon {
		line-height: 1.5rem;
		width: 100%;
		height: 100%;
		font-size: 1rem;
	}
}

/* Overflow Select compact form factor */

.sapUiSizeCompact {

	.sapMTabStripSelectListItemCloseBtn {
		width: 2rem;

		.sapMBtnContent {
			line-height: 1.25rem;
		}

		.sapMBtnInner  {
			height: 1.25rem;
			min-width: 1.25rem;
			width: 1.25rem;
		}

		.sapMBtnIcon {
			line-height: 1.15rem;
			font-size: 0.75rem;
		}

		.sapMBtnInner.sapMBtnIconFirst {
			padding: 0;

			.sapMBtnIcon {
				margin: 0;
				padding: 0;
			}
		}
	}

	.sapMTSItemCloseBtnCnt .sapMBtn {
		height: 2.5rem;
	}

	.sapMTSOverflowSelectListItem.sapMSelectListItemBase {
		height: 2.5rem;
	}

	.sapMTSOverflowSelectListItem.sapMSelectListItem {
		line-height: 2.5rem;
	}
}
/* END "TabStrip.less" */

/* START "Text.less" */
/* =========================== */
/* CSS for control sap.m/Text  */
/* Base theme                  */
/* =========================== */

@_sap_m_Text_TextShadowInTransparentToolbar: none;

.sapMText {
	display: inline-block;
	box-sizing: border-box;
	white-space: pre-line;
	word-wrap: break-word; /* works only if browser knows the width */
	cursor: text;
	font-size: @sapMFontMediumSize;
	font-family: var(--sapFontFamily);
	line-height: normal;
	color: var(--sapTextColor);
}

html[data-sap-ui-browser='sf10'] {

	.sapUiVltCell,
	.sapuiVltCell {

		.sapMText {
			display: block;
		}
	}
}

/* BCP: 0020751295 0000279538 2020 */
.sapMText.sapMTextMaxLineWrapper {
	/* needed for line clamp to work */
	overflow: hidden;
}

.sapMTextNoWrap,
.sapMTextMaxLine {
	overflow: hidden;
}

.sapMTextMaxLine {
	display: block;
}

.sapMTextNoWrap {
	text-overflow: ellipsis;
	white-space: nowrap;
	word-wrap: normal; /* overwrite */
}

.sapMTextRenderWhitespaceWrap {
	white-space: pre-wrap;
}

.sapMTextRenderWhitespace {
	white-space: pre;
}

.sapMTextLineClamp {
	display: -webkit-box;
	-webkit-box-orient: vertical;
}

.sapMTextBreakWord {
	/* to break all words even browser does not know the width */
	word-break: break-all;
	word-break: break-word;  /* Non standard for old webkits */
}

.sapMTextMaxWidth {
	max-width: 100%;
}

.sapMLIBActionable .sapMText {
	cursor: pointer;
}

/*Necessary to align vertically to labels */
.sapUiForm .sapMText {
	overflow: hidden;
}

.sapUiFormEdit .sapMText {
	display: block;
	min-height: 3rem; /* for empty text */
	line-height: 1.375rem;
	padding-top: 0.8125rem;
	padding-bottom: 0.8125rem;
}

.sapUiForm:not(.sapUiFormEdit) .sapMText {
	min-height: 1rem; /* for empty text */
}

.sapMLIBActive .sapMText,
.sapMLIBActive .sapMListTblCell .sapMText,
.sapMLIBActive .sapMListTblSubCntRow .sapMText {
	color: var(--sapList_Active_TextColor);
}

.sapMLIBUnread .sapMText,
.sapMLIBUnread + .sapMListTblSubRow .sapMText {
	font-family: var(--sapFontFamily);
	font-weight: bold;
}

/* HorizontalLayout */
.sapUiHLayoutChildWrapper > .sapMText {
	vertical-align: text-bottom;
}

/* Transparent Toolbar Context*/
.sapMTB-Transparent-CTX .sapMText {
	font-weight: @sapUiFontHeaderWeight;
	font-family: var(--sapFontHeaderFamily);
	text-shadow: @_sap_m_Text_TextShadowInTransparentToolbar;
	color: var(--sapGroup_TitleTextColor);
}

/* Compact size */
.sapUiSizeCompact .sapUiFormEdit .sapMText {
	min-height: 2rem; /* for empty text */
	padding-top: 0.3125rem;
	padding-bottom: 0.3125rem;
}

html[data-sap-ui-browser^='cr'] .sapUiSizeCompact .sapUiFormEdit .sapMText {
	line-height: 1.375rem;
}

/* Condensed size / Table Specifics */
.sapUiTableCell .sapMText,
.sapMListTblCell > .sapMText.sapMTextNoWrap {
	vertical-align: middle;
}

.sapUiSizeCondensed .sapUiTableDataCell .sapMText {
	line-height: 1.5rem;
}

.sapUiTableHeaderDataCell .sapMText,
.sapMColumnHeader .sapMText {
	font-weight: @sapUiColumnHeaderFontWeight;
	font-size: @sapUiColumnHeaderFontSize;
	color: var(--sapList_HeaderTextColor);
}

.sapMText .sapMEmptyIndicator.sapMEmptyIndicatorAuto {
	display: none;
}

.sapMShowEmpty-CTX .sapMText .sapMEmptyIndicator.sapMEmptyIndicatorAuto {
	display: inline-block;
}
/* END "Text.less" */

/* START "TextArea.less" */
/* =============================== */
/* CSS for control sap.m/TextArea  */
/* Base theme                      */
/* =============================== */

@_sapMTextAreaPadding: 0.4375rem 0.5625rem;
@_sapMTextAreaPadding_Compact: 0.125rem 0.4375rem;
@_sapMTextAreaReadonlyPadding: 0.4375rem 0.5625rem;
@_sapMTextAreaReadonlyPadding_Compact: 0.125rem 0.4375rem;
@_sapMTextAreaValueStateSuccessPadding: 0.4375rem 0.5625rem;
@_sapMTextAreaValueStateSuccessPadding_Compact: 0.125rem 0.4375rem;
@_sapMTextAreaValueStateInformationPadding: 0.375rem 0.5rem;
@_sapMTextAreaValueStateInformationPadding_Compact: 0.0625rem 0.375rem;
@_sapMTextAreaValueStateErrorWarningPadding: 0.375rem 0.5rem;
@_sapMTextAreaValueStateErrorWarningPadding_Compact: 0.0625rem 0.375rem;
@_sapMTextAreaContentWrapperPadding: 0;
@_sapMTextAreaContentWrapperValueStateSuccessPadding: 0;
@_sapMTextAreaContentWrapperValueStateInformationPadding: 0;
@_sapMTextAreaContentWrapperValueStateErrorWarningPadding: 0;

/* uses sapMInputBase styles */

.sapMTextAreaInner {
	overflow: auto; /* remove scrollbars if not needed */
	-webkit-transform: none; /* transform is blocking scroll */
	resize: none;
	margin: 0;
}

.sapMTextArea .sapMInputBaseContentWrapper {
	-webkit-overflow-scrolling: touch;
	height: 100%;
	width: 100%;
	line-height: 0;
	display: block;
	padding: @_sapMTextAreaContentWrapperPadding;
}

.sapMTextArea .sapMTextAreaInner {
	-webkit-overflow-scrolling: touch;
	line-height: 1.4;
	height: 100%;
	width: 100%;
}

.sapMTextAreaWithCounter {

	& .sapMTextAreaCounter {
		float:right;
		overflow: hidden;
		font-family: var(--sapFontFamily);
		font-size: var(--sapFontSmallSize);
		color: var(--sapContent_LabelColor);
		padding: 0.3725rem 0.125rem 0.5rem;
	}

	& .sapMInputBaseContentWrapper {
		height: calc(~"100% - 1.5rem");
	}
}

.sapMTextAreaGrow {
	overflow: hidden;
	position: absolute;
	height: 100%;
	top: 0;
}

/* ensure that the mirror div has the same styles like the inner text area */
.sapMTextAreaMirror {
	/* should not grow more than screen size */
	max-height: calc(100vh - 6rem);
	white-space: pre-wrap;
	border: none;
	overflow: hidden;
	line-height: 1.4;
	visibility: hidden;
	word-break: break-word;
	font-size: var(--sapFontSize);
}

.sapMInputBase.sapMTextArea {
	height: auto;
	min-height: 2.5rem;
	min-width: 6rem;

	.sapMTextAreaInner {
		line-height: 1.4;
		-webkit-overflow-scrolling: touch; /* IOS only */
	}

	&.sapMInputBaseReadonly .sapMInputBaseReadonlyWrapper {
		background-image: none;
	}
}

.sapMInputBase.sapMTextArea {

	.sapMTextAreaInner,
	.sapMTextAreaMirror {
		padding: @_sapMTextAreaPadding;
	}

	.sapMInputBaseContentWrapperState.sapMInputBaseContentWrapperError,
	.sapMInputBaseContentWrapperState.sapMInputBaseContentWrapperWarning {
		padding: @_sapMTextAreaContentWrapperValueStateErrorWarningPadding;

		.sapMTextAreaInner,
		.sapMTextAreaMirror {
			padding: @_sapMTextAreaValueStateErrorWarningPadding;
		}
	}
	.sapMInputBaseContentWrapperState.sapMInputBaseContentWrapperSuccess {
		padding: @_sapMTextAreaContentWrapperValueStateSuccessPadding;

		.sapMTextAreaInner,
		.sapMTextAreaMirror {
			padding: @_sapMTextAreaValueStateSuccessPadding;
		}
	}
	.sapMInputBaseContentWrapperState.sapMInputBaseContentWrapperInformation {
		padding: @_sapMTextAreaContentWrapperValueStateInformationPadding;

		.sapMTextAreaInner,
		.sapMTextAreaMirror {
			padding: @_sapMTextAreaValueStateInformationPadding;
		}
	}
}

.sapMInputBase.sapMTextArea.sapMInputBaseReadonly:not(.sapMInputBaseDisabled) {

	.sapMTextAreaInner,
	.sapMTextAreaMirror {
		padding: @_sapMTextAreaReadonlyPadding;
	}
}

.sapMFocus.sapMTextArea {

	& .sapMInputBaseContentWrapper {
		outline: none;

		.sapMInputBaseInner {
			outline: var(--sapContent_FocusWidth) var(--sapContent_FocusStyle) var(--sapContent_FocusColor);
			outline-offset: -2px;
		}
	}
}

/**** Compact Size ****/
.sapUiSizeCompact {

	.sapMInputBase.sapMTextArea {
		min-height: 1.625rem;
		margin: 0.1875rem 0;
	}

	.sapMTextAreaWithCounter {

		& .sapMTextAreaCounter {
			padding-top: 0.3125rem;
		}
	}

	.sapMInputBase.sapMTextArea.sapMInputBaseReadonly:not(.sapMInputBaseDisabled) {

		.sapMTextAreaInner,
		.sapMTextAreaMirror {
			padding: @_sapMTextAreaReadonlyPadding_Compact;
		}
	}
}

.sapUiSizeCompact .sapMInputBase.sapMTextArea {

	.sapMTextAreaInner,
	.sapMTextAreaMirror {
		padding: @_sapMTextAreaPadding_Compact;
	}

	.sapMInputBaseContentWrapperState.sapMInputBaseContentWrapperError,
	.sapMInputBaseContentWrapperState.sapMInputBaseContentWrapperWarning {

		.sapMTextAreaInner,
		.sapMTextAreaMirror {
			padding: @_sapMTextAreaValueStateErrorWarningPadding_Compact;
		}
	}

	.sapMInputBaseContentWrapperState.sapMInputBaseContentWrapperSuccess {

		.sapMTextAreaInner,
		.sapMTextAreaMirror {
			padding: @_sapMTextAreaValueStateSuccessPadding_Compact;
		}
	}

	.sapMInputBaseContentWrapperState.sapMInputBaseContentWrapperInformation {

		.sapMTextAreaInner,
		.sapMTextAreaMirror {
			padding: @_sapMTextAreaValueStateInformationPadding_Compact;
		}
	}
}

/**
	Fix for iOS. When growing is enabled, the mirrored, hidden content should have pixel perfect proportions
	as the textarea. Otherwise, words could break differently and this would lead to inconsistent height.
	For some reason iOS devices need wider padding on left & right in order to have this consistency.
 */
html.sap-tablet,
html.sap-phone {

	&[data-sap-ui-os*='iOS'] .sapMInputBase.sapMTextArea .sapMTextAreaMirror {
		padding-left: 15px;
		padding-right: 15px;
	}
}
/* END "TextArea.less" */

/* START "Tile.less" */
/* =========================== */
/* CSS for control sap.m/Tile  */
/* Base theme                  */
/* =========================== */

.sapMTile {
	display: inline-block;
	transition-property: transform3d;
	transition-duration: 0.2s;
	width: 12rem;
	height: 14rem;
	margin: 0.5rem;
	-webkit-touch-callout: none;
	background-color: var(--sapTile_Background);
	border: 1px solid var(--sapTile_BorderColor);
	border-radius: 0.125rem;
	box-shadow: @sapUiShadowLevel0;
}

.sapMTile * { /* FIXME!!! (star rule with any-parent selector...) */
	-webkit-user-select: none;
	-webkit-user-drag: none;
	-moz-user-select: -moz-none;
}

.sapMTCEditable .sapMTile {
	background-color: fade(@sapUiTileBackground, 50);
}

.sapMTile.sapMTileActive {
	background: var(--sapHighlightColor);
}

html.sap-phone .sapMTile {
	width: 8.5rem;
	height: 10rem;
	margin: 0.5rem;
}

html.sap-phone .sapMTileContent {
	width: 8.5rem;
	height: 10rem;
}

.sapMTileContent {
	overflow: hidden;
	width: 12rem;
	height: 14rem;
}

.sapMTileDrag {
	opacity: 0.8;
	z-index: 10;
	transition-property: none;
}

.sapMNavItemFlipping .sapMTile {
	opacity: 0.01;
}

.sapMTile:focus {
	outline: 1px dotted var(--sapContent_FocusColor);
	position: relative;
}

.sapMTile .sapMTileDrag {
	opacity: 0.6;
	box-shadow: 0 1px 3px fade(@sapUiContentShadowColor,60);
}
/* END "Tile.less" */

/* START "TileContent.less" */
/* ================================== */
/* CSS for control sap.m/TileContent  */
/* Base theme                         */
/* ================================== */

.sapMTileCnt {
	display: inline-block;
	box-sizing: border-box;
	position: relative;
	height: 6.375rem;
    vertical-align: top;

	/* FrameType: Auto */
	&.sapMFrameTypeAuto {
		min-width: 9rem;
	}

	/* FrameType: TwoByOne */
	&.sapMFrameTypeTwoByOne {
		width: 100%;
	}

	/* FrameType: OneByOne */
	&.sapMFrameTypeOneByOne {
		width: 100%;
	}
}

/* Improved behaviour for TileContent in GenericTile */
.sapMGT .sapMTileCnt.sapMFrameTypeAuto:only-of-type {
	width: 100%;
}

.sapMGT .sapMTileCnt.sapMFrameTypeAuto:not(:only-of-type) {
	width: ~"calc(50% - 0.25rem)"; //0.5rem margin between the contents
}

/* Footer CSS */
.sapMTileCntFtrTxt {
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
	text-align: right;
	position: absolute;
	right: 0;
	left: 0;
	box-sizing: border-box;
	height: 1rem;
	bottom: 1rem;
	margin-left: 1rem;
	margin-right: 1rem;
	line-height: normal;
	font-family: @sapUiFontFamily;
	font-size: @sapMFontMediumSize;
	font-weight: normal;
}

.sapMGT .sapMTileCntFtrTxt {
	text-align: left;
}

html[dir=rtl] .sapMGT .sapMTileCntFtrTxt {
	text-align: right;
}

.sapMTileCntFooterTextColorNeutral {
	color: @sapUiTileTextColor;
}

.sapMTileCntFooterTextColorGood {
	color: @sapUiPositiveText;
}

.sapMTileCntFooterTextColorError {
	color: @sapUiNegativeText;
}

.sapMTileCntFooterTextColorCritical {
	color: @sapUiCriticalText;
}

html[dir=rtl] .sapMTileCntFtrTxt {
	text-align: left;
}

/* Content */
.sapMTileCntContent {
	overflow: visible;
	display: inline-block;
	width: 100%;
	box-sizing: border-box;
}

.sapMTileCntContent > .sapMTcInnerMarker {
	vertical-align: bottom;
	line-height: normal;
}

.sapMTileCntContent {
	height: 3.875rem;
	margin-bottom: 0.25rem;
	margin-top: 0.25rem;
	padding-left: 1rem;
	padding-right: 1rem;
}

.sapMTileCnt + .sapMTileCnt {
	margin-left: 0.5rem;
}

.sapMTileCntContent > .sapMNwC {
	background-color: inherit;
}

.sapMTileCnt.News {
	height: 6.125rem;
	margin-top: 0.125rem;
	background-color: @sapUiTileBackground;

	.sapMTileCntContent {
		padding-left: 1rem;
		padding-right: 1rem;
	}

	.sapMTileCntFtrTxt {
		margin-left: 1rem;
		margin-right: 1rem;
	}

	.sapMTileCntContent > .sapMTcInnerMarker {
		vertical-align: top;
	}
}

.Stretch {
	.sapMTileCnt.News{
		height: 100%;

		.sapMTileCntContent {
			margin-top: 1rem;
		}

		.sapMTileCntFtrTxt {
			font-size: @sapMFontSmallSize;
		}
	}
}

// Styles when the slide tile has only one tile inside it

.Stretch {
	.sapMTileCnt.News.sapMTileFooterPresent{
		.sapMTileCntContent {
			height: -webkit-calc(~'100% - 50px') !important;
		}
	}
	&.sapMTileSmallPhone {
		.sapMTileCnt.News.sapMTileFooterPresent{
			.sapMTileCntContent {
				height: -webkit-calc(~'100% - 32px') !important;
			}
		}
	}
	.sapMTileCnt.News {
		.sapMTileCntContent {
			height: -webkit-calc(~'100% - 32px');
		}
	}
	&.sapMTileSmallPhone {
		.sapMTileCnt.News {
			.sapMTileCntContent {
				height: -webkit-calc(~'100% - 12px');
			}
		}
	}
}

// Styles when the slide tile has only one tile inside it

.sapMST.sapMSTIndicatorVisible.Stretch {
	.sapMTileCnt.News {
		.sapMNwCCTxt,.sapMNwCSbh {
			.sapMFT {
				text-overflow: initial;
				 white-space: initial;
			}
		}
		&.sapMTileFooterPresent {
			.sapMTileCntContent {
				height: -webkit-calc(~'100% - 65px') !important;
			}
		}
		.sapMTileCntContent {
			height: -webkit-calc(~'100% - 51px') !important;
		}
	}
	.sapMTileCntFtrTxt {
		bottom: 1.875rem;
	}
}

.sapMST.sapMSTIndicatorVisible.Stretch.sapMTileSmallPhone {
	.sapMTileCnt.News {
		&.sapMTileFooterPresent {
			.sapMTileCntContent {
				height: -webkit-calc(~'100% - 54px') !important;
			}
		}
		.sapMTileCntContent {
			height: -webkit-calc(~'100% - 40px') !important;
		}
	}
	.sapMTileCntFtrTxt {
		bottom: 1.875rem;
	}
}

.sapMTileSmallPhone {
	.sapMTileCnt {
		height: 5.5rem;
	}

	.sapMTileCntFtrTxt {
		height: 1.063rem;
		line-height: normal;
		bottom: 0.5rem;
		margin-left: 0.5rem;
		margin-right: 0.5rem;
		font-size: @sapMFontSmallSize;
	}

	.sapMTileCntContent {
		margin-bottom: 0.25rem;
		margin-top: 0.25rem;
		padding-left: 0.5rem;
		padding-right: 0.5rem;
	}

	.sapMTileCnt.News {
		height: 5.25rem;

		.sapMTileCntContent {
			padding-left: 0.5rem;
			padding-right: 0.5rem;
			margin-top: 0.25rem;
		}

		.sapMTileCntFtrTxt {
			margin-left: 0.5rem;
			margin-right: 0.5rem;
		}
	}

	.sapMTileSmallPhone.Stretch {
		.sapMTileCnt.News {
			height: 100%;
		}
		.sapMTileFtrCnt > .sapMTileNavContainer {
			padding-bottom: 1rem;
		}	
	}
}
.sapMGTStateLoading {
	&.sapMTileSmallPhone:not(.sapMGTActionMode) {
		.sapMTileCntContentShimmerPlaceholderWithDescription{
			padding-top: 0;
		}
		.sapMTileCntContentShimmerPlaceholderItem {
			.sapMTileCntContentShimmerPlaceholderRows {
				.sapMTileCntContentShimmerPlaceholderItemTextFooter, .sapMTileCntContentShimmerPlaceholderItemBox {
					top:0rem;
				}
			}
		}
	}
	.sapMTileCntContentShimmerPlaceholderItem {
		.sapMTileCntContentShimmerPlaceholderRows {
			.sapMTileCntContentShimmerPlaceholderItemTextFooter, .sapMTileCntContentShimmerPlaceholderItemBox {
				top:1rem;
			}
		}
	}
	.sapMTileCntContentShimmerPlaceholderWithDescription {
		padding: 1rem 0rem 0rem 0rem;
	}
}
.sapMGT.sapMTileSmallPhone.sapMGTStateLoading.OneByOne {
	.sapMTileCntContentShimmerPlaceholderItemTextFooter{
		width: 5.875rem;
	}
}
.sapMTileCntContentShimmerPlaceholderItem {
	.sapMTileCntContentShimmerPlaceholderRows {
		.sapMTileCntContentShimmerPlaceholderItemTextFooter, .sapMTileCntContentShimmerPlaceholderItemBox {
			top:0.85rem;
		}
	}
}

.sapMGTStateLoaded.TwoByHalf, .sapMGTStateLoaded.OneByHalf,
.sapMGTStateDisabled.TwoByHalf, .sapMGTStateDisabled.OneByHalf,
.sapMGTStateFailed.TwoByHalf, .sapMGTStateFailed.OneByHalf  {
	.sapMTileCntContentShimmerPlaceholderItem.sapMTileCntContentShimmerPlaceholderWithDescription {
		padding: 0rem 1rem 0rem 1rem;
	}

}


.sapMTileCnt.sapMTileCntDisabled {
	position: relative;

	&:after {
		.sapMGTOverlayMixin();
		display: block;
		content: "";
		background-color: var(--sapTile_OverlayBackground);
		opacity: 0.7;
	}
}
/* END "TileContent.less" */

/* START "TileContainer.less" */
/* ==================================== */
/* CSS for control sap.m/TileContainer  */
/* Base theme                           */
/* ==================================== */

.sapMTC {
	top: 0;
	bottom: 0;
	left: 0;
	right: 0;
	position: relative;
	overflow: hidden;
}

.sapMTC .sapMTCScrl {
	padding: 0;
	position: absolute;
	left: 0;
	top: 0;
	overflow: visible;
}

.sapMTC:focus,
.sapMTC .sapMTCScroller:focus {
	outline: none;
}

.sapMTC .sapMTCCnt {
	height: 100%;
	width: 100%;
	overflow: visible;
	position: absolute;
	left: 0;
	top: 0;
	-webkit-touch-callout: none;
	-webkit-user-select: none;
	-moz-user-select: none;
}

html.sap-tablet .sapMTC .sapMTCScrl,
html.sap-phone .sapMTC .sapMTCScrl {
	padding: 0;
}

.sapMTC.sapMTCEditable .sapMTCCnt {
	opacity : 1;
}

.sapMTC .sapMTCCnt.sapMTCAnim {
	transition-property: transform3d;
	transition-duration: 0.4s;
}

.sapMTC .sapMTCPager {
	position: absolute;
	bottom: 0;
	box-sizing: border-box;
	width: 100%;
	height: 1.875rem;
	padding: 0.313rem;
	text-align: center;
}

.sapMTC .sapMTCPager span {
	display: inline-block;
	background-color: var(--sapContent_ForegroundColor);
	border-radius: 0.25rem;
	height: 0.5rem;
	width: 0.5rem;
	margin-right: 0.5rem;
	margin-bottom: 1px;
	cursor: default;
}

.sapMTC .sapMTCBlind {
	background-color: var(--sapGroup_ContentBackground);
	position: absolute;
	top: 0;
	left: 0;
	bottom: 0;
	right: 0;
	display:none;
	box-shadow: @sapUiShadowLevel1;
}

.sapMTC .sapMTCPager span.sapMTCActive {
	background-color: var(--sapContent_NonInteractiveIconColor);
}

.sapMTC .sapMTCScroller {
	position: absolute;
	display: none;
	top: 41%;
	border-radius: 50%;
	background-color: var(--sapButton_Background);
	height: 3rem;
	width: 3rem;
	opacity: 0;
	cursor: default;
	transition-property: opacity, left, right;
	transition-duration: 0.4s;
	box-shadow: @sapUiShadowLevel1;
	padding: 0.25rem;
	box-sizing: border-box;
}

.sapMTC .sapMTCScroller .sapMTCInner {
	background-color: var(--sapUiButtonBackground);
	height: 100%;
	width: 100%;
	border-radius: 50%;
	font-family: var(--sapContent_IconFontFamily);
	font-size: 1.313rem;
	vertical-align: middle;
	color: var(--sapButton_IconColor);
	padding-top: 0.5rem;
	box-sizing: border-box;
}

.sapMTC:hover .sapMTCScroller {
	opacity: 0.1;
}

.sapMTC:hover .sapMTCScroller:hover {
	opacity: 1;
}

.sapMTC .sapMTCScroller.sapMTCLeft {
	left: -100px;
}

.sapMTC .sapMTCScroller.sapMTCLeft .sapMTCInner::before {
	content: '\e067';
	padding-left: 0.813rem;
}

.sapMTC:hover .sapMTCScroller.sapMTCLeft {
	left: 1rem;
}

.sapMTC .sapMTCScroller.sapMTCRight {
	right: -100px;
}

.sapMTC:hover .sapMTCScroller.sapMTCRight {
	right: 1rem;
}

.sapMTC .sapMTCScroller.sapMTCRight .sapMTCInner::before {
	content: '\e066';
	padding-left: 1rem;
}

.sapMTC .sapMTile,
.sapMTC .sapMCustomTile {
	position: absolute;
	top: 0;
	left: 0;
}

html[dir=rtl] .sapMTC .sapMTile,
html[dir=rtl] .sapMTC .sapMCustomTile {
	left: auto;
	right: 0;
}

html.sap-tablet .sapMTC .sapMTCScroller,
html.sap-phone .sapMTC .sapMTCScroller {
	display : none;
}

.sapMTC .sapMTCRemove {
	display: none;
}

.sapMTC.sapMTCEditable .sapMTile {
	cursor: move;
	box-shadow: @sapUiShadowLevel0;
}

.sapMTC.sapMTCEditable .sapMTCRemove {
	position: absolute;
	display: block;
	height: 2rem;
	width: 2rem;
	border-radius: 50%;
	font-family: var(--sapContent_IconFontFamily);
	font-size: 1rem;
	line-height: 2rem;
	text-align: center;
	box-sizing: border-box;
	margin-top: -0.5rem;
	margin-left: -0.5rem;
	z-index: 1;
	cursor: default;
	background-color: var(--sapUiButtonBackground);
	border: 1px solid var(--sapButton_BorderColor);
	color: var(--sapButton_IconColor);
}

html[data-sap-ui-browser^='ff'] .sapMTC.sapMTCEditable .sapMTile .sapMTCRemove {
	width: 2rem;
}

.sapMTC.sapMTCEditable .sapMTile .sapMTCNoRemove {
	height: 1.5rem;
	width: 1.563rem;
	margin-top: -0.5em;
	margin-left: -0.5em;
	z-index: 1;
	visibility: hidden;
}

.sapMTC.sapMTCEditable .sapMTCRemove:hover {
	background-color: var(--sapButton_Hover_Background);
}

.sapMTC.sapMTCEditable .sapMTCRemove:active {
	background-color: var(--sapButton_Active_Background);
	color: var(--sapButton_Active_TextColor);
}

.sapMTC.sapMTCEditable .sapMTCRemove::before {
	content: '\e03e';
}

/* use dir-specific rules to prevent mirroring in dir=rtl mode */
.sapMTC .sapMTCEdgeRight,
html[dir=rtl] .sapMTC .sapMTCEdgeLeft {
	position: absolute;
	top: 0;
	right: 0;
	width: 2.5rem;
	height: 100%;
	border-right: 0.125rem solid fade(lighten(@sapUiHighlight,20),60);
	opacity: 0.01;
	background: linear-gradient(to right, rgba(255,255,255,0) 0%, rgba(255,255,255,1) 76%, rgba(255,255,255,1) 100%);
}

/* use dir-specific rules to prevent mirroring in dir=rtl mode */
.sapMTC .sapMTCEdgeLeft,
html[dir=rtl] .sapMTC .sapMTCEdgeRight {
	position: absolute;
	top: 0;
	left: 0;
	width: 2.5rem;
	height: 100%;
	border-left: 0.125rem solid fade(lighten(@sapUiHighlight,20),60);
	opacity: 0.01;
	background: linear-gradient(to right, rgba(255,255,255,1) 0%,rgba(255,255,255,1) 24%,rgba(255,255,255,0) 100%);
}

html.sap-phone .sapMTC .sapMTCEdgeLeft,
html.sap-phone .sapMTC .sapMTCEdgeRight {
	width: 1.25rem;
}
/* END "TileContainer.less" */

/* START "TimePicker.less" */
/* ================================= */
/* CSS for control sap.m/TimePicker  */
/* Base theme                        */
/* ================================= */

/* TimePicker Input Field */

.sapMInputBase.sapMTimePickerHiddenInput {
	display: none;
}

/* TimePicker Dropdown*/

.sapMTimePickerDropDown {
	overflow: hidden !important;
	-webkit-user-select: none;
	-moz-user-select: none;
	user-select: none;
	box-shadow: @sapUiShadowLevel1 !important; /* Need to override this with important because of popover stronger selectors */

	.sapMTPClocksContainer .sapMSegB .sapMSegBBtn .sapMSegBBtnInner {
		text-overflow: initial;
	}

}

/* COMPACT MODE STYLES */

.sapUiSizeCompact {

	/* TimePicker Dropdown */
	.sapMTimePickerDropDown {
		box-shadow: @sapUiShadowLevel1 !important; /* Need to override this with important because of popover stronger selectors */
	}
}

/* PHONE STYLES */
html.sap-phone {

	/* TimePicker Dropdown */
	.sapMTimePickerDropDown {

		/* This centers content in landscape & portrait mode */
		.sapMDialogScrollCont {
			height: 100%;
			padding: 0;
		}
	}
}

/* DESKTOP STYLES */
html.sap-desktop .sapMValueStateHeaderRoot {
	margin-right: -12px; /* fixing a right margin set from popover */
	max-width: 100%;
}
/* END "TimePicker.less" */

/* START "TimePickerInputs.less" */
/* ====================================== */
/* CSS for control sap.m/TimePickerInputs */
/* Base theme                             */
/* ====================================== */

.sapMTPInputsContainer {
	min-width: 12.5rem;
	padding: 1rem;
	display: flex;
	justify-content: center;
	align-items: center;

	& > span {
		min-width: 0.5rem;
		padding: 0 0.125rem;
		text-align: center;
		font-family: var(--sapFontFamily);
		font-size: @sapMFontMediumSize;
		color: var(--sapTextColor);
	}
}

.sapMTPNow {
	margin-left: 0.5rem;
}
/* END "TimePickerInputs.less" */

/* START "TimePickerClock.less" */
/* ====================================== */
/* CSS for control sap.m/TimePickerClock  */
/* Base theme                             */
/* ====================================== */

/* clock dial related classes */

.sapMTPClock {
	position: relative;
	width: auto;
	padding: 0.5625rem;
	outline: none;
	display: none;
}

.sapMTPClock.sapMTPCFadeIn {
	display: block;
}

.sapMTPCDial {
	position: relative;
	width: auto;
	display: block;
	border-radius: 100%;
	background-color: var(--sapLegend_WorkingBackground);
	text-align: center;
	z-index: 1;
}

.sapMTPCDial::before {
	content: attr(data-label);
	display: flex;
	align-items: center;
	justify-content: center;
	position: absolute;
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	color: var(--sapContent_LabelColor);
	top: 2.75rem;
	left: 2.75rem;
	bottom: 2.75rem;
	right: 2.75rem;
	border-radius: 100%;
	z-index: 2;
}

.sapMTPCInner .sapMTPCDial::before {
	background-color: var(--sapLegend_WorkingBackground);
}

.sapMTPCDial::after {
	content: '';
	display: block;
	padding-bottom: 100%;
}

.sapMTPClockCover {
	position: absolute;
	top: -0.25rem;
	left: -0.25rem;
	bottom: -0.25rem;
	right: -0.25rem;
	border-radius: 100%;
	z-index: 10;
	touch-action: none;
}

/* units related classes */
.sapMTPCMidDot {
	display: block;
	box-sizing: border-box;
	width: 0.125rem;
	height: 0.1875rem;
	margin: 0 auto 0.375rem auto;
	border-radius: 100%;
	background-color: var(--sapField_BorderColor);
}

.sapMTPCDot {
	display: block;
	box-sizing: border-box;
	width: 0.25rem;
	height: 0.3125rem;
	margin: 0 auto 0.25rem auto;
	border-radius: 100%;
	background-color: var(--sapField_BorderColor);
}

.sapMTPCMarker {
	box-sizing: border-box;
	width: 0.25rem;
	height: 0.5625rem;
	margin: 0 auto;
	border-radius: 100%;
	background-color: var(--sapButton_Selected_Background);
	border: 0.0625rem solid var(--sapButton_Selected_BorderColor);
}

.sapMTPCHoverMarker {
	box-sizing: border-box;
	width: 0.25rem;
	height: 0.5625rem;
	margin: 0 auto;
	border-radius: 100%;
	background-color: var(--sapList_Hover_Background);
	border: 0.0625rem solid var(--sapGroup_TitleBorderColor);
}

.sapMTPCHoveredItem {
	display: block;
}

.sapMTPCItem {
	position: absolute;
	top: 0px;
	left: 50%;
	display: inline-block;
	width: 2.75rem;
	height: 100%;
	z-index: 3;
	-webkit-touch-callout: none; /* iOS Safari */
	-webkit-user-select: none; /* Safari */
	user-select: none; /* Chrome, Edge, Opera and Firefox */

	&.sapMTPCMarker {
		z-index: 4;
	}

	&.sapMTPCHoverMarker {
		z-index: 5;
	}
}

.sapMTPCNumber {
	box-sizing: border-box;
	display: inline-block;
	width: 2.75rem;
	height: 2.75rem;
	border-radius: 100%;
	line-height: 2.75rem;
	text-align: center;
	vertical-align: top;
	font-family: var(--sapFontFamily);
	font-size:@sapMFontMediumSize;
	color: var(--sapTextColor);
	border: 0.0625rem solid transparent;

	&.sapMTPCNumberHover:not(.sapMTPCSelected) {
		background-color: var(--sapList_Hover_Background);
	}
}

.sapMTPCInvisible {
	visibility: hidden;
}

.sapMTPCSelected {
	box-sizing: border-box;
	color: var(--sapButton_Selected_TextColor);
	background-color: var(--sapButton_Selected_Background);
	border: 0.0625rem solid var(--sapButton_Selected_BorderColor);
}

.sapMTPCHovered {
	box-sizing: border-box;
	background-color: var(--sapList_Hover_Background);
	border: 0.0625rem solid var(--sapGroup_TitleBorderColor);
}

/* Degree related classes */
.degreeMixIn(@class, @angle) {
	.@{class} {
		transform: translate(-50%) rotate(@angle);

		.sapMTPCNumber {
			transform: rotate(-@angle);
		}
	}

	html[dir=rtl] .@{class} {
		transform: translate(-50%) rotate(-@angle);

		.sapMTPCNumber {
			transform: rotate(@angle);
		}
	}
}

.sapMTPNow {
	margin-left: 0.5rem;
}

.sapUiSizeCompact {

	.sapMTPCItem {
		width: 2rem;
	}

	.sapMTPCNumber {
		width: 2rem;
		height: 2rem;
		line-height: 2rem;
	}
}

.degreeMixIn(sapMTPCDeg6, 6deg);
.degreeMixIn(sapMTPCDeg12, 12deg);
.degreeMixIn(sapMTPCDeg15, 15deg);
.degreeMixIn(sapMTPCDeg18, 18deg);
.degreeMixIn(sapMTPCDeg24, 24deg);
.degreeMixIn(sapMTPCDeg30, 30deg);
.degreeMixIn(sapMTPCDeg36, 36deg);
.degreeMixIn(sapMTPCDeg42, 42deg);
.degreeMixIn(sapMTPCDeg45, 45deg);
.degreeMixIn(sapMTPCDeg48, 48deg);
.degreeMixIn(sapMTPCDeg54, 54deg);
.degreeMixIn(sapMTPCDeg60, 60deg);
.degreeMixIn(sapMTPCDeg66, 66deg);
.degreeMixIn(sapMTPCDeg72, 72deg);
.degreeMixIn(sapMTPCDeg75, 75deg);
.degreeMixIn(sapMTPCDeg78, 78deg);
.degreeMixIn(sapMTPCDeg84, 84deg);
.degreeMixIn(sapMTPCDeg90, 90deg);
.degreeMixIn(sapMTPCDeg96, 96deg);
.degreeMixIn(sapMTPCDeg102, 102deg);
.degreeMixIn(sapMTPCDeg105, 105deg);
.degreeMixIn(sapMTPCDeg108, 108deg);
.degreeMixIn(sapMTPCDeg114, 114deg);
.degreeMixIn(sapMTPCDeg120, 120deg);
.degreeMixIn(sapMTPCDeg126, 126deg);
.degreeMixIn(sapMTPCDeg132, 132deg);
.degreeMixIn(sapMTPCDeg135, 135deg);
.degreeMixIn(sapMTPCDeg138, 138deg);
.degreeMixIn(sapMTPCDeg144, 144deg);
.degreeMixIn(sapMTPCDeg150, 150deg);
.degreeMixIn(sapMTPCDeg156, 156deg);
.degreeMixIn(sapMTPCDeg162, 162deg);
.degreeMixIn(sapMTPCDeg165, 165deg);
.degreeMixIn(sapMTPCDeg168, 168deg);
.degreeMixIn(sapMTPCDeg174, 174deg);
.degreeMixIn(sapMTPCDeg180, 180deg);
.degreeMixIn(sapMTPCDeg186, 186deg);
.degreeMixIn(sapMTPCDeg192, 192deg);
.degreeMixIn(sapMTPCDeg195, 195deg);
.degreeMixIn(sapMTPCDeg198, 198deg);
.degreeMixIn(sapMTPCDeg204, 204deg);
.degreeMixIn(sapMTPCDeg210, 210deg);
.degreeMixIn(sapMTPCDeg216, 216deg);
.degreeMixIn(sapMTPCDeg222, 222deg);
.degreeMixIn(sapMTPCDeg225, 225deg);
.degreeMixIn(sapMTPCDeg228, 228deg);
.degreeMixIn(sapMTPCDeg234, 234deg);
.degreeMixIn(sapMTPCDeg240, 240deg);
.degreeMixIn(sapMTPCDeg246, 246deg);
.degreeMixIn(sapMTPCDeg252, 252deg);
.degreeMixIn(sapMTPCDeg255, 255deg);
.degreeMixIn(sapMTPCDeg258, 258deg);
.degreeMixIn(sapMTPCDeg264, 264deg);
.degreeMixIn(sapMTPCDeg270, 270deg);
.degreeMixIn(sapMTPCDeg276, 276deg);
.degreeMixIn(sapMTPCDeg282, 282deg);
.degreeMixIn(sapMTPCDeg285, 285deg);
.degreeMixIn(sapMTPCDeg288, 288deg);
.degreeMixIn(sapMTPCDeg294, 294deg);
.degreeMixIn(sapMTPCDeg300, 300deg);
.degreeMixIn(sapMTPCDeg306, 306deg);
.degreeMixIn(sapMTPCDeg312, 312deg);
.degreeMixIn(sapMTPCDeg315, 315deg);
.degreeMixIn(sapMTPCDeg318, 318deg);
.degreeMixIn(sapMTPCDeg324, 324deg);
.degreeMixIn(sapMTPCDeg330, 330deg);
.degreeMixIn(sapMTPCDeg336, 336deg);
.degreeMixIn(sapMTPCDeg342, 342deg);
.degreeMixIn(sapMTPCDeg345, 345deg);
.degreeMixIn(sapMTPCDeg348, 348deg);
.degreeMixIn(sapMTPCDeg354, 354deg);
.degreeMixIn(sapMTPCDeg360, 360deg);

/* animation CSS classes */

.sapMTPClock.sapMTPCDisplay {
	display: block;
}

.sapMTPCFadeIn:not(.sapMTPCSkipAnimation):not(.sapMTPCFadeOut) {

	.sapMTPCItems .sapMTPCNumber,
	.sapMTPCDial::before,
	.sapMTPCItems .sapMTPCMidDot,
	.sapMTPCSelectedItem .sapMTPCNumber,
	.sapMTPCMarker {
		opacity: 0;
	}
}

.sapMTPCFadeIn.sapMTPCFadeOut:not(.sapMTPCSkipAnimation) {

	.sapMTPCHoveredItem {
		display: none;
	}

	.sapMTPCItems .sapMTPCNumber {
		animation-name: fadeOutClockItems;
		animation-delay: 0.15s;
		animation-duration: 0.2s;
		animation-timing-function: ease-out;
		animation-fill-mode: forwards;
	}

	.sapMTPCDial::before {
		animation-name: fadeOutClockLabel;
		animation-duration: 0.35s;
		animation-timing-function: ease-out;
		animation-fill-mode: forwards;
	}

	.sapMTPCSelectedItem .sapMTPCNumber,
	.sapMTPCMarker {
		animation-name: fadeOutSelectedItem;
		animation-delay: 0.1s;
		animation-duration: 0.15s;
		animation-timing-function: ease-out;
		animation-fill-mode: forwards;
	}

	.sapMTPCItems .sapMTPCMidDot {
		animation-name: fadeOutSelectedItem;
		animation-delay: 0.15s;
		animation-duration: 0.2s;
		animation-timing-function: ease-out;
		animation-fill-mode: forwards;
	}
}

.sapMTPCFadeIn:not(.sapMTPCSkipAnimation) {

	.sapMTPCItems .sapMTPCNumber {
		animation-name: fadeInClockItems;
		animation-delay: 0.15s;
		animation-duration: 0.2s;
		animation-timing-function: ease-in;
		animation-fill-mode: forwards;
	}

	.sapMTPCDial::before {
		animation-name: fadeInClockLabel;
		animation-duration: 0.35s;
		animation-timing-function: ease-in;
		animation-fill-mode: forwards;
	}

	.sapMTPCSelectedItem .sapMTPCNumber,
	.sapMTPCMarker {
		animation-name: fadeInSelectedItem;
		animation-delay: 0.1s;
		animation-duration: 0.15s;
		animation-timing-function: ease-in;
		animation-fill-mode: forwards;
	}

	.sapMTPCItems .sapMTPCMidDot {
		animation-name: fadeInSelectedItem;
		animation-delay: 0.15s;
		animation-duration: 0.2s;
		animation-timing-function: ease-in;
		animation-fill-mode: forwards;
	}
}

/* @keyframes */

@keyframes fadeOutClockItems {

	from {
		opacity: 1;
		display: block;
	}

	to {
		opacity: 0;
		display: block;
	}
}

@keyframes fadeOutClockLabel {

	from {
		opacity: 1;
		display: flex;
		transform: translateY(0);
	}

	to {
		opacity: 0;
		display: flex;
		transform: translateY(-16px);
	}
}

@keyframes fadeOutSelectedItem {

	from {
		opacity: 1;
		display: block;
	}

	to {
		opacity: 0;
		display: block;
	}
}

@keyframes fadeInClockItems {

	from {
		opacity: 0;
		display: block;
	}

	to {
		opacity: 1;
		display: block;
	}
}

@keyframes fadeInClockLabel {

	from {
		opacity: 0;
		display: flex;
		transform: translateY(16px);
	}

	to {
		opacity: 1;
		display: flex;
		transform: translateY(0px);
	}
}

@keyframes fadeInSelectedItem {

	from {
		opacity: 0;
		display: block;
	}

	50% {
		opacity: 0.5;
		display: block;
	}

	to {
		opacity: 1;
		display: block;
	}
}
/* END "TimePickerClock.less" */

/* START "TimePickerClocks.less" */
/* ====================================== */
/* CSS for control sap.m/TimePickerClocks */
/* Base theme                             */
/* ====================================== */

.sapMTPClocksContainer {
	margin: 0 auto;
	box-sizing: border-box;
	width: 20rem;
	height: 23.75rem;
	padding: 1rem;
	text-align: center;
}

.sapMTPCButtons {
	display: flex;
	justify-content: center;
	align-items: center;
	padding-bottom: 1rem;
	width: 18rem;

	& .sapMBtn {
		width: 2.5625rem;
	}

	& .sapMTPNow {
		margin-left: 0.75rem;
	}

	& .sapMSegB {
		min-width: 5.375rem;
	}

	& > span {
		min-width: 0.5rem;
		padding: 0 0.125rem;
		text-align: center;
		font-family: var(--sapFontFamily);
		font-size: @sapMFontMediumSize;
		color: var(--sapTextColor);
	}
}

.sapMTPCClocks {
	display: block;
	text-align: center;
	width: 18rem;
	height: 18rem;
	touch-action: none;
}

.sapMTPClocksContainer > .sapMTPCButtons > div.sapMBtnBase.sapMBtn {
	box-sizing: border-box;
	display: inline-block;
}

.sapUiSizeCompact {

	.sapMTPClocksContainer {
		width: 18.5rem;
		height: 21.5rem;
	}

	.sapMTPCButtons {
		width: 17rem;

		& .sapMBtn {
			width: 2.3125rem;
		}
	}

	.sapMTPCClocks {
		width: 17rem;
		height: 17rem;
	}
}
/* END "TimePickerClocks.less" */

/* START "TimePickerSliders.less" */
/* ======================================= */
/* CSS for control sap.m/TimePickerSliders */
/* Base Styles                             */
/* ======================================= */

.TPSliderItemHidden {
	display: none;
}

.sapMTimePickerContainer {
	width: 100%;
	height: 100%;
	min-width: 20rem;
	box-sizing: border-box;
	padding: 1rem;
	text-align: center;
	position: relative;

	&::before {
		content: '';
		display: inline-block;
		height: 100%;
		vertical-align: middle;
	}

	.sapMTimePickerContainerLabel {
		display: none;
		box-sizing: border-box;
	}

	.sapMTPDisabled {
		pointer-events: none;
	}

	.sapMTPColumn {
		display: inline-block;
		overflow: hidden;
		height: 3rem;
		vertical-align: middle;
		text-align: center;
		box-sizing: border-box;
		cursor: pointer;
		margin-top: 2rem;

		.sapMTimePickerLabel {
			display: none;
			box-sizing: border-box;
			height: 2rem;
			line-height: 2rem;
			width: 4.5rem;
			text-align: center;
		}

		.sapMTimePickerItemArrows {
			display: none;
			box-sizing: border-box;
		}

		.sapMTimePickerSlider {
			overflow: hidden;
			height: 100%;
			box-sizing: border-box;

			.sapMTimePickerItem {
				height: 3rem;
				line-height: 3rem;
				width: 4.5rem;
				box-sizing: border-box;
			}

			.sapMTPPickerSelectionFrame {
				width: 4.5rem;
				height: 3rem;
				position: absolute;
				box-sizing: border-box;
				display: none;
			}

			& > ul {
				list-style: none;
				margin: 0;
				padding: 0;
			}
		}

		&.sapMTPSliderExpanded {
			height: 100%;
			cursor: default;
			margin: 0;

			.sapMTimePickerLabel {
				display: block;
			}

			.sapMTimePickerItemArrows {
				display: none;
			}

			.sapMTimePickerSlider {
				height: calc(100% - 2rem);
				max-height: 100%; /* because of the SliderValuesN classes below */

				.sapMTPPickerSelectionFrame {
					display: block;
				}
			}

			.sapMTimePickerSlider.SliderValues3 {
				height: 9rem;
			}

			.sapMTimePickerSlider.SliderValues4 {
				height: 12rem;
			}

			.sapMTimePickerSlider.SliderValues5 {
				height: 15rem;
			}

			.sapMTimePickerSlider.SliderValues6 {
				height: 18rem;
			}

			.sapMTimePickerSlider.SliderValues7 {
				height: 21rem;
			}

			.sapMTimePickerSlider.SliderValues8 {
				height: 24rem;
			}

			.sapMTimePickerSlider.SliderValues9 {
				height: 27rem;
			}

			.sapMTimePickerSlider.SliderValues10 {
				height: 30rem;
			}

			.sapMTimePickerSlider.SliderValues11 {
				height: 33rem;
			}

			.sapMTimePickerSlider.SliderValues12 {
				height: 36rem;
			}
		}
	}

	.sapMTimePickerContainerLabel {
		font-size: var(--sapFontHeader5Size);
		color: @sapUiCalendarColorToday;
	}

	.sapMTPColumn {

		.sapMTimePickerLabel {
			font-size: var(--sapFontSmallSize);
			color: var(--sapContent_LabelColor);
			text-align: center;
			vertical-align: middle;
		}

		.sapMTimePickerSlider {

			.sapMTimePickerItem {
				background: var(--sapSelectedColor);
				border: 1px solid var(--sapList_BorderColor);
				font-size: @sapMFontMediumSize;
				color: var(--sapContent_ContrastTextColor);
				text-align: center;

				&:hover {
					background: var(--sapHighlightColor);
					color: var(--sapContent_ContrastTextColor);
				}

				&:active {
					background: var(--sapHighlightColor);
					color: var(--sapContent_ContrastTextColor);
				}

				&:focus {
					outline: 1px dotted var(--sapContent_ContrastFocusColor);
					outline-offset: -3px;
				}
			}
		}

		&.sapMTPSliderExpanded {

			.sapMTimePickerSlider {

				.sapMTimePickerItem {
					background: var(--sapList_Background);
					color: var(--sapTextColor);
					border: 1px solid var(--sapList_BorderColor);

					&:hover {
						background: var(--sapHighlightColor);
					}

					&:active {
						background: var(--sapHighlightColor);
						color: var(--sapContent_ContrastTextColor);
					}

					&:focus {
						outline: 1px dotted var(--sapContent_FocusColor);
						outline-offset: -3px;
					}
				}

				.sapMTPPickerSelectionFrame {
					outline: 0.125rem solid var(--sapContent_ForegroundBorderColor);
					outline-offset: -0.313rem;

					&:hover {

						& + ul > li.sapMTimePickerItem.sapMTimePickerItemSelected {
							background: var(--sapHighlightColor);
						}
					}

					&:active {

						& + ul > li.sapMTimePickerItem.sapMTimePickerItemSelected {
							background: var(--sapSelectedColor);
							color: var(--sapContent_ContrastTextColor);
						}
					}
				}
			}
		}

		&:focus {
			outline: none;
		}
	}
}

.sapUiSizeCompact {

	.sapMTimePickerContainer {
		padding: 0.5rem;
		margin: auto;

		.sapMTPColumn {
			height: 2rem;

			.sapMTimePickerLabel {
				width: 4rem;
			}

			.sapMTimePickerSlider {

				.sapMTimePickerItem {
					height: 2rem;
					line-height: 2rem;
					width: 4rem;
					box-sizing: border-box;
				}

				.sapMTPPickerSelectionFrame {
					width: 4rem;
					height: 2rem;
				}
			}

			&.sapMTPSliderExpanded {
				height: 100%;
				cursor: default;
				margin: 0;

				.sapMTimePickerItemArrows {
					display: block;
				}

				.sapMTimePickerSlider {
					height: ~"calc(100% - 6rem)";
					max-height: ~"calc(100% - 6rem)";
				}

				.sapMTimePickerSlider.SliderValues3 {
					height: 6rem;
				}

				.sapMTimePickerSlider.SliderValues4 {
					height: 8rem;
				}

				.sapMTimePickerSlider.SliderValues5 {
					height: 10rem;
				}

				.sapMTimePickerSlider.SliderValues6 {
					height: 12rem;
				}

				.sapMTimePickerSlider.SliderValues7 {
					height: 14rem;
				}

				.sapMTimePickerSlider.SliderValues8 {
					height: 16rem;
				}

				.sapMTimePickerSlider.SliderValues9 {
					height: 18rem;
				}

				.sapMTimePickerSlider.SliderValues10 {
					height: 20rem;
				}

				.sapMTimePickerSlider.SliderValues11 {
					height: 22rem;
				}

				.sapMTimePickerSlider.SliderValues12 {
					height: 24rem;
				}
			}
		}
	}
}

.sap-phone {

	.sapMTimePickerContainer {
		max-height: 38rem; /* 12 * item height + label's height */
		height: 100%;
		padding: 1rem;
		box-sizing: border-box;
		margin: auto;
		text-align: center;

		&::before {
			display: none;
		}

		.sapMTimePickerContainerLabel {
			position: absolute;
			top: 0;
			left: 50%;
			transform: translate(-50%, 0);
			display: block;
			height: 2rem;
			line-height: 2rem;
			font-size: var(--sapFontHeader5Size);
			color: @sapUiCalendarColorToday;
		}

		.sapMTPColumn {
			display: inline-block;
			height: 3rem;
			text-align: center;
			overflow: hidden;
			cursor: pointer;
			vertical-align: middle;
			box-sizing: border-box;

			.sapMTimePickerLabel {
				height: 2rem;
				width: 4.5rem;
				display: none;
				font-size: var(--sapFontSmallSize);
				color: var(--sapContent_LabelColor);
				text-align: center;
				vertical-align: middle;
			}

			.sapMTimePickerItemArrows {
				display: none;
			}

			.sapMTimePickerSlider {
				height: 100%;
				/* for Android 4.1 - 4.3 calc is not supported, so use 100% instead */
				max-height: 100%;
				overflow: hidden;

				.sapMTimePickerItem {
					height: 3rem;
					line-height: 3rem;
					width: 4.5rem;
					box-sizing: border-box;
					background: var(--sapSelectedColor);
					border: 1px solid var(--sapList_BorderColor);
					font-size: @sapMFontMediumSize;
					color: var(--sapContent_ContrastTextColor);
					text-align: center;

					&:active {
						background: var(--sapHighlightColor);
						color: var(--sapContent_ContrastTextColor);
					}

					&:focus {
						outline: 1px dotted var(--sapContent_ContrastFocusColor);
						outline-offset: -3px;
					}
				}

				.sapMTPPickerSelectionFrame {
					width: 4.5rem;
					height: 3rem;
					position: absolute;
					display: none;
					box-sizing: border-box;
				}
			}

			&.sapMTPSliderExpanded {
				height: 100%;
				cursor: default;

				.sapMTimePickerLabel {
					display: block;
				}

				.sapMTimePickerItemArrows {
					display: none;
				}

				.sapMTimePickerSlider {
					height: calc(100% - 2rem);
					/* for Android 4.1 - 4.3 calc is not supported, so use 100% instead */
					max-height: 100%;
					overflow: hidden;

					.sapMTimePickerItem {
						height: 3rem;
						line-height: 3rem;
						width: 4.5rem;
						box-sizing: border-box;
						background: var(--sapList_Background);
						color: var(--sapTextColor);
						border: 1px solid var(--sapList_BorderColor);

						&:active {
							background: var(--sapHighlightColor);
							color: var(--sapContent_ContrastTextColor);
						}

						&:focus {
							outline: 1px dotted var(--sapContent_FocusColor);
							outline-offset: -3px;
						}
					}

					.sapMTPPickerSelectionFrame {
						display: block;
						outline: 0.125rem solid var(--sapContent_ForegroundBorderColor);
						outline-offset: -0.313rem;
					}
				}

				.sapMTimePickerSlider.SliderValues3 {
					height: 9rem;
				}

				.sapMTimePickerSlider.SliderValues4 {
					height: 12rem;
				}

				.sapMTimePickerSlider.SliderValues5 {
					height: 15rem;
				}

				.sapMTimePickerSlider.SliderValues6 {
					height: 18rem;
				}

				.sapMTimePickerSlider.SliderValues7 {
					height: 21rem;
				}

				.sapMTimePickerSlider.SliderValues8 {
					height: 24rem;
				}

				.sapMTimePickerSlider.SliderValues9 {
					height: 27rem;
				}

				.sapMTimePickerSlider.SliderValues10 {
					height: 30rem;
				}

				.sapMTimePickerSlider.SliderValues11 {
					height: 33rem;
				}

				.sapMTimePickerSlider.SliderValues12 {
					height: 36rem;
				}
			}

			&:focus {
				outline: none;
			}
		}
	}
}
/* END "TimePickerSliders.less" */

/* START "Title.less" */
/* ============================ */
/* CSS for control sap.m/Title  */
/* Base theme                   */
/* ============================ */

.sapMTitle {
	position: relative;
	display: inline-block;
	margin: 0;
	font-weight: @sapUiFontHeaderWeight;
	color: var(--sapGroup_TitleTextColor);
	font-family: var(--sapFontHeaderFamily);
	box-sizing: border-box;
	overflow: hidden;
	text-overflow: ellipsis;
	cursor: text;
	text-shadow: @sapUiShadowText;
}

.sapMTitle.sapMTitleWrap {
	white-space: pre-line;
	word-wrap: break-word;
}

.sapMTitle.sapMTitleNoWrap {
	white-space: nowrap;
}

.sapMTitleMaxWidth {
	max-width: 100%;
}

/* Level Styling */

/* Level Auto*/
.sapMTitleStyleAuto {
	font-size: var(--sapFontHeader5Size);
}

.sapMTB > .sapMTitleStyleAuto {
	font-size: var(--sapGroup_Title_FontSize);
}

/* Level H1 */
.sapMTitleStyleH1 {
	font-size: var(--sapFontHeader1Size);
}

/* Level H2 */
.sapMTitleStyleH2 {
	font-size: var(--sapFontHeader2Size);
}

/* Level H3 */
.sapMTitleStyleH3 {
	font-size: var(--sapFontHeader3Size);
}

/* Level H4 */
.sapMTitleStyleH4 {
	font-size: var(--sapFontHeader4Size);
}

/* Level H5 */
.sapMTitleStyleH5 {
	font-size: var(--sapFontHeader5Size);
}

/* Level H6 */
.sapMTitleStyleH6 {
	font-size: var(--sapFontHeader6Size);
}

.sapMIBar-CTX .sapMTitle {
	color: var(--sapPageHeader_TextColor);
	text-shadow: @sapUiShadowText;
}

.sapMTB-Transparent-CTX .sapMTitle {
	text-shadow: none;
}

.sapMIBar-CTX.sapMFooter-CTX .sapMTitle {
	color: var(--sapPageFooter_TextColor);
	text-shadow: @sapUiShadowText;
}

/* DynamicPageTitle Heading-CTX */
.sapFDynamicPageTitleHeading-CTX > .sapMTitle {
	margin-top: 0;
}

/* Link in Title */
.sapMTitle .sapMLnk {
	font-size: inherit;
	vertical-align: middle;
}
/* END "Title.less" */

/* START "ToggleButton.less" */
/* =================================== */
/* CSS for control sap.m/ToggleButton  */
/* Base theme                          */
/* =================================== */

/* Common styles */

/* toggled without context */
html.sap-desktop .sapMToggleBtnPressed,
.sapMToggleBtnPressed:not(.sapMBtnDisabled),
/* Bar context */
html.sap-desktop .sapMIBar-CTX .sapMToggleBtnPressed,
.sapMIBar-CTX .sapMToggleBtnPressed:not(.sapMBtnDisabled),
/* Toolbar transparent context */
html.sap-desktop .sapMTB-Transparent-CTX .sapMBtnInner.sapMToggleBtnPressed,
.sapMTB-Transparent-CTX .sapMBtnInner.sapMToggleBtnPressed:not(.sapMBtnDisabled) {
	background-image: none;
	background-color: var(--sapButton_Selected_Background);
	border-color: var(--sapButton_Selected_BorderColor);
	color: var(--sapButton_Selected_TextColor);
	text-shadow: none;

	& .sapUiIcon {
		color: var(--sapButton_Selected_TextColor);
	}

	&.sapMBtnReject {
		background-color: var(--sapButton_Reject_Selected_Background);
		border-color: var(--sapButton_Reject_Selected_BorderColor);
		color: var(--sapButton_Reject_Selected_TextColor);

		& .sapUiIcon {
			color: var(--sapButton_Reject_Selected_TextColor);
		}
	}

	&.sapMBtnAccept {
		background-color: var(--sapButton_Accept_Selected_Background);
		border-color: var(--sapButton_Accept_Selected_BorderColor);
		color: var(--sapButton_Accept_Selected_TextColor);

		& .sapUiIcon {
			color: var(--sapButton_Accept_Selected_TextColor);
		}
	}

	&.sapMBtnAttention {
		background-color: var(--sapButton_Attention_Selected_Background);
		border-color: var(--sapButton_Attention_Selected_BorderColor);
		color: var(--sapButton_Attention_Selected_TextColor);

		& .sapUiIcon {
			color: var(--sapButton_Attention_Selected_TextColor);
		}
	}
}

/*hover and down on toggled*/
.sapMBtn:hover .sapMBtnHoverable.sapMToggleBtnPressed, /* in Firefox the button receives the hover state, not it's inner child */
.sapMBtn .sapMBtnHoverable.sapMToggleBtnPressed:hover,
/* Toolbar transparent context */
.sapMTB-Transparent-CTX .sapMBtnHoverable.sapMToggleBtnPressed:hover {
	background-image: none;
	background-color: @sapUiToggleButtonPressedHoverBackground;
}

/* toggled - icon has to be adapted */
span.sapMBtnInner.sapMToggleBtnPressed .sapMBtnIcon,
span.sapMBtnInner.sapMToggleBtnPressed.sapMBtnTransparent > .sapMBtnIcon,
html.sap-desktop .sapMToggleBtnPressed.sapMBtnTransparent > .sapMBtnIcon {
	color: @sapUiToggleButtonPressedTextColor;
}

/* Focus of toggled button */
.sapMBtn:focus > .sapMBtnInner.sapMToggleBtnPressed.sapMFocusable {
	outline-color: var(--sapContent_ContrastFocusColor);
}

/* hover and down on toggled button part 1 */
.sapMBtn:hover > .sapMBtnHoverable.sapMToggleBtnPressed:not(.sapMBtnActive),
.sapMBtn .sapMBtnHoverable.sapMToggleBtnPressed:hover {
	background-color: var(--sapButton_Selected_Hover_Background);
	border-color: var(--sapButton_Selected_Hover_BorderColor);
	color: var(--sapButton_Selected_TextColor);
	text-shadow: none;

	&.sapMBtnReject {
		background-color: darken(@sapUiButtonRejectActiveBackground, 5);
		border-color: var(--sapButton_Reject_Active_BorderColor);
		color: contrast(lighten(@sapUiButtonRejectActiveBackground, 5), @sapUiBaseText, @sapUiContentContrastTextColor, @sapUiContentContrastTextThreshold);
	}

	&.sapMBtnAccept {
		background-color: darken(@sapUiButtonAcceptActiveBackground, 5);
		border-color: var(--sapButton_Accept_Active_BorderColor);
		color: contrast(lighten(@sapUiButtonAcceptActiveBackground, 5), @sapUiBaseText, @sapUiContentContrastTextColor, @sapUiContentContrastTextThreshold);
	}

	&.sapMBtnAttention {
		background-color: darken(@sapUiButtonAttentionActiveBackground, 5);
		border-color: var(--sapButton_Attention_Active_BorderColor);
		color: contrast(lighten(@sapUiButtonAttentionActiveBackground, 5), @sapUiBaseText, @sapUiContentContrastTextColor, @sapUiContentContrastTextThreshold);
	}
}
/* END "ToggleButton.less" */

/* START "Token.less" */
/* ============================ */
/* CSS for control sap.m/Token  */
/* Base theme                   */
/* ============================ */

@_sap_m_Token_TokenPadding: 0.5rem;
@_sap_m_Token_TokenCompactPadding: 0.25rem;
@_sap_m_Token_LeftPadding: 0.375rem;
@_sap_m_Token_IconWidth: @sapMFontMediumSize + (2 * @_sap_m_Token_TokenPadding) - @_sap_m_Token_LeftPadding;
@_sap_m_Token_CompactIconWidth: @sapMFontSmallSize + @_sap_m_Token_TokenCompactPadding;
@_sap_m_Token_TokenHeight: 1.625rem;
@_sap_m_Token_TokenCompactHeight: 1.25rem;

.sapMToken {
	display: inline-block;
	position: relative;
	overflow: hidden;
	box-sizing: border-box;
	font-size: @sapMFontMediumSize;
	padding-left: @_sap_m_Token_LeftPadding;
	height: @_sap_m_Token_TokenHeight;
	background: @sapUiButtonTokenBackground;
	border: @sapUiButtonBorderWidth solid @sapUiButtonTokenBorderColor;
	border-radius: @sapUiButtonBorderCornerRadius;
	color: @sapUiButtonTextColor;
}

.sapMToken.sapMHiddenToken {
	display: none;
}

.sapMToken.sapMTokenReadOnly {
	padding: 0 0.375rem;
	color: @sapUiContentForegroundTextColor;
}

html.sap-desktop .sapMToken:not(.sapMTokenReadOnly):hover {
	background: @sapUiButtonHoverBackground;
	border: @sapUiButtonBorderWidth solid @sapUiButtonHoverBorderColor;
}

.sapMTokenText {
	display: inline-block;
	overflow: hidden;
	white-space: nowrap;

	// at least for Microsoft Surface Tablet we experienced problems when not setting these properties - touches on the Token seemed to be propageted to the Span inside the Token-Div and thereby (part of) the text contained in the Span would appear selected
	-webkit-touch-callout: none;
	-webkit-user-select: none;
	-moz-user-select: none;
	user-select: none;

	pointer-events: none;

	line-height: 1.5rem;
	max-width: calc(~"100% - " @_sap_m_Token_IconWidth);
	box-sizing: border-box;
}

.sapMToken.sapMTokenTruncated .sapMTokenText {
	text-overflow: ellipsis;
}

.sapUiSizeCompact .sapMTokenText {
	max-width: calc(~"100%  - " @_sap_m_Token_CompactIconWidth);
	font-size: @sapMFontMediumSize;
}

.sapMTokenReadOnly .sapMTokenText,
.sapMTokenizerReadonly .sapMTokenText {
	max-width: 100%;
}

.sapMTokenIcon {
	vertical-align: top;
	display: inline-block;
	line-height: 1.625rem;
	padding: 0 0.5rem;
	color: @sapUiContentIconColor;
	font-size: 0.75rem;
	box-sizing: border-box;
	touch-action: none;
	right: -@sapUiButtonBorderWidth;
}

.sapUiSizeCompact .sapMToken {
	font-size: @sapMFontSmallSize;
	padding-left: @_sap_m_Token_TokenCompactPadding;
	height: @_sap_m_Token_TokenCompactHeight;
}

.sapUiSizeCompact .sapMToken.sapMTokenReadOnly {
	padding: 0 0.25rem;
}

.sapUiSizeCompact .sapMTokenText {
	line-height: 1.125rem;
}

.sapUiSizeCompact .sapMToken .sapMTokenIcon {
	font-size: @sapMFontSmallSize;
	padding: 0 0.25rem;
	line-height: 1.1875rem;
}

html.sap-desktop .sapMToken.sapMTokenSelected:not(.sapMTokenReadOnly):hover {
	background: @sapUiToggleButtonPressedHoverBackground;
	border: 1px solid @sapUiToggleButtonPressedHoverBorderColor;
}

.sapMTokenSelected,
.sapMToken.sapMTokenReadOnly.sapMTokenSelected {
	color: @sapUiButtonSelectedTextColor;
	background: @sapUiButtonSelectedBackground;
	border: @sapUiButtonBorderWidth solid @sapUiButtonSelectedBorderColor;
}

.sapMTokenSelected .sapMTokenIcon,
.sapMTokenSelected .sapMTokenText {
	color: @sapUiButtonSelectedTextColor;
}

.sapUiSizeCompact .sapMListTbl .sapMToken .sapUiIcon:not(.sapMLIBImgNav) {
	font-size: 0.75rem;
}
/* END "Token.less" */

/* START "Tokenizer.less" */
/* ================================ */
/* CSS for control sap.m/Tokenizer  */
/* Base theme                       */
/* ================================ */
@_sap_m_Tokenizer_OutlineOffset: 0.1875rem;
@_sap_m_Tokenizer_Token_MarginRight: 0.375rem;
@_sap_m_Tokenizer_FontSizeRatioCozy: (@sapMFontMediumSize / @sapFontLargeSize);
@_sap_m_Tokenizer_FontSizeRatioCompact: 1;

.sapMTokenizer {
	padding-left: 0.375rem;
	position: relative;
	display: inline-block;
	white-space: nowrap;
	overflow: hidden;
	box-sizing: border-box;
	vertical-align: inherit;
}

.sapMTokenizer .sapMToken {
	margin-top: 0.25rem;
	margin-bottom: 0.25rem;
	margin-right: @_sap_m_Tokenizer_Token_MarginRight;
	line-height: normal;
	vertical-align: middle;
}

.sapMTokenizer.sapMTokenizerReadonly {
	.sapMToken {
		padding: 0 0.375rem;
		}

	.sapMTokenIcon {
		display: none;
	}

	.sapMTokenizerIndicator {
		color: @sapUiLink;

		&:hover{
			color: @sapUiLinkHover;
		}
	}
}

.sapMTokenizer.sapMTokenizerDisabled {
	opacity: var(--sapContent_DisabledOpacity);
	pointer-events: none;

	.sapMToken.sapMTokenSelected::before {
		background-color: none;
		height: 0;
	}
}

.sapMTokenizerClip {
	position: absolute;
	top: 0;
	left: 0;
	width: 1px;
	height: 1px;
	display: block;
	font-size: 1px;
	z-index: -1;
	color: transparent;
	background: transparent;
	overflow: hidden;
	border: none;
	padding: 0;
	resize: none;
	outline: none;
	-webkit-user-select: text;
	user-select: text;
}

.sapMTokenizerIndicator {
	display: inline-block;
	cursor: pointer;
	font-size: @sapMFontMediumSize;
	line-height: 2rem;
	overflow: hidden;
	vertical-align: middle;

	&:hover {
		text-decoration: underline;
	}
	&:active {
		text-decoration: none;
	}
}

.sapUiSizeCompact .sapMTokenizer {
	padding-left: 0.25rem;

	.sapMToken {
		height: 1.25rem;
		margin-top: 0.125rem;
		margin-bottom: 0.125rem;
		margin-right: 0.25rem;
		max-width: calc(~"100% - 0.25rem");
	}
}

.sapUiSizeCompact .sapMTokenizerIndicator {
	line-height: 1.5rem;
}

.sapMTokenizerScrollContainer {
	display: inline-block;
	-moz-user-select: none;
	-webkit-user-select: none;
	height: 100%;
}

.sapMTokenizer.sapMTokenizerOneLongToken .sapMTokenizerScrollContainer{
	width: 100%;
	overflow: hidden;

	.sapMToken {
		width: calc(~"100% - " @_sap_m_Tokenizer_Token_MarginRight);
	}
}

.sapMTokenizerScrollContainerNoVisibleTokens {
	display: none;
}

.sapMTokenizerTokensPopup {
	.sapMListUl > .sapMLIB.sapMSLIWrapping {
		min-height: 2.5rem;
	}

	.sapMSLIWrapping > .sapMLIBContent > .sapMSLIDiv {
		padding: 0.25rem 0;
	}
}

.sapUiSizeCompact {
	.sapMTokenizerTokensPopup,
	&.sapMTokenizerTokensPopup {
		.sapMListUl > .sapMLIB.sapMSLIWrapping {
			min-height: 2rem;
		}

		.sapMSLIWrapping > .sapMLIBContent > .sapMSLIDiv {
			padding: 0.25rem 0;
		}
	}
}
/* END "Tokenizer.less" */

/* START "Toolbar.less" */
/* ============================== */
/* CSS for control sap.m/Toolbar  */
/* Base theme                     */
/* ============================== */

@_sap_m_Toolbar_ShrinkItem_MinWidth: 2.5rem;
@_sap_m_Toolbar_Height: 2rem;

//we want the first visible element on the row to have specific padding-left value
.sapMIBar.sapMTB .sapMBarChild {
	margin-left: 0.25rem;
}

.sapMIBar.sapMTB .sapMBarChild ~ .sapMBarChild {
	margin: 0 0 0 0.5rem;
}

.sapMIBar.sapMTB .sapMBarChild.sapMBarChildFirstChild {
	margin-left: 0;
}

.sapMIBar.sapMTB.sapMTBHeader-CTX {
	padding-left: 1rem;
}

.sapMTB {
	white-space: nowrap;
	overflow: hidden;
	-webkit-align-items: center;
	-webkit-box-align: center;
	align-items: center;
	box-sizing: border-box;
	-webkit-text-size-adjust: none;
	-moz-text-size-adjust: none;
	height: 3rem;
	padding-left: 0.25rem;
	padding-right: 0.25rem;
	background: @sapUiToolbarBackground;
}

.sapMTB.sapMTB-Transparent-CTX {
	background: transparent;
}

.sapMIBar.sapMTB .sapMTBHiddenElement {
	position: absolute;
	visibility: hidden;
}

.sapMTBFocused {
	outline: 0;
}

.sapMTBStandard {
	border-bottom: solid 0.0625rem @sapUiGroupTitleBorderColor;
}

.sapMTBClear {
	border-bottom: none;
}

/* Info Toolbar Context */
.sapMIBar.sapMTB.sapMTB-Info-CTX {
	padding: 0 1rem;
}

.sapMIBar.sapMTB.sapMTB-Info-CTX > .sapUiIcon:first-child {
	padding: 1px;
}

/* Background and text colors for different states */
.sapMTB-Info-CTX > .sapUiIcon:focus {
	outline: @sapUiContentFocusWidth @sapUiContentFocusStyle @sapUiContentFocusColor;
	outline-offset: 0.0625rem;
}

.sapMTBActive,
.sapMTBActive > * {
	cursor: pointer;
}

.sapMTBShrinkItem:not(.sapMSF) {
	min-width: @_sap_m_Toolbar_ShrinkItem_MinWidth; /* default min-width value for the shrinkable items */
}

/* NewFlex */
.sapMTBNewFlex {
	display: -webkit-flex;
	display: flex;
}

.sapMTBNewFlex > * {
	-webkit-flex-shrink: 0;
	flex-shrink: 0;
}

.sapMTBNewFlex > .sapMTBShrinkItem {
	-webkit-flex-shrink: 1;
	flex-shrink: 1;
}

/*** Toolbar inside of Form as Form- and Container- header ***/
.sapUiFormContainerToolbar > .sapMTB.sapMIBar {
	border-bottom: none;
	padding: 0;
}

.sapUiFormCLContainer .sapMIBar.sapMTB .sapMBarChild:first-child,
.sapUiRespGrid .sapMIBar .sapMBarChild:first-child {
	margin-left: 0.25rem;
}

/* Solid Toolbar Context */
.sapMTB-Solid-CTX.sapMTB {
	background: @sapUiListHeaderBackground;
}

/* Info Toolbar Context */
.sapMTB-Info-CTX.sapMTB {
	background: @sapUiInfobarBackground;
	color: contrast(@sapUiInfobarBackground, @sapUiListActiveTextColor, @sapUiListTextColor, @sapUiContentContrastTextThreshold);
	height: @_sap_m_Toolbar_Height;

	.sapMText {
		white-space: nowrap;
		overflow: hidden;
		text-overflow: ellipsis;
	}

	> .sapMTitle {
		font-size: @sapMFontHeader6Size;
		text-shadow: none;
	}

	&.sapMTBActive {
		&:hover {
			background: @sapUiInfobarHoverBackground;
		}

		&:active {
			background: @sapUiInfobarActiveBackground;
		}
	}
}

/********** COMPACT DESIGN ************/

/* toolbar has 2rem height in compact mode */
.sapUiSizeCompact .sapMTB {
	height: @_sap_m_Toolbar_Height;
}

/* class used to get a height of 3rem in compact mode */
.sapUiSizeCompact .sapMTB.sapMTBHeader-CTX {
	height: @sapUiElementLineHeight;
}

/* default min-width value for the shrinkable items */
.sapUiSizeCompact .sapMTBShrinkItem:not(.sapMSF),
.sapUiSizeCondensed .sapUiTableCell .sapMTBShrinkItem{
	min-width: 2rem;
}

.sapFDynamicPageTitleContent-CTX .sapMTB.sapMTBStandard {
	border-bottom: none;
}
/* END "Toolbar.less" */

/* START "ToolbarSeparator.less" */
/* ======================================= */
/* CSS for control sap.m/ToolbarSeparator  */
/* Base theme                              */
/* ======================================= */

.sapMTBSeparator {
	width: 0.0625rem;
	height: 2rem;
	background: @sapUiToolbarSeparatorColor;

	&.sapMTBSeparatorOverflowToolbar {
		height: 0.0625rem;
		margin: 0.25rem 0.5rem;
		width: auto;
	}
}

.sapUiSizeCompact .sapMTBSeparator {
	height: 1.5rem;

	&.sapMTBSeparatorOverflowToolbar {
		height: 0.0625rem;
		margin: 0.1875rem 0.375rem;
	}
}
/* END "ToolbarSeparator.less" */

/* START "ToolbarSpacer.less" */
/* ==================================== */
/* CSS for control sap.m/ToolbarSpacer  */
/* Base theme                           */
/* ==================================== */

.sapMTBSpacer {
	/* should never have padding/margin/border */
	margin: 0 !important;
	padding: 0 !important;
	border: 0 !important;

	/* without height chrome ignores the element */
	height: 1px;
}

.sapMTBSpacerFlex {
	-webkit-flex-grow: 1;
	-webkit-box-flex: 1;
	flex: auto;
}
/* END "ToolbarSpacer.less" */

/* START "TreeItemBase.less" */
/* =================================== */
/* CSS for control sap.m/TreeItemBase  */
/* Base theme                          */
/* =================================== */

.sapMTreeItemBase {
	min-height: var(--sapElement_Height);
	font-family: var(--sapFontFamily);
	font-size: @sapMFontMediumSize;
	padding-left: 0;
	color: var(--sapList_TextColor);
	padding-top: 0.25rem;
	padding-bottom: 0.25rem;
}

.sapMTreeItemBaseExpander {
	width: var(--sapElement_LineHeight);
	height: var(--sapElement_Height);
	line-height: var(--sapElement_Height);
	font-size: 1rem;
	color: var(--sapContent_IconColor);
	flex-shrink: 0;
}

.sapMTreeItemBaseLeaf > .sapMTreeItemBaseExpander {
	opacity: 0;
	pointer-events: none;
}

.sapMLIBActive .sapMTreeItemBaseExpander {
	color: var(--sapList_Active_TextColor);
}

.sapMListHighlight .sapMTreeItemBaseExpander {
	margin-left: 0.375rem;
}

.sapMTreeItemBaseChildren {
	background-color: var(--sapList_AlternatingBackground);
	border-bottom-color: transparent;
}

.sapMTIBWrapTitle {
	white-space: nowrap;
	overflow: hidden;
	text-overflow: ellipsis;
}

.sapMTreeItemBase > .sapMLIBContent {
	display: flex;
	align-items: center;
}

.sapUiSizeCompact .sapMTreeItemBase {
	min-height: 1.5rem;
}

.sapUiSizeCompact .sapMTreeItemBaseWrapping {
	min-height: 1.5rem;
}

.sapUiSizeCompact .sapMTreeItemBaseExpander {
	font-size: 0.75rem;
	width: 2rem;
	height: 1.5rem;
	line-height: 1.5rem;
}
/* END "TreeItemBase.less" */

/* START "UploadCollection.less" */
/* ======================================= */
/* CSS for control sap.m/UploadCollection  */
/* Base theme                              */
/* ======================================= */

/* until the central focus concept is implemented, a workaround is built in sap.m.UploadCollection so that all focus frames are visible */
/* to remove the workaround the changes below are necessary: */
/* for .sapMLIB.sapMUCItem.sapMUCListSingleItem, .sapMLIB.sapMUCItem.sapMUCListFirstItem,.sapMLIB.sapMUCItem.sapMUCListLastItem, .sapMLIB.sapMUCItem.sapMUCListItem change padding-left from 0.9rem to 1rem    */
/* for .sapMUCFileName.sapMLnk delete margin-left:	0.1rem */
/* for .sapMUCItemIcon, .sapMUCItemImage change margin-right from 0.65rem to 0.75rem, delete margin-top: 0.1rem; margin-bottom:	0.1rem; margin-left: 0.1rem; */
/* for .sapMUCAttrContainer, .sapMUCStatusContainer delete margin-left: 0.1rem*/

/* List Header */
.sapMTB.sapMUCListHeader {
	width:				100%;
	height:				3rem;
	padding-left:		1rem;
	padding-right:		1rem;
	border-bottom-width: 0.0625rem;
	border-bottom-style: solid;
	white-space:		nowrap;
	overflow:			hidden;
	text-overflow:		ellipsis;
	vertical-align:		middle;
}

.sapMTB.sapMUCListHeader > .sapMBarChild:first-child {
	margin-left: 0;
}

/* List with borders*/
.sapMLIB.sapMUCItem.sapMUCListSingleItem,
.sapMLIB.sapMUCItem.sapMUCListFirstItem,
.sapMLIB.sapMUCItem.sapMUCListLastItem,
.sapMLIB.sapMUCItem.sapMUCListItem {
	padding-left:			0.9rem;       /* a workaround until the central focus concept is implemented( after that make it 1rem); 0.1rem gets the image/icon so that the left side of the keyboard frame is displayed */
	padding-top:			1rem;
	padding-bottom: 		1rem;
	border-bottom-width: 	0.0625rem;
	border-bottom-style: 	solid;
	padding-right:			1rem;
}

/* List containing no borders */
.sapMListShowSeparatorsNone {
	.sapMLIB.sapMUCListFirstItem,
	.sapMLIB.sapMUCListLastItem,
	.sapMLIB.sapMUCListItem {
		border-bottom-color: 	transparent; /* this is the central switch to have inner separators or not */
	}
}

/* List containing no borders */
.sapMListShowSeparatorsNone {
	.sapMLIB.sapMUCListLastItem,
	.sapMLIB.sapMUCListItem {
		padding-top: 0.5rem;
	}
}

/* List containing no borders */
.sapMListShowSeparatorsNone {
	.sapMLIB.sapMUCListFirstItem,
	.sapMLIB.sapMUCListItem {
		padding-bottom: 0.5rem;
	}
}

/* List with a single item containing no borders */
.sapMListShowSeparatorsNone .sapMLIB.sapMUCListSingleItem {
	padding-top: 1rem;
	padding-bottom: 1rem;
}

/* Item icon */
.sapMUCItemIconInactive,
.sapMUCItemIcon {
	font-size: 2.5rem;
}

.sapMImg.sapMUCItemImage{
	background-size:	cover;
}

.sapMUCItemIconInactive,
.sapMUCItemIcon,
.sapMUCItemImage {
	height:				3rem;
	width:				3rem;
	margin-right:		0.75rem; /* because of the keyboard frame it should be margin and not padding */
	margin-top:			0.1rem; /* is needed so that keyboard frame is visible; after the central focus concept is implemented delete the margin */
	margin-bottom:		0.1rem; /* is needed so that keyboard frame is visible; after the central focus concept is implemented delete the margin */
	margin-left:		0.1rem; /* is needed so that keyboard frame is visible; after the central focus concept is implemented delete the margin */
	flex:				none;
}

/*if no image is provided a standard placeholder is displayed */
.sapMUCItemPlaceholderInactive,
.sapMUCItemPlaceholder {
	line-height: 3rem;
}

/* Text: File name */
.sapMUCFileName.sapMLnk {
	font-family:		@sapUiFontFamily;
	font-weight:		normal;
	padding-top:		0.25rem; /* Padding top instead of padding bottom due to in-line placement with Objectmarker */
	overflow:			hidden;
	text-overflow:		ellipsis;
	opacity:			100;
	margin-left:		0.1rem;
}

.sapMUCObjectMarker {
	margin-left: 0.5rem;
}

.sapMUCObjectMarker.sapMObjectMarkerText:not(.sapMObjectMarkerIcon) {
	padding-top: 0.25rem;
}

.sapMUCObjectMarker.sapMObjectMarkerText,
.sapMUCObjectMarker.sapMObjectMarkerIcon.sapMObjectMarkerText .sapUiIcon {
	vertical-align: bottom;
}

.sapMUCObjectMarkerContainer {
	display: inline-block;
	position: absolute;
}

/*positioning of the Progress label */
.sapMLabel.sapMUCProgress {
	display: block;
	padding-top: 0.5rem;
}

/* Busy Indicator */
.sapMBusyIndicator.sapMUCloadingIcon {
	padding-top:		1.25rem;
	padding-bottom:		1.25rem;
	padding-right:		1.3rem;
	padding-left:		0.5rem;
	margin-top:			0rem;
}

.sapMBusyIndicator.sapMUCloadingIcon .sapMBusySvg {
	height:				2rem;
	width:				2rem;
}

/* No data indicator */
.sapMUCNoDataPage {
	padding: 1rem;
	text-align: center;
	background: @sapUiGroupContentBackground;
	display: block;

	.sapUiIcon {
		color: fade(@sapUiContentNonInteractiveIconColor, 50);
	}

	.sapMUCNoDataText {
		padding: 1rem 0rem;
		line-height: 2rem;
		font-size: @sapMFontHeader2Size;
		color: @sapUiGroupTitleTextColor;
	}

	.sapMUCNoDataDescription {
		padding: 0rem 0rem 2rem 0rem;
		font-size: @sapMFontHeader5Size;
		color: @sapUiContentLabelColor;
	}
}

/* Drag && Drop */
.sapMUC {
	position: relative;

	.sapMUCDragDropOverlay:not(.sapMUCDragDropOverlayHide) ~ .sapMList {
		.sapMUCNoDataPage {
			visibility: hidden;
		}
	}

	.sapMUCDragDropOverlay {
		position: absolute;
		top: 3rem;
		right: 0rem;
		left: 0rem;
		bottom: 0rem;
		background-color: fade(@sapUiDragAndDropBackground, 80);
		z-index: 1;

		&.sapMUCDragDropOverlayHide {
			display: none;
		}

		&:after {
			display: block;
			content: "";
			position: absolute;
			top: 0rem;
			left: 0rem;
			bottom: 0rem;
			right: 0rem;
			margin: 0.5rem;
			border: 0.125rem dashed @sapUiDragAndDropBorderColor;
		}

		.sapMUCDragDropIndicator {
			position: absolute;
			top: 50%;
			left: 50%;
			transform: translate(-50%, -50%);

			.sapUiIcon {
				position: relative;
				display: block;
				color: @sapUiDragAndDropColor;
				padding-bottom: 1rem;
			}

			.sapMText {
				position: relative;
				font-size: @sapMFontHeader4Size;
				color: @sapUiDragAndDropColor;
			}
		}

		&.sapMUCDropIndicator:after {
			background-color: @sapUiDragAndDropActiveBackground;
			border: 0.125rem solid @sapUiDragAndDropActiveBorderColor;
		}

		&.sapMUCDropIndicator {
			.sapMUCDragDropIndicator {
				.sapMText {
					color: @sapUiDragAndDropActiveColor;
				}
				.sapUiIcon {
					color: @sapUiDragAndDropActiveColor;
				}
			}
		}
	}
}

/* ************************************************************************************* */
/* local fixes: should be done by used simple controls                                   */
/* ************************************************************************************* */
.sapMTB.sapMUCListHeader .sapMBtnPaddingLeft {
	padding-left:		0rem;
}

.sapMUC .sapUiIcon:focus {
	outline-style: @sapUiContentFocusStyle;
	outline-width: @sapUiContentFocusWidth;
}
/* ************************************************************************************* */


.sapMUCListFirstItem > .sapMLIBContent,
.sapMUCListLastItem > .sapMLIBContent,
.sapMUCListSingleItem > .sapMLIBContent,
.sapMUCListItem > .sapMLIBContent {
	display:		flex;
	padding:		0;
}

/*Container for a textContainer and a button container */
@media(min-width: 30rem) {
	.sapMUCTextButtonContainer {
		display:		flex;
		width: 			100%;
		min-width: 		0; /* to avoid FF bug with flex*/
	}
}

.sapMUCTextButtonContainer {
	min-width: 		0; /* to avoid FF bug with truncation */
	width: 			100%; /* it is necessary for the file name truncation in IE10 */
}

/*text container holding the file name, a container for attributes and a container for statuses */
@media(min-width: 30rem) {
	.sapMUCTextContainer {
		flex:	 			10 10 auto;
		padding-top: 		0.125rem;
		padding-bottom: 	0.125rem;
		display: 			block;
	}
}

.sapMUCTextContainer.sapMUCEditMode {
	padding-top: 0rem;
}

.sapMUCTextContainer {
	width: 100%;
	white-space: nowrap; /* necessary for text ellipsis of the file name in FF */
	overflow: hidden;
}

/*display the attributes inline */
.sapMUCAttrContainer .sapMObjectAttributeDiv {
	display: inline;
}

/* display the statuses inline */
.sapMUCStatusContainer .sapMObjStatus {
	display: inline;
}

/* overwrite display: -webkit-box so that the text does not get truncated */
.sapMUCAttr .sapMTextMaxLine {
	display: initial;
	overflow: visible;
}

.sapMUCAttrContainer, .sapMUCStatusContainer {
	padding-top: 0.375rem;
	font-weight: normal;
	line-height: normal;
	white-space: pre-line;
	margin-left: 0.1rem;
	margin-right: 0.1rem;
}

@media(min-width: 30rem) {
	/* !important is necessary here for overriding inline style of sap.m.input */
	.sapMUCEditBox >.sapMInputBaseInner {
		width: 60% !important;
	}
}

.sapMInputBase.sapMUCEditBox {
	margin-bottom: 0.1875rem;
}

@media(min-width: 30rem) {
	.sapMUCButtonContainer {
		white-space: nowrap; /* so that buttons are displayed in a row */
		padding-left: 1rem;
		padding-top: 0rem;
		align-self: center; /* for buttons alignment at the center */
		flex: none;
	}
}

@media(max-width: 30rem) {
	.sapMUCButtonContainer {
		padding-top: 0.75rem;
	}
}

/* this padding is only needed if both buttons are displayed */
/* for the width less than 30 rem */
.sapMBtn.sapMUCFirstButton {
	padding-right: 0.5rem;
}

/* this padding is only needed if both buttons are displayed */
/* for the width >=30rem */
@media(min-width: 30rem) {
	.sapMBtn.sapMUCFirstButton {
		padding-right: 0.125rem;
	}
}

/* for compact form: for the width less than 30rem */
.sapUiSizeCompact .sapMBtn.sapMUCFirstButton {
	padding-right: 0.5rem;
}

/* for compact form: this padding is only needed if both buttons are displayed */
@media(min-width: 30rem) {
	.sapUiSizeCompact .sapMBtn.sapMUCFirstButton {
		padding-right: 0.125rem;
	}
}

.sapMUCButtonContainer > .sapMBtn {
	padding-top: 0rem; /* the padding of the sap.m.Button should be removed */
	padding-bottom: 0rem; /* the padding of the sap.m.Button should be removed */
	height:			2.5rem;
}


/*compact form */
.sapUiSizeCompact .sapMUCButtonContainer > .sapMBtn {
	height: 1.625rem;
}

.sapMUCSeparator {
	display:inline-block;
}

.sapMUCList .sapMLIB {
	align-items: center;
}

.sapMUCItem.sapMCLI>.sapMLIBSelectM {
	margin-top: -0.8125rem;
	margin-left: -0.875rem;
	margin-right: -0.125rem;
}

.sapUiSizeCompact .sapMUCItem>.sapMLIBSelectM {
	margin-left: -0.9rem;
	padding-top: 0.3rem;
	margin-right: -0.1rem;
}

.sapUiSizeCompact .sapMUCItem .sapMLIBSelectM {
	padding-left: 0.3rem;
	width: 2.75rem;
}

.sapUiSizeCozy .sapMUCItem .sapMLIBSelectM {
	padding-left: 0;
	width: 2.75rem;
}

/* Item Icon */
.sapMUCItemIcon {
	color: @sapUiContentIconColor;
}
/* Item Icon Inactive */
.sapMUCItemIconInactive {
	color: @sapUiContentNonInteractiveIconColor;
}

/*if no image is provided a standard placeholder is displayed */
.sapMUCItemPlaceholder {
	color: @sapUiContentIconColor;
}

.sapMUCItemPlaceholderInactive {
	color: @sapUiContentNonInteractiveIconColor;
}

/*if no link is provided the file name is displayed as an disabled link */
.sapMLnkDsbl.sapMUCFileName {
	color: @sapUiListTextColor;
	text-decoration: none;
}

/* disabled state */
.sapMBtnDisabled.sapMUCDeleteBtn > .sapMBtnInner > .sapMBtnIcon {
	color: @sapUiContentIconColor;
}

.sapMUCAttrContainer {
	color: @sapUiContentLabelColor;
}

/* Text: Filename */
.sapMUCFileName.sapMLnk {
	font-size: @sapMFontLargeSize; /* it cannot be moved to base as the LESS constant is not known there */
}

.sapMUCAttrContainer, .sapMUCStatusContainer {
	font-size: @sapMFontMediumSize; /* it cannot be moved to base as the LESS constant is not known there */
}

/*for width > 30rem the buttons should be displayed as buttons of type transparent */
@media(min-width: 30rem) {
	.sapMUCButtonContainer >.sapMBtn >.sapMBtnInner {
		background-image: none;
		background-color: @sapUiButtonLiteBackground;
		border-color: @sapUiButtonLiteBorderColor;
		text-shadow: none;
	}
}

/* an edit and delete button in a hover state; it should look like a transparent button in a hover state */
@media(min-width: 30rem) {
	.sapMUCEditBtn:hover >.sapMBtnHoverable, .sapMUCDeleteBtn:hover >.sapMBtnHoverable {
		background-color: @sapUiButtonLiteHoverBackground;
		border-color: @sapUiButtonLiteHoverBorderColor;
	}
}

/*an edit and delete button in a pressed state; it should look like a transparent button in a pressed state */
@media(min-width: 30rem) {
	:not(.sapMBtnDisabled).sapMUCEditBtn > .sapMBtnActive,
	:not(.sapMBtnDisabled).sapMUCEditBtn:hover > .sapMBtnActive,
	:not(.sapMBtnDisabled).sapMUCDeleteBtn > .sapMBtnActive,
	:not(.sapMBtnDisabled).sapMUCDeleteBtn:hover > .sapMBtnActive {
		background-color: @sapUiButtonLiteActiveBackground;
		border-color: @sapUiButtonLiteActiveBorderColor;
	}
}
/* END "UploadCollection.less" */

/* START "UploadSet.less" */
.sapMUSFileName {
	flex-shrink: 0;
}

.sapMUSObjectMarkerContainer {
	display: inline-block;
	flex: none;
}

.sapMUSProgressBox {
	min-width: 120px;
	padding-left: 1rem;
}

.sapMUSProgressIndicator {
	padding: 5px;
}

.sapMUSTextInnerContainer {
	display: flex;
}

.sapMUSTFileUploaderVisibility {
	visibility: hidden;
	display: none !important;
}

.sapMUSObjectMarkersAsStatusContainer {
	display: flex;
	gap: 0.5rem;
	align-items: end;
	margin-left: 0.5rem;
}

@media(min-width: 30rem) {
	.sapMUSButtonContainer {
		white-space: nowrap; /* so that buttons are displayed in a row */
		padding-left: 1rem;
		padding-top: 0rem;
		align-self: center; /* for buttons alignment at the center */
		flex: none;
		order: 3;
	}
}

@media(max-width: 30rem) {
	.sapMUSButtonContainer {
		padding-top: 0.75rem;
	}
}

.sapMUSButtonContainer > .sapMBtn {
	padding-top: 0rem; /* the padding of the sap.m.Button should be removed */
	padding-bottom: 0rem; /* the padding of the sap.m.Button should be removed */
	height:			2.5rem;
}


/*compact form */
.sapUiSizeCompact .sapMUSButtonContainer > .sapMBtn {
	height: 1.625rem;
}
/*for width > 30rem the buttons should be displayed as buttons of type transparent */
@media(min-width: 30rem) {
	.sapMUSButtonContainer >.sapMBtn >.sapMBtnInner {
		background-image: none;
		background-color: @sapUiButtonLiteBackground;
		border-color: @sapUiButtonLiteBorderColor;
		text-shadow: none;
	}
}
/* an edit and delete button in a hover state; it should look like a transparent button in a hover state */
@media(min-width: 30rem) {
	.sapMUCEditBtn:hover >.sapMBtnHoverable, .sapMUCDeleteBtn:hover >.sapMBtnHoverable {
		background-color: @sapUiButtonLiteHoverBackground;
		border-color: @sapUiButtonLiteHoverBorderColor;
	}
}

/*an edit and delete button in a pressed state; it should look like a transparent button in a pressed state */
@media(min-width: 30rem) {
	:not(.sapMBtnDisabled).sapMUCEditBtn > .sapMBtnActive,
	:not(.sapMBtnDisabled).sapMUCEditBtn:hover > .sapMBtnActive,
	:not(.sapMBtnDisabled).sapMUCDeleteBtn > .sapMBtnActive,
	:not(.sapMBtnDisabled).sapMUCDeleteBtn:hover > .sapMBtnActive {
		background-color: @sapUiButtonLiteActiveBackground;
		border-color: @sapUiButtonLiteActiveBorderColor;
	}
}
/* END "UploadSet.less" */

/* START "UploadSetwithTable.less" */
.sapMUSTItemIconInactive,
.sapMUSTItemIcon {
	text-align: initial;
}

.sapMUSTItemIconInactive,
.sapMUSTItemIcon,
.sapMUSTItemImage {
	height:				3rem;
	width:				3rem;
	margin-top:			0.1rem; /* is needed so that keyboard frame is visible; after the central focus concept is implemented delete the margin */
	margin-bottom:		0.1rem; /* is needed so that keyboard frame is visible; after the central focus concept is implemented delete the margin */
	margin-left:		0.1rem; /* is needed so that keyboard frame is visible; after the central focus concept is implemented delete the margin */
	flex:				none;
}

.sapMUSTItemIcon {
	color: @sapUiContentIconColor;
}
/* Item Icon Inactive */
.sapMUSTItemIconInactive {
	color: @sapUiContentNonInteractiveIconColor;
}

.sapMUSTItemIcon {
	font-size: 1.25rem!important;
	display: flex;
	align-items: center;
	justify-content: center;
	background: var(--sapAvatar_10_Background);
	border-radius: 4px;
	color: var(--sapContent_ImagePlaceholderForegroundColor);
}

.sapMUST .sapMIllustratedMessage .sapMIllustratedMessageMainContent {
	justify-content: center;
	flex-direction: row;
}
.sapMUST .sapMIllustratedMessage {
	padding: 0px;
	text-align: left;
}

.sapMUST .sapMIllustratedMessage.sapMIllustratedMessage-Spot .sapMIllustration {
	width: 4rem;
	height: 5rem;
	margin: 0 1rem;
}

.sapMUST .sapMIllustratedMessageAdditionalContent,.sapMUST .sapMIllustratedMessage.sapMIllustratedMessage-Spot .sapMIllustratedMessageAdditionalContent {
	margin-top: 0px;
	margin-bottom: 0px;
}

// alignment for illustrated figure captions
.sapMUST .sapMIllustratedMessage.sapMIllustratedMessage-Spot figcaption {
	margin-top: 1rem;
}

.sapMUST .sapMListTblCell.sapMListTblCellNoData {
	padding: 0px;
}

.sapMUSTRemovePopoverContainer {
	max-width: 20rem !important;
	max-height: 11rem !important;
}

// UploadSetwith Table plugin styles 

.sapMUSTP-IllustratedMessage-container .sapMIllustratedMessageMainContent {
	justify-content: center;
	flex-direction: row;
}

.sapMUSTP-IllustratedMessage-container.sapMIllustratedMessage {
	text-align: left;
}

.sapMUSTP-IllustratedMessage-container.sapMIllustratedMessage.sapMIllustratedMessage-Spot .sapMIllustration {
	height: 4.75rem;
}

.sapMUSTP-IllustratedMessage-container .sapMIllustratedMessageAdditionalContent,.sapMUSTP-IllustratedMessage-container.sapMIllustratedMessage.sapMIllustratedMessage-Spot .sapMIllustratedMessageAdditionalContent {
	margin-top: 0px;
	margin-bottom: 0px;
}
/* END "UploadSetwithTable.less" */

/* START "ValueStateMessage.less" */
/* ======================================== */
/* CSS for control sap.m/ValueStateMessage  */
/* Base theme                               */
/* ======================================== */

.sapMValueStateMessage {
	box-sizing: border-box;
	background: @sapUiFieldBackground;
	display: inline-block;
	color: @sapUiBaseText;
	font-size: @sapMFontSmallSize;
	font-family: @sapUiFontFamily;
	padding: 0.3rem 0.625rem;
	min-width: 6rem;
	max-width: 22rem;
	white-space: pre-line;
	word-wrap: break-word;
	overflow: hidden;
	text-overflow: ellipsis;

	.sapMFT {
		font-size: @sapMFontSmallSize;
	}
}

.sapMValueStateMessageError {
	background: @sapErrorBackground;
}

.sapMValueStateMessageWarning {
	background: @sapWarningBackground;
}

.sapMValueStateMessageInformation {
	background: @sapInformationBackground;
}
/* END "ValueStateMessage.less" */

/* START "VariantManagement.less" */
/* ============================================== */
/* CSS for control sap.m/VariantManagement    */
/* Base theme                                     */
/* ============================================== */

.sapMVarMngmt {
	text-align: center;
	display: inline-block;
}

.sapMVarMngmt .sapUiHLayout.sapMVarMngmtLayout .sapUiHLayoutChildWrapper:first-child>div {
   vertical-align: middle;
}
.sapMVarMngmt .sapUiHLayout.sapMVarMngmtLayout .sapUiHLayoutChildWrapper:nth-child(2)>span {
   vertical-align: top;
   padding-bottom: 8px;
}
.sapMVarMngmt .sapUiHLayout.sapMVarMngmtLayout .sapUiHLayoutChildWrapper:last-child>button {
   vertical-align: middle;
}

.sapMVarMngmt .sapUiHLayout.sapMVarMngmtLayout {
   display: grid;
   grid-template-columns: minmax(0, 1fr) 12px 32px;
   align-items: center;
}
.sapMVarMngmt .sapUiHLayout.sapMVarMngmtLayout.sapMVarMngmtLayoutModifiedHidden {
   grid-template-columns: minmax(0, 1fr) 1px 32px;
}
.sapMVarMngmt .sapUiHLayout.sapMVarMngmtLayout .sapUiHLayoutChildWrapper:first-child {
	grid-column: 1;
	grid-row: 1;
}
.sapMVarMngmt .sapUiHLayout.sapMVarMngmtLayout .sapUiHLayoutChildWrapper:nth-child(2) {
	grid-column: 2;
	grid-row: 1;
}
.sapMVarMngmt .sapUiHLayout.sapMVarMngmtLayout .sapUiHLayoutChildWrapper:nth-child(3) {
	grid-column: 3;
	grid-row: 1;
}

.sapMVarMngmt .sapMTitle.sapMVarMngmtTitle  {
    color: @sapUiLink;
	text-shadow: @sapUiContentTextShadow;
}

.sapMVarMngmt .sapMText.sapMVarMngmtTitle  {
    color: @sapUiLink;
    text-shadow: @sapUiContentTextShadow;
    font-size: 1rem;
    white-space: nowrap;
    vertical-align: middle;
    position: relative;
    overflow: hidden;
    text-overflow: ellipsis;
}

.sapMVarMngmt .sapMVarMngmtModified {
	color: @sapUiLink;
	font-weight: bold;
	padding-left: 0.125rem;
	padding-right: 0.125rem;
}

.sapMVarMngmtErrorPopover>div {
	padding-top: 1rem;
}

.sapMVarMngmtErrorPopover>div:last-child {
	padding-left: 4rem;
	padding-right: 4rem;
}

.sapMVarMngmt .sapMVarMngmtClickable:hover {
	color: @sapUiLinkHoverColor;
	cursor: pointer;
}

.sapMVarMngmt .sapMVarMngmtClickable:active {
	color: @sapUiLinkActiveColor;
	cursor: pointer;
}

.sapMVarMngmt .sapMVarMngmtDisabled {
	opacity: 0.5;
}

.sapMVarMngmtPopover .sapMSelectList {
	border: 0rem solid;
}

.sapMVarMngmtFavColor {
	color: @sapUiContentMarkerIconColor;
}

.sapMVarMngmtFavNonInteractiveColor {
	color: @sapUiContentNonInteractiveIconColor;
}

.sapMVarMngmtSaveDialog  .sapUiRespGrid.sapUiRespGridVSpace1 > div {
	margin-bottom: 0rem;
	margin: 0rem;
 }

.sapMVarMngmtSaveDialog .sapMVarMngmtSaveDialogLabel.sapMLabel::after {
	content: ":";
}

.sapMVarMngmtRolesEdit {
	padding-left: 0.5rem;
}

.sapMVarMngmtPopover .sapMIllustratedMessage.sapMVarMngmtIllustratedMessage  {
    padding-top: 0rem;
    padding-bottom: 0rem;
}
/* END "VariantManagement.less" */

/* START "ViewSettingsDialog.less" */
/* ========================================= */
/* CSS for control sap.m/ViewSettingsDialog  */
/* Base theme                                */
/* ========================================= */

.sapMVSD.sapMDialog.sapUiPopupWithPadding:not(.sapUiNoContentPadding) .sapMDialogScrollCont,
.sapMVSD.sapMDialog .sapMDialogScrollCont {
	padding: 0;
}

.sapMTB.sapMVSDFilterHeaderToolbar {
	padding-left: 0;
	padding-right: 1rem;
	width: 100%;

	.sapMBarChild.sapMCb {
		margin-left: 0;

		.sapMCbLabel {
			overflow: hidden;
			white-space: nowrap;
			text-overflow: ellipsis;
		}

		.sapMCbMark {
			white-space: normal;
		}
	}

	&.sapMIBar.sapMTBHeader-CTX.sapMListHdrTBar {
		padding-left: 0;
	}
}

/* needed for removing not needed padding from the dialog which depends on the compatible version */
.sapMVSD.sapUiPopupWithPadding .sapMDialogScrollCont.sapMDialogStretchContent {
	height: inherit;
}

.sapMSF.sapMVSDFilterSearchField,
.sapUiSizeCompact .sapMSF.sapMVSDFilterSearchField {
	padding: 0.25rem;
}

.sapUiSizeCompact .sapMTB.sapMListHdrTBar.sapMVSDFilterHeaderToolbar {
	height: 2rem;
	padding-left: 0.5rem;

	.sapMLabel.sapMCbLabel {
		overflow: hidden;
		text-overflow: ellipsis;
	}
}

.sapUiSizeCompact .sapMSF.sapMVSDFilterSearchField > .sapMSFF {
	height: 1.5rem;
}

html.sap-desktop .sapMTB.sapMListHdrTBar.sapMVSDFilterHeaderToolbar {

	.sapMCb {
		outline: none;
		padding-right: 0.5rem;
	}
}

.sapMSF.sapMVSDFilterSearchField {
	border-bottom: 1px solid var(--sapGroup_TitleBorderColor);
}

.sapMVSD .sapMTitle {
	font-size: var(--sapFontHeader5Size);
	color: var(--sapContent_LabelColor);
}

.sapUiSizeCompact .sapMTB.sapMListHdrTBar.sapMVSDFilterHeaderToolbar {

	.sapMLabel.sapMCbLabel {
		font-size: @sapMFontMediumSize;
	}

	.sapMCb.sapMBarChild:only-child {
		padding-left: 2rem;
	}

	&.sapMIBar.sapMTBHeader-CTX {
		padding-left: 0.1875rem;
	}
}

.sapUiSizeCozy .sapMTB.sapMListHdrTBar.sapMVSDFilterHeaderToolbar {

	.sapMLabel.sapMCbLabel {
		font-size: var(--sapFontLargeSize);
	}
}

.sapMVSD {

	.sapMTB {
		padding: 0 1rem 0 0.5rem;
	}

	.sapMLIB > .sapMLIBSelectS:first-child,
	.sapMLIB > .sapMLIBSelectM:first-child {
		margin-left: -0.6875rem;
	}

	.sapMPageSubHeader .sapMVSDBar {
		padding-bottom: 0.25rem;
	}

	.sapMTB.sapMVSDFilterHeaderToolbar .sapMBarChild.sapMCb {
		margin-left: 0.3125rem;
	}

	.sapMPage > .sapMPageHeader + .sapMPageSubHeader + section {
		top: 5.75rem;
	}

	.sapMPage .sapMVSDCustomTabContent {
		max-width: 99%;
	}
}

.sapUiSizeCompact .sapMVSD {

	.sapMSFI {
		height: 1.375rem;
		line-height: 1.375rem;
	}

	.sapMSFB {
		height: 1.375rem;
		line-height: 1.375rem;
	}

	.sapMLIB > .sapMLIBSelectS:first-child,
	.sapMLIB > .sapMLIBSelectM:first-child {
		margin-left: -0.5rem;
	}

	.sapMTB.sapMListHdrTBar.sapMVSDFilterHeaderToolbar {
		padding-left: 0.1875rem;
	}

	.sapMPage > .sapMPageHeader + .sapMPageSubHeader + section {
		top: 4.75rem;
	}
}

html.sap-phone .sapMVSDSeg {
	max-width: min-content;
}
/* END "ViewSettingsDialog.less" */

/* START "WheelSlider.less" */
/* ================================== */
/* CSS for control sap.m/WheelSlider  */
/* Base Styles                        */
/* ================================== */

.sapMWS {
	display: inline-block;
	overflow: hidden;
	height: 3rem;
	vertical-align: middle;
	text-align: center;
	box-sizing: border-box;
	cursor: pointer;
	margin-top: 2rem;

	.sapMWSLabel {
		display: none;
		box-sizing: border-box;
		height: 2rem;
		line-height: 2rem;
		width: 4.5rem;
		text-align: center;
		font-size: var(--sapFontSmallSize);
		color: var(--sapContent_LabelColor);
		text-align: center;
		vertical-align: middle;
	}

	.sapMWSArrows {
		display: none;
		box-sizing: border-box;
	}

	.sapMWSInner {
		overflow: hidden;
		height: 100%;
		box-sizing: border-box;
		-webkit-user-select: none;
		-moz-user-select: none;
		user-select: none;

		.sapMWSItem {
			height: 3rem;
			line-height: 3rem;
			width: 4.5rem;
			box-sizing: border-box;
			background: var(--sapSelectedColor);
			border: 1px solid var(--sapList_BorderColor);
			font-size: @sapMFontMediumSize;
			color: var(--sapContent_ContrastTextColor);
			text-align: center;

			&:hover {
				background: var(--sapHighlightColor);
				color: var(--sapContent_ContrastTextColor);
			}

			&:active {
				background: var(--sapHighlightColor);
				color: var(--sapContent_ContrastTextColor);
			}

			&:focus {
				outline: 1px dotted var(--sapContent_ContrastFocusColor);
				outline-offset: -3px;
			}
		}

		.sapMWSSelectionFrame {
			width: 4.5rem;
			height: 3rem;
			position: absolute;
			box-sizing: border-box;
			display: none;
		}

		& > ul {
			list-style: none;
			margin: 0;
			padding: 0;
		}
	}

	&.sapMWSExpanded {
		height: 100%;
		cursor: default;
		margin: 0;

		.sapMWSLabel {
			display: block;
		}

		.sapMWSArrows {
			display: none;
		}

		.sapMWSInner {
			height: calc(100% - 2rem);
			max-height: 100%; /* because of the SliderValuesN classes below */

			.sapMWSItem {
				background: var(--sapList_Background);
				color: var(--sapTextColor);
				border: 1px solid var(--sapList_BorderColor);

				&:hover {
					background: var(--sapHighlightColor);
				}

				&:active {
					background: var(--sapHighlightColor);
					color: var(--sapContent_ContrastTextColor);
				}

				&:focus {
					outline: 1px dotted var(--sapContent_FocusColor);
					outline-offset: -3px;
				}
			}

			.sapMWSSelectionFrame {
				display: block;
				outline: 0.125rem solid var(--sapContent_ForegroundBorderColor);
				outline-offset: -0.313rem;

				&:hover {

					& + ul > li.sapMWSItem.sapMWSItemSelected {
						background: var(--sapHighlightColor);
					}
				}

				&:active {

					& + ul > li.sapMWSItem.sapMWSItemSelected {
						background: var(--sapSelectedColor);
						color: var(--sapContent_ContrastTextColor);
					}
				}
			}
		}

		.sapMWSInner.SliderValues3 {
			height: 9rem;
		}

		.sapMWSInner.SliderValues4 {
			height: 12rem;
		}

		.sapMWSInner.SliderValues5 {
			height: 15rem;
		}

		.sapMWSInner.SliderValues6 {
			height: 18rem;
		}

		.sapMWSInner.SliderValues7 {
			height: 21rem;
		}

		.sapMWSInner.SliderValues8 {
			height: 24rem;
		}

		.sapMWSInner.SliderValues9 {
			height: 27rem;
		}

		.sapMWSInner.SliderValues10 {
			height: 30rem;
		}

		.sapMWSInner.SliderValues11 {
			height: 33rem;
		}

		.sapMWSInner.SliderValues12 {
			height: 36rem;
		}
	}

	&:focus {
		outline: none;
	}
}

.sapUiSizeCompact {

	.sapMWS {
		height: 2rem;

		.sapMWSLabel {
			width: 4rem;
		}

		.sapMWSInner {

			.sapMWSItem {
				height: 2rem;
				line-height: 2rem;
				width: 4rem;
				box-sizing: border-box;
			}

			.sapMWSSelectionFrame {
				width: 4rem;
				height: 2rem;
			}
		}

		&.sapMWSExpanded {
			height: 100%;
			cursor: default;
			margin: 0;

			.sapMWSArrows {
				display: block;
			}

			.sapMWSInner {
				height: ~"calc(100% - 6rem)";
				max-height: ~"calc(100% - 6rem)";
			}

			.sapMWSInner.SliderValues3 {
				height: 6rem;
			}

			.sapMWSInner.SliderValues4 {
				height: 8rem;
			}

			.sapMWSInner.SliderValues5 {
				height: 10rem;
			}

			.sapMWSInner.SliderValues6 {
				height: 12rem;
			}

			.sapMWSInner.SliderValues7 {
				height: 14rem;
			}

			.sapMWSInner.SliderValues8 {
				height: 16rem;
			}

			.sapMWSInner.SliderValues9 {
				height: 18rem;
			}

			.sapMWSInner.SliderValues10 {
				height: 20rem;
			}

			.sapMWSInner.SliderValues11 {
				height: 22rem;
			}

			.sapMWSInner.SliderValues12 {
				height: 24rem;
			}
		}
	}
}

.sap-phone {

	.sapMWS {
		display: inline-block;
		height: 3rem;
		text-align: center;
		overflow: hidden;
		cursor: pointer;
		vertical-align: middle;
		box-sizing: border-box;

		.sapMWSLabel {
			height: 2rem;
			width: 4.5rem;
			display: none;
			font-size: var(--sapFontSmallSize);
			color: var(--sapContent_LabelColor);
			text-align: center;
			vertical-align: middle;
		}

		.sapMWSArrows {
			display: none;
		}

		.sapMWSInner {
			height: 100%;
			/* for Android 4.1 - 4.3 calc is not supported, so use 100% instead */
			max-height: 100%;
			overflow: hidden;

			.sapMWSItem {
				height: 3rem;
				line-height: 3rem;
				width: 4.5rem;
				box-sizing: border-box;
				background: var(--sapSelectedColor);
				border: 1px solid var(--sapList_BorderColor);
				font-size: @sapMFontMediumSize;
				color: var(--sapContent_ContrastTextColor);
				text-align: center;

				&:active {
					background: var(--sapHighlightColor);
					color: var(--sapContent_ContrastTextColor);
				}

				&:focus {
					outline: 1px dotted var(--sapContent_ContrastFocusColor);
					outline-offset: -3px;
				}
			}

			.sapMWSSelectionFrame {
				width: 4.5rem;
				height: 3rem;
				position: absolute;
				display: none;
				box-sizing: border-box;
			}
		}

		&.sapMWSExpanded {
			height: 100%;
			cursor: default;

			.sapMWSLabel {
				display: block;
			}

			.sapMWSArrows {
				display: none;
			}

			.sapMWSInner {
				height: calc(100% - 2rem);
				/* for Android 4.1 - 4.3 calc is not supported, so use 100% instead */
				max-height: 100%;
				overflow: hidden;

				.sapMWSItem {
					height: 3rem;
					line-height: 3rem;
					width: 4.5rem;
					box-sizing: border-box;
					background: var(--sapList_Background);
					color: var(--sapTextColor);
					border: 1px solid var(--sapList_BorderColor);

					&:active {
						background: var(--sapHighlightColor);
						color: var(--sapContent_ContrastTextColor);
					}

					&:focus {
						outline: 1px dotted var(--sapContent_FocusColor);
						outline-offset: -3px;
					}
				}

				.sapMWSSelectionFrame {
					display: block;
					outline: 0.125rem solid var(--sapContent_ForegroundBorderColor);
					outline-offset: -0.313rem;
				}
			}

			.sapMWSInner.SliderValues3 {
				height: 9rem;
			}

			.sapMWSInner.SliderValues4 {
				height: 12rem;
			}

			.sapMWSInner.SliderValues5 {
				height: 15rem;
			}

			.sapMWSInner.SliderValues6 {
				height: 18rem;
			}

			.sapMWSInner.SliderValues7 {
				height: 21rem;
			}

			.sapMWSInner.SliderValues8 {
				height: 24rem;
			}

			.sapMWSInner.SliderValues9 {
				height: 27rem;
			}

			.sapMWSInner.SliderValues10 {
				height: 30rem;
			}

			.sapMWSInner.SliderValues11 {
				height: 33rem;
			}

			.sapMWSInner.SliderValues12 {
				height: 36rem;
			}
		}

		&:focus {
			outline: none;
		}
	}
}
/* END "WheelSlider.less" */

/* START "WheelSliderContainer.less" */
/* =========================================== */
/* CSS for control sap.m/WheelSliderContainer  */
/* Base Styles                                 */
/* =========================================== */

.sapMWSContainer {
	width: 100%;
	height: 100%;
	min-width: 20rem;
	box-sizing: border-box;
	padding: 1rem;
	text-align: center;
	position: relative;

	&::before {
		content: '';
		display: inline-block;
		height: 100%;
		vertical-align: middle;
	}

	.sapMWSContainerLabel {
		display: none;
		box-sizing: border-box;
	}

	.sapMWSContainerLabel {
		font-size: var(--sapFontHeader5Size);
		color: @sapUiCalendarColorToday;
	}
}

.sapUiSizeCompact {

	.sapMWSContainer {
		padding: 0.5rem;
		margin: auto;
	}
}

.sap-phone {

	.sapMWSContainer {
		max-height: 38rem; /* 12 * item height + label's height */
		height: 100%;
		padding: 1rem;
		box-sizing: border-box;
		margin: auto;
		text-align: center;

		&::before {
			content: '';
			display: inline-block;
			height: 100%;
			margin-right: -0.25rem;
			vertical-align: middle;
		}

		.sapMWSContainerLabel {
			position: absolute;
			top: 0;
			left: 50%;
			transform: translate(-50%, 0);
			display: block;
			height: 2rem;
			line-height: 2rem;
			font-size: var(--sapFontHeader5Size);
			color: @sapUiCalendarColorToday;
		}
	}
}
/* END "WheelSliderContainer.less" */

/* START "Wizard.less" */
/* ============================= */
/* CSS for control sap.m/Wizard  */
/* Base theme                    */
/* ============================= */

@_sap_m_Wizard_NavHeight: 4rem;
@_sap_m_WizardStep_Animation_Duration: 0.4s;

.sapMWizard {
	height: 100%;
	position: relative;
	counter-reset: steps-counter;
	overflow: hidden;
	background-color: transparent;

	.sapMWizardStepContainer {
		position: relative;
		height: ~"calc(100% - 4rem)"; /* 4rem is the height of the Progress Navigator*/
		box-sizing: border-box;
		padding: 0 1rem 1rem 1rem;
	}

	.sapMWizardStep:first-child {
		margin-top: 1rem;
	}

	.sapMWizardStep  {
		position: relative;
		padding-bottom: 2rem;
		display: none;
	}

	.sapMWizardStepActivated {
		display: block;
	}

	&.sapMWizardModeScroll .sapMWizardStepTitle::before {
		counter-increment: steps-counter;
		content: counter(steps-counter) ". ";
	}

	.sapMWizardNextButtonHidden {
		bottom: -3rem;
		opacity: 0;
		animation-name: hideWizardButton
	}

	.sapMWizardNextButtonVisible {
		margin-top: 0.5rem;
		bottom: 0rem;
		opacity: 1;
		animation-name: showWizardButton
	}

	.sapMDialogSection {
		height: auto;
	}


	&.sapMWizardBgStandard .sapMWizardStepContainer {
		background-color: @sapUiBaseBG;
	}

	&.sapMWizardBgSolid .sapMWizardStepContainer {
		background-color: @sapUiBaseBG;
	}

	&.sapMWizardBgTransparent .sapMWizardStepContainer {
		background-color: transparent;
	}

	&.sapMWizardBgList .sapMWizardStepContainer {
		background-color: @sapUiGroupContentBackground;
	}
}

.sapMWizard:not(.sapMWizardModePage) .sapMWizardLastActivatedStep {
	min-height: ~"calc(100% - 1rem)"; /* 1rem is the margin from the top of the container*/
}

html[data-sap-ui-animation='on'] .sapMWizardNextButtonVisible,
html[data-sap-ui-animation='on'] .sapMWizardNextButtonVisible {
	animation-timing-function: linear;
	animation-duration: @_sap_m_WizardStep_Animation_Duration;
}

@keyframes showWizardButton {
	from {opacity: 0; bottom: -3rem}
	to {opacity: 1; bottom: 0rem}
}

@keyframes hideWizardButton {
	from {opacity: 1; bottom: 0rem}
	to {opacity: 0; bottom: -3rem}
}

html.sap-desktop .sapMWizard .sapMWizardNextButtonVisible,
html.sap-tablet .sapMWizard .sapMWizardNextButtonVisible {
	display: block;
}

html.sap-phone .sapMWizard {
	.sapMWizardNextButtonVisible {
		position: fixed;
		left: 50%;
		-webkit-transform: translate(-50%, 0);
		transform: translate(-50%, 0);
		bottom: 3rem;
	}
}
/* END "Wizard.less" */

/* START "WizardProgressNavigator.less" */
/* ============================================== */
/* CSS for control sap.m/WizardProgressNavigator  */
/* Base theme                                     */
/* ============================================== */

@_sap_m_WizardProgressNavigator_NavHeight: 4rem;
@_sap_m_WizardProgressNavigator_ListHeight: 2rem;
@_sap_m_WizardProgressNavigator_ListItemHeight: 3rem;
@_sap_m_WizardProgressNavigator_StepWidth: 2rem;
@_sap_m_WizardProgressNavigator_StepPadding: 0.5rem;
@_sap_m_WizardProgressNavigator_SmallSeparator: 0.25rem;
@_sap_m_WizardProgressNavigator_StepUnderlineHeight: 0.125rem;
@_sap_m_WizardProgressNavigator_StepOverlapOnPhone: 1.75rem;

.sapMWizardProgressNav {
	box-sizing: border-box;
	height: @_sap_m_WizardProgressNavigator_NavHeight;
	padding: 1rem 1rem 0 1rem;
	-webkit-user-select: none;
	-moz-user-select: none;
	user-select: none;
	background: @sapUiObjectHeaderBackground;
	font-size: @sapMFontMediumSize;
	box-shadow: @sapUiShadowHeader;
}

.sapMWizardProgressNavList,
.sapMWizardProgressNavListVarying {
	display: table;
	table-layout: fixed;
	position: relative;
	list-style: none;
	margin: 0;
	padding: 0;
	width: 100%;
	height: @_sap_m_WizardProgressNavigator_ListItemHeight;
}

.sapMWizardProgressNavStepContainer {
	display: inline-block;
	cursor: pointer;
	position: relative;
	text-align: center;
	text-decoration: none;
}

.sapMWizardProgressNavStep[data-sap-ui-wpn-step-open="true"] .sapMWizardProgressNavStepContainer {
	max-width: 100%;
}

.sapMWizardProgressNavStep {
	display: table-cell;
	position: relative;
	padding: 0 @_sap_m_WizardProgressNavigator_StepPadding;

	&:focus {
		outline: none;

		.sapMWizardProgressNavStepContainer {
			outline: @sapUiContentFocusWidth @sapUiContentFocusStyle @sapUiContentFocusColor;
			outline-offset: 1px;
		}
	}
}

// the separator connecting every two steps
.sapMWizardProgressNavStep::after {
	content: "";
	display: block;
	position: absolute;
	top: @_sap_m_WizardProgressNavigator_ListHeight / 2;
	left: @_sap_m_WizardProgressNavigator_StepWidth + (2 * @_sap_m_WizardProgressNavigator_StepPadding);
	right: 0;
	border-bottom-width: 1px;
	border-bottom-style: solid;
	z-index: -1;
}

// we do not have a separator at the end for known number of steps
.sapMWizardProgressNavList .sapMWizardProgressNavStep:last-child::after {
	display: none;
}

// we have a dashed separator at the end when the number of steps can change
.sapMWizardProgressNavListVarying .sapMWizardProgressNavStep:last-child::after {
	display: block;
	border-bottom-style: dashed;
}

// the currently selected step
.sapMWizardProgressNavStep[data-sap-ui-wpn-step-current="true"] {

	// the current step is underlined as to have a visual cue
	.sapMWizardProgressNavStepContainer::after {
		content: "";
		display: block;
		position: absolute;
		width: 100%;
		height: @_sap_m_WizardProgressNavigator_StepUnderlineHeight;

		// negative values are needed for the right positioning - (2rem - 4rem) / 2 = -1
		// bottom:0 positions it at the bottom of the list but we want it at the bottom of the nav
		bottom: (@_sap_m_WizardProgressNavigator_ListHeight - @_sap_m_WizardProgressNavigator_NavHeight) / 2;
	}
	//all steps after the current one are right aligned so that they dont overflow to the right
	& ~ .sapMWizardProgressNavStep > .sapMWizardProgressNavStepContainer {
		right: 0;
	}
}

// the step up to which progress has been reached
.sapMWizardProgressNavStep[data-sap-ui-wpn-step-active="true"] {

	//all steps after the active one have the default cursor and no focus
	& ~ .sapMWizardProgressNavStep > .sapMWizardProgressNavStepContainer {
		cursor: default;
		display: inline;
	}

	& ~ .sapMWizardProgressNavStep:focus > .sapMWizardProgressNavStepContainer {
		outline: none;
	}
}

// all steps that are grouped
.sapMWizardProgressNavStep[data-sap-ui-wpn-step-open="false"] {
	width: @_sap_m_WizardProgressNavigator_StepWidth - @_sap_m_WizardProgressNavigator_StepOverlapOnPhone;
	padding: 0;

	.sapMWizardProgressNavStepContainer {
		position: absolute;
		top: 0;
	}

	.sapMWizardProgressNavStepTitleContainer {
		display: none;
	}
}

// when there are no titles and the number of steps is know the last step should align to the end
.sapMWizardProgressNavList.sapMWizardProgressNavListNoTitles .sapMWizardProgressNavStep:last-child {
	width: @_sap_m_WizardProgressNavigator_StepWidth;
}

// override the previous rule when more than one step is grouped because we want the last step to be
// shrunk and show below the previous step with [data-sap-ui-wpn-step-open="false"]
.sapMWizardProgressNavStep[data-sap-ui-wpn-step-open="false"] + .sapMWizardProgressNavStep[data-sap-ui-wpn-step-open="false"] {
	width: @_sap_m_WizardProgressNavigator_StepWidth - @_sap_m_WizardProgressNavigator_StepOverlapOnPhone;
}

// select the first grouped step on the right and style it so that it does not collapse
.sapMWizardProgressNavStep[data-sap-ui-wpn-step-open="true"] + .sapMWizardProgressNavStep[data-sap-ui-wpn-step-open="false"] {
	width: @_sap_m_WizardProgressNavigator_StepWidth;
	padding-left: @_sap_m_WizardProgressNavigator_StepPadding;
}

// select the last grouped step on the left and style it so that it does not collapse
// add the needed padding to the right so that the separator between the steps is visible
// the second selector is needed for a corner case
.sapMWizardProgressNavStep[data-sap-ui-wpn-step-open-prev="true"],
.sapMWizardProgressNavStep[data-sap-ui-wpn-step-open="false"] +  .sapMWizardProgressNavStep[data-sap-ui-wpn-step-open-prev="true"] {
	width: @_sap_m_WizardProgressNavigator_StepWidth;
	padding-right: @_sap_m_WizardProgressNavigator_StepPadding + @_sap_m_WizardProgressNavigator_SmallSeparator;

	&::after {
		left: @_sap_m_WizardProgressNavigator_StepWidth + @_sap_m_WizardProgressNavigator_StepPadding;
	}
}

.sapMWizardProgressNavStepCircle {
	display: inline-block;
	box-sizing: border-box;
	vertical-align: middle;
	text-align: center;
	width: @_sap_m_WizardProgressNavigator_StepWidth;
	height: @_sap_m_WizardProgressNavigator_ListHeight;
	line-height: @_sap_m_WizardProgressNavigator_ListHeight;
	border-width: 1px;
	border-style: solid;
	border-radius: 50%;
	color: @sapUiBaseText;
	background: @sapUiObjectHeaderBackground;
	border-color: @sapUiObjectHeaderBorderColor;
	font-size: @sapMFontMediumSize;
	.sapUiIcon {
		font-size: @sapMFontLargeSize;
	}
}

.sapMSplitContainerDetail .sapMWizardProgressNav {
	padding: 1rem 1rem 0 1rem;
}

html.sap-phone .sapMWizardProgressNav {
	padding: 1rem 0.5rem;
}

.sapMWizardProgressNavStep:hover .sapMWizardProgressNavStepCircle {
	background: @sapUiHighlight;
}

.sapMWizardProgressNavStepIcon {
	font-size: @sapMFontLargeSize;
	cursor: pointer;
}

.sapMWizardProgressNavStep::after {
	border-bottom-color: @sapUiObjectHeaderBorderColor;
}

.sapMWizardProgressNavStep[data-sap-ui-wpn-step-current="true"] {
	& .sapMWizardProgressNavStepContainer::after {
		background: @sapUiSelected;
		height: 0.25rem;
	}

	& .sapMWizardProgressNavStepCircle {
		background: @sapUiSelected;
		color: @sapUiContentContrastIconColor;
	}

	& ~ .sapMWizardProgressNavStep .sapMWizardProgressNavStepIcon {
		cursor: default;
	}
}


.sapMWizardProgressNavStepTitleContainer {
	display: inline-block;
	vertical-align: middle;
	text-align: left;
	padding: 0 @_sap_m_WizardProgressNavigator_StepPadding;
	max-height: 3rem;
	overflow: hidden;
	max-width: ~"calc(100% - 3rem)";
	background: @sapUiObjectHeaderBackground;
}

.sapMWizardProgressNavStepTitle {
	overflow: hidden;
	max-width: 100%;
	max-height: 2.875rem;
	line-height: @_sap_m_WizardProgressNavigator_ListHeight  / 2;
	font-size: @sapMFontMediumSize;
	color: @sapUiGroupTitleTextColor;
	vertical-align: middle;
	display: -webkit-inline-box;
	-webkit-box-orient: vertical;
	-webkit-line-clamp: 2;
}

.sapMWizardProgressNavStepTitleOptional {
	-webkit-line-clamp: 1;
	line-height: 100%;
}

.sapMWizardProgressNavStepLabelOptional {
	display: table-row;
	font-size: @sapMFontSmallSize;
	color: @sapUiContentLabelColor;
	overflow: hidden;
}
/* END "WizardProgressNavigator.less" */

/* START "WizardStep.less" */
/* ================================= */
/* CSS for control sap.m/WizardStep  */
/* Base theme                        */
/* ================================= */

.sapMWizardStep {
	box-sizing: border-box;

	.sapMWizardStepTitle {
		font-size: 1.25rem;
		font-weight: normal;
		margin-top: 0;
		margin-bottom: 1rem;
	}

	.sapUiFormResLayout > div {
		background-color: inherit;
	}
}

.sapMWizardStep .sapMWizardStepTitle {
	color: @sapUiGroupTitleTextColor;
}
/* END "WizardStep.less" */


/* import plugin less files */
/* START "CellSelector.less" */
.sapMPluginsCellSelectorRsz {
	position: absolute;
	z-index: 7;
	box-sizing: border-box;

	&.sapMPluginsVerticalBorder {
		width: 7px;
		margin-left: -3.5px;
		cursor: ew-resize;

		html[dir=rtl] & {
			margin-left: 0;
			margin-right: -3.5px;
		}
	}

	&.sapMPluginsHorizontalBorder {
		height: 7px;
		margin-top: -3.5px;
		cursor: ns-resize;
	}

	&.sapMPluginsEdge {
		height: 10px !important;
		width: 10px !important;
		margin-top: -5px;
		margin-left: -5px;

		html[dir=rtl] & {
			margin-left: 0;
			margin-right: -5px;
		}

		&.sapMPluginsNESW {
			cursor: nesw-resize;
		}

		&.sapMPluginsNWSE {
			cursor: nwse-resize;
		}

		/* Needed, because (for some reason) RTL converts nwse-resize to the opposite nesw-resize (alhtough we do not want this to happen) */
		html[dir=rtl] &.sapMPluginsNESW {
			cursor: nwse-resize;
		}

		html[dir=rtl] &.sapMPluginsNWSE {
			cursor: nesw-resize;
		}
	}
}

.sapMPluginsCellSelectorRsz:hover {
	background-color: @sapList_HighlightColor;
	opacity: 0.5;
}

.sapMPluginsCellSelectorRszActive {
	opacity: 1 !important;
}

.sapMPluginsCellSelectorSelected {
	background-color: var(--sapList_Hover_SelectionBackground);
}

.sapMLIBSelected > .sapMPluginsCellSelectorSelected {
	border-bottom: var(--sapList_BorderWidth) var(--sapList_SelectionBorderColor) solid;
}

.sapMPluginsCellSelectorTop {
	box-shadow: inset 0 var(--sapList_BorderWidth) var(--sapList_SelectionBorderColor);
}

.sapMPluginsCellSelectorRight {
	box-shadow: inset ~"calc(var(--sapList_BorderWidth) * -1)" 0 var(--sapList_SelectionBorderColor);
}

.sapMPluginsCellSelectorBottom {
	box-shadow: inset 0 ~"calc(var(--sapList_BorderWidth) * -1)" var(--sapList_SelectionBorderColor);
}

.sapMPluginsCellSelectorLeft {
	box-shadow: inset var(--sapList_BorderWidth) 0 var(--sapList_SelectionBorderColor);
}

.sapMPluginsCellSelectorBottom {
	box-shadow: inset 0 ~"calc(var(--sapList_BorderWidth) * -1)" var(--sapList_SelectionBorderColor);
}

.sapMPluginsCellSelectorTop.sapMPluginsCellSelectorRight {
	box-shadow: inset 0 var(--sapList_BorderWidth) var(--sapList_SelectionBorderColor),
				inset ~"calc(var(--sapList_BorderWidth) * -1)" 0 var(--sapList_SelectionBorderColor);
}

.sapMPluginsCellSelectorTop.sapMPluginsCellSelectorBottom {
	box-shadow: inset 0 var(--sapList_BorderWidth) var(--sapList_SelectionBorderColor),
				inset 0 ~"calc(var(--sapList_BorderWidth) * -1)" var(--sapList_SelectionBorderColor);
}

.sapMPluginsCellSelectorTop.sapMPluginsCellSelectorLeft {
	box-shadow: inset 0 var(--sapList_BorderWidth) var(--sapList_SelectionBorderColor),
				inset var(--sapList_BorderWidth) 0 var(--sapList_SelectionBorderColor);
}

.sapMPluginsCellSelectorRight.sapMPluginsCellSelectorBottom {
	box-shadow: inset ~"calc(var(--sapList_BorderWidth) * -1)" 0 var(--sapList_SelectionBorderColor),
				inset 0 ~"calc(var(--sapList_BorderWidth) * -1)" var(--sapList_SelectionBorderColor);
}

.sapMPluginsCellSelectorRight.sapMPluginsCellSelectorLeft {
	box-shadow: inset ~"calc(var(--sapList_BorderWidth) * -1)" 0 var(--sapList_SelectionBorderColor),
				inset var(--sapList_BorderWidth) 0 var(--sapList_SelectionBorderColor);
}

.sapMPluginsCellSelectorBottom.sapMPluginsCellSelectorLeft {
	box-shadow: inset 0 ~"calc(var(--sapList_BorderWidth) * -1)" var(--sapList_SelectionBorderColor),
				inset var(--sapList_BorderWidth) 0 var(--sapList_SelectionBorderColor);
}

.sapMPluginsCellSelectorTop.sapMPluginsCellSelectorRight.sapMPluginsCellSelectorBottom {
	box-shadow: inset 0 var(--sapList_BorderWidth) var(--sapList_SelectionBorderColor),
				inset ~"calc(var(--sapList_BorderWidth) * -1)" 0 var(--sapList_SelectionBorderColor),
				inset 0 ~"calc(var(--sapList_BorderWidth) * -1)" var(--sapList_SelectionBorderColor);
}

.sapMPluginsCellSelectorTop.sapMPluginsCellSelectorRight.sapMPluginsCellSelectorLeft {
	box-shadow: inset 0 var(--sapList_BorderWidth) var(--sapList_SelectionBorderColor),
				inset ~"calc(var(--sapList_BorderWidth) * -1)" 0 var(--sapList_SelectionBorderColor),
				inset var(--sapList_BorderWidth) 0 var(--sapList_SelectionBorderColor);
}

.sapMPluginsCellSelectorTop.sapMPluginsCellSelectorBottom.sapMPluginsCellSelectorLeft {
	box-shadow: inset 0 var(--sapList_BorderWidth) var(--sapList_SelectionBorderColor),
				inset 0 ~"calc(var(--sapList_BorderWidth) * -1)" var(--sapList_SelectionBorderColor),
				inset var(--sapList_BorderWidth) 0 var(--sapList_SelectionBorderColor);
}

.sapMPluginsCellSelectorRight.sapMPluginsCellSelectorBottom.sapMPluginsCellSelectorLeft {
	box-shadow: inset ~"calc(var(--sapList_BorderWidth) * -1)" 0 var(--sapList_SelectionBorderColor),
				inset 0 ~"calc(var(--sapList_BorderWidth) * -1)" var(--sapList_SelectionBorderColor),
				inset var(--sapList_BorderWidth) 0 var(--sapList_SelectionBorderColor);
}

.sapMPluginsCellSelectorTop.sapMPluginsCellSelectorRight.sapMPluginsCellSelectorBottom.sapMPluginsCellSelectorLeft {
	box-shadow: inset 0 0 0 var(--sapList_BorderWidth) var(--sapList_SelectionBorderColor);
}
/* END "CellSelector.less" */

/* START "ColumnResizer.less" */
.sapMPluginsColumnResizerContainer {
	position: relative;
}

.sapMPluginsColumnResizerResizing {
	cursor: col-resize;
}

.sapMPluginsColumnResizerResizable {
	border-right: 1px solid var(--sapList_BorderColor);
}

.sapMPluginsColumnResizerHandle {
	position: absolute;
	z-index: 2;
	top: 0;
	bottom: 0;
	left: -10000px;
	width: 0.1875rem;
	padding: 0 0.125rem;
	margin-left: -0.25rem;
	cursor: col-resize;
	background-clip: content-box;
	background-color: var(--sapContent_DragAndDropActiveColor);
	-webkit-user-select: none;
	user-select: none;
}

.sapMPluginsColumnResizerHandleCircle {
	content: '';
	position: absolute;
	top: 1.5rem;
	left: -0.4375rem;
	width: 1rem;
	height: 1rem;
	border-radius: 1rem;
	border: 0.125rem solid var(--sapContent_DragAndDropActiveColor);
	background: @sapUiWhiteBG;
	pointer-events: none;
	margin-top: -0.125rem;
}

.sapMPluginsColumnResizerSizeDetector {
	position: absolute;
	left: -1000px;
	visibility: hidden;
	display: flex;
	flex-direction: column;
}
/* END "ColumnResizer.less" */

/* START "PasteProvider.less" */
/* ==================================== */
/* CSS for control sap.m/PasteProvider  */
/* Base theme                           */
/* ==================================== */

.sapMPluginsPasteProviderMessage {
	outline: 0;
	padding: 2rem;
	font-size: @sapFontSize;
	font-family: var(--sapFontFamily);
	color: var(--sapTextColor);
}

.sapMPluginsPasteProviderShortCut {
	font-size: var(--sapFontLargeSize);
	font-family: var(--sapFontBoldFamily);
}

.sapMPluginsPasteProviderHighlight {
	position: relative;
}

.sapMPluginsPasteProviderHighlight::after {
	content: '';
	background: @sapUiDragAndDropActiveBackground;
	border: 0.125rem solid @sapUiDragAndDropActiveBorderColor;
	position: absolute;
	top: 0;
	left: 0;
	right: 0;
	bottom: 0;
	z-index: 99;
	pointer-events: none;
}
/* END "PasteProvider.less" */

/* START "ContextMenuSetting.less" */
.sapMContextMenuSettingContentOpacity {
	opacity: var(--sapContent_DisabledOpacity);
}
/* END "ContextMenuSetting.less" */
